@@ -69,7 +69,6 @@ struct HelloWorldExample : public TestBase
 
 int exampleMain(int argc, char** argv)
 {
-    initDebugCallback();
     HelloWorldExample example;
     example.parseOption(argc, argv);
     return example.run();
@@ -127,13 +126,7 @@ int HelloWorldExample::createComputePipelineFromShader()
 
     sessionDesc.targets = &targetDesc;
     sessionDesc.targetCount = 1;
-
-    std::vector<slang::CompilerOptionEntry> options;
-    options.push_back(
-        {slang::CompilerOptionName::EmitSpirvDirectly,
-         {slang::CompilerOptionValueKind::Int, 1, 0, nullptr, nullptr}});
-    sessionDesc.compilerOptionEntries = options.data();
-    sessionDesc.compilerOptionEntryCount = options.size();
+    sessionDesc.compilerOptionEntryCount = 0;
 
     ComPtr<slang::ISession> session;
     RETURN_ON_FAIL(slangGlobalSession->createSession(sessionDesc, session.writeRef()));
