@@ -587,12 +587,6 @@ struct ReflectingPrinting
                 auto containerVarLayout = typeLayout->getContainerVarLayout();
                 auto elementVarLayout = typeLayout->getElementVarLayout();
 
-                key("container");
-                {
-                    SCOPED_OBJECT();
-                    printOffsets(containerVarLayout, accessPath);
-                }
-
                 AccessPath innerOffsets = accessPath;
                 innerOffsets.deepestConstantBufer = innerOffsets.leaf;
                 if (containerVarLayout->getTypeLayout()->getSize(
@@ -601,6 +595,12 @@ struct ReflectingPrinting
                     innerOffsets.deepestParameterBlock = innerOffsets.leaf;
                 }
 
+                key("container");
+                {
+                    SCOPED_OBJECT();
+                    printOffsets(containerVarLayout, innerOffsets);
+                }
+
                 key("content");
                 {
                     SCOPED_OBJECT();
