@@ -18,9 +18,15 @@
 # This script is meant to be run by
 # https://github.com/google/oss-fuzz/blob/master/projects/miniz/Dockerfile
 
+cat << "EOF" > miniz_export.h
+#ifndef MINIZ_EXPORT
+#define MINIZ_EXPORT
+#endif
+EOF
+
 mkdir build
 cd build
-cmake .. -DAMALGAMATE_SOURCES=ON -DBUILD_SHARED_LIBS=OFF -DBUILD_FUZZERS=ON
+cmake .. -DAMALGAMATE_SOURCES=ON -DBUILD_SHARED_LIBS=OFF -DBUILD_FUZZERS=ON -DBUILD_TESTS=OFF
 make -j$(nproc)
 cd ..
 
@@ -39,4 +45,4 @@ done
 rm -f $OUT/zip_fuzzer_seed_corpus.zip
 zip $OUT/zip_fuzzer_seed_corpus.zip $OUT/seed_corpus.zip
 
-cp tests/zip.dict $OUT/zip_fuzzer.dict
\ No newline at end of file
+cp tests/zip.dict $OUT/zip_fuzzer.dict
