@@ -2,6 +2,7 @@
 
 #include "rhi-shared.h"
 #include "device.h"
+#include "format-conversion.h"
 
 namespace rhi {
 
@@ -27,7 +28,7 @@ void RenderPassEncoder::writeRenderState()
     cmd.state = m_renderState;
     cmd.pipeline = m_pipeline;
     m_commandEncoder->getPipelineSpecializationArgs(m_pipeline, m_rootObject, cmd.specializationArgs);
-    if (!SLANG_SUCCEEDED(m_commandEncoder->getBindingData(m_rootObject, cmd.bindingData)))
+    if (SLANG_FAILED(m_commandEncoder->getBindingData(m_rootObject, cmd.bindingData)))
     {
         m_commandEncoder->getDevice()
             ->handleMessage(DebugMessageType::Error, DebugMessageSource::Layer, "Failed to get binding data");
@@ -42,7 +43,7 @@ IShaderObject* RenderPassEncoder::bindPipeline(IRenderPipeline* pipeline)
     {
         m_pipeline = pipeline;
         ShaderProgram* program = checked_cast<ShaderProgram*>(pipeline->getProgram());
-        if (!SLANG_SUCCEEDED(m_commandEncoder->getDevice()->createRootShaderObject(program, m_rootObject.writeRef())))
+        if (SLANG_FAILED(m_commandEncoder->getDevice()->createRootShaderObject(program, m_rootObject.writeRef())))
             return nullptr;
         return m_rootObject;
     }
@@ -137,15 +138,13 @@ void RenderPassEncoder::drawMeshTasks(uint32_t x, uint32_t y, uint32_t z)
     }
 }
 
-void RenderPassEncoder::pushDebugGroup(const char* name, float rgbColor[3])
+void RenderPassEncoder::pushDebugGroup(const char* name, const MarkerColor& color)
 {
     if (m_commandList)
     {
         commands::PushDebugGroup cmd;
         cmd.name = name;
-        cmd.rgbColor[0] = rgbColor[0];
-        cmd.rgbColor[1] = rgbColor[1];
-        cmd.rgbColor[2] = rgbColor[2];
+        cmd.color = color;
         m_commandList->write(std::move(cmd));
     }
 }
@@ -159,15 +158,24 @@ void RenderPassEncoder::popDebugGroup()
     }
 }
 
-void RenderPassEncoder::insertDebugMarker(const char* name, float rgbColor[3])
+void RenderPassEncoder::insertDebugMarker(const char* name, const MarkerColor& color)
 {
     if (m_commandList)
     {
         commands::InsertDebugMarker cmd;
         cmd.name = name;
-        cmd.rgbColor[0] = rgbColor[0];
-        cmd.rgbColor[1] = rgbColor[1];
-        cmd.rgbColor[2] = rgbColor[2];
+        cmd.color = color;
+        m_commandList->write(std::move(cmd));
+    }
+}
+
+void RenderPassEncoder::writeTimestamp(IQueryPool* queryPool, uint32_t queryIndex)
+{
+    if (m_commandList)
+    {
+        commands::WriteTimestamp cmd;
+        cmd.queryPool = checked_cast<QueryPool*>(queryPool);
+        cmd.queryIndex = queryIndex;
         m_commandList->write(std::move(cmd));
     }
 }
@@ -203,7 +211,7 @@ void ComputePassEncoder::writeComputeState()
     commands::SetComputeState cmd;
     cmd.pipeline = m_pipeline;
     m_commandEncoder->getPipelineSpecializationArgs(m_pipeline, m_rootObject, cmd.specializationArgs);
-    if (!SLANG_SUCCEEDED(m_commandEncoder->getBindingData(m_rootObject, cmd.bindingData)))
+    if (SLANG_FAILED(m_commandEncoder->getBindingData(m_rootObject, cmd.bindingData)))
     {
         m_commandEncoder->getDevice()
             ->handleMessage(DebugMessageType::Error, DebugMessageSource::Layer, "Failed to get binding data");
@@ -218,7 +226,7 @@ IShaderObject* ComputePassEncoder::bindPipeline(IComputePipeline* pipeline)
     {
         m_pipeline = pipeline;
         ShaderProgram* program = checked_cast<ShaderProgram*>(pipeline->getProgram());
-        if (!SLANG_SUCCEEDED(m_commandEncoder->getDevice()->createRootShaderObject(program, m_rootObject.writeRef())))
+        if (SLANG_FAILED(m_commandEncoder->getDevice()->createRootShaderObject(program, m_rootObject.writeRef())))
             return nullptr;
         return m_rootObject;
     }
@@ -258,15 +266,13 @@ void ComputePassEncoder::dispatchComputeIndirect(BufferOffsetPair argBuffer)
     }
 }
 
-void ComputePassEncoder::pushDebugGroup(const char* name, float rgbColor[3])
+void ComputePassEncoder::pushDebugGroup(const char* name, const MarkerColor& color)
 {
     if (m_commandList)
     {
         commands::PushDebugGroup cmd;
         cmd.name = name;
-        cmd.rgbColor[0] = rgbColor[0];
-        cmd.rgbColor[1] = rgbColor[1];
-        cmd.rgbColor[2] = rgbColor[2];
+        cmd.color = color;
         m_commandList->write(std::move(cmd));
     }
 }
@@ -280,15 +286,24 @@ void ComputePassEncoder::popDebugGroup()
     }
 }
 
-void ComputePassEncoder::insertDebugMarker(const char* name, float rgbColor[3])
+void ComputePassEncoder::insertDebugMarker(const char* name, const MarkerColor& color)
 {
     if (m_commandList)
     {
         commands::InsertDebugMarker cmd;
         cmd.name = name;
-        cmd.rgbColor[0] = rgbColor[0];
-        cmd.rgbColor[1] = rgbColor[1];
-        cmd.rgbColor[2] = rgbColor[2];
+        cmd.color = color;
+        m_commandList->write(std::move(cmd));
+    }
+}
+
+void ComputePassEncoder::writeTimestamp(IQueryPool* queryPool, uint32_t queryIndex)
+{
+    if (m_commandList)
+    {
+        commands::WriteTimestamp cmd;
+        cmd.queryPool = checked_cast<QueryPool*>(queryPool);
+        cmd.queryIndex = queryIndex;
         m_commandList->write(std::move(cmd));
     }
 }
@@ -325,7 +340,7 @@ void RayTracingPassEncoder::writeRayTracingState()
     cmd.pipeline = m_pipeline;
     cmd.shaderTable = m_shaderTable;
     m_commandEncoder->getPipelineSpecializationArgs(m_pipeline, m_rootObject, cmd.specializationArgs);
-    if (!SLANG_SUCCEEDED(m_commandEncoder->getBindingData(m_rootObject, cmd.bindingData)))
+    if (SLANG_FAILED(m_commandEncoder->getBindingData(m_rootObject, cmd.bindingData)))
     {
         m_commandEncoder->getDevice()
             ->handleMessage(DebugMessageType::Error, DebugMessageSource::Layer, "Failed to get binding data");
@@ -342,7 +357,7 @@ IShaderObject* RayTracingPassEncoder::bindPipeline(IRayTracingPipeline* pipeline
         m_pipeline = pipeline;
         m_shaderTable = shaderTable;
         ShaderProgram* program = checked_cast<ShaderProgram*>(pipeline->getProgram());
-        if (!SLANG_SUCCEEDED(m_commandEncoder->getDevice()->createRootShaderObject(program, m_rootObject.writeRef())))
+        if (SLANG_FAILED(m_commandEncoder->getDevice()->createRootShaderObject(program, m_rootObject.writeRef())))
             return nullptr;
         return m_rootObject;
     }
@@ -377,15 +392,13 @@ void RayTracingPassEncoder::dispatchRays(uint32_t rayGenShaderIndex, uint32_t wi
     }
 }
 
-void RayTracingPassEncoder::pushDebugGroup(const char* name, float rgbColor[3])
+void RayTracingPassEncoder::pushDebugGroup(const char* name, const MarkerColor& color)
 {
     if (m_commandList)
     {
         commands::PushDebugGroup cmd;
         cmd.name = name;
-        cmd.rgbColor[0] = rgbColor[0];
-        cmd.rgbColor[1] = rgbColor[1];
-        cmd.rgbColor[2] = rgbColor[2];
+        cmd.color = color;
         m_commandList->write(std::move(cmd));
     }
 }
@@ -399,15 +412,24 @@ void RayTracingPassEncoder::popDebugGroup()
     }
 }
 
-void RayTracingPassEncoder::insertDebugMarker(const char* name, float rgbColor[3])
+void RayTracingPassEncoder::insertDebugMarker(const char* name, const MarkerColor& color)
 {
     if (m_commandList)
     {
         commands::InsertDebugMarker cmd;
         cmd.name = name;
-        cmd.rgbColor[0] = rgbColor[0];
-        cmd.rgbColor[1] = rgbColor[1];
-        cmd.rgbColor[2] = rgbColor[2];
+        cmd.color = color;
+        m_commandList->write(std::move(cmd));
+    }
+}
+
+void RayTracingPassEncoder::writeTimestamp(IQueryPool* queryPool, uint32_t queryIndex)
+{
+    if (m_commandList)
+    {
+        commands::WriteTimestamp cmd;
+        cmd.queryPool = checked_cast<QueryPool*>(queryPool);
+        cmd.queryIndex = queryIndex;
         m_commandList->write(std::move(cmd));
     }
 }
@@ -696,6 +718,13 @@ void CommandEncoder::clearTextureUint(ITexture* texture, SubresourceRange subres
     cmd.texture = texture;
     cmd.subresourceRange = checked_cast<Texture*>(texture)->resolveSubresourceRange(subresourceRange);
     ::memcpy(cmd.clearValue, clearValue, sizeof(cmd.clearValue));
+    // Different APIs/drivers appear to handle out-of-range clear values differently.
+    // To ensure consistent results we clamp the values here.
+    FormatConversionFuncs funcs = getFormatConversionFuncs(texture->getDesc().format);
+    if (funcs.clampIntFunc)
+    {
+        funcs.clampIntFunc(cmd.clearValue);
+    }
     m_commandList->write(std::move(cmd));
 }
 
@@ -705,6 +734,13 @@ void CommandEncoder::clearTextureSint(ITexture* texture, SubresourceRange subres
     cmd.texture = texture;
     cmd.subresourceRange = checked_cast<Texture*>(texture)->resolveSubresourceRange(subresourceRange);
     ::memcpy(cmd.clearValue, clearValue, sizeof(cmd.clearValue));
+    // Different APIs/drivers appear to handle out-of-range clear values differently.
+    // To ensure consistent results we clamp the values here.
+    FormatConversionFuncs funcs = getFormatConversionFuncs(texture->getDesc().format);
+    if (funcs.clampIntFunc)
+    {
+        funcs.clampIntFunc(cmd.clearValue);
+    }
     m_commandList->write(std::move(cmd));
 }
 
@@ -804,11 +840,27 @@ void CommandEncoder::deserializeAccelerationStructure(IAccelerationStructure* ds
     m_commandList->write(std::move(cmd));
 }
 
-void CommandEncoder::convertCooperativeVectorMatrix(const ConvertCooperativeVectorMatrixDesc* descs, uint32_t descCount)
+void CommandEncoder::executeClusterOperation(const ClusterOperationDesc& desc)
+{
+    commands::ExecuteClusterOperation cmd;
+    cmd.desc = desc;
+    m_commandList->write(std::move(cmd));
+}
+
+void CommandEncoder::convertCooperativeVectorMatrix(
+    IBuffer* dstBuffer,
+    const CooperativeVectorMatrixDesc* dstDescs,
+    IBuffer* srcBuffer,
+    const CooperativeVectorMatrixDesc* srcDescs,
+    uint32_t matrixCount
+)
 {
     commands::ConvertCooperativeVectorMatrix cmd;
-    cmd.descs = descs;
-    cmd.descCount = descCount;
+    cmd.dstBuffer = checked_cast<Buffer*>(dstBuffer);
+    cmd.dstDescs = dstDescs;
+    cmd.srcBuffer = checked_cast<Buffer*>(srcBuffer);
+    cmd.srcDescs = srcDescs;
+    cmd.matrixCount = matrixCount;
     m_commandList->write(std::move(cmd));
 }
 
@@ -829,13 +881,18 @@ void CommandEncoder::setTextureState(ITexture* texture, SubresourceRange subreso
     m_commandList->write(std::move(cmd));
 }
 
-void CommandEncoder::pushDebugGroup(const char* name, float rgbColor[3])
+void CommandEncoder::globalBarrier()
+{
+    commands::GlobalBarrier cmd;
+    m_commandList->write(std::move(cmd));
+}
+
+
+void CommandEncoder::pushDebugGroup(const char* name, const MarkerColor& color)
 {
     commands::PushDebugGroup cmd;
     cmd.name = name;
-    cmd.rgbColor[0] = rgbColor[0];
-    cmd.rgbColor[1] = rgbColor[1];
-    cmd.rgbColor[2] = rgbColor[2];
+    cmd.color = color;
     m_commandList->write(std::move(cmd));
 }
 
@@ -845,13 +902,11 @@ void CommandEncoder::popDebugGroup()
     m_commandList->write(std::move(cmd));
 }
 
-void CommandEncoder::insertDebugMarker(const char* name, float rgbColor[3])
+void CommandEncoder::insertDebugMarker(const char* name, const MarkerColor& color)
 {
     commands::InsertDebugMarker cmd;
     cmd.name = name;
-    cmd.rgbColor[0] = rgbColor[0];
-    cmd.rgbColor[1] = rgbColor[1];
-    cmd.rgbColor[2] = rgbColor[2];
+    cmd.color = color;
     m_commandList->write(std::move(cmd));
 }
 
