@@ -1,9 +1,9 @@
 #pragma once
 
-namespace rhi {
-
 #include <unordered_map>
 
+namespace rhi {
+
 /// Given a mapping function, create a reverse map from To to From.
 /// The mapping function func must be bijective.
 /// The reverse map will return defaultValue if the value is not found.
@@ -15,7 +15,9 @@ auto reverseMap(Func func, From min, From max, From defaultValue = From(0))
         std::unordered_map<To, From> map;
         for (int i = int(min); i <= int(max); i++)
         {
-            map[func(From(i))] = From(i);
+            const From fromI = From(i);
+            const To key = func(fromI); // Fixed MSVC warning C4709: comma operator within a subscript expression
+            map[key] = fromI;
         }
         return map;
     }();
