@@ -9,7 +9,8 @@ class ShaderProgramImpl : public ShaderProgram
 public:
     RefPtr<RootShaderObjectLayoutImpl> m_rootShaderObjectLayout;
 
-    ~ShaderProgramImpl() {}
+    ShaderProgramImpl(Device* device, const ShaderProgramDesc& desc);
+    ~ShaderProgramImpl();
 
     virtual ShaderObjectLayout* getRootShaderObjectLayout() override;
 };
