@@ -1,6 +1,7 @@
 #pragma once
 
 #include "cuda-api.h"
+#include "optix-api.h"
 
 #include "../rhi-shared.h"
 #include "../slang-context.h"
@@ -13,9 +14,11 @@
 
 namespace rhi::cuda {
 
+class AdapterImpl;
 class BufferImpl;
 class TextureImpl;
 class TextureViewImpl;
+class SamplerImpl;
 class ShaderObjectLayoutImpl;
 class RootShaderObjectLayoutImpl;
 class ShaderProgramImpl;
@@ -28,6 +31,7 @@ class CommandEncoderImpl;
 class CommandQueueImpl;
 class AccelerationStructureImpl;
 class ShaderTableImpl;
+class HeapImpl;
 struct BindingDataImpl;
 struct BindingCache;
 
