@@ -7,6 +7,7 @@
 #include "cuda-acceleration-structure.h"
 #include "cuda-shader-table.h"
 #include "cuda-shader-object.h"
+#include "cuda-utils.h"
 #include "../command-list.h"
 #include "../strings.h"
 
@@ -22,13 +23,11 @@ public:
     bool m_computeStateValid = false;
     RefPtr<ComputePipelineImpl> m_computePipeline;
 
-#if SLANG_RHI_ENABLE_OPTIX
     bool m_rayTracingPassActive = false;
     bool m_rayTracingStateValid = false;
     RefPtr<RayTracingPipelineImpl> m_rayTracingPipeline;
     RefPtr<ShaderTableImpl> m_shaderTable;
-    ShaderTableImpl::Instance* m_shaderTableInstance = nullptr;
-#endif
+    optix::ShaderBindingTable* m_shaderBindingTable = nullptr;
 
     BindingDataImpl* m_bindingData = nullptr;
 
@@ -71,9 +70,11 @@ public:
     void cmdQueryAccelerationStructureProperties(const commands::QueryAccelerationStructureProperties& cmd);
     void cmdSerializeAccelerationStructure(const commands::SerializeAccelerationStructure& cmd);
     void cmdDeserializeAccelerationStructure(const commands::DeserializeAccelerationStructure& cmd);
+    void cmdExecuteClusterOperation(const commands::ExecuteClusterOperation& cmd);
     void cmdConvertCooperativeVectorMatrix(const commands::ConvertCooperativeVectorMatrix& cmd);
     void cmdSetBufferState(const commands::SetBufferState& cmd);
     void cmdSetTextureState(const commands::SetTextureState& cmd);
+    void cmdGlobalBarrier(const commands::GlobalBarrier& cmd);
     void cmdPushDebugGroup(const commands::PushDebugGroup& cmd);
     void cmdPopDebugGroup(const commands::PopDebugGroup& cmd);
     void cmdInsertDebugMarker(const commands::InsertDebugMarker& cmd);
@@ -113,7 +114,7 @@ Result CommandExecutor::execute(CommandBufferImpl* commandBuffer)
     return SLANG_OK;
 }
 
-#define NOT_SUPPORTED(x) m_device->warning(x " command is not supported!")
+#define NOT_SUPPORTED(x) m_device->printWarning(x " command is not supported!")
 
 void CommandExecutor::cmdCopyBuffer(const commands::CopyBuffer& cmd)
 {
@@ -198,18 +199,18 @@ void CommandExecutor::cmdCopyTexture(const commands::CopyTexture& cmd)
             CUDA_MEMCPY3D copyParam = {};
             copyParam.srcMemoryType = CU_MEMORYTYPE_ARRAY;
             copyParam.srcArray = srcArray;
-            copyParam.srcXInBytes = srcOffset.x * formatInfo.blockSizeInBytes;
-            copyParam.srcY = srcOffset.y;
+            copyParam.srcXInBytes = widthInBlocks(formatInfo, srcOffset.x) * formatInfo.blockSizeInBytes;
+            copyParam.srcY = heightInBlocks(formatInfo, srcOffset.y);
             copyParam.srcZ = srcOffset.z + srcLayer;
 
             copyParam.dstMemoryType = CU_MEMORYTYPE_ARRAY;
             copyParam.dstArray = dstArray;
-            copyParam.dstXInBytes = dstOffset.x * formatInfo.blockSizeInBytes;
-            copyParam.dstY = dstOffset.y;
+            copyParam.dstXInBytes = widthInBlocks(formatInfo, dstOffset.x) * formatInfo.blockSizeInBytes;
+            copyParam.dstY = heightInBlocks(formatInfo, dstOffset.y);
             copyParam.dstZ = dstOffset.z + dstLayer;
 
-            copyParam.WidthInBytes = adjustedExtent.width * formatInfo.blockSizeInBytes;
-            copyParam.Height = adjustedExtent.height;
+            copyParam.WidthInBytes = widthInBlocks(formatInfo, adjustedExtent.width) * formatInfo.blockSizeInBytes;
+            copyParam.Height = heightInBlocks(formatInfo, adjustedExtent.height);
             copyParam.Depth = adjustedExtent.depth;
 
             SLANG_CUDA_ASSERT_ON_FAIL(cuMemcpy3D(&copyParam));
@@ -273,16 +274,16 @@ void CommandExecutor::cmdCopyTextureToBuffer(const commands::CopyTextureToBuffer
     CUDA_MEMCPY3D copyParam = {};
     copyParam.srcMemoryType = CU_MEMORYTYPE_ARRAY;
     copyParam.srcArray = srcArray;
-    copyParam.srcXInBytes = srcOffset.x * formatInfo.blockSizeInBytes;
-    copyParam.srcY = srcOffset.y;
+    copyParam.srcXInBytes = widthInBlocks(formatInfo, srcOffset.x) * formatInfo.blockSizeInBytes;
+    copyParam.srcY = heightInBlocks(formatInfo, srcOffset.y);
     copyParam.srcZ = z;
 
     copyParam.dstMemoryType = CU_MEMORYTYPE_DEVICE;
     copyParam.dstDevice = (CUdeviceptr)((uint8_t*)dst->m_cudaMemory + dstOffset);
     copyParam.dstPitch = dstRowPitch;
 
-    copyParam.WidthInBytes = adjustedExtent.width * formatInfo.blockSizeInBytes;
-    copyParam.Height = adjustedExtent.height;
+    copyParam.WidthInBytes = widthInBlocks(formatInfo, adjustedExtent.width) * formatInfo.blockSizeInBytes;
+    copyParam.Height = heightInBlocks(formatInfo, adjustedExtent.height);
     copyParam.Depth = adjustedExtent.depth;
 
     SLANG_CUDA_ASSERT_ON_FAIL(cuMemcpy3D(&copyParam));
@@ -339,14 +340,14 @@ void CommandExecutor::cmdUploadTextureData(const commands::UploadTextureData& cm
             CUDA_MEMCPY3D copyParam = {};
             copyParam.dstMemoryType = CU_MEMORYTYPE_ARRAY;
             copyParam.dstArray = dstArray;
-            copyParam.dstXInBytes = cmd.offset.x * formatInfo.blockSizeInBytes;
-            copyParam.dstY = cmd.offset.y;
+            copyParam.dstXInBytes = widthInBlocks(formatInfo, cmd.offset.x) * formatInfo.blockSizeInBytes;
+            copyParam.dstY = heightInBlocks(formatInfo, cmd.offset.y);
             copyParam.dstZ = cmd.offset.z + layer;
             copyParam.srcMemoryType = CU_MEMORYTYPE_DEVICE;
             copyParam.srcDevice = (CUdeviceptr)((uint8_t*)buffer->m_cudaMemory + bufferOffset);
             copyParam.srcPitch = srLayout->rowPitch;
-            copyParam.WidthInBytes = srLayout->size.width * formatInfo.blockSizeInBytes;
-            copyParam.Height = srLayout->size.height;
+            copyParam.WidthInBytes = widthInBlocks(formatInfo, srLayout->size.width) * formatInfo.blockSizeInBytes;
+            copyParam.Height = heightInBlocks(formatInfo, srLayout->size.height);
             copyParam.Depth = srLayout->size.depth;
             SLANG_CUDA_ASSERT_ON_FAIL(cuMemcpy3D(&copyParam));
 
@@ -436,43 +437,45 @@ void CommandExecutor::cmdDispatchCompute(const commands::DispatchCompute& cmd)
     ComputePipelineImpl* computePipeline = m_computePipeline;
     BindingDataImpl* bindingData = m_bindingData;
 
-    SLANG_RHI_ASSERT(computePipeline->m_kernelIndex < bindingData->entryPointCount);
-    const auto& entryPointData = bindingData->entryPoints[computePipeline->m_kernelIndex];
+    SLANG_RHI_ASSERT(computePipeline->m_entryPointIndex < bindingData->entryPointCount);
+    const auto& entryPointData = bindingData->entryPoints[computePipeline->m_entryPointIndex];
 
     // Copy global parameter data to the `SLANG_globalParams` symbol.
+    if (computePipeline->m_globalParams)
     {
-        CUdeviceptr globalParamsSymbol = 0;
-        size_t globalParamsSymbolSize = 0;
-        CUresult result = cuModuleGetGlobal(
-            &globalParamsSymbol,
-            &globalParamsSymbolSize,
-            computePipeline->m_module,
-            "SLANG_globalParams"
-        );
-        if (result == CUDA_SUCCESS)
+        // TODO: Slang sometimes computes the size of the global parameters layout incorrectly.
+        // Instead of the assert, we currently warn about this mismatch once.
+        // SLANG_RHI_ASSERT(computePipeline->m_globalParamsSize == bindingData->globalParamsSize);
+        if (computePipeline->m_globalParamsSize != bindingData->globalParamsSize &&
+            !computePipeline->m_warnedAboutGlobalParamsSizeMismatch)
         {
-            SLANG_RHI_ASSERT(globalParamsSymbolSize == bindingData->globalParamsSize);
-            SLANG_CUDA_ASSERT_ON_FAIL(
-                cuMemcpyAsync(globalParamsSymbol, bindingData->globalParams, globalParamsSymbolSize, m_stream)
+            m_device->printWarning(
+                "Warning: Incorrect global parameter size (expected %llu, got %llu) for pipeline %s",
+                computePipeline->m_globalParamsSize,
+                bindingData->globalParamsSize,
+                computePipeline->m_entryPointName.c_str()
             );
+            computePipeline->m_warnedAboutGlobalParamsSizeMismatch = true;
         }
+        SLANG_CUDA_ASSERT_ON_FAIL(cuMemcpyAsync(
+            computePipeline->m_globalParams,
+            bindingData->globalParams,
+            min(bindingData->globalParamsSize, computePipeline->m_globalParamsSize),
+            m_stream
+        ));
     }
-    //
+
     // The argument data for the entry-point parameters are already
     // stored in host memory, as expected by cuLaunchKernel.
-    //
-    SLANG_RHI_ASSERT(entryPointData.size >= computePipeline->m_paramBufferSize);
     void* extraOptions[] = {
         CU_LAUNCH_PARAM_BUFFER_POINTER,
         (void*)entryPointData.data,
         CU_LAUNCH_PARAM_BUFFER_SIZE,
-        (void*)&computePipeline->m_paramBufferSize,
+        (void*)&entryPointData.size,
         CU_LAUNCH_PARAM_END,
     };
 
-    // Once we have all the necessary data extracted and/or
-    // set up, we can launch the kernel and see what happens.
-    //
+    // Once we have all the necessary data extracted and/or set up, we can launch the kernel.
     SLANG_CUDA_ASSERT_ON_FAIL(cuLaunchKernel(
         computePipeline->m_function,
         cmd.x,
@@ -497,157 +500,75 @@ void CommandExecutor::cmdDispatchComputeIndirect(const commands::DispatchCompute
 void CommandExecutor::cmdBeginRayTracingPass(const commands::BeginRayTracingPass& cmd)
 {
     SLANG_UNUSED(cmd);
-#if SLANG_RHI_ENABLE_OPTIX
     m_rayTracingPassActive = true;
-#else
-    NOT_SUPPORTED(beginRayTracingPass);
-#endif
 }
 
 void CommandExecutor::cmdEndRayTracingPass(const commands::EndRayTracingPass& cmd)
 {
     SLANG_UNUSED(cmd);
-#if SLANG_RHI_ENABLE_OPTIX
     m_rayTracingPassActive = false;
-#else
-    NOT_SUPPORTED(endRayTracingPass);
-#endif
 }
 
 void CommandExecutor::cmdSetRayTracingState(const commands::SetRayTracingState& cmd)
 {
-#if SLANG_RHI_ENABLE_OPTIX
     if (!m_rayTracingPassActive)
         return;
 
     m_rayTracingPipeline = checked_cast<RayTracingPipelineImpl*>(cmd.pipeline);
     m_bindingData = static_cast<BindingDataImpl*>(cmd.bindingData);
     m_shaderTable = checked_cast<ShaderTableImpl*>(cmd.shaderTable);
-    m_shaderTableInstance = m_shaderTable ? m_shaderTable->getInstance(m_rayTracingPipeline) : nullptr;
+    m_shaderBindingTable = m_shaderTable ? m_shaderTable->getShaderBindingTable(m_rayTracingPipeline) : nullptr;
     m_rayTracingStateValid = m_rayTracingPipeline && m_bindingData && m_shaderTable;
-#else
-    SLANG_UNUSED(cmd);
-    NOT_SUPPORTED(setRayTracingState);
-#endif
 }
 
 void CommandExecutor::cmdDispatchRays(const commands::DispatchRays& cmd)
 {
-#if SLANG_RHI_ENABLE_OPTIX
     if (!m_rayTracingStateValid)
         return;
 
+    if (!m_device->m_ctx.optixContext)
+        return;
+
     BindingDataImpl* bindingData = m_bindingData;
-
-    OptixShaderBindingTable sbt = m_shaderTableInstance->sbt;
-    sbt.raygenRecord += cmd.rayGenShaderIndex * m_shaderTableInstance->raygenRecordSize;
-
-    SLANG_OPTIX_ASSERT_ON_FAIL(optixLaunch(
-        m_rayTracingPipeline->m_pipeline,
+    m_device->m_ctx.optixContext->dispatchRays(
         m_stream,
-        bindingData->globalParams,
-        bindingData->globalParamsSize,
-        &sbt,
+        m_rayTracingPipeline->m_optixPipeline,
+        bindingData,
+        m_shaderBindingTable,
+        cmd.rayGenShaderIndex,
         cmd.width,
         cmd.height,
         cmd.depth
-    ));
-#else
-    SLANG_UNUSED(cmd);
-    NOT_SUPPORTED(dispatchRays);
-#endif
+    );
 }
 
 void CommandExecutor::cmdBuildAccelerationStructure(const commands::BuildAccelerationStructure& cmd)
 {
-#if SLANG_RHI_ENABLE_OPTIX
-    AccelerationStructureBuildDescConverter converter;
-    if (converter.convert(cmd.desc, m_device->m_debugCallback) != SLANG_OK)
+    if (!m_device->m_ctx.optixContext)
         return;
 
-    AccelerationStructureImpl* dst = checked_cast<AccelerationStructureImpl*>(cmd.dst);
-
-    short_vector<OptixAccelEmitDesc, 8> emittedProperties;
-    for (uint32_t i = 0; i < cmd.propertyQueryCount; i++)
-    {
-        if (cmd.queryDescs[i].queryType == QueryType::AccelerationStructureCompactedSize)
-        {
-            PlainBufferProxyQueryPoolImpl* queryPool =
-                checked_cast<PlainBufferProxyQueryPoolImpl*>(cmd.queryDescs[i].queryPool);
-            OptixAccelEmitDesc property = {};
-            property.type = OPTIX_PROPERTY_TYPE_COMPACTED_SIZE;
-            property.result = queryPool->m_buffer + cmd.queryDescs[i].firstQueryIndex * sizeof(uint64_t);
-            emittedProperties.push_back(property);
-        }
-    }
-
-    SLANG_OPTIX_ASSERT_ON_FAIL(optixAccelBuild(
-        m_device->m_ctx.optixContext,
+    m_device->m_ctx.optixContext->buildAccelerationStructure(
         m_stream,
-        &converter.buildOptions,
-        converter.buildInputs.data(),
-        converter.buildInputs.size(),
-        cmd.scratchBuffer.getDeviceAddress(),
-        checked_cast<BufferImpl*>(cmd.scratchBuffer.buffer)->m_desc.size - cmd.scratchBuffer.offset,
-        dst->m_buffer,
-        dst->m_desc.size,
-        &dst->m_handle,
-        emittedProperties.empty() ? nullptr : emittedProperties.data(),
-        emittedProperties.size()
-    ));
-#else  // SLANG_RHI_ENABLE_OPTIX
-    SLANG_UNUSED(cmd);
-    NOT_SUPPORTED(buildAccelerationStructure);
-#endif // SLANG_RHI_ENABLE_OPTIX
+        cmd.desc,
+        checked_cast<AccelerationStructureImpl*>(cmd.dst),
+        checked_cast<AccelerationStructureImpl*>(cmd.src),
+        cmd.scratchBuffer,
+        cmd.propertyQueryCount,
+        cmd.queryDescs
+    );
 }
 
 void CommandExecutor::cmdCopyAccelerationStructure(const commands::CopyAccelerationStructure& cmd)
 {
-#if SLANG_RHI_ENABLE_OPTIX
-    AccelerationStructureImpl* dst = checked_cast<AccelerationStructureImpl*>(cmd.dst);
-    AccelerationStructureImpl* src = checked_cast<AccelerationStructureImpl*>(cmd.src);
+    if (!m_device->m_ctx.optixContext)
+        return;
 
-    switch (cmd.mode)
-    {
-    case AccelerationStructureCopyMode::Clone:
-    {
-#if 0
-                OptixRelocationInfo relocInfo = {};
-                optixAccelGetRelocationInfo(m_commandBuffer->m_device->m_ctx.optixContext, src->m_handle, &relocInfo);
-
-                // TODO setup inputs
-                OptixRelocateInput relocInput = {};
-
-                cuMemcpyDtoD(dst->m_buffer, src->m_buffer, src->m_desc.size);
-
-                optixAccelRelocate(
-                    m_commandBuffer->m_device->m_ctx.optixContext,
-                    m_stream,
-                    &relocInfo,
-                    &relocInput,
-                    1,
-                    dst->m_buffer,
-                    dst->m_desc.size,
-                    &dst->m_handle
-                );
-                break;
-#endif
-    }
-    case AccelerationStructureCopyMode::Compact:
-        SLANG_OPTIX_ASSERT_ON_FAIL(optixAccelCompact(
-            m_device->m_ctx.optixContext,
-            m_stream,
-            src->m_handle,
-            dst->m_buffer,
-            dst->m_desc.size,
-            &dst->m_handle
-        ));
-        break;
-    }
-#else  // SLANG_RHI_ENABLE_OPTIX
-    SLANG_UNUSED(cmd);
-    NOT_SUPPORTED(copyAccelerationStructure);
-#endif // SLANG_RHI_ENABLE_OPTIX
+    m_device->m_ctx.optixContext->copyAccelerationStructure(
+        m_stream,
+        checked_cast<AccelerationStructureImpl*>(cmd.dst),
+        checked_cast<AccelerationStructureImpl*>(cmd.src),
+        cmd.mode
+    );
 }
 
 void CommandExecutor::cmdQueryAccelerationStructureProperties(const commands::QueryAccelerationStructureProperties& cmd)
@@ -668,10 +589,27 @@ void CommandExecutor::cmdDeserializeAccelerationStructure(const commands::Deseri
     NOT_SUPPORTED(S_CommandEncoder_deserializeAccelerationStructure);
 }
 
+void CommandExecutor::cmdExecuteClusterOperation(const commands::ExecuteClusterOperation& cmd)
+{
+    if (!m_device->m_ctx.optixContext)
+        return;
+
+    m_device->m_ctx.optixContext->executeClusterOperation(m_stream, cmd.desc);
+}
+
 void CommandExecutor::cmdConvertCooperativeVectorMatrix(const commands::ConvertCooperativeVectorMatrix& cmd)
 {
-    SLANG_UNUSED(cmd);
-    NOT_SUPPORTED(S_CommandEncoder_convertCooperativeVectorMatrix);
+    if (!m_device->m_ctx.optixContext)
+        return;
+
+    m_device->m_ctx.optixContext->convertCooperativeVectorMatrix(
+        m_stream,
+        cmd.dstBuffer->getDeviceAddress(),
+        cmd.dstDescs,
+        cmd.srcBuffer->getDeviceAddress(),
+        cmd.srcDescs,
+        cmd.matrixCount
+    );
 }
 
 void CommandExecutor::cmdSetBufferState(const commands::SetBufferState& cmd)
@@ -684,6 +622,11 @@ void CommandExecutor::cmdSetTextureState(const commands::SetTextureState& cmd)
     SLANG_UNUSED(cmd);
 }
 
+void CommandExecutor::cmdGlobalBarrier(const commands::GlobalBarrier& cmd)
+{
+    SLANG_UNUSED(cmd);
+}
+
 void CommandExecutor::cmdPushDebugGroup(const commands::PushDebugGroup& cmd)
 {
     SLANG_UNUSED(cmd);
@@ -715,27 +658,179 @@ void CommandExecutor::cmdExecuteCallback(const commands::ExecuteCallback& cmd)
 CommandQueueImpl::CommandQueueImpl(Device* device, QueueType type)
     : CommandQueue(device, type)
 {
-    SLANG_CUDA_ASSERT_ON_FAIL(cuStreamCreate(&m_stream, 0));
 }
 
 CommandQueueImpl::~CommandQueueImpl()
 {
-    SLANG_CUDA_ASSERT_ON_FAIL(cuStreamSynchronize(m_stream));
-    SLANG_CUDA_ASSERT_ON_FAIL(cuStreamDestroy(m_stream));
+    SLANG_CUDA_CTX_SCOPE(getDevice<DeviceImpl>());
+
+    // Block on all events completing
+    for (const auto& ev : m_submitEvents)
+    {
+        SLANG_CUDA_ASSERT_ON_FAIL(cuEventSynchronize(ev.event));
+    }
+
+    // Retire finished command buffers, which should be all of them
+    retireCommandBuffers();
+    SLANG_RHI_ASSERT(m_commandBuffersInFlight.empty());
+
+    // Sync/destroy the stream
+    if (m_stream)
+    {
+        SLANG_CUDA_ASSERT_ON_FAIL(cuStreamSynchronize(m_stream));
+        SLANG_CUDA_ASSERT_ON_FAIL(cuStreamDestroy(m_stream));
+    }
+}
+
+Result CommandQueueImpl::init()
+{
+    SLANG_CUDA_CTX_SCOPE(getDevice<DeviceImpl>());
+
+    // On CUDA, treat the graphics stream as the default stream, identified
+    // by a NULL ptr. When we support async compute queues on D3D/Vulkan,
+    // they will be equivalent to secondary, non-default streams in CUDA.
+    if (m_type == QueueType::Graphics)
+    {
+        m_stream = nullptr;
+    }
+    else
+    {
+        SLANG_RETURN_ON_FAIL(cuStreamCreate(&m_stream, 0));
+    }
+
+    return SLANG_OK;
+}
+
+Result CommandQueueImpl::createCommandBuffer(CommandBufferImpl** outCommandBuffer)
+{
+    RefPtr<CommandBufferImpl> commandBuffer = new CommandBufferImpl(m_device);
+    returnRefPtr(outCommandBuffer, commandBuffer);
+    return SLANG_OK;
+}
+
+Result CommandQueueImpl::getOrCreateCommandBuffer(CommandBufferImpl** outCommandBuffer)
+{
+    std::lock_guard<std::mutex> lock(m_mutex);
+    RefPtr<CommandBufferImpl> commandBuffer;
+    if (m_commandBuffersPool.empty())
+    {
+        SLANG_RETURN_ON_FAIL(createCommandBuffer(commandBuffer.writeRef()));
+    }
+    else
+    {
+        commandBuffer = m_commandBuffersPool.front();
+        m_commandBuffersPool.pop_front();
+        commandBuffer->setInternalReferenceCount(0);
+    }
+    returnRefPtr(outCommandBuffer, commandBuffer);
+    return SLANG_OK;
+}
+
+void CommandQueueImpl::retireCommandBuffer(CommandBufferImpl* commandBuffer)
+{
+    commandBuffer->reset();
+    {
+        std::lock_guard<std::mutex> lock(m_mutex);
+        m_commandBuffersPool.push_back(commandBuffer);
+        commandBuffer->setInternalReferenceCount(1);
+    }
+}
+
+Result CommandQueueImpl::retireCommandBuffers()
+{
+    // Run fence logic so m_lastFinishedID is up to date.
+    SLANG_RETURN_ON_FAIL(updateFence());
+
+    // Retire command buffers that're passed the submission ID
+    auto cbIt = m_commandBuffersInFlight.begin();
+    while (cbIt != m_commandBuffersInFlight.end())
+    {
+        RefPtr<CommandBufferImpl>& commandBuffer = *cbIt;
+        if (commandBuffer->m_submissionID > m_lastFinishedID)
+            break;
+
+        retireCommandBuffer(commandBuffer);
+        cbIt = m_commandBuffersInFlight.erase(cbIt);
+    }
+
+    // Flush all device heaps
+    SLANG_RETURN_ON_FAIL(getDevice<DeviceImpl>()->flushHeaps());
+
+    return SLANG_OK;
 }
 
 Result CommandQueueImpl::createCommandEncoder(ICommandEncoder** outEncoder)
 {
-    RefPtr<CommandEncoderImpl> encoder = new CommandEncoderImpl(m_device);
+    SLANG_CUDA_CTX_SCOPE(getDevice<DeviceImpl>());
+
+    RefPtr<CommandEncoderImpl> encoder = new CommandEncoderImpl(m_device, this);
     SLANG_RETURN_ON_FAIL(encoder->init());
     returnComPtr(outEncoder, encoder);
     return SLANG_OK;
 }
 
+Result CommandQueueImpl::signalFence(CUstream stream, uint64_t* outId)
+{
+    // Increment submit count
+    m_lastSubmittedID++;
+
+    // Record submission event so we can detect completion
+    SubmitEvent ev;
+    ev.submitID = m_lastSubmittedID;
+    SLANG_CUDA_RETURN_ON_FAIL(cuEventCreate(&ev.event, CU_EVENT_DISABLE_TIMING));
+    SLANG_CUDA_RETURN_ON_FAIL(cuEventRecord(ev.event, stream));
+    m_submitEvents.push_back(ev);
+
+    if (outId)
+        *outId = m_lastSubmittedID;
+    return SLANG_OK;
+}
+
+Result CommandQueueImpl::updateFence()
+{
+    // Iterate the submit events to update the last finished ID
+    auto submitIt = m_submitEvents.begin();
+    while (submitIt != m_submitEvents.end())
+    {
+        CUresult result = cuEventQuery(submitIt->event);
+        if (result == CUDA_SUCCESS)
+        {
+            // Event is complete.
+            // We aren't recycling, so all we have to do is destroy the event
+            SLANG_CUDA_ASSERT_ON_FAIL(cuEventDestroy(submitIt->event));
+            m_lastFinishedID = submitIt->submitID;
+
+            // Remove the event from the list.
+            submitIt = m_submitEvents.erase(submitIt);
+        }
+        else if (result == CUDA_ERROR_NOT_READY)
+        {
+            // Not ready means event hasn't been triggered yet, so it's still in-flight.
+            // As command buffers are ordered, this should mean that all subsequent events
+            // are also still in-flight, so we can stop checking.
+            break;
+        }
+        else
+        {
+            SLANG_CUDA_RETURN_ON_FAIL_REPORT(result, this);
+        }
+    }
+    return SLANG_OK;
+}
+
+
 Result CommandQueueImpl::submit(const SubmitDesc& desc)
 {
     SLANG_CUDA_CTX_SCOPE(getDevice<DeviceImpl>());
 
+    // Check if we need to retire command buffers that have completed.
+    retireCommandBuffers();
+
+    // Select either the queue's default stream or the stream
+    // specified in the descriptor,and switch to it for the scope
+    // of this submission.
+    CUstream requestedStream = desc.cudaStream == kInvalidCUDAStream ? m_stream : (CUstream)desc.cudaStream;
+
     // Wait for fences.
     for (uint32_t i = 0; i < desc.waitFenceCount; ++i)
     {
@@ -751,8 +846,18 @@ Result CommandQueueImpl::submit(const SubmitDesc& desc)
     // Execute command buffers.
     for (uint32_t i = 0; i < desc.commandBufferCount; i++)
     {
-        CommandExecutor executor(getDevice<DeviceImpl>(), m_stream);
-        SLANG_RETURN_ON_FAIL(executor.execute(checked_cast<CommandBufferImpl*>(desc.commandBuffers[i])));
+        // Get/execute the buffer.
+        CommandBufferImpl* commandBuffer = checked_cast<CommandBufferImpl*>(desc.commandBuffers[i]);
+        CommandExecutor executor(getDevice<DeviceImpl>(), requestedStream);
+        SLANG_RETURN_ON_FAIL(executor.execute(commandBuffer));
+
+        // Signal main fence
+        uint64_t submissionID;
+        SLANG_RETURN_ON_FAIL(signalFence(requestedStream, &submissionID));
+
+        // Record the command buffer + corresponding submit ID
+        commandBuffer->m_submissionID = submissionID;
+        m_commandBuffersInFlight.push_back(commandBuffer);
     }
 
     // Signal fences.
@@ -761,11 +866,6 @@ Result CommandQueueImpl::submit(const SubmitDesc& desc)
         SLANG_RETURN_ON_FAIL(desc.signalFences[i]->setCurrentValue(desc.signalFenceValues[i]));
     }
 
-    SLANG_CUDA_ASSERT_ON_FAIL(cuStreamSynchronize(m_stream));
-
-    for (uint32_t i = 0; i < desc.commandBufferCount; i++)
-        checked_cast<CommandBufferImpl*>(desc.commandBuffers[i])->reset();
-
     return SLANG_OK;
 }
 
@@ -773,8 +873,15 @@ Result CommandQueueImpl::waitOnHost()
 {
     SLANG_CUDA_CTX_SCOPE(getDevice<DeviceImpl>());
 
-    SLANG_CUDA_RETURN_ON_FAIL(cuStreamSynchronize(m_stream));
-    SLANG_CUDA_RETURN_ON_FAIL(cuCtxSynchronize());
+    SLANG_CUDA_RETURN_ON_FAIL_REPORT(cuStreamSynchronize(m_stream), this);
+    SLANG_CUDA_RETURN_ON_FAIL_REPORT(cuCtxSynchronize(), this);
+
+    // Retire command buffers that have completed.
+    retireCommandBuffers();
+
+    // If there's any left, it represents a bug in slang-rhi
+    SLANG_RHI_ASSERT(m_commandBuffersInFlight.empty());
+
     return SLANG_OK;
 }
 
@@ -787,21 +894,38 @@ Result CommandQueueImpl::getNativeHandle(NativeHandle* outHandle)
 
 // CommandEncoderImpl
 
-CommandEncoderImpl::CommandEncoderImpl(Device* device)
+CommandEncoderImpl::CommandEncoderImpl(Device* device, CommandQueueImpl* queue)
     : CommandEncoder(device)
+    , m_queue(queue)
 {
 }
 
+CommandEncoderImpl::~CommandEncoderImpl()
+{
+    SLANG_CUDA_CTX_SCOPE(getDevice<DeviceImpl>());
+
+    // If the command buffer was not used, return it to the pool.
+    if (m_commandBuffer)
+    {
+        m_queue->retireCommandBuffer(m_commandBuffer);
+    }
+}
+
 Result CommandEncoderImpl::init()
 {
-    m_commandBuffer = new CommandBufferImpl(m_device);
+    SLANG_CUDA_CTX_SCOPE(getDevice<DeviceImpl>());
+
+    SLANG_RETURN_ON_FAIL(m_queue->getOrCreateCommandBuffer(m_commandBuffer.writeRef()));
     m_commandList = &m_commandBuffer->m_commandList;
     return SLANG_OK;
 }
 
 Result CommandEncoderImpl::getBindingData(RootShaderObject* rootObject, BindingData*& outBindingData)
 {
+    SLANG_CUDA_CTX_SCOPE(getDevice<DeviceImpl>());
+
     rootObject->trackResources(m_commandBuffer->m_trackedObjects);
+
     BindingDataBuilder builder;
     builder.m_device = getDevice<DeviceImpl>();
     builder.m_bindingCache = &m_commandBuffer->m_bindingCache;
@@ -838,11 +962,13 @@ Result CommandEncoderImpl::getNativeHandle(NativeHandle* outHandle)
 CommandBufferImpl::CommandBufferImpl(Device* device)
     : CommandBuffer(device)
 {
+    SLANG_CUDA_CTX_SCOPE(getDevice<DeviceImpl>());
     m_constantBufferPool.init(checked_cast<DeviceImpl*>(device));
 }
 
 Result CommandBufferImpl::reset()
 {
+    SLANG_CUDA_CTX_SCOPE(getDevice<DeviceImpl>());
     m_bindingCache.reset();
     m_constantBufferPool.reset();
     return CommandBuffer::reset();
