@@ -18,10 +18,13 @@ public:
 
     std::vector<Module> m_modules;
 
+    ShaderProgramImpl(Device* device, const ShaderProgramDesc& desc);
     ~ShaderProgramImpl();
 
-    virtual Result createShaderModule(slang::EntryPointReflection* entryPointInfo, ComPtr<ISlangBlob> kernelCode)
-        override;
+    virtual Result createShaderModule(
+        slang::EntryPointReflection* entryPointInfo,
+        ComPtr<ISlangBlob> kernelCode
+    ) override;
 
     virtual ShaderObjectLayout* getRootShaderObjectLayout() override;
 };
