@@ -4,14 +4,18 @@
 #include "cuda-device.h"
 #include "cuda-texture.h"
 #include "cuda-command.h"
+#include "cuda-utils.h"
 
 #include "vulkan/vk-api.h"
-#include "vulkan/vk-util.h"
-#include "vulkan/vk-helper-functions.h"
+#include "vulkan/vk-utils.h"
 
 #include "core/reverse-map.h"
 #include "core/short_vector.h"
 
+#if SLANG_WINDOWS_FAMILY
+#include <dxgi1_4.h>
+#endif
+
 #if !SLANG_WINDOWS_FAMILY
 #include <unistd.h>
 #endif
@@ -81,7 +85,6 @@ public:
     uint32_t m_currentFrameIndex = 0;
     short_vector<VkImage> m_swapchainImages;
     uint32_t m_currentSwapchainImageIndex = -1;
-    bool m_configured = false;
 
 public:
     ~SurfaceImpl();
@@ -101,15 +104,17 @@ public:
     void destroySharedTexture(SharedTexture& sharedTexture);
 
     virtual SLANG_NO_THROW Result SLANG_MCALL configure(const SurfaceConfig& config) override;
+    virtual SLANG_NO_THROW Result SLANG_MCALL unconfigure() override;
     virtual SLANG_NO_THROW Result SLANG_MCALL acquireNextImage(ITexture** outTexture) override;
     virtual SLANG_NO_THROW Result SLANG_MCALL present() override;
 };
 
-static auto translateVkFormat =
-    reverseMap<Format, VkFormat>(vk::VulkanUtil::getVkFormat, Format::Undefined, Format::_Count);
+static auto translateVkFormat = reverseMap<Format, VkFormat>(vk::getVkFormat, Format::Undefined, Format::_Count);
 
 SurfaceImpl::~SurfaceImpl()
 {
+    SLANG_CUDA_CTX_SCOPE(m_deviceImpl);
+
     destroySwapchain();
 
     if (m_surface)
@@ -135,6 +140,8 @@ Result SurfaceImpl::init(DeviceImpl* device, WindowHandle windowHandle)
     m_deviceImpl = device;
     m_windowHandle = windowHandle;
 
+    SLANG_CUDA_CTX_SCOPE(m_deviceImpl);
+
     SLANG_RETURN_ON_FAIL(createVulkanInstance());
 
     switch (windowHandle.type)
@@ -146,7 +153,8 @@ Result SurfaceImpl::init(DeviceImpl* device, WindowHandle windowHandle)
         surfaceCreateInfo.sType = VK_STRUCTURE_TYPE_WIN32_SURFACE_CREATE_INFO_KHR;
         surfaceCreateInfo.hinstance = ::GetModuleHandle(nullptr);
         surfaceCreateInfo.hwnd = (HWND)windowHandle.handleValues[0];
-        SLANG_VK_RETURN_ON_FAIL(m_api.vkCreateWin32SurfaceKHR(m_api.m_instance, &surfaceCreateInfo, nullptr, &m_surface)
+        SLANG_VK_RETURN_ON_FAIL(
+            m_api.vkCreateWin32SurfaceKHR(m_api.m_instance, &surfaceCreateInfo, nullptr, &m_surface)
         );
         break;
     }
@@ -157,7 +165,8 @@ Result SurfaceImpl::init(DeviceImpl* device, WindowHandle windowHandle)
         surfaceCreateInfo.sType = VK_STRUCTURE_TYPE_XLIB_SURFACE_CREATE_INFO_KHR;
         surfaceCreateInfo.dpy = (Display*)windowHandle.handleValues[0];
         surfaceCreateInfo.window = (Window)windowHandle.handleValues[1];
-        SLANG_VK_RETURN_ON_FAIL(m_api.vkCreateXlibSurfaceKHR(m_api.m_instance, &surfaceCreateInfo, nullptr, &m_surface)
+        SLANG_VK_RETURN_ON_FAIL(
+            m_api.vkCreateXlibSurfaceKHR(m_api.m_instance, &surfaceCreateInfo, nullptr, &m_surface)
         );
         break;
     }
@@ -177,11 +186,13 @@ Result SurfaceImpl::init(DeviceImpl* device, WindowHandle windowHandle)
     for (uint32_t i = 0; i < formatCount; ++i)
     {
         Format format = translateVkFormat(surfaceFormats[i].format);
+        // Skip BGR formats that are not supported by the CUDA backend.
+        if (format == Format::BGRA8Unorm || format == Format::BGRA8UnormSrgb || format == Format::BGRX8Unorm ||
+            format == Format::BGRX8UnormSrgb)
+            continue;
         if (format != Format::Undefined)
             m_supportedFormats.push_back(format);
-        // if (format == Format::BGRA8Unorm)
-        //     preferredFormat = format;
-        if (format == Format::RGBA8Unorm)
+        if (format == Format::RGBA8UnormSrgb)
             preferredFormat = format;
     }
     if (preferredFormat == Format::Undefined && !m_supportedFormats.empty())
@@ -190,8 +201,7 @@ Result SurfaceImpl::init(DeviceImpl* device, WindowHandle windowHandle)
     }
 
     m_info.preferredFormat = preferredFormat;
-    m_info.supportedUsage = TextureUsage::Present | TextureUsage::RenderTarget | TextureUsage::UnorderedAccess |
-                            TextureUsage::CopyDestination;
+    m_info.supportedUsage = TextureUsage::Present | TextureUsage::UnorderedAccess | TextureUsage::CopyDestination;
     m_info.formats = m_supportedFormats.data();
     m_info.formatCount = (uint32_t)m_supportedFormats.size();
 
@@ -200,7 +210,9 @@ Result SurfaceImpl::init(DeviceImpl* device, WindowHandle windowHandle)
 
 Result SurfaceImpl::createVulkanInstance()
 {
-    SLANG_RETURN_ON_FAIL(m_module.init(false));
+    SLANG_CUDA_CTX_SCOPE(m_deviceImpl);
+
+    SLANG_RETURN_ON_FAIL(m_module.init());
     SLANG_RETURN_ON_FAIL(m_api.initGlobalProcs(m_module));
 
     VkApplicationInfo applicationInfo = {VK_STRUCTURE_TYPE_APPLICATION_INFO};
@@ -260,6 +272,8 @@ Result SurfaceImpl::createVulkanInstance()
 
 Result SurfaceImpl::createVulkanDevice()
 {
+    SLANG_CUDA_CTX_SCOPE(m_deviceImpl);
+
     uint32_t physicalDeviceCount = 0;
     SLANG_VK_RETURN_ON_FAIL(m_api.vkEnumeratePhysicalDevices(m_instance, &physicalDeviceCount, nullptr));
 
@@ -383,6 +397,8 @@ Result SurfaceImpl::createVulkanDevice()
 
 Result SurfaceImpl::createSwapchain()
 {
+    SLANG_CUDA_CTX_SCOPE(m_deviceImpl);
+
     VkExtent2D imageExtent = {m_config.width, m_config.height};
 
     // It is necessary to query the caps -> otherwise the LunarG verification layer will
@@ -431,7 +447,7 @@ Result SurfaceImpl::createSwapchain()
         return SLANG_FAIL;
     }
 
-    VkFormat format = vk::VulkanUtil::getVkFormat(m_config.format);
+    VkFormat format = vk::getVkFormat(m_config.format);
     VkSwapchainKHR oldSwapchain = VK_NULL_HANDLE;
 
     VkSwapchainCreateInfoKHR swapchainDesc = {VK_STRUCTURE_TYPE_SWAPCHAIN_CREATE_INFO_KHR};
@@ -469,6 +485,8 @@ Result SurfaceImpl::createSwapchain()
 
 void SurfaceImpl::destroySwapchain()
 {
+    SLANG_CUDA_CTX_SCOPE(m_deviceImpl);
+
     if (m_queue != VK_NULL_HANDLE)
     {
         m_api.vkQueueWaitIdle(m_queue);
@@ -489,6 +507,8 @@ void SurfaceImpl::destroySwapchain()
 
 Result SurfaceImpl::createFrameData(FrameData& frameData)
 {
+    SLANG_CUDA_CTX_SCOPE(m_deviceImpl);
+
     frameData = {0};
 
     // Create command pool.
@@ -570,7 +590,10 @@ Result SurfaceImpl::createFrameData(FrameData& frameData)
         externalSemaphoreHandleDesc.type = CU_EXTERNAL_SEMAPHORE_HANDLE_TYPE_TIMELINE_SEMAPHORE_FD;
         externalSemaphoreHandleDesc.handle.fd = (int)frameData.sharedSemaphoreHandle.value;
 #endif
-        SLANG_CUDA_RETURN_ON_FAIL(cuImportExternalSemaphore(&frameData.cudaSemaphore, &externalSemaphoreHandleDesc));
+        SLANG_CUDA_RETURN_ON_FAIL_REPORT(
+            cuImportExternalSemaphore(&frameData.cudaSemaphore, &externalSemaphoreHandleDesc),
+            m_deviceImpl.get()
+        );
     }
 
     SLANG_RETURN_ON_FAIL(createSharedTexture(frameData.sharedTexture));
@@ -580,6 +603,8 @@ Result SurfaceImpl::createFrameData(FrameData& frameData)
 
 void SurfaceImpl::destroyFrameData(FrameData& frameData)
 {
+    SLANG_CUDA_CTX_SCOPE(m_deviceImpl);
+
     if (frameData.commandBuffer)
     {
         m_api.vkFreeCommandBuffers(m_device, frameData.commandPool, 1, &frameData.commandBuffer);
@@ -618,19 +643,22 @@ void SurfaceImpl::destroyFrameData(FrameData& frameData)
 
 Result SurfaceImpl::createSharedTexture(SharedTexture& sharedTexture)
 {
+    SLANG_CUDA_CTX_SCOPE(m_deviceImpl);
+
     VkImageCreateInfo imageInfo = {VK_STRUCTURE_TYPE_IMAGE_CREATE_INFO};
     imageInfo.imageType = VK_IMAGE_TYPE_2D;
     imageInfo.extent = VkExtent3D{m_config.width, m_config.height, 1};
     imageInfo.mipLevels = 1;
     imageInfo.arrayLayers = 1;
-    imageInfo.format = vk::VulkanUtil::getVkFormat(m_config.format);
+    imageInfo.format = vk::getVkFormat(m_config.format);
     imageInfo.tiling = VK_IMAGE_TILING_OPTIMAL;
     imageInfo.usage = VK_IMAGE_USAGE_TRANSFER_SRC_BIT | VK_IMAGE_USAGE_TRANSFER_DST_BIT;
     imageInfo.sharingMode = VK_SHARING_MODE_EXCLUSIVE;
     imageInfo.samples = VK_SAMPLE_COUNT_1_BIT;
     imageInfo.initialLayout = VK_IMAGE_LAYOUT_UNDEFINED;
 
-    VkExternalMemoryImageCreateInfo externalMemoryImageCreateInfo = {VK_STRUCTURE_TYPE_EXTERNAL_MEMORY_IMAGE_CREATE_INFO
+    VkExternalMemoryImageCreateInfo externalMemoryImageCreateInfo = {
+        VK_STRUCTURE_TYPE_EXTERNAL_MEMORY_IMAGE_CREATE_INFO
     };
     VkExternalMemoryHandleTypeFlags extMemoryHandleType =
 #if SLANG_WINDOWS_FAMILY
@@ -708,16 +736,14 @@ Result SurfaceImpl::createSharedTexture(SharedTexture& sharedTexture)
     // Create CUDA texture.
     TextureDesc textureDesc = {};
     textureDesc.type = TextureType::Texture2D;
-    textureDesc.memoryType = MemoryType::DeviceLocal;
-    textureDesc.usage =
-        TextureUsage::RenderTarget | TextureUsage::CopyDestination | TextureUsage::Present | TextureUsage::Shared;
-    textureDesc.defaultState = ResourceState::RenderTarget;
     textureDesc.size.width = m_config.width;
     textureDesc.size.height = m_config.height;
-    textureDesc.size.depth = 0;
+    textureDesc.size.depth = 1;
     textureDesc.arrayLength = 1;
     textureDesc.mipCount = 1;
     textureDesc.format = m_config.format;
+    textureDesc.usage = m_config.usage;
+    textureDesc.defaultState = ResourceState::Present;
     SLANG_RETURN_ON_FAIL(m_deviceImpl->createTextureFromSharedHandle(
         sharedTexture.sharedHandle,
         textureDesc,
@@ -730,6 +756,8 @@ Result SurfaceImpl::createSharedTexture(SharedTexture& sharedTexture)
 
 void SurfaceImpl::destroySharedTexture(SharedTexture& sharedTexture)
 {
+    SLANG_CUDA_CTX_SCOPE(m_deviceImpl);
+
     sharedTexture.cudaTexture = nullptr;
     if (sharedTexture.sharedHandle)
     {
@@ -751,6 +779,8 @@ void SurfaceImpl::destroySharedTexture(SharedTexture& sharedTexture)
 
 Result SurfaceImpl::configure(const SurfaceConfig& config)
 {
+    SLANG_CUDA_CTX_SCOPE(m_deviceImpl);
+
     setConfig(config);
 
     if (m_config.width == 0 || m_config.height == 0)
@@ -761,6 +791,10 @@ Result SurfaceImpl::configure(const SurfaceConfig& config)
     {
         m_config.format = m_info.preferredFormat;
     }
+    if (m_config.usage == TextureUsage::None)
+    {
+        m_config.usage = m_info.supportedUsage;
+    }
 
     m_configured = false;
     destroySwapchain();
@@ -770,8 +804,26 @@ Result SurfaceImpl::configure(const SurfaceConfig& config)
     return SLANG_OK;
 }
 
+Result SurfaceImpl::unconfigure()
+{
+    SLANG_CUDA_CTX_SCOPE(m_deviceImpl);
+
+    if (!m_configured)
+    {
+        return SLANG_OK;
+    }
+
+    m_configured = false;
+    destroySwapchain();
+    return SLANG_OK;
+}
+
 Result SurfaceImpl::acquireNextImage(ITexture** outTexture)
 {
+    SLANG_CUDA_CTX_SCOPE(m_deviceImpl);
+
+    *outTexture = nullptr;
+
     if (!m_configured)
     {
         return SLANG_FAIL;
@@ -794,7 +846,7 @@ Result SurfaceImpl::acquireNextImage(ITexture** outTexture)
         &m_currentSwapchainImageIndex
     );
 
-    if (result != VK_SUCCESS)
+    if (result != VK_SUCCESS && result != VK_SUBOPTIMAL_KHR)
     {
         return SLANG_FAIL;
     }
@@ -805,6 +857,8 @@ Result SurfaceImpl::acquireNextImage(ITexture** outTexture)
 
 Result SurfaceImpl::present()
 {
+    SLANG_CUDA_CTX_SCOPE(m_deviceImpl);
+
     if (!m_configured)
     {
         return SLANG_FAIL;
@@ -819,6 +873,9 @@ Result SurfaceImpl::present()
     VkImage swapchainImage = m_swapchainImages[m_currentSwapchainImageIndex];
     VkImage sharedImage = frameData.sharedTexture.vulkanImage;
 
+    // On classic graphics devices surface presentation would syncronize with the graphics queue. This
+    // is emulated in CUDA by treating the default (NULL) CUDA stream as the graphics queue.
+
     // Signal semaphore on CUDA stream.
     // This would be the preferred way, but leads to Vulkan validation errors because
     // the validation layer cannot se the signal sent from the CUDA stream.
@@ -830,7 +887,8 @@ Result SurfaceImpl::present()
     }
 #endif
 
-    // Synchronize the CUDA stream and manually signal semaphore on Vulkan (avoid validation errors above).
+    // As the semaphore approach doesn't currently work, call cuStreamSynchronize, which blocks
+    // the host until the default CUDA stream is completely drained.
 #if 1
     {
         cuStreamSynchronize(m_deviceImpl->m_queue->m_stream);
@@ -1027,6 +1085,8 @@ Result SurfaceImpl::present()
 
 Result DeviceImpl::createSurface(WindowHandle windowHandle, ISurface** outSurface)
 {
+    SLANG_CUDA_CTX_SCOPE(this);
+
     RefPtr<SurfaceImpl> surface = new SurfaceImpl();
     SLANG_RETURN_ON_FAIL(surface->init(this, windowHandle));
     returnComPtr(outSurface, surface);
