@@ -6,7 +6,7 @@
 #include "d3d11-pipeline.h"
 #include "d3d11-shader-program.h"
 #include "d3d11-input-layout.h"
-#include "d3d11-helper-functions.h"
+#include "d3d11-utils.h"
 #include "d3d11-shader-object.h"
 #include "../command-list.h"
 #include "../strings.h"
@@ -82,9 +82,11 @@ public:
     void cmdQueryAccelerationStructureProperties(const commands::QueryAccelerationStructureProperties& cmd);
     void cmdSerializeAccelerationStructure(const commands::SerializeAccelerationStructure& cmd);
     void cmdDeserializeAccelerationStructure(const commands::DeserializeAccelerationStructure& cmd);
+    void cmdExecuteClusterOperation(const commands::ExecuteClusterOperation& cmd);
     void cmdConvertCooperativeVectorMatrix(const commands::ConvertCooperativeVectorMatrix& cmd);
     void cmdSetBufferState(const commands::SetBufferState& cmd);
     void cmdSetTextureState(const commands::SetTextureState& cmd);
+    void cmdGlobalBarrier(const commands::GlobalBarrier& cmd);
     void cmdPushDebugGroup(const commands::PushDebugGroup& cmd);
     void cmdPopDebugGroup(const commands::PopDebugGroup& cmd);
     void cmdInsertDebugMarker(const commands::InsertDebugMarker& cmd);
@@ -125,7 +127,7 @@ Result CommandExecutor::execute(CommandBufferImpl* commandBuffer)
     return SLANG_OK;
 }
 
-#define NOT_SUPPORTED(x) m_device->warning(x " command is not supported!")
+#define NOT_SUPPORTED(x) m_device->printWarning(x " command is not supported!")
 
 void CommandExecutor::cmdCopyBuffer(const commands::CopyBuffer& cmd)
 {
@@ -140,8 +142,139 @@ void CommandExecutor::cmdCopyBuffer(const commands::CopyBuffer& cmd)
 
 void CommandExecutor::cmdCopyTexture(const commands::CopyTexture& cmd)
 {
-    SLANG_UNUSED(cmd);
-    NOT_SUPPORTED(S_CommandEncoder_copyTexture);
+    TextureImpl* dst = checked_cast<TextureImpl*>(cmd.dst);
+    TextureImpl* src = checked_cast<TextureImpl*>(cmd.src);
+
+    SubresourceRange dstSubresource = cmd.dstSubresource;
+    const Offset3D& dstOffset = cmd.dstOffset;
+    SubresourceRange srcSubresource = cmd.srcSubresource;
+    const Offset3D& srcOffset = cmd.srcOffset;
+    const Extent3D& extent = cmd.extent;
+
+    // Fast path for copying whole resource.
+    if (dstSubresource.layerCount == 0 && dstSubresource.mipCount == 0 && srcSubresource.layerCount == 0 &&
+        srcSubresource.mipCount == 0 && srcOffset.isZero() && dstOffset.isZero() && extent.isWholeTexture())
+    {
+        m_immediateContext->CopyResource(dst->m_resource, src->m_resource);
+        return;
+    }
+
+    // If we couldn't use the fast CopyResource path, need to ensure that the subresource ranges are valid.
+    if (dstSubresource.layerCount == 0)
+        dstSubresource.layerCount = dst->m_desc.getLayerCount();
+    if (dstSubresource.mipCount == 0)
+        dstSubresource.mipCount = dst->m_desc.mipCount;
+    if (srcSubresource.layerCount == 0)
+        srcSubresource.layerCount = src->m_desc.getLayerCount();
+    if (srcSubresource.mipCount == 0)
+        srcSubresource.mipCount = src->m_desc.mipCount;
+
+    // Validate subresource ranges
+    SLANG_RHI_ASSERT(srcSubresource.layer + srcSubresource.layerCount <= src->m_desc.getLayerCount());
+    SLANG_RHI_ASSERT(dstSubresource.layer + dstSubresource.layerCount <= dst->m_desc.getLayerCount());
+    SLANG_RHI_ASSERT(srcSubresource.mip + srcSubresource.mipCount <= src->m_desc.mipCount);
+    SLANG_RHI_ASSERT(dstSubresource.mip + dstSubresource.mipCount <= dst->m_desc.mipCount);
+
+    // Validate matching dimensions between source and destination
+    SLANG_RHI_ASSERT(srcSubresource.layerCount == dstSubresource.layerCount);
+    SLANG_RHI_ASSERT(srcSubresource.mipCount == dstSubresource.mipCount);
+
+    Extent3D srcTextureSize = src->m_desc.size;
+
+    uint32_t planeCount = getPlaneSliceCount(dst->m_format);
+    SLANG_RHI_ASSERT(planeCount == getPlaneSliceCount(src->m_format));
+    SLANG_RHI_ASSERT(planeCount > 0);
+
+    for (uint32_t planeIndex = 0; planeIndex < planeCount; planeIndex++)
+    {
+        for (uint32_t layer = 0; layer < dstSubresource.layerCount; layer++)
+        {
+            for (uint32_t mipOffset = 0; mipOffset < dstSubresource.mipCount; mipOffset++)
+            {
+                uint32_t srcMip = srcSubresource.mip + mipOffset;
+                uint32_t dstMip = dstSubresource.mip + mipOffset;
+
+                // Calculate adjusted extents. Note it is required and enforced
+                // by debug layer that if 'remaining texture' is used, src and
+                // dst offsets are the same.
+                Extent3D srcMipSize = calcMipSize(srcTextureSize, srcMip);
+                Extent3D adjustedExtent = extent;
+                if (adjustedExtent.width == kRemainingTextureSize)
+                {
+                    SLANG_RHI_ASSERT(srcOffset.x == dstOffset.x);
+                    adjustedExtent.width = srcMipSize.width - srcOffset.x;
+                }
+                if (adjustedExtent.height == kRemainingTextureSize)
+                {
+                    SLANG_RHI_ASSERT(srcOffset.y == dstOffset.y);
+                    adjustedExtent.height = srcMipSize.height - srcOffset.y;
+                }
+                if (adjustedExtent.depth == kRemainingTextureSize)
+                {
+                    SLANG_RHI_ASSERT(srcOffset.z == dstOffset.z);
+                    adjustedExtent.depth = srcMipSize.depth - srcOffset.z;
+                }
+
+                // Validate source and destination parameters
+                SLANG_RHI_ASSERT(srcOffset.x + adjustedExtent.width <= srcMipSize.width);
+                SLANG_RHI_ASSERT(srcOffset.y + adjustedExtent.height <= srcMipSize.height);
+                SLANG_RHI_ASSERT(srcOffset.z + adjustedExtent.depth <= srcMipSize.depth);
+                SLANG_RHI_ASSERT(srcMip < src->m_desc.mipCount);
+
+                UINT dstSubresourceIndex = getSubresourceIndex(
+                    dstMip,
+                    dstSubresource.layer + layer,
+                    planeIndex,
+                    dst->m_desc.mipCount,
+                    dst->m_desc.getLayerCount()
+                );
+                UINT srcSubresourceIndex = getSubresourceIndex(
+                    srcMip,
+                    srcSubresource.layer + layer,
+                    planeIndex,
+                    src->m_desc.mipCount,
+                    src->m_desc.getLayerCount()
+                );
+
+                if (srcOffset.isZero() && dstOffset.isZero() && adjustedExtent == srcMipSize)
+                {
+                    // If copying whole texture region, pass nullptr. This is required for
+                    // copying certain resources such as depth-stencil or multisampled textures.
+                    m_immediateContext->CopySubresourceRegion(
+                        dst->m_resource,
+                        dstSubresourceIndex,
+                        0,
+                        0,
+                        0,
+                        src->m_resource,
+                        srcSubresourceIndex,
+                        nullptr
+                    );
+                }
+                else
+                {
+                    D3D11_BOX srcBox = {};
+                    srcBox.left = srcOffset.x;
+                    srcBox.top = srcOffset.y;
+                    srcBox.front = srcOffset.z;
+                    srcBox.right = srcBox.left + adjustedExtent.width;
+                    srcBox.bottom = srcBox.top + adjustedExtent.height;
+                    srcBox.back = srcBox.front + adjustedExtent.depth;
+
+                    m_immediateContext->CopySubresourceRegion(
+                        dst->m_resource,
+                        dstSubresourceIndex,
+                        dstOffset.x,
+                        dstOffset.y,
+                        dstOffset.z,
+                        src->m_resource,
+                        srcSubresourceIndex,
+                        &srcBox
+                    );
+                }
+            }
+        }
+    }
 }
 
 void CommandExecutor::cmdCopyTextureToBuffer(const commands::CopyTextureToBuffer& cmd)
@@ -371,7 +504,9 @@ void CommandExecutor::cmdSetRenderState(const commands::SetRenderState& cmd)
     {
         m_renderPipeline = checked_cast<RenderPipelineImpl*>(cmd.pipeline);
 
-        m_immediateContext->IASetInputLayout(m_renderPipeline->m_inputLayout->m_layout);
+        m_immediateContext->IASetInputLayout(
+            m_renderPipeline->m_inputLayout ? m_renderPipeline->m_inputLayout->m_layout : nullptr
+        );
         m_immediateContext->IASetPrimitiveTopology(m_renderPipeline->m_primitiveTopology);
         m_immediateContext->VSSetShader(m_renderPipeline->m_vertexShader, nullptr, 0);
         m_immediateContext->RSSetState(m_renderPipeline->m_rasterizerState);
@@ -467,7 +602,7 @@ void CommandExecutor::cmdSetRenderState(const commands::SetRenderState& cmd)
         {
             m_immediateContext->IASetIndexBuffer(
                 checked_cast<BufferImpl*>(state.indexBuffer.buffer)->m_buffer,
-                D3DUtil::getIndexFormat(state.indexFormat),
+                getIndexFormat(state.indexFormat),
                 (UINT)state.indexBuffer.offset
             );
         }
@@ -552,7 +687,7 @@ void CommandExecutor::cmdDrawIndirect(const commands::DrawIndirect& cmd)
     // D3D11 does not support sourcing the count from a buffer.
     if (cmd.countBuffer)
     {
-        m_device->warning(S_RenderPassEncoder_drawIndirect " with countBuffer not supported");
+        m_device->printWarning(S_RenderPassEncoder_drawIndirect " with countBuffer not supported");
         return;
     }
 
@@ -569,7 +704,7 @@ void CommandExecutor::cmdDrawIndexedIndirect(const commands::DrawIndexedIndirect
     // D3D11 does not support sourcing the count from a buffer.
     if (cmd.countBuffer)
     {
-        m_device->warning(S_RenderPassEncoder_drawIndirect " with countBuffer not supported");
+        m_device->printWarning(S_RenderPassEncoder_drawIndirect " with countBuffer not supported");
         return;
     }
 
@@ -708,6 +843,12 @@ void CommandExecutor::cmdDeserializeAccelerationStructure(const commands::Deseri
     NOT_SUPPORTED(S_CommandEncoder_deserializeAccelerationStructure);
 }
 
+void CommandExecutor::cmdExecuteClusterOperation(const commands::ExecuteClusterOperation& cmd)
+{
+    SLANG_UNUSED(cmd);
+    NOT_SUPPORTED(S_CommandEncoder_executeClusterOperation);
+}
+
 void CommandExecutor::cmdConvertCooperativeVectorMatrix(const commands::ConvertCooperativeVectorMatrix& cmd)
 {
     SLANG_UNUSED(cmd);
@@ -724,6 +865,11 @@ void CommandExecutor::cmdSetTextureState(const commands::SetTextureState& cmd)
     SLANG_UNUSED(cmd);
 }
 
+void CommandExecutor::cmdGlobalBarrier(const commands::GlobalBarrier& cmd)
+{
+    SLANG_UNUSED(cmd);
+}
+
 void CommandExecutor::cmdPushDebugGroup(const commands::PushDebugGroup& cmd)
 {
     SLANG_UNUSED(cmd);
