@@ -1,7 +1,7 @@
 #define _CRT_SECURE_NO_WARNINGS
 #include "d3d11-device.h"
 #include "d3d11-buffer.h"
-#include "d3d11-helper-functions.h"
+#include "d3d11-utils.h"
 #include "d3d11-query.h"
 #include "d3d11-sampler.h"
 #include "d3d11-shader-object-layout.h"
@@ -22,29 +22,44 @@
 
 namespace rhi::d3d11 {
 
+inline Result getAdaptersImpl(std::vector<AdapterImpl>& outAdapters)
+{
+    std::vector<ComPtr<IDXGIAdapter>> dxgiAdapters;
+    SLANG_RETURN_ON_FAIL(enumAdapters(dxgiAdapters));
+
+    for (const auto& dxgiAdapter : dxgiAdapters)
+    {
+        AdapterInfo info = getAdapterInfo(dxgiAdapter);
+        info.deviceType = DeviceType::D3D11;
+
+        AdapterImpl adapter;
+        adapter.m_info = info;
+        adapter.m_dxgiAdapter = dxgiAdapter;
+        outAdapters.push_back(adapter);
+    }
+
+    // Mark default adapter (prefer discrete if available).
+    markDefaultAdapter(outAdapters);
+
+    return SLANG_OK;
+}
+
+std::vector<AdapterImpl>& getAdapters()
+{
+    static std::vector<AdapterImpl> adapters;
+    static Result initResult = getAdaptersImpl(adapters);
+    SLANG_UNUSED(initResult);
+    return adapters;
+}
+
 DeviceImpl::DeviceImpl() {}
 
 DeviceImpl::~DeviceImpl() {}
 
 Result DeviceImpl::initialize(const DeviceDesc& desc)
 {
-    SLANG_RETURN_ON_FAIL(
-        m_slangContext
-            .initialize(desc.slang, SLANG_DXBC, "sm_5_0", std::array{slang::PreprocessorMacroDesc{"__D3D11__", "1"}})
-    );
-
     SLANG_RETURN_ON_FAIL(Device::initialize(desc));
 
-    // Initialize DeviceInfo
-    {
-        m_info.deviceType = DeviceType::D3D11;
-        m_info.apiName = "D3D11";
-        static const float kIdentity[] = {1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1};
-        ::memcpy(m_info.identityProjectionMatrix, kIdentity, sizeof(kIdentity));
-    }
-
-    m_desc = desc;
-
     // Rather than statically link against D3D, we load it dynamically.
     SharedLibraryHandle d3dModule;
 #if SLANG_WINDOWS_FAMILY
@@ -54,15 +69,7 @@ Result DeviceImpl::initialize(const DeviceDesc& desc)
 #endif
     if (SLANG_FAILED(loadSharedLibrary(libName, d3dModule)))
     {
-        fprintf(stderr, "error: failed to load '%s'\n", libName);
-        return SLANG_FAIL;
-    }
-
-    PFN_D3D11_CREATE_DEVICE_AND_SWAP_CHAIN D3D11CreateDeviceAndSwapChain_ =
-        (PFN_D3D11_CREATE_DEVICE_AND_SWAP_CHAIN)findSymbolAddressByName(d3dModule, "D3D11CreateDeviceAndSwapChain");
-    if (!D3D11CreateDeviceAndSwapChain_)
-    {
-        fprintf(stderr, "error: failed load symbol 'D3D11CreateDeviceAndSwapChain'\n");
+        printError("Failed to load '%s'\n", libName);
         return SLANG_FAIL;
     }
 
@@ -70,10 +77,16 @@ Result DeviceImpl::initialize(const DeviceDesc& desc)
         (PFN_D3D11_CREATE_DEVICE)findSymbolAddressByName(d3dModule, "D3D11CreateDevice");
     if (!D3D11CreateDevice_)
     {
-        fprintf(stderr, "error: failed load symbol 'D3D11CreateDevice'\n");
+        printError("Failed to load symbol 'D3D11CreateDevice'\n");
         return SLANG_FAIL;
     }
 
+    m_dxgiFactory = getDXGIFactory();
+
+    AdapterImpl* adapter = nullptr;
+    SLANG_RETURN_ON_FAIL(selectAdapter(this, getAdapters(), desc, adapter));
+    m_dxgiAdapter = adapter->m_dxgiAdapter;
+
     // We will ask for the highest feature level that can be supported.
     const D3D_FEATURE_LEVEL featureLevels[] = {
         D3D_FEATURE_LEVEL_11_1,
@@ -84,200 +97,105 @@ Result DeviceImpl::initialize(const DeviceDesc& desc)
         D3D_FEATURE_LEVEL_9_2,
         D3D_FEATURE_LEVEL_9_1,
     };
-    D3D_FEATURE_LEVEL featureLevel = D3D_FEATURE_LEVEL_9_1;
-    const int totalNumFeatureLevels = SLANG_COUNT_OF(featureLevels);
-    bool isHardwareDevice = false;
 
+    D3D_FEATURE_LEVEL featureLevel = D3D_FEATURE_LEVEL(0);
+
+    // When creating the D3D11 device we need to consider:
+    // - Creation may fail if debug layer is requested but not available on the system.
+    // - Creation may fail if feature level 11_1 is requested on a system that does not support it.
+    // To handle this we try a few combinations until one works or we run out of options.
+    // The order of flags is important, as we want to try the most specific options first.
+
+    enum CreateFlag
     {
-        // On a machine that does not have an up-to-date version of D3D installed,
-        // the `D3D11CreateDeviceAndSwapChain` call will fail with `E_INVALIDARG`
-        // if you ask for feature level 11_1 (DeviceCheckFlag::UseFullFeatureLevel).
-        // The workaround is to call `D3D11CreateDeviceAndSwapChain` the first time
-        // with 11_1 and then back off to 11_0 if that fails.
+        UseDebug = 1,
+        Use11_1 = 2
+    };
+    int createFlags[] = {UseDebug | Use11_1, UseDebug, Use11_1, 0};
+    int usedCreateFlags = 0;
 
-        FlagCombiner combiner;
-        // TODO: we should probably provide a command-line option
-        // to override UseDebug of default rather than leave it
-        // up to each back-end to specify.
-
-#if SLANG_RHI_DEBUG
-        /// First try debug then non debug
-        combiner.add(DeviceCheckFlag::UseDebug, ChangeType::OnOff);
-#else
-        /// Don't bother with debug
-        combiner.add(DeviceCheckFlag::UseDebug, ChangeType::Off);
-#endif
-        /// First try hardware, then reference
-        combiner.add(DeviceCheckFlag::UseHardwareDevice, ChangeType::OnOff);
-        /// First try fully featured, then degrade features
-        combiner.add(DeviceCheckFlag::UseFullFeatureLevel, ChangeType::OnOff);
-
-        const int numCombinations = combiner.getNumCombinations();
-        Result res = SLANG_FAIL;
-        for (int i = 0; i < numCombinations; ++i)
-        {
-            const auto deviceCheckFlags = combiner.getCombination(i);
-            isHardwareDevice = (deviceCheckFlags & DeviceCheckFlag::UseHardwareDevice) != 0;
-            D3DUtil::createFactory(deviceCheckFlags, m_dxgiFactory);
-
-            // If we have an adapter set on the desc, look it up.
-            ComPtr<IDXGIAdapter> adapter;
-            if (desc.adapterLUID)
-            {
-                std::vector<ComPtr<IDXGIAdapter>> dxgiAdapters;
-                D3DUtil::findAdapters(deviceCheckFlags, desc.adapterLUID, m_dxgiFactory, dxgiAdapters);
-                if (dxgiAdapters.empty())
-                {
-                    continue;
-                }
-                adapter = dxgiAdapters[0];
-            }
-
-            // The adapter can be nullptr - that just means 'default', but when so we need to select the driver type
-            D3D_DRIVER_TYPE driverType = D3D_DRIVER_TYPE_UNKNOWN;
-            if (adapter == nullptr)
-            {
-                // If we don't have an adapter, select directly
-                driverType = (deviceCheckFlags & DeviceCheckFlag::UseHardwareDevice) ? D3D_DRIVER_TYPE_HARDWARE
-                                                                                     : D3D_DRIVER_TYPE_REFERENCE;
-            }
-
-            const int startFeatureIndex = (deviceCheckFlags & DeviceCheckFlag::UseFullFeatureLevel) ? 0 : 1;
-            const UINT deviceFlags = (deviceCheckFlags & DeviceCheckFlag::UseDebug) ? D3D11_CREATE_DEVICE_DEBUG : 0;
-
-            res = D3D11CreateDevice_(
-                adapter,
-                driverType,
-                nullptr,
-                deviceFlags,
-                &featureLevels[startFeatureIndex],
-                totalNumFeatureLevels - startFeatureIndex,
-                D3D11_SDK_VERSION,
-                m_device.writeRef(),
-                &featureLevel,
-                m_immediateContext.writeRef()
-            );
+    Result result = SLANG_FAIL;
+    for (uint32_t i = isDebugLayersEnabled() ? 0 : 2; i < SLANG_COUNT_OF(createFlags); i++)
+    {
+        usedCreateFlags = createFlags[i];
+        bool useDebug = (usedCreateFlags & UseDebug) != 0;
+        bool use11_1 = (usedCreateFlags & Use11_1) != 0;
+        result = D3D11CreateDevice_(
+            m_dxgiAdapter,
+            D3D_DRIVER_TYPE_UNKNOWN,
+            nullptr,
+            useDebug ? D3D11_CREATE_DEVICE_DEBUG : 0,
+            use11_1 ? featureLevels : featureLevels + 1,
+            use11_1 ? SLANG_COUNT_OF(featureLevels) : SLANG_COUNT_OF(featureLevels) - 1,
+            D3D11_SDK_VERSION,
+            m_device.writeRef(),
+            &featureLevel,
+            m_immediateContext.writeRef()
+        );
+        if (SUCCEEDED(result))
+            break;
+    }
+    if (FAILED(result))
+    {
+        printError("D3D11CreateDevice failed: %08x\n", result);
+        return SLANG_FAIL;
+    }
+    if (isDebugLayersEnabled() && (usedCreateFlags & UseDebug) == 0)
+    {
+        printWarning("Debug layer requested but not available.\n");
+    }
 
 #if SLANG_RHI_ENABLE_AFTERMATH
-            if (SLANG_SUCCEEDED(res))
-            {
-                if (deviceCheckFlags & DeviceCheckFlag::UseDebug)
-                {
-                    // Initialize Nsight Aftermath for this device.
-                    // This combination of flags is not necessarily appropriate for real world usage
-                    const uint32_t aftermathFlags =
-                        GFSDK_Aftermath_FeatureFlags_EnableMarkers |      // Enable event marker tracking.
-                        GFSDK_Aftermath_FeatureFlags_CallStackCapturing | // Enable automatic call stack event markers.
-                        GFSDK_Aftermath_FeatureFlags_EnableResourceTracking |    // Enable tracking of resources.
-                        GFSDK_Aftermath_FeatureFlags_GenerateShaderDebugInfo |   // Generate debug information for
-                                                                                 // shaders.
-                        GFSDK_Aftermath_FeatureFlags_EnableShaderErrorReporting; // Enable additional runtime shader
-                                                                                 // error reporting.
+    if (desc.enableAftermath && adapter->isNVIDIA())
+    {
+        // Initialize Nsight Aftermath for this device.
+        // This combination of flags is not necessarily appropriate for real world usage
+        const uint32_t aftermathFlags =
+            GFSDK_Aftermath_FeatureFlags_EnableMarkers | GFSDK_Aftermath_FeatureFlags_CallStackCapturing |
+            GFSDK_Aftermath_FeatureFlags_EnableResourceTracking | GFSDK_Aftermath_FeatureFlags_GenerateShaderDebugInfo |
+            GFSDK_Aftermath_FeatureFlags_EnableShaderErrorReporting;
 
-                    auto initResult =
-                        GFSDK_Aftermath_DX11_Initialize(GFSDK_Aftermath_Version_API, aftermathFlags, m_device);
+        auto initResult = GFSDK_Aftermath_DX11_Initialize(GFSDK_Aftermath_Version_API, aftermathFlags, m_device);
 
-                    if (initResult != GFSDK_Aftermath_Result_Success)
-                    {
-                        SLANG_RHI_ASSERT_FAILURE("Unable to initialize aftermath");
-                        // Unable to initialize aftermath
-                        return SLANG_FAIL;
-                    }
-                }
-            }
+        if (initResult != GFSDK_Aftermath_Result_Success)
+        {
+            printWarning("Failed to initialize aftermath: %d\n", int(initResult));
+        }
+    }
 #endif
 
-            // Check if successfully constructed - if so we are done.
-            if (SLANG_SUCCEEDED(res))
-            {
-                break;
-            }
-        }
-        // If res is failure, means all styles have have failed, and so initialization fails.
-        if (SLANG_FAILED(res))
-        {
-            return res;
-        }
-        // Check we have a swap chain, context and device
-        SLANG_RHI_ASSERT(m_immediateContext && m_device);
+    SLANG_RHI_ASSERT(m_device && m_immediateContext);
+    SLANG_RETURN_ON_FAIL(m_immediateContext->QueryInterface(m_immediateContext1.writeRef()));
 
-        SLANG_RETURN_ON_FAIL(m_immediateContext->QueryInterface(m_immediateContext1.writeRef()));
-
-        ComPtr<IDXGIDevice> dxgiDevice;
-        if (m_device->QueryInterface(dxgiDevice.writeRef()) == 0)
-        {
-            ComPtr<IDXGIAdapter> dxgiAdapter;
-            dxgiDevice->GetAdapter(dxgiAdapter.writeRef());
-            DXGI_ADAPTER_DESC adapterDesc;
-            dxgiAdapter->GetDesc(&adapterDesc);
-            m_adapterName = string::from_wstring(adapterDesc.Description);
-            m_info.adapterName = m_adapterName.data();
-        }
+    // Initialize device info
+    {
+        m_info.deviceType = DeviceType::D3D11;
+        m_info.apiName = "D3D11";
     }
 
-    addFeature(isHardwareDevice ? Feature::HardwareDevice : Feature::SoftwareDevice);
-    addFeature(Feature::ParameterBlock);
-    addFeature(Feature::Surface);
-    addFeature(Feature::Rasterization);
-    addFeature(Feature::CustomBorderColor);
-    addFeature(Feature::TimestampQuery);
-
-    // NVAPI
-#if SLANG_RHI_ENABLE_NVAPI
+    // Query adapter name & LUID
     {
-        if (SLANG_SUCCEEDED(NVAPIUtil::initialize()))
-        {
-            m_nvapiShaderExtension = NVAPIShaderExtension{desc.nvapiExtUavSlot, desc.nvapiExtRegisterSpace};
-            if (m_nvapiShaderExtension)
-            {
-                if (isSupportedNVAPIOp(m_device, NV_EXTN_OP_UINT64_ATOMIC))
-                {
-                    addFeature(Feature::AtomicInt64);
-                }
-                if (isSupportedNVAPIOp(m_device, NV_EXTN_OP_FP16_ATOMIC))
-                {
-                    addFeature(Feature::AtomicHalf);
-                }
-                if (isSupportedNVAPIOp(m_device, NV_EXTN_OP_FP32_ATOMIC))
-                {
-                    addFeature(Feature::AtomicFloat);
-                }
-                if (isSupportedNVAPIOp(m_device, NV_EXTN_OP_GET_SPECIAL))
-                {
-                    addFeature(Feature::RealtimeClock);
-                }
-            }
-        }
-    }
-#endif // SLANG_RHI_ENABLE_NVAPI
-
-    // Check double precision support
-    {
-        D3D11_FEATURE_DATA_DOUBLES doublePrecisionFeature = {};
-        if (SUCCEEDED(m_device->CheckFeatureSupport(
-                D3D11_FEATURE_DOUBLES,
-                &doublePrecisionFeature,
-                sizeof(doublePrecisionFeature)
-            )) &&
-            doublePrecisionFeature.DoublePrecisionFloatShaderOps)
-        {
-            addFeature(Feature::Double);
-        }
+        DXGI_ADAPTER_DESC adapterDesc;
+        m_dxgiAdapter->GetDesc(&adapterDesc);
+        m_adapterName = string::from_wstring(adapterDesc.Description);
+        m_info.adapterName = m_adapterName.data();
+        m_info.adapterLUID = getAdapterLUID(adapterDesc.AdapterLuid);
     }
 
+    // Query timestamp frequency
     {
-        // Create a TIMESTAMP_DISJOINT query object to query/update frequency info.
         D3D11_QUERY_DESC disjointQueryDesc = {};
         disjointQueryDesc.Query = D3D11_QUERY_TIMESTAMP_DISJOINT;
         SLANG_RETURN_ON_FAIL(m_device->CreateQuery(&disjointQueryDesc, m_disjointQuery.writeRef()));
         m_immediateContext->Begin(m_disjointQuery);
         m_immediateContext->End(m_disjointQuery);
         D3D11_QUERY_DATA_TIMESTAMP_DISJOINT disjointData = {};
+        m_immediateContext->Flush();
         m_immediateContext->GetData(m_disjointQuery, &disjointData, sizeof(disjointData), 0);
         m_info.timestampFrequency = disjointData.Frequency;
     }
 
-    // Get device limits.
+    // Query device limits
     {
         uint32_t maxTextureDimensionUV = 2048;
         if (featureLevel >= D3D_FEATURE_LEVEL_9_3)
@@ -320,6 +238,7 @@ Result DeviceImpl::initialize(const DeviceDesc& desc)
         }
 
         DeviceLimits limits = {};
+        limits.maxBufferSize = 0x80000000ull; // Assume 2GB
         limits.maxTextureDimension1D = maxTextureDimensionUV;
         limits.maxTextureDimension2D = maxTextureDimensionUV;
         limits.maxTextureDimension3D = maxTextureDimensionW;
@@ -351,7 +270,141 @@ Result DeviceImpl::initialize(const DeviceDesc& desc)
         m_info.limits = limits;
     }
 
+    // Initialize features & capabilities
+    bool isSoftwareDevice = adapter->m_info.adapterType == AdapterType::Software;
+    addFeature(isSoftwareDevice ? Feature::SoftwareDevice : Feature::HardwareDevice);
+    addFeature(Feature::ParameterBlock);
+    addFeature(Feature::Surface);
+    addFeature(Feature::Rasterization);
+    addFeature(Feature::CustomBorderColor);
+    if (m_info.timestampFrequency > 0)
+    {
+        addFeature(Feature::TimestampQuery);
+    }
+
+    addCapability(Capability::hlsl);
+
+// Initialize NVAPI
+#if SLANG_RHI_ENABLE_NVAPI
+    {
+        if (adapter->isNVIDIA() && SLANG_SUCCEEDED(NVAPIUtil::initialize()))
+        {
+            m_nvapiShaderExtension = NVAPIShaderExtension{desc.nvapiExtUavSlot, desc.nvapiExtRegisterSpace};
+            if (m_nvapiShaderExtension)
+            {
+                addCapability(Capability::hlsl_nvapi);
+
+                if (isSupportedNVAPIOp(m_device, NV_EXTN_OP_UINT64_ATOMIC))
+                {
+                    addFeature(Feature::AtomicInt64);
+                }
+                if (isSupportedNVAPIOp(m_device, NV_EXTN_OP_FP16_ATOMIC))
+                {
+                    addFeature(Feature::AtomicHalf);
+                }
+                if (isSupportedNVAPIOp(m_device, NV_EXTN_OP_FP32_ATOMIC))
+                {
+                    addFeature(Feature::AtomicFloat);
+                }
+                if (isSupportedNVAPIOp(m_device, NV_EXTN_OP_GET_SPECIAL))
+                {
+                    addFeature(Feature::RealtimeClock);
+                }
+            }
+        }
+    }
+#endif // SLANG_RHI_ENABLE_NVAPI
+
+    // Check double precision support
+    {
+        D3D11_FEATURE_DATA_DOUBLES doublePrecisionFeature = {};
+        if (SUCCEEDED(m_device->CheckFeatureSupport(
+                D3D11_FEATURE_DOUBLES,
+                &doublePrecisionFeature,
+                sizeof(doublePrecisionFeature)
+            )) &&
+            doublePrecisionFeature.DoublePrecisionFloatShaderOps)
+        {
+            addFeature(Feature::Double);
+        }
+    }
+
+    // Initialize format support table
+    for (size_t formatIndex = 0; formatIndex < size_t(Format::_Count); ++formatIndex)
+    {
+        Format format = Format(formatIndex);
+        const FormatMapping& formatMapping = getFormatMapping(format);
+        FormatSupport formatSupport = FormatSupport::None;
+
+#define UPDATE_FLAGS(d3dFlags, formatSupportFlags)                                                                     \
+    formatSupport |= (flags & d3dFlags) ? formatSupportFlags : FormatSupport::None;
+
+        D3D11_FEATURE_DATA_FORMAT_SUPPORT d3dFormatSupport = {formatMapping.srvFormat};
+        if (SLANG_SUCCEEDED(
+                m_device->CheckFeatureSupport(D3D11_FEATURE_FORMAT_SUPPORT, &d3dFormatSupport, sizeof(d3dFormatSupport))
+            ))
+        {
+            UINT flags = d3dFormatSupport.OutFormatSupport;
+            UPDATE_FLAGS(D3D11_FORMAT_SUPPORT_BUFFER, FormatSupport::Buffer);
+            UPDATE_FLAGS(D3D11_FORMAT_SUPPORT_IA_VERTEX_BUFFER, FormatSupport::VertexBuffer);
+            UPDATE_FLAGS(D3D11_FORMAT_SUPPORT_IA_INDEX_BUFFER, FormatSupport::IndexBuffer);
+            UPDATE_FLAGS(D3D11_FORMAT_SUPPORT_TEXTURE1D, FormatSupport::Texture);
+            UPDATE_FLAGS(D3D11_FORMAT_SUPPORT_TEXTURE2D, FormatSupport::Texture);
+            UPDATE_FLAGS(D3D11_FORMAT_SUPPORT_TEXTURE3D, FormatSupport::Texture);
+            UPDATE_FLAGS(D3D11_FORMAT_SUPPORT_TEXTURECUBE, FormatSupport::Texture);
+            UPDATE_FLAGS(D3D11_FORMAT_SUPPORT_SHADER_LOAD, FormatSupport::ShaderLoad);
+            UPDATE_FLAGS(D3D11_FORMAT_SUPPORT_SHADER_SAMPLE, FormatSupport::ShaderSample);
+            UPDATE_FLAGS(D3D11_FORMAT_SUPPORT_RENDER_TARGET, FormatSupport::RenderTarget);
+            UPDATE_FLAGS(D3D11_FORMAT_SUPPORT_BLENDABLE, FormatSupport::Blendable);
+            UPDATE_FLAGS(D3D11_FORMAT_SUPPORT_DEPTH_STENCIL, FormatSupport::DepthStencil);
+            UPDATE_FLAGS(D3D11_FORMAT_SUPPORT_MULTISAMPLE_RESOLVE, FormatSupport::Resolvable);
+            UPDATE_FLAGS(D3D11_FORMAT_SUPPORT_MULTISAMPLE_RENDERTARGET, FormatSupport::Multisampling);
+            UPDATE_FLAGS(
+                D3D11_FORMAT_SUPPORT_TYPED_UNORDERED_ACCESS_VIEW,
+                FormatSupport::ShaderUavLoad | FormatSupport::ShaderUavStore
+            );
+        }
+        D3D11_FEATURE_DATA_FORMAT_SUPPORT2 d3dFormatSupport2 = {formatMapping.srvFormat};
+        if (SLANG_SUCCEEDED(m_device->CheckFeatureSupport(
+                D3D11_FEATURE_FORMAT_SUPPORT2,
+                &d3dFormatSupport2,
+                sizeof(d3dFormatSupport2)
+            )))
+        {
+            UINT flags = d3dFormatSupport2.OutFormatSupport2;
+            if (is_set(formatSupport, FormatSupport::ShaderUavStore))
+            {
+                UPDATE_FLAGS(
+                    (D3D11_FORMAT_SUPPORT2_UAV_ATOMIC_ADD | D3D11_FORMAT_SUPPORT2_UAV_ATOMIC_BITWISE_OPS |
+                     D3D11_FORMAT_SUPPORT2_UAV_ATOMIC_COMPARE_STORE_OR_COMPARE_EXCHANGE |
+                     D3D11_FORMAT_SUPPORT2_UAV_ATOMIC_EXCHANGE | D3D11_FORMAT_SUPPORT2_UAV_ATOMIC_SIGNED_MIN_OR_MAX |
+                     D3D11_FORMAT_SUPPORT2_UAV_ATOMIC_UNSIGNED_MIN_OR_MAX),
+                    FormatSupport::ShaderAtomic
+                );
+            }
+        }
+
+#undef UPDATE_FLAGS
+
+        if (formatSupport != FormatSupport::None)
+        {
+            formatSupport |= FormatSupport::CopySource | FormatSupport::CopyDestination;
+        }
+
+        m_formatSupport[formatIndex] = formatSupport;
+    }
+
+    // Initialize slang context
+    SLANG_RETURN_ON_FAIL(m_slangContext.initialize(
+        desc.slang,
+        SLANG_DXBC,
+        "sm_5_0",
+        getCapabilities(),
+        std::array{slang::PreprocessorMacroDesc{"__D3D11__", "1"}}
+    ));
+
     m_queue = new CommandQueueImpl(this, QueueType::Graphics);
+    m_queue->setInternalReferenceCount(1);
 
     return SLANG_OK;
 }
@@ -360,8 +413,8 @@ Result DeviceImpl::readTexture(
     ITexture* texture,
     uint32_t layer,
     uint32_t mip,
-    ISlangBlob** outBlob,
-    SubresourceLayout* outLayout
+    const SubresourceLayout& layout,
+    void* outData
 )
 {
     auto textureImpl = checked_cast<TextureImpl*>(texture);
@@ -380,10 +433,6 @@ Result DeviceImpl::readTexture(
     // needs to be made, it is kept alive for the duration of the function.
     ComPtr<ITexture> tempTexture;
 
-    // Calculate layout info.
-    SubresourceLayout layout;
-    SLANG_RETURN_ON_FAIL(texture->getSubresourceLayout(mip, &layout));
-
     TextureImpl* stagingTextureImpl = nullptr;
     uint32_t subResourceIdx = D3D11CalcSubresource(mip, layer, desc.mipCount);
     if (desc.memoryType == MemoryType::ReadBack)
@@ -454,10 +503,6 @@ Result DeviceImpl::readTexture(
         subResourceIdx = 0;
     }
 
-    // Output layout
-    if (outLayout)
-        *outLayout = layout;
-
     // Now just read back texels from the staging textures
     {
         D3D11_MAPPED_SUBRESOURCE mappedResource;
@@ -468,7 +513,7 @@ Result DeviceImpl::readTexture(
 
         auto blob = OwnedBlob::create(layout.sizeInBytes);
         uint8_t* srcBuffer = (uint8_t*)mappedResource.pData;
-        uint8_t* dstBuffer = (uint8_t*)blob->getBufferPointer();
+        uint8_t* dstBuffer = (uint8_t*)outData;
 
         // Data should be the same, but alignment may not be, so the row copy
         // needs to be the minimum of the two row sizes.
@@ -492,7 +537,6 @@ Result DeviceImpl::readTexture(
         // Make sure to unmap
         m_immediateContext->Unmap(stagingTextureImpl->m_resource.get(), subResourceIdx);
 
-        returnComPtr(outBlob, blob);
         return SLANG_OK;
     }
 }
@@ -546,31 +590,22 @@ Result DeviceImpl::getTextureRowAlignment(Format format, Size* outAlignment)
     return SLANG_OK;
 }
 
-Result DeviceImpl::getFormatSupport(Format format, FormatSupport* outFormatSupport)
-{
-    SLANG_RETURN_ON_FAIL(Device::getFormatSupport(format, outFormatSupport));
-
-    // Disable formats for which we have no mapping
-    if (D3DUtil::getFormatMapping(format).srvFormat == DXGI_FORMAT_UNKNOWN)
-        *outFormatSupport = FormatSupport::None;
-
-    return SLANG_OK;
-}
-
 Result DeviceImpl::createShaderProgram(
     const ShaderProgramDesc& desc,
     IShaderProgram** outProgram,
     ISlangBlob** outDiagnosticBlob
 )
 {
-    RefPtr<ShaderProgramImpl> shaderProgram = new ShaderProgramImpl();
-    shaderProgram->init(desc);
-    SLANG_RETURN_ON_FAIL(RootShaderObjectLayoutImpl::create(
-        this,
-        shaderProgram->linkedProgram,
-        shaderProgram->linkedProgram->getLayout(),
-        shaderProgram->m_rootObjectLayout.writeRef()
-    ));
+    RefPtr<ShaderProgramImpl> shaderProgram = new ShaderProgramImpl(this, desc);
+    SLANG_RETURN_ON_FAIL(shaderProgram->init());
+    SLANG_RETURN_ON_FAIL(
+        RootShaderObjectLayoutImpl::create(
+            this,
+            shaderProgram->linkedProgram,
+            shaderProgram->linkedProgram->getLayout(),
+            shaderProgram->m_rootObjectLayout.writeRef()
+        )
+    );
     returnComPtr(outProgram, shaderProgram);
     return SLANG_OK;
 }
@@ -597,3 +632,21 @@ Result DeviceImpl::createRootShaderObjectLayout(
 }
 
 } // namespace rhi::d3d11
+
+namespace rhi {
+
+IAdapter* getD3D11Adapter(uint32_t index)
+{
+    std::vector<d3d11::AdapterImpl>& adapters = d3d11::getAdapters();
+    return index < adapters.size() ? &adapters[index] : nullptr;
+}
+
+Result createD3D11Device(const DeviceDesc* desc, IDevice** outDevice)
+{
+    RefPtr<d3d11::DeviceImpl> result = new d3d11::DeviceImpl();
+    SLANG_RETURN_ON_FAIL(result->initialize(*desc));
+    returnComPtr(outDevice, result);
+    return SLANG_OK;
+}
+
+} // namespace rhi
