@@ -4,8 +4,6 @@
 
 namespace rhi::d3d12 {
 
-#if SLANG_RHI_DXR
-
 AccelerationStructureImpl::AccelerationStructureImpl(Device* device, const AccelerationStructureDesc& desc)
     : AccelerationStructure(device, desc)
 {
@@ -15,8 +13,15 @@ AccelerationStructureImpl::~AccelerationStructureImpl()
 {
     DeviceImpl* device = getDevice<DeviceImpl>();
 
+    if (m_descriptorHandle)
+    {
+        device->m_bindlessDescriptorSet->freeHandle(m_descriptorHandle);
+    }
+
     if (m_descriptor)
+    {
         device->m_cpuCbvSrvUavHeap->free(m_descriptor);
+    }
 }
 
 Result AccelerationStructureImpl::getNativeHandle(NativeHandle* outHandle)
@@ -36,6 +41,26 @@ DeviceAddress AccelerationStructureImpl::getDeviceAddress()
     return m_buffer->getDeviceAddress();
 }
 
+Result AccelerationStructureImpl::getDescriptorHandle(DescriptorHandle* outHandle)
+{
+    DeviceImpl* device = getDevice<DeviceImpl>();
+
+    if (!device->m_bindlessDescriptorSet)
+    {
+        return SLANG_E_NOT_AVAILABLE;
+    }
+
+    if (!m_descriptorHandle)
+    {
+        SLANG_RETURN_ON_FAIL(
+            device->m_bindlessDescriptorSet->allocAccelerationStructureHandle(this, &m_descriptorHandle)
+        );
+    }
+
+    *outHandle = m_descriptorHandle;
+    return SLANG_OK;
+}
+
 Result AccelerationStructureBuildDescConverter::convert(
     const AccelerationStructureBuildDesc& buildDesc,
     IDebugCallback* callback
@@ -46,6 +71,12 @@ Result AccelerationStructureBuildDescConverter::convert(
         return SLANG_E_INVALID_ARG;
     }
 
+    // Motion blur is not supported in D3D12
+    if (is_set(buildDesc.flags, AccelerationStructureBuildFlags::CreateMotion))
+    {
+        return SLANG_E_NOT_AVAILABLE;
+    }
+
     AccelerationStructureBuildInputType type = buildDesc.inputs[0].type;
     for (uint32_t i = 1; i < buildDesc.inputCount; ++i)
     {
@@ -99,12 +130,12 @@ Result AccelerationStructureBuildDescConverter::convert(
             geomDesc.Triangles.VertexBuffer.StartAddress = triangles.vertexBuffers[0].getDeviceAddress();
             geomDesc.Triangles.VertexBuffer.StrideInBytes = triangles.vertexStride;
             geomDesc.Triangles.VertexCount = triangles.vertexCount;
-            geomDesc.Triangles.VertexFormat = D3DUtil::getVertexFormat(triangles.vertexFormat);
+            geomDesc.Triangles.VertexFormat = getVertexFormat(triangles.vertexFormat);
             if (triangles.indexBuffer)
             {
                 geomDesc.Triangles.IndexBuffer = triangles.indexBuffer.getDeviceAddress();
                 geomDesc.Triangles.IndexCount = triangles.indexCount;
-                geomDesc.Triangles.IndexFormat = D3DUtil::getIndexFormat(triangles.indexFormat);
+                geomDesc.Triangles.IndexFormat = getIndexFormat(triangles.indexFormat);
             }
             else
             {
@@ -209,6 +240,12 @@ Result AccelerationStructureBuildDescConverterNVAPI::convert(
         return SLANG_E_INVALID_ARG;
     }
 
+    // Motion blur is not supported in D3D12
+    if (is_set(buildDesc.flags, AccelerationStructureBuildFlags::CreateMotion))
+    {
+        return SLANG_E_NOT_AVAILABLE;
+    }
+
     AccelerationStructureBuildInputType type = buildDesc.inputs[0].type;
     for (uint32_t i = 1; i < buildDesc.inputCount; ++i)
     {
@@ -263,12 +300,12 @@ Result AccelerationStructureBuildDescConverterNVAPI::convert(
             geomDesc.triangles.VertexBuffer.StartAddress = triangles.vertexBuffers[0].getDeviceAddress();
             geomDesc.triangles.VertexBuffer.StrideInBytes = triangles.vertexStride;
             geomDesc.triangles.VertexCount = triangles.vertexCount;
-            geomDesc.triangles.VertexFormat = D3DUtil::getVertexFormat(triangles.vertexFormat);
+            geomDesc.triangles.VertexFormat = getVertexFormat(triangles.vertexFormat);
             if (triangles.indexBuffer)
             {
                 geomDesc.triangles.IndexBuffer = triangles.indexBuffer.getDeviceAddress();
                 geomDesc.triangles.IndexCount = triangles.indexCount;
-                geomDesc.triangles.IndexFormat = D3DUtil::getIndexFormat(triangles.indexFormat);
+                geomDesc.triangles.IndexFormat = getIndexFormat(triangles.indexFormat);
             }
             else
             {
@@ -325,16 +362,16 @@ Result AccelerationStructureBuildDescConverterNVAPI::convert(
             geomDesc.spheres.vertexCount = spheres.vertexCount;
             geomDesc.spheres.vertexPositionBuffer.StartAddress = spheres.vertexPositionBuffers[0].getDeviceAddress();
             geomDesc.spheres.vertexPositionBuffer.StrideInBytes = spheres.vertexPositionStride;
-            geomDesc.spheres.vertexPositionFormat = D3DUtil::getVertexFormat(spheres.vertexPositionFormat);
+            geomDesc.spheres.vertexPositionFormat = getVertexFormat(spheres.vertexPositionFormat);
             geomDesc.spheres.vertexRadiusBuffer.StartAddress = spheres.vertexRadiusBuffers[0].getDeviceAddress();
             geomDesc.spheres.vertexRadiusBuffer.StrideInBytes = spheres.vertexRadiusStride;
-            geomDesc.spheres.vertexRadiusFormat = D3DUtil::getVertexFormat(spheres.vertexRadiusFormat);
+            geomDesc.spheres.vertexRadiusFormat = getVertexFormat(spheres.vertexRadiusFormat);
             if (spheres.indexBuffer)
             {
                 geomDesc.spheres.indexCount = spheres.indexCount;
                 geomDesc.spheres.indexBuffer.StartAddress = spheres.indexBuffer.getDeviceAddress();
                 geomDesc.spheres.indexBuffer.StrideInBytes = spheres.indexFormat == IndexFormat::Uint32 ? 4 : 2;
-                geomDesc.spheres.indexFormat = D3DUtil::getIndexFormat(spheres.indexFormat);
+                geomDesc.spheres.indexFormat = getIndexFormat(spheres.indexFormat);
             }
             else
             {
@@ -361,21 +398,21 @@ Result AccelerationStructureBuildDescConverterNVAPI::convert(
                 return SLANG_E_INVALID_ARG;
             }
             NVAPI_D3D12_RAYTRACING_GEOMETRY_DESC_EX& geomDesc = geomDescs[i];
-            geomDesc.type = NVAPI_D3D12_RAYTRACING_GEOMETRY_TYPE_SPHERES_EX;
+            geomDesc.type = NVAPI_D3D12_RAYTRACING_GEOMETRY_TYPE_LSS_EX;
             geomDesc.flags = translateGeometryFlags(lss.flags);
             geomDesc.lss.vertexCount = lss.vertexCount;
             geomDesc.lss.vertexPositionBuffer.StartAddress = lss.vertexPositionBuffers[0].getDeviceAddress();
             geomDesc.lss.vertexPositionBuffer.StrideInBytes = lss.vertexPositionStride;
-            geomDesc.lss.vertexPositionFormat = D3DUtil::getVertexFormat(lss.vertexPositionFormat);
+            geomDesc.lss.vertexPositionFormat = getVertexFormat(lss.vertexPositionFormat);
             geomDesc.lss.vertexRadiusBuffer.StartAddress = lss.vertexRadiusBuffers[0].getDeviceAddress();
             geomDesc.lss.vertexRadiusBuffer.StrideInBytes = lss.vertexRadiusStride;
-            geomDesc.lss.vertexRadiusFormat = D3DUtil::getVertexFormat(lss.vertexRadiusFormat);
+            geomDesc.lss.vertexRadiusFormat = getVertexFormat(lss.vertexRadiusFormat);
             if (lss.indexBuffer)
             {
                 geomDesc.lss.indexCount = lss.indexCount;
                 geomDesc.lss.indexBuffer.StartAddress = lss.indexBuffer.getDeviceAddress();
                 geomDesc.lss.indexBuffer.StrideInBytes = lss.indexFormat == IndexFormat::Uint32 ? 4 : 2;
-                geomDesc.lss.indexFormat = D3DUtil::getIndexFormat(lss.indexFormat);
+                geomDesc.lss.indexFormat = getIndexFormat(lss.indexFormat);
             }
             else
             {
@@ -474,8 +511,119 @@ NVAPI_D3D12_RAYTRACING_LSS_ENDCAP_MODE AccelerationStructureBuildDescConverterNV
     }
 }
 
+NVAPI_D3D12_RAYTRACING_MULTI_INDIRECT_CLUSTER_OPERATION_FLAGS translateClusterOperationFlags(
+    ClusterOperationFlags flags
+)
+{
+    unsigned int result = NVAPI_D3D12_RAYTRACING_MULTI_INDIRECT_CLUSTER_OPERATION_FLAG_NONE;
+    if (is_set(flags, ClusterOperationFlags::FastTrace))
+    {
+        result |= NVAPI_D3D12_RAYTRACING_MULTI_INDIRECT_CLUSTER_OPERATION_FLAG_FAST_TRACE;
+    }
+    if (is_set(flags, ClusterOperationFlags::FastBuild))
+    {
+        result |= NVAPI_D3D12_RAYTRACING_MULTI_INDIRECT_CLUSTER_OPERATION_FLAG_FAST_BUILD;
+    }
+    if (is_set(flags, ClusterOperationFlags::NoOverlap))
+    {
+        result |= NVAPI_D3D12_RAYTRACING_MULTI_INDIRECT_CLUSTER_OPERATION_FLAG_NO_OVERLAP;
+    }
+    if (is_set(flags, ClusterOperationFlags::AllowOMM))
+    {
+        result |= NVAPI_D3D12_RAYTRACING_MULTI_INDIRECT_CLUSTER_OPERATION_FLAG_ALLOW_OMM;
+    }
+    return NVAPI_D3D12_RAYTRACING_MULTI_INDIRECT_CLUSTER_OPERATION_FLAGS(result);
+}
+
+NVAPI_D3D12_RAYTRACING_MULTI_INDIRECT_CLUSTER_OPERATION_MOVE_TYPE translateClusterOperationMoveType(
+    ClusterOperationMoveType type
+)
+{
+    switch (type)
+    {
+    case ClusterOperationMoveType::BottomLevel:
+        return NVAPI_D3D12_RAYTRACING_MULTI_INDIRECT_CLUSTER_OPERATION_MOVE_TYPE_BOTTOM_LEVEL_ACCELERATION_STRUCTURE;
+    case ClusterOperationMoveType::ClusterLevel:
+        return NVAPI_D3D12_RAYTRACING_MULTI_INDIRECT_CLUSTER_OPERATION_MOVE_TYPE_CLUSTER_LEVEL_ACCELERATION_STRUCTURE;
+    case ClusterOperationMoveType::Template:
+        return NVAPI_D3D12_RAYTRACING_MULTI_INDIRECT_CLUSTER_OPERATION_MOVE_TYPE_TEMPLATE;
+    }
+    return NVAPI_D3D12_RAYTRACING_MULTI_INDIRECT_CLUSTER_OPERATION_MOVE_TYPE(0);
+}
+
+NVAPI_D3D12_RAYTRACING_MULTI_INDIRECT_CLUSTER_OPERATION_INPUTS translateClusterOperationParams(
+    const ClusterOperationParams& params
+)
+{
+    NVAPI_D3D12_RAYTRACING_MULTI_INDIRECT_CLUSTER_OPERATION_INPUTS inputs = {};
+
+    inputs.maxArgCount = params.maxArgCount;
+    inputs.flags = translateClusterOperationFlags(params.flags);
+    inputs.mode = translateClusterOperationMode(params.mode);
+
+    switch (params.type)
+    {
+    case ClusterOperationType::CLASFromTriangles:
+        inputs.type = NVAPI_D3D12_RAYTRACING_MULTI_INDIRECT_CLUSTER_OPERATION_TYPE_BUILD_CLAS_FROM_TRIANGLES;
+        break;
+    case ClusterOperationType::BLASFromCLAS:
+        inputs.type = NVAPI_D3D12_RAYTRACING_MULTI_INDIRECT_CLUSTER_OPERATION_TYPE_BUILD_BLAS_FROM_CLAS;
+        break;
+    case ClusterOperationType::TemplatesFromTriangles:
+        inputs.type =
+            NVAPI_D3D12_RAYTRACING_MULTI_INDIRECT_CLUSTER_OPERATION_TYPE_BUILD_CLUSTER_TEMPLATES_FROM_TRIANGLES;
+        break;
+    case ClusterOperationType::CLASFromTemplates:
+        inputs.type = NVAPI_D3D12_RAYTRACING_MULTI_INDIRECT_CLUSTER_OPERATION_TYPE_INSTANTIATE_CLUSTER_TEMPLATES;
+        break;
+    case ClusterOperationType::MoveObjects:
+        inputs.type = NVAPI_D3D12_RAYTRACING_MULTI_INDIRECT_CLUSTER_OPERATION_TYPE_MOVE_CLUSTER_OBJECT;
+        break;
+    }
+
+    switch (inputs.type)
+    {
+    case NVAPI_D3D12_RAYTRACING_MULTI_INDIRECT_CLUSTER_OPERATION_TYPE_MOVE_CLUSTER_OBJECT:
+        inputs.movesDesc.type = translateClusterOperationMoveType(params.move.type);
+        inputs.movesDesc.maxBytesMoved = params.move.maxSize;
+        break;
+    case NVAPI_D3D12_RAYTRACING_MULTI_INDIRECT_CLUSTER_OPERATION_TYPE_BUILD_BLAS_FROM_CLAS:
+        inputs.clasDesc.maxTotalClasCount = params.blas.maxTotalClasCount;
+        inputs.clasDesc.maxClasCountPerArg = params.blas.maxClasCount;
+        break;
+    case NVAPI_D3D12_RAYTRACING_MULTI_INDIRECT_CLUSTER_OPERATION_TYPE_BUILD_CLAS_FROM_TRIANGLES:
+    case NVAPI_D3D12_RAYTRACING_MULTI_INDIRECT_CLUSTER_OPERATION_TYPE_BUILD_CLUSTER_TEMPLATES_FROM_TRIANGLES:
+    case NVAPI_D3D12_RAYTRACING_MULTI_INDIRECT_CLUSTER_OPERATION_TYPE_INSTANTIATE_CLUSTER_TEMPLATES:
+        inputs.trianglesDesc.vertexFormat = getVertexFormat(params.clas.vertexFormat);
+        inputs.trianglesDesc.maxGeometryIndexValue = params.clas.maxGeometryIndex;
+        inputs.trianglesDesc.maxUniqueGeometryCountPerArg = params.clas.maxUniqueGeometryCount;
+        inputs.trianglesDesc.maxTriangleCountPerArg = params.clas.maxTriangleCount;
+        inputs.trianglesDesc.maxVertexCountPerArg = params.clas.maxVertexCount;
+        inputs.trianglesDesc.maxTotalTriangleCount = params.clas.maxTotalTriangleCount;
+        inputs.trianglesDesc.maxTotalVertexCount = params.clas.maxTotalVertexCount;
+        inputs.trianglesDesc.minPositionTruncateBitCount = params.clas.minPositionTruncateBitCount;
+        break;
+    default:
+        break;
+    }
+
+    return inputs;
+}
+
+NVAPI_D3D12_RAYTRACING_MULTI_INDIRECT_CLUSTER_OPERATION_MODE translateClusterOperationMode(ClusterOperationMode mode)
+{
+    switch (mode)
+    {
+    case ClusterOperationMode::ImplicitDestinations:
+        return NVAPI_D3D12_RAYTRACING_MULTI_INDIRECT_CLUSTER_OPERATION_MODE_IMPLICIT_DESTINATIONS;
+    case ClusterOperationMode::ExplicitDestinations:
+        return NVAPI_D3D12_RAYTRACING_MULTI_INDIRECT_CLUSTER_OPERATION_MODE_EXPLICIT_DESTINATIONS;
+    case ClusterOperationMode::GetSizes:
+        return NVAPI_D3D12_RAYTRACING_MULTI_INDIRECT_CLUSTER_OPERATION_MODE_GET_SIZES;
+    }
+    return NVAPI_D3D12_RAYTRACING_MULTI_INDIRECT_CLUSTER_OPERATION_MODE(0);
+}
+
 #endif // SLANG_RHI_ENABLE_NVAPI
 
-#endif // SLANG_RHI_DXR
-
 } // namespace rhi::d3d12
