@@ -10,7 +10,7 @@
 #include "d3d12-fence.h"
 #include "d3d12-query.h"
 #include "d3d12-input-layout.h"
-#include "d3d12-helper-functions.h"
+#include "d3d12-utils.h"
 #include "../state-tracking.h"
 #include "../strings.h"
 #include "../format-conversion.h"
@@ -101,9 +101,11 @@ public:
     void cmdQueryAccelerationStructureProperties(const commands::QueryAccelerationStructureProperties& cmd);
     void cmdSerializeAccelerationStructure(const commands::SerializeAccelerationStructure& cmd);
     void cmdDeserializeAccelerationStructure(const commands::DeserializeAccelerationStructure& cmd);
+    void cmdExecuteClusterOperation(const commands::ExecuteClusterOperation& cmd);
     void cmdConvertCooperativeVectorMatrix(const commands::ConvertCooperativeVectorMatrix& cmd);
     void cmdSetBufferState(const commands::SetBufferState& cmd);
     void cmdSetTextureState(const commands::SetTextureState& cmd);
+    void cmdGlobalBarrier(const commands::GlobalBarrier& cmd);
     void cmdPushDebugGroup(const commands::PushDebugGroup& cmd);
     void cmdPopDebugGroup(const commands::PopDebugGroup& cmd);
     void cmdInsertDebugMarker(const commands::InsertDebugMarker& cmd);
@@ -160,7 +162,7 @@ Result CommandRecorder::record(CommandBufferImpl* commandBuffer)
     return SLANG_OK;
 }
 
-#define NOT_SUPPORTED(x) m_device->warning(S_CommandEncoder_##x " command is not supported!")
+#define NOT_SUPPORTED(x) m_device->printWarning(S_CommandEncoder_##x " command is not supported!")
 
 void CommandRecorder::cmdCopyBuffer(const commands::CopyBuffer& cmd)
 {
@@ -228,8 +230,8 @@ void CommandRecorder::cmdCopyTexture(const commands::CopyTexture& cmd)
 
     Extent3D srcTextureSize = src->m_desc.size;
 
-    uint32_t planeCount = D3DUtil::getPlaneSliceCount(dst->m_format);
-    SLANG_RHI_ASSERT(planeCount == D3DUtil::getPlaneSliceCount(src->m_format));
+    uint32_t planeCount = getPlaneSliceCount(dst->m_format);
+    SLANG_RHI_ASSERT(planeCount == getPlaneSliceCount(src->m_format));
     SLANG_RHI_ASSERT(planeCount > 0);
 
     for (uint32_t planeIndex = 0; planeIndex < planeCount; planeIndex++)
@@ -272,7 +274,7 @@ void CommandRecorder::cmdCopyTexture(const commands::CopyTexture& cmd)
 
                 dstRegion.Type = D3D12_TEXTURE_COPY_TYPE_SUBRESOURCE_INDEX;
                 dstRegion.pResource = dst->m_resource.getResource();
-                dstRegion.SubresourceIndex = D3DUtil::getSubresourceIndex(
+                dstRegion.SubresourceIndex = getSubresourceIndex(
                     dstMip,
                     dstSubresource.layer + layer,
                     planeIndex,
@@ -283,7 +285,7 @@ void CommandRecorder::cmdCopyTexture(const commands::CopyTexture& cmd)
                 D3D12_TEXTURE_COPY_LOCATION srcRegion = {};
                 srcRegion.Type = D3D12_TEXTURE_COPY_TYPE_SUBRESOURCE_INDEX;
                 srcRegion.pResource = src->m_resource.getResource();
-                srcRegion.SubresourceIndex = D3DUtil::getSubresourceIndex(
+                srcRegion.SubresourceIndex = getSubresourceIndex(
                     srcMip,
                     srcSubresource.layer + layer,
                     planeIndex,
@@ -365,7 +367,7 @@ void CommandRecorder::cmdCopyTextureToBuffer(const commands::CopyTextureToBuffer
     D3D12_TEXTURE_COPY_LOCATION srcRegion = {};
     srcRegion.Type = D3D12_TEXTURE_COPY_TYPE_SUBRESOURCE_INDEX;
     srcRegion.SubresourceIndex =
-        D3DUtil::getSubresourceIndex(srcMip, srcLayer, 0, src->m_desc.mipCount, src->m_desc.arrayLength);
+        getSubresourceIndex(srcMip, srcLayer, 0, src->m_desc.mipCount, src->m_desc.arrayLength);
     srcRegion.pResource = src->m_resource.getResource();
 
     // Setup the destination resource.
@@ -550,7 +552,7 @@ void CommandRecorder::cmdUploadTextureData(const commands::UploadTextureData& cm
             dstRegion.pResource = dst->m_resource.getResource();
 
             dstRegion.SubresourceIndex =
-                D3DUtil::getSubresourceIndex(mip, layer, 0, dst->m_desc.mipCount, dst->m_desc.arrayLength);
+                getSubresourceIndex(mip, layer, 0, dst->m_desc.mipCount, dst->m_desc.arrayLength);
 
             D3D12_TEXTURE_COPY_LOCATION srcRegion = {};
             srcRegion.Type = D3D12_TEXTURE_COPY_TYPE_PLACED_FOOTPRINT;
@@ -848,7 +850,7 @@ void CommandRecorder::cmdSetRenderState(const commands::SetRenderState& cmd)
             D3D12_INDEX_BUFFER_VIEW indexBufferView;
             indexBufferView.BufferLocation = buffer->m_resource.getResource()->GetGPUVirtualAddress() + offset;
             indexBufferView.SizeInBytes = UINT(buffer->m_desc.size - offset);
-            indexBufferView.Format = D3DUtil::getIndexFormat(state.indexFormat);
+            indexBufferView.Format = getIndexFormat(state.indexFormat);
             m_cmdList->IASetIndexBuffer(&indexBufferView);
         }
         else
@@ -1018,6 +1020,9 @@ void CommandRecorder::cmdSetComputeState(const commands::SetComputeState& cmd)
     }
 
     m_computeStateValid = true;
+
+    m_renderStateValid = false;
+    m_rayTracingStateValid = false;
 }
 
 void CommandRecorder::cmdDispatchCompute(const commands::DispatchCompute& cmd)
@@ -1121,6 +1126,9 @@ void CommandRecorder::cmdSetRayTracingState(const commands::SetRayTracingState&
     }
 
     m_rayTracingStateValid = true;
+
+    m_renderStateValid = false;
+    m_computeStateValid = false;
 }
 
 void CommandRecorder::cmdDispatchRays(const commands::DispatchRays& cmd)
@@ -1140,11 +1148,125 @@ void CommandRecorder::cmdBuildAccelerationStructure(const commands::BuildAcceler
 {
     AccelerationStructureImpl* dst = checked_cast<AccelerationStructureImpl*>(cmd.dst);
     AccelerationStructureImpl* src = checked_cast<AccelerationStructureImpl*>(cmd.src);
+    BufferImpl* scratchBuffer = checked_cast<BufferImpl*>(cmd.scratchBuffer.buffer);
+
+    requireBufferState(dst->m_buffer, ResourceState::AccelerationStructureWrite);
+    if (src)
+        requireBufferState(src->m_buffer, ResourceState::AccelerationStructureRead);
+    requireBufferState(scratchBuffer, ResourceState::UnorderedAccess);
+
+    for (uint32_t inputIndex = 0; inputIndex < cmd.desc.inputCount; ++inputIndex)
+    {
+        const AccelerationStructureBuildInput& input = cmd.desc.inputs[inputIndex];
+        switch (input.type)
+        {
+        case AccelerationStructureBuildInputType::Instances:
+            if (input.instances.instanceBuffer.buffer)
+            {
+                requireBufferState(
+                    checked_cast<BufferImpl*>(input.instances.instanceBuffer.buffer),
+                    ResourceState::AccelerationStructureBuildInput
+                );
+            }
+            break;
+        case AccelerationStructureBuildInputType::Triangles:
+            for (uint32_t i = 0; i < input.triangles.vertexBufferCount; ++i)
+            {
+                if (input.triangles.vertexBuffers[i].buffer)
+                {
+                    requireBufferState(
+                        checked_cast<BufferImpl*>(input.triangles.vertexBuffers[i].buffer),
+                        ResourceState::AccelerationStructureBuildInput
+                    );
+                }
+            }
+            if (input.triangles.indexBuffer.buffer)
+            {
+                requireBufferState(
+                    checked_cast<BufferImpl*>(input.triangles.indexBuffer.buffer),
+                    ResourceState::AccelerationStructureBuildInput
+                );
+            }
+            if (input.triangles.preTransformBuffer.buffer)
+            {
+                requireBufferState(
+                    checked_cast<BufferImpl*>(input.triangles.preTransformBuffer.buffer),
+                    ResourceState::AccelerationStructureBuildInput
+                );
+            }
+            break;
+        case AccelerationStructureBuildInputType::ProceduralPrimitives:
+            for (uint32_t i = 0; i < input.proceduralPrimitives.aabbBufferCount; ++i)
+            {
+                if (input.proceduralPrimitives.aabbBuffers[i].buffer)
+                {
+                    requireBufferState(
+                        checked_cast<BufferImpl*>(input.proceduralPrimitives.aabbBuffers[i].buffer),
+                        ResourceState::AccelerationStructureBuildInput
+                    );
+                }
+            }
+            break;
+        case AccelerationStructureBuildInputType::Spheres:
+            for (uint32_t i = 0; i < input.spheres.vertexBufferCount; ++i)
+            {
+                if (input.spheres.vertexPositionBuffers[i].buffer)
+                {
+                    requireBufferState(
+                        checked_cast<BufferImpl*>(input.spheres.vertexPositionBuffers[i].buffer),
+                        ResourceState::AccelerationStructureBuildInput
+                    );
+                }
+                if (input.spheres.vertexRadiusBuffers[i].buffer)
+                {
+                    requireBufferState(
+                        checked_cast<BufferImpl*>(input.spheres.vertexRadiusBuffers[i].buffer),
+                        ResourceState::AccelerationStructureBuildInput
+                    );
+                }
+            }
+            if (input.spheres.indexBuffer.buffer)
+            {
+                requireBufferState(
+                    checked_cast<BufferImpl*>(input.spheres.indexBuffer.buffer),
+                    ResourceState::AccelerationStructureBuildInput
+                );
+            }
+            break;
+        case AccelerationStructureBuildInputType::LinearSweptSpheres:
+            for (uint32_t i = 0; i < input.linearSweptSpheres.vertexBufferCount; ++i)
+            {
+                if (input.linearSweptSpheres.vertexPositionBuffers[i].buffer)
+                {
+                    requireBufferState(
+                        checked_cast<BufferImpl*>(input.linearSweptSpheres.vertexPositionBuffers[i].buffer),
+                        ResourceState::AccelerationStructureBuildInput
+                    );
+                }
+                if (input.linearSweptSpheres.vertexRadiusBuffers[i].buffer)
+                {
+                    requireBufferState(
+                        checked_cast<BufferImpl*>(input.linearSweptSpheres.vertexRadiusBuffers[i].buffer),
+                        ResourceState::AccelerationStructureBuildInput
+                    );
+                }
+            }
+            if (input.linearSweptSpheres.indexBuffer.buffer)
+            {
+                requireBufferState(
+                    checked_cast<BufferImpl*>(input.linearSweptSpheres.indexBuffer.buffer),
+                    ResourceState::AccelerationStructureBuildInput
+                );
+            }
+            break;
+        }
+    }
+
+    commitBarriers();
 
 #if SLANG_RHI_ENABLE_NVAPI
-    if (NVAPIUtil::isAvailable())
+    if (m_device->m_nvapiEnabled)
     {
-
         NVAPI_D3D12_BUILD_RAYTRACING_ACCELERATION_STRUCTURE_DESC_EX desc = {};
         desc.destAccelerationStructureData = dst->getDeviceAddress();
         desc.sourceAccelerationStructureData = src ? src->getDeviceAddress() : 0;
@@ -1187,8 +1309,13 @@ void CommandRecorder::cmdBuildAccelerationStructure(const commands::BuildAcceler
 
 void CommandRecorder::cmdCopyAccelerationStructure(const commands::CopyAccelerationStructure& cmd)
 {
-    auto dst = checked_cast<AccelerationStructureImpl*>(cmd.dst);
-    auto src = checked_cast<AccelerationStructureImpl*>(cmd.src);
+    AccelerationStructureImpl* dst = checked_cast<AccelerationStructureImpl*>(cmd.dst);
+    AccelerationStructureImpl* src = checked_cast<AccelerationStructureImpl*>(cmd.src);
+
+    requireBufferState(dst->m_buffer, ResourceState::AccelerationStructureWrite);
+    requireBufferState(src->m_buffer, ResourceState::AccelerationStructureRead);
+    commitBarriers();
+
     D3D12_RAYTRACING_ACCELERATION_STRUCTURE_COPY_MODE copyMode;
     switch (cmd.mode)
     {
@@ -1221,7 +1348,13 @@ void CommandRecorder::cmdQueryAccelerationStructureProperties(const commands::Qu
 
 void CommandRecorder::cmdSerializeAccelerationStructure(const commands::SerializeAccelerationStructure& cmd)
 {
-    auto src = checked_cast<AccelerationStructureImpl*>(cmd.src);
+    BufferImpl* dstBuffer = checked_cast<BufferImpl*>(cmd.dst.buffer);
+    AccelerationStructureImpl* src = checked_cast<AccelerationStructureImpl*>(cmd.src);
+
+    requireBufferState(dstBuffer, ResourceState::UnorderedAccess);
+    requireBufferState(src->m_buffer, ResourceState::AccelerationStructureRead);
+    commitBarriers();
+
     m_cmdList4->CopyRaytracingAccelerationStructure(
         cmd.dst.getDeviceAddress(),
         src->getDeviceAddress(),
@@ -1231,7 +1364,13 @@ void CommandRecorder::cmdSerializeAccelerationStructure(const commands::Serializ
 
 void CommandRecorder::cmdDeserializeAccelerationStructure(const commands::DeserializeAccelerationStructure& cmd)
 {
-    auto dst = checked_cast<AccelerationStructureImpl*>(cmd.dst);
+    AccelerationStructureImpl* dst = checked_cast<AccelerationStructureImpl*>(cmd.dst);
+    BufferImpl* srcBuffer = checked_cast<BufferImpl*>(cmd.src.buffer);
+
+    requireBufferState(dst->m_buffer, ResourceState::AccelerationStructureWrite);
+    requireBufferState(srcBuffer, ResourceState::ShaderResource);
+    commitBarriers();
+
     m_cmdList4->CopyRaytracingAccelerationStructure(
         dst->getDeviceAddress(),
         cmd.src.getDeviceAddress(),
@@ -1239,17 +1378,125 @@ void CommandRecorder::cmdDeserializeAccelerationStructure(const commands::Deseri
     );
 }
 
+void CommandRecorder::cmdExecuteClusterOperation(const commands::ExecuteClusterOperation& cmd)
+{
+#if SLANG_RHI_ENABLE_NVAPI
+    if (!m_device->m_nvapiEnabled)
+        return;
+
+    const ClusterOperationDesc& desc = cmd.desc;
+
+    if (desc.params.maxArgCount == 0)
+        return;
+
+    BufferImpl* argCountBuffer = checked_cast<BufferImpl*>(desc.argCountBuffer.buffer);
+    BufferImpl* argsBuffer = checked_cast<BufferImpl*>(desc.argsBuffer.buffer);
+    BufferImpl* scratchBuffer = checked_cast<BufferImpl*>(desc.scratchBuffer.buffer);
+    BufferImpl* addressesBuffer = checked_cast<BufferImpl*>(desc.addressesBuffer.buffer);
+    BufferImpl* resultBuffer = checked_cast<BufferImpl*>(desc.resultBuffer.buffer);
+    BufferImpl* sizesBuffer = checked_cast<BufferImpl*>(desc.sizesBuffer.buffer);
+
+    requireBufferState(argsBuffer, ResourceState::ShaderResource);
+    if (argCountBuffer)
+        requireBufferState(argCountBuffer, ResourceState::ShaderResource);
+    requireBufferState(scratchBuffer, ResourceState::UnorderedAccess);
+    if (addressesBuffer)
+        requireBufferState(addressesBuffer, ResourceState::UnorderedAccess);
+    if (resultBuffer)
+        requireBufferState(resultBuffer, ResourceState::AccelerationStructureWrite);
+    if (sizesBuffer)
+        requireBufferState(sizesBuffer, ResourceState::UnorderedAccess);
+    commitBarriers();
+
+    NVAPI_D3D12_RAYTRACING_MULTI_INDIRECT_CLUSTER_OPERATION_DESC nvapiDesc = {};
+    nvapiDesc.inputs = translateClusterOperationParams(desc.params);
+    nvapiDesc.addressResolutionFlags =
+        NVAPI_D3D12_RAYTRACING_MULTI_INDIRECT_CLUSTER_OPERATION_ADDRESS_RESOLUTION_FLAG_NONE;
+
+    // Input buffers
+    if (desc.argCountBuffer)
+    {
+        nvapiDesc.indirectArgCount = desc.argCountBuffer.getDeviceAddress();
+    }
+    if (desc.argsBuffer)
+    {
+        nvapiDesc.indirectArgArray.StartAddress = desc.argsBuffer.getDeviceAddress();
+        nvapiDesc.indirectArgArray.StrideInBytes = desc.argsBufferStride;
+    }
+    nvapiDesc.batchScratchData = desc.scratchBuffer.getDeviceAddress();
+
+    // Input / output buffers
+    if (desc.addressesBuffer)
+    {
+        nvapiDesc.destinationAddressArray.StartAddress = desc.addressesBuffer.getDeviceAddress();
+        nvapiDesc.destinationAddressArray.StrideInBytes = desc.addressesBufferStride;
+    }
+
+    // Output buffers
+    if (desc.resultBuffer)
+    {
+        nvapiDesc.batchResultData = desc.resultBuffer.getDeviceAddress();
+    }
+    if (sizesBuffer)
+    {
+        nvapiDesc.resultSizeArray.StartAddress = desc.sizesBuffer.getDeviceAddress();
+        nvapiDesc.resultSizeArray.StrideInBytes = desc.sizesBufferStride;
+    }
+
+    NVAPI_RAYTRACING_EXECUTE_MULTI_INDIRECT_CLUSTER_OPERATION_PARAMS clusterOpParams = {};
+    clusterOpParams.version = NVAPI_RAYTRACING_EXECUTE_MULTI_INDIRECT_CLUSTER_OPERATION_PARAMS_VER;
+    clusterOpParams.pDesc = &nvapiDesc;
+
+    SLANG_RHI_NVAPI_CHECK(NvAPI_D3D12_RaytracingExecuteMultiIndirectClusterOperation(m_cmdList4, &clusterOpParams));
+
+#else  // SLANG_RHI_ENABLE_NVAPI
+    SLANG_UNUSED(cmd);
+    NOT_SUPPORTED(executeClusterOperation);
+#endif // SLANG_RHI_ENABLE_NVAPI
+}
+
 void CommandRecorder::cmdConvertCooperativeVectorMatrix(const commands::ConvertCooperativeVectorMatrix& cmd)
 {
 #if SLANG_RHI_ENABLE_NVAPI
-    short_vector<NVAPI_CONVERT_COOPERATIVE_VECTOR_MATRIX_DESC> descs;
-    for (uint32_t i = 0; i < cmd.descCount; i++)
+    BufferImpl* dstBuffer = checked_cast<BufferImpl*>(cmd.dstBuffer);
+    BufferImpl* srcBuffer = checked_cast<BufferImpl*>(cmd.srcBuffer);
+
+    requireBufferState(dstBuffer, ResourceState::UnorderedAccess);
+    requireBufferState(srcBuffer, ResourceState::ShaderResource);
+    commitBarriers();
+
+    short_vector<NVAPI_CONVERT_COOPERATIVE_VECTOR_MATRIX_DESC> nvDescs;
+    for (uint32_t i = 0; i < cmd.matrixCount; i++)
     {
-        descs.push_back(translateConvertCooperativeVectorMatrixDesc(cmd.descs[i], true));
+        const CooperativeVectorMatrixDesc& dstDesc = cmd.dstDescs[i];
+        const CooperativeVectorMatrixDesc& srcDesc = cmd.srcDescs[i];
+        NVAPI_CONVERT_COOPERATIVE_VECTOR_MATRIX_DESC nvDesc = {};
+        nvDesc.version = NVAPI_CONVERT_COOPERATIVE_VECTOR_MATRIX_DESC_VER1;
+        nvDesc.srcSize = srcDesc.size;
+        nvDesc.srcData.bIsDeviceAlloc = true;
+        nvDesc.srcData.deviceAddress = srcBuffer->getDeviceAddress() + srcDesc.offset;
+        nvDesc.pDstSize = (size_t*)&dstDesc.size;
+        nvDesc.dstData.bIsDeviceAlloc = true;
+        nvDesc.dstData.deviceAddress = dstBuffer->getDeviceAddress() + dstDesc.offset;
+        nvDesc.srcComponentType = translateCooperativeVectorComponentType(srcDesc.componentType);
+        nvDesc.dstComponentType = translateCooperativeVectorComponentType(dstDesc.componentType);
+        nvDesc.numRows = srcDesc.rowCount;
+        nvDesc.numColumns = srcDesc.colCount;
+        nvDesc.srcLayout = translateCooperativeVectorMatrixLayout(srcDesc.layout);
+        nvDesc.srcStride = srcDesc.rowColumnStride;
+        nvDesc.dstLayout = translateCooperativeVectorMatrixLayout(dstDesc.layout);
+        nvDesc.dstStride = dstDesc.rowColumnStride;
+        nvDescs.push_back(nvDesc);
     }
-    SLANG_RHI_NVAPI_CHECK(
-        NvAPI_D3D12_ConvertCooperativeVectorMatrixMultiple(m_device->m_device, m_cmdList, descs.data(), descs.size())
-    );
+    SLANG_RHI_NVAPI_CHECK(NvAPI_D3D12_ConvertCooperativeVectorMatrixMultiple(
+        m_device->m_device,
+        m_cmdList,
+        nvDescs.data(),
+        (NvU32)nvDescs.size()
+    ));
+
+    requireBufferState(dstBuffer, ResourceState::ShaderResource);
+    commitBarriers();
 #else
     SLANG_UNUSED(cmd);
 #endif
@@ -1265,15 +1512,26 @@ void CommandRecorder::cmdSetTextureState(const commands::SetTextureState& cmd)
     m_stateTracking.setTextureState(checked_cast<TextureImpl*>(cmd.texture), cmd.subresourceRange, cmd.state);
 }
 
+void CommandRecorder::cmdGlobalBarrier(const commands::GlobalBarrier& cmd)
+{
+    // Global barrier on D3D12 is implemented with a UAV barrier pointing at null resource.
+    // https://learn.microsoft.com/en-us/windows/win32/direct3d12/using-resource-barriers-to-synchronize-resource-states-in-direct3d-12
+    // TODO: Look at using D3D12 advanced barriers when available, currently only experimental in agility sdk though.
+    D3D12_RESOURCE_BARRIER barrier = {};
+    barrier.Type = D3D12_RESOURCE_BARRIER_TYPE_UAV;
+    barrier.UAV.pResource = nullptr;
+    m_cmdList->ResourceBarrier(1, &barrier);
+}
+
 void CommandRecorder::cmdPushDebugGroup(const commands::PushDebugGroup& cmd)
 {
     auto beginEvent = m_device->m_BeginEventOnCommandList;
     if (beginEvent)
     {
         UINT64 color = 0xff000000;
-        color |= uint8_t(cmd.rgbColor[0] * 255.0f) << 16;
-        color |= uint8_t(cmd.rgbColor[1] * 255.0f) << 8;
-        color |= uint8_t(cmd.rgbColor[2] * 255.0f);
+        color |= uint8_t(cmd.color.r * 255.0f) << 16;
+        color |= uint8_t(cmd.color.g * 255.0f) << 8;
+        color |= uint8_t(cmd.color.b * 255.0f);
         beginEvent(m_cmdList, color, cmd.name);
     }
 }
@@ -1293,9 +1551,9 @@ void CommandRecorder::cmdInsertDebugMarker(const commands::InsertDebugMarker& cm
     if (setMarker)
     {
         UINT64 color = 0xff000000;
-        color |= uint8_t(cmd.rgbColor[0] * 255.0f) << 16;
-        color |= uint8_t(cmd.rgbColor[1] * 255.0f) << 8;
-        color |= uint8_t(cmd.rgbColor[2] * 255.0f);
+        color |= uint8_t(cmd.color.r * 255.0f) << 16;
+        color |= uint8_t(cmd.color.g * 255.0f) << 8;
+        color |= uint8_t(cmd.color.b * 255.0f);
         setMarker(m_cmdList, color, cmd.name);
     }
 }
@@ -1389,88 +1647,79 @@ void CommandRecorder::requireTextureState(TextureImpl* texture, SubresourceRange
 
 void CommandRecorder::commitBarriers()
 {
+    if (testing::gDebugDisableStateTracking)
+        return;
+
     short_vector<D3D12_RESOURCE_BARRIER, 16> barriers;
 
     for (const auto& bufferBarrier : m_stateTracking.getBufferBarriers())
     {
         BufferImpl* buffer = checked_cast<BufferImpl*>(bufferBarrier.buffer);
         D3D12_RESOURCE_BARRIER barrier = {};
-        bool isUAVBarrier =
-            (bufferBarrier.stateBefore == bufferBarrier.stateAfter &&
-             bufferBarrier.stateAfter == ResourceState::UnorderedAccess);
-        if (isUAVBarrier)
-        {
-            barrier.Type = D3D12_RESOURCE_BARRIER_TYPE_UAV;
-            barrier.UAV.pResource = buffer->m_resource;
-        }
-        else
+        D3D12_RESOURCE_STATES stateBefore = translateResourceState(bufferBarrier.stateBefore);
+        D3D12_RESOURCE_STATES stateAfter = translateResourceState(bufferBarrier.stateAfter);
+        // Acceleration structure buffers need to be treated specially.
+        // D3D12 doesn't allow to transition to/from D3D12_RESOURCE_STATE_RAYTRACING_ACCELERATION_STRUCTURE state.
+        // Instead, UAV barriers are used to synchronize accesses.
+        if (stateBefore != stateAfter &&
+            ((stateBefore & D3D12_RESOURCE_STATE_RAYTRACING_ACCELERATION_STRUCTURE) == 0) &&
+            ((stateAfter & D3D12_RESOURCE_STATE_RAYTRACING_ACCELERATION_STRUCTURE) == 0))
         {
             barrier.Type = D3D12_RESOURCE_BARRIER_TYPE_TRANSITION;
             barrier.Transition.pResource = buffer->m_resource;
-            barrier.Transition.StateBefore = D3DUtil::getResourceState(bufferBarrier.stateBefore);
-            barrier.Transition.StateAfter = D3DUtil::getResourceState(bufferBarrier.stateAfter);
-            barrier.Transition.Subresource = 0;
-            if (barrier.Transition.StateBefore == barrier.Transition.StateAfter)
-            {
-                continue;
-            }
+            barrier.Transition.StateBefore = stateBefore;
+            barrier.Transition.StateAfter = stateAfter;
+            barrier.Transition.Subresource = D3D12_RESOURCE_BARRIER_ALL_SUBRESOURCES;
+            barriers.push_back(barrier);
+        }
+        else if ((bufferBarrier.stateBefore == ResourceState::AccelerationStructureWrite &&
+                  bufferBarrier.stateAfter == ResourceState::AccelerationStructureRead) ||
+                 (bufferBarrier.stateAfter == ResourceState::AccelerationStructureRead &&
+                  bufferBarrier.stateBefore == ResourceState::AccelerationStructureWrite) ||
+                 ((stateAfter & D3D12_RESOURCE_STATE_UNORDERED_ACCESS) != 0))
+        {
+            barrier.Type = D3D12_RESOURCE_BARRIER_TYPE_UAV;
+            barrier.UAV.pResource = buffer->m_resource;
+            barriers.push_back(barrier);
         }
-        barriers.push_back(barrier);
     }
 
     for (const auto& textureBarrier : m_stateTracking.getTextureBarriers())
     {
         TextureImpl* texture = checked_cast<TextureImpl*>(textureBarrier.texture);
         D3D12_RESOURCE_BARRIER barrier = {};
-        if (textureBarrier.entireTexture)
+        D3D12_RESOURCE_STATES stateBefore = translateResourceState(textureBarrier.stateBefore);
+        D3D12_RESOURCE_STATES stateAfter = translateResourceState(textureBarrier.stateAfter);
+        if (stateBefore != stateAfter)
         {
-            bool isUAVBarrier =
-                (textureBarrier.stateBefore == textureBarrier.stateAfter &&
-                 textureBarrier.stateAfter == ResourceState::UnorderedAccess);
-            if (isUAVBarrier)
+            barrier.Type = D3D12_RESOURCE_BARRIER_TYPE_TRANSITION;
+            barrier.Transition.pResource = texture->m_resource;
+            barrier.Transition.StateBefore = stateBefore;
+            barrier.Transition.StateAfter = stateAfter;
+            if (textureBarrier.entireTexture)
             {
-                barrier.Type = D3D12_RESOURCE_BARRIER_TYPE_UAV;
-                barrier.UAV.pResource = texture->m_resource;
+                barrier.Transition.Subresource = D3D12_RESOURCE_BARRIER_ALL_SUBRESOURCES;
+                barriers.push_back(barrier);
             }
             else
             {
-                barrier.Type = D3D12_RESOURCE_BARRIER_TYPE_TRANSITION;
-                barrier.Transition.pResource = texture->m_resource;
-                barrier.Transition.StateBefore = D3DUtil::getResourceState(textureBarrier.stateBefore);
-                barrier.Transition.StateAfter = D3DUtil::getResourceState(textureBarrier.stateAfter);
-                barrier.Transition.Subresource = D3D12_RESOURCE_BARRIER_ALL_SUBRESOURCES;
-                if (barrier.Transition.StateBefore == barrier.Transition.StateAfter)
+                uint32_t mipCount = texture->m_desc.mipCount;
+                uint32_t layerCount = texture->m_desc.getLayerCount();
+                DXGI_FORMAT d3dFormat = getMapFormat(texture->m_desc.format);
+                uint32_t planeCount = getPlaneSliceCount(d3dFormat);
+                for (uint32_t planeIndex = 0; planeIndex < planeCount; ++planeIndex)
                 {
-                    continue;
+                    barrier.Transition.Subresource =
+                        getSubresourceIndex(textureBarrier.mip, textureBarrier.layer, planeIndex, mipCount, layerCount);
+                    barriers.push_back(barrier);
                 }
             }
-            barriers.push_back(barrier);
         }
-        else
+        else if ((stateAfter & D3D12_RESOURCE_STATE_UNORDERED_ACCESS) != 0)
         {
-            uint32_t mipCount = texture->m_desc.mipCount;
-            uint32_t layerCount = texture->m_desc.getLayerCount();
-            DXGI_FORMAT d3dFormat = D3DUtil::getMapFormat(texture->m_desc.format);
-            uint32_t planeCount = D3DUtil::getPlaneSliceCount(d3dFormat);
-            barrier.Type = D3D12_RESOURCE_BARRIER_TYPE_TRANSITION;
-            barrier.Transition.pResource = texture->m_resource;
-            barrier.Transition.StateBefore = D3DUtil::getResourceState(textureBarrier.stateBefore);
-            barrier.Transition.StateAfter = D3DUtil::getResourceState(textureBarrier.stateAfter);
-            if (barrier.Transition.StateBefore == barrier.Transition.StateAfter)
-            {
-                continue;
-            }
-            for (uint32_t planeIndex = 0; planeIndex < planeCount; ++planeIndex)
-            {
-                barrier.Transition.Subresource = D3DUtil::getSubresourceIndex(
-                    textureBarrier.mip,
-                    textureBarrier.layer,
-                    planeIndex,
-                    mipCount,
-                    layerCount
-                );
-                barriers.push_back(barrier);
-            }
+            barrier.Type = D3D12_RESOURCE_BARRIER_TYPE_UAV;
+            barrier.UAV.pResource = texture->m_resource;
+            barriers.push_back(barrier);
         }
     }
 
@@ -1529,17 +1778,20 @@ Result CommandQueueImpl::getOrCreateCommandBuffer(CommandBufferImpl** outCommand
     {
         commandBuffer = m_commandBuffersPool.front();
         m_commandBuffersPool.pop_front();
+        commandBuffer->setInternalReferenceCount(0);
     }
     returnRefPtr(outCommandBuffer, commandBuffer);
     return SLANG_OK;
 }
 
-void CommandQueueImpl::retireUnfinishedCommandBuffer(CommandBufferImpl* commandBuffer)
+void CommandQueueImpl::retireCommandBuffer(CommandBufferImpl* commandBuffer)
 {
-    std::lock_guard<std::mutex> lock(m_mutex);
-    commandBuffer->m_d3dCommandList->Close();
     commandBuffer->reset();
-    m_commandBuffersPool.push_back(commandBuffer);
+    {
+        std::lock_guard<std::mutex> lock(m_mutex);
+        m_commandBuffersPool.push_back(commandBuffer);
+        commandBuffer->setInternalReferenceCount(1);
+    }
 }
 
 void CommandQueueImpl::retireCommandBuffers()
@@ -1552,17 +1804,16 @@ void CommandQueueImpl::retireCommandBuffers()
     {
         if (commandBuffer->m_submissionID <= lastFinishedID)
         {
-            commandBuffer->reset();
-            {
-                std::lock_guard<std::mutex> lock(m_mutex);
-                m_commandBuffersPool.push_back(commandBuffer);
-            }
+            retireCommandBuffer(commandBuffer);
         }
         else
         {
             m_commandBuffersInFlight.push_back(commandBuffer);
         }
     }
+
+    // Flush all device heaps
+    getDevice<DeviceImpl>()->flushHeaps();
 }
 
 uint64_t CommandQueueImpl::updateLastFinishedID()
@@ -1654,7 +1905,9 @@ CommandEncoderImpl::~CommandEncoderImpl()
     // If the command buffer was not used, return it to the pool.
     if (m_commandBuffer)
     {
-        m_queue->retireUnfinishedCommandBuffer(m_commandBuffer);
+        // Need to close the d3d12 command list because we never recorded commands.
+        m_commandBuffer->m_d3dCommandList->Close();
+        m_queue->retireCommandBuffer(m_commandBuffer);
     }
 }
 
