@@ -4,9 +4,11 @@
 #include "d3d12-shader-program.h"
 #include "d3d12-shader-object-layout.h"
 #include "d3d12-input-layout.h"
+#include "d3d12-utils.h"
 
 #include "core/stable_vector.h"
 #include "core/string.h"
+#include "core/sha1.h"
 
 #include <climits>
 
@@ -14,8 +16,192 @@
 
 namespace rhi::d3d12 {
 
-RenderPipelineImpl::RenderPipelineImpl(Device* device)
-    : RenderPipeline(device)
+void hashShader(SHA1& sha1, const D3D12_SHADER_BYTECODE& shaderBytecode)
+{
+    if (shaderBytecode.pShaderBytecode && shaderBytecode.BytecodeLength)
+    {
+        sha1.update(shaderBytecode.pShaderBytecode, shaderBytecode.BytecodeLength);
+    }
+}
+
+template<typename T>
+void hashValue(SHA1& sha1, const T& value)
+{
+    sha1.update(&value, sizeof(value));
+}
+
+void hashString(SHA1& sha1, const char* str)
+{
+    if (str)
+    {
+        sha1.update(str, strlen(str));
+    }
+}
+
+inline void hashDevice(SHA1& sha1, DeviceImpl* device)
+{
+    const AdapterLUID& luid = device->getInfo().adapterLUID;
+    sha1.update(luid.luid, sizeof(luid.luid));
+}
+
+inline void hashPipelineDesc(SHA1& sha1, const D3D12_GRAPHICS_PIPELINE_STATE_DESC* desc)
+{
+    hashShader(sha1, desc->VS);
+    hashShader(sha1, desc->PS);
+    hashShader(sha1, desc->DS);
+    hashShader(sha1, desc->HS);
+    hashShader(sha1, desc->GS);
+    for (uint32_t i = 0; i < desc->StreamOutput.NumEntries; ++i)
+    {
+        const D3D12_SO_DECLARATION_ENTRY& entry = desc->StreamOutput.pSODeclaration[i];
+        hashValue(sha1, entry.Stream);
+        hashString(sha1, entry.SemanticName);
+        hashValue(sha1, entry.SemanticIndex);
+        hashValue(sha1, entry.StartComponent);
+        hashValue(sha1, entry.ComponentCount);
+        hashValue(sha1, entry.OutputSlot);
+    }
+    for (uint32_t i = 0; i < desc->StreamOutput.NumStrides; ++i)
+    {
+        hashValue(sha1, desc->StreamOutput.pBufferStrides[i]);
+    }
+    hashValue(sha1, desc->StreamOutput.RasterizedStream);
+    hashValue(sha1, desc->BlendState);
+    hashValue(sha1, desc->SampleMask);
+    hashValue(sha1, desc->RasterizerState);
+    hashValue(sha1, desc->DepthStencilState);
+    for (uint32_t i = 0; i < desc->InputLayout.NumElements; ++i)
+    {
+        const D3D12_INPUT_ELEMENT_DESC& element = desc->InputLayout.pInputElementDescs[i];
+        hashString(sha1, element.SemanticName);
+        hashValue(sha1, element.SemanticIndex);
+        hashValue(sha1, element.Format);
+        hashValue(sha1, element.InputSlot);
+        hashValue(sha1, element.AlignedByteOffset);
+        hashValue(sha1, element.InputSlotClass);
+        hashValue(sha1, element.InstanceDataStepRate);
+    }
+    hashValue(sha1, desc->IBStripCutValue);
+    hashValue(sha1, desc->PrimitiveTopologyType);
+    hashValue(sha1, desc->NumRenderTargets);
+    for (uint32_t i = 0; i < desc->NumRenderTargets; i++)
+    {
+        hashValue(sha1, desc->RTVFormats[i]);
+    }
+    hashValue(sha1, desc->DSVFormat);
+    hashValue(sha1, desc->SampleDesc);
+    hashValue(sha1, desc->NodeMask);
+    hashValue(sha1, desc->Flags);
+}
+
+inline void hashPipelineDesc(SHA1& sha1, const D3D12_COMPUTE_PIPELINE_STATE_DESC* desc)
+{
+    hashShader(sha1, desc->CS);
+    hashValue(sha1, desc->NodeMask);
+    hashValue(sha1, desc->Flags);
+}
+
+inline void getPipelineCacheKey(
+    DeviceImpl* device,
+    const D3D12_GRAPHICS_PIPELINE_STATE_DESC* desc,
+    ISlangBlob** outBlob
+)
+{
+    SHA1 sha1;
+    hashDevice(sha1, device);
+    hashPipelineDesc(sha1, desc);
+    SHA1::Digest digest = sha1.getDigest();
+    ComPtr<ISlangBlob> blob = OwnedBlob::create(digest.data(), digest.size());
+    returnComPtr(outBlob, blob);
+}
+
+inline void getPipelineCacheKey(DeviceImpl* device, const D3D12_COMPUTE_PIPELINE_STATE_DESC* desc, ISlangBlob** outBlob)
+{
+    SHA1 sha1;
+    hashDevice(sha1, device);
+    hashPipelineDesc(sha1, desc);
+    SHA1::Digest digest = sha1.getDigest();
+    ComPtr<ISlangBlob> blob = OwnedBlob::create(digest.data(), digest.size());
+    returnComPtr(outBlob, blob);
+}
+
+template<typename PipelineDesc, typename PipelineState>
+Result createPipelineWithCache(
+    DeviceImpl* device,
+    PipelineDesc* desc,
+    Result (*createPipelineFunc)(DeviceImpl* device, PipelineDesc* desc, PipelineState** outPipeline),
+    PipelineState** outPipeline,
+    bool& outCached,
+    size_t& outCacheSize
+)
+{
+    outCached = false;
+    outCacheSize = 0;
+
+    // Early out if cache is not enabled.
+    if (!device->m_persistentPipelineCache)
+    {
+        return createPipelineFunc(device, desc, outPipeline);
+    }
+
+    bool writeCache = true;
+    ComPtr<ISlangBlob> pipelineCacheKey;
+    ComPtr<ISlangBlob> pipelineCacheData;
+    PipelineState* pipeline = nullptr;
+
+    // Create pipeline cache key.
+    getPipelineCacheKey(device, desc, pipelineCacheKey.writeRef());
+
+    // Query pipeline cache.
+    if (SLANG_FAILED(device->m_persistentPipelineCache->queryCache(pipelineCacheKey, pipelineCacheData.writeRef())))
+    {
+        pipelineCacheData = nullptr;
+    }
+
+    // Try create pipeline from cache.
+    if (pipelineCacheData)
+    {
+        desc->CachedPSO.pCachedBlob = pipelineCacheData->getBufferPointer();
+        desc->CachedPSO.CachedBlobSizeInBytes = pipelineCacheData->getBufferSize();
+        if (createPipelineFunc(device, desc, &pipeline) == SLANG_OK)
+        {
+            writeCache = false;
+            outCached = true;
+            outCacheSize = pipelineCacheData->getBufferSize();
+        }
+        else
+        {
+            desc->CachedPSO.pCachedBlob = nullptr;
+            desc->CachedPSO.CachedBlobSizeInBytes = 0;
+            pipeline = nullptr;
+        }
+    }
+
+    // Create pipeline if not found in cache.
+    if (!pipeline)
+    {
+        SLANG_RETURN_ON_FAIL(createPipelineFunc(device, desc, &pipeline));
+    }
+
+    // Write to the cache.
+    if (writeCache)
+    {
+        ComPtr<ID3DBlob> cachedBlob;
+        if (SLANG_SUCCEEDED(pipeline->GetCachedBlob(cachedBlob.writeRef())) && cachedBlob)
+        {
+            pipelineCacheData = UnownedBlob::create(cachedBlob->GetBufferPointer(), cachedBlob->GetBufferSize());
+            device->m_persistentPipelineCache->writeCache(pipelineCacheKey, pipelineCacheData);
+            outCacheSize = pipelineCacheData->getBufferSize();
+        }
+    }
+
+    *outPipeline = pipeline;
+    return SLANG_OK;
+}
+
+
+RenderPipelineImpl::RenderPipelineImpl(Device* device, const RenderPipelineDesc& desc)
+    : RenderPipeline(device, desc)
 {
 }
 
@@ -28,25 +214,29 @@ Result RenderPipelineImpl::getNativeHandle(NativeHandle* outHandle)
 
 Result DeviceImpl::createRenderPipeline2(const RenderPipelineDesc& desc, IRenderPipeline** outPipeline)
 {
+    TimePoint startTime = Timer::now();
+
     ShaderProgramImpl* program = checked_cast<ShaderProgramImpl*>(desc.program);
     SLANG_RHI_ASSERT(!program->m_shaders.empty());
     InputLayoutImpl* inputLayout = checked_cast<InputLayoutImpl*>(desc.inputLayout);
 
     ComPtr<ID3D12PipelineState> pipelineState;
+    bool cached = false;
+    size_t cacheSize = 0;
 
     // A helper to fill common fields between graphics and mesh pipeline descs
     const auto fillCommonGraphicsState = [&](auto& psoDesc)
     {
         psoDesc.pRootSignature = program->m_rootObjectLayout->m_rootSignature;
 
-        psoDesc.PrimitiveTopologyType = D3DUtil::getPrimitiveTopologyType(desc.primitiveTopology);
+        psoDesc.PrimitiveTopologyType = translatePrimitiveTopologyType(desc.primitiveTopology);
 
         uint32_t numRenderTargets = desc.targetCount;
 
         {
             if (desc.depthStencil.format != Format::Undefined)
             {
-                psoDesc.DSVFormat = D3DUtil::getFormatMapping(desc.depthStencil.format).rtvFormat;
+                psoDesc.DSVFormat = getFormatMapping(desc.depthStencil.format).rtvFormat;
             }
             else
             {
@@ -55,7 +245,7 @@ Result DeviceImpl::createRenderPipeline2(const RenderPipelineDesc& desc, IRender
             psoDesc.NumRenderTargets = numRenderTargets;
             for (uint32_t i = 0; i < numRenderTargets; i++)
             {
-                psoDesc.RTVFormats[i] = D3DUtil::getFormatMapping(desc.targets[i].format).rtvFormat;
+                psoDesc.RTVFormats[i] = getFormatMapping(desc.targets[i].format).rtvFormat;
             }
 
             psoDesc.SampleDesc.Count = desc.multisample.sampleCount;
@@ -65,8 +255,8 @@ Result DeviceImpl::createRenderPipeline2(const RenderPipelineDesc& desc, IRender
 
         {
             auto& rs = psoDesc.RasterizerState;
-            rs.FillMode = D3DUtil::getFillMode(desc.rasterizer.fillMode);
-            rs.CullMode = D3DUtil::getCullMode(desc.rasterizer.cullMode);
+            rs.FillMode = translateFillMode(desc.rasterizer.fillMode);
+            rs.CullMode = translateCullMode(desc.rasterizer.cullMode);
             rs.FrontCounterClockwise = desc.rasterizer.frontFace == FrontFaceMode::CounterClockwise ? TRUE : FALSE;
             rs.DepthBias = desc.rasterizer.depthBias;
             rs.DepthBiasClamp = desc.rasterizer.depthBiasClamp;
@@ -84,20 +274,20 @@ Result DeviceImpl::createRenderPipeline2(const RenderPipelineDesc& desc, IRender
             D3D12_BLEND_DESC& blend = psoDesc.BlendState;
             blend.IndependentBlendEnable = FALSE;
             blend.AlphaToCoverageEnable = desc.multisample.alphaToCoverageEnable ? TRUE : FALSE;
-            blend.RenderTarget[0].RenderTargetWriteMask = (uint8_t)RenderTargetWriteMask::EnableAll;
+            blend.RenderTarget[0].RenderTargetWriteMask = (UINT8)RenderTargetWriteMask::All;
             for (uint32_t i = 0; i < numRenderTargets; i++)
             {
                 auto& d3dDesc = blend.RenderTarget[i];
                 d3dDesc.BlendEnable = desc.targets[i].enableBlend ? TRUE : FALSE;
-                d3dDesc.BlendOp = D3DUtil::getBlendOp(desc.targets[i].color.op);
-                d3dDesc.BlendOpAlpha = D3DUtil::getBlendOp(desc.targets[i].alpha.op);
-                d3dDesc.DestBlend = D3DUtil::getBlendFactor(desc.targets[i].color.dstFactor);
-                d3dDesc.DestBlendAlpha = D3DUtil::getBlendFactor(desc.targets[i].alpha.dstFactor);
+                d3dDesc.BlendOp = translateBlendOp(desc.targets[i].color.op);
+                d3dDesc.BlendOpAlpha = translateBlendOp(desc.targets[i].alpha.op);
+                d3dDesc.DestBlend = translateBlendFactor(desc.targets[i].color.dstFactor);
+                d3dDesc.DestBlendAlpha = translateBlendFactor(desc.targets[i].alpha.dstFactor);
                 d3dDesc.LogicOp = D3D12_LOGIC_OP_NOOP;
                 d3dDesc.LogicOpEnable = FALSE;
-                d3dDesc.RenderTargetWriteMask = desc.targets[i].writeMask;
-                d3dDesc.SrcBlend = D3DUtil::getBlendFactor(desc.targets[i].color.srcFactor);
-                d3dDesc.SrcBlendAlpha = D3DUtil::getBlendFactor(desc.targets[i].alpha.srcFactor);
+                d3dDesc.RenderTargetWriteMask = (UINT8)desc.targets[i].writeMask;
+                d3dDesc.SrcBlend = translateBlendFactor(desc.targets[i].color.srcFactor);
+                d3dDesc.SrcBlendAlpha = translateBlendFactor(desc.targets[i].alpha.srcFactor);
             }
             auto equalBlendState = [](const ColorTargetDesc& a, const ColorTargetDesc& b)
             {
@@ -126,15 +316,15 @@ Result DeviceImpl::createRenderPipeline2(const RenderPipelineDesc& desc, IRender
             ds.DepthEnable = desc.depthStencil.depthTestEnable;
             ds.DepthWriteMask =
                 desc.depthStencil.depthWriteEnable ? D3D12_DEPTH_WRITE_MASK_ALL : D3D12_DEPTH_WRITE_MASK_ZERO;
-            ds.DepthFunc = D3DUtil::getComparisonFunc(desc.depthStencil.depthFunc);
+            ds.DepthFunc = translateComparisonFunc(desc.depthStencil.depthFunc);
             ds.StencilEnable = desc.depthStencil.stencilEnable;
             ds.StencilReadMask = (UINT8)desc.depthStencil.stencilReadMask;
             ds.StencilWriteMask = (UINT8)desc.depthStencil.stencilWriteMask;
-            ds.FrontFace = D3DUtil::translateStencilOpDesc(desc.depthStencil.frontFace);
-            ds.BackFace = D3DUtil::translateStencilOpDesc(desc.depthStencil.backFace);
+            ds.FrontFace = translateStencilOpDesc(desc.depthStencil.frontFace);
+            ds.BackFace = translateStencilOpDesc(desc.depthStencil.backFace);
         }
 
-        psoDesc.PrimitiveTopologyType = D3DUtil::getPrimitiveTopologyType(desc.primitiveTopology);
+        psoDesc.PrimitiveTopologyType = translatePrimitiveTopologyType(desc.primitiveTopology);
     };
 
     if (program->isMeshShaderProgram())
@@ -159,22 +349,9 @@ Result DeviceImpl::createRenderPipeline2(const RenderPipelineDesc& desc, IRender
             }
         }
         fillCommonGraphicsState(meshDesc);
-        if (m_pipelineCreationAPIDispatcher)
-        {
-            SLANG_RETURN_ON_FAIL(m_pipelineCreationAPIDispatcher->createMeshPipeline(
-                this,
-                program->linkedProgram.get(),
-                &meshDesc,
-                (void**)pipelineState.writeRef()
-            ));
-        }
-        else
-        {
-            CD3DX12_PIPELINE_STATE_STREAM2 meshStateStream{meshDesc};
-            D3D12_PIPELINE_STATE_STREAM_DESC streamDesc{sizeof(meshStateStream), &meshStateStream};
-
-            SLANG_RETURN_ON_FAIL(m_device5->CreatePipelineState(&streamDesc, IID_PPV_ARGS(pipelineState.writeRef())));
-        }
+        CD3DX12_PIPELINE_STATE_STREAM2 meshStateStream{meshDesc};
+        D3D12_PIPELINE_STATE_STREAM_DESC streamDesc{sizeof(meshStateStream), &meshStateStream};
+        SLANG_RETURN_ON_FAIL(m_device5->CreatePipelineState(&streamDesc, IID_PPV_ARGS(pipelineState.writeRef())));
     }
     else
     {
@@ -211,60 +388,78 @@ Result DeviceImpl::createRenderPipeline2(const RenderPipelineDesc& desc, IRender
 
         fillCommonGraphicsState(graphicsDesc);
 
-        if (m_pipelineCreationAPIDispatcher)
-        {
-            SLANG_RETURN_ON_FAIL(m_pipelineCreationAPIDispatcher->createRenderPipeline(
-                this,
-                program->linkedProgram.get(),
-                &graphicsDesc,
-                (void**)pipelineState.writeRef()
-            ));
-        }
-        else
-        {
+        Result result = createPipelineWithCache<D3D12_GRAPHICS_PIPELINE_STATE_DESC, ID3D12PipelineState>(
+            this,
+            &graphicsDesc,
+            [](DeviceImpl* device,
+               D3D12_GRAPHICS_PIPELINE_STATE_DESC* desc,
+               ID3D12PipelineState** outPipeline) -> Result
+            {
 #if SLANG_RHI_ENABLE_NVAPI
-            if (m_nvapiShaderExtension)
-            {
-                NVAPI_D3D12_PSO_SET_SHADER_EXTENSION_SLOT_DESC extensionDesc;
-                extensionDesc.baseVersion = NV_PSO_EXTENSION_DESC_VER;
-                extensionDesc.psoExtension = NV_PSO_SET_SHADER_EXTENSION_SLOT_AND_SPACE;
-                extensionDesc.version = NV_SET_SHADER_EXTENSION_SLOT_DESC_VER;
-                extensionDesc.uavSlot = m_nvapiShaderExtension.uavSlot;
-                extensionDesc.registerSpace = m_nvapiShaderExtension.registerSpace;
+                if (device->m_nvapiShaderExtension)
+                {
+                    NVAPI_D3D12_PSO_SET_SHADER_EXTENSION_SLOT_DESC extensionDesc;
+                    extensionDesc.baseVersion = NV_PSO_EXTENSION_DESC_VER;
+                    extensionDesc.psoExtension = NV_PSO_SET_SHADER_EXTENSION_SLOT_AND_SPACE;
+                    extensionDesc.version = NV_SET_SHADER_EXTENSION_SLOT_DESC_VER;
+                    extensionDesc.uavSlot = device->m_nvapiShaderExtension.uavSlot;
+                    extensionDesc.registerSpace = device->m_nvapiShaderExtension.registerSpace;
 
-                const NVAPI_D3D12_PSO_EXTENSION_DESC* extensions[] = {&extensionDesc};
+                    const NVAPI_D3D12_PSO_EXTENSION_DESC* extensions[] = {&extensionDesc};
 
-                // Now create the PSO.
-                SLANG_RHI_NVAPI_RETURN_ON_FAIL(NvAPI_D3D12_CreateGraphicsPipelineState(
-                    m_device,
-                    &graphicsDesc,
-                    SLANG_COUNT_OF(extensions),
-                    extensions,
-                    pipelineState.writeRef()
-                ));
-            }
-            else
+                    NvAPI_Status status = NvAPI_D3D12_CreateGraphicsPipelineState(
+                        device->m_device,
+                        desc,
+                        SLANG_COUNT_OF(extensions),
+                        extensions,
+                        outPipeline
+                    );
+                    return status == NVAPI_OK ? SLANG_OK : SLANG_FAIL;
+                }
+                else
 #endif // SLANG_RHI_ENABLE_NVAPI
-            {
-                SLANG_RETURN_ON_FAIL(
-                    m_device->CreateGraphicsPipelineState(&graphicsDesc, IID_PPV_ARGS(pipelineState.writeRef()))
-                );
-            }
-        }
+                {
+                    HRESULT hr = device->m_device->CreateGraphicsPipelineState(desc, IID_PPV_ARGS(outPipeline));
+                    return hr == S_OK ? SLANG_OK : SLANG_FAIL;
+                }
+            },
+            pipelineState.writeRef(),
+            cached,
+            cacheSize
+        );
+        SLANG_RETURN_ON_FAIL(result);
     }
 
-    RefPtr<RenderPipelineImpl> pipeline = new RenderPipelineImpl(this);
+    if (desc.label)
+    {
+        pipelineState->SetName(string::to_wstring(desc.label).c_str());
+    }
+
+    // Report the pipeline creation time.
+    if (m_shaderCompilationReporter)
+    {
+        m_shaderCompilationReporter->reportCreatePipeline(
+            program,
+            ShaderCompilationReporter::PipelineType::Render,
+            startTime,
+            Timer::now(),
+            cached,
+            cacheSize
+        );
+    }
+
+    RefPtr<RenderPipelineImpl> pipeline = new RenderPipelineImpl(this, desc);
     pipeline->m_program = program;
     pipeline->m_inputLayout = inputLayout;
     pipeline->m_rootObjectLayout = program->m_rootObjectLayout;
     pipeline->m_pipelineState = pipelineState;
-    pipeline->m_primitiveTopology = D3DUtil::getPrimitiveTopology(desc.primitiveTopology);
+    pipeline->m_primitiveTopology = translatePrimitiveTopology(desc.primitiveTopology);
     returnComPtr(outPipeline, pipeline);
     return SLANG_OK;
 }
 
-ComputePipelineImpl::ComputePipelineImpl(Device* device)
-    : ComputePipeline(device)
+ComputePipelineImpl::ComputePipelineImpl(Device* device, const ComputePipelineDesc& desc)
+    : ComputePipeline(device, desc)
 {
 }
 
@@ -277,11 +472,11 @@ Result ComputePipelineImpl::getNativeHandle(NativeHandle* outHandle)
 
 Result DeviceImpl::createComputePipeline2(const ComputePipelineDesc& desc, IComputePipeline** outPipeline)
 {
+    TimePoint startTime = Timer::now();
+
     ShaderProgramImpl* program = checked_cast<ShaderProgramImpl*>(desc.program);
     SLANG_RHI_ASSERT(!program->m_shaders.empty());
 
-    ComPtr<ID3D12PipelineState> pipelineState;
-
     // Describe and create the compute pipeline state object
     D3D12_COMPUTE_PIPELINE_STATE_DESC computeDesc = {};
     computeDesc.pRootSignature = desc.d3d12RootSignatureOverride
@@ -289,48 +484,67 @@ Result DeviceImpl::createComputePipeline2(const ComputePipelineDesc& desc, IComp
                                      : program->m_rootObjectLayout->m_rootSignature;
     computeDesc.CS = {program->m_shaders[0].code.data(), SIZE_T(program->m_shaders[0].code.size())};
 
-    if (m_pipelineCreationAPIDispatcher)
-    {
-        SLANG_RETURN_ON_FAIL(m_pipelineCreationAPIDispatcher->createComputePipeline(
-            this,
-            program->linkedProgram.get(),
-            &computeDesc,
-            (void**)pipelineState.writeRef()
-        ));
-    }
-    else
-    {
+    ComPtr<ID3D12PipelineState> pipelineState;
+    bool cached = false;
+    size_t cacheSize = 0;
+    Result result = createPipelineWithCache<D3D12_COMPUTE_PIPELINE_STATE_DESC, ID3D12PipelineState>(
+        this,
+        &computeDesc,
+        [](DeviceImpl* device, D3D12_COMPUTE_PIPELINE_STATE_DESC* desc, ID3D12PipelineState** outPipeline) -> Result
+        {
 #if SLANG_RHI_ENABLE_NVAPI
-        if (m_nvapiShaderExtension)
-        {
-            NVAPI_D3D12_PSO_SET_SHADER_EXTENSION_SLOT_DESC extensionDesc;
-            extensionDesc.baseVersion = NV_PSO_EXTENSION_DESC_VER;
-            extensionDesc.psoExtension = NV_PSO_SET_SHADER_EXTENSION_SLOT_AND_SPACE;
-            extensionDesc.version = NV_SET_SHADER_EXTENSION_SLOT_DESC_VER;
-            extensionDesc.uavSlot = m_nvapiShaderExtension.uavSlot;
-            extensionDesc.registerSpace = m_nvapiShaderExtension.registerSpace;
+            if (device->m_nvapiShaderExtension)
+            {
+                NVAPI_D3D12_PSO_SET_SHADER_EXTENSION_SLOT_DESC extensionDesc;
+                extensionDesc.baseVersion = NV_PSO_EXTENSION_DESC_VER;
+                extensionDesc.psoExtension = NV_PSO_SET_SHADER_EXTENSION_SLOT_AND_SPACE;
+                extensionDesc.version = NV_SET_SHADER_EXTENSION_SLOT_DESC_VER;
+                extensionDesc.uavSlot = device->m_nvapiShaderExtension.uavSlot;
+                extensionDesc.registerSpace = device->m_nvapiShaderExtension.registerSpace;
 
-            const NVAPI_D3D12_PSO_EXTENSION_DESC* extensions[] = {&extensionDesc};
+                const NVAPI_D3D12_PSO_EXTENSION_DESC* extensions[] = {&extensionDesc};
 
-            // Now create the PSO.
-            SLANG_RHI_NVAPI_RETURN_ON_FAIL(NvAPI_D3D12_CreateComputePipelineState(
-                m_device,
-                &computeDesc,
-                SLANG_COUNT_OF(extensions),
-                extensions,
-                pipelineState.writeRef()
-            ));
-        }
-        else
+                NvAPI_Status status = NvAPI_D3D12_CreateComputePipelineState(
+                    device->m_device,
+                    desc,
+                    SLANG_COUNT_OF(extensions),
+                    extensions,
+                    outPipeline
+                );
+                return status == NVAPI_OK ? SLANG_OK : SLANG_FAIL;
+            }
+            else
 #endif // SLANG_RHI_ENABLE_NVAPI
-        {
-            SLANG_RETURN_ON_FAIL(
-                m_device->CreateComputePipelineState(&computeDesc, IID_PPV_ARGS(pipelineState.writeRef()))
-            );
-        }
+            {
+                HRESULT hr = device->m_device->CreateComputePipelineState(desc, IID_PPV_ARGS(outPipeline));
+                return hr == S_OK ? SLANG_OK : SLANG_FAIL;
+            }
+        },
+        pipelineState.writeRef(),
+        cached,
+        cacheSize
+    );
+    SLANG_RETURN_ON_FAIL(result);
+
+    if (desc.label)
+    {
+        pipelineState->SetName(string::to_wstring(desc.label).c_str());
     }
 
-    RefPtr<ComputePipelineImpl> pipeline = new ComputePipelineImpl(this);
+    // Report the pipeline creation time.
+    if (m_shaderCompilationReporter)
+    {
+        m_shaderCompilationReporter->reportCreatePipeline(
+            program,
+            ShaderCompilationReporter::PipelineType::Compute,
+            startTime,
+            Timer::now(),
+            cached,
+            cacheSize
+        );
+    }
+
+    RefPtr<ComputePipelineImpl> pipeline = new ComputePipelineImpl(this, desc);
     pipeline->m_program = program;
     pipeline->m_rootObjectLayout = program->m_rootObjectLayout;
     pipeline->m_pipelineState = pipelineState;
@@ -338,8 +552,8 @@ Result DeviceImpl::createComputePipeline2(const ComputePipelineDesc& desc, IComp
     return SLANG_OK;
 }
 
-RayTracingPipelineImpl::RayTracingPipelineImpl(Device* device)
-    : RayTracingPipeline(device)
+RayTracingPipelineImpl::RayTracingPipelineImpl(Device* device, const RayTracingPipelineDesc& desc)
+    : RayTracingPipeline(device, desc)
 {
 }
 
@@ -357,19 +571,16 @@ Result DeviceImpl::createRayTracingPipeline2(const RayTracingPipelineDesc& desc,
         return SLANG_E_NOT_AVAILABLE;
     }
 
-#if SLANG_RHI_DXR
+    TimePoint startTime = Timer::now();
+
     ShaderProgramImpl* program = checked_cast<ShaderProgramImpl*>(desc.program);
     SLANG_RHI_ASSERT(!program->m_shaders.empty());
 
     ComPtr<ID3D12StateObject> stateObject;
 
-    auto slangGlobalScope = program->linkedProgram;
-    auto programLayout = slangGlobalScope->getLayout();
-
     std::vector<D3D12_STATE_SUBOBJECT> subObjects;
     stable_vector<D3D12_DXIL_LIBRARY_DESC> dxilLibraries;
     stable_vector<D3D12_HIT_GROUP_DESC> hitGroups;
-    stable_vector<ComPtr<ISlangBlob>> codeBlobs;
     stable_vector<D3D12_EXPORT_DESC> exports;
     stable_vector<const wchar_t*> strPtrs;
     ComPtr<ISlangBlob> diagnostics;
@@ -392,33 +603,14 @@ Result DeviceImpl::createRayTracingPipeline2(const RayTracingPipelineDesc& desc,
     pipelineConfigSubobject.pDesc = &pipelineConfig;
     subObjects.push_back(pipelineConfigSubobject);
 
-    auto compileShader =
-        [&](slang::EntryPointLayout* entryPointInfo, slang::IComponentType* component, SlangInt entryPointIndex)
+    for (const ShaderBinary& shader : program->m_shaders)
     {
-        ComPtr<ISlangBlob> codeBlob;
-        auto compileResult = getEntryPointCodeFromShaderCache(
-            component,
-            entryPointIndex,
-            0,
-            codeBlob.writeRef(),
-            diagnostics.writeRef()
-        );
-        if (diagnostics.get())
-        {
-            handleMessage(
-                compileResult == SLANG_OK ? DebugMessageType::Warning : DebugMessageType::Error,
-                DebugMessageSource::Slang,
-                (char*)diagnostics->getBufferPointer()
-            );
-        }
-        SLANG_RETURN_ON_FAIL(compileResult);
-        codeBlobs.push_back(codeBlob);
         D3D12_DXIL_LIBRARY_DESC library = {};
-        library.DXILLibrary.BytecodeLength = codeBlob->getBufferSize();
-        library.DXILLibrary.pShaderBytecode = codeBlob->getBufferPointer();
+        library.DXILLibrary.BytecodeLength = shader.code.size();
+        library.DXILLibrary.pShaderBytecode = shader.code.data();
         library.NumExports = 1;
         D3D12_EXPORT_DESC exportDesc = {};
-        exportDesc.Name = getWStr(entryPointInfo->getNameOverride());
+        exportDesc.Name = getWStr(shader.entryPointInfo->getNameOverride());
         exportDesc.ExportToRename = nullptr;
         exportDesc.Flags = D3D12_EXPORT_FLAG_NONE;
         exports.push_back(exportDesc);
@@ -429,24 +621,6 @@ Result DeviceImpl::createRayTracingPipeline2(const RayTracingPipelineDesc& desc,
         dxilLibraries.push_back(library);
         dxilSubObject.pDesc = &dxilLibraries.back();
         subObjects.push_back(dxilSubObject);
-        return SLANG_OK;
-    };
-
-    if (program->linkedEntryPoints.empty())
-    {
-        for (SlangUInt i = 0; i < programLayout->getEntryPointCount(); i++)
-        {
-            SLANG_RETURN_ON_FAIL(
-                compileShader(programLayout->getEntryPointByIndex(i), program->linkedProgram, (SlangInt)i)
-            );
-        }
-    }
-    else
-    {
-        for (auto& entryPoint : program->linkedEntryPoints)
-        {
-            SLANG_RETURN_ON_FAIL(compileShader(entryPoint->getLayout()->getEntryPointByIndex(0), entryPoint, 0));
-        }
     }
 
     for (uint32_t i = 0; i < desc.hitGroupCount; i++)
@@ -494,23 +668,37 @@ Result DeviceImpl::createRayTracingPipeline2(const RayTracingPipelineDesc& desc,
     globalSignatureSubobject.pDesc = &globalSignatureDesc;
     subObjects.push_back(globalSignatureSubobject);
 
-    if (m_pipelineCreationAPIDispatcher)
-    {
-        SLANG_RETURN_ON_FAIL(m_pipelineCreationAPIDispatcher->beforeCreateRayTracingState(this, slangGlobalScope));
-    }
-    else
-    {
 #if SLANG_RHI_ENABLE_NVAPI
-        if (m_nvapiShaderExtension)
+    bool nvapiResetPipelineStateOptions = false;
+    if (m_nvapiShaderExtension)
+    {
+        SLANG_RHI_NVAPI_RETURN_ON_FAIL(NvAPI_D3D12_SetNvShaderExtnSlotSpaceLocalThread(
+            m_device,
+            m_nvapiShaderExtension.uavSlot,
+            m_nvapiShaderExtension.registerSpace
+        ));
+
+        if (is_set(desc.flags, RayTracingPipelineFlags::EnableLinearSweptSpheres) ||
+            is_set(desc.flags, RayTracingPipelineFlags::EnableSpheres) ||
+            is_set(desc.flags, RayTracingPipelineFlags::EnableClusters))
         {
-            SLANG_RHI_NVAPI_RETURN_ON_FAIL(NvAPI_D3D12_SetNvShaderExtnSlotSpace(
-                m_device,
-                m_nvapiShaderExtension.uavSlot,
-                m_nvapiShaderExtension.registerSpace
-            ));
+            NVAPI_D3D12_SET_CREATE_PIPELINE_STATE_OPTIONS_PARAMS params = {};
+            params.version = NVAPI_D3D12_SET_CREATE_PIPELINE_STATE_OPTIONS_PARAMS_VER;
+
+            if (is_set(desc.flags, RayTracingPipelineFlags::EnableLinearSweptSpheres))
+                params.flags = NVAPI_D3D12_PIPELINE_CREATION_STATE_FLAGS_ENABLE_LSS_SUPPORT;
+            if (is_set(desc.flags, RayTracingPipelineFlags::EnableSpheres))
+                params.flags = NVAPI_D3D12_PIPELINE_CREATION_STATE_FLAGS_ENABLE_SPHERE_SUPPORT;
+            if (is_set(desc.flags, RayTracingPipelineFlags::EnableClusters))
+                params.flags = NVAPI_D3D12_PIPELINE_CREATION_STATE_FLAGS_ENABLE_CLUSTER_SUPPORT;
+
+            // TODO: This sets global state!
+            // Need to revisit if createRayTracingPipeline2 can get called from multiple threads.
+            SLANG_RHI_NVAPI_RETURN_ON_FAIL(NvAPI_D3D12_SetCreatePipelineStateOptions(m_device5, &params));
+            nvapiResetPipelineStateOptions = true;
         }
-#endif // SLANG_RHI_ENABLE_NVAPI
     }
+#endif // SLANG_RHI_ENABLE_NVAPI
 
     D3D12_STATE_OBJECT_DESC rtpsoDesc = {};
     rtpsoDesc.Type = D3D12_STATE_OBJECT_TYPE_RAYTRACING_PIPELINE;
@@ -518,29 +706,45 @@ Result DeviceImpl::createRayTracingPipeline2(const RayTracingPipelineDesc& desc,
     rtpsoDesc.pSubobjects = subObjects.data();
     SLANG_RETURN_ON_FAIL(m_device5->CreateStateObject(&rtpsoDesc, IID_PPV_ARGS(stateObject.writeRef())));
 
-    if (m_pipelineCreationAPIDispatcher)
-    {
-        SLANG_RETURN_ON_FAIL(m_pipelineCreationAPIDispatcher->afterCreateRayTracingState(this, slangGlobalScope));
-    }
-    else
-    {
 #if SLANG_RHI_ENABLE_NVAPI
-        if (m_nvapiShaderExtension)
+    if (m_nvapiShaderExtension)
+    {
+        SLANG_RHI_NVAPI_RETURN_ON_FAIL(NvAPI_D3D12_SetNvShaderExtnSlotSpaceLocalThread(m_device, 0xffffffff, 0));
+
+        if (nvapiResetPipelineStateOptions)
         {
-            SLANG_RHI_NVAPI_RETURN_ON_FAIL(NvAPI_D3D12_SetNvShaderExtnSlotSpace(m_device, 0xffffffff, 0));
+            NVAPI_D3D12_SET_CREATE_PIPELINE_STATE_OPTIONS_PARAMS params = {};
+            params.version = NVAPI_D3D12_SET_CREATE_PIPELINE_STATE_OPTIONS_PARAMS_VER;
+
+            SLANG_RHI_NVAPI_RETURN_ON_FAIL(NvAPI_D3D12_SetCreatePipelineStateOptions(m_device5, &params));
         }
+    }
 #endif // SLANG_RHI_ENABLE_NVAPI
+
+    if (desc.label)
+    {
+        stateObject->SetName(string::to_wstring(desc.label).c_str());
     }
 
-    RefPtr<RayTracingPipelineImpl> pipeline = new RayTracingPipelineImpl(this);
+    // Report the pipeline creation time.
+    if (m_shaderCompilationReporter)
+    {
+        m_shaderCompilationReporter->reportCreatePipeline(
+            program,
+            ShaderCompilationReporter::PipelineType::RayTracing,
+            startTime,
+            Timer::now(),
+            false,
+            0
+        );
+    }
+
+    RefPtr<RayTracingPipelineImpl> pipeline = new RayTracingPipelineImpl(this, desc);
     pipeline->m_program = program;
     pipeline->m_rootObjectLayout = program->m_rootObjectLayout;
     pipeline->m_stateObject = stateObject;
     returnComPtr(outPipeline, pipeline);
     return SLANG_OK;
-#else
-    return SLANG_E_NOT_AVAILABLE;
-#endif
 }
 
 } // namespace rhi::d3d12
