@@ -4,15 +4,7 @@
 
 #if SLANG_LINUX_FAMILY
 
-#pragma push_macro("WIN32_LEAN_AND_MEAN")
-#pragma push_macro("NOMINMAX")
-#undef WIN32_LEAN_AND_MEAN
-#define WIN32_LEAN_AND_MEAN
-#undef NOMINMAX
-#define NOMINMAX
 #include <windows.h>
-#pragma pop_macro("NOMINMAX")
-#pragma pop_macro("WIN32_LEAN_AND_MEAN")
 
 ////////////////////////////////////////////////////////////////
 //
