@@ -64,22 +64,12 @@ public:
     struct SubObjectRangeOffset : BindingOffset
     {
         SubObjectRangeOffset() {}
-
-        SubObjectRangeOffset(slang::VariableLayoutReflection* varLayout);
-
-        /// The offset for "pending" ordinary data related to this range
-        uint32_t pendingOrdinaryData = 0;
     };
 
     /// Stride information for a sub-object range
     struct SubObjectRangeStride : BindingOffset
     {
         SubObjectRangeStride() {}
-
-        SubObjectRangeStride(slang::TypeLayoutReflection* typeLayout);
-
-        /// The strid for "pending" ordinary data related to this range
-        uint32_t pendingOrdinaryData = 0;
     };
 
     /// Information about a sub-objecrt range
@@ -247,6 +237,19 @@ public:
     ComPtr<slang::IComponentType> m_program;
     slang::ProgramLayout* m_programLayout = nullptr;
 
+#if SLANG_RHI_ENABLE_NVAPI
+    // Slang differentiates between explicit and implicit use of NVAPI:
+    // - Explicit use happens when a shader manually includes the NVAPI header.
+    //   In this case the program reflection data contains the global NVAPI UAV slot "g_NvidiaExt"
+    //   and the shader object handling automatically assigns a null-descriptor to it.
+    // - Implicit use happens when a shader doesn't manually include the NVAPI header, but uses
+    //   a standard library call that is implemented using an NVAPI shader intrinsic.
+    //   In that case the program reflection data does not contain "g_NvidiaExt" and slang-rhi creates
+    //   the resource implicitly by adding an extra descriptor range to the root descriptor set
+    //   and then assigns a null-descriptor when binding it.
+    bool m_hasImplicitDescriptorRangeForNVAPI = false;
+#endif
+
     std::vector<EntryPointInfo> m_entryPoints;
 
     ComPtr<ID3D12RootSignature> m_rootSignature;
@@ -345,25 +348,6 @@ public:
             }
         };
 
-        struct BindingRegisterOffsetPair
-        {
-            BindingRegisterOffset primary;
-            BindingRegisterOffset pending;
-
-            BindingRegisterOffsetPair() {}
-
-            BindingRegisterOffsetPair(slang::VariableLayoutReflection* varLayout)
-                : primary(varLayout)
-                , pending(varLayout->getPendingDataLayout())
-            {
-            }
-
-            void operator+=(const BindingRegisterOffsetPair& other)
-            {
-                primary += other.primary;
-                pending += other.pending;
-            }
-        };
         /// Add a new descriptor set to the layout being computed.
         ///
         /// Note that a "descriptor set" in the layout may amount to
@@ -448,15 +432,15 @@ public:
         void addAsConstantBuffer(
             slang::TypeLayoutReflection* typeLayout,
             uint32_t physicalDescriptorSetIndex,
-            BindingRegisterOffsetPair containerOffset,
-            BindingRegisterOffsetPair elementOffset
+            BindingRegisterOffset containerOffset,
+            BindingRegisterOffset elementOffset
         );
 
         void addAsValue(
             slang::TypeLayoutReflection* typeLayout,
             uint32_t physicalDescriptorSetIndex,
-            BindingRegisterOffsetPair containerOffset,
-            BindingRegisterOffsetPair elementOffset
+            BindingRegisterOffset containerOffset,
+            BindingRegisterOffset elementOffset
         );
 
         D3D12_ROOT_SIGNATURE_DESC1& build();
