@@ -2,7 +2,7 @@
 #include "d3d12-buffer.h"
 #include "d3d12-texture.h"
 #include "d3d12-device.h"
-#include "d3d12-helper-functions.h"
+#include "d3d12-utils.h"
 #include "d3d12-sampler.h"
 #include "d3d12-shader-object-layout.h"
 #include "d3d12-acceleration-structure.h"
@@ -36,6 +36,42 @@ inline BindingDataImpl::RootParameter createRootUAV(UINT index, D3D12_GPU_VIRTUA
     return result;
 }
 
+inline void writeBufferState(BindingDataBuilder* builder, BufferImpl* buffer, ResourceState state)
+{
+    BindingDataImpl* bindingData = builder->m_bindingData;
+    if (bindingData->bufferStateCount >= bindingData->bufferStateCapacity)
+    {
+        bindingData->bufferStateCapacity *= 2;
+        BindingDataImpl::BufferState* newBufferStates =
+            builder->m_allocator->allocate<BindingDataImpl::BufferState>(bindingData->bufferStateCapacity);
+        std::memcpy(
+            newBufferStates,
+            bindingData->bufferStates,
+            bindingData->bufferStateCount * sizeof(BindingDataImpl::BufferState)
+        );
+        bindingData->bufferStates = newBufferStates;
+    }
+    bindingData->bufferStates[bindingData->bufferStateCount++] = {buffer, state};
+}
+
+inline void writeTextureState(BindingDataBuilder* builder, TextureViewImpl* textureView, ResourceState state)
+{
+    BindingDataImpl* bindingData = builder->m_bindingData;
+    if (bindingData->textureStateCount >= bindingData->textureStateCapacity)
+    {
+        bindingData->textureStateCapacity *= 2;
+        BindingDataImpl::TextureState* newTextureStates =
+            builder->m_allocator->allocate<BindingDataImpl::TextureState>(bindingData->textureStateCapacity);
+        std::memcpy(
+            newTextureStates,
+            bindingData->textureStates,
+            bindingData->textureStateCount * sizeof(BindingDataImpl::TextureState)
+        );
+        bindingData->textureStates = newTextureStates;
+    }
+    bindingData->textureStates[bindingData->textureStateCount++] = {textureView, state};
+}
+
 Result BindingDataBuilder::bindAsRoot(
     RootShaderObject* shaderObject,
     RootShaderObjectLayoutImpl* specializedLayout,
@@ -49,11 +85,14 @@ Result BindingDataBuilder::bindAsRoot(
     m_bindingData = bindingData;
 
     // TODO(shaderobject): allocate actual number of buffer/texture resources
-    uint32_t bufferCount = 100;
-    uint32_t textureCount = 100;
-    m_bindingData->bufferStates = m_allocator->allocate<BindingDataImpl::BufferState>(bufferCount);
+    // For now we use a fixed starting capacity and grow as needed.
+    m_bindingData->bufferStateCapacity = 1024;
+    m_bindingData->bufferStates =
+        m_allocator->allocate<BindingDataImpl::BufferState>(m_bindingData->bufferStateCapacity);
     m_bindingData->bufferStateCount = 0;
-    m_bindingData->textureStates = m_allocator->allocate<BindingDataImpl::TextureState>(textureCount);
+    m_bindingData->textureStateCapacity = 1024;
+    m_bindingData->textureStates =
+        m_allocator->allocate<BindingDataImpl::TextureState>(m_bindingData->textureStateCapacity);
     m_bindingData->textureStateCount = 0;
 
     uint32_t rootParameterCount = specializedLayout->m_rootSignatureTotalParameterCount;
@@ -74,7 +113,8 @@ Result BindingDataBuilder::bindAsRoot(
     rootOffset.rootParam += specializedLayout->m_rootSignatureRootParameterCount;
 
     DescriptorSet descriptorSet;
-    SLANG_RETURN_ON_FAIL(allocateDescriptorSets(shaderObject, /* inout */ rootOffset, specializedLayout, descriptorSet)
+    SLANG_RETURN_ON_FAIL(
+        allocateDescriptorSets(shaderObject, /* inout */ rootOffset, specializedLayout, descriptorSet)
     );
 
     SLANG_RETURN_ON_FAIL(
@@ -96,6 +136,28 @@ Result BindingDataBuilder::bindAsRoot(
         );
     }
 
+#if SLANG_RHI_ENABLE_NVAPI
+    // Bind null-descriptor to implicit descriptor range for the NVAPI UAV descriptor if needed.
+    if (specializedLayout->m_hasImplicitDescriptorRangeForNVAPI)
+    {
+        SLANG_RHI_ASSERT(descriptorSet.resources.count > 0);
+
+        // The NVAPI UAV descriptor should be at the very last position in the descriptor table
+        // since we added the NVAPI UAV range last during root signature creation.
+        uint32_t nvapiDescriptorIndex = descriptorSet.resources.count - 1;
+
+        D3D12_CPU_DESCRIPTOR_HANDLE nullUavDescriptor =
+            m_device->getNullDescriptor(slang::BindingType::MutableRawBuffer, SLANG_STRUCTURED_BUFFER);
+
+        m_device->m_device->CopyDescriptorsSimple(
+            1,
+            descriptorSet.resources.getCpuHandle(nvapiDescriptorIndex),
+            nullUavDescriptor,
+            D3D12_DESCRIPTOR_HEAP_TYPE_CBV_SRV_UAV
+        );
+    }
+#endif
+
     outBindingData = bindingData;
 
     return SLANG_OK;
@@ -198,7 +260,8 @@ Result BindingDataBuilder::bindAsParameterBlock(
     // Next we bind the object into that descriptor set as if it were being used
     // as a `ConstantBuffer<X>`.
     //
-    SLANG_RETURN_ON_FAIL(bindAsConstantBuffer(shaderObject, descriptorSet, subOffset, rootParamIndex, specializedLayout)
+    SLANG_RETURN_ON_FAIL(
+        bindAsConstantBuffer(shaderObject, descriptorSet, subOffset, rootParamIndex, specializedLayout)
     );
 
     return SLANG_OK;
@@ -248,7 +311,7 @@ Result BindingDataBuilder::bindAsValue(
                 {
                     SLANG_RHI_ASSERT(resourceIndex + i < descriptorSet.resources.count);
                     descriptor = isSrv ? textureView->getSRV() : textureView->getUAV();
-                    m_bindingData->textureStates[m_bindingData->textureStateCount++] = {textureView, requiredState};
+                    writeTextureState(this, textureView, requiredState);
                 }
                 else
                 {
@@ -278,10 +341,7 @@ Result BindingDataBuilder::bindAsValue(
                 if (textureView)
                 {
                     textureDescriptor = textureView->getSRV();
-                    m_bindingData->textureStates[m_bindingData->textureStateCount++] = {
-                        textureView,
-                        ResourceState::UnorderedAccess
-                    };
+                    writeTextureState(this, textureView, ResourceState::UnorderedAccess);
                 }
                 else
                 {
@@ -406,7 +466,7 @@ Result BindingDataBuilder::bindAsValue(
                 }
                 if (buffer)
                 {
-                    m_bindingData->bufferStates[m_bindingData->bufferStateCount++] = {buffer, requiredState};
+                    writeBufferState(this, buffer, requiredState);
                 }
             }
             break;
@@ -431,15 +491,26 @@ Result BindingDataBuilder::bindAsValue(
                 else
                 {
                     SLANG_RHI_ASSERT(resourceIndex + i < descriptorSet.resources.count);
+                    D3D12_CPU_DESCRIPTOR_HANDLE descriptor = {};
                     if (as)
                     {
-                        d3dDevice->CopyDescriptorsSimple(
-                            1,
-                            descriptorSet.resources.getCpuHandle(resourceIndex + i),
-                            as->m_descriptor.cpuHandle,
-                            D3D12_DESCRIPTOR_HEAP_TYPE_CBV_SRV_UAV
-                        );
+                        descriptor = as->m_descriptor.cpuHandle;
                     }
+                    else
+                    {
+                        descriptor =
+                            m_device->getNullDescriptor(bindingRangeInfo.bindingType, bindingRangeInfo.resourceShape);
+                    }
+                    d3dDevice->CopyDescriptorsSimple(
+                        1,
+                        descriptorSet.resources.getCpuHandle(resourceIndex + i),
+                        descriptor,
+                        D3D12_DESCRIPTOR_HEAP_TYPE_CBV_SRV_UAV
+                    );
+                }
+                if (as)
+                {
+                    writeBufferState(this, as->m_buffer, ResourceState::AccelerationStructureRead);
                 }
             }
             break;
