@@ -21,8 +21,12 @@ public:
     RefPtr<RootShaderObjectLayoutImpl> m_rootObjectLayout;
     std::vector<ShaderBinary> m_shaders;
 
-    virtual Result createShaderModule(slang::EntryPointReflection* entryPointInfo, ComPtr<ISlangBlob> kernelCode)
-        override;
+    ShaderProgramImpl(Device* device, const ShaderProgramDesc& desc);
+
+    virtual Result createShaderModule(
+        slang::EntryPointReflection* entryPointInfo,
+        ComPtr<ISlangBlob> kernelCode
+    ) override;
 
     virtual ShaderObjectLayout* getRootShaderObjectLayout() override;
 };
