@@ -88,12 +88,12 @@ void DebugRenderPassEncoder::drawMeshTasks(uint32_t x, uint32_t y, uint32_t z)
     baseObject->drawMeshTasks(x, y, z);
 }
 
-void DebugRenderPassEncoder::pushDebugGroup(const char* name, float rgbColor[3])
+void DebugRenderPassEncoder::pushDebugGroup(const char* name, const MarkerColor& color)
 {
     SLANG_RHI_API_FUNC;
     m_commandEncoder->requireOpen();
     m_commandEncoder->requireRenderPass();
-    baseObject->pushDebugGroup(name, rgbColor);
+    baseObject->pushDebugGroup(name, color);
 }
 
 void DebugRenderPassEncoder::popDebugGroup()
@@ -104,12 +104,20 @@ void DebugRenderPassEncoder::popDebugGroup()
     baseObject->popDebugGroup();
 }
 
-void DebugRenderPassEncoder::insertDebugMarker(const char* name, float rgbColor[3])
+void DebugRenderPassEncoder::insertDebugMarker(const char* name, const MarkerColor& color)
 {
     SLANG_RHI_API_FUNC;
     m_commandEncoder->requireOpen();
     m_commandEncoder->requireRenderPass();
-    baseObject->insertDebugMarker(name, rgbColor);
+    baseObject->insertDebugMarker(name, color);
+}
+
+void DebugRenderPassEncoder::writeTimestamp(IQueryPool* queryPool, uint32_t queryIndex)
+{
+    SLANG_RHI_API_FUNC;
+    m_commandEncoder->requireOpen();
+    m_commandEncoder->requireRenderPass();
+    baseObject->writeTimestamp(getInnerObj(queryPool), queryIndex);
 }
 
 void DebugRenderPassEncoder::end()
@@ -162,12 +170,12 @@ void DebugComputePassEncoder::dispatchComputeIndirect(BufferOffsetPair argBuffer
     baseObject->dispatchComputeIndirect(argBuffer);
 }
 
-void DebugComputePassEncoder::pushDebugGroup(const char* name, float rgbColor[3])
+void DebugComputePassEncoder::pushDebugGroup(const char* name, const MarkerColor& color)
 {
     SLANG_RHI_API_FUNC;
     m_commandEncoder->requireOpen();
     m_commandEncoder->requireComputePass();
-    baseObject->pushDebugGroup(name, rgbColor);
+    baseObject->pushDebugGroup(name, color);
 }
 
 void DebugComputePassEncoder::popDebugGroup()
@@ -178,12 +186,20 @@ void DebugComputePassEncoder::popDebugGroup()
     baseObject->popDebugGroup();
 }
 
-void DebugComputePassEncoder::insertDebugMarker(const char* name, float rgbColor[3])
+void DebugComputePassEncoder::insertDebugMarker(const char* name, const MarkerColor& color)
 {
     SLANG_RHI_API_FUNC;
     m_commandEncoder->requireOpen();
     m_commandEncoder->requireComputePass();
-    baseObject->insertDebugMarker(name, rgbColor);
+    baseObject->insertDebugMarker(name, color);
+}
+
+void DebugComputePassEncoder::writeTimestamp(IQueryPool* queryPool, uint32_t queryIndex)
+{
+    SLANG_RHI_API_FUNC;
+    m_commandEncoder->requireOpen();
+    m_commandEncoder->requireComputePass();
+    baseObject->writeTimestamp(getInnerObj(queryPool), queryIndex);
 }
 
 void DebugComputePassEncoder::end()
@@ -237,12 +253,12 @@ void DebugRayTracingPassEncoder::dispatchRays(
     baseObject->dispatchRays(rayGenShaderIndex, width, height, depth);
 }
 
-void DebugRayTracingPassEncoder::pushDebugGroup(const char* name, float rgbColor[3])
+void DebugRayTracingPassEncoder::pushDebugGroup(const char* name, const MarkerColor& color)
 {
     SLANG_RHI_API_FUNC;
     m_commandEncoder->requireOpen();
     m_commandEncoder->requireRayTracingPass();
-    baseObject->pushDebugGroup(name, rgbColor);
+    baseObject->pushDebugGroup(name, color);
 }
 
 void DebugRayTracingPassEncoder::popDebugGroup()
@@ -253,12 +269,20 @@ void DebugRayTracingPassEncoder::popDebugGroup()
     baseObject->popDebugGroup();
 }
 
-void DebugRayTracingPassEncoder::insertDebugMarker(const char* name, float rgbColor[3])
+void DebugRayTracingPassEncoder::insertDebugMarker(const char* name, const MarkerColor& color)
 {
     SLANG_RHI_API_FUNC;
     m_commandEncoder->requireOpen();
     m_commandEncoder->requireRayTracingPass();
-    baseObject->insertDebugMarker(name, rgbColor);
+    baseObject->insertDebugMarker(name, color);
+}
+
+void DebugRayTracingPassEncoder::writeTimestamp(IQueryPool* queryPool, uint32_t queryIndex)
+{
+    SLANG_RHI_API_FUNC;
+    m_commandEncoder->requireOpen();
+    m_commandEncoder->requireRayTracingPass();
+    baseObject->writeTimestamp(getInnerObj(queryPool), queryIndex);
 }
 
 void DebugRayTracingPassEncoder::end()
@@ -729,15 +753,116 @@ void DebugCommandEncoder::deserializeAccelerationStructure(IAccelerationStructur
     baseObject->deserializeAccelerationStructure(dst, src);
 }
 
+void DebugCommandEncoder::executeClusterOperation(const ClusterOperationDesc& desc)
+{
+    SLANG_RHI_API_FUNC;
+    requireOpen();
+    requireNoPass();
+
+    if (SLANG_FAILED(validateClusterOperationParams(ctx, desc.params)))
+    {
+        return;
+    }
+    if (!desc.argCountBuffer && desc.params.maxArgCount == 0)
+    {
+        RHI_VALIDATION_ERROR("argCountBuffer is not provided and params.maxArgCount is 0");
+        return;
+    }
+    if (!desc.argsBuffer)
+    {
+        RHI_VALIDATION_ERROR("argsBuffer must be provided");
+        return;
+    }
+    if (desc.argsBufferStride == 0)
+    {
+        RHI_VALIDATION_ERROR("argsBufferStride must not be 0");
+        return;
+    }
+    if (!desc.scratchBuffer)
+    {
+        RHI_VALIDATION_ERROR("scratchBuffer must be provided");
+        return;
+    }
+    switch (desc.params.mode)
+    {
+    case ClusterOperationMode::ImplicitDestinations:
+        if (!desc.addressesBuffer)
+        {
+            RHI_VALIDATION_ERROR("addressesBuffer must be provided in ClusterOperationMode::ImplicitDestinations mode");
+            return;
+        }
+        if (!desc.resultBuffer)
+        {
+            RHI_VALIDATION_ERROR("resultBuffer must be provided in ClusterOperationMode::ImplicitDestinations mode");
+            return;
+        }
+        break;
+    case ClusterOperationMode::ExplicitDestinations:
+        if (!desc.addressesBuffer)
+        {
+            RHI_VALIDATION_ERROR("addressesBuffer must be provided in ClusterOperationMode::ExplicitDestinations mode");
+            return;
+        }
+        break;
+    case ClusterOperationMode::GetSizes:
+        if (!desc.sizesBuffer)
+        {
+            RHI_VALIDATION_ERROR("sizesBuffer must be provided in ClusterOperationMode::GetSizes mode");
+            return;
+        }
+        break;
+    default:
+        RHI_VALIDATION_ERROR("Unknown cluster operation mode");
+        return;
+    }
+
+    if (desc.addressesBufferStride != 0 && desc.addressesBufferStride < 8)
+    {
+        RHI_VALIDATION_ERROR("addressesBufferStride must be 0 or >= 8");
+        return;
+    }
+    if (desc.sizesBufferStride != 0 && desc.sizesBufferStride < 4)
+    {
+        RHI_VALIDATION_ERROR("sizesBufferStride must be 0 or >= 4");
+        return;
+    }
+
+    baseObject->executeClusterOperation(desc);
+}
+
 void DebugCommandEncoder::convertCooperativeVectorMatrix(
-    const ConvertCooperativeVectorMatrixDesc* infos,
-    uint32_t infoCount
+    IBuffer* dstBuffer,
+    const CooperativeVectorMatrixDesc* dstDescs,
+    IBuffer* srcBuffer,
+    const CooperativeVectorMatrixDesc* srcDescs,
+    uint32_t matrixCount
 )
 {
     SLANG_RHI_API_FUNC;
     requireOpen();
     requireNoPass();
-    baseObject->convertCooperativeVectorMatrix(infos, infoCount);
+
+    if (!dstBuffer)
+    {
+        RHI_VALIDATION_ERROR("Destination buffer must be valid");
+        return;
+    }
+    if (!srcBuffer)
+    {
+        RHI_VALIDATION_ERROR("Source buffer must be valid");
+        return;
+    }
+
+    SLANG_RETURN_VOID_ON_FAIL(validateConvertCooperativeVectorMatrix(
+        ctx,
+        dstBuffer->getDesc().size,
+        dstDescs,
+        srcBuffer->getDesc().size,
+        srcDescs,
+        matrixCount
+    ));
+
+    baseObject->convertCooperativeVectorMatrix(dstBuffer, dstDescs, srcBuffer, srcDescs, matrixCount);
 }
 
 void DebugCommandEncoder::setBufferState(IBuffer* buffer, ResourceState state)
@@ -756,12 +881,20 @@ void DebugCommandEncoder::setTextureState(ITexture* texture, SubresourceRange su
     baseObject->setTextureState(texture, subresourceRange, state);
 }
 
-void DebugCommandEncoder::pushDebugGroup(const char* name, float rgbColor[3])
+void DebugCommandEncoder::globalBarrier()
 {
     SLANG_RHI_API_FUNC;
     requireOpen();
     requireNoPass();
-    baseObject->pushDebugGroup(name, rgbColor);
+    baseObject->globalBarrier();
+}
+
+void DebugCommandEncoder::pushDebugGroup(const char* name, const MarkerColor& color)
+{
+    SLANG_RHI_API_FUNC;
+    requireOpen();
+    requireNoPass();
+    baseObject->pushDebugGroup(name, color);
 }
 
 void DebugCommandEncoder::popDebugGroup()
@@ -772,12 +905,12 @@ void DebugCommandEncoder::popDebugGroup()
     baseObject->popDebugGroup();
 }
 
-void DebugCommandEncoder::insertDebugMarker(const char* name, float rgbColor[3])
+void DebugCommandEncoder::insertDebugMarker(const char* name, const MarkerColor& color)
 {
     SLANG_RHI_API_FUNC;
     requireOpen();
     requireNoPass();
-    baseObject->insertDebugMarker(name, rgbColor);
+    baseObject->insertDebugMarker(name, color);
 }
 
 void DebugCommandEncoder::writeTimestamp(IQueryPool* pool, uint32_t index)
