@@ -1,6 +1,7 @@
 #include "debug-device.h"
 #include "debug-command-queue.h"
 #include "debug-fence.h"
+#include "debug-heap.h"
 #include "debug-helper-functions.h"
 #include "debug-query.h"
 #include "debug-shader-object.h"
@@ -30,11 +31,46 @@ Result DebugDevice::getNativeDeviceHandles(DeviceNativeHandles* outHandles)
     return baseObject->getNativeDeviceHandles(outHandles);
 }
 
-Result DebugDevice::getFeatures(const char** outFeatures, size_t bufferSize, uint32_t* outFeatureCount)
+Result DebugDevice::getFeatures(uint32_t* outFeatureCount, Feature* outFeatures)
 {
     SLANG_RHI_API_FUNC;
 
-    return baseObject->getFeatures(outFeatures, bufferSize, outFeatureCount);
+    return baseObject->getFeatures(outFeatureCount, outFeatures);
+}
+
+bool DebugDevice::hasFeature(Feature feature)
+{
+    SLANG_RHI_API_FUNC;
+
+    return baseObject->hasFeature(feature);
+}
+
+bool DebugDevice::hasFeature(const char* feature)
+{
+    SLANG_RHI_API_FUNC;
+
+    return baseObject->hasFeature(feature);
+}
+
+Result DebugDevice::getCapabilities(uint32_t* outCapabilityCount, Capability* outCapabilities)
+{
+    SLANG_RHI_API_FUNC;
+
+    return baseObject->getCapabilities(outCapabilityCount, outCapabilities);
+}
+
+bool DebugDevice::hasCapability(Capability capability)
+{
+    SLANG_RHI_API_FUNC;
+
+    return baseObject->hasCapability(capability);
+}
+
+bool DebugDevice::hasCapability(const char* capability)
+{
+    SLANG_RHI_API_FUNC;
+
+    return baseObject->hasCapability(capability);
 }
 
 Result DebugDevice::getFormatSupport(Format format, FormatSupport* outFormatSupport)
@@ -52,20 +88,6 @@ DebugDevice::DebugDevice(DeviceType deviceType, IDebugCallback* debugCallback)
     RHI_VALIDATION_INFO("Debug layer is enabled.");
 }
 
-bool DebugDevice::hasFeature(Feature feature)
-{
-    SLANG_RHI_API_FUNC;
-
-    return baseObject->hasFeature(feature);
-}
-
-bool DebugDevice::hasFeature(const char* feature)
-{
-    SLANG_RHI_API_FUNC;
-
-    return baseObject->hasFeature(feature);
-}
-
 Result DebugDevice::getSlangSession(slang::ISession** outSlangSession)
 {
     SLANG_RHI_API_FUNC;
@@ -198,13 +220,14 @@ Result DebugDevice::createTexture(const TextureDesc& desc, const SubresourceData
         break;
     }
 
-    TextureDesc patchedDesc = fixupTextureDesc(desc);
+    TextureDesc patchedDesc = desc;
     std::string label;
     if (!patchedDesc.label)
     {
         label = createTextureLabel(patchedDesc);
         patchedDesc.label = label.c_str();
     }
+
     return baseObject->createTexture(patchedDesc, initData, outTexture);
 }
 
@@ -231,13 +254,14 @@ Result DebugDevice::createBuffer(const BufferDesc& desc, const void* initData, I
 {
     SLANG_RHI_API_FUNC;
 
-    BufferDesc patchedDesc = fixupBufferDesc(desc);
+    BufferDesc patchedDesc = desc;
     std::string label;
     if (!patchedDesc.label)
     {
         label = createBufferLabel(patchedDesc);
         patchedDesc.label = label.c_str();
     }
+
     return baseObject->createBuffer(patchedDesc, initData, outBuffer);
 }
 
@@ -329,11 +353,30 @@ Result DebugDevice::createSampler(const SamplerDesc& desc, ISampler** outSampler
         RHI_VALIDATION_ERROR("Invalid address W mode");
         return SLANG_E_INVALID_ARG;
     }
+    if (ctx->deviceType == DeviceType::WGPU && (desc.addressU == TextureAddressingMode::ClampToBorder ||
+                                                desc.addressV == TextureAddressingMode::ClampToBorder ||
+                                                desc.addressW == TextureAddressingMode::ClampToBorder))
+    {
+        RHI_VALIDATION_ERROR("WebGPU doesn't support ClampToBorder mode");
+        return SLANG_E_INVALID_ARG;
+    }
+    if (ctx->deviceType == DeviceType::WGPU &&
+        (desc.addressU == TextureAddressingMode::MirrorOnce || desc.addressV == TextureAddressingMode::MirrorOnce ||
+         desc.addressW == TextureAddressingMode::MirrorOnce))
+    {
+        RHI_VALIDATION_ERROR("WebGPU doesn't support MirrorOnce mode");
+        return SLANG_E_INVALID_ARG;
+    }
     if (desc.comparisonFunc > ComparisonFunc::Always)
     {
         RHI_VALIDATION_ERROR("Invalid comparison func");
         return SLANG_E_INVALID_ARG;
     }
+    if (desc.maxAnisotropy > 1 &&
+        (desc.minFilter == TextureFilteringMode::Point || desc.minFilter == TextureFilteringMode::Point))
+    {
+        RHI_VALIDATION_WARNING("maxAnisotropy > 1 can only be set when neither min and mag filter is Point");
+    }
 
     if (desc.addressU == TextureAddressingMode::ClampToBorder ||
         desc.addressV == TextureAddressingMode::ClampToBorder || desc.addressW == TextureAddressingMode::ClampToBorder)
@@ -370,6 +413,7 @@ Result DebugDevice::createSampler(const SamplerDesc& desc, ISampler** outSampler
         label = createSamplerLabel(patchedDesc);
         patchedDesc.label = label.c_str();
     }
+
     return baseObject->createSampler(patchedDesc, outSampler);
 }
 
@@ -384,6 +428,7 @@ Result DebugDevice::createTextureView(ITexture* texture, const TextureViewDesc&
         label = createTextureViewLabel(patchedDesc);
         patchedDesc.label = label.c_str();
     }
+
     return baseObject->createTextureView(texture, patchedDesc, outView);
 }
 
@@ -393,10 +438,17 @@ Result DebugDevice::getAccelerationStructureSizes(
 )
 {
     SLANG_RHI_API_FUNC;
-    validateAccelerationStructureBuildDesc(ctx, desc);
+    SLANG_RETURN_ON_FAIL(validateAccelerationStructureBuildDesc(ctx, desc));
     return baseObject->getAccelerationStructureSizes(desc, outSizes);
 }
 
+Result DebugDevice::getClusterOperationSizes(const ClusterOperationParams& params, ClusterOperationSizes* outSizes)
+{
+    SLANG_RHI_API_FUNC;
+    SLANG_RETURN_ON_FAIL(validateClusterOperationParams(ctx, params));
+    return baseObject->getClusterOperationSizes(params, outSizes);
+}
+
 Result DebugDevice::createAccelerationStructure(
     const AccelerationStructureDesc& desc,
     IAccelerationStructure** outAccelerationStructure
@@ -404,7 +456,15 @@ Result DebugDevice::createAccelerationStructure(
 {
     SLANG_RHI_API_FUNC;
 
-    return baseObject->createAccelerationStructure(desc, outAccelerationStructure);
+    AccelerationStructureDesc patchedDesc = desc;
+    std::string label;
+    if (!patchedDesc.label)
+    {
+        label = createAccelerationStructureLabel(patchedDesc);
+        patchedDesc.label = label.c_str();
+    }
+
+    return baseObject->createAccelerationStructure(patchedDesc, outAccelerationStructure);
 }
 
 Result DebugDevice::createSurface(WindowHandle windowHandle, ISurface** outSurface)
@@ -495,28 +555,141 @@ Result DebugDevice::createShaderProgram(
 {
     SLANG_RHI_API_FUNC;
 
-    return baseObject->createShaderProgram(desc, outProgram, outDiagnostics);
+    ShaderProgramDesc patchedDesc = desc;
+    std::string label;
+    if (!patchedDesc.label)
+    {
+        label = createShaderProgramLabel(patchedDesc);
+        patchedDesc.label = label.c_str();
+    }
+
+    return baseObject->createShaderProgram(patchedDesc, outProgram, outDiagnostics);
 }
 
 Result DebugDevice::createRenderPipeline(const RenderPipelineDesc& desc, IRenderPipeline** outPipeline)
 {
     SLANG_RHI_API_FUNC;
 
-    return baseObject->createRenderPipeline(desc, outPipeline);
+    if (desc.program == nullptr)
+    {
+        RHI_VALIDATION_ERROR("Program must be specified");
+        return SLANG_E_INVALID_ARG;
+    }
+    if (ctx->deviceType == DeviceType::WGPU && desc.primitiveTopology == PrimitiveTopology::PatchList)
+    {
+        RHI_VALIDATION_ERROR("WebGPU doesn't support PatchList topology");
+        return SLANG_E_INVALID_ARG;
+    }
+    if (ctx->deviceType == DeviceType::Metal && desc.primitiveTopology == PrimitiveTopology::PatchList)
+    {
+        RHI_VALIDATION_ERROR("Metal doesn't support PatchList topology");
+        return SLANG_E_INVALID_ARG;
+    }
+
+    RenderPipelineDesc patchedDesc = desc;
+    std::string label;
+    if (!patchedDesc.label)
+    {
+        label = createRenderPipelineLabel(patchedDesc);
+        patchedDesc.label = label.c_str();
+    }
+
+    return baseObject->createRenderPipeline(patchedDesc, outPipeline);
 }
 
 Result DebugDevice::createComputePipeline(const ComputePipelineDesc& desc, IComputePipeline** outPipeline)
 {
     SLANG_RHI_API_FUNC;
 
-    return baseObject->createComputePipeline(desc, outPipeline);
+    if (desc.program == nullptr)
+    {
+        RHI_VALIDATION_ERROR("Program must be specified");
+        return SLANG_E_INVALID_ARG;
+    }
+
+    ComputePipelineDesc patchedDesc = desc;
+    std::string label;
+    if (!patchedDesc.label)
+    {
+        label = createComputePipelineLabel(patchedDesc);
+        patchedDesc.label = label.c_str();
+    }
+
+    return baseObject->createComputePipeline(patchedDesc, outPipeline);
 }
 
 Result DebugDevice::createRayTracingPipeline(const RayTracingPipelineDesc& desc, IRayTracingPipeline** outPipeline)
 {
     SLANG_RHI_API_FUNC;
 
-    return baseObject->createRayTracingPipeline(desc, outPipeline);
+    if (desc.program == nullptr)
+    {
+        RHI_VALIDATION_ERROR("Program must be specified");
+        return SLANG_E_INVALID_ARG;
+    }
+
+    RayTracingPipelineDesc patchedDesc = desc;
+    std::string label;
+    if (!patchedDesc.label)
+    {
+        label = createRayTracingPipelineLabel(patchedDesc);
+        patchedDesc.label = label.c_str();
+    }
+
+    return baseObject->createRayTracingPipeline(patchedDesc, outPipeline);
+}
+
+Result DebugDevice::getCompilationReportList(ISlangBlob** outReportListBlob)
+{
+    SLANG_RHI_API_FUNC;
+
+    return baseObject->getCompilationReportList(outReportListBlob);
+}
+
+Result DebugDevice::readTexture(
+    ITexture* texture,
+    uint32_t layer,
+    uint32_t mip,
+    const SubresourceLayout& layout,
+    void* outData
+)
+{
+    const TextureDesc& desc = texture->getDesc();
+
+    if (layer > desc.getLayerCount())
+    {
+        RHI_VALIDATION_ERROR("Layer out of bounds");
+        return SLANG_E_INVALID_ARG;
+    }
+    if (mip > desc.mipCount)
+    {
+        RHI_VALIDATION_ERROR("Mip out of bounds");
+        return SLANG_E_INVALID_ARG;
+    }
+
+    switch (desc.type)
+    {
+    case TextureType::Texture2DMS:
+    case TextureType::Texture2DMSArray:
+        RHI_VALIDATION_ERROR("Multisample textures cannot be read");
+        return SLANG_E_INVALID_ARG;
+    default:
+        break;
+    }
+
+    SubresourceLayout expectedLayout;
+    SLANG_RETURN_ON_FAIL(texture->getSubresourceLayout(mip, &expectedLayout));
+    if (layout.size.width != expectedLayout.size.width || layout.size.height != expectedLayout.size.height ||
+        layout.size.depth != expectedLayout.size.depth || layout.colPitch != expectedLayout.colPitch ||
+        layout.rowPitch != expectedLayout.rowPitch || layout.slicePitch != expectedLayout.slicePitch ||
+        layout.sizeInBytes != expectedLayout.sizeInBytes || layout.blockWidth != expectedLayout.blockWidth ||
+        layout.blockHeight != expectedLayout.blockHeight || layout.rowCount != expectedLayout.rowCount)
+    {
+        RHI_VALIDATION_ERROR("Layout does not match the expected layout");
+        return SLANG_E_INVALID_ARG;
+    }
+
+    return baseObject->readTexture(texture, layer, mip, layout, outData);
 }
 
 Result DebugDevice::readTexture(
@@ -565,18 +738,26 @@ Result DebugDevice::readBuffer(IBuffer* buffer, size_t offset, size_t size, ISla
     return baseObject->readBuffer(buffer, offset, size, outBlob);
 }
 
-const DeviceInfo& DebugDevice::getDeviceInfo() const
+const DeviceInfo& DebugDevice::getInfo() const
 {
     SLANG_RHI_API_FUNC;
-    return baseObject->getDeviceInfo();
+    return baseObject->getInfo();
 }
 
 Result DebugDevice::createQueryPool(const QueryPoolDesc& desc, IQueryPool** outPool)
 {
     SLANG_RHI_API_FUNC;
+
+    QueryPoolDesc patchedDesc = desc;
+    std::string label;
+    if (!patchedDesc.label)
+    {
+        label = createQueryPoolLabel(patchedDesc);
+        patchedDesc.label = label.c_str();
+    }
+
     RefPtr<DebugQueryPool> result = new DebugQueryPool(ctx);
-    result->desc = desc;
-    SLANG_RETURN_ON_FAIL(baseObject->createQueryPool(desc, result->baseObject.writeRef()));
+    SLANG_RETURN_ON_FAIL(baseObject->createQueryPool(patchedDesc, result->baseObject.writeRef()));
     returnComPtr(outPool, result);
     return SLANG_OK;
 }
@@ -584,8 +765,17 @@ Result DebugDevice::createQueryPool(const QueryPoolDesc& desc, IQueryPool** outP
 Result DebugDevice::createFence(const FenceDesc& desc, IFence** outFence)
 {
     SLANG_RHI_API_FUNC;
+
+    FenceDesc patchedDesc = desc;
+    std::string label;
+    if (!patchedDesc.label)
+    {
+        label = createFenceLabel(patchedDesc);
+        patchedDesc.label = label.c_str();
+    }
+
     RefPtr<DebugFence> result = new DebugFence(ctx);
-    SLANG_RETURN_ON_FAIL(baseObject->createFence(desc, result->baseObject.writeRef()));
+    SLANG_RETURN_ON_FAIL(baseObject->createFence(patchedDesc, result->baseObject.writeRef()));
     returnComPtr(outFence, result);
     return SLANG_OK;
 }
@@ -607,6 +797,22 @@ Result DebugDevice::waitForFences(
     return baseObject->waitForFences(fenceCount, innerFences.data(), fenceValues, waitForAll, timeout);
 }
 
+Result DebugDevice::createHeap(const HeapDesc& desc, IHeap** outHeap)
+{
+    HeapDesc patchedDesc = desc;
+    std::string label;
+    if (!patchedDesc.label)
+    {
+        label = createHeapLabel(patchedDesc);
+        patchedDesc.label = label.c_str();
+    }
+
+    RefPtr<DebugHeap> result = new DebugHeap(ctx);
+    SLANG_RETURN_ON_FAIL(baseObject->createHeap(patchedDesc, result->baseObject.writeRef()));
+    returnComPtr(outHeap, result);
+    return SLANG_OK;
+}
+
 Result DebugDevice::getTextureAllocationInfo(const TextureDesc& desc, size_t* outSize, size_t* outAlignment)
 {
     SLANG_RHI_API_FUNC;
@@ -619,16 +825,91 @@ Result DebugDevice::getTextureRowAlignment(Format format, size_t* outAlignment)
     return baseObject->getTextureRowAlignment(format, outAlignment);
 }
 
-Result DebugDevice::getCooperativeVectorProperties(CooperativeVectorProperties* properties, uint32_t* propertyCount)
+Result DebugDevice::getCooperativeVectorProperties(CooperativeVectorProperties* properties, uint32_t* propertiesCount)
 {
     SLANG_RHI_API_FUNC;
-    return baseObject->getCooperativeVectorProperties(properties, propertyCount);
+    return baseObject->getCooperativeVectorProperties(properties, propertiesCount);
 }
 
-Result DebugDevice::convertCooperativeVectorMatrix(const ConvertCooperativeVectorMatrixDesc* descs, uint32_t descCount)
+Result DebugDevice::getCooperativeVectorMatrixSize(
+    uint32_t rowCount,
+    uint32_t colCount,
+    CooperativeVectorComponentType componentType,
+    CooperativeVectorMatrixLayout layout,
+    size_t rowColumnStride,
+    size_t* outSize
+)
 {
     SLANG_RHI_API_FUNC;
-    return baseObject->convertCooperativeVectorMatrix(descs, descCount);
+
+    if (rowCount < 1 || rowCount > 128)
+    {
+        RHI_VALIDATION_ERROR("Row count must be in the range [1, 128]");
+        return SLANG_E_INVALID_ARG;
+    }
+    if (colCount < 1 || colCount > 128)
+    {
+        RHI_VALIDATION_ERROR("Column count must be in the range [1, 128]");
+        return SLANG_E_INVALID_ARG;
+    }
+    switch (layout)
+    {
+    case CooperativeVectorMatrixLayout::RowMajor:
+    case CooperativeVectorMatrixLayout::ColumnMajor:
+        break;
+    case CooperativeVectorMatrixLayout::InferencingOptimal:
+    case CooperativeVectorMatrixLayout::TrainingOptimal:
+        if (rowColumnStride != 0)
+        {
+            RHI_VALIDATION_ERROR("Row/Column stride must be zero for optimal layouts");
+            return SLANG_E_INVALID_ARG;
+        }
+        break;
+    default:
+        RHI_VALIDATION_ERROR("Invalid matrix layout");
+        return SLANG_E_INVALID_ARG;
+    }
+
+    return baseObject
+        ->getCooperativeVectorMatrixSize(rowCount, colCount, componentType, layout, rowColumnStride, outSize);
+}
+
+Result DebugDevice::convertCooperativeVectorMatrix(
+    void* dstBuffer,
+    size_t dstBufferSize,
+    const CooperativeVectorMatrixDesc* dstDescs,
+    const void* srcBuffer,
+    size_t srcBufferSize,
+    const CooperativeVectorMatrixDesc* srcDescs,
+    uint32_t matrixCount
+)
+{
+    SLANG_RHI_API_FUNC;
+
+    if (!dstBuffer)
+    {
+        RHI_VALIDATION_ERROR("Destination buffer must be valid");
+        return SLANG_E_INVALID_ARG;
+    }
+    if (!srcBuffer)
+    {
+        RHI_VALIDATION_ERROR("Source buffer must be valid");
+        return SLANG_E_INVALID_ARG;
+    }
+
+    SLANG_RETURN_ON_FAIL(
+        validateConvertCooperativeVectorMatrix(ctx, dstBufferSize, dstDescs, srcBufferSize, srcDescs, matrixCount)
+    );
+
+    return baseObject->convertCooperativeVectorMatrix(
+        dstBuffer,
+        dstBufferSize,
+        dstDescs,
+        srcBuffer,
+        srcBufferSize,
+        srcDescs,
+        matrixCount
+    );
 }
 
 Result DebugDevice::createShaderTable(const ShaderTableDesc& desc, IShaderTable** outTable)
@@ -638,10 +919,11 @@ Result DebugDevice::createShaderTable(const ShaderTableDesc& desc, IShaderTable*
     return baseObject->createShaderTable(desc, outTable);
 }
 
-Result DebugDevice::getShaderCacheStats(size_t* outCacheHitCount, size_t* outCacheMissCount, size_t* outCacheSize)
+Result DebugDevice::reportHeaps(HeapReport* heapReports, uint32_t* heapCount)
 {
     SLANG_RHI_API_FUNC;
-    return baseObject->getShaderCacheStats(outCacheHitCount, outCacheMissCount, outCacheSize);
+
+    return baseObject->reportHeaps(heapReports, heapCount);
 }
 
 } // namespace rhi::debug
