@@ -26,8 +26,9 @@ public:
     State m_state = State::Initial;
 
     virtual SLANG_NO_THROW const SurfaceInfo& SLANG_MCALL getInfo() override;
-    virtual SLANG_NO_THROW const SurfaceConfig& SLANG_MCALL getConfig() override;
+    virtual SLANG_NO_THROW const SurfaceConfig* SLANG_MCALL getConfig() override;
     virtual SLANG_NO_THROW Result SLANG_MCALL configure(const SurfaceConfig& config) override;
+    virtual SLANG_NO_THROW Result SLANG_MCALL unconfigure() override;
     virtual SLANG_NO_THROW Result SLANG_MCALL acquireNextImage(ITexture** outTexture) override;
     virtual SLANG_NO_THROW Result SLANG_MCALL present() override;
 };
