@@ -1,11 +1,18 @@
 #include "device.h"
 
 #include "rhi-shared.h"
+#include "shader.h"
+#include "heap.h"
 
 #include <algorithm>
+#include <cstdarg>
 
 namespace rhi {
 
+namespace testing {
+bool gDebugDisableStateTracking = false;
+} // namespace testing
+
 // ----------------------------------------------------------------------------
 // ShaderCache
 // ----------------------------------------------------------------------------
@@ -69,24 +76,68 @@ ShaderComponentID ShaderCache::getComponentId(ComponentKey key)
     return resultId;
 }
 
+RefPtr<Pipeline> ShaderCache::getSpecializedPipeline(PipelineKey programKey)
+{
+    auto it = specializedPipelines.find(programKey);
+    if (it != specializedPipelines.end())
+        return it->second;
+    return nullptr;
+}
+
 void ShaderCache::addSpecializedPipeline(PipelineKey key, RefPtr<Pipeline> specializedPipeline)
 {
     specializedPipelines[key] = specializedPipeline;
 }
 
+void ShaderCache::free()
+{
+    componentIds = decltype(componentIds)();
+    specializedPipelines = decltype(specializedPipelines)();
+}
+
 // ----------------------------------------------------------------------------
 // Device
 // ----------------------------------------------------------------------------
 
-// Feature lookup
+void Device::printMessage(DebugMessageType type, DebugMessageSource source, const char* message, ...)
+{
+    va_list args;
+    va_start(args, message);
+    char buffer[4096];
+    vsnprintf(buffer, sizeof(buffer), message, args);
+    va_end(args);
+    handleMessage(type, source, buffer);
+}
 
-#define SLANG_RHI_FEATURE_X(id, name) name,
-static const std::array<const char*, size_t(Feature::_Count)> kFeatureNames = {SLANG_RHI_FEATURES(SLANG_RHI_FEATURE_X)};
-#undef SLANG_RHI_FEATURE_X
+void Device::printInfo(const char* message, ...)
+{
+    va_list args;
+    va_start(args, message);
+    char buffer[4096];
+    vsnprintf(buffer, sizeof(buffer), message, args);
+    va_end(args);
+    handleMessage(DebugMessageType::Info, DebugMessageSource::Layer, buffer);
+}
 
-#define SLANG_RHI_FEATURE_X(id, name) {name, Feature::id},
-static const std::unordered_map<std::string_view, Feature> kFeatureNameMap = {SLANG_RHI_FEATURES(SLANG_RHI_FEATURE_X)};
-#undef SLANG_RHI_FEATURE_X
+void Device::printWarning(const char* message, ...)
+{
+    va_list args;
+    va_start(args, message);
+    char buffer[4096];
+    vsnprintf(buffer, sizeof(buffer), message, args);
+    va_end(args);
+    handleMessage(DebugMessageType::Warning, DebugMessageSource::Layer, buffer);
+}
+
+void Device::printError(const char* message, ...)
+{
+    va_list args;
+    va_start(args, message);
+    char buffer[4096];
+    vsnprintf(buffer, sizeof(buffer), message, args);
+    va_end(args);
+    handleMessage(DebugMessageType::Error, DebugMessageSource::Layer, buffer);
+}
 
 Result Device::createShaderObject(ShaderObjectLayout* layout, ShaderObject** outObject)
 {
@@ -117,8 +168,8 @@ Result Device::getSpecializedProgram(
         RefPtr<ShaderProgram> specializedProgram;
         SLANG_RETURN_ON_FAIL(specializeProgram(program, specializationArgs, specializedProgram.writeRef()));
         program->m_specializedPrograms[key] = specializedProgram;
-        // Program is owned by the cache
-        specializedProgram->comFree();
+        // Program is owned by the cache (which is owned by the device).
+        specializedProgram->breakStrongReferenceToDevice();
         returnRefPtr(outSpecializedProgram, specializedProgram);
         return SLANG_OK;
     }
@@ -172,6 +223,22 @@ Result Device::getConcretePipeline(
     Pipeline*& outPipeline
 )
 {
+    // Virtual pipelines are created due to 2 reasons:
+    //
+    // 1) When the user creates a pipeline with a program that has undefined specialization arguments, there is no way
+    // to compile target code to create the backend pipeline resources during creation time. Therefore, a virtual
+    // pipeline is created which needs to be specialized later (during dispatch time, when specialization arguments are
+    // known).
+    //
+    // 2) The user requests deferred compilation of target code when creating a pipeline, in which case the
+    // backend resources are not created until they are needed (during dispatch time).
+    //
+    // This code handles both these cases. For the specialization case, we query the device's pipeline cache for
+    // a specialized pipeline that matches the specialization arguments provided. If not found, we specialize
+    // the program, compile the target code, create the backend resources and cache the specialized pipeline
+    // For the deferred case, we simply create the backend resources and keep a reference to the concrete pipeline
+    // in the virtual one for later use.
+
     // If this is already a concrete pipeline, then we are done.
     if (!pipeline->isVirtual())
     {
@@ -179,73 +246,96 @@ Result Device::getConcretePipeline(
         return SLANG_OK;
     }
 
-    // Create key for looking up cached pipelines.
+    // Return early if we previously created a concrete pipeline for this virtual pipeline.
+    outPipeline = pipeline->getConcretePipeline();
+    if (outPipeline)
+    {
+        return SLANG_OK;
+    }
+
+    // Check if program is specializable (if it has specialization arguments).
+    bool isSpecializable = pipeline->m_program->isSpecializable();
+
+    // If the pipeline is specializable, collect specialization arguments from bound shader objects
+    // and lookup the cache.
     PipelineKey pipelineKey;
     pipelineKey.pipeline = pipeline;
-
-    // If the pipeline is specializable, collect specialization arguments from bound shader objects.
-    if (pipeline->m_program->isSpecializable())
+    if (isSpecializable)
     {
         if (!specializationArgs)
+        {
             return SLANG_FAIL;
+        }
         for (const auto& componentID : specializationArgs->componentIDs)
         {
             pipelineKey.specializationArgs.push_back(componentID);
         }
+        pipelineKey.updateHash();
+        outPipeline = m_shaderCache.getSpecializedPipeline(pipelineKey);
+        if (outPipeline)
+        {
+            return SLANG_OK;
+        }
     }
 
-    // Look up pipeline in cache.
-    pipelineKey.updateHash();
-    RefPtr<Pipeline> concretePipeline = m_shaderCache.getSpecializedPipeline(pipelineKey);
-    if (!concretePipeline)
+    // At this point we need to create a new concrete pipeline.
+    RefPtr<ShaderProgram> program = pipeline->m_program;
+    if (isSpecializable)
     {
-        // Specialize program if needed.
-        RefPtr<ShaderProgram> program = pipeline->m_program;
-        if (program->isSpecializable())
-        {
-            RefPtr<ShaderProgram> specializedProgram;
-            SLANG_RETURN_ON_FAIL(specializeProgram(program, *specializationArgs, specializedProgram.writeRef()));
-            program = specializedProgram;
-            // Program is owned by the specialized pipeline.
-            program->comFree();
-        }
+        RefPtr<ShaderProgram> specializedProgram;
+        SLANG_RETURN_ON_FAIL(specializeProgram(program, *specializationArgs, specializedProgram.writeRef()));
+        program = specializedProgram;
+    }
 
-        // Ensure sure shaders are compiled.
-        SLANG_RETURN_ON_FAIL(program->compileShaders(this));
+    // Ensure sure shaders are compiled.
+    SLANG_RETURN_ON_FAIL(program->compileShaders(this));
 
-        switch (pipeline->getType())
-        {
-        case PipelineType::Render:
-        {
-            RenderPipelineDesc desc = checked_cast<VirtualRenderPipeline*>(pipeline)->m_desc;
-            desc.program = program;
-            ComPtr<IRenderPipeline> renderPipeline;
-            SLANG_RETURN_ON_FAIL(createRenderPipeline2(desc, renderPipeline.writeRef()));
-            concretePipeline = checked_cast<RenderPipeline*>(renderPipeline.get());
-            break;
-        }
-        case PipelineType::Compute:
-        {
-            ComputePipelineDesc desc = checked_cast<VirtualComputePipeline*>(pipeline)->m_desc;
-            desc.program = program;
-            ComPtr<IComputePipeline> computePipeline;
-            SLANG_RETURN_ON_FAIL(createComputePipeline2(desc, computePipeline.writeRef()));
-            concretePipeline = checked_cast<ComputePipeline*>(computePipeline.get());
-            break;
-        }
-        case PipelineType::RayTracing:
-        {
-            RayTracingPipelineDesc desc = checked_cast<VirtualRayTracingPipeline*>(pipeline)->m_desc;
-            desc.program = program;
-            ComPtr<IRayTracingPipeline> rayTracingPipeline;
-            SLANG_RETURN_ON_FAIL(createRayTracingPipeline2(desc, rayTracingPipeline.writeRef()));
-            concretePipeline = checked_cast<RayTracingPipeline*>(rayTracingPipeline.get());
-            break;
-        }
-        }
+    // Create a new concrete pipeline.
+    RefPtr<Pipeline> concretePipeline;
+    switch (pipeline->getType())
+    {
+    case PipelineType::Render:
+    {
+        RenderPipelineDesc desc = checked_cast<VirtualRenderPipeline*>(pipeline)->m_desc;
+        desc.program = program;
+        ComPtr<IRenderPipeline> renderPipeline;
+        SLANG_RETURN_ON_FAIL(createRenderPipeline2(desc, renderPipeline.writeRef()));
+        concretePipeline = checked_cast<RenderPipeline*>(renderPipeline.get());
+        break;
+    }
+    case PipelineType::Compute:
+    {
+        ComputePipelineDesc desc = checked_cast<VirtualComputePipeline*>(pipeline)->m_desc;
+        desc.program = program;
+        ComPtr<IComputePipeline> computePipeline;
+        SLANG_RETURN_ON_FAIL(createComputePipeline2(desc, computePipeline.writeRef()));
+        concretePipeline = checked_cast<ComputePipeline*>(computePipeline.get());
+        break;
+    }
+    case PipelineType::RayTracing:
+    {
+        RayTracingPipelineDesc desc = checked_cast<VirtualRayTracingPipeline*>(pipeline)->m_desc;
+        desc.program = program;
+        ComPtr<IRayTracingPipeline> rayTracingPipeline;
+        SLANG_RETURN_ON_FAIL(createRayTracingPipeline2(desc, rayTracingPipeline.writeRef()));
+        concretePipeline = checked_cast<RayTracingPipeline*>(rayTracingPipeline.get());
+        break;
+    }
+    }
+
+    if (isSpecializable)
+    {
+        // Cache the specialized pipeline for later use.
         m_shaderCache.addSpecializedPipeline(pipelineKey, concretePipeline);
         // Pipeline is owned by the cache.
         concretePipeline->breakStrongReferenceToDevice();
+        // Program is owned by the specialized pipeline (which is owned by the cache).
+        concretePipeline->m_program->breakStrongReferenceToDevice();
+    }
+    else
+    {
+        // Store the concrete pipeline in the virtual one.
+        pipeline->setConcretePipeline(concretePipeline);
     }
 
     outPipeline = concretePipeline;
@@ -274,41 +364,78 @@ Result Device::createRayTracingPipeline2(const RayTracingPipelineDesc& desc, IRa
 }
 
 Result Device::getEntryPointCodeFromShaderCache(
-    slang::IComponentType* program,
-    SlangInt entryPointIndex,
-    SlangInt targetIndex,
+    ShaderProgram* program,
+    slang::IComponentType* componentType,
+    const char* entryPointName,
+    uint32_t entryPointIndex,
+    uint32_t targetIndex,
     slang::IBlob** outCode,
     slang::IBlob** outDiagnostics
 )
 {
-    // Immediately call getEntryPointCode if shader cache is not available.
-    if (!m_persistentShaderCache)
-    {
-        return program->getEntryPointCode(entryPointIndex, targetIndex, outCode, outDiagnostics);
-    }
-
-    // Hash all relevant state for generating the entry point shader code to use as a key
-    // for the shader cache.
-    ComPtr<ISlangBlob> hashBlob;
-    program->getEntryPointHash(entryPointIndex, targetIndex, hashBlob.writeRef());
-
-    // Query the shader cache.
+    TimePoint startTime = Timer::now();
     ComPtr<ISlangBlob> codeBlob;
-    if (m_persistentShaderCache->queryCache(hashBlob, codeBlob.writeRef()) != SLANG_OK)
+    ComPtr<ISlangBlob> hashBlob;
+
+    if (m_persistentShaderCache)
     {
-        // No cached entry found. Generate the code and add it to the cache.
-        SLANG_RETURN_ON_FAIL(
-            program->getEntryPointCode(entryPointIndex, targetIndex, codeBlob.writeRef(), outDiagnostics)
-        );
-        m_persistentShaderCache->writeCache(hashBlob, codeBlob);
-        m_shaderCacheMisses++;
-    }
-    else
-    {
-        m_shaderCacheHits++;
+        // Hash all relevant state for generating the entry point shader code to use as a key
+        // for the shader cache.
+        componentType->getEntryPointHash(entryPointIndex, targetIndex, hashBlob.writeRef());
+
+        // Query the shader cache.
+        if (m_persistentShaderCache->queryCache(hashBlob, codeBlob.writeRef()) == SLANG_OK)
+        {
+            if (m_shaderCompilationReporter)
+            {
+                m_shaderCompilationReporter->reportCompileEntryPoint(
+                    program,
+                    entryPointName,
+                    startTime,
+                    Timer::now(),
+                    0.0,
+                    0.0,
+                    true,
+                    codeBlob->getBufferSize()
+                );
+            }
+
+            returnComPtr(outCode, codeBlob);
+            return SLANG_OK;
+        }
     }
 
-    *outCode = codeBlob.detach();
+    // Cached entry not found, generate the code and measure compilation time.
+    double startTotalTime, endTotalTime;
+    double startDownstreamTime, endDownstreamTime;
+    componentType->getSession()->getGlobalSession()->getCompilerElapsedTime(&startTotalTime, &startDownstreamTime);
+    SLANG_RETURN_ON_FAIL(
+        componentType->getEntryPointCode(entryPointIndex, targetIndex, codeBlob.writeRef(), outDiagnostics)
+    );
+    componentType->getSession()->getGlobalSession()->getCompilerElapsedTime(&endTotalTime, &endDownstreamTime);
+
+    // Write the generated code to the shader cache if available.
+    if (m_persistentShaderCache)
+    {
+        m_persistentShaderCache->writeCache(hashBlob, codeBlob);
+    }
+
+    // Report compilation time.
+    if (m_shaderCompilationReporter)
+    {
+        m_shaderCompilationReporter->reportCompileEntryPoint(
+            program,
+            entryPointName,
+            startTime,
+            Timer::now(),
+            endTotalTime - startTotalTime,
+            endDownstreamTime - startDownstreamTime,
+            false,
+            codeBlob->getBufferSize()
+        );
+    }
+
+    returnComPtr(outCode, codeBlob);
     return SLANG_OK;
 }
 
@@ -327,22 +454,26 @@ IDevice* Device::getInterface(const Guid& guid)
 
 Result Device::initialize(const DeviceDesc& desc)
 {
+    m_info = {};
+    m_info.deviceType = desc.deviceType;
+
     m_featureSet.fill(false);
+    m_capabilitySet.fill(false);
+    m_formatSupport.fill(FormatSupport::None);
 
     m_debugCallback = desc.debugCallback ? desc.debugCallback : NullDebugCallback::getInstance();
 
+    if (desc.enableCompilationReports)
+    {
+        m_shaderCompilationReporter = new ShaderCompilationReporter(this);
+    }
+
     m_persistentShaderCache = desc.persistentShaderCache;
+    m_persistentPipelineCache = desc.persistentPipelineCache;
 
     m_uploadHeap.initialize(this, desc.stagingHeapPageSize, MemoryType::Upload);
     m_readbackHeap.initialize(this, desc.stagingHeapPageSize, MemoryType::ReadBack);
 
-    if (desc.apiCommandDispatcher)
-    {
-        desc.apiCommandDispatcher->queryInterface(
-            IPipelineCreationAPIDispatcher::getTypeGuid(),
-            (void**)m_pipelineCreationAPIDispatcher.writeRef()
-        );
-    }
     return SLANG_OK;
 }
 
@@ -352,30 +483,62 @@ void Device::addFeature(Feature feature)
     m_featureSet[size_t(feature)] = true;
 }
 
+void Device::addCapability(Capability capability)
+{
+    SLANG_RHI_ASSERT(size_t(capability) < size_t(Capability::_Count));
+    m_capabilitySet[size_t(capability)] = true;
+}
+
+std::vector<Capability> Device::getCapabilities()
+{
+    std::vector<Capability> capabilities;
+    for (size_t i = 0; i < m_capabilitySet.size(); i++)
+    {
+        if (m_capabilitySet[i])
+        {
+            capabilities.push_back(Capability(i));
+        }
+    }
+    return capabilities;
+}
+
 Result Device::getNativeDeviceHandles(DeviceNativeHandles* outHandles)
 {
     return SLANG_OK;
 }
 
-Result Device::getFeatures(const char** outFeatures, size_t bufferSize, uint32_t* outFeatureCount)
+Result Device::getFeatures(uint32_t* outFeatureCount, Feature* outFeatures)
 {
-    if (!outFeatures)
+    if (!outFeatureCount)
     {
         return SLANG_E_INVALID_ARG;
     }
-
-    size_t featureCount = 0;
-    for (size_t i = 0; i < m_featureSet.size(); i++)
+    if (outFeatures)
     {
-        if (m_featureSet[i] && featureCount < bufferSize)
+        uint32_t featureIndex = 0;
+        for (size_t i = 0; i < m_featureSet.size(); i++)
         {
-            outFeatures[featureCount] = kFeatureNames[i];
-            featureCount++;
+            if (m_featureSet[i])
+            {
+                if (featureIndex < *outFeatureCount)
+                {
+                    outFeatures[featureIndex++] = Feature(i);
+                }
+                else
+                {
+                    return SLANG_E_INVALID_ARG;
+                }
+            }
         }
     }
-    if (outFeatureCount)
+    else
     {
-        *outFeatureCount = uint32_t(featureCount);
+        uint32_t featureCount = 0;
+        for (size_t i = 0; i < m_featureSet.size(); i++)
+        {
+            featureCount += m_featureSet[i] ? 1 : 0;
+        }
+        *outFeatureCount = featureCount;
     }
     return SLANG_OK;
 }
@@ -387,6 +550,12 @@ bool Device::hasFeature(Feature feature)
 
 bool Device::hasFeature(const char* feature)
 {
+#define SLANG_RHI_FEATURES_X(id, name) {name, Feature::id},
+    static const std::unordered_map<std::string_view, Feature> kFeatureNameMap = {
+        SLANG_RHI_FEATURES(SLANG_RHI_FEATURES_X)
+    };
+#undef SLANG_RHI_FEATURES_X
+
     auto it = kFeatureNameMap.find(feature);
     if (it != kFeatureNameMap.end())
     {
@@ -395,23 +564,70 @@ bool Device::hasFeature(const char* feature)
     return false;
 }
 
+Result Device::getCapabilities(uint32_t* outCapabilityCount, Capability* outCapabilities)
+{
+    if (!outCapabilityCount)
+    {
+        return SLANG_E_INVALID_ARG;
+    }
+    if (outCapabilities)
+    {
+        uint32_t capabilityIndex = 0;
+        for (size_t i = 0; i < m_capabilitySet.size(); i++)
+        {
+            if (m_capabilitySet[i])
+            {
+                if (capabilityIndex < *outCapabilityCount)
+                {
+                    outCapabilities[capabilityIndex++] = Capability(i);
+                }
+                else
+                {
+                    return SLANG_E_INVALID_ARG;
+                }
+            }
+        }
+    }
+    else
+    {
+        uint32_t capabilityCount = 0;
+        for (size_t i = 0; i < m_capabilitySet.size(); i++)
+        {
+            capabilityCount += m_capabilitySet[i] ? 1 : 0;
+        }
+        *outCapabilityCount = capabilityCount;
+    }
+    return SLANG_OK;
+}
+
+bool Device::hasCapability(Capability capability)
+{
+    return size_t(capability) < size_t(Capability::_Count) ? m_capabilitySet[size_t(capability)] : false;
+}
+
+bool Device::hasCapability(const char* capability)
+{
+#define SLANG_RHI_CAPABILITIES_X(id) {#id, Capability::id},
+    static const std::unordered_map<std::string_view, Capability> kCapabilityMap = {
+        SLANG_RHI_CAPABILITIES(SLANG_RHI_CAPABILITIES_X)
+    };
+#undef SLANG_RHI_CAPABILITIES_X
+
+    auto it = kCapabilityMap.find(capability);
+    if (it != kCapabilityMap.end())
+    {
+        return hasCapability(it->second);
+    }
+    return false;
+}
+
 Result Device::getFormatSupport(Format format, FormatSupport* outFormatSupport)
 {
-    SLANG_UNUSED(format);
-    FormatSupport support = FormatSupport::None;
-    support |= FormatSupport::Buffer;
-    support |= FormatSupport::IndexBuffer;
-    support |= FormatSupport::VertexBuffer;
-    support |= FormatSupport::Texture;
-    support |= FormatSupport::DepthStencil;
-    support |= FormatSupport::RenderTarget;
-    support |= FormatSupport::Blendable;
-    support |= FormatSupport::ShaderLoad;
-    support |= FormatSupport::ShaderSample;
-    support |= FormatSupport::ShaderUavLoad;
-    support |= FormatSupport::ShaderUavStore;
-    support |= FormatSupport::ShaderAtomic;
-    *outFormatSupport = support;
+    if (size_t(format) >= m_formatSupport.size() || !outFormatSupport)
+    {
+        return SLANG_E_INVALID_ARG;
+    }
+    *outFormatSupport = m_formatSupport[size_t(format)];
     return SLANG_OK;
 }
 
@@ -470,7 +686,7 @@ Result Device::createInputLayout(const InputLayoutDesc& desc, IInputLayout** out
 Result Device::createRenderPipeline(const RenderPipelineDesc& desc, IRenderPipeline** outPipeline)
 {
     ShaderProgram* program = checked_cast<ShaderProgram*>(desc.program);
-    bool createVirtual = program->isSpecializable();
+    bool createVirtual = desc.deferTargetCompilation | program->isSpecializable();
     if (createVirtual)
     {
         RefPtr<VirtualRenderPipeline> pipeline = new VirtualRenderPipeline(this, desc);
@@ -487,7 +703,7 @@ Result Device::createRenderPipeline(const RenderPipelineDesc& desc, IRenderPipel
 Result Device::createComputePipeline(const ComputePipelineDesc& desc, IComputePipeline** outPipeline)
 {
     ShaderProgram* program = checked_cast<ShaderProgram*>(desc.program);
-    bool createVirtual = program->isSpecializable();
+    bool createVirtual = desc.deferTargetCompilation | program->isSpecializable();
     if (createVirtual)
     {
         RefPtr<VirtualComputePipeline> pipeline = new VirtualComputePipeline(this, desc);
@@ -504,7 +720,7 @@ Result Device::createComputePipeline(const ComputePipelineDesc& desc, IComputePi
 Result Device::createRayTracingPipeline(const RayTracingPipelineDesc& desc, IRayTracingPipeline** outPipeline)
 {
     ShaderProgram* program = checked_cast<ShaderProgram*>(desc.program);
-    bool createVirtual = program->isSpecializable();
+    bool createVirtual = desc.deferTargetCompilation | program->isSpecializable();
     if (createVirtual)
     {
         RefPtr<VirtualRayTracingPipeline> pipeline = new VirtualRayTracingPipeline(this, desc);
@@ -518,6 +734,15 @@ Result Device::createRayTracingPipeline(const RayTracingPipelineDesc& desc, IRay
     }
 }
 
+Result Device::getCompilationReportList(ISlangBlob** outReportListBlob)
+{
+    if (!m_shaderCompilationReporter)
+    {
+        return SLANG_E_NOT_AVAILABLE;
+    }
+    return m_shaderCompilationReporter->getCompilationReportList(outReportListBlob);
+}
+
 Result Device::createShaderObject(
     slang::ISession* slangSession,
     slang::TypeReflection* type,
@@ -564,6 +789,13 @@ Result Device::getAccelerationStructureSizes(
     return SLANG_E_NOT_AVAILABLE;
 }
 
+Result Device::getClusterOperationSizes(const ClusterOperationParams& params, ClusterOperationSizes* outSizes)
+{
+    SLANG_UNUSED(params);
+    SLANG_UNUSED(outSizes);
+    return SLANG_E_NOT_AVAILABLE;
+}
+
 Result Device::createAccelerationStructure(
     const AccelerationStructureDesc& desc,
     IAccelerationStructure** outAccelerationStructure
@@ -604,12 +836,19 @@ Result Device::waitForFences(
     return SLANG_E_NOT_AVAILABLE;
 }
 
+Result Device::createHeap(const HeapDesc& desc, IHeap** outHeap)
+{
+    SLANG_UNUSED(desc);
+    SLANG_UNUSED(outHeap);
+    return SLANG_E_NOT_AVAILABLE;
+}
+
 Result Device::readTexture(
     ITexture* texture,
     uint32_t layer,
     uint32_t mip,
-    ISlangBlob** outBlob,
-    SubresourceLayout* outLayout
+    const SubresourceLayout& layout,
+    void* outData
 )
 {
     ComPtr<ICommandQueue> queue;
@@ -618,9 +857,6 @@ Result Device::readTexture(
     ComPtr<ICommandEncoder> commandEncoder;
     SLANG_RETURN_ON_FAIL(queue->createCommandEncoder(commandEncoder.writeRef()));
 
-    SubresourceLayout layout;
-    SLANG_RETURN_ON_FAIL(texture->getSubresourceLayout(mip, &layout));
-
     StagingHeap::Allocation stagingAllocation;
     SLANG_RETURN_ON_FAIL(m_readbackHeap.alloc(layout.sizeInBytes, {}, &stagingAllocation));
 
@@ -642,12 +878,30 @@ Result Device::readTexture(
     void* mappedData;
     SLANG_RETURN_ON_FAIL(m_readbackHeap.map(stagingAllocation, &mappedData));
 
-    auto blob = OwnedBlob::create(mappedData, layout.sizeInBytes);
+    std::memcpy(outData, mappedData, layout.sizeInBytes);
 
     SLANG_RETURN_ON_FAIL(m_readbackHeap.unmap(stagingAllocation));
 
     m_readbackHeap.free(stagingAllocation);
 
+    return SLANG_OK;
+}
+
+Result Device::readTexture(
+    ITexture* texture,
+    uint32_t layer,
+    uint32_t mip,
+    ISlangBlob** outBlob,
+    SubresourceLayout* outLayout
+)
+{
+    SubresourceLayout layout;
+    SLANG_RETURN_ON_FAIL(texture->getSubresourceLayout(mip, &layout));
+
+    auto blob = OwnedBlob::create(layout.sizeInBytes);
+
+    SLANG_RETURN_ON_FAIL(readTexture(texture, layer, mip, layout, (void*)blob->getBufferPointer()));
+
     if (outLayout)
         *outLayout = layout;
 
@@ -684,44 +938,103 @@ Result Device::createSurface(WindowHandle windowHandle, ISurface** outSurface)
     return SLANG_E_NOT_AVAILABLE;
 }
 
-Result Device::getCooperativeVectorProperties(CooperativeVectorProperties* properties, uint32_t* propertyCount)
+Result Device::getCooperativeVectorProperties(CooperativeVectorProperties* properties, uint32_t* propertiesCount)
 {
+    if (!propertiesCount)
+    {
+        return SLANG_E_INVALID_ARG;
+    }
     if (m_cooperativeVectorProperties.empty())
     {
         return SLANG_E_NOT_AVAILABLE;
     }
-
-    if (*propertyCount == 0)
+    if (properties)
     {
-        *propertyCount = uint32_t(m_cooperativeVectorProperties.size());
-        return SLANG_OK;
+        uint32_t count = min(*propertiesCount, uint32_t(m_cooperativeVectorProperties.size()));
+        ::memcpy(properties, m_cooperativeVectorProperties.data(), count * sizeof(CooperativeVectorProperties));
+        Result result = count == *propertiesCount ? SLANG_OK : SLANG_E_BUFFER_TOO_SMALL;
+        *propertiesCount = count;
+        return result;
     }
     else
     {
-        uint32_t count = min(*propertyCount, uint32_t(m_cooperativeVectorProperties.size()));
-        ::memcpy(properties, m_cooperativeVectorProperties.data(), count * sizeof(CooperativeVectorProperties));
-        Result result = count == *propertyCount ? SLANG_OK : SLANG_E_BUFFER_TOO_SMALL;
-        *propertyCount = count;
-        return result;
+        *propertiesCount = uint32_t(m_cooperativeVectorProperties.size());
+        return SLANG_OK;
     }
 }
 
-Result Device::convertCooperativeVectorMatrix(const ConvertCooperativeVectorMatrixDesc* descs, uint32_t descCount)
+Result Device::getCooperativeVectorMatrixSize(
+    uint32_t rowCount,
+    uint32_t colCount,
+    CooperativeVectorComponentType componentType,
+    CooperativeVectorMatrixLayout layout,
+    size_t rowColumnStride,
+    size_t* outSize
+)
 {
-    SLANG_UNUSED(descs);
-    SLANG_UNUSED(descCount);
+    SLANG_UNUSED(rowCount);
+    SLANG_UNUSED(colCount);
+    SLANG_UNUSED(componentType);
+    SLANG_UNUSED(layout);
+    SLANG_UNUSED(rowColumnStride);
+    SLANG_UNUSED(outSize);
     return SLANG_E_NOT_AVAILABLE;
 }
 
-
-Result Device::getShaderCacheStats(size_t* outCacheHitCount, size_t* outCacheMissCount, size_t* outCacheSize)
+Result Device::convertCooperativeVectorMatrix(
+    void* dstBuffer,
+    size_t dstBufferSize,
+    const CooperativeVectorMatrixDesc* dstDescs,
+    const void* srcBuffer,
+    size_t srcBufferSize,
+    const CooperativeVectorMatrixDesc* srcDescs,
+    uint32_t matrixCount
+)
 {
-    if (outCacheHitCount)
-        *outCacheHitCount = m_shaderCacheHits;
-    if (outCacheMissCount)
-        *outCacheMissCount = m_shaderCacheMisses;
-    if (outCacheSize)
-        *outCacheSize = m_shaderCache.getSize();
+    SLANG_UNUSED(dstBuffer);
+    SLANG_UNUSED(dstBufferSize);
+    SLANG_UNUSED(dstDescs);
+    SLANG_UNUSED(srcBuffer);
+    SLANG_UNUSED(srcBufferSize);
+    SLANG_UNUSED(srcDescs);
+    SLANG_UNUSED(matrixCount);
+    return SLANG_E_NOT_AVAILABLE;
+}
+
+Result Device::reportHeaps(HeapReport* heapReports, uint32_t* heapCount)
+{
+    if (!heapCount)
+        return SLANG_E_INVALID_ARG;
+
+    uint32_t totalHeapCount = static_cast<uint32_t>(m_globalHeaps.size());
+
+    // If only querying count, return early
+    if (!heapReports)
+    {
+        *heapCount = totalHeapCount;
+        return SLANG_OK;
+    }
+
+    // If buffer is provided, it must be large enough
+    if (*heapCount < totalHeapCount)
+        return SLANG_E_BUFFER_TOO_SMALL;
+
+    // Fill heap reports
+    for (uint32_t i = 0; i < totalHeapCount; i++)
+    {
+        SLANG_RETURN_ON_FAIL(m_globalHeaps[i]->report(&heapReports[i]));
+    }
+
+    *heapCount = totalHeapCount;
+    return SLANG_OK;
+}
+
+Result Device::flushHeaps()
+{
+    for (Heap* heap : m_globalHeaps)
+    {
+        SLANG_RETURN_ON_FAIL(heap->flush());
+    }
     return SLANG_OK;
 }
 
