@@ -7,8 +7,7 @@
 #include "metal-pipeline.h"
 #include "metal-acceleration-structure.h"
 #include "metal-shader-object.h"
-#include "metal-helper-functions.h"
-#include "metal-util.h"
+#include "metal-utils.h"
 #include "../strings.h"
 
 namespace rhi::metal {
@@ -92,9 +91,11 @@ public:
     void cmdQueryAccelerationStructureProperties(const commands::QueryAccelerationStructureProperties& cmd);
     void cmdSerializeAccelerationStructure(const commands::SerializeAccelerationStructure& cmd);
     void cmdDeserializeAccelerationStructure(const commands::DeserializeAccelerationStructure& cmd);
+    void cmdExecuteClusterOperation(const commands::ExecuteClusterOperation& cmd);
     void cmdConvertCooperativeVectorMatrix(const commands::ConvertCooperativeVectorMatrix& cmd);
     void cmdSetBufferState(const commands::SetBufferState& cmd);
     void cmdSetTextureState(const commands::SetTextureState& cmd);
+    void cmdGlobalBarrier(const commands::GlobalBarrier& cmd);
     void cmdPushDebugGroup(const commands::PushDebugGroup& cmd);
     void cmdPopDebugGroup(const commands::PopDebugGroup& cmd);
     void cmdInsertDebugMarker(const commands::InsertDebugMarker& cmd);
@@ -144,7 +145,7 @@ Result CommandRecorder::record(CommandBufferImpl* commandBuffer)
     return SLANG_OK;
 }
 
-#define NOT_SUPPORTED(x) m_device->warning(x " command is not supported!")
+#define NOT_SUPPORTED(x) m_device->printWarning(x " command is not supported!")
 
 void CommandRecorder::cmdCopyBuffer(const commands::CopyBuffer& cmd)
 {
@@ -271,7 +272,7 @@ void CommandRecorder::cmdClearTextureDepthStencil(const commands::ClearTextureDe
         NS::TransferPtr(MTL::RenderPassDescriptor::alloc()->init());
 
     // Setup depth stencil attachment
-    if (MetalUtil::isDepthFormat(texture->m_pixelFormat) && cmd.clearDepth)
+    if (isDepthFormat(texture->m_pixelFormat) && cmd.clearDepth)
     {
         MTL::RenderPassDepthAttachmentDescriptor* depthAttachment = renderPassDesc->depthAttachment();
         depthAttachment->setLoadAction(MTL::LoadActionClear);
@@ -279,7 +280,7 @@ void CommandRecorder::cmdClearTextureDepthStencil(const commands::ClearTextureDe
         depthAttachment->setClearDepth(cmd.depthValue);
         depthAttachment->setTexture(texture->m_texture.get());
     }
-    if (MetalUtil::isStencilFormat(texture->m_pixelFormat) && cmd.clearStencil)
+    if (isStencilFormat(texture->m_pixelFormat) && cmd.clearStencil)
     {
         MTL::RenderPassStencilAttachmentDescriptor* stencilAttachment = renderPassDesc->stencilAttachment();
         stencilAttachment->setLoadAction(MTL::LoadActionClear);
@@ -297,12 +298,12 @@ void CommandRecorder::cmdClearTextureDepthStencil(const commands::ClearTextureDe
             uint32_t mip = cmd.subresourceRange.mip + mipOffset;
 
             // Set the level and slice for this iteration
-            if (MetalUtil::isDepthFormat(texture->m_pixelFormat) && cmd.clearDepth)
+            if (isDepthFormat(texture->m_pixelFormat) && cmd.clearDepth)
             {
                 renderPassDesc->depthAttachment()->setLevel(mip);
                 renderPassDesc->depthAttachment()->setSlice(layerIndex);
             }
-            if (MetalUtil::isStencilFormat(texture->m_pixelFormat) && cmd.clearStencil)
+            if (isStencilFormat(texture->m_pixelFormat) && cmd.clearStencil)
             {
                 renderPassDesc->stencilAttachment()->setLevel(mip);
                 renderPassDesc->stencilAttachment()->setSlice(layerIndex);
@@ -399,18 +400,18 @@ void CommandRecorder::cmdBeginRenderPass(const commands::BeginRenderPass& cmd)
         visitView(view);
 
         MTL::RenderPassColorAttachmentDescriptor* colorAttachment = renderPassDesc->colorAttachments()->object(i);
-        colorAttachment->setLoadAction(MetalUtil::translateLoadOp(attachment.loadOp));
-        colorAttachment->setStoreAction(
-            MetalUtil::translateStoreOp(attachment.storeOp, attachment.resolveTarget != nullptr)
-        );
+        colorAttachment->setLoadAction(translateLoadOp(attachment.loadOp));
+        colorAttachment->setStoreAction(translateStoreOp(attachment.storeOp, attachment.resolveTarget != nullptr));
         if (attachment.loadOp == LoadOp::Clear)
         {
-            colorAttachment->setClearColor(MTL::ClearColor(
-                attachment.clearValue[0],
-                attachment.clearValue[1],
-                attachment.clearValue[2],
-                attachment.clearValue[3]
-            ));
+            colorAttachment->setClearColor(
+                MTL::ClearColor(
+                    attachment.clearValue[0],
+                    attachment.clearValue[1],
+                    attachment.clearValue[2],
+                    attachment.clearValue[3]
+                )
+            );
         }
         colorAttachment->setTexture(view->m_textureView.get());
         colorAttachment->setResolveTexture(
@@ -430,12 +431,12 @@ void CommandRecorder::cmdBeginRenderPass(const commands::BeginRenderPass& cmd)
             return;
         visitView(view);
 
-        MTL::PixelFormat pixelFormat = MetalUtil::translatePixelFormat(view->m_desc.format);
-        if (MetalUtil::isDepthFormat(pixelFormat))
+        MTL::PixelFormat pixelFormat = translatePixelFormat(view->m_desc.format);
+        if (isDepthFormat(pixelFormat))
         {
             MTL::RenderPassDepthAttachmentDescriptor* depthAttachment = renderPassDesc->depthAttachment();
-            depthAttachment->setLoadAction(MetalUtil::translateLoadOp(attachment.depthLoadOp));
-            depthAttachment->setStoreAction(MetalUtil::translateStoreOp(attachment.depthStoreOp, false));
+            depthAttachment->setLoadAction(translateLoadOp(attachment.depthLoadOp));
+            depthAttachment->setStoreAction(translateStoreOp(attachment.depthStoreOp, false));
             if (attachment.depthLoadOp == LoadOp::Clear)
             {
                 depthAttachment->setClearDepth(attachment.depthClearValue);
@@ -444,11 +445,11 @@ void CommandRecorder::cmdBeginRenderPass(const commands::BeginRenderPass& cmd)
             depthAttachment->setLevel(view->m_desc.subresourceRange.mip);
             depthAttachment->setSlice(view->m_desc.subresourceRange.layer);
         }
-        if (MetalUtil::isStencilFormat(pixelFormat))
+        if (isStencilFormat(pixelFormat))
         {
             MTL::RenderPassStencilAttachmentDescriptor* stencilAttachment = renderPassDesc->stencilAttachment();
-            stencilAttachment->setLoadAction(MetalUtil::translateLoadOp(attachment.stencilLoadOp));
-            stencilAttachment->setStoreAction(MetalUtil::translateStoreOp(attachment.stencilStoreOp, false));
+            stencilAttachment->setLoadAction(translateLoadOp(attachment.stencilLoadOp));
+            stencilAttachment->setStoreAction(translateStoreOp(attachment.stencilStoreOp, false));
             if (attachment.stencilLoadOp == LoadOp::Clear)
             {
                 stencilAttachment->setClearStencil(attachment.stencilClearValue);
@@ -603,13 +604,13 @@ void CommandRecorder::cmdSetRenderState(const commands::SetRenderState& cmd)
     }
 
     const RasterizerDesc& rasterizer = m_renderPipeline->m_rasterizerDesc;
-    encoder->setFrontFacingWinding(MetalUtil::translateWinding(rasterizer.frontFace));
-    encoder->setCullMode(MetalUtil::translateCullMode(rasterizer.cullMode));
+    encoder->setFrontFacingWinding(translateWinding(rasterizer.frontFace));
+    encoder->setCullMode(translateCullMode(rasterizer.cullMode));
     encoder->setDepthClipMode(
         rasterizer.depthClipEnable ? MTL::DepthClipModeClip : MTL::DepthClipModeClamp
     ); // TODO correct?
     encoder->setDepthBias(rasterizer.depthBias, rasterizer.slopeScaledDepthBias, rasterizer.depthBiasClamp);
-    encoder->setTriangleFillMode(MetalUtil::translateTriangleFillMode(rasterizer.fillMode));
+    encoder->setTriangleFillMode(translateTriangleFillMode(rasterizer.fillMode));
     // encoder->setBlendColor(); // not supported by rhi
     if (m_useDepthStencil)
     {
@@ -827,6 +828,12 @@ void CommandRecorder::cmdDeserializeAccelerationStructure(const commands::Deseri
     NOT_SUPPORTED(S_CommandEncoder_deserializeAccelerationStructure);
 }
 
+void CommandRecorder::cmdExecuteClusterOperation(const commands::ExecuteClusterOperation& cmd)
+{
+    SLANG_UNUSED(cmd);
+    NOT_SUPPORTED(S_CommandEncoder_executeClusterOperation);
+}
+
 void CommandRecorder::cmdConvertCooperativeVectorMatrix(const commands::ConvertCooperativeVectorMatrix& cmd)
 {
     SLANG_UNUSED(cmd);
@@ -843,9 +850,14 @@ void CommandRecorder::cmdSetTextureState(const commands::SetTextureState& cmd)
     SLANG_UNUSED(cmd);
 }
 
+void CommandRecorder::cmdGlobalBarrier(const commands::GlobalBarrier& cmd)
+{
+    SLANG_UNUSED(cmd);
+}
+
 void CommandRecorder::cmdPushDebugGroup(const commands::PushDebugGroup& cmd)
 {
-    NS::SharedPtr<NS::String> string = MetalUtil::createString(cmd.name);
+    NS::SharedPtr<NS::String> string = createString(cmd.name);
     m_commandBuffer->pushDebugGroup(string.get());
 }
 
@@ -857,7 +869,7 @@ void CommandRecorder::cmdPopDebugGroup(const commands::PopDebugGroup& cmd)
 void CommandRecorder::cmdInsertDebugMarker(const commands::InsertDebugMarker& cmd)
 {
     SLANG_UNUSED(cmd);
-    // NS::SharedPtr<NS::String> string = MetalUtil::createString(cmd.name);
+    // NS::SharedPtr<NS::String> string = createString(cmd.name);
     // m_commandBuffer->insertDebugSignpost(string.get());
 }
 
@@ -985,6 +997,9 @@ void CommandQueueImpl::retireCommandBuffers()
             m_commandBuffersInFlight.push_back(commandBuffer);
         }
     }
+
+    // Flush all device heaps
+    getDevice<DeviceImpl>()->flushHeaps();
 }
 
 uint64_t CommandQueueImpl::updateLastFinishedID()
