@@ -3,6 +3,7 @@
 #include "metal-buffer.h"
 #include "metal-texture.h"
 #include "metal-sampler.h"
+#include <slang.h>
 
 namespace rhi::metal {
 
@@ -88,18 +89,9 @@ Result BindingDataBuilder::bindAsRoot(
     m_bindingData->usedRWResourceCount = 0;
     m_bindingData->usedRWResources = m_allocator->allocate<MTL::Resource*>(m_bindingData->usedRWResourceCapacity);
 
-    // When binding an entire root shader object, we need to deal with
-    // the way that specialization might have allocated space for "pending"
-    // parameter data after all the primary parameters.
-    //
-    // We start by initializing an offset that will store zeros for the
-    // primary data, an the computed offset from the specialized layout
-    // for pending data.
+    // Initialize binding offset for shader parameters.
     //
     BindingOffset offset;
-#if 0
-    offset.pending = layout->getPendingDataOffset();
-#endif
 
     // Note: We could *almost* call `bindAsConstantBuffer()` here to bind
     // the state of the root object itself, but there is an important
@@ -189,7 +181,11 @@ Result BindingDataBuilder::bindAsParameterBlock(
 
     BufferImpl* argumentBuffer = nullptr;
     SLANG_RETURN_ON_FAIL(writeArgumentBuffer(shaderObject, specializedLayout, argumentBuffer));
-    SLANG_RETURN_ON_FAIL(setBuffer(m_bindingData, inOffset.buffer, argumentBuffer->m_buffer.get()));
+
+    if (argumentBuffer)
+    {
+        SLANG_RETURN_ON_FAIL(setBuffer(m_bindingData, inOffset.buffer, argumentBuffer->m_buffer.get()));
+    }
 
     return SLANG_OK;
 }
@@ -327,13 +323,8 @@ Result BindingDataBuilder::bindAsValue(
             //
             if (subObjectLayout)
             {
-                // The data for objects in this range will always be bound into
-                // the "pending" allocation for the parent block/buffer/object.
-                // As a result, the offset for the first object in the range
-                // will come from the `pending` part of the range's offset.
-                //
-                SimpleBindingOffset objOffset = rangeOffset.pending;
-                SimpleBindingOffset objStride = rangeStride.pending;
+                SimpleBindingOffset objOffset = {};
+                SimpleBindingOffset objStride = {};
 
                 for (Index i = 0; i < count; ++i)
                 {
@@ -404,6 +395,14 @@ Result BindingDataBuilder::writeArgumentBuffer(
 {
     auto argumentBufferTypeLayout = specializedLayout->getParameterBlockTypeLayout();
 
+    // If the argument buffer has no fields, we don't need to create one, note this is legal because there could be an
+    // empty struct type in AST. We need to handle this correctly.
+    if (argumentBufferTypeLayout->getFieldCount() == 0)
+    {
+        outArgumentBuffer = nullptr;
+        return SLANG_OK;
+    }
+
     ComPtr<IBuffer> argumentBuffer;
     BufferDesc argumentBufferDesc = {};
     argumentBufferDesc.size = argumentBufferTypeLayout->getSize();
@@ -421,6 +420,15 @@ Result BindingDataBuilder::writeArgumentBuffer(
     //
     uint8_t* argumentData = (uint8_t*)argumentBufferImpl->m_buffer->contents();
 
+    // Write all ordinary data early to prevent overwriting gpu-addresses
+    // we write below.
+    SLANG_RETURN_ON_FAIL(writeOrdinaryDataIntoArgumentBuffer(
+        argumentBufferTypeLayout,
+        shaderObject->getElementTypeLayout(),
+        (uint8_t*)argumentData,
+        (uint8_t*)shaderObject->m_data.data()
+    ));
+
     for (uint32_t bindingRangeIndex = 0; bindingRangeIndex < specializedLayout->getBindingRangeCount();
          ++bindingRangeIndex)
     {
@@ -556,4 +564,37 @@ Result BindingDataBuilder::writeArgumentBuffer(
     return SLANG_OK;
 }
 
+Result BindingDataBuilder::writeOrdinaryDataIntoArgumentBuffer(
+    slang::TypeLayoutReflection* argumentBufferTypeLayout,
+    slang::TypeLayoutReflection* defaultTypeLayout,
+    uint8_t* argumentBuffer,
+    uint8_t* srcData
+)
+{
+    // If we are pure data, just copy it over from srcData.
+    if (defaultTypeLayout->getCategoryCount() == 1)
+    {
+        if (defaultTypeLayout->getCategoryByIndex(0) == slang::ParameterCategory::Uniform)
+        {
+            // Just write the uniform data.
+            memcpy(argumentBuffer, srcData, defaultTypeLayout->getSize());
+        }
+        return SLANG_OK;
+    }
+
+    for (unsigned int i = 0; i < argumentBufferTypeLayout->getFieldCount(); i++)
+    {
+        auto argumentBufferField = argumentBufferTypeLayout->getFieldByIndex(i);
+        auto defaultLayoutField = defaultTypeLayout->getFieldByIndex(i);
+        // If the field is mixed type, recurse.
+        SLANG_RETURN_ON_FAIL(writeOrdinaryDataIntoArgumentBuffer(
+            argumentBufferField->getTypeLayout(),
+            defaultLayoutField->getTypeLayout(),
+            argumentBuffer + argumentBufferField->getOffset(),
+            srcData + defaultLayoutField->getOffset()
+        ));
+    }
+    return SLANG_OK;
+}
+
 } // namespace rhi::metal
