@@ -1,5 +1,7 @@
 #pragma once
 
+#include <cstdint>
+
 namespace rhi {
 
 class DeviceChild;
@@ -10,6 +12,7 @@ class Texture;
 class QueryPool;
 class InputLayout;
 class ShaderTable;
+class Heap;
 
 class CommandBuffer;
 class CommandEncoder;
@@ -39,7 +42,8 @@ class VirtualRayTracingPipeline;
 
 struct SpecializationKey;
 class ShaderProgram;
-
+class ShaderCompilationReporter;
 
 typedef uint32_t ShaderComponentID;
+
 } // namespace rhi
