@@ -1,7 +1,7 @@
 #include "vk-acceleration-structure.h"
 #include "vk-device.h"
 #include "vk-buffer.h"
-#include "vk-util.h"
+#include "vk-utils.h"
 
 namespace rhi::vk {
 
@@ -36,6 +36,26 @@ DeviceAddress AccelerationStructureImpl::getDeviceAddress()
     return m_buffer->getDeviceAddress();
 }
 
+Result AccelerationStructureImpl::getDescriptorHandle(DescriptorHandle* outHandle)
+{
+    DeviceImpl* device = getDevice<DeviceImpl>();
+
+    if (!device->m_bindlessDescriptorSet)
+    {
+        return SLANG_E_NOT_AVAILABLE;
+    }
+
+    if (!m_descriptorHandle)
+    {
+        SLANG_RETURN_ON_FAIL(
+            device->m_bindlessDescriptorSet->allocAccelerationStructureHandle(this, &m_descriptorHandle)
+        );
+    }
+
+    *outHandle = m_descriptorHandle;
+    return SLANG_OK;
+}
+
 Result AccelerationStructureBuildDescConverter::convert(
     const AccelerationStructureBuildDesc& buildDesc,
     IDebugCallback* debugCallback
@@ -46,6 +66,11 @@ Result AccelerationStructureBuildDescConverter::convert(
         return SLANG_E_INVALID_ARG;
     }
 
+    if (buildDesc.motionOptions.keyCount > kMaxAccelerationStructureMotionKeyCount)
+    {
+        return SLANG_E_INVALID_ARG;
+    }
+
     AccelerationStructureBuildInputType type = buildDesc.inputs[0].type;
     for (uint32_t i = 1; i < buildDesc.inputCount; ++i)
     {
@@ -97,10 +122,20 @@ Result AccelerationStructureBuildDescConverter::convert(
     }
     case AccelerationStructureBuildInputType::Triangles:
     {
+        bool useMotion = is_set(buildDesc.flags, AccelerationStructureBuildFlags::CreateMotion);
+        if (useMotion)
+        {
+            motionTrianglesDatas.resize(buildDesc.inputCount);
+        }
+
         for (uint32_t i = 0; i < buildDesc.inputCount; ++i)
         {
             const AccelerationStructureBuildInputTriangles& triangles = buildDesc.inputs[i].triangles;
-            if (triangles.vertexBufferCount != 1)
+            if (triangles.vertexBufferCount > kMaxAccelerationStructureMotionKeyCount)
+            {
+                return SLANG_E_INVALID_ARG;
+            }
+            if (useMotion && triangles.vertexBufferCount < 2)
             {
                 return SLANG_E_INVALID_ARG;
             }
@@ -110,7 +145,7 @@ Result AccelerationStructureBuildDescConverter::convert(
             geometry.geometryType = VK_GEOMETRY_TYPE_TRIANGLES_KHR;
             geometry.flags = translateGeometryFlags(triangles.flags);
             geometry.geometry.triangles.sType = VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_GEOMETRY_TRIANGLES_DATA_KHR;
-            geometry.geometry.triangles.vertexFormat = VulkanUtil::getVkFormat(triangles.vertexFormat);
+            geometry.geometry.triangles.vertexFormat = getVkFormat(triangles.vertexFormat);
             geometry.geometry.triangles.vertexData.deviceAddress = triangles.vertexBuffers[0].getDeviceAddress();
             geometry.geometry.triangles.vertexStride = triangles.vertexStride;
             geometry.geometry.triangles.maxVertex = triangles.vertexCount - 1;
@@ -128,6 +163,16 @@ Result AccelerationStructureBuildDescConverter::convert(
             geometry.geometry.triangles.transformData.deviceAddress =
                 triangles.preTransformBuffer ? triangles.preTransformBuffer.getDeviceAddress() : 0;
 
+            if (useMotion)
+            {
+                VkAccelerationStructureGeometryMotionTrianglesDataNV& motionData = motionTrianglesDatas[i];
+                motionData.sType = VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_GEOMETRY_MOTION_TRIANGLES_DATA_NV;
+                motionData.pNext = nullptr;
+                motionData.vertexData.deviceAddress = triangles.vertexBuffers[1].getDeviceAddress();
+
+                geometry.geometry.triangles.pNext = &motionData;
+            }
+
             primitiveCounts[i] = max(triangles.vertexCount, triangles.indexCount) / 3;
         }
 
@@ -136,6 +181,12 @@ Result AccelerationStructureBuildDescConverter::convert(
     }
     case AccelerationStructureBuildInputType::ProceduralPrimitives:
     {
+        // Motion is only supported for triangles in Vulkan
+        if (is_set(buildDesc.flags, AccelerationStructureBuildFlags::CreateMotion))
+        {
+            return SLANG_E_NOT_AVAILABLE;
+        }
+
         for (uint32_t i = 0; i < buildDesc.inputCount; ++i)
         {
             const AccelerationStructureBuildInputProceduralPrimitives& proceduralPrimitives =
@@ -161,6 +212,12 @@ Result AccelerationStructureBuildDescConverter::convert(
     }
     case AccelerationStructureBuildInputType::Spheres:
     {
+        // Motion is only supported for triangles in Vulkan
+        if (is_set(buildDesc.flags, AccelerationStructureBuildFlags::CreateMotion))
+        {
+            return SLANG_E_NOT_AVAILABLE;
+        }
+
         spheresDatas.resize(buildDesc.inputCount);
         for (uint32_t i = 0; i < buildDesc.inputCount; ++i)
         {
@@ -173,10 +230,10 @@ Result AccelerationStructureBuildDescConverter::convert(
             VkAccelerationStructureGeometrySpheresDataNV& spheresData = spheresDatas[i];
             spheresData.sType = VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_GEOMETRY_SPHERES_DATA_NV;
 
-            spheresData.vertexFormat = VulkanUtil::getVkFormat(spheres.vertexPositionFormat);
+            spheresData.vertexFormat = getVkFormat(spheres.vertexPositionFormat);
             spheresData.vertexData.deviceAddress = spheres.vertexPositionBuffers[0].getDeviceAddress();
             spheresData.vertexStride = spheres.vertexPositionStride;
-            spheresData.radiusFormat = VulkanUtil::getVkFormat(spheres.vertexRadiusFormat);
+            spheresData.radiusFormat = getVkFormat(spheres.vertexRadiusFormat);
             spheresData.radiusData.deviceAddress = spheres.vertexRadiusBuffers[0].getDeviceAddress();
             spheresData.radiusStride = spheres.vertexRadiusStride;
             if (spheres.indexBuffer)
@@ -207,6 +264,12 @@ Result AccelerationStructureBuildDescConverter::convert(
     }
     case AccelerationStructureBuildInputType::LinearSweptSpheres:
     {
+        // Motion is only supported for triangles in Vulkan
+        if (is_set(buildDesc.flags, AccelerationStructureBuildFlags::CreateMotion))
+        {
+            return SLANG_E_NOT_AVAILABLE;
+        }
+
         linearSweptSpheresDatas.resize(buildDesc.inputCount);
         for (uint32_t i = 0; i < buildDesc.inputCount; ++i)
         {
@@ -217,12 +280,12 @@ Result AccelerationStructureBuildDescConverter::convert(
             }
 
             VkAccelerationStructureGeometryLinearSweptSpheresDataNV& lssData = linearSweptSpheresDatas[i];
-            lssData.sType = VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_GEOMETRY_SPHERES_DATA_NV;
+            lssData.sType = VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_GEOMETRY_LINEAR_SWEPT_SPHERES_DATA_NV;
 
-            lssData.vertexFormat = VulkanUtil::getVkFormat(lss.vertexPositionFormat);
+            lssData.vertexFormat = getVkFormat(lss.vertexPositionFormat);
             lssData.vertexData.deviceAddress = lss.vertexPositionBuffers[0].getDeviceAddress();
             lssData.vertexStride = lss.vertexPositionStride;
-            lssData.radiusFormat = VulkanUtil::getVkFormat(lss.vertexRadiusFormat);
+            lssData.radiusFormat = getVkFormat(lss.vertexRadiusFormat);
             lssData.radiusData.deviceAddress = lss.vertexRadiusBuffers[0].getDeviceAddress();
             lssData.radiusStride = lss.vertexRadiusStride;
             if (lss.indexBuffer)
@@ -285,6 +348,10 @@ VkBuildAccelerationStructureFlagsKHR AccelerationStructureBuildDescConverter::tr
     {
         result |= VK_BUILD_ACCELERATION_STRUCTURE_PREFER_FAST_TRACE_BIT_KHR;
     }
+    if (is_set(flags, AccelerationStructureBuildFlags::CreateMotion))
+    {
+        result |= VK_BUILD_ACCELERATION_STRUCTURE_MOTION_BIT_NV;
+    }
     return result;
 }
 
@@ -330,5 +397,120 @@ VkRayTracingLssPrimitiveEndCapsModeNV AccelerationStructureBuildDescConverter::t
     }
 }
 
+VkBuildAccelerationStructureFlagsKHR translateClusterOperationFlags(ClusterOperationFlags flags)
+{
+    VkBuildAccelerationStructureFlagsKHR result = 0;
+    if (is_set(flags, ClusterOperationFlags::FastTrace))
+    {
+        result |= VK_BUILD_ACCELERATION_STRUCTURE_PREFER_FAST_TRACE_BIT_KHR;
+    }
+    if (is_set(flags, ClusterOperationFlags::FastBuild))
+    {
+        result |= VK_BUILD_ACCELERATION_STRUCTURE_PREFER_FAST_BUILD_BIT_KHR;
+    }
+    if (is_set(flags, ClusterOperationFlags::AllowOMM))
+    {
+        result |= VK_BUILD_ACCELERATION_STRUCTURE_ALLOW_DISABLE_OPACITY_MICROMAPS_BIT_EXT;
+    }
+    return result;
+}
+
+VkClusterAccelerationStructureTypeNV translateClusterOperationMoveType(ClusterOperationMoveType type)
+{
+    switch (type)
+    {
+    case ClusterOperationMoveType::BottomLevel:
+        return VK_CLUSTER_ACCELERATION_STRUCTURE_TYPE_CLUSTERS_BOTTOM_LEVEL_NV;
+    case ClusterOperationMoveType::ClusterLevel:
+        return VK_CLUSTER_ACCELERATION_STRUCTURE_TYPE_TRIANGLE_CLUSTER_NV;
+    case ClusterOperationMoveType::Template:
+        return VK_CLUSTER_ACCELERATION_STRUCTURE_TYPE_TRIANGLE_CLUSTER_TEMPLATE_NV;
+    }
+    return VkClusterAccelerationStructureTypeNV(0);
+}
+
+VkClusterAccelerationStructureInputInfoNV translateClusterOperationParams(
+    const ClusterOperationParams& params,
+    VkClusterAccelerationStructureClustersBottomLevelInputNV& bottomLevelInput,
+    VkClusterAccelerationStructureTriangleClusterInputNV& triangleClusterInput,
+    VkClusterAccelerationStructureMoveObjectsInputNV& moveObjectsInput
+)
+{
+    VkClusterAccelerationStructureInputInfoNV info = {VK_STRUCTURE_TYPE_CLUSTER_ACCELERATION_STRUCTURE_INPUT_INFO_NV};
+
+    info.maxAccelerationStructureCount = params.maxArgCount;
+    info.flags = translateClusterOperationFlags(params.flags);
+    info.opMode = translateClusterOperationMode(params.mode);
+
+    switch (params.type)
+    {
+    case ClusterOperationType::CLASFromTriangles:
+        info.opType = VK_CLUSTER_ACCELERATION_STRUCTURE_OP_TYPE_BUILD_TRIANGLE_CLUSTER_NV;
+        break;
+    case ClusterOperationType::BLASFromCLAS:
+        info.opType = VK_CLUSTER_ACCELERATION_STRUCTURE_OP_TYPE_BUILD_CLUSTERS_BOTTOM_LEVEL_NV;
+        break;
+    case ClusterOperationType::TemplatesFromTriangles:
+        info.opType = VK_CLUSTER_ACCELERATION_STRUCTURE_OP_TYPE_BUILD_TRIANGLE_CLUSTER_TEMPLATE_NV;
+        break;
+    case ClusterOperationType::CLASFromTemplates:
+        info.opType = VK_CLUSTER_ACCELERATION_STRUCTURE_OP_TYPE_INSTANTIATE_TRIANGLE_CLUSTER_NV;
+        break;
+    case ClusterOperationType::MoveObjects:
+        info.opType = VK_CLUSTER_ACCELERATION_STRUCTURE_OP_TYPE_MOVE_OBJECTS_NV;
+        break;
+    }
+
+    bottomLevelInput = {VK_STRUCTURE_TYPE_CLUSTER_ACCELERATION_STRUCTURE_CLUSTERS_BOTTOM_LEVEL_INPUT_NV};
+    triangleClusterInput = {VK_STRUCTURE_TYPE_CLUSTER_ACCELERATION_STRUCTURE_TRIANGLE_CLUSTER_INPUT_NV};
+    moveObjectsInput = {VK_STRUCTURE_TYPE_CLUSTER_ACCELERATION_STRUCTURE_MOVE_OBJECTS_INPUT_NV};
+
+    switch (info.opType)
+    {
+    case VK_CLUSTER_ACCELERATION_STRUCTURE_OP_TYPE_MOVE_OBJECTS_NV:
+        moveObjectsInput.type = translateClusterOperationMoveType(params.move.type);
+        moveObjectsInput.noMoveOverlap = is_set(params.flags, ClusterOperationFlags::NoOverlap) ? VK_TRUE : VK_FALSE;
+        moveObjectsInput.maxMovedBytes = params.move.maxSize;
+        info.opInput.pMoveObjects = &moveObjectsInput;
+        break;
+    case VK_CLUSTER_ACCELERATION_STRUCTURE_OP_TYPE_BUILD_CLUSTERS_BOTTOM_LEVEL_NV:
+        bottomLevelInput.maxTotalClusterCount = params.blas.maxTotalClasCount;
+        bottomLevelInput.maxClusterCountPerAccelerationStructure = params.blas.maxClasCount;
+        info.opInput.pClustersBottomLevel = &bottomLevelInput;
+        break;
+    case VK_CLUSTER_ACCELERATION_STRUCTURE_OP_TYPE_BUILD_TRIANGLE_CLUSTER_NV:
+    case VK_CLUSTER_ACCELERATION_STRUCTURE_OP_TYPE_BUILD_TRIANGLE_CLUSTER_TEMPLATE_NV:
+    case VK_CLUSTER_ACCELERATION_STRUCTURE_OP_TYPE_INSTANTIATE_TRIANGLE_CLUSTER_NV:
+        triangleClusterInput.maxGeometryIndexValue = params.clas.maxGeometryIndex;
+        triangleClusterInput.vertexFormat = getVkFormat(params.clas.vertexFormat);
+        triangleClusterInput.maxGeometryIndexValue = params.clas.maxGeometryIndex;
+        triangleClusterInput.maxClusterUniqueGeometryCount = params.clas.maxUniqueGeometryCount;
+        triangleClusterInput.maxClusterTriangleCount = params.clas.maxTriangleCount;
+        triangleClusterInput.maxClusterVertexCount = params.clas.maxVertexCount;
+        triangleClusterInput.maxTotalTriangleCount = params.clas.maxTotalTriangleCount;
+        triangleClusterInput.maxTotalVertexCount = params.clas.maxTotalVertexCount;
+        triangleClusterInput.minPositionTruncateBitCount = params.clas.minPositionTruncateBitCount;
+        info.opInput.pTriangleClusters = &triangleClusterInput;
+        break;
+    default:
+        break;
+    }
+
+    return info;
+}
+
+VkClusterAccelerationStructureOpModeNV translateClusterOperationMode(ClusterOperationMode mode)
+{
+    switch (mode)
+    {
+    case ClusterOperationMode::ImplicitDestinations:
+        return VK_CLUSTER_ACCELERATION_STRUCTURE_OP_MODE_IMPLICIT_DESTINATIONS_NV;
+    case ClusterOperationMode::ExplicitDestinations:
+        return VK_CLUSTER_ACCELERATION_STRUCTURE_OP_MODE_EXPLICIT_DESTINATIONS_NV;
+    case ClusterOperationMode::GetSizes:
+        return VK_CLUSTER_ACCELERATION_STRUCTURE_OP_MODE_COMPUTE_SIZES_NV;
+    }
+    return VkClusterAccelerationStructureOpModeNV(0);
+}
 
 } // namespace rhi::vk
