@@ -7,8 +7,7 @@
 #include "vk-acceleration-structure.h"
 #include "vk-shader-table.h"
 #include "vk-pipeline.h"
-#include "vk-util.h"
-#include "vk-helper-functions.h"
+#include "vk-utils.h"
 #include "vk-shader-object.h"
 #include "vk-shader-object-layout.h"
 #include "../command-list.h"
@@ -44,6 +43,10 @@ public:
     RenderState m_renderState;
     RefPtr<RenderPipelineImpl> m_renderPipeline;
 
+    bool m_preparedRenderStateValid = false;
+    RenderState m_preparedRenderState;
+    BindingDataImpl* m_preparedRenderBindingData = nullptr;
+
     bool m_computePassActive = false;
     bool m_computeStateValid = false;
     RefPtr<ComputePipelineImpl> m_computePipeline;
@@ -100,9 +103,11 @@ public:
     void cmdQueryAccelerationStructureProperties(const commands::QueryAccelerationStructureProperties& cmd);
     void cmdSerializeAccelerationStructure(const commands::SerializeAccelerationStructure& cmd);
     void cmdDeserializeAccelerationStructure(const commands::DeserializeAccelerationStructure& cmd);
+    void cmdExecuteClusterOperation(const commands::ExecuteClusterOperation& cmd);
     void cmdConvertCooperativeVectorMatrix(const commands::ConvertCooperativeVectorMatrix& cmd);
     void cmdSetBufferState(const commands::SetBufferState& cmd);
     void cmdSetTextureState(const commands::SetTextureState& cmd);
+    void cmdGlobalBarrier(const commands::GlobalBarrier& cmd);
     void cmdPushDebugGroup(const commands::PushDebugGroup& cmd);
     void cmdPopDebugGroup(const commands::PopDebugGroup& cmd);
     void cmdInsertDebugMarker(const commands::InsertDebugMarker& cmd);
@@ -113,6 +118,7 @@ public:
 
     void setBindings(BindingDataImpl* bindingData, VkPipelineBindPoint bindPoint);
 
+    void requireBindingStates(BindingDataImpl* bindingData);
     void requireBufferState(BufferImpl* buffer, ResourceState state);
     void requireTextureState(TextureImpl* texture, SubresourceRange subresourceRange, ResourceState state);
     void commitBarriers();
@@ -123,13 +129,6 @@ public:
         uint32_t queryCount,
         const AccelerationStructureQueryDesc* queryDescs
     );
-
-    void accelerationStructureBarrier(
-        uint32_t accelerationStructureCount,
-        IAccelerationStructure* const* accelerationStructures,
-        AccessFlag srcAccess,
-        AccessFlag destAccess
-    );
 };
 
 Result CommandRecorder::record(CommandBufferImpl* commandBuffer)
@@ -188,7 +187,7 @@ Result CommandRecorder::record(CommandBufferImpl* commandBuffer)
     return SLANG_OK;
 }
 
-#define NOT_SUPPORTED(x) m_device->warning(x " command is not supported!")
+#define NOT_SUPPORTED(x) m_device->printWarning(x " command is not supported!")
 
 void CommandRecorder::cmdCopyBuffer(const commands::CopyBuffer& cmd)
 {
@@ -267,12 +266,12 @@ void CommandRecorder::cmdCopyTexture(const commands::CopyTexture& cmd)
             VkImageLayout dstImageLayout = VK_IMAGE_LAYOUT_TRANSFER_DST_OPTIMAL;
 
             VkImageCopy region = {};
-            region.srcSubresource.aspectMask = VulkanUtil::getAspectMask(TextureAspect::All, src->m_vkformat);
+            region.srcSubresource.aspectMask = getAspectMaskFromFormat(src->m_vkformat);
             region.srcSubresource.baseArrayLayer = srcSubresource.layer + layer;
             region.srcSubresource.mipLevel = srcMip;
             region.srcSubresource.layerCount = 1;
             region.srcOffset = {(int32_t)srcOffset.x, (int32_t)srcOffset.y, (int32_t)srcOffset.z};
-            region.dstSubresource.aspectMask = VulkanUtil::getAspectMask(TextureAspect::All, dst->m_vkformat);
+            region.dstSubresource.aspectMask = getAspectMaskFromFormat(dst->m_vkformat);
             region.dstSubresource.baseArrayLayer = dstSubresource.layer + layer;
             region.dstSubresource.mipLevel = dstMip;
             region.dstSubresource.layerCount = 1;
@@ -340,7 +339,7 @@ void CommandRecorder::cmdCopyTextureToBuffer(const commands::CopyTextureToBuffer
     region.bufferOffset = dstOffset;
     region.bufferRowLength = rowLengthInTexels;
     region.bufferImageHeight = 0;
-    region.imageSubresource.aspectMask = VulkanUtil::getAspectMask(TextureAspect::All, src->m_vkformat);
+    region.imageSubresource.aspectMask = getAspectMaskFromFormat(src->m_vkformat);
     region.imageSubresource.mipLevel = srcMip;
     region.imageSubresource.baseArrayLayer = srcLayer;
     region.imageSubresource.layerCount = 1;
@@ -587,8 +586,8 @@ void CommandRecorder::cmdBeginRenderPass(const commands::BeginRenderPass& cmd)
         // Determine render area
         const TextureViewDesc& viewDesc = view->m_desc;
         const TextureDesc& textureDesc = view->m_texture->m_desc;
-        uint32_t width = calcMipSize(viewDesc.subresourceRange.mip, textureDesc.size.width);
-        uint32_t height = calcMipSize(viewDesc.subresourceRange.mip, textureDesc.size.height);
+        uint32_t width = calcMipSize(textureDesc.size.width, viewDesc.subresourceRange.mip);
+        uint32_t height = calcMipSize(textureDesc.size.height, viewDesc.subresourceRange.mip);
         renderArea.extent.width = min(renderArea.extent.width, width);
         renderArea.extent.height = min(renderArea.extent.height, height);
         uint32_t attachmentLayerCount = (textureDesc.type == TextureType::Texture3D)
@@ -598,12 +597,12 @@ void CommandRecorder::cmdBeginRenderPass(const commands::BeginRenderPass& cmd)
 
         // Create attachment info
         VkRenderingAttachmentInfoKHR attachmentInfo = {VK_STRUCTURE_TYPE_RENDERING_ATTACHMENT_INFO_KHR};
-        attachmentInfo.imageView = checked_cast<TextureViewImpl*>(attachment.view)->getView().imageView;
+        attachmentInfo.imageView = checked_cast<TextureViewImpl*>(attachment.view)->getRenderTargetView().imageView;
         attachmentInfo.imageLayout = VK_IMAGE_LAYOUT_COLOR_ATTACHMENT_OPTIMAL;
         if (attachment.resolveTarget)
         {
             attachmentInfo.resolveMode = VK_RESOLVE_MODE_AVERAGE_BIT;
-            attachmentInfo.resolveImageView = resolveView->getView().imageView;
+            attachmentInfo.resolveImageView = resolveView->getRenderTargetView().imageView;
             attachmentInfo.resolveImageLayout = VK_IMAGE_LAYOUT_COLOR_ATTACHMENT_OPTIMAL;
         }
         attachmentInfo.loadOp = translateLoadOp(attachment.loadOp);
@@ -633,27 +632,27 @@ void CommandRecorder::cmdBeginRenderPass(const commands::BeginRenderPass& cmd)
         // Determine render area
         const TextureViewDesc& viewDesc = view->m_desc;
         const TextureDesc& textureDesc = view->m_texture->m_desc;
-        uint32_t width = calcMipSize(viewDesc.subresourceRange.mip, textureDesc.size.width);
-        uint32_t height = calcMipSize(viewDesc.subresourceRange.mip, textureDesc.size.height);
+        uint32_t width = calcMipSize(textureDesc.size.width, viewDesc.subresourceRange.mip);
+        uint32_t height = calcMipSize(textureDesc.size.height, viewDesc.subresourceRange.mip);
         renderArea.extent.width = min(renderArea.extent.width, width);
         renderArea.extent.height = min(renderArea.extent.height, height);
 
         // Create attachment info
-        if (VulkanUtil::isDepthFormat(view->m_texture->m_vkformat))
+        if (isDepthFormat(view->m_texture->m_vkformat))
         {
             hasDepthAttachment = true;
             const auto& dsa = *desc.depthStencilAttachment;
-            depthAttachmentInfo.imageView = checked_cast<TextureViewImpl*>(dsa.view)->getView().imageView;
+            depthAttachmentInfo.imageView = checked_cast<TextureViewImpl*>(dsa.view)->getRenderTargetView().imageView;
             depthAttachmentInfo.imageLayout = VK_IMAGE_LAYOUT_DEPTH_STENCIL_ATTACHMENT_OPTIMAL;
             depthAttachmentInfo.loadOp = translateLoadOp(dsa.depthLoadOp);
             depthAttachmentInfo.storeOp = translateStoreOp(dsa.depthStoreOp);
             depthAttachmentInfo.clearValue.depthStencil.depth = dsa.depthClearValue;
         }
-        if (VulkanUtil::isStencilFormat(view->m_texture->m_vkformat))
+        if (isStencilFormat(view->m_texture->m_vkformat))
         {
             hasStencilAttachment = true;
             const auto& dsa = *desc.depthStencilAttachment;
-            stencilAttachmentInfo.imageView = checked_cast<TextureViewImpl*>(dsa.view)->getView().imageView;
+            stencilAttachmentInfo.imageView = checked_cast<TextureViewImpl*>(dsa.view)->getRenderTargetView().imageView;
             stencilAttachmentInfo.imageLayout = VK_IMAGE_LAYOUT_DEPTH_STENCIL_ATTACHMENT_OPTIMAL;
             stencilAttachmentInfo.loadOp = translateLoadOp(dsa.stencilLoadOp);
             stencilAttachmentInfo.storeOp = translateStoreOp(dsa.stencilStoreOp);
@@ -691,14 +690,22 @@ void CommandRecorder::prepareSetRenderState(const commands::SetRenderState& cmd)
 {
     const RenderState& state = cmd.state;
 
-    bool updateVertexBuffers = !arraysEqual(
-        state.vertexBufferCount,
-        m_renderState.vertexBufferCount,
-        state.vertexBuffers,
-        m_renderState.vertexBuffers
-    );
-    bool updateIndexBuffer =
-        state.indexFormat != m_renderState.indexFormat || state.indexBuffer != m_renderState.indexBuffer;
+    bool updateBindings = !m_preparedRenderStateValid || cmd.bindingData != m_preparedRenderBindingData;
+
+    bool updateVertexBuffers = !m_preparedRenderStateValid || !arraysEqual(
+                                                                  state.vertexBufferCount,
+                                                                  m_renderState.vertexBufferCount,
+                                                                  state.vertexBuffers,
+                                                                  m_renderState.vertexBuffers
+                                                              );
+    bool updateIndexBuffer = !m_preparedRenderStateValid || state.indexFormat != m_renderState.indexFormat ||
+                             state.indexBuffer != m_renderState.indexBuffer;
+
+    if (updateBindings)
+    {
+        m_preparedRenderBindingData = static_cast<BindingDataImpl*>(cmd.bindingData);
+        requireBindingStates(m_preparedRenderBindingData);
+    }
 
     if (updateVertexBuffers)
     {
@@ -717,6 +724,9 @@ void CommandRecorder::prepareSetRenderState(const commands::SetRenderState& cmd)
             requireBufferState(buffer, ResourceState::IndexBuffer);
         }
     }
+
+    m_preparedRenderStateValid = true;
+    m_preparedRenderState = state;
 }
 
 void CommandRecorder::cmdSetRenderState(const commands::SetRenderState& cmd)
@@ -1002,10 +1012,16 @@ void CommandRecorder::cmdSetComputeState(const commands::SetComputeState& cmd)
     if (updateBindings)
     {
         m_bindingData = static_cast<BindingDataImpl*>(cmd.bindingData);
+        requireBindingStates(m_bindingData);
+        commitBarriers();
         setBindings(m_bindingData, VK_PIPELINE_BIND_POINT_COMPUTE);
     }
 
     m_computeStateValid = true;
+
+    m_renderStateValid = false;
+    m_preparedRenderStateValid = false;
+    m_rayTracingStateValid = false;
 }
 
 void CommandRecorder::cmdDispatchCompute(const commands::DispatchCompute& cmd)
@@ -1058,6 +1074,8 @@ void CommandRecorder::cmdSetRayTracingState(const commands::SetRayTracingState&
     if (updateBindings)
     {
         m_bindingData = static_cast<BindingDataImpl*>(cmd.bindingData);
+        requireBindingStates(m_bindingData);
+        commitBarriers();
         setBindings(m_bindingData, VK_PIPELINE_BIND_POINT_RAY_TRACING_KHR);
     }
 
@@ -1068,13 +1086,13 @@ void CommandRecorder::cmdSetRayTracingState(const commands::SetRayTracingState&
         BufferImpl* shaderTableBuffer = m_shaderTable->getBuffer(m_rayTracingPipeline);
         DeviceAddress shaderTableAddr = shaderTableBuffer->getDeviceAddress();
 
-        auto rtProps = api.m_rtProperties;
+        auto rtpProps = api.m_rayTracingPipelineProperties;
         size_t alignedHandleSize =
-            math::calcAligned2(rtProps.shaderGroupHandleSize, rtProps.shaderGroupHandleAlignment);
+            math::calcAligned2(rtpProps.shaderGroupHandleSize, rtpProps.shaderGroupHandleAlignment);
 
         // Raygen index is set at dispatch time.
         m_rayGenTableAddr = shaderTableAddr;
-        m_raygenSBT.stride = math::calcAligned2(alignedHandleSize, rtProps.shaderGroupBaseAlignment);
+        m_raygenSBT.stride = math::calcAligned2(alignedHandleSize, rtpProps.shaderGroupBaseAlignment);
         m_raygenSBT.deviceAddress = shaderTableAddr;
         m_raygenSBT.size = m_raygenSBT.stride;
 
@@ -1092,6 +1110,10 @@ void CommandRecorder::cmdSetRayTracingState(const commands::SetRayTracingState&
     }
 
     m_rayTracingStateValid = true;
+
+    m_computeStateValid = false;
+    m_renderStateValid = false;
+    m_preparedRenderStateValid = false;
 }
 
 void CommandRecorder::cmdDispatchRays(const commands::DispatchRays& cmd)
@@ -1119,13 +1141,129 @@ void CommandRecorder::cmdBuildAccelerationStructure(const commands::BuildAcceler
     if (converter.convert(cmd.desc, m_device->m_debugCallback) != SLANG_OK)
         return;
 
-    converter.buildInfo.dstAccelerationStructure = checked_cast<AccelerationStructureImpl*>(cmd.dst)->m_vkHandle;
-    if (cmd.src)
+    AccelerationStructureImpl* dst = checked_cast<AccelerationStructureImpl*>(cmd.dst);
+    AccelerationStructureImpl* src = checked_cast<AccelerationStructureImpl*>(cmd.src);
+    BufferImpl* scratchBuffer = checked_cast<BufferImpl*>(cmd.scratchBuffer.buffer);
+
+    requireBufferState(dst->m_buffer, ResourceState::AccelerationStructureWrite);
+    converter.buildInfo.dstAccelerationStructure = dst->m_vkHandle;
+    if (src)
     {
-        converter.buildInfo.srcAccelerationStructure = checked_cast<AccelerationStructureImpl*>(cmd.src)->m_vkHandle;
+        requireBufferState(src->m_buffer, ResourceState::AccelerationStructureRead);
+        converter.buildInfo.srcAccelerationStructure = src->m_vkHandle;
     }
+    requireBufferState(scratchBuffer, ResourceState::UnorderedAccess);
     converter.buildInfo.scratchData.deviceAddress = cmd.scratchBuffer.getDeviceAddress();
 
+    for (uint32_t inputIndex = 0; inputIndex < cmd.desc.inputCount; ++inputIndex)
+    {
+        const AccelerationStructureBuildInput& input = cmd.desc.inputs[inputIndex];
+        switch (input.type)
+        {
+        case AccelerationStructureBuildInputType::Instances:
+            if (input.instances.instanceBuffer.buffer)
+            {
+                requireBufferState(
+                    checked_cast<BufferImpl*>(input.instances.instanceBuffer.buffer),
+                    ResourceState::AccelerationStructureBuildInput
+                );
+            }
+            break;
+        case AccelerationStructureBuildInputType::Triangles:
+            for (uint32_t i = 0; i < input.triangles.vertexBufferCount; ++i)
+            {
+                if (input.triangles.vertexBuffers[i].buffer)
+                {
+                    requireBufferState(
+                        checked_cast<BufferImpl*>(input.triangles.vertexBuffers[i].buffer),
+                        ResourceState::AccelerationStructureBuildInput
+                    );
+                }
+            }
+            if (input.triangles.indexBuffer.buffer)
+            {
+                requireBufferState(
+                    checked_cast<BufferImpl*>(input.triangles.indexBuffer.buffer),
+                    ResourceState::AccelerationStructureBuildInput
+                );
+            }
+            if (input.triangles.preTransformBuffer.buffer)
+            {
+                requireBufferState(
+                    checked_cast<BufferImpl*>(input.triangles.preTransformBuffer.buffer),
+                    ResourceState::AccelerationStructureBuildInput
+                );
+            }
+            break;
+        case AccelerationStructureBuildInputType::ProceduralPrimitives:
+            for (uint32_t i = 0; i < input.proceduralPrimitives.aabbBufferCount; ++i)
+            {
+                if (input.proceduralPrimitives.aabbBuffers[i].buffer)
+                {
+                    requireBufferState(
+                        checked_cast<BufferImpl*>(input.proceduralPrimitives.aabbBuffers[i].buffer),
+                        ResourceState::AccelerationStructureBuildInput
+                    );
+                }
+            }
+            break;
+        case AccelerationStructureBuildInputType::Spheres:
+            for (uint32_t i = 0; i < input.spheres.vertexBufferCount; ++i)
+            {
+                if (input.spheres.vertexPositionBuffers[i].buffer)
+                {
+                    requireBufferState(
+                        checked_cast<BufferImpl*>(input.spheres.vertexPositionBuffers[i].buffer),
+                        ResourceState::AccelerationStructureBuildInput
+                    );
+                }
+                if (input.spheres.vertexRadiusBuffers[i].buffer)
+                {
+                    requireBufferState(
+                        checked_cast<BufferImpl*>(input.spheres.vertexRadiusBuffers[i].buffer),
+                        ResourceState::AccelerationStructureBuildInput
+                    );
+                }
+            }
+            if (input.spheres.indexBuffer.buffer)
+            {
+                requireBufferState(
+                    checked_cast<BufferImpl*>(input.spheres.indexBuffer.buffer),
+                    ResourceState::AccelerationStructureBuildInput
+                );
+            }
+            break;
+        case AccelerationStructureBuildInputType::LinearSweptSpheres:
+            for (uint32_t i = 0; i < input.linearSweptSpheres.vertexBufferCount; ++i)
+            {
+                if (input.linearSweptSpheres.vertexPositionBuffers[i].buffer)
+                {
+                    requireBufferState(
+                        checked_cast<BufferImpl*>(input.linearSweptSpheres.vertexPositionBuffers[i].buffer),
+                        ResourceState::AccelerationStructureBuildInput
+                    );
+                }
+                if (input.linearSweptSpheres.vertexRadiusBuffers[i].buffer)
+                {
+                    requireBufferState(
+                        checked_cast<BufferImpl*>(input.linearSweptSpheres.vertexRadiusBuffers[i].buffer),
+                        ResourceState::AccelerationStructureBuildInput
+                    );
+                }
+            }
+            if (input.linearSweptSpheres.indexBuffer.buffer)
+            {
+                requireBufferState(
+                    checked_cast<BufferImpl*>(input.linearSweptSpheres.indexBuffer.buffer),
+                    ResourceState::AccelerationStructureBuildInput
+                );
+            }
+            break;
+        }
+    }
+
+    commitBarriers();
+
     std::vector<VkAccelerationStructureBuildRangeInfoKHR> rangeInfos;
     rangeInfos.resize(converter.primitiveCounts.size());
     for (size_t i = 0; i < converter.primitiveCounts.size(); i++)
@@ -1142,16 +1280,22 @@ void CommandRecorder::cmdBuildAccelerationStructure(const commands::BuildAcceler
 
     if (cmd.propertyQueryCount)
     {
-        accelerationStructureBarrier(1, &cmd.dst, AccessFlag::Write, AccessFlag::Read);
         queryAccelerationStructureProperties(1, &cmd.dst, cmd.propertyQueryCount, cmd.queryDescs);
     }
 }
 
 void CommandRecorder::cmdCopyAccelerationStructure(const commands::CopyAccelerationStructure& cmd)
 {
+    AccelerationStructureImpl* dst = checked_cast<AccelerationStructureImpl*>(cmd.dst);
+    AccelerationStructureImpl* src = checked_cast<AccelerationStructureImpl*>(cmd.src);
+
+    requireBufferState(dst->m_buffer, ResourceState::AccelerationStructureWrite);
+    requireBufferState(src->m_buffer, ResourceState::AccelerationStructureRead);
+    commitBarriers();
+
     VkCopyAccelerationStructureInfoKHR copyInfo = {VK_STRUCTURE_TYPE_COPY_ACCELERATION_STRUCTURE_INFO_KHR};
-    copyInfo.src = checked_cast<AccelerationStructureImpl*>(cmd.src)->m_vkHandle;
-    copyInfo.dst = checked_cast<AccelerationStructureImpl*>(cmd.dst)->m_vkHandle;
+    copyInfo.dst = dst->m_vkHandle;
+    copyInfo.src = src->m_vkHandle;
     switch (cmd.mode)
     {
     case AccelerationStructureCopyMode::Clone:
@@ -1176,10 +1320,17 @@ void CommandRecorder::cmdQueryAccelerationStructureProperties(const commands::Qu
 
 void CommandRecorder::cmdSerializeAccelerationStructure(const commands::SerializeAccelerationStructure& cmd)
 {
+    BufferImpl* dstBuffer = checked_cast<BufferImpl*>(cmd.dst.buffer);
+    AccelerationStructureImpl* src = checked_cast<AccelerationStructureImpl*>(cmd.src);
+
+    requireBufferState(dstBuffer, ResourceState::UnorderedAccess);
+    requireBufferState(src->m_buffer, ResourceState::AccelerationStructureRead);
+    commitBarriers();
+
     VkCopyAccelerationStructureToMemoryInfoKHR copyInfo = {
         VK_STRUCTURE_TYPE_COPY_ACCELERATION_STRUCTURE_TO_MEMORY_INFO_KHR
     };
-    copyInfo.src = checked_cast<AccelerationStructureImpl*>(cmd.src)->m_vkHandle;
+    copyInfo.src = src->m_vkHandle;
     copyInfo.dst.deviceAddress = cmd.dst.getDeviceAddress();
     copyInfo.mode = VK_COPY_ACCELERATION_STRUCTURE_MODE_SERIALIZE_KHR;
     m_api.vkCmdCopyAccelerationStructureToMemoryKHR(m_cmdBuffer, &copyInfo);
@@ -1187,23 +1338,119 @@ void CommandRecorder::cmdSerializeAccelerationStructure(const commands::Serializ
 
 void CommandRecorder::cmdDeserializeAccelerationStructure(const commands::DeserializeAccelerationStructure& cmd)
 {
+    AccelerationStructureImpl* dst = checked_cast<AccelerationStructureImpl*>(cmd.dst);
+    BufferImpl* srcBuffer = checked_cast<BufferImpl*>(cmd.src.buffer);
+
+    requireBufferState(dst->m_buffer, ResourceState::AccelerationStructureWrite);
+    requireBufferState(srcBuffer, ResourceState::ShaderResource);
+    commitBarriers();
+
     VkCopyMemoryToAccelerationStructureInfoKHR copyInfo = {
         VK_STRUCTURE_TYPE_COPY_MEMORY_TO_ACCELERATION_STRUCTURE_INFO_KHR
     };
     copyInfo.src.deviceAddress = cmd.src.getDeviceAddress();
-    copyInfo.dst = checked_cast<AccelerationStructureImpl*>(cmd.dst)->m_vkHandle;
+    copyInfo.dst = dst->m_vkHandle;
     copyInfo.mode = VK_COPY_ACCELERATION_STRUCTURE_MODE_DESERIALIZE_KHR;
     m_api.vkCmdCopyMemoryToAccelerationStructureKHR(m_cmdBuffer, &copyInfo);
 }
 
+void CommandRecorder::cmdExecuteClusterOperation(const commands::ExecuteClusterOperation& cmd)
+{
+    if (!m_api.vkCmdBuildClusterAccelerationStructureIndirectNV)
+        return;
+
+    const ClusterOperationDesc& desc = cmd.desc;
+
+    if (desc.params.maxArgCount == 0)
+        return;
+
+    BufferImpl* argCountBuffer = checked_cast<BufferImpl*>(desc.argCountBuffer.buffer);
+    BufferImpl* argsBuffer = checked_cast<BufferImpl*>(desc.argsBuffer.buffer);
+    BufferImpl* scratchBuffer = checked_cast<BufferImpl*>(desc.scratchBuffer.buffer);
+    BufferImpl* addressesBuffer = checked_cast<BufferImpl*>(desc.addressesBuffer.buffer);
+    BufferImpl* resultBuffer = checked_cast<BufferImpl*>(desc.resultBuffer.buffer);
+    BufferImpl* sizesBuffer = checked_cast<BufferImpl*>(desc.sizesBuffer.buffer);
+
+    requireBufferState(argsBuffer, ResourceState::ShaderResource);
+    if (argCountBuffer)
+        requireBufferState(argCountBuffer, ResourceState::ShaderResource);
+    requireBufferState(scratchBuffer, ResourceState::UnorderedAccess);
+    if (addressesBuffer)
+        requireBufferState(addressesBuffer, ResourceState::UnorderedAccess);
+    if (resultBuffer)
+        requireBufferState(resultBuffer, ResourceState::AccelerationStructureWrite);
+    if (sizesBuffer)
+        requireBufferState(sizesBuffer, ResourceState::UnorderedAccess);
+    commitBarriers();
+
+    VkClusterAccelerationStructureClustersBottomLevelInputNV bottomLevelInput;
+    VkClusterAccelerationStructureTriangleClusterInputNV triangleClusterInput;
+    VkClusterAccelerationStructureMoveObjectsInputNV moveObjectsInput;
+
+    VkClusterAccelerationStructureCommandsInfoNV commandsInfo = {
+        VK_STRUCTURE_TYPE_CLUSTER_ACCELERATION_STRUCTURE_COMMANDS_INFO_NV
+    };
+    commandsInfo.input =
+        translateClusterOperationParams(desc.params, bottomLevelInput, triangleClusterInput, moveObjectsInput);
+    commandsInfo.scratchData = scratchBuffer->getDeviceAddress();
+    commandsInfo.dstImplicitData = desc.resultBuffer ? desc.resultBuffer.getDeviceAddress() : 0;
+    if (desc.addressesBuffer)
+    {
+        commandsInfo.dstAddressesArray.deviceAddress = desc.addressesBuffer.getDeviceAddress();
+        commandsInfo.dstAddressesArray.size = desc.addressesBuffer.getSize();
+        commandsInfo.dstAddressesArray.stride = desc.addressesBufferStride;
+    }
+    if (desc.sizesBuffer)
+    {
+        commandsInfo.dstSizesArray.deviceAddress = desc.sizesBuffer.getDeviceAddress();
+        commandsInfo.dstSizesArray.size = desc.sizesBuffer.getSize();
+        commandsInfo.dstSizesArray.stride = desc.sizesBufferStride;
+    }
+    if (desc.argsBuffer)
+    {
+        commandsInfo.srcInfosArray.deviceAddress = desc.argsBuffer.getDeviceAddress();
+        commandsInfo.srcInfosArray.size = desc.argsBuffer.getSize();
+        commandsInfo.srcInfosArray.stride = desc.argsBufferStride;
+    }
+    commandsInfo.srcInfosCount = desc.argCountBuffer ? desc.argCountBuffer.getDeviceAddress() : 0;
+    commandsInfo.addressResolutionFlags = VkClusterAccelerationStructureAddressResolutionFlagBitsNV(0);
+
+    m_api.vkCmdBuildClusterAccelerationStructureIndirectNV(m_cmdBuffer, &commandsInfo);
+}
+
 void CommandRecorder::cmdConvertCooperativeVectorMatrix(const commands::ConvertCooperativeVectorMatrix& cmd)
 {
+    BufferImpl* dstBuffer = checked_cast<BufferImpl*>(cmd.dstBuffer);
+    BufferImpl* srcBuffer = checked_cast<BufferImpl*>(cmd.srcBuffer);
+
+    requireBufferState(dstBuffer, ResourceState::UnorderedAccess);
+    requireBufferState(srcBuffer, ResourceState::ShaderResource);
+    commitBarriers();
+
     short_vector<VkConvertCooperativeVectorMatrixInfoNV> infos;
-    for (uint32_t i = 0; i < cmd.descCount; ++i)
+    for (uint32_t i = 0; i < cmd.matrixCount; ++i)
     {
-        infos.push_back(VulkanUtil::translateConvertCooperativeVectorMatrixDesc(cmd.descs[i]));
+        const CooperativeVectorMatrixDesc& dstDesc = cmd.dstDescs[i];
+        const CooperativeVectorMatrixDesc& srcDesc = cmd.srcDescs[i];
+        VkConvertCooperativeVectorMatrixInfoNV info = {VK_STRUCTURE_TYPE_CONVERT_COOPERATIVE_VECTOR_MATRIX_INFO_NV};
+        info.srcSize = srcDesc.size;
+        info.srcData.deviceAddress = srcBuffer->getDeviceAddress() + srcDesc.offset;
+        info.pDstSize = (size_t*)&dstDesc.size;
+        info.dstData.deviceAddress = dstBuffer->getDeviceAddress() + dstDesc.offset;
+        info.srcComponentType = translateCooperativeVectorComponentType(srcDesc.componentType);
+        info.dstComponentType = translateCooperativeVectorComponentType(dstDesc.componentType);
+        info.numRows = srcDesc.rowCount;
+        info.numColumns = srcDesc.colCount;
+        info.srcLayout = translateCooperativeVectorMatrixLayout(srcDesc.layout);
+        info.srcStride = srcDesc.rowColumnStride;
+        info.dstLayout = translateCooperativeVectorMatrixLayout(dstDesc.layout);
+        info.dstStride = dstDesc.rowColumnStride;
+        infos.push_back(info);
     }
     m_api.vkCmdConvertCooperativeVectorMatrixNV(m_cmdBuffer, infos.size(), infos.data());
+
+    requireBufferState(dstBuffer, ResourceState::ShaderResource);
+    commitBarriers();
 }
 
 void CommandRecorder::cmdSetBufferState(const commands::SetBufferState& cmd)
@@ -1216,6 +1463,31 @@ void CommandRecorder::cmdSetTextureState(const commands::SetTextureState& cmd)
     m_stateTracking.setTextureState(checked_cast<TextureImpl*>(cmd.texture), cmd.subresourceRange, cmd.state);
 }
 
+void CommandRecorder::cmdGlobalBarrier(const commands::GlobalBarrier& cmd)
+{
+    // On vulkan the global barrier is a memory barrier that:
+    // - captures all stages both before and after the barrier
+    // - ensures that all reads after the barrier see all writes before the barrier
+
+    VkMemoryBarrier memoryBarrier = {};
+    memoryBarrier.sType = VK_STRUCTURE_TYPE_MEMORY_BARRIER;
+    memoryBarrier.srcAccessMask = VK_ACCESS_MEMORY_WRITE_BIT;
+    memoryBarrier.dstAccessMask = VK_ACCESS_MEMORY_READ_BIT;
+
+    m_api.vkCmdPipelineBarrier(
+        m_cmdBuffer,
+        VK_PIPELINE_STAGE_ALL_COMMANDS_BIT,
+        VK_PIPELINE_STAGE_ALL_COMMANDS_BIT,
+        VkDependencyFlags(0),
+        1,
+        &memoryBarrier,
+        0,
+        nullptr,
+        0,
+        nullptr
+    );
+}
+
 void CommandRecorder::cmdPushDebugGroup(const commands::PushDebugGroup& cmd)
 {
     if (!m_api.vkCmdBeginDebugUtilsLabelEXT)
@@ -1223,9 +1495,9 @@ void CommandRecorder::cmdPushDebugGroup(const commands::PushDebugGroup& cmd)
 
     VkDebugUtilsLabelEXT label = {VK_STRUCTURE_TYPE_DEBUG_UTILS_LABEL_EXT};
     label.pLabelName = cmd.name;
-    label.color[0] = cmd.rgbColor[0];
-    label.color[1] = cmd.rgbColor[1];
-    label.color[2] = cmd.rgbColor[2];
+    label.color[0] = cmd.color.r;
+    label.color[1] = cmd.color.g;
+    label.color[2] = cmd.color.b;
     label.color[3] = 1.0f;
     m_api.vkCmdBeginDebugUtilsLabelEXT(m_cmdBuffer, &label);
 }
@@ -1246,9 +1518,9 @@ void CommandRecorder::cmdInsertDebugMarker(const commands::InsertDebugMarker& cm
 
     VkDebugUtilsLabelEXT label = {VK_STRUCTURE_TYPE_DEBUG_UTILS_LABEL_EXT};
     label.pLabelName = cmd.name;
-    label.color[0] = cmd.rgbColor[0];
-    label.color[1] = cmd.rgbColor[1];
-    label.color[2] = cmd.rgbColor[2];
+    label.color[0] = cmd.color.r;
+    label.color[1] = cmd.color.g;
+    label.color[2] = cmd.color.b;
     label.color[3] = 1.0f;
     m_api.vkCmdInsertDebugUtilsLabelEXT(m_cmdBuffer, &label);
 }
@@ -1268,24 +1540,7 @@ void CommandRecorder::cmdExecuteCallback(const commands::ExecuteCallback& cmd)
 
 void CommandRecorder::setBindings(BindingDataImpl* bindingData, VkPipelineBindPoint bindPoint)
 {
-    // First, we transition all resources to the required states.
-    for (uint32_t i = 0; i < bindingData->bufferStateCount; ++i)
-    {
-        const auto& bufferState = bindingData->bufferStates[i];
-        requireBufferState(bufferState.buffer, bufferState.state);
-    }
-    for (uint32_t i = 0; i < bindingData->textureStateCount; ++i)
-    {
-        const auto& textureState = bindingData->textureStates[i];
-        requireTextureState(
-            textureState.textureView->m_texture,
-            textureState.textureView->m_desc.subresourceRange,
-            textureState.state
-        );
-    }
-    commitBarriers();
-
-    // Then we set all push constants.
+    // Set push constants.
     for (uint32_t i = 0; i < bindingData->pushConstantCount; ++i)
     {
         VkPushConstantRange range = bindingData->pushConstantRanges[i];
@@ -1300,7 +1555,7 @@ void CommandRecorder::setBindings(BindingDataImpl* bindingData, VkPipelineBindPo
         );
     }
 
-    // Finally, we bind all descriptor sets.
+    // Bind descriptor sets.
     if (bindingData->descriptorSetCount)
     {
         m_api.vkCmdBindDescriptorSets(
@@ -1316,6 +1571,24 @@ void CommandRecorder::setBindings(BindingDataImpl* bindingData, VkPipelineBindPo
     }
 }
 
+void CommandRecorder::requireBindingStates(BindingDataImpl* bindingData)
+{
+    for (uint32_t i = 0; i < bindingData->bufferStateCount; ++i)
+    {
+        const auto& bufferState = bindingData->bufferStates[i];
+        requireBufferState(bufferState.buffer, bufferState.state);
+    }
+    for (uint32_t i = 0; i < bindingData->textureStateCount; ++i)
+    {
+        const auto& textureState = bindingData->textureStates[i];
+        requireTextureState(
+            textureState.textureView->m_texture,
+            textureState.textureView->m_desc.subresourceRange,
+            textureState.state
+        );
+    }
+}
+
 void CommandRecorder::requireBufferState(BufferImpl* buffer, ResourceState state)
 {
     m_stateTracking.setBufferState(buffer, state);
@@ -1328,6 +1601,9 @@ void CommandRecorder::requireTextureState(TextureImpl* texture, SubresourceRange
 
 void CommandRecorder::commitBarriers()
 {
+    if (testing::gDebugDisableStateTracking)
+        return;
+
     short_vector<VkBufferMemoryBarrier, 16> bufferBarriers;
     short_vector<VkImageMemoryBarrier, 16> imageBarriers;
 
@@ -1432,7 +1708,7 @@ void CommandRecorder::commitBarriers()
             texture->m_isSwapchainInitialState = false;
         }
         barrier.newLayout = translateImageLayout(textureBarrier.stateAfter);
-        barrier.subresourceRange.aspectMask = getAspectMaskFromFormat(VulkanUtil::getVkFormat(texture->m_desc.format));
+        barrier.subresourceRange.aspectMask = getAspectMaskFromFormat(getVkFormat(texture->m_desc.format));
         barrier.subresourceRange.baseArrayLayer = textureBarrier.entireTexture ? 0 : textureBarrier.layer;
         barrier.subresourceRange.baseMipLevel = textureBarrier.entireTexture ? 0 : textureBarrier.mip;
         barrier.subresourceRange.layerCount = textureBarrier.entireTexture ? VK_REMAINING_ARRAY_LAYERS : 1;
@@ -1458,10 +1734,16 @@ void CommandRecorder::queryAccelerationStructureProperties(
 {
     short_vector<VkAccelerationStructureKHR> vkHandles;
     vkHandles.resize(accelerationStructureCount);
-    for (uint32_t i = 0; i < accelerationStructureCount; i++)
+
+    for (uint32_t i = 0; i < accelerationStructureCount; ++i)
     {
+        AccelerationStructureImpl* asImpl = checked_cast<AccelerationStructureImpl*>(accelerationStructures[i]);
+        requireBufferState(asImpl->m_buffer, ResourceState::AccelerationStructureRead);
         vkHandles[i] = checked_cast<AccelerationStructureImpl*>(accelerationStructures[i])->m_vkHandle;
     }
+
+    commitBarriers();
+
     for (uint32_t i = 0; i < queryCount; i++)
     {
         VkQueryType queryType;
@@ -1496,47 +1778,6 @@ void CommandRecorder::queryAccelerationStructureProperties(
     }
 }
 
-void CommandRecorder::accelerationStructureBarrier(
-    uint32_t accelerationStructureCount,
-    IAccelerationStructure* const* accelerationStructures,
-    AccessFlag srcAccess,
-    AccessFlag destAccess
-)
-{
-    short_vector<VkBufferMemoryBarrier> memBarriers;
-    memBarriers.resize(accelerationStructureCount);
-    for (uint32_t i = 0; i < accelerationStructureCount; i++)
-    {
-        memBarriers[i].sType = VK_STRUCTURE_TYPE_BUFFER_MEMORY_BARRIER;
-        memBarriers[i].pNext = nullptr;
-        memBarriers[i].dstAccessMask = translateAccelerationStructureAccessFlag(destAccess);
-        memBarriers[i].srcAccessMask = translateAccelerationStructureAccessFlag(srcAccess);
-        memBarriers[i].srcQueueFamilyIndex = m_device->m_queueFamilyIndex;
-        memBarriers[i].dstQueueFamilyIndex = m_device->m_queueFamilyIndex;
-
-        auto asImpl = checked_cast<AccelerationStructureImpl*>(accelerationStructures[i]);
-        memBarriers[i].buffer = asImpl->m_buffer->m_buffer.m_buffer;
-        memBarriers[i].offset = 0;
-        memBarriers[i].size = asImpl->m_buffer->m_desc.size;
-    }
-    m_device->m_api.vkCmdPipelineBarrier(
-        m_cmdBuffer,
-        VK_PIPELINE_STAGE_ACCELERATION_STRUCTURE_BUILD_BIT_KHR | VK_PIPELINE_STAGE_BOTTOM_OF_PIPE_BIT,
-        VK_PIPELINE_STAGE_ACCELERATION_STRUCTURE_BUILD_BIT_KHR | VK_PIPELINE_STAGE_TOP_OF_PIPE_BIT |
-            VK_PIPELINE_STAGE_TRANSFER_BIT | VK_PIPELINE_STAGE_VERTEX_SHADER_BIT |
-            VK_PIPELINE_STAGE_FRAGMENT_SHADER_BIT | VK_PIPELINE_STAGE_COMPUTE_SHADER_BIT |
-            VK_PIPELINE_STAGE_RAY_TRACING_SHADER_BIT_KHR,
-        0,
-        0,
-        nullptr,
-        (uint32_t)memBarriers.size(),
-        memBarriers.data(),
-        0,
-        nullptr
-    );
-}
-
-
 // CommandQueueImpl
 
 CommandQueueImpl::CommandQueueImpl(Device* device, QueueType type)
@@ -1585,16 +1826,20 @@ Result CommandQueueImpl::getOrCreateCommandBuffer(CommandBufferImpl** outCommand
     {
         commandBuffer = m_commandBuffersPool.front();
         m_commandBuffersPool.pop_front();
+        commandBuffer->setInternalReferenceCount(0);
     }
     returnRefPtr(outCommandBuffer, commandBuffer);
     return SLANG_OK;
 }
 
-void CommandQueueImpl::retireUnfinishedCommandBuffer(CommandBufferImpl* commandBuffer)
+void CommandQueueImpl::retireCommandBuffer(CommandBufferImpl* commandBuffer)
 {
-    std::lock_guard<std::mutex> lock(m_mutex);
     commandBuffer->reset();
-    m_commandBuffersPool.push_back(commandBuffer);
+    {
+        std::lock_guard<std::mutex> lock(m_mutex);
+        m_commandBuffersPool.push_back(commandBuffer);
+        commandBuffer->setInternalReferenceCount(1);
+    }
 }
 
 void CommandQueueImpl::retireCommandBuffers()
@@ -1607,17 +1852,16 @@ void CommandQueueImpl::retireCommandBuffers()
     {
         if (commandBuffer->m_submissionID <= lastFinishedID)
         {
-            commandBuffer->reset();
-            {
-                std::lock_guard<std::mutex> lock(m_mutex);
-                m_commandBuffersPool.push_back(commandBuffer);
-            }
+            retireCommandBuffer(commandBuffer);
         }
         else
         {
             m_commandBuffersInFlight.push_back(commandBuffer);
         }
     }
+
+    // Flush all device heaps
+    getDevice<DeviceImpl>()->flushHeaps();
 }
 
 uint64_t CommandQueueImpl::updateLastFinishedID()
@@ -1629,7 +1873,7 @@ uint64_t CommandQueueImpl::updateLastFinishedID()
 Result CommandQueueImpl::createCommandEncoder(ICommandEncoder** outEncoder)
 {
     RefPtr<CommandEncoderImpl> encoder = new CommandEncoderImpl(m_device, this);
-    encoder->init();
+    SLANG_RETURN_ON_FAIL(encoder->init());
     returnComPtr(outEncoder, encoder);
     return SLANG_OK;
 }
@@ -1760,7 +2004,7 @@ CommandEncoderImpl::~CommandEncoderImpl()
     // If the command buffer was not used, return it to the pool.
     if (m_commandBuffer)
     {
-        m_queue->retireUnfinishedCommandBuffer(m_commandBuffer);
+        m_queue->retireCommandBuffer(m_commandBuffer);
     }
 }
 
@@ -1830,7 +2074,7 @@ Result CommandBufferImpl::init()
     m_descriptorSetAllocator.init(&device->m_api);
 
     VkCommandPoolCreateInfo createInfo = {VK_STRUCTURE_TYPE_COMMAND_POOL_CREATE_INFO};
-    createInfo.flags = VK_COMMAND_POOL_CREATE_TRANSIENT_BIT | VK_COMMAND_POOL_CREATE_RESET_COMMAND_BUFFER_BIT;
+    createInfo.flags = VK_COMMAND_POOL_CREATE_TRANSIENT_BIT;
     createInfo.queueFamilyIndex = m_queue->m_queueFamilyIndex;
     SLANG_VK_RETURN_ON_FAIL(
         device->m_api.vkCreateCommandPool(device->m_api.m_device, &createInfo, nullptr, &m_commandPool)
@@ -1840,7 +2084,8 @@ Result CommandBufferImpl::init()
     allocInfo.commandPool = m_commandPool;
     allocInfo.level = VK_COMMAND_BUFFER_LEVEL_PRIMARY;
     allocInfo.commandBufferCount = 1;
-    SLANG_VK_RETURN_ON_FAIL(device->m_api.vkAllocateCommandBuffers(device->m_api.m_device, &allocInfo, &m_commandBuffer)
+    SLANG_VK_RETURN_ON_FAIL(
+        device->m_api.vkAllocateCommandBuffers(device->m_api.m_device, &allocInfo, &m_commandBuffer)
     );
 
     return SLANG_OK;
@@ -1850,7 +2095,7 @@ Result CommandBufferImpl::reset()
 {
     DeviceImpl* device = getDevice<DeviceImpl>();
     m_commandList.reset();
-    SLANG_VK_RETURN_ON_FAIL(device->m_api.vkResetCommandBuffer(m_commandBuffer, 0));
+    SLANG_VK_RETURN_ON_FAIL(device->m_api.vkResetCommandPool(device->m_device, m_commandPool, 0));
     m_constantBufferPool.reset();
     m_descriptorSetAllocator.reset();
     m_bindingCache.reset();
