@@ -11,9 +11,11 @@ public:
     ~SamplerImpl();
 
     virtual SLANG_NO_THROW Result SLANG_MCALL getNativeHandle(NativeHandle* outHandle) override;
+    virtual SLANG_NO_THROW Result SLANG_MCALL getDescriptorHandle(DescriptorHandle* outHandle) override;
 
 public:
     VkSampler m_sampler;
+    DescriptorHandle m_descriptorHandle;
 };
 
 } // namespace rhi::vk
