@@ -1,6 +1,7 @@
 #include "vk-shader-object-layout.h"
 #include "vk-device.h"
-#include "vk-util.h"
+#include "vk-bindless-descriptor-set.h"
+#include "vk-utils.h"
 
 namespace rhi::vk {
 
@@ -143,11 +144,14 @@ void ShaderObjectLayoutImpl::Builder::_addDescriptorRangesAsValue(
             auto vkDescriptorType = _mapDescriptorType(slangDescriptorType);
             VkDescriptorSetLayoutBinding vkBindingRangeDesc = {};
             vkBindingRangeDesc.binding =
-                offset.binding +
-                (uint32_t
-                )typeLayout->getDescriptorSetDescriptorRangeIndexOffset(slangDescriptorSetIndex, descriptorRangeIndex);
-            vkBindingRangeDesc.descriptorCount = (uint32_t
-            )typeLayout->getDescriptorSetDescriptorRangeDescriptorCount(slangDescriptorSetIndex, descriptorRangeIndex);
+                offset.binding + (uint32_t)typeLayout->getDescriptorSetDescriptorRangeIndexOffset(
+                                     slangDescriptorSetIndex,
+                                     descriptorRangeIndex
+                                 );
+            vkBindingRangeDesc.descriptorCount = (uint32_t)typeLayout->getDescriptorSetDescriptorRangeDescriptorCount(
+                slangDescriptorSetIndex,
+                descriptorRangeIndex
+            );
             vkBindingRangeDesc.descriptorType = vkDescriptorType;
             vkBindingRangeDesc.stageFlags = VK_SHADER_STAGE_ALL;
 
@@ -181,16 +185,7 @@ void ShaderObjectLayoutImpl::Builder::_addDescriptorRangesAsValue(
             break;
 
         case slang::BindingType::ExistentialValue:
-            // An interest/existential-typed sub-object range will only contribute
-            // descriptor ranges to a parent object in the case where it has been
-            // specialied, which is precisely the case where the Slang reflection
-            // information will tell us about its "pending" layout.
-            //
-            if (auto pendingTypeLayout = subObjectTypeLayout->getPendingDataTypeLayout())
-            {
-                BindingOffset pendingOffset = BindingOffset(subObjectRangeOffset.pending);
-                _addDescriptorRangesAsValue(pendingTypeLayout, pendingOffset);
-            }
+            // Interface-type ranges are no longer supported after pending data removal.
             break;
 
         case slang::BindingType::ConstantBuffer:
@@ -419,7 +414,6 @@ void ShaderObjectLayoutImpl::Builder::addBindingRanges(slang::TypeLayoutReflecti
     for (SlangInt r = 0; r < subObjectRangeCount; ++r)
     {
         SlangInt bindingRangeIndex = typeLayout->getSubObjectRangeBindingRangeIndex(r);
-        auto& bindingRange = m_bindingRanges[bindingRangeIndex];
         auto slangBindingType = typeLayout->getBindingRangeType(bindingRangeIndex);
         slang::TypeLayoutReflection* slangLeafTypeLayout = typeLayout->getBindingRangeLeafTypeLayout(bindingRangeIndex);
 
@@ -447,15 +441,6 @@ void ShaderObjectLayoutImpl::Builder::addBindingRanges(slang::TypeLayoutReflecti
         break;
 
         case slang::BindingType::ExistentialValue:
-            if (auto pendingTypeLayout = slangLeafTypeLayout->getPendingDataTypeLayout())
-            {
-                ShaderObjectLayoutImpl::createForElementType(
-                    m_device,
-                    m_session,
-                    pendingTypeLayout,
-                    subObjectLayout.writeRef()
-                );
-            }
             break;
         }
 
@@ -470,8 +455,6 @@ void ShaderObjectLayoutImpl::Builder::addBindingRanges(slang::TypeLayoutReflecti
         //
         subObjectRange.offset = SubObjectRangeOffset(typeLayout->getSubObjectRangeOffset(r));
         subObjectRange.stride = SubObjectRangeStride(slangLeafTypeLayout);
-        subObjectRange.pendingOrdinaryDataOffset = subObjectRange.offset.pendingOrdinaryData;
-        subObjectRange.pendingOrdinaryDataStride = subObjectRange.stride.pendingOrdinaryData;
 
         switch (slangBindingType)
         {
@@ -493,17 +476,7 @@ void ShaderObjectLayoutImpl::Builder::addBindingRanges(slang::TypeLayoutReflecti
                 m_totalBindingCount += subObjectLayout->getTotalBindingCount();
                 m_childPushConstantRangeCount += subObjectLayout->getTotalPushConstantRangeCount();
 
-                // An interface-type range that includes ordinary data can
-                // increase the size of the ordinary data buffer we need to
-                // allocate for the parent object.
-                //
-                uint32_t ordinaryDataEnd = subObjectRange.offset.pendingOrdinaryData +
-                                           (uint32_t)bindingRange.count * subObjectRange.stride.pendingOrdinaryData;
-
-                if (ordinaryDataEnd > m_totalOrdinaryDataSize)
-                {
-                    m_totalOrdinaryDataSize = ordinaryDataEnd;
-                }
+                // Interface-type ranges are no longer supported after pending data removal.
             }
             break;
 
@@ -584,16 +557,6 @@ Result ShaderObjectLayoutImpl::createForElementType(
     BindingOffset elementOffset;
     elementOffset.binding = ordinaryDataBufferCount;
 
-    // Furthermore, any `binding`s that arise due to "pending" data
-    // in the type of the object (due to specialization for existential types)
-    // will need to come after all the other `binding`s that were
-    // part of the "primary" (unspecialized) data.
-    //
-    uint32_t primaryDescriptorCount =
-        ordinaryDataBufferCount +
-        (uint32_t)builder.m_elementTypeLayout->getSize(SLANG_PARAMETER_CATEGORY_DESCRIPTOR_TABLE_SLOT);
-    elementOffset.pending.binding = primaryDescriptorCount;
-
     // Once we've computed the offset information, we simply add the
     // descriptor ranges as if things were declared as a `ConstantBuffer<X>`,
     // since that is how things will be laid out inside the parameter block.
@@ -664,7 +627,7 @@ void EntryPointLayout::Builder::addEntryPointParams(slang::EntryPointLayout* ent
 {
     m_slangEntryPointLayout = entryPointLayout;
     setElementTypeLayout(entryPointLayout->getTypeLayout());
-    m_shaderStageFlag = VulkanUtil::getShaderStage(entryPointLayout->getStage());
+    m_shaderStageFlag = translateShaderStage(entryPointLayout->getStage());
 
     // Note: we do not bother adding any descriptor sets/ranges here,
     // because the descriptor ranges of an entry point will simply
@@ -739,7 +702,6 @@ Result RootShaderObjectLayoutImpl::_init(const Builder* builder)
     m_program = builder->m_program;
     m_programLayout = builder->m_programLayout;
     m_entryPoints = _Move(builder->m_entryPoints);
-    m_pendingDataOffset = builder->m_pendingDataOffset;
     m_device = device;
 
     // If the program has unbound specialization parameters,
@@ -771,6 +733,14 @@ Result RootShaderObjectLayoutImpl::_init(const Builder* builder)
     // Once we've collected the information across the entire
     // tree of sub-objects
 
+    // Add bindless descriptor set layout if needed.
+    // We currently assume that the bindless descriptor set is always the last,
+    // following all other descriptor sets, without any gaps.
+    if (m_device->m_bindlessDescriptorSet)
+    {
+        m_vkDescriptorSetLayouts.push_back(m_device->m_bindlessDescriptorSet->m_descriptorSetLayout);
+    }
+
     // Now call Vulkan API to create a pipeline layout.
     VkPipelineLayoutCreateInfo pipelineLayoutCreateInfo = {};
     pipelineLayoutCreateInfo.sType = VK_STRUCTURE_TYPE_PIPELINE_LAYOUT_CREATE_INFO;
@@ -778,6 +748,20 @@ Result RootShaderObjectLayoutImpl::_init(const Builder* builder)
     pipelineLayoutCreateInfo.pSetLayouts = m_vkDescriptorSetLayouts.data();
     if (m_allPushConstantRanges.size())
     {
+        uint32_t totalPushConstantSize = 0;
+        for (const auto& range : m_allPushConstantRanges)
+        {
+            totalPushConstantSize = std::max(totalPushConstantSize, range.offset + range.size);
+        }
+        if (totalPushConstantSize > m_device->m_api.m_deviceProperties.limits.maxPushConstantsSize)
+        {
+            m_device->printError(
+                "Total push constant size (%u) exceeds the maximum allowed (%u).",
+                totalPushConstantSize,
+                m_device->m_api.m_deviceProperties.limits.maxPushConstantsSize
+            );
+            return SLANG_FAIL;
+        }
         pipelineLayoutCreateInfo.pushConstantRangeCount = (uint32_t)m_allPushConstantRanges.size();
         pipelineLayoutCreateInfo.pPushConstantRanges = m_allPushConstantRanges.data();
     }
@@ -919,7 +903,7 @@ void RootShaderObjectLayoutImpl::Builder::addGlobalParams(slang::VariableLayoutR
     // While we expect that the parameter in the global scope start
     // at an offset of zero, it is also worth querying the offset
     // information because it could impact the locations assigned
-    // to "pending" data in the case of static specialization.
+    // for handling static specialization cases.
     //
     BindingOffset offset(globalsLayout);
 
@@ -936,11 +920,8 @@ void RootShaderObjectLayoutImpl::Builder::addGlobalParams(slang::VariableLayoutR
     //
     _addDescriptorRangesAsValue(globalsLayout->getTypeLayout(), offset);
 
-    // We want to keep track of the offset that was applied to "pending"
-    // data because we will need it again later when it comes time to
-    // actually bind things.
+    // Binding offset handling has been simplified after pending data removal.
     //
-    m_pendingDataOffset = offset.pending;
 }
 
 void RootShaderObjectLayoutImpl::Builder::addEntryPoint(EntryPointLayout* entryPointLayout)
@@ -949,13 +930,12 @@ void RootShaderObjectLayoutImpl::Builder::addEntryPoint(EntryPointLayout* entryP
     auto entryPointVarLayout = slangEntryPointLayout->getVarLayout();
 
     // The offset information for each entry point needs to
-    // be adjusted by any offset for "pending" data that
+    // be handled uniformly now that pending data has been removed.
     // was recorded in the global-scope layout.
     //
     // TODO(tfoley): Double-check that this is correct.
 
     BindingOffset entryPointOffset(entryPointVarLayout);
-    entryPointOffset.pending += m_pendingDataOffset;
 
     EntryPointInfo info;
     info.layout = entryPointLayout;
