@@ -61,22 +61,12 @@ struct SimpleBindingOffset
 
 // While a "simple" binding offset representation will work in many cases,
 // once we need to deal with layout for programs with interface-type parameters
-// that have been statically specialized, we also need to track the offset
-// for where to bind any "pending" data that arises from the process of static
-// specialization.
-//
-// In order to conveniently track both the "primary" and "pending" offset information,
-// we will define a more complete `BindingOffset` type that combines simple
-// binding offsets for the primary and pending parts.
+// that have been statically specialized.
 
 /// A representation of the offset at which to bind a shader parameter or sub-object
 struct BindingOffset : SimpleBindingOffset
 {
-    // Offsets for "primary" data are stored directly in the `BindingOffset`
-    // via the inheritance from `SimpleBindingOffset`.
 
-    /// Offset for any "pending" data
-    SimpleBindingOffset pending;
 
     /// Create a default (zero) offset
     BindingOffset() {}
@@ -90,7 +80,6 @@ struct BindingOffset : SimpleBindingOffset
     /// Create an offset based on offset information in the given Slang `varLayout`
     BindingOffset(slang::VariableLayoutReflection* varLayout)
         : SimpleBindingOffset(varLayout)
-        , pending(varLayout->getPendingDataLayout())
     {
     }
 
@@ -98,11 +87,7 @@ struct BindingOffset : SimpleBindingOffset
     void operator+=(const SimpleBindingOffset& offset) { SimpleBindingOffset::operator+=(offset); }
 
     /// Add any values in the given `offset`
-    void operator+=(const BindingOffset& offset)
-    {
-        SimpleBindingOffset::operator+=(offset);
-        pending += offset.pending;
-    }
+    void operator+=(const BindingOffset& offset) { SimpleBindingOffset::operator+=(offset); }
 };
 
 class ShaderObjectLayoutImpl : public ShaderObjectLayout
@@ -148,14 +133,7 @@ public:
         SubObjectRangeOffset(slang::VariableLayoutReflection* varLayout)
             : BindingOffset(varLayout)
         {
-            if (auto pendingLayout = varLayout->getPendingDataLayout())
-            {
-                pendingOrdinaryData = (uint32_t)pendingLayout->getOffset(SLANG_PARAMETER_CATEGORY_UNIFORM);
-            }
         }
-
-        /// The offset for "pending" ordinary data related to this range
-        uint32_t pendingOrdinaryData = 0;
     };
 
     /// Stride information for a sub-object range
@@ -163,16 +141,7 @@ public:
     {
         SubObjectRangeStride() {}
 
-        SubObjectRangeStride(slang::TypeLayoutReflection* typeLayout)
-        {
-            if (auto pendingLayout = typeLayout->getPendingDataTypeLayout())
-            {
-                pendingOrdinaryData = (uint32_t)pendingLayout->getStride();
-            }
-        }
-
-        /// The stride for "pending" ordinary data related to this range
-        uint32_t pendingOrdinaryData = 0;
+        SubObjectRangeStride(slang::TypeLayoutReflection* typeLayout) {}
     };
 
     /// Information about a logical binding range as reported by Slang reflection
@@ -453,9 +422,6 @@ public:
         slang::IComponentType* m_program;
         slang::ProgramLayout* m_programLayout;
         std::vector<EntryPointInfo> m_entryPoints;
-
-        /// Offset to apply to "pending" data from this object, sub-objects, and entry points
-        SimpleBindingOffset m_pendingDataOffset;
     };
 
     uint32_t findEntryPointIndex(VkShaderStageFlags stage);
@@ -469,7 +435,6 @@ public:
         RootShaderObjectLayoutImpl** outLayout
     );
 
-    const SimpleBindingOffset& getPendingDataOffset() const { return m_pendingDataOffset; }
 
     slang::IComponentType* getSlangProgram() const { return m_program; }
     slang::ProgramLayout* getSlangProgramLayout() const { return m_programLayout; }
@@ -517,7 +482,6 @@ public:
     std::vector<VkPushConstantRange> m_allPushConstantRanges;
     uint32_t m_totalPushConstantSize = 0;
 
-    SimpleBindingOffset m_pendingDataOffset;
     DeviceImpl* m_device = nullptr;
 };
 
