@@ -5,6 +5,7 @@
 #include "vk-sampler.h"
 #include "vk-acceleration-structure.h"
 #include "vk-shader-object-layout.h"
+#include "vk-bindless-descriptor-set.h"
 
 #include "../state-tracking.h"
 
@@ -108,10 +109,18 @@ inline void writeAccelerationStructureDescriptor(
     AccelerationStructureImpl* as
 )
 {
+    // The Vulkan spec states: If the nullDescriptor feature is not enabled, each element of
+    // pAccelerationStructures must not be VK_NULL_HANDLE
+    if (!as && !device->m_api.m_extendedFeatures.robustness2Features.nullDescriptor)
+    {
+        SLANG_RHI_ASSERT_FAILURE("nullDescriptor feature is not available on the device");
+        return;
+    }
+
     VkWriteDescriptorSetAccelerationStructureKHR writeAS = {};
     writeAS.sType = VK_STRUCTURE_TYPE_WRITE_DESCRIPTOR_SET_ACCELERATION_STRUCTURE_KHR;
     writeAS.accelerationStructureCount = 1;
-    VkAccelerationStructureKHR nullHandle = VK_NULL_HANDLE;
+    static const VkAccelerationStructureKHR nullHandle = VK_NULL_HANDLE;
     writeAS.pAccelerationStructures = as ? &as->m_vkHandle : &nullHandle;
 
     VkWriteDescriptorSet write = {};
@@ -181,6 +190,42 @@ inline void writeSamplerDescriptor(
     writeDescriptor(device, write);
 }
 
+inline void writeBufferState(BindingDataBuilder* builder, BufferImpl* buffer, ResourceState state)
+{
+    BindingDataImpl* bindingData = builder->m_bindingData;
+    if (bindingData->bufferStateCount >= bindingData->bufferStateCapacity)
+    {
+        bindingData->bufferStateCapacity *= 2;
+        BindingDataImpl::BufferState* newBufferStates =
+            builder->m_allocator->allocate<BindingDataImpl::BufferState>(bindingData->bufferStateCapacity);
+        std::memcpy(
+            newBufferStates,
+            bindingData->bufferStates,
+            bindingData->bufferStateCount * sizeof(BindingDataImpl::BufferState)
+        );
+        bindingData->bufferStates = newBufferStates;
+    }
+    bindingData->bufferStates[bindingData->bufferStateCount++] = {buffer, state};
+}
+
+inline void writeTextureState(BindingDataBuilder* builder, TextureViewImpl* textureView, ResourceState state)
+{
+    BindingDataImpl* bindingData = builder->m_bindingData;
+    if (bindingData->textureStateCount >= bindingData->textureStateCapacity)
+    {
+        bindingData->textureStateCapacity *= 2;
+        BindingDataImpl::TextureState* newTextureStates =
+            builder->m_allocator->allocate<BindingDataImpl::TextureState>(bindingData->textureStateCapacity);
+        std::memcpy(
+            newTextureStates,
+            bindingData->textureStates,
+            bindingData->textureStateCount * sizeof(BindingDataImpl::TextureState)
+        );
+        bindingData->textureStates = newTextureStates;
+    }
+    bindingData->textureStates[bindingData->textureStateCount++] = {textureView, state};
+}
+
 
 Result BindingDataBuilder::bindAsRoot(
     RootShaderObject* shaderObject,
@@ -195,16 +240,26 @@ Result BindingDataBuilder::bindAsRoot(
     m_bindingCache->bindingData.push_back(m_bindingData);
 
     // TODO(shaderobject): we should count number of buffers/textures in the layout and allocate appropriately
-    uint32_t slotCount = 100; // specializedLayout->getSlotCount() is not the total number of slots of all subobjects
-    m_bindingData->bufferStates = m_allocator->allocate<BindingDataImpl::BufferState>(slotCount);
+    // For now we use a fixed starting capacity and grow as needed.
+    m_bindingData->bufferStateCapacity = 1024;
+    m_bindingData->bufferStates =
+        m_allocator->allocate<BindingDataImpl::BufferState>(m_bindingData->bufferStateCapacity);
     m_bindingData->bufferStateCount = 0;
-    m_bindingData->textureStates = m_allocator->allocate<BindingDataImpl::TextureState>(slotCount);
+    m_bindingData->textureStateCapacity = 1024;
+    m_bindingData->textureStates =
+        m_allocator->allocate<BindingDataImpl::TextureState>(m_bindingData->textureStateCapacity);
     m_bindingData->textureStateCount = 0;
 
     m_bindingData->pipelineLayout = specializedLayout->m_pipelineLayout;
 
-    m_bindingData->descriptorSets =
-        m_allocator->allocate<VkDescriptorSet>(specializedLayout->getTotalDescriptorSetCount());
+    uint32_t totalDescriptorSetCount = specializedLayout->getTotalDescriptorSetCount();
+    if (m_device->m_bindlessDescriptorSet)
+    {
+        // The bindless descriptor set is always the last descriptor set in the pipeline layout.
+        // We need to add one more descriptor set to the count to account for it.
+        totalDescriptorSetCount++;
+    }
+    m_bindingData->descriptorSets = m_allocator->allocate<VkDescriptorSet>(totalDescriptorSetCount);
     m_bindingData->descriptorSetCount = 0;
 
     m_pushConstantRanges = specializedLayout->getAllPushConstantRanges();
@@ -214,7 +269,6 @@ Result BindingDataBuilder::bindAsRoot(
     m_bindingData->pushConstantCount = 0;
 
     BindingOffset offset = {};
-    offset.pending = specializedLayout->getPendingDataOffset();
 
     // Note: the operations here are quite similar to what `bindAsParameterBlock` does.
     // The key difference in practice is that we do *not* make use of the adjustment
@@ -252,6 +306,13 @@ Result BindingDataBuilder::bindAsRoot(
         SLANG_RETURN_ON_FAIL(bindAsEntryPoint(entryPoint, entryPointInfo.offset, entryPointLayout));
     }
 
+    // Assign bindless descriptor set to the last slot if available.
+    if (m_device->m_bindlessDescriptorSet)
+    {
+        m_bindingData->descriptorSets[m_bindingData->descriptorSetCount++] =
+            m_device->m_bindlessDescriptorSet->m_descriptorSet;
+    }
+
     outBindingData = m_bindingData;
 
     return SLANG_OK;
@@ -290,8 +351,7 @@ Result BindingDataBuilder::bindAsEntryPoint(
         // We expect that the size of the range as reflected matches the
         // amount of ordinary data stored on this object.
         //
-        // TODO: This would not be the case if specialization for interface-type
-        // parameters led to the entry point having "pending" ordinary data.
+        // Note: Entry points with ordinary data are handled uniformly now.
         //
         SLANG_RHI_ASSERT(pushConstantRange.size == shaderObject->m_data.size());
 
@@ -390,7 +450,7 @@ Result BindingDataBuilder::bindAsValue(
                 writeTextureDescriptor(device, descriptorSet, binding, i, descriptorType, textureView);
                 if (textureView)
                 {
-                    m_bindingData->textureStates[m_bindingData->textureStateCount++] = {textureView, requiredState};
+                    writeTextureState(this, textureView, requiredState);
                 }
             }
             break;
@@ -407,7 +467,7 @@ Result BindingDataBuilder::bindAsValue(
                 writeTextureSamplerDescriptor(device, descriptorSet, binding, i, textureView, sampler);
                 if (textureView)
                 {
-                    m_bindingData->textureStates[m_bindingData->textureStateCount++] = {textureView, requiredState};
+                    writeTextureState(this, textureView, requiredState);
                 }
             }
             break;
@@ -439,7 +499,7 @@ Result BindingDataBuilder::bindAsValue(
                 writePlainBufferDescriptor(device, descriptorSet, binding, i, descriptorType, buffer, slot.bufferRange);
                 if (buffer)
                 {
-                    m_bindingData->bufferStates[m_bindingData->bufferStateCount++] = {buffer, requiredState};
+                    writeBufferState(this, buffer, requiredState);
                 }
             }
             break;
@@ -470,7 +530,7 @@ Result BindingDataBuilder::bindAsValue(
                 );
                 if (buffer)
                 {
-                    m_bindingData->bufferStates[m_bindingData->bufferStateCount++] = {buffer, requiredState};
+                    writeBufferState(this, buffer, requiredState);
                 }
             }
             break;
@@ -483,6 +543,10 @@ Result BindingDataBuilder::bindAsValue(
                 const ResourceSlot& slot = shaderObject->m_slots[slotIndex + i];
                 AccelerationStructureImpl* as = checked_cast<AccelerationStructureImpl*>(slot.resource.get());
                 writeAccelerationStructureDescriptor(device, descriptorSet, binding, i, as);
+                if (as)
+                {
+                    writeBufferState(this, as->m_buffer, ResourceState::AccelerationStructureRead);
+                }
             }
             break;
         }
@@ -566,27 +630,8 @@ Result BindingDataBuilder::bindAsValue(
             //
             if (subObjectLayout)
             {
-                // Second, the offset where we want to start binding for existential-type
-                // ranges is a bit different, because we don't wnat to bind at the "primary"
-                // offset that got passed down, but instead at the "pending" offset.
-                //
-                // For the purposes of nested binding, what used to be the pending offset
-                // will now be used as the primary offset.
-                //
-                SimpleBindingOffset objOffset = rangeOffset.pending;
-                SimpleBindingOffset objStride = rangeStride.pending;
-                for (uint32_t i = 0; i < count; ++i)
-                {
-                    // An existential-type sub-object is always bound just as a value,
-                    // which handles its nested bindings and descriptor sets, but
-                    // does not deal with ordianry data. The ordinary data should
-                    // have been handled as part of the buffer for a parent object
-                    // already.
-                    //
-                    ShaderObject* subObject = shaderObject->m_objects[subObjectIndex + i];
-                    SLANG_RETURN_ON_FAIL(bindAsValue(subObject, BindingOffset(objOffset), subObjectLayout));
-                    objOffset += objStride;
-                }
+                // Interface-typed sub-object ranges are no longer supported
+                // after pending data layout APIs have been removed.
             }
             break;
         case slang::BindingType::RawBuffer:
@@ -644,13 +689,8 @@ Result BindingDataBuilder::bindAsParameterBlock(
     offset.bindingSet = m_bindingData->descriptorSetCount;
     offset.binding = 0;
 
-    // TODO: We should also be writing to `offset.pending` here,
-    // because any resource/sampler bindings related to "pending"
-    // data should *also* be writing into the chosen set.
-    //
-    // The challenge here is that we need to compute the right
-    // value for `offset.pending.binding`, so that it writes after
-    // all the other bindings.
+    // Note: Interface-type binding handling has been simplified
+    // now that pending data layout APIs have been removed.
 
     // Writing the bindings for a parameter block is relatively easy:
     // we just need to allocate the descriptor set(s) needed for this
