@@ -4,7 +4,7 @@
 #include "wgpu-texture.h"
 #include "wgpu-pipeline.h"
 #include "wgpu-shader-object.h"
-#include "wgpu-util.h"
+#include "wgpu-utils.h"
 
 #include "../strings.h"
 
@@ -83,9 +83,11 @@ public:
     void cmdQueryAccelerationStructureProperties(const commands::QueryAccelerationStructureProperties& cmd);
     void cmdSerializeAccelerationStructure(const commands::SerializeAccelerationStructure& cmd);
     void cmdDeserializeAccelerationStructure(const commands::DeserializeAccelerationStructure& cmd);
+    void cmdExecuteClusterOperation(const commands::ExecuteClusterOperation& cmd);
     void cmdConvertCooperativeVectorMatrix(const commands::ConvertCooperativeVectorMatrix& cmd);
     void cmdSetBufferState(const commands::SetBufferState& cmd);
     void cmdSetTextureState(const commands::SetTextureState& cmd);
+    void cmdGlobalBarrier(const commands::GlobalBarrier& cmd);
     void cmdPushDebugGroup(const commands::PushDebugGroup& cmd);
     void cmdPopDebugGroup(const commands::PopDebugGroup& cmd);
     void cmdInsertDebugMarker(const commands::InsertDebugMarker& cmd);
@@ -97,9 +99,9 @@ public:
 
 Result CommandRecorder::record(CommandBufferImpl* commandBuffer)
 {
-    auto existingError = m_device->getAndClearLastError();
+    auto existingError = m_device->getAndClearLastUncapturedError();
     if (existingError != WGPUErrorType_NoError)
-        m_device->warning("Web GPU device had reported error before command record.");
+        m_device->printWarning("Web GPU device had reported error before command record.");
 
     m_commandEncoder = m_ctx.api.wgpuDeviceCreateCommandEncoder(m_ctx.device, nullptr);
     SLANG_RHI_DEFERRED({ m_ctx.api.wgpuCommandEncoderRelease(m_commandEncoder); });
@@ -138,7 +140,7 @@ Result CommandRecorder::record(CommandBufferImpl* commandBuffer)
         return SLANG_FAIL;
     }
 
-    auto lastError = m_device->getAndClearLastError();
+    auto lastError = m_device->getAndClearLastUncapturedError();
     if (lastError != WGPUErrorType_NoError)
     {
         return SLANG_FAIL;
@@ -147,7 +149,7 @@ Result CommandRecorder::record(CommandBufferImpl* commandBuffer)
     return SLANG_OK;
 }
 
-#define NOT_SUPPORTED(x) m_device->warning(x " command is not supported!")
+#define NOT_SUPPORTED(x) m_device->printWarning(x " command is not supported!")
 
 void CommandRecorder::cmdCopyBuffer(const commands::CopyBuffer& cmd)
 {
@@ -237,13 +239,13 @@ void CommandRecorder::cmdCopyTexture(const commands::CopyTexture& cmd)
             uint32_t srcZ = cmd.srcOffset.z + cmd.srcSubresource.layer + layer;
             uint32_t dstZ = cmd.dstOffset.z + cmd.dstSubresource.layer + layer;
 
-            WGPUImageCopyTexture source = {};
+            WGPUTexelCopyTextureInfo source = {};
             source.texture = src->m_texture;
             source.origin = {cmd.srcOffset.x, cmd.srcOffset.y, srcZ};
             source.mipLevel = srcMip;
             source.aspect = WGPUTextureAspect_All;
 
-            WGPUImageCopyTexture destination = {};
+            WGPUTexelCopyTextureInfo destination = {};
             destination.texture = dst->m_texture;
             destination.origin = {cmd.dstOffset.x, cmd.dstOffset.y, dstZ};
             destination.mipLevel = dstMip;
@@ -307,13 +309,13 @@ void CommandRecorder::cmdCopyTextureToBuffer(const commands::CopyTextureToBuffer
     SLANG_RHI_ASSERT(srcLayer == 0 || srcOffset.z == 0);
     uint32_t z = srcOffset.z + srcLayer;
 
-    WGPUImageCopyTexture source = {};
+    WGPUTexelCopyTextureInfo source = {};
     source.texture = src->m_texture;
     source.origin = {srcOffset.x, srcOffset.y, z};
     source.mipLevel = srcMip;
     source.aspect = WGPUTextureAspect_All;
 
-    WGPUImageCopyBuffer destination = {};
+    WGPUTexelCopyBufferInfo destination = {};
     destination.buffer = dst->m_buffer;
     destination.layout.offset = dstOffset;
     destination.layout.bytesPerRow = dstRowPitch;
@@ -365,7 +367,7 @@ void CommandRecorder::cmdUploadTextureData(const commands::UploadTextureData& cm
         {
             uint32_t mip = subresourceRange.mip + mipOffset;
 
-            WGPUImageCopyBuffer srcRegion;
+            WGPUTexelCopyBufferInfo srcRegion;
             srcRegion.buffer = buffer->m_buffer;
             srcRegion.layout.bytesPerRow = srLayout->rowPitch;
             srcRegion.layout.rowsPerImage = srLayout->rowCount;
@@ -376,7 +378,7 @@ void CommandRecorder::cmdUploadTextureData(const commands::UploadTextureData& cm
             SLANG_RHI_ASSERT(layer == 0 || cmd.offset.z == 0);
             uint32_t z = cmd.offset.z + layer;
 
-            WGPUImageCopyTexture dstRegion;
+            WGPUTexelCopyTextureInfo dstRegion;
             dstRegion.aspect = WGPUTextureAspect_All;
             dstRegion.mipLevel = mip;
             dstRegion.origin = {(uint32_t)cmd.offset.x, (uint32_t)cmd.offset.y, z};
@@ -451,11 +453,13 @@ void CommandRecorder::cmdBeginRenderPass(const commands::BeginRenderPass& cmd)
 
     endPassEncoder();
     m_renderPassEncoder = m_ctx.api.wgpuCommandEncoderBeginRenderPass(m_commandEncoder, &passDesc);
+    m_renderPassActive = true;
 }
 
 void CommandRecorder::cmdEndRenderPass(const commands::EndRenderPass& cmd)
 {
     endPassEncoder();
+    m_renderPassActive = false;
 }
 
 void CommandRecorder::cmdSetRenderState(const commands::SetRenderState& cmd)
@@ -646,11 +650,14 @@ void CommandRecorder::cmdDrawMeshTasks(const commands::DrawMeshTasks& cmd)
 
 void CommandRecorder::cmdBeginComputePass(const commands::BeginComputePass& cmd)
 {
+    endPassEncoder();
+    m_computePassEncoder = m_ctx.api.wgpuCommandEncoderBeginComputePass(m_commandEncoder, nullptr);
     m_computePassActive = true;
 }
 
 void CommandRecorder::cmdEndComputePass(const commands::EndComputePass& cmd)
 {
+    endPassEncoder();
     m_computePassActive = false;
 }
 
@@ -662,11 +669,6 @@ void CommandRecorder::cmdSetComputeState(const commands::SetComputeState& cmd)
     bool updatePipeline = !m_computeStateValid || cmd.pipeline != m_computePipeline;
     bool updateBindings = updatePipeline || cmd.bindingData != m_bindingData;
 
-    if (!m_computePassEncoder)
-    {
-        m_computePassEncoder = m_ctx.api.wgpuCommandEncoderBeginComputePass(m_commandEncoder, nullptr);
-    }
-
     WGPUComputePassEncoder encoder = m_computePassEncoder;
 
     if (updatePipeline)
@@ -765,6 +767,12 @@ void CommandRecorder::cmdDeserializeAccelerationStructure(const commands::Deseri
     NOT_SUPPORTED(S_CommandEncoder_deserializeAccelerationStructure);
 }
 
+void CommandRecorder::cmdExecuteClusterOperation(const commands::ExecuteClusterOperation& cmd)
+{
+    SLANG_UNUSED(cmd);
+    NOT_SUPPORTED(S_CommandEncoder_executeClusterOperation);
+}
+
 void CommandRecorder::cmdConvertCooperativeVectorMatrix(const commands::ConvertCooperativeVectorMatrix& cmd)
 {
     SLANG_UNUSED(cmd);
@@ -781,34 +789,57 @@ void CommandRecorder::cmdSetTextureState(const commands::SetTextureState& cmd)
     SLANG_UNUSED(cmd);
 }
 
+void CommandRecorder::cmdGlobalBarrier(const commands::GlobalBarrier& cmd)
+{
+    SLANG_UNUSED(cmd);
+}
+
 void CommandRecorder::cmdPushDebugGroup(const commands::PushDebugGroup& cmd)
 {
-    if (m_renderPassEncoder)
-        m_ctx.api.wgpuRenderPassEncoderPushDebugGroup(m_renderPassEncoder, cmd.name);
-    else if (m_computePassEncoder)
-        m_ctx.api.wgpuComputePassEncoderPushDebugGroup(m_computePassEncoder, cmd.name);
+    if (m_renderPassActive)
+    {
+        m_ctx.api.wgpuRenderPassEncoderPushDebugGroup(m_renderPassEncoder, translateString(cmd.name));
+    }
+    else if (m_computePassActive)
+    {
+        m_ctx.api.wgpuComputePassEncoderPushDebugGroup(m_computePassEncoder, translateString(cmd.name));
+    }
     else
-        m_ctx.api.wgpuCommandEncoderPushDebugGroup(m_commandEncoder, cmd.name);
+    {
+        m_ctx.api.wgpuCommandEncoderPushDebugGroup(m_commandEncoder, translateString(cmd.name));
+    }
 }
 
 void CommandRecorder::cmdPopDebugGroup(const commands::PopDebugGroup& cmd)
 {
-    if (m_renderPassEncoder)
+    if (m_renderPassActive)
+    {
         m_ctx.api.wgpuRenderPassEncoderPopDebugGroup(m_renderPassEncoder);
-    else if (m_computePassEncoder)
+    }
+    else if (m_computePassActive)
+    {
         m_ctx.api.wgpuComputePassEncoderPopDebugGroup(m_computePassEncoder);
+    }
     else
+    {
         m_ctx.api.wgpuCommandEncoderPopDebugGroup(m_commandEncoder);
+    }
 }
 
 void CommandRecorder::cmdInsertDebugMarker(const commands::InsertDebugMarker& cmd)
 {
-    if (m_renderPassEncoder)
-        m_ctx.api.wgpuRenderPassEncoderInsertDebugMarker(m_renderPassEncoder, cmd.name);
-    else if (m_computePassEncoder)
-        m_ctx.api.wgpuComputePassEncoderInsertDebugMarker(m_computePassEncoder, cmd.name);
+    if (m_renderPassActive)
+    {
+        m_ctx.api.wgpuRenderPassEncoderInsertDebugMarker(m_renderPassEncoder, translateString(cmd.name));
+    }
+    else if (m_computePassActive)
+    {
+        m_ctx.api.wgpuComputePassEncoderInsertDebugMarker(m_computePassEncoder, translateString(cmd.name));
+    }
     else
-        m_ctx.api.wgpuCommandEncoderInsertDebugMarker(m_commandEncoder, cmd.name);
+    {
+        m_ctx.api.wgpuCommandEncoderInsertDebugMarker(m_commandEncoder, translateString(cmd.name));
+    }
 }
 
 void CommandRecorder::cmdWriteTimestamp(const commands::WriteTimestamp& cmd)
@@ -911,13 +942,15 @@ Result CommandQueueImpl::waitOnHost()
 
     // Wait for the command buffer to finish executing
     {
-        WGPUQueueWorkDoneStatus status = WGPUQueueWorkDoneStatus_Unknown;
-        WGPUQueueWorkDoneCallbackInfo2 callbackInfo = {};
+        WGPUQueueWorkDoneStatus status = WGPUQueueWorkDoneStatus(0);
+        WGPUQueueWorkDoneCallbackInfo callbackInfo = {};
         callbackInfo.mode = WGPUCallbackMode_WaitAnyOnly;
         callbackInfo.callback = [](WGPUQueueWorkDoneStatus status_, void* userdata1, void* userdata2)
-        { *(WGPUQueueWorkDoneStatus*)userdata1 = status_; };
+        {
+            *(WGPUQueueWorkDoneStatus*)userdata1 = status_;
+        };
         callbackInfo.userdata1 = &status;
-        WGPUFuture future = device->m_ctx.api.wgpuQueueOnSubmittedWorkDone2(m_queue, callbackInfo);
+        WGPUFuture future = device->m_ctx.api.wgpuQueueOnSubmittedWorkDone(m_queue, callbackInfo);
         constexpr size_t futureCount = 1;
         WGPUFutureWaitInfo futures[futureCount] = {{future}};
         uint64_t timeoutNS = UINT64_MAX;
