@@ -14,13 +14,13 @@ public:
     WGPUSurface m_surface = nullptr;
     WGPUPresentMode m_vsyncOffMode = WGPUPresentMode(0);
     WGPUPresentMode m_vsyncOnMode = WGPUPresentMode(0);
-    bool m_configured = false;
 
     ~SurfaceImpl();
 
     Result init(DeviceImpl* device, WindowHandle windowHandle);
 
     virtual SLANG_NO_THROW Result SLANG_MCALL configure(const SurfaceConfig& config) override;
+    virtual SLANG_NO_THROW Result SLANG_MCALL unconfigure() override;
     virtual SLANG_NO_THROW Result SLANG_MCALL acquireNextImage(ITexture** outTexture) override;
     virtual SLANG_NO_THROW Result SLANG_MCALL present() override;
 };
