@@ -1,6 +1,5 @@
 #include "testing.h"
 
-#include "texture-utils.h"
 #include "texture-test.h"
 #include "resource-desc-utils.h"
 
@@ -10,7 +9,7 @@
 using namespace rhi;
 using namespace rhi::testing;
 
-GPU_TEST_CASE("cmd-copy-texture-full", D3D12 | Vulkan | WGPU | CUDA)
+GPU_TEST_CASE("cmd-copy-texture-full", D3D11 | D3D12 | Vulkan | WGPU | CUDA)
 {
     TextureTestOptions options(device);
     options.addVariants(
@@ -64,7 +63,7 @@ GPU_TEST_CASE("cmd-copy-texture-full", D3D12 | Vulkan | WGPU | CUDA)
     );
 }
 
-GPU_TEST_CASE("cmd-copy-texture-arrayrange", D3D12 | Vulkan | WGPU | CUDA)
+GPU_TEST_CASE("cmd-copy-texture-arrayrange", D3D11 | D3D12 | Vulkan | WGPU | CUDA)
 {
     TextureTestOptions options(device);
     options.addVariants(TTShape::All, TTArray::On, TTMip::Both, TTFmtDepth::Off);
@@ -118,7 +117,7 @@ GPU_TEST_CASE("cmd-copy-texture-arrayrange", D3D12 | Vulkan | WGPU | CUDA)
     );
 }
 
-GPU_TEST_CASE("cmd-copy-texture-miprange", D3D12 | Vulkan | WGPU | CUDA)
+GPU_TEST_CASE("cmd-copy-texture-miprange", D3D11 | D3D12 | Vulkan | WGPU | CUDA)
 {
     TextureTestOptions options(device);
     options.addVariants(TTShape::All, TTArray::Both, TTMip::On, TTFmtDepth::Off);
@@ -173,7 +172,7 @@ GPU_TEST_CASE("cmd-copy-texture-miprange", D3D12 | Vulkan | WGPU | CUDA)
     );
 }
 
-GPU_TEST_CASE("cmd-copy-texture-fromarray", D3D12 | Vulkan | WGPU | CUDA)
+GPU_TEST_CASE("cmd-copy-texture-fromarray", D3D11 | D3D12 | Vulkan | WGPU | CUDA)
 {
     TextureTestOptions options(device);
     options.addVariants(TTShape::D1 | TTShape::D2, TTArray::On, TTMip::Both, TTFmtDepth::Off);
@@ -221,7 +220,7 @@ GPU_TEST_CASE("cmd-copy-texture-fromarray", D3D12 | Vulkan | WGPU | CUDA)
     );
 }
 
-GPU_TEST_CASE("cmd-copy-texture-toarray", D3D12 | Vulkan | WGPU | CUDA)
+GPU_TEST_CASE("cmd-copy-texture-toarray", D3D11 | D3D12 | Vulkan | WGPU | CUDA)
 {
     TextureTestOptions options(device);
     options.addVariants(TTShape::D1 | TTShape::D2, TTArray::On, TTMip::Both, TTFmtDepth::Off, TextureInitMode::None);
@@ -272,7 +271,7 @@ GPU_TEST_CASE("cmd-copy-texture-toarray", D3D12 | Vulkan | WGPU | CUDA)
     );
 }
 
-GPU_TEST_CASE("cmd-copy-texture-fromslice", D3D12 | Vulkan | WGPU | CUDA)
+GPU_TEST_CASE("cmd-copy-texture-fromslice", D3D11 | D3D12 | Vulkan | WGPU | CUDA)
 {
     TextureTestOptions options(device);
     options.addVariants(TTShape::D3, TTArray::Off, TTMip::Both, TTFmtDepth::Off);
@@ -324,7 +323,7 @@ GPU_TEST_CASE("cmd-copy-texture-fromslice", D3D12 | Vulkan | WGPU | CUDA)
     );
 }
 
-GPU_TEST_CASE("cmd-copy-texture-arrayfromslice", D3D12 | Vulkan | WGPU | CUDA)
+GPU_TEST_CASE("cmd-copy-texture-arrayfromslice", D3D11 | D3D12 | Vulkan | WGPU | CUDA)
 {
     TextureTestOptions options(device);
     options.addVariants(TTShape::D3, TTArray::Off, TTMip::Both, TTFmtDepth::Off);
@@ -377,7 +376,7 @@ GPU_TEST_CASE("cmd-copy-texture-arrayfromslice", D3D12 | Vulkan | WGPU | CUDA)
     );
 }
 
-GPU_TEST_CASE("cmd-copy-texture-toslice", D3D12 | Vulkan | WGPU | CUDA)
+GPU_TEST_CASE("cmd-copy-texture-toslice", D3D11 | D3D12 | Vulkan | WGPU | CUDA)
 {
     TextureTestOptions options(device);
     options.addVariants(TTShape::D3, TTArray::Off, TTMip::Off, TTFmtDepth::Off, TextureInitMode::Invalid);
@@ -447,7 +446,7 @@ GPU_TEST_CASE("cmd-copy-texture-toslice", D3D12 | Vulkan | WGPU | CUDA)
     );
 }
 
-GPU_TEST_CASE("cmd-copy-texture-offset-nomip", D3D12 | Vulkan | WGPU | CUDA)
+GPU_TEST_CASE("cmd-copy-texture-offset-nomip", D3D11 | D3D12 | Vulkan | WGPU | CUDA)
 {
     TextureTestOptions options(device);
     options.addVariants(TTShape::All, TTArray::Both, TTFmtDepth::Off);
@@ -497,7 +496,7 @@ GPU_TEST_CASE("cmd-copy-texture-offset-nomip", D3D12 | Vulkan | WGPU | CUDA)
     );
 }
 
-GPU_TEST_CASE("cmd-copy-texture-sizeoffset-nomip", D3D12 | Vulkan | WGPU | CUDA)
+GPU_TEST_CASE("cmd-copy-texture-sizeoffset-nomip", D3D11 | D3D12 | Vulkan | WGPU | CUDA)
 {
     TextureTestOptions options(device);
     options.addVariants(TTShape::All, TTArray::Both, TTFmtDepth::Off);
@@ -547,7 +546,7 @@ GPU_TEST_CASE("cmd-copy-texture-sizeoffset-nomip", D3D12 | Vulkan | WGPU | CUDA)
     );
 }
 
-GPU_TEST_CASE("cmd-copy-texture-smalltolarge", D3D12 | Vulkan | WGPU | CUDA)
+GPU_TEST_CASE("cmd-copy-texture-smalltolarge", D3D11 | D3D12 | Vulkan | WGPU | CUDA)
 {
     TextureTestOptions options(device);
     options.addVariants(TTShape::All, TTArray::Both, TTFmtDepth::Off);
@@ -595,7 +594,7 @@ GPU_TEST_CASE("cmd-copy-texture-smalltolarge", D3D12 | Vulkan | WGPU | CUDA)
     );
 }
 
-GPU_TEST_CASE("cmd-copy-texture-largetosmall", D3D12 | Vulkan | WGPU | CUDA)
+GPU_TEST_CASE("cmd-copy-texture-largetosmall", D3D11 | D3D12 | Vulkan | WGPU | CUDA)
 {
     TextureTestOptions options(device);
     options.addVariants(TTShape::All, TTArray::Both, TTFmtDepth::Off);
@@ -648,7 +647,7 @@ GPU_TEST_CASE("cmd-copy-texture-largetosmall", D3D12 | Vulkan | WGPU | CUDA)
     );
 }
 
-GPU_TEST_CASE("cmd-copy-texture-acrossmips", D3D12 | Vulkan | WGPU | CUDA)
+GPU_TEST_CASE("cmd-copy-texture-acrossmips", D3D11 | D3D12 | Vulkan | WGPU | CUDA)
 {
     TextureTestOptions options(device);
     options.addVariants(TTShape::All, TTArray::Both, TTMip::On, TTFmtDepth::Off);
@@ -697,7 +696,7 @@ GPU_TEST_CASE("cmd-copy-texture-acrossmips", D3D12 | Vulkan | WGPU | CUDA)
     );
 }
 
-GPU_TEST_CASE("cmd-copy-texture-offset-mip1", D3D12 | Vulkan | WGPU | CUDA)
+GPU_TEST_CASE("cmd-copy-texture-offset-mip1", D3D11 | D3D12 | Vulkan | WGPU | CUDA)
 {
     TextureTestOptions options(device);
     options.addVariants(TTShape::All, TTArray::Both, TTMip::On, TTFmtDepth::Off);
@@ -756,7 +755,7 @@ GPU_TEST_CASE("cmd-copy-texture-offset-mip1", D3D12 | Vulkan | WGPU | CUDA)
     );
 }
 
-GPU_TEST_CASE("cmd-copy-texture-offset-mip1", D3D12 | Vulkan | WGPU | CUDA)
+GPU_TEST_CASE("cmd-copy-texture-offset-mip1", D3D11 | D3D12 | Vulkan | WGPU | CUDA)
 {
     TextureTestOptions options(device);
     options.addVariants(TTShape::All, TTArray::Both, TTMip::On, TTFmtDepth::Off);
