@@ -8,7 +8,7 @@
 using namespace rhi;
 using namespace rhi::testing;
 
-class VirtualShaderCache : public IPersistentShaderCache
+class VirtualShaderCache : public IPersistentCache
 {
 public:
     struct Stats
@@ -47,7 +47,10 @@ public:
             auto it = std::min_element(
                 entries.begin(),
                 entries.end(),
-                [](const auto& a, const auto& b) { return a.second.ticket < b.second.ticket; }
+                [](const auto& a, const auto& b)
+                {
+                    return a.second.ticket < b.second.ticket;
+                }
             );
             entries.erase(it);
         }
@@ -85,9 +88,9 @@ public:
 
     virtual SLANG_NO_THROW Result SLANG_MCALL queryInterface(const SlangUUID& uuid, void** outObject) override
     {
-        if (uuid == IPersistentShaderCache::getTypeGuid())
+        if (uuid == IPersistentCache::getTypeGuid())
         {
-            *outObject = static_cast<IPersistentShaderCache*>(this);
+            *outObject = static_cast<IPersistentCache*>(this);
             return SLANG_OK;
         }
         return SLANG_E_NO_INTERFACE;
@@ -117,13 +120,12 @@ static VirtualShaderCache shaderCache;
 struct ShaderCacheTest
 {
     GpuTestContext* ctx;
-    DeviceType deviceType;
     std::filesystem::path tempDirectory;
 
     ComPtr<IDevice> device;
     ComPtr<IComputePipeline> computePipeline;
     ComPtr<IRenderPipeline> renderPipeline;
-    ComPtr<IBuffer> buffer;
+    ComPtr<IBuffer> rwBuffer;
 
     std::string computeShaderA = std::string(
         R"(
@@ -179,7 +181,8 @@ struct ShaderCacheTest
     void createDevice()
     {
         DeviceDesc deviceDesc = {};
-        deviceDesc.deviceType = deviceType;
+        deviceDesc.deviceType = ctx->deviceType;
+        deviceDesc.adapter = getSelectedDeviceAdapter(ctx->deviceType);
         deviceDesc.slang.slangGlobalSession = ctx->slangGlobalSession;
         auto searchPaths = getSlangSearchPaths();
         std::string tempDirectoryStr = tempDirectory.string();
@@ -219,20 +222,19 @@ struct ShaderCacheTest
         bufferDesc.defaultState = ResourceState::UnorderedAccess;
         bufferDesc.memoryType = MemoryType::DeviceLocal;
 
-        REQUIRE_CALL(device->createBuffer(bufferDesc, (void*)initialData, buffer.writeRef()));
+        REQUIRE_CALL(device->createBuffer(bufferDesc, (void*)initialData, rwBuffer.writeRef()));
     }
 
     void freeComputeResources()
     {
-        buffer = nullptr;
+        rwBuffer = nullptr;
         computePipeline = nullptr;
     }
 
     void createComputePipeline(const char* moduleName, const char* entryPointName)
     {
         ComPtr<IShaderProgram> shaderProgram;
-        slang::ProgramLayout* slangReflection;
-        REQUIRE_CALL(loadComputeProgram(device, shaderProgram, moduleName, entryPointName, slangReflection));
+        REQUIRE_CALL(loadAndLinkProgram(device, moduleName, entryPointName, shaderProgram.writeRef()));
 
         ComputePipelineDesc pipelineDesc = {};
         pipelineDesc.program = shaderProgram.get();
@@ -242,7 +244,7 @@ struct ShaderCacheTest
     void createComputePipeline(std::string shaderSource)
     {
         ComPtr<IShaderProgram> shaderProgram;
-        REQUIRE_CALL(loadComputeProgramFromSource(device, shaderProgram, shaderSource));
+        REQUIRE_CALL(loadComputeProgramFromSource(device, shaderSource, shaderProgram.writeRef()));
 
         ComputePipelineDesc pipelineDesc = {};
         pipelineDesc.program = shaderProgram.get();
@@ -256,7 +258,7 @@ struct ShaderCacheTest
         auto passEncoder = commandEncoder->beginComputePass();
         auto rootObject = passEncoder->bindPipeline(computePipeline);
         ShaderCursor entryPointCursor(rootObject->getEntryPoint(0));
-        entryPointCursor["buffer"].setBinding(buffer);
+        entryPointCursor["buffer"].setBinding(rwBuffer);
         passEncoder->dispatchCompute(4, 1, 1);
         passEncoder->end();
         queue->submit(commandEncoder->finish());
@@ -266,7 +268,7 @@ struct ShaderCacheTest
     bool checkOutput(const std::vector<float>& expectedOutput)
     {
         ComPtr<ISlangBlob> bufferBlob;
-        device->readBuffer(buffer, 0, 4 * sizeof(float), bufferBlob.writeRef());
+        device->readBuffer(rwBuffer, 0, 4 * sizeof(float), bufferBlob.writeRef());
         REQUIRE(bufferBlob);
         REQUIRE(bufferBlob->getBufferSize() == expectedOutput.size() * sizeof(float));
         return ::memcmp(bufferBlob->getBufferPointer(), expectedOutput.data(), bufferBlob->getBufferSize()) == 0;
@@ -296,10 +298,9 @@ struct ShaderCacheTest
 
     VirtualShaderCache::Stats getStats() { return shaderCache.stats; }
 
-    void run(GpuTestContext* ctx_, DeviceType deviceType_, std::string tempDirectory_)
+    void run(GpuTestContext* ctx_, std::string tempDirectory_)
     {
         ctx = ctx_;
-        deviceType = deviceType_;
         tempDirectory = tempDirectory_;
 
         shaderCache.clear();
@@ -516,18 +517,18 @@ struct ShaderCacheTestImportInclude : ShaderCacheTest
 // One shader featuring multiple kinds of shader objects that can be bound.
 struct ShaderCacheTestSpecialization : ShaderCacheTest
 {
-    slang::ProgramLayout* slangReflection;
+    slang::ProgramLayout* slangReflection = nullptr;
 
     void createComputePipeline()
     {
         ComPtr<IShaderProgram> shaderProgram;
 
-        REQUIRE_CALL(loadComputeProgram(
+        REQUIRE_CALL(loadAndLinkProgram(
             device,
-            shaderProgram,
             "test-shader-cache-specialization",
             "computeMain",
-            slangReflection
+            shaderProgram.writeRef(),
+            &slangReflection
         ));
 
         ComputePipelineDesc pipelineDesc = {};
@@ -554,7 +555,7 @@ struct ShaderCacheTestSpecialization : ShaderCacheTest
         transformer->finalize();
 
         ShaderCursor entryPointCursor(rootObject->getEntryPoint(0));
-        entryPointCursor["buffer"].setBinding(buffer);
+        entryPointCursor["buffer"].setBinding(rwBuffer);
         entryPointCursor["transformer"].setObject(transformer);
 
         passEncoder->dispatchCompute(1, 1, 1);
@@ -664,7 +665,7 @@ struct ShaderCacheTestGraphics : ShaderCacheTest
     ComPtr<ITextureView> colorBufferView;
     ComPtr<IInputLayout> inputLayout;
 
-    ComPtr<IBuffer> createVertexBuffer(IDevice* device)
+    ComPtr<IBuffer> createVertexBuffer()
     {
         const Vertex vertices[] = {
             {0, 0, 0.5},
@@ -672,29 +673,29 @@ struct ShaderCacheTestGraphics : ShaderCacheTest
             {0, 1, 0.5},
         };
 
-        BufferDesc vertexBufferDesc;
-        vertexBufferDesc.size = sizeof(vertices);
-        vertexBufferDesc.usage = BufferUsage::VertexBuffer;
-        vertexBufferDesc.defaultState = ResourceState::VertexBuffer;
-        ComPtr<IBuffer> vertexBuffer = device->createBuffer(vertexBufferDesc, vertices);
-        REQUIRE(vertexBuffer != nullptr);
-        return vertexBuffer;
+        BufferDesc bufferDesc;
+        bufferDesc.size = sizeof(vertices);
+        bufferDesc.usage = BufferUsage::VertexBuffer;
+        bufferDesc.defaultState = ResourceState::VertexBuffer;
+        ComPtr<IBuffer> buffer = device->createBuffer(bufferDesc, vertices);
+        REQUIRE(buffer != nullptr);
+        return buffer;
     }
 
-    ComPtr<ITexture> createColorBuffer(IDevice* device)
+    ComPtr<ITexture> createColorBuffer()
     {
-        TextureDesc colorBufferDesc;
-        colorBufferDesc.type = TextureType::Texture2D;
-        colorBufferDesc.size.width = kWidth;
-        colorBufferDesc.size.height = kHeight;
-        colorBufferDesc.size.depth = 1;
-        colorBufferDesc.mipCount = 1;
-        colorBufferDesc.format = format;
-        colorBufferDesc.usage = TextureUsage::RenderTarget | TextureUsage::CopySource;
-        colorBufferDesc.defaultState = ResourceState::RenderTarget;
-        ComPtr<ITexture> colorBuffer = device->createTexture(colorBufferDesc, nullptr);
-        REQUIRE(colorBuffer != nullptr);
-        return colorBuffer;
+        TextureDesc textureDesc;
+        textureDesc.type = TextureType::Texture2D;
+        textureDesc.size.width = kWidth;
+        textureDesc.size.height = kHeight;
+        textureDesc.size.depth = 1;
+        textureDesc.mipCount = 1;
+        textureDesc.format = format;
+        textureDesc.usage = TextureUsage::RenderTarget | TextureUsage::CopySource;
+        textureDesc.defaultState = ResourceState::RenderTarget;
+        ComPtr<ITexture> texture = device->createTexture(textureDesc, nullptr);
+        REQUIRE(texture != nullptr);
+        return texture;
     }
 
     void createGraphicsResources()
@@ -715,8 +716,8 @@ struct ShaderCacheTestGraphics : ShaderCacheTest
         inputLayout = device->createInputLayout(inputLayoutDesc);
         REQUIRE(inputLayout != nullptr);
 
-        vertexBuffer = createVertexBuffer(device);
-        colorBuffer = createColorBuffer(device);
+        vertexBuffer = createVertexBuffer();
+        colorBuffer = createColorBuffer();
 
         TextureViewDesc colorBufferViewDesc = {};
         colorBufferViewDesc.format = format;
@@ -735,15 +736,9 @@ struct ShaderCacheTestGraphics : ShaderCacheTest
     void createGraphicsPipeline()
     {
         ComPtr<IShaderProgram> shaderProgram;
-        slang::ProgramLayout* slangReflection;
-        REQUIRE_CALL(loadGraphicsProgram(
-            device,
-            shaderProgram,
-            "test-shader-cache-graphics",
-            "vertexMain",
-            "fragmentMain",
-            slangReflection
-        ));
+        REQUIRE_CALL(
+            loadProgram(device, "test-shader-cache-graphics", {"vertexMain", "fragmentMain"}, shaderProgram.writeRef())
+        );
 
         ColorTargetDesc target;
         target.format = format;
@@ -771,7 +766,7 @@ struct ShaderCacheTestGraphics : ShaderCacheTest
         renderPass.colorAttachmentCount = 1;
         auto passEncoder = commandEncoder->beginRenderPass(renderPass);
 
-        auto rootObject = passEncoder->bindPipeline(renderPipeline);
+        passEncoder->bindPipeline(renderPipeline);
         RenderState state;
         state.viewports[0] = Viewport::fromSize(kWidth, kHeight);
         state.viewportCount = 1;
@@ -845,8 +840,6 @@ struct ShaderCacheTestGraphicsSplit : ShaderCacheTestGraphics
             composedProgram.writeRef()
         ));
 
-        slang::ProgramLayout* slangReflection = composedProgram->getLayout();
-
         std::vector<slang::IComponentType*> entryPoints;
         entryPoints.push_back(vertexEntryPoint);
         entryPoints.push_back(fragmentEntryPoint);
@@ -898,101 +891,53 @@ struct ShaderCacheTestGraphicsSplit : ShaderCacheTestGraphics
 };
 
 template<typename T>
-void runTest(GpuTestContext* ctx, DeviceType deviceType)
+void runTest(GpuTestContext* ctx)
 {
     std::string tempDirectory = getCaseTempDirectory();
     T test;
-    test.run(ctx, deviceType, tempDirectory);
+    test.run(ctx, tempDirectory);
 }
 
 // TODO
 // These tests are super expensive because they re-create devices.
 // This is needed because slang doesn't support reloading modules at this time.
 
-TEST_CASE("shader-cache-source-file")
+GPU_TEST_CASE("shader-cache-source-file", D3D12 | Vulkan | DontCreateDevice)
 {
-    runGpuTests(
-        runTest<ShaderCacheTestSourceFile>,
-        {
-            DeviceType::D3D12,
-            DeviceType::Vulkan,
-        }
-    );
+    runTest<ShaderCacheTestSourceFile>(ctx);
 }
 
-TEST_CASE("shader-cache-source-string")
+GPU_TEST_CASE("shader-cache-source-string", D3D12 | Vulkan | DontCreateDevice)
 {
-    runGpuTests(
-        runTest<ShaderCacheTestSourceString>,
-        {
-            DeviceType::D3D12,
-            DeviceType::Vulkan,
-        }
-    );
+    runTest<ShaderCacheTestSourceString>(ctx);
 }
 
-TEST_CASE("shader-cache-entry-point")
+GPU_TEST_CASE("shader-cache-entry-point", D3D12 | Vulkan | DontCreateDevice)
 {
-    runGpuTests(
-        runTest<ShaderCacheTestEntryPoint>,
-        {
-            DeviceType::D3D12,
-            DeviceType::Vulkan,
-        }
-    );
+    runTest<ShaderCacheTestEntryPoint>(ctx);
 }
 
-TEST_CASE("shader-cache-import-include")
+GPU_TEST_CASE("shader-cache-import-include", D3D12 | Vulkan | DontCreateDevice)
 {
-    runGpuTests(
-        runTest<ShaderCacheTestImportInclude>,
-        {
-            DeviceType::D3D12,
-            DeviceType::Vulkan,
-        }
-    );
+    runTest<ShaderCacheTestImportInclude>(ctx);
 }
 
-TEST_CASE("shader-cache-specialization")
+GPU_TEST_CASE("shader-cache-specialization", D3D12 | Vulkan | DontCreateDevice)
 {
-    runGpuTests(
-        runTest<ShaderCacheTestSpecialization>,
-        {
-            DeviceType::D3D12,
-            DeviceType::Vulkan,
-        }
-    );
+    runTest<ShaderCacheTestSpecialization>(ctx);
 }
 
-TEST_CASE("shader-cache-eviction")
+GPU_TEST_CASE("shader-cache-eviction", D3D12 | Vulkan | DontCreateDevice)
 {
-    runGpuTests(
-        runTest<ShaderCacheTestEviction>,
-        {
-            DeviceType::D3D12,
-            DeviceType::Vulkan,
-        }
-    );
+    runTest<ShaderCacheTestEviction>(ctx);
 }
 
-TEST_CASE("shader-cache-graphics")
+GPU_TEST_CASE("shader-cache-graphics", D3D12 | Vulkan | DontCreateDevice)
 {
-    runGpuTests(
-        runTest<ShaderCacheTestEviction>,
-        {
-            DeviceType::D3D12,
-            DeviceType::Vulkan,
-        }
-    );
+    runTest<ShaderCacheTestGraphics>(ctx);
 }
 
-TEST_CASE("shader-cache-graphics-split")
+GPU_TEST_CASE("shader-cache-graphics-split", D3D12 | Vulkan | DontCreateDevice)
 {
-    runGpuTests(
-        runTest<ShaderCacheTestGraphicsSplit>,
-        {
-            DeviceType::D3D12,
-            DeviceType::Vulkan,
-        }
-    );
+    runTest<ShaderCacheTestGraphicsSplit>(ctx);
 }
