@@ -27,10 +27,10 @@ struct TextureTest
 
     const void* expectedTextureData;
 
-    void init(IDevice* device, Format format, RefPtr<ValidationTextureFormatBase> validationFormat, TextureType type)
+    void init(IDevice* device_, Format format, RefPtr<ValidationTextureFormatBase> validationFormat_, TextureType type)
     {
-        this->device = device;
-        this->validationFormat = validationFormat;
+        this->device = device_;
+        this->validationFormat = validationFormat_;
 
         this->textureInfo = new TextureInfo();
         this->textureInfo->format = format;
@@ -44,15 +44,15 @@ struct TextureAccessTest : TextureTest
     bool readWrite;
 
     void init(
-        IDevice* device,
+        IDevice* device_,
         Format format,
-        RefPtr<ValidationTextureFormatBase> validationFormat,
+        RefPtr<ValidationTextureFormatBase> validationFormat_,
         TextureType type,
-        bool readWrite
+        bool readWrite_
     )
     {
-        TextureTest::init(device, format, validationFormat, type);
-        this->readWrite = readWrite;
+        TextureTest::init(device_, format, validationFormat_, type);
+        this->readWrite = readWrite_;
     }
 
     std::string getShaderEntryPoint()
@@ -75,6 +75,8 @@ struct TextureAccessTest : TextureTest
         case TextureType::TextureCube:
             name += "Cube";
             break;
+        default:
+            FAIL("Unsupported texture type");
         }
 
         return name;
@@ -114,8 +116,7 @@ struct TextureAccessTest : TextureTest
     void submitShaderWork(const char* entryPoint)
     {
         ComPtr<IShaderProgram> shaderProgram;
-        slang::ProgramLayout* slangReflection;
-        REQUIRE_CALL(loadComputeProgram(device, shaderProgram, "test-texture-types", entryPoint, slangReflection));
+        REQUIRE_CALL(loadProgram(device, "test-texture-types", entryPoint, shaderProgram.writeRef()));
 
         ComputePipelineDesc pipelineDesc = {};
         pipelineDesc.program = shaderProgram.get();
@@ -175,22 +176,21 @@ struct TextureAccessTest : TextureTest
         if (readWrite)
         {
             ComPtr<ISlangBlob> textureBlob;
-            size_t rowPitch;
-            size_t pixelSize;
-            REQUIRE_CALL(device->readTexture(texture, 0, 0, textureBlob.writeRef(), &rowPitch, &pixelSize));
+            SubresourceLayout layout;
+            REQUIRE_CALL(device->readTexture(texture, 0, 0, textureBlob.writeRef(), &layout));
             auto textureValues = (uint8_t*)textureBlob->getBufferPointer();
 
             ValidationTextureData textureResults;
             textureResults.extent = textureInfo->extent;
             textureResults.textureData = textureValues;
-            textureResults.pitches.x = (uint32_t)pixelSize;
-            textureResults.pitches.y = (uint32_t)rowPitch;
+            textureResults.pitches.x = (uint32_t)layout.colPitch;
+            textureResults.pitches.y = (uint32_t)layout.rowPitch;
             textureResults.pitches.z = textureResults.extent.height * textureResults.pitches.y;
 
             ValidationTextureData originalData;
             originalData.extent = textureInfo->extent;
             originalData.textureData = textureInfo->subresourceDatas.data();
-            originalData.pitches.x = (uint32_t)pixelSize;
+            originalData.pitches.x = (uint32_t)layout.colPitch;
             originalData.pitches.y = textureInfo->extent.width * originalData.pitches.x;
             originalData.pitches.z = textureInfo->extent.height * originalData.pitches.y;
 
@@ -333,15 +333,9 @@ struct RenderTargetTests : TextureTest
         REQUIRE(inputLayout != nullptr);
 
         ComPtr<IShaderProgram> shaderProgram;
-        slang::ProgramLayout* slangReflection;
-        REQUIRE_CALL(loadGraphicsProgram(
-            device,
-            shaderProgram,
-            "test-texture-types",
-            "vertexMain",
-            "fragmentMain",
-            slangReflection
-        ));
+        REQUIRE_CALL(
+            loadProgram(device, "test-texture-types", {"vertexMain", "fragmentMain"}, shaderProgram.writeRef())
+        );
 
         ColorTargetDesc target;
         target.format = textureInfo->format;
@@ -430,23 +424,22 @@ struct RenderTargetTests : TextureTest
     void checkTestResults()
     {
         ComPtr<ISlangBlob> textureBlob;
-        size_t rowPitch;
-        size_t pixelSize;
+        SubresourceLayout layout;
         if (sampleCount > 1)
         {
-            REQUIRE_CALL(device->readTexture(texture, 0, 0, textureBlob.writeRef(), &rowPitch, &pixelSize));
+            REQUIRE_CALL(device->readTexture(texture, 0, 0, textureBlob.writeRef(), &layout));
         }
         else
         {
-            REQUIRE_CALL(device->readTexture(renderTexture, 0, 0, textureBlob.writeRef(), &rowPitch, &pixelSize));
+            REQUIRE_CALL(device->readTexture(renderTexture, 0, 0, textureBlob.writeRef(), &layout));
         }
         auto textureValues = (float*)textureBlob->getBufferPointer();
 
         ValidationTextureData textureResults;
         textureResults.extent = textureInfo->extent;
         textureResults.textureData = textureValues;
-        textureResults.pitches.x = (uint32_t)pixelSize;
-        textureResults.pitches.y = (uint32_t)rowPitch;
+        textureResults.pitches.x = (uint32_t)layout.colPitch;
+        textureResults.pitches.y = (uint32_t)layout.rowPitch;
         textureResults.pitches.z = textureResults.extent.height * textureResults.pitches.y;
 
         validateTextureValues(textureResults);
