@@ -413,14 +413,10 @@ void TextureData::checkMipLevelsEqual(
     const Subresource& thisSubresource = getSubresource(thisLayer, thisMipLevel);
     const SubresourceLayout& thisLayout = thisSubresource.layout;
 
+    ComPtr<ISlangBlob> textureBlob;
     SubresourceLayout textureLayout;
-    REQUIRE_CALL(textureImpl->getSubresourceLayout(textureMipLevel, kDefaultAlignment, &textureLayout));
-
-    ComPtr<ISlangBlob> blob;
-    Size rowPitch;
-    REQUIRE_CALL(
-        textureImpl->getDevice()->readTexture(textureImpl, textureLayer, textureMipLevel, blob.writeRef(), &rowPitch)
-    );
+    REQUIRE_CALL(textureImpl->getDevice()
+                     ->readTexture(textureImpl, textureLayer, textureMipLevel, textureBlob.writeRef(), &textureLayout));
 
     // For compressed textures, raise error if attempting to check non-aligned blocks
     if (formatInfo.blockWidth > 1)
@@ -453,7 +449,7 @@ void TextureData::checkMipLevelsEqual(
             thisSubresource.data.get(),
             thisLayout,
             thisOffset,
-            blob->getBufferPointer(),
+            textureBlob->getBufferPointer(),
             textureLayout,
             textureOffset,
             textureExtent
@@ -496,7 +492,7 @@ void TextureData::checkMipLevelsEqual(
                             thisSubresource.data.get(),
                             thisLayout,
                             {offsetX, offsetY, offsetZ},
-                            blob->getBufferPointer(),
+                            textureBlob->getBufferPointer(),
                             textureLayout,
                             {offsetX, offsetY, offsetZ},
                             {sizeX, sizeY, sizeZ}
@@ -533,21 +529,17 @@ void TextureData::checkSliceEqual(
     const Subresource& thisSubresource = getSubresource(thisLayer, thisMipLevel);
     const SubresourceLayout& thisLayout = thisSubresource.layout;
 
+    ComPtr<ISlangBlob> textureBlob;
     SubresourceLayout textureLayout;
-    REQUIRE_CALL(textureImpl->getSubresourceLayout(textureMipLevel, kDefaultAlignment, &textureLayout));
-
-    ComPtr<ISlangBlob> blob;
-    Size rowPitch;
-    REQUIRE_CALL(
-        textureImpl->getDevice()->readTexture(textureImpl, textureLayer, textureMipLevel, blob.writeRef(), &rowPitch)
-    );
+    REQUIRE_CALL(textureImpl->getDevice()
+                     ->readTexture(textureImpl, textureLayer, textureMipLevel, textureBlob.writeRef(), &textureLayout));
 
     // Calculate overall dimensions in blocks rather than pixels to handle compressed textures.
     uint32_t rowCount = thisLayout.rowCount;
     uint32_t colCount = thisLayout.size.width / formatInfo.blockWidth;
 
     uint8_t* thisData = thisSubresource.data.get();
-    uint8_t* textureData = (uint8_t*)blob->getBufferPointer();
+    uint8_t* textureData = (uint8_t*)textureBlob->getBufferPointer();
 
     // Iterate rows
     for (uint32_t row = 0; row < rowCount; row++)
@@ -597,12 +589,14 @@ void TextureData::checkEqualFloat(ITexture* texture, float epsilon) const
         {
             const Subresource& sr = getSubresource(layer, mip);
 
-            ComPtr<ISlangBlob> blob;
-            Size rowPitch;
-            REQUIRE_CALL(textureImpl->getDevice()->readTexture(textureImpl, layer, mip, blob.writeRef(), &rowPitch));
+            ComPtr<ISlangBlob> textureBlob;
+            SubresourceLayout textureLayout;
+            REQUIRE_CALL(
+                textureImpl->getDevice()->readTexture(textureImpl, layer, mip, textureBlob.writeRef(), &textureLayout)
+            );
 
             const uint8_t* expectedSlice = sr.data.get();
-            const uint8_t* actualSlice = (uint8_t*)blob->getBufferPointer();
+            const uint8_t* actualSlice = (uint8_t*)textureBlob->getBufferPointer();
 
             for (uint32_t slice = 0; slice < sr.layout.size.depth; slice++)
             {
@@ -632,11 +626,11 @@ void TextureData::checkEqualFloat(ITexture* texture, float epsilon) const
                     CHECK(isEqual);
 
                     expectedRow += sr.layout.rowPitch;
-                    actualRow += rowPitch;
+                    actualRow += textureLayout.rowPitch;
                 }
 
                 expectedSlice += sr.layout.slicePitch;
-                actualSlice += rowPitch * sr.layout.rowCount;
+                actualSlice += textureLayout.rowPitch * sr.layout.rowCount;
             }
         }
     }
@@ -686,13 +680,16 @@ void TextureData::clearUint(const uint32_t clearValue[4]) const
     }
 }
 
-void TextureData::clearUint(uint32_t layer, uint32_t mip, const uint32_t clearValue[4]) const
+void TextureData::clearUint(uint32_t layer, uint32_t mip, const uint32_t clearValue_[4]) const
 {
     const Subresource& subresource = getSubresource(layer, mip);
     FormatConversionFuncs funcs = getFormatConversionFuncs(desc.format);
     SLANG_RHI_ASSERT(funcs.packIntFunc);
+    SLANG_RHI_ASSERT(funcs.clampIntFunc);
     size_t pixelSize = formatInfo.blockSizeInBytes / formatInfo.pixelsPerBlock;
     uint8_t pixelData[16];
+    uint32_t clearValue[4] = {clearValue_[0], clearValue_[1], clearValue_[2], clearValue_[3]};
+    funcs.clampIntFunc(clearValue);
     funcs.packIntFunc(clearValue, pixelData);
     for (uint32_t depth = 0; depth < subresource.layout.size.depth; depth++)
     {
@@ -710,19 +707,14 @@ void TextureData::clearUint(uint32_t layer, uint32_t mip, const uint32_t clearVa
 
 void TextureData::clearSint(const int32_t clearValue[4]) const
 {
-    uint32_t clearValueUint[4];
-    truncateBySintFormat(desc.format, reinterpret_cast<const uint32_t*>(clearValue), clearValueUint);
-    clearUint(clearValueUint);
+    clearUint(reinterpret_cast<const uint32_t*>(clearValue));
 }
 
 void TextureData::clearSint(uint32_t layer, uint32_t mip, const int32_t clearValue[4]) const
 {
-    uint32_t clearValueUint[4];
-    truncateBySintFormat(desc.format, reinterpret_cast<const uint32_t*>(clearValue), clearValueUint);
-    clearUint(layer, mip, clearValueUint);
+    clearUint(layer, mip, reinterpret_cast<const uint32_t*>(clearValue));
 }
 
-
 //----------------------------------------------------------
 // TextureTestOptions
 //----------------------------------------------------------
@@ -1221,6 +1213,11 @@ void TextureTestOptions::filterFormat(int state, TextureTestVariant variant)
         if (!is_set(support, FormatSupport::Texture))
             return;
 
+        // Skip if format doesn't support UAV access.
+        if (is_set(testTexture.desc.usage, TextureUsage::UnorderedAccess) &&
+            (!is_set(support, FormatSupport::ShaderUavLoad) || !is_set(support, FormatSupport::ShaderUavStore)))
+            return;
+
         const FormatInfo& info = getFormatInfo(format);
 
         // Metal doesn't support writing into depth textures.
@@ -1268,28 +1265,28 @@ void TextureTestOptions::applyTextureSize(int state, TextureTestVariant variant)
         // Make adjustments for power of 2 sizes. We need to increment dimensions
         // that aren't only 1 pixel wide until they're a non-power of 2 multiple
         // of the block size.
-        for (auto& testTexture : variant.descriptors)
+        for (auto& testDescA : variant.descriptors)
         {
-            const FormatInfo& info = getFormatInfo(testTexture.desc.format);
+            const FormatInfo& info = getFormatInfo(testDescA.desc.format);
             if (!info.supportsNonPowerOf2)
                 return;
 
-            for (auto& testTexture : variant.descriptors)
+            for (auto& testDescB : variant.descriptors)
             {
-                if (testTexture.desc.size.width > 1)
+                if (testDescB.desc.size.width > 1)
                 {
-                    while (math::isPowerOf2(testTexture.desc.size.width))
-                        testTexture.desc.size.width += info.blockWidth;
+                    while (math::isPowerOf2(testDescB.desc.size.width))
+                        testDescB.desc.size.width += info.blockWidth;
                 }
-                if (testTexture.desc.size.height > 1)
+                if (testDescB.desc.size.height > 1)
                 {
-                    while (math::isPowerOf2(testTexture.desc.size.height))
-                        testTexture.desc.size.height += info.blockHeight;
+                    while (math::isPowerOf2(testDescB.desc.size.height))
+                        testDescB.desc.size.height += info.blockHeight;
                 }
-                if (testTexture.desc.size.depth > 1)
+                if (testDescB.desc.size.depth > 1)
                 {
-                    while (math::isPowerOf2(testTexture.desc.size.depth))
-                        testTexture.desc.size.depth++;
+                    while (math::isPowerOf2(testDescB.desc.size.depth))
+                        testDescB.desc.size.depth++;
                 }
             }
         }
