@@ -289,6 +289,8 @@ SLANG_HIERARCHICAL_ENUM(ArtifactStyle, SLANG_ARTIFACT_STYLE, SLANG_ARTIFACT_STYL
         return Desc::make(Kind::Source, Payload::C, Style::Kernel, 0);
     case SLANG_CPP_SOURCE:
         return Desc::make(Kind::Source, Payload::Cpp, Style::Kernel, 0);
+    case SLANG_CPP_HEADER:
+        return Desc::make(Kind::Source, Payload::Cpp, Style::Kernel, 0);
     case SLANG_HOST_CPP_SOURCE:
         return Desc::make(Kind::Source, Payload::Cpp, Style::Host, 0);
     case SLANG_CPP_PYTORCH_BINDING:
@@ -303,6 +305,8 @@ SLANG_HIERARCHICAL_ENUM(ArtifactStyle, SLANG_ARTIFACT_STYLE, SLANG_ARTIFACT_STYL
         return Desc::make(Kind::HostCallable, Payload::HostCPU, Style::Kernel, 0);
     case SLANG_CUDA_SOURCE:
         return Desc::make(Kind::Source, Payload::CUDA, Style::Kernel, 0);
+    case SLANG_CUDA_HEADER:
+        return Desc::make(Kind::Source, Payload::CUDA, Style::Kernel, 0);
         // TODO(JS):
         // Not entirely clear how best to represent PTX here. We could mark as 'Assembly'.
         // Saying it is 'Executable' implies it is Binary (which PTX isn't). Executable also
@@ -325,6 +329,19 @@ SLANG_HIERARCHICAL_ENUM(ArtifactStyle, SLANG_ARTIFACT_STYLE, SLANG_ARTIFACT_STYL
         return Desc::make(Kind::Assembly, Payload::WGSL_SPIRV, Style::Kernel, 0);
     case SLANG_WGSL_SPIRV:
         return Desc::make(Kind::ObjectCode, Payload::WGSL_SPIRV, Style::Kernel, 0);
+
+    case SLANG_HOST_VM:
+        return Desc::make(Kind::ObjectCode, Payload::UniversalCPU, Style::Host, 0);
+
+    case SLANG_HOST_OBJECT_CODE:
+        return Desc::make(Kind::ObjectCode, Payload::UniversalCPU, Style::Host, 0);
+
+    case SLANG_HOST_LLVM_IR:
+        return Desc::make(Kind::Assembly, Payload::LLVMIR, Style::Host, 0);
+
+    case SLANG_SHADER_LLVM_IR:
+        return Desc::make(Kind::Assembly, Payload::LLVMIR, Style::Kernel, 0);
+
     default:
         break;
     }
@@ -584,7 +601,8 @@ static const KindExtension g_cpuKindExts[] = {
 
 /* static */ bool ArtifactDescUtil::isCpuLikeTarget(const ArtifactDesc& desc)
 {
-    if (isDerivedFrom(desc.kind, ArtifactKind::CompileBinary))
+    if (isDerivedFrom(desc.kind, ArtifactKind::CompileBinary) ||
+        isDerivedFrom(desc.kind, ArtifactKind::ObjectCode))
     {
         return isDerivedFrom(desc.payload, ArtifactPayload::CPULike);
     }
@@ -593,6 +611,11 @@ static const KindExtension g_cpuKindExts[] = {
         // We'll assume C/C++ are targetting CPU, although that is perhaps somewhat arguable.
         return desc.payload == Payload::C || desc.payload == Payload::Cpp;
     }
+    else if (isDerivedFrom(desc.kind, ArtifactKind::Assembly))
+    {
+        // We'll assume LLVM IR is targeting CPU (even more arguable...)
+        return desc.payload == Payload::LLVMIR;
+    }
 
     return false;
 }
