@@ -512,6 +512,22 @@ SlangResult DXCDownstreamCompiler::compile(const CompileOptions& inOptions, IArt
         break;
     }
 
+    switch (options.denormalModeFp32)
+    {
+    default:
+    case CompileOptions::FloatingPointDenormalMode::Any:
+        break;
+
+    case CompileOptions::FloatingPointDenormalMode::Preserve:
+        args.add(L"-denorm");
+        args.add(L"preserve");
+        break;
+
+    case CompileOptions::FloatingPointDenormalMode::FlushToZero:
+        args.add(L"-denorm");
+        args.add(L"ftz");
+        break;
+    }
 
     switch (options.optimizationLevel)
     {
@@ -537,9 +553,11 @@ SlangResult DXCDownstreamCompiler::compile(const CompileOptions& inOptions, IArt
     case DebugInfoType::None:
         break;
 
+    // DXC only supports -Zi (full debug info) or -Zs (small PDB without debug metadata).
+    // Testing shows -Zs generates NO debug metadata at all (!DIFile, !DICompileUnit, etc.)
+    // For all debug levels, we use -Zi to generate full debug info
     default:
         args.add(L"-Zi");
-        break;
     }
 
     // Slang strives to produce correct code, and by default
