@@ -127,27 +127,35 @@ protected:
         nvrtcProgram m_program;
     };
 
-    SlangResult _findCUDAIncludePath(String& outIncludePath);
+    SlangResult _findCUDAIncludePath(String& outPath);
     SlangResult _getCUDAIncludePath(String& outIncludePath);
 
     SlangResult _findOptixIncludePath(String& outIncludePath);
     SlangResult _getOptixIncludePath(String& outIncludePath);
 
-    SlangResult _maybeAddHalfSupport(const CompileOptions& options, CommandLine& ioCmdLine);
-    SlangResult _maybeAddOptixSupport(const CompileOptions& options, CommandLine& ioCmdLine);
+    SlangResult _maybeAddHalfSupport(
+        const CompileOptions& options,
+        CommandLine& ioCmdLine,
+        IArtifactDiagnostics* diagnostics);
+    SlangResult _maybeAddOptixSupport(
+        const CompileOptions& options,
+        CommandLine& ioCmdLine,
+        IArtifactDiagnostics* diagnostics);
 
 #define SLANG_NVTRC_MEMBER_FUNCS(ret, name, params) ret(*m_##name) params;
 
     SLANG_NVRTC_FUNCS(SLANG_NVTRC_MEMBER_FUNCS);
 
-    // Holds list of paths passed in where cuda_fp16.h is found. Does *NOT* include cuda_fp16.h.
+    // Holds list of paths passed in where cuda_fp16.h is found. Does *NOT*
+    // include cuda_fp16.h.
     List<String> m_cudaFp16FoundPaths;
 
     bool m_cudaIncludeSearched = false;
     // Holds location of where include (for cuda_fp16.h) is found.
     String m_cudaIncludePath;
 
-    // Holds list of paths passed in where optix.h is found. Does *NOT* include optix.h.
+    // Holds list of paths passed in where optix.h is found. Does *NOT* include
+    // optix.h.
     List<String> m_optixFoundPaths;
 
     bool m_optixIncludeSearched = false;
@@ -234,7 +242,8 @@ static SlangResult _parseNVRTCLine(
     List<UnownedStringSlice> split;
     if (_hasDriveLetter(line))
     {
-        // The drive letter has :, which confuses things, so skip that and then fix up first entry
+        // The drive letter has :, which confuses things, so skip that and then fix
+        // up first entry
         UnownedStringSlice lineWithoutDrive(line.begin() + 2, line.end());
         StringUtil::split(lineWithoutDrive, ':', split);
         split[0] = UnownedStringSlice(line.begin(), split[0].end());
@@ -246,7 +255,8 @@ static SlangResult _parseNVRTCLine(
 
     if (split.getCount() >= 3)
     {
-        // tests/cuda/cuda-compile.cu(7): warning: variable "c" is used before its value is set
+        // tests/cuda/cuda-compile.cu(7): warning: variable "c" is used before its
+        // value is set
         const auto split1 = split[1].trim();
 
         Severity severity = Severity::Unknown;
@@ -267,8 +277,8 @@ static SlangResult _parseNVRTCLine(
             //
             // Anything ending with " warning:" or " error:" in effect.
 
-            // We can expand to include character after as this is split1, as must be followed by at
-            // a minimum : (as the split has at least 3 parts).
+            // We can expand to include character after as this is split1, as must be
+            // followed by at a minimum : (as the split has at least 3 parts).
             const UnownedStringSlice expandSplit1(split1.begin(), split1.end() + 1);
 
             if (expandSplit1.endsWith(toSlice(" error:")))
@@ -297,12 +307,12 @@ static SlangResult _parseNVRTCLine(
             return SLANG_OK;
         }
 
-        // TODO(JS): Note here if it's not possible to determine a line as being the main
-        // diagnostics we fall through to it potentially being a note.
+        // TODO(JS): Note here if it's not possible to determine a line as being the
+        // main diagnostics we fall through to it potentially being a note.
         //
-        // That could mean a valid diagnostic (from NVRTCs point of view) is ignored/noted, because
-        // this code can't parse it. Ideally that situation would lead to an error such that we can
-        // detect and things will fail.
+        // That could mean a valid diagnostic (from NVRTCs point of view) is
+        // ignored/noted, because this code can't parse it. Ideally that situation
+        // would lead to an error such that we can detect and things will fail.
         //
         // So we might want to revisit this determination in the future.
     }
@@ -320,8 +330,8 @@ static SlangResult _parseNVRTCLine(
     return SLANG_OK;
 }
 
-/* An implementation of Path::Visitor that can be used for finding NVRTC shared library
- * installations. */
+/* An implementation of Path::Visitor that can be used for finding NVRTC shared
+ * library installations. */
 struct NVRTCPathVisitor : Path::Visitor
 {
     struct Candidate
@@ -364,7 +374,6 @@ struct NVRTCPathVisitor : Path::Visitor
     }
     void sortCandidates() { m_candidates.sort(_orderCandiate); }
 
-
 #if SLANG_WINDOWS_FAMILY
     SlangResult getVersion(const UnownedStringSlice& filename, SemanticVersion& outVersion)
     {
@@ -392,7 +401,8 @@ struct NVRTCPathVisitor : Path::Visitor
             StringUtil::split(versionSlice, '_', slices);
             if (slices.getCount() >= 2)
             {
-                // We don't bother checking for error here, if it's not parsable, it will be 0
+                // We don't bother checking for error here, if it's not parsable, it
+                // will be 0
                 StringUtil::parseInt(slices[1], patch);
             }
             majorMinorSlice = slices[0];
@@ -420,8 +430,8 @@ struct NVRTCPathVisitor : Path::Visitor
 #else
     // How the path is constructed depends on platform
     // https://docs.nvidia.com/cuda/nvrtc/index.html
-    // TODO(JS): Handle version number depending on the platform - it's different for
-    // Windows/OSX/Linux
+    // TODO(JS): Handle version number depending on the platform - it's different
+    // for Windows/OSX/Linux
     SlangResult getVersion(const UnownedStringSlice& filename, SemanticVersion& outVersion)
     {
         SLANG_UNUSED(filename);
@@ -448,7 +458,6 @@ struct NVRTCPathVisitor : Path::Visitor
                 }
             }
 
-
             if (filename.getLength() >= m_prefix.getLength() &&
                 filename.subString(0, m_prefix.getLength())
                     .caseInsensitiveEquals(m_prefix.getUnownedSlice()))
@@ -460,9 +469,9 @@ struct NVRTCPathVisitor : Path::Visitor
                     version = SemanticVersion();
                 }
 
-                // We may want to add multiple versions, if they are in different locations - as
-                // there may be multiple entries in the PATH, and only one works. We'll only know
-                // which works by loading
+                // We may want to add multiple versions, if they are in different
+                // locations - as there may be multiple entries in the PATH, and only
+                // one works. We'll only know which works by loading
 
 #if 0
                 // We already found this version, so let's not add it again
@@ -538,10 +547,11 @@ static UnownedStringSlice _getNVRTCBaseName()
 #endif
 }
 
-// Candidates are in m_candidates list. Will be ordered from the oldest to newest (in version
-// number)
+// Candidates are in m_candidates list. Will be ordered from the oldest to
+// newest (in version number)
 static SlangResult _findNVRTC(NVRTCPathVisitor& visitor)
 {
+#if SLANG_WINDOWS_FAMILY && SLANG_PTR_IS_64
     // First try the instance path (if supported on platform)
     {
         StringBuilder instancePath;
@@ -551,16 +561,23 @@ static SlangResult _findNVRTC(NVRTCPathVisitor& visitor)
         }
     }
 
-    // If we don't have a candidate try CUDA_PATH
+    // If we don't have a candidate, try CUDA_PATH
     if (!visitor.hasCandidates())
     {
         StringBuilder buf;
-        if (!SLANG_SUCCEEDED(PlatformUtil::getEnvironmentVariable(
+        if (SLANG_SUCCEEDED(PlatformUtil::getEnvironmentVariable(
                 UnownedStringSlice::fromLiteral("CUDA_PATH"),
                 buf)))
         {
             // Look for candidates in the directory
             visitor.findInDirectory(Path::combine(buf, "bin"));
+
+            // Since CUDA 13.0, cuda dlls have been moved to bin\x64
+            String bin64Dir = Path::combine(buf, "bin\\x64");
+            if (File::exists(bin64Dir))
+            {
+                visitor.findInDirectory(bin64Dir);
+            }
         }
     }
 
@@ -583,22 +600,30 @@ static SlangResult _findNVRTC(NVRTCPathVisitor& visitor)
             // We are going to search the paths in order
             for (const auto& path : paths)
             {
-                // PATH can have the same path multiple times. If we have already searched this
-                // path, we don't need to again
+                // PATH can have the same path multiple times. If we have already
+                // searched this path, we don't need to again
                 if (!pool.has(path))
                 {
                     pool.add(path);
 
                     Path::split(path, splitPath);
 
-                    // We could search every path, but here we restrict to paths that look like CUDA
-                    // installations. It's a path that contains a CUDA directory and has bin
+                    // We could search every path, but here we restrict to paths that look
+                    // like CUDA installations. It's a path that contains a CUDA directory
+                    // and has bin
                     if (splitPath.indexOf("CUDA") >= 0 &&
                         splitPath[splitPath.getCount() - 1].caseInsensitiveEquals(
                             UnownedStringSlice::fromLiteral("bin")))
                     {
                         // Okay lets search it
                         visitor.findInDirectory(path);
+
+                        // Since CUDA 13.0, cuda dlls have been moved to bin\x64
+                        String bin64Dir = Path::combine(buf, "bin\\x64");
+                        if (File::exists(bin64Dir))
+                        {
+                            visitor.findInDirectory(bin64Dir);
+                        }
                     }
                 }
             }
@@ -609,12 +634,16 @@ static SlangResult _findNVRTC(NVRTCPathVisitor& visitor)
     visitor.sortCandidates();
 
     return SLANG_OK;
+#else
+    SLANG_UNUSED(visitor);
+
+    return SLANG_E_NOT_FOUND;
+#endif
 }
 
 static const UnownedStringSlice g_fp16HeaderName = UnownedStringSlice::fromLiteral("cuda_fp16.h");
 static const UnownedStringSlice g_optixHeaderName = UnownedStringSlice::fromLiteral("optix.h");
 
-
 SlangResult _findFileInIncludePath(
     const String& path,
     const UnownedStringSlice& filename,
@@ -651,8 +680,8 @@ SlangResult NVRTCDownstreamCompiler::_findCUDAIncludePath(String& outPath)
 {
     outPath = String();
 
-    // Try looking up from a symbol. This will work as long as the nvrtc is loaded somehow from a
-    // dll/sharedlibrary And the header is included from there
+    // Try looking up from a symbol. This will work as long as the nvrtc is loaded
+    // somehow from a dll/sharedlibrary And the header is included from there
     {
         String libPath = SharedLibraryUtils::getSharedLibraryFileName((void*)m_nvrtcCreateProgram);
         if (libPath.getLength())
@@ -664,26 +693,77 @@ SlangResult NVRTCDownstreamCompiler::_findCUDAIncludePath(String& outPath)
                 return SLANG_OK;
             }
 
-            // See if the shared library is in the SDK, as if so we know how to find the includes
-            // TODO(JS):
-            // This directory structure is correct for windows perhaps could be different elsewhere.
+            // See if the shared library is in the CUDA SDK install path as, if so, we know how to
+            // find the includes from there
             {
                 List<UnownedStringSlice> pathSlices;
                 Path::split(parentPath.getUnownedSlice(), pathSlices);
-
-                // This -2 split holds the version number.
                 const auto pathSplitCount = pathSlices.getCount();
-                if (pathSplitCount >= 3 && pathSlices[pathSplitCount - 1] == toSlice("bin") &&
-                    pathSlices[pathSplitCount - 3] == toSlice("CUDA"))
-                {
-                    // We want to make sure that one of these paths is CUDA...
-                    const auto sdkPath = Path::getParentDirectory(parentPath);
 
-                    if (SLANG_SUCCEEDED(_findFileInIncludePath(sdkPath, g_fp16HeaderName, outPath)))
+#if SLANG_WINDOWS_FAMILY
+                // Expected sequence on Windows:
+                // CUDA / <version> / bin [ / x64 ]
+                if (pathSplitCount >= 3)
+                {
+                    // Figure out if this is 13.0+ and in an x64 subdirectory
+                    auto pathOffset = 0;
+                    if (pathSplitCount >= 4 && pathSlices[pathSplitCount - 1] == toSlice("x64"))
                     {
-                        return SLANG_OK;
+                        pathOffset = 1;
+                    }
+                    // Check for CUDA directory pattern
+                    if (pathSlices[pathSplitCount - pathOffset - 1] == toSlice("bin") &&
+                        pathSlices[pathSplitCount - pathOffset - 3] == toSlice("CUDA"))
+                    {
+                        auto sdkPath = Path::getParentDirectory(parentPath);
+                        if (pathOffset == 1)
+                        {
+                            sdkPath = Path::getParentDirectory(sdkPath);
+                        }
+
+                        if (SLANG_SUCCEEDED(
+                                _findFileInIncludePath(sdkPath, g_fp16HeaderName, outPath)))
+                        {
+                            return SLANG_OK;
+                        }
                     }
                 }
+#else
+                // Expected sequence on Linux for cuda 13+:
+                // cuda-<version> / targets / <arch> / lib
+                if (pathSplitCount >= 4)
+                {
+                    if (pathSlices[pathSplitCount - 1] == toSlice("lib") &&
+                        pathSlices[pathSplitCount - 3] == toSlice("targets") &&
+                        pathSlices[pathSplitCount - 4].subString(0, 4) == toSlice("cuda"))
+                    {
+                        // Headers are under <arch> / include
+                        const auto sdkPath = Path::getParentDirectory(parentPath);
+
+                        if (SLANG_SUCCEEDED(
+                                _findFileInIncludePath(sdkPath, g_fp16HeaderName, outPath)))
+                        {
+                            return SLANG_OK;
+                        }
+                    }
+                }
+                // Expected sequence on Linux for cuda 12:
+                // cuda-<version> / lib[64]
+                if (pathSplitCount >= 2)
+                {
+                    if (pathSlices[pathSplitCount - 1].subString(0, 3) == toSlice("lib") &&
+                        pathSlices[pathSplitCount - 2].subString(0, 4) == toSlice("cuda"))
+                    {
+                        const auto sdkPath = Path::getParentDirectory(parentPath);
+
+                        if (SLANG_SUCCEEDED(
+                                _findFileInIncludePath(sdkPath, g_fp16HeaderName, outPath)))
+                        {
+                            return SLANG_OK;
+                        }
+                    }
+                }
+#endif
             }
         }
     }
@@ -706,10 +786,13 @@ SlangResult NVRTCDownstreamCompiler::_findCUDAIncludePath(String& outPath)
     }
 
 #if SLANG_LINUX_FAMILY
-    // Try /usr/include
-    {
-        String includePath = "/usr/include";
+    List<String> candidatePaths;
+    candidatePaths.add("/usr/local/include");
+    candidatePaths.add("/usr/local/cuda/include");
+    candidatePaths.add("/usr/include");
 
+    for (const String& includePath : candidatePaths)
+    {
         if (File::exists(Path::combine(includePath, g_fp16HeaderName)))
         {
             outPath = includePath;
@@ -717,7 +800,6 @@ SlangResult NVRTCDownstreamCompiler::_findCUDAIncludePath(String& outPath)
         }
     }
 #endif
-
     return SLANG_E_NOT_FOUND;
 }
 
@@ -740,6 +822,32 @@ SlangResult NVRTCDownstreamCompiler::_findOptixIncludePath(String& outPath)
 {
     outPath = String();
 
+    // First try to find OptiX headers in the local external/optix-dev/include
+    // directory relative to the executable path
+    {
+        StringBuilder instancePathBuilder;
+        if (SLANG_SUCCEEDED(PlatformUtil::getInstancePath(instancePathBuilder)))
+        {
+            // Get executable path, then go up to project root
+            // instancePathBuilder already contains the bin directory path
+            // Executable is in build/Debug/bin or build/Release/bin
+            // Go up 3 levels: bin -> Debug/Release -> build -> project root
+            String binPath = instancePathBuilder;
+            String buildTypeDir = Path::getParentDirectory(binPath);
+            String buildDir = Path::getParentDirectory(buildTypeDir);
+            String projectRoot = Path::getParentDirectory(buildDir);
+
+            String localOptixPath =
+                Path::combine(Path::combine(projectRoot, "external/optix-dev"), "include");
+            String optixHeader = Path::combine(localOptixPath, g_optixHeaderName);
+
+            if (File::exists(optixHeader))
+            {
+                outPath = localOptixPath;
+                return SLANG_OK;
+            }
+        }
+    }
     List<String> rootPaths;
 
 #if SLANG_WINDOWS_FAMILY
@@ -786,10 +894,10 @@ SlangResult NVRTCDownstreamCompiler::_findOptixIncludePath(String& outPath)
             // Paths are expected to look like ".\OptiX SDK X.X.X"
             auto versionString = path.subString(path.lastIndexOf(' ') + 1, path.getLength());
 #else
-            // Paths are expected to look like "./NVIDIA-OptiX-SDK-X.X.X-suffix"
-            auto versionString = path.subString(0, path.lastIndexOf('-'));
-            versionString =
-                versionString.subString(path.lastIndexOf('-') + 1, versionString.getLength());
+            // Paths are expected to look like "NVIDIA-OptiX-SDK-X.X.X-suffix"
+            const UnownedStringSlice pathPrefix("NVIDIA-OptiX-SDK-");
+            auto versionString = path.tail(pathPrefix.getLength());
+            versionString = versionString.head(versionString.indexOf('-'));
 #endif
             if (SLANG_SUCCEEDED(SemanticVersion::parse(versionString, '.', optixPath.version)))
             {
@@ -847,7 +955,8 @@ SlangResult NVRTCDownstreamCompiler::_getOptixIncludePath(String& outPath)
 
 SlangResult NVRTCDownstreamCompiler::_maybeAddHalfSupport(
     const DownstreamCompileOptions& options,
-    CommandLine& ioCmdLine)
+    CommandLine& ioCmdLine,
+    IArtifactDiagnostics* diagnostics)
 {
     if ((options.flags & DownstreamCompileOptions::Flag::EnableFloat16) == 0)
     {
@@ -881,7 +990,14 @@ SlangResult NVRTCDownstreamCompiler::_maybeAddHalfSupport(
     }
 
     String includePath;
-    SLANG_RETURN_ON_FAIL(_getCUDAIncludePath(includePath));
+    if (SLANG_FAILED(_getCUDAIncludePath(includePath)))
+    {
+        String msg = "Failed to locate CUDA headers (cuda_fp16.h) required for half/float16 "
+                     "support. Please install CUDA Toolkit or set CUDA_PATH environment variable.";
+        diagnostics->setRaw(SliceUtil::asCharSlice(msg));
+        diagnostics->requireErrorDiagnostic();
+        return SLANG_E_NOT_FOUND;
+    }
 
     // Add the found include path
     ioCmdLine.addArg("-I");
@@ -894,7 +1010,8 @@ SlangResult NVRTCDownstreamCompiler::_maybeAddHalfSupport(
 
 SlangResult NVRTCDownstreamCompiler::_maybeAddOptixSupport(
     const DownstreamCompileOptions& options,
-    CommandLine& ioCmdLine)
+    CommandLine& ioCmdLine,
+    IArtifactDiagnostics* diagnostics)
 {
     // First check if we know if one of the include paths contains optix.h
     for (const auto& includePath : options.includePaths)
@@ -923,7 +1040,14 @@ SlangResult NVRTCDownstreamCompiler::_maybeAddOptixSupport(
     }
 
     String includePath;
-    SLANG_RETURN_ON_FAIL(_getOptixIncludePath(includePath));
+    if (SLANG_FAILED(_getOptixIncludePath(includePath)))
+    {
+        String msg = "Failed to locate OptiX headers (optix.h) required for OptiX ray tracing "
+                     "support. Please install OptiX SDK.";
+        diagnostics->setRaw(SliceUtil::asCharSlice(msg));
+        diagnostics->requireErrorDiagnostic();
+        return SLANG_E_NOT_FOUND;
+    }
 
     // Add the found include path
     ioCmdLine.addArg("-I");
@@ -954,6 +1078,11 @@ SlangResult NVRTCDownstreamCompiler::compile(
 
     IArtifact* sourceArtifact = options.sourceArtifacts[0];
 
+    // Create artifact and diagnostics early so we can report errors from any stage
+    auto artifact = ArtifactUtil::createArtifactForCompileTarget(options.targetType);
+    auto diagnostics = ArtifactDiagnostics::create();
+    ArtifactUtil::addAssociated(artifact, diagnostics);
+
     CommandLine cmdLine;
 
     // --dopt option is only available in CUDA 11.7 and later
@@ -1028,7 +1157,24 @@ SlangResult NVRTCDownstreamCompiler::compile(
         cmdLine.addArg(asString(include));
     }
 
-    SLANG_RETURN_ON_FAIL(_maybeAddHalfSupport(options, cmdLine));
+    if (SLANG_FAILED(_maybeAddHalfSupport(options, cmdLine, diagnostics)))
+    {
+        diagnostics->setResult(SLANG_FAIL);
+        *outArtifact = artifact.detach();
+        return SLANG_FAIL;
+    }
+
+    // Previously, we relied on `_maybeAddHalfSupport` to find the cuda include path, but
+    // that was incorrect. Because if there is no half support needed, we will not add the
+    // CUDA include path at all. So we always try to find and add the CUDA include path here.
+    String includePath;
+    if (_getCUDAIncludePath(includePath) == SLANG_OK)
+    {
+        // Add the found include path
+        cmdLine.addArg("-I");
+        cmdLine.addArg(includePath);
+    }
+
 
     // Neither of these options are strictly required, for general use of nvrtc,
     // but are enabled to make use withing Slang work more smoothly
@@ -1038,9 +1184,9 @@ SlangResult NVRTCDownstreamCompiler::compile(
         cmdLine.addArg("-std=c++17");
 
         // Disable all warnings
-        // This is arguably too much - but nvrtc does not appear to have a mechanism to switch off
-        // individual warnings. I tried the -Xcudafe mechanism but that does not appear to work for
-        // nvrtc
+        // This is arguably too much - but nvrtc does not appear to have a mechanism
+        // to switch off individual warnings. I tried the -Xcudafe mechanism but
+        // that does not appear to work for nvrtc
         cmdLine.addArg("-w");
     }
 
@@ -1051,10 +1197,18 @@ SlangResult NVRTCDownstreamCompiler::compile(
         SemanticVersion version(3);
 
         // Newer releases of NVRTC only support newer CUDA architectures.
-        if (m_desc.version.m_major >= 12)
+        if (m_desc.version.m_major > 12 ||
+            (m_desc.version.m_major == 12 && m_desc.version.m_minor >= 8))
         {
-            // NVRTC in CUDA 12 only supports `compute_50` and up
-            // (with everything before `compute_52` being deprecated).
+            // NVRTC 12.8+ warns about architectures prior to compute_75 being deprecated
+            // The exact warning message is:
+            //   nvrtc 12.8: nvrtc: warning : Architectures prior to '<compute/sm>_75' are
+            //   deprecated and may be removed in a future release
+            version = SemanticVersion(7, 5);
+        }
+        else if (m_desc.version.m_major == 12)
+        {
+            // NVRTC 12.0 supports `compute_50` and up
             version = SemanticVersion(5, 0);
         }
         else if (m_desc.version.m_major == 11)
@@ -1093,16 +1247,22 @@ SlangResult NVRTCDownstreamCompiler::compile(
     List<const char*> headers;
     List<const char*> headerIncludeNames;
 
-    // If compiling for OptiX, we need to add the appropriate search paths to the command line.
+    // If compiling for OptiX, we need to add the appropriate search paths to the
+    // command line.
     //
     if (options.pipelineType == PipelineType::RayTracing)
     {
-        SLANG_RETURN_ON_FAIL(_maybeAddOptixSupport(options, cmdLine));
+        if (SLANG_FAILED(_maybeAddOptixSupport(options, cmdLine, diagnostics)))
+        {
+            diagnostics->setResult(SLANG_FAIL);
+            *outArtifact = artifact.detach();
+            return SLANG_FAIL;
+        }
     }
 
     // Add any compiler specific options
-    // NOTE! If these clash with any previously set options (as set via other flags)
-    // compilation might fail.
+    // NOTE! If these clash with any previously set options (as set via other
+    // flags) compilation might fail.
     if (options.compilerSpecificArguments.count > 0)
     {
         for (auto compilerSpecificArg : options.compilerSpecificArguments)
@@ -1145,11 +1305,6 @@ SlangResult NVRTCDownstreamCompiler::compile(
 
     res = m_nvrtcCompileProgram(program, int(dstOptions.getCount()), dstOptions.getBuffer());
 
-    auto artifact = ArtifactUtil::createArtifactForCompileTarget(options.targetType);
-    auto diagnostics = ArtifactDiagnostics::create();
-
-    ArtifactUtil::addAssociated(artifact, diagnostics);
-
     ComPtr<ISlangBlob> blob;
 
     diagnostics->setResult(_asResult(res));
@@ -1191,8 +1346,8 @@ SlangResult NVRTCDownstreamCompiler::compile(
         Index lastIndex = lines.getCount();
 
         // Look for the first blank line after this point.
-        // We'll assume any information after that blank line to the end of the diagnostic
-        // is compilation summary information.
+        // We'll assume any information after that blank line to the end of the
+        // diagnostic is compilation summary information.
         for (Index i = lastIndex - 1; i >= 0; --i)
         {
             if (lines[i].getLength() == 0)
@@ -1222,7 +1377,8 @@ SlangResult NVRTCDownstreamCompiler::compile(
             else if (lineRes != SLANG_E_NOT_FOUND)
             {
                 // If there is an error exit
-                // But if SLANG_E_NOT_FOUND that just means this line couldn't be parsed, so ignore.
+                // But if SLANG_E_NOT_FOUND that just means this line couldn't be
+                // parsed, so ignore.
                 return lineRes;
             }
         }
@@ -1271,8 +1427,8 @@ SlangResult NVRTCDownstreamCompiler::convert(
 
     // PTX is 'binary like' and 'assembly like' so we allow conversion either way
     // We do it by just getting as a blob and sharing that blob.
-    // A more sophisticated implementation could proxy to the original artifact, but this
-    // is simpler, and probably fine in most scenarios.
+    // A more sophisticated implementation could proxy to the original artifact,
+    // but this is simpler, and probably fine in most scenarios.
     ComPtr<ISlangBlob> blob;
     SLANG_RETURN_ON_FAIL(from->loadBlob(ArtifactKeep::Yes, blob.writeRef()));
 
@@ -1342,9 +1498,9 @@ static SlangResult _findAndLoadNVRTC(
         // a library simply named `nvrtc` (well, `libnvrtc`) which
         // is expected to match whatever the user has installed.
         //
-        // On Windows an installation could place the version of nvrtc it uses in the same directory
-        // as the slang binary, such that it's loaded.
-        // Using this name also allows a ISlangSharedLibraryLoader to easily identify what is
+        // On Windows an installation could place the version of nvrtc it uses in
+        // the same directory as the slang binary, such that it's loaded. Using this
+        // name also allows a ISlangSharedLibraryLoader to easily identify what is
         // required and perhaps load a specific version
         if (SLANG_FAILED(loader->loadSharedLibrary("nvrtc", library.writeRef())))
         {
