@@ -17,6 +17,7 @@ struct ExecuteUnitTestArgs
     String moduleName;
     String testName;
     uint32_t enabledApis;
+    bool enableDebugLayers;
 
     static const UnownedStringSlice g_methodName;
     static const StructRttiInfo g_rttiInfo;
@@ -41,6 +42,7 @@ struct ExecutionResult
 {
     String stdOut;
     String stdError;
+    String debugLayer;
     int32_t result = SLANG_OK;
     int32_t returnCode = 0; ///< As returned if invoked as command line
 
