@@ -339,6 +339,7 @@ protected:
     DictionaryType dict;
 
 private:
+    void init() {} // Base case for recursion
     template<typename... Args>
     void init(const T& v, Args... args)
     {
@@ -406,6 +407,8 @@ public:
 template<typename T>
 class HashSet : public HashSetBase<T, Dictionary<T, _DummyClass>>
 {
+public:
+    using HashSetBase<T, Dictionary<T, _DummyClass>>::HashSetBase;
 };
 
 template<typename TKey, typename TValue>
@@ -525,7 +528,7 @@ private:
             OrderedDictionary<TKey, TValue> newDict;
             newDict.m_bucketCountMinusOne = newSize - 1;
             newDict.m_hashMap = new LinkedNode<KeyValuePair<TKey, TValue>>*[newSize];
-            newDict.m_marks.resizeAndClear(newSize * 2);
+            newDict.m_marks.resizeAndUnsetAll(newSize * 2);
             if (m_hashMap)
             {
                 for (auto& kvPair : *this)
@@ -618,7 +621,7 @@ public:
     {
         m_count = 0;
         m_kvPairs.clear();
-        m_marks.clear();
+        m_marks.resize(0);
     }
     template<typename T>
     bool containsKey(const T& key) const
