@@ -336,6 +336,7 @@ RttiTypeFuncs RttiUtil::getDefaultTypeFuncs(const RttiInfo* rttiInfo)
     case RttiInfo::Kind::List:
         return ListFuncs::getFuncs();
     case RttiInfo::Kind::Struct:
+    case RttiInfo::Kind::Optional:
         return StructArrayFuncs::getFuncs();
     default:
         break;
@@ -556,6 +557,10 @@ static bool _isStructDefault(const StructRttiInfo* type, const void* src)
             const OtherRttiInfo* otherRttiInfo = static_cast<const OtherRttiInfo*>(rttiInfo);
             return otherRttiInfo->m_isDefaultFunc && otherRttiInfo->m_isDefaultFunc(rttiInfo, src);
         }
+    case RttiInfo::Kind::Optional:
+        {
+            return *(const bool*)src == (_getIntDefaultValue(defaultValue) != 0);
+        }
     default:
         {
             return false;
@@ -573,39 +578,17 @@ static bool _isStructDefault(const StructRttiInfo* type, const void* src)
     // setting the count if it is <= capacity just sets the count (ie things aren't released(!)).
 
     List<Byte>& dstList = *(List<Byte>*)dst;
-    const Index oldCount = dstList.getCount();
-    if (oldCount == count)
-    {
-        return SLANG_OK;
-    }
-    if (count < oldCount)
-    {
-        dstList.unsafeShrinkToCount(count);
-        return SLANG_OK;
-    }
 
     const auto typeFuncs = typeMap->getFuncsForType(elementType);
     SLANG_ASSERT(typeFuncs.isValid());
 
-    const Index dstCapacity = dstList.getCapacity();
-    void* oldBuffer = dstList.detachBuffer();
-
     void* newBuffer = ::malloc(count * elementType->m_size);
     // Initialize it all first
     typeFuncs.ctorArray(typeMap, elementType, newBuffer, count);
 
-    typeFuncs.copyArray(typeMap, elementType, newBuffer, oldBuffer, oldCount);
-
     // Attach the new buffer
     dstList.attachBuffer((Byte*)newBuffer, count, count);
 
-    // Free the old buffer
-    if (oldBuffer)
-    {
-        typeFuncs.dtorArray(typeMap, elementType, oldBuffer, dstCapacity);
-        ::free(oldBuffer);
-    }
-
     return SLANG_OK;
 }
 
@@ -660,6 +643,11 @@ static bool _isStructDefault(const StructRttiInfo* type, const void* src)
 
             return true;
         }
+    case RttiInfo::Kind::Optional:
+        {
+            const OptionalRttiInfo* optionalRttiInfo = static_cast<const OptionalRttiInfo*>(type);
+            return canMemCpy(optionalRttiInfo->m_elementType);
+        }
     default:
         {
             return type->isBuiltIn();
@@ -723,6 +711,11 @@ static bool _isStructDefault(const StructRttiInfo* type, const void* src)
 
             return true;
         }
+    case RttiInfo::Kind::Optional:
+        {
+            const OptionalRttiInfo* optionalRttiInfo = static_cast<const OptionalRttiInfo*>(type);
+            return canZeroInit(optionalRttiInfo->m_elementType);
+        }
     default:
         {
             return type->isBuiltIn();
@@ -783,6 +776,11 @@ static bool _isStructDefault(const StructRttiInfo* type, const void* src)
             } while (structRttiInfo);
             return false;
         }
+    case RttiInfo::Kind::Optional:
+        {
+            const OptionalRttiInfo* optionalRttiInfo = static_cast<const OptionalRttiInfo*>(type);
+            return hasDtor(optionalRttiInfo->m_elementType);
+        }
     default:
         {
             return !type->isBuiltIn();
@@ -894,6 +892,19 @@ static bool _isStructDefault(const StructRttiInfo* type, const void* src)
 
             return;
         }
+    case RttiInfo::Kind::Optional:
+        {
+            const OptionalRttiInfo* optionalRttiInfo =
+                static_cast<const OptionalRttiInfo*>(rttiInfo);
+            ctorArray(typeMap, GetRttiInfo<bool>::get(), dst, stride, count);
+            ctorArray(
+                typeMap,
+                optionalRttiInfo->m_elementType,
+                dst + optionalRttiInfo->m_valueOffset,
+                stride,
+                count);
+            return;
+        }
     }
 
     SLANG_ASSERT(!"Unexpected");
@@ -1005,6 +1016,20 @@ static bool _isStructDefault(const StructRttiInfo* type, const void* src)
 
             return;
         }
+    case RttiInfo::Kind::Optional:
+        {
+            const OptionalRttiInfo* optionalRttiInfo =
+                static_cast<const OptionalRttiInfo*>(rttiInfo);
+            copyArray(typeMap, GetRttiInfo<bool>::get(), dst, src, stride, count);
+            copyArray(
+                typeMap,
+                optionalRttiInfo->m_elementType,
+                dst + optionalRttiInfo->m_valueOffset,
+                src + optionalRttiInfo->m_valueOffset,
+                stride,
+                count);
+            return;
+        }
     }
 
     SLANG_ASSERT(!"Unexpected");
@@ -1091,6 +1116,19 @@ static bool _isStructDefault(const StructRttiInfo* type, const void* src)
 
             return;
         }
+    case RttiInfo::Kind::Optional:
+        {
+            const OptionalRttiInfo* optionalRttiInfo =
+                static_cast<const OptionalRttiInfo*>(rttiInfo);
+            dtorArray(typeMap, GetRttiInfo<bool>::get(), dst, stride, count);
+            dtorArray(
+                typeMap,
+                optionalRttiInfo->m_elementType,
+                dst + optionalRttiInfo->m_valueOffset,
+                stride,
+                count);
+            return;
+        }
     }
 
     SLANG_ASSERT(!"Unexpected");
