@@ -11,6 +11,7 @@
 #include "slang-compiler.h"
 #include "slang-internal.h"
 #include "slang-repro.h"
+#include "slang-tag-version.h"
 
 // implementation of C interface
 
@@ -65,24 +66,60 @@ SlangResult tryLoadBuiltinModuleFromCache(
     return SLANG_OK;
 }
 
-// Attempt to load a precompiled builtin module from slang-xxx-module.dll.
+// Attempt to load a precompiled builtin module from slang-xxx-module.
+// The module name is versioned on Mac and Linux platforms, but not on Windows.
 SlangResult tryLoadBuiltinModuleFromDLL(
     slang::IGlobalSession* globalSession,
     slang::BuiltinModuleName builtinModuleName)
 {
-    Slang::String moduleFileName =
-        Slang::String("slang-") + Slang::getBuiltinModuleNameStr(builtinModuleName) + "-module";
+    // Construct the versioned module name: slang-{module}-module[-{version}]
+    // e.g., "slang-glsl-module-2025.19.1" which becomes:
+    //   - Linux: libslang-glsl-module-2025.19.1.so
+    //   - macOS: libslang-glsl-module-2025.19.1.dylib
+    //   - Windows: slang-glsl-module.dll
+    // Modules are runtime-loaded libraries. We need to version these because
+    // they end up deployed on Mac and Linux platforms in a directory that ends
+    // up in the library path.
+    Slang::String versionString =
+        (SLANG_WINDOWS_FAMILY) ? "" : Slang::String("-") + SLANG_VERSION_NUMERIC;
+    Slang::String moduleFileName = Slang::String("slang-") +
+                                   Slang::getBuiltinModuleNameStr(builtinModuleName) + "-module" +
+                                   versionString;
 
     Slang::SharedLibrary::Handle libHandle = nullptr;
 
     SLANG_RETURN_ON_FAIL(Slang::SharedLibrary::load(moduleFileName.getBuffer(), libHandle));
     if (!libHandle)
         return SLANG_FAIL;
-    void* ptr = Slang::SharedLibrary::findSymbolAddressByName(libHandle, "slang_getEmbeddedModule");
-    if (!ptr)
+
+    // Check if the module is the same version as the slang dll.
+    void* emBuildTagPtr = Slang::SharedLibrary::findSymbolAddressByName(
+        libHandle,
+        "slang_getEmbeddedModuleBuildTagString");
+    if (!emBuildTagPtr)
+    {
+        Slang::SharedLibrary::unload(libHandle);
         return SLANG_FAIL;
+    }
+    typedef const char*(GetEmbeddedModuleBuildTagStringFunc)();
+    auto getEmbeddedModuleBuildTagString = (GetEmbeddedModuleBuildTagStringFunc*)emBuildTagPtr;
+    const char* buildTagString = getEmbeddedModuleBuildTagString();
+    if (strcmp(buildTagString, SLANG_TAG_VERSION) != 0)
+    {
+        Slang::SharedLibrary::unload(libHandle);
+        return SLANG_FAIL;
+    }
+
+    // Load the embedded module.
+    void* emPtr =
+        Slang::SharedLibrary::findSymbolAddressByName(libHandle, "slang_getEmbeddedModule");
+    if (!emPtr)
+    {
+        Slang::SharedLibrary::unload(libHandle);
+        return SLANG_FAIL;
+    }
     typedef ISlangBlob*(GetEmbeddedModuleFunc)();
-    auto getEmbeddedModule = (GetEmbeddedModuleFunc*)ptr;
+    auto getEmbeddedModule = (GetEmbeddedModuleFunc*)emPtr;
     auto blob = getEmbeddedModule();
     SLANG_RETURN_ON_FAIL(globalSession->loadBuiltinModule(
         builtinModuleName,
@@ -982,3 +1019,87 @@ SLANG_API void spSetDiagnosticFlags(slang::ICompileRequest* request, SlangDiagno
 
     request->setDiagnosticFlags(flags);
 }
+
+/* !!!!!!!!!!!!!!!!!!!!!!!!!!!!! Blob Creation !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! */
+
+SLANG_EXTERN_C SLANG_API ISlangBlob* slang_createBlob(const void* data, size_t size)
+{
+    // Disallow empty blobs.
+    if (!data || size == 0)
+        return nullptr;
+
+    Slang::ComPtr<ISlangBlob> blob = Slang::RawBlob::create(data, size);
+    if (!blob)
+        return nullptr;
+
+    return blob.detach();
+}
+
+/* !!!!!!!!!!!!!!!!!!!!!!!!!!!!! Module Loading !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! */
+
+SLANG_EXTERN_C SLANG_API slang::IModule* slang_loadModuleFromSource(
+    slang::ISession* session,
+    const char* moduleName,
+    const char* path,
+    const char* source,
+    size_t sourceSize,
+    ISlangBlob** outDiagnostics)
+{
+    if (!session || !moduleName || !path || !source || sourceSize == 0)
+        return nullptr;
+
+    // Create a blob from the source data using the slang_createBlob function.
+    Slang::ComPtr<ISlangBlob> sourceBlob;
+    sourceBlob = slang_createBlob(source, sourceSize);
+    if (!sourceBlob)
+        return nullptr;
+
+    // Load the module using the existing blob-based API.
+    return session->loadModuleFromSource(moduleName, path, sourceBlob, outDiagnostics);
+}
+
+SLANG_EXTERN_C SLANG_API slang::IModule* slang_loadModuleFromIRBlob(
+    slang::ISession* session,
+    const char* moduleName,
+    const char* path,
+    const void* source,
+    size_t sourceSize,
+    ISlangBlob** outDiagnostics)
+{
+    if (!session || !moduleName || !path || !source || sourceSize == 0)
+        return nullptr;
+
+    // Create a blob from the source data using the slang_createBlob function.
+    Slang::ComPtr<ISlangBlob> sourceBlob;
+    sourceBlob = slang_createBlob(source, sourceSize);
+    if (!sourceBlob)
+        return nullptr;
+
+    // Load the module using the existing IR blob-based API.
+    return session->loadModuleFromIRBlob(moduleName, path, sourceBlob, outDiagnostics);
+}
+
+SLANG_EXTERN_C SLANG_API SlangResult slang_loadModuleInfoFromIRBlob(
+    slang::ISession* session,
+    const void* source,
+    size_t sourceSize,
+    SlangInt& outModuleVersion,
+    const char*& outModuleCompilerVersion,
+    const char*& outModuleName)
+{
+    if (!session || !source || sourceSize == 0)
+        return SLANG_E_INVALID_ARG;
+
+    // Create a blob from the source data using the slang_createBlob function.
+    Slang::ComPtr<ISlangBlob> sourceBlob;
+    sourceBlob = slang_createBlob(source, sourceSize);
+    if (!sourceBlob)
+        return SLANG_E_INVALID_ARG;
+
+    // Load module info using the existing IR blob-based API.
+    return session->loadModuleInfoFromIRBlob(
+        sourceBlob,
+        outModuleVersion,
+        outModuleCompilerVersion,
+        outModuleName);
+}
