@@ -2,11 +2,13 @@
 
 #pragma once
 
-#include "slang-ast-base.h.fiddle"
 #include "slang-ast-forward-declarations.h"
 #include "slang-ast-support-types.h"
 #include "slang-capability.h"
 
+//
+#include "slang-ast-base.h.fiddle"
+
 // This file defines the primary base classes for the hierarchy of
 // AST nodes and related objects. For example, this is where the
 // basic `Decl`, `Stmt`, `Expr`, `type`, etc. definitions come from.
@@ -27,8 +29,8 @@ class NodeBase
     // Note that the astBuilder is not stored in the NodeBase derived types by default.
     SLANG_FORCE_INLINE void init(ASTNodeType inAstNodeType, ASTBuilder* inAstBuilder)
     {
-        SLANG_UNUSED(inAstBuilder);
         astNodeType = inAstNodeType;
+        _astBuilder = inAstBuilder;
 #ifdef _DEBUG
         _initDebug(inAstNodeType, inAstBuilder);
 #endif
@@ -41,11 +43,20 @@ class NodeBase
     /// The type of the node. ASTNodeType(-1) is an invalid node type, and shouldn't appear on any
     /// correctly constructed (through ASTBuilder) NodeBase derived class.
     /// The actual type is set when constructed on the ASTBuilder.
-    FIDDLE() ASTNodeType astNodeType = ASTNodeType(-1);
+    ASTNodeType astNodeType = ASTNodeType(-1);
 
 #ifdef _DEBUG
-    SLANG_UNREFLECTED int32_t _debugUID = 0;
+    int32_t _debugUID = 0;
 #endif
+
+    /// Get the AST builder that was used to allocate this node.
+    ASTBuilder* getASTBuilder() { return _astBuilder; }
+
+private:
+    friend class ASTBuilder;
+
+    /// The AST builder that was used to allocate this node.
+    ASTBuilder* _astBuilder = nullptr;
 };
 
 // Casting of NodeBase
@@ -112,7 +123,6 @@ class Scope : public NodeBase
     // The parent of this scope (where lookup should go if nothing is found locally)
     Scope* parent = nullptr;
 
-    SLANG_UNREFLECTED
     // The next sibling of this scope (a peer for lookup)
     Scope* nextSibling = nullptr;
 };
@@ -146,6 +156,24 @@ struct ValNodeOperand
 
     ValNodeOperand() { values.intOperand = 0; }
 
+    int64_t getIntConstant() const
+    {
+        SLANG_ASSERT(kind == ValNodeOperandKind::ConstantValue);
+        return values.intOperand;
+    }
+
+    Val* getVal() const
+    {
+        SLANG_ASSERT(kind == ValNodeOperandKind::ValNode);
+        return (Val*)values.nodeOperand;
+    }
+
+    Decl* getDecl() const
+    {
+        SLANG_ASSERT(kind == ValNodeOperandKind::ASTNode);
+        return (Decl*)values.nodeOperand;
+    }
+
     explicit ValNodeOperand(NodeBase* node)
     {
         if constexpr (sizeof(values.nodeOperand) < sizeof(values.intOperand))
@@ -424,23 +452,11 @@ class Val : public NodeBase
     Val* resolveImpl();
     Val* resolve();
 
-    Val* getOperand(Index index) const
-    {
-        SLANG_ASSERT(m_operands[index].kind == ValNodeOperandKind::ValNode);
-        return (Val*)m_operands[index].values.nodeOperand;
-    }
+    Val* getOperand(Index index) const { return m_operands[index].getVal(); }
 
-    Decl* getDeclOperand(Index index) const
-    {
-        SLANG_ASSERT(m_operands[index].kind == ValNodeOperandKind::ASTNode);
-        return (Decl*)(m_operands[index].values.nodeOperand);
-    }
+    Decl* getDeclOperand(Index index) const { return m_operands[index].getDecl(); }
 
-    int64_t getIntConstOperand(Index index) const
-    {
-        SLANG_ASSERT(m_operands[index].kind == ValNodeOperandKind::ConstantValue);
-        return m_operands[index].values.intOperand;
-    }
+    int64_t getIntConstOperand(Index index) const { return m_operands[index].getIntConstant(); }
 
     Index getOperandCount() const { return m_operands.getCount(); }
 
@@ -473,7 +489,7 @@ protected:
 
 private:
     mutable Val* m_resolvedVal = nullptr;
-    SLANG_UNREFLECTED mutable Index m_resolvedValEpoch = 0;
+    mutable Index m_resolvedValEpoch = 0;
 };
 
 template<int N, typename T, typename... Ts>
@@ -579,7 +595,21 @@ protected:
     // semantic checking, since Val deduplication requires the entire semantic checking process to
     // stick with one ASTBuilder.
     // Call getCurrentASTBuilder() to obtain the right ASTBuilder for semantic checking.
-    SLANG_UNREFLECTED ASTBuilder* m_astBuilderForReflection;
+    ASTBuilder* m_astBuilderForReflection;
+};
+
+struct TypePair
+{
+    Type* type0;
+    Type* type1;
+    HashCode getHashCode() const
+    {
+        return combineHash(Slang::getHashCode(type0), Slang::getHashCode(type1));
+    }
+    bool operator==(const TypePair& other) const
+    {
+        return type0 == other.type0 && type1 == other.type1;
+    }
 };
 
 template<typename T>
@@ -643,12 +673,7 @@ class DeclRefBase : public Val
     SourceLoc getNameLoc() const;
     SourceLoc getLoc() const;
     DeclRefBase* getParent();
-    String toString() const
-    {
-        StringBuilder sb;
-        const_cast<DeclRefBase*>(this)->toText(sb);
-        return sb.produceString();
-    }
+    String toString() const;
     DeclRefBase* getBase();
     void toText(StringBuilder& out);
 };
@@ -744,9 +769,9 @@ public:
     DeclRefBase* getDefaultDeclRef();
 
     FIDDLE() NameLoc nameAndLoc;
-    FIDDLE() CapabilitySet inferredCapabilityRequirements;
+    FIDDLE() CapabilitySetVal* inferredCapabilityRequirements = nullptr;
 
-    FIDDLE() RefPtr<MarkupEntry> markup;
+    RefPtr<MarkupEntry> markup;
 
     Name* getName() const { return nameAndLoc.name; }
     SourceLoc getNameLoc() const { return nameAndLoc.loc; }
@@ -754,8 +779,15 @@ public:
 
     DeclCheckStateExt checkState = DeclCheckState::Unchecked;
 
-    // The next declaration defined in the same container with the same name
-    Decl* nextInContainerWithSameName = nullptr;
+    /// The previous declaration defined in the same `ContainerDecl`
+    /// that has the same name as this declaration.
+    ///
+    /// Note: it is not recommended to ever access this member directly;
+    /// instead, code should use the `ContainerDecl::getPrevDeclWithSameName()`
+    /// method, which ensures that the `_prevInContainerWithSameName` fields
+    /// have been properly set for all declarations in that container.
+    ///
+    FIDDLE() Decl* _prevInContainerWithSameName = nullptr;
 
     bool isChecked(DeclCheckState state) const { return checkState >= state; }
     void setCheckState(DeclCheckState state)
@@ -766,12 +798,12 @@ public:
     bool isChildOf(Decl* other) const;
 
     // Track the decl reference that caused the requirement of a capability atom.
-    SLANG_UNREFLECTED List<ProvenenceNodeWithLoc> capabilityRequirementProvenance;
+    List<ProvenenceNodeWithLoc> capabilityRequirementProvenance;
 
-    SLANG_UNREFLECTED bool hiddenFromLookup = false;
+    bool hiddenFromLookup = false;
 
 private:
-    SLANG_UNREFLECTED DeclRefBase* m_defaultDeclRef = nullptr;
+    DeclRefBase* m_defaultDeclRef = nullptr;
 };
 
 FIDDLE(abstract)
