@@ -555,8 +555,6 @@ struct ASTDumpContext
     }
     void dump(const ExpandedSpecializationArg& arg) { dump(arg.witness); }
 
-    void dump(const TransparentMemberInfo& memInfo) { dump(memInfo.decl); }
-
     void dumpRemaining()
     {
         // Have to keep checking count, as dumping objects can add objects
@@ -570,6 +568,32 @@ struct ASTDumpContext
         }
     }
 
+    void dump(ContainerDeclDirectMemberDecls const& decls)
+    {
+        m_writer->emit(" { \n");
+        m_writer->indent();
+        bool first = true;
+        for (auto decl : decls.getDecls())
+        {
+            if (!first)
+            {
+                m_writer->emit(",\n");
+            }
+            first = false;
+
+            if (decl)
+            {
+                dump(decl);
+            }
+            else
+            {
+                m_writer->emit("null");
+            }
+        }
+        m_writer->dedent();
+        m_writer->emit("}");
+    }
+
     void dump(ContainerDecl* decl)
     {
         if (auto moduleDecl = dynamicCast<ModuleDecl>(decl))
@@ -775,6 +799,8 @@ struct ASTDumpAccess
 %for _,T in ipairs(Slang.NodeBase.subclasses) do
     static void dump_($T * node, ASTDumpContext & context)
     {
+        SLANG_UNUSED(node);
+        SLANG_UNUSED(context);
 %   if T.directSuperClass then
         dump_(static_cast<$(T.directSuperClass)*>(node), context);
 %   end
