@@ -2,6 +2,8 @@
 #pragma once
 
 #include "slang-ast-base.h"
+
+//
 #include "slang-ast-expr.h.fiddle"
 
 FIDDLE()
@@ -33,7 +35,6 @@ class DeclRefExpr : public Expr
     // The original expr before DeclRef resolution.
     Expr* originalExpr = nullptr;
 
-    SLANG_UNREFLECTED
     // The scope in which to perform lookup
     Scope* scope = nullptr;
 };
@@ -80,7 +81,7 @@ class OverloadedExpr2 : public Expr
     Expr* base = nullptr;
 
     // The lookup result that was ambiguous
-    List<Expr*> candidiateExprs;
+    List<Expr*> candidateExprs;
 };
 
 FIDDLE(abstract)
@@ -139,6 +140,12 @@ class StringLiteralExpr : public LiteralExpr
     FIDDLE() String value;
 };
 
+FIDDLE()
+class MakeArrayFromElementExpr : public Expr
+{
+    FIDDLE(...)
+};
+
 // An initializer list, e.g. `{ 1, 2, 3 }`
 FIDDLE()
 class InitializerListExpr : public Expr
@@ -304,13 +311,17 @@ class StaticMemberExpr : public DeclRefExpr
     SourceLoc memberOperatorLoc;
 };
 
+FIDDLE()
 struct MatrixCoord
 {
+    FIDDLE(...)
+
     bool operator==(const MatrixCoord& rhs) const { return row == rhs.row && col == rhs.col; };
     bool operator!=(const MatrixCoord& rhs) const { return !(*this == rhs); };
+
     // Rows and columns are zero indexed
-    int row;
-    int col;
+    FIDDLE() Int32 row;
+    FIDDLE() Int32 col;
 };
 
 FIDDLE()
@@ -491,6 +502,31 @@ class CountOfExpr : public SizeOfLikeExpr
     FIDDLE(...)
 };
 
+/// Expression for compile-time bit casting from floating-point to integer.
+/// __floatAsInt(expr) reinterprets the bits of a floating-point value as an integer.
+/// The input can be:
+///   - A floating-point literal with suffix: 1.0h (half), 1.0f or 1.0 (float), 1.0lf (double)
+///   - An explicit cast: half(x), float(x), double(x)
+/// Returns:
+///   - half -> int16_t
+///   - float -> int32_t (int)
+///   - double -> int64_t
+/// This is evaluated at compile-time when the argument is a constant.
+FIDDLE()
+class FloatBitCastExpr : public Expr
+{
+    FIDDLE(...)
+    /// The floating-point expression being bit-cast
+    FIDDLE() Expr* value = nullptr;
+};
+
+FIDDLE()
+class AddressOfExpr : public Expr
+{
+    FIDDLE(...)
+    FIDDLE() Expr* arg = nullptr;
+};
+
 FIDDLE()
 class MakeOptionalExpr : public Expr
 {
@@ -567,16 +603,26 @@ FIDDLE()
 class ParenExpr : public Expr
 {
     FIDDLE(...)
-    Expr* base = nullptr;
+    FIDDLE() Expr* base = nullptr;
 };
 
+// An expression that constructs a tuple `(arg1, arg2, ...)`
+//
+FIDDLE()
+class TupleExpr : public Expr
+{
+    FIDDLE(...)
+    List<Expr*> elements;
+};
+
+
 // An object-oriented `this` expression, used to
 // refer to the current instance of an enclosing type.
 FIDDLE()
 class ThisExpr : public Expr
 {
     FIDDLE(...)
-    SLANG_UNREFLECTED
+
     Scope* scope = nullptr;
 };
 
@@ -586,7 +632,7 @@ FIDDLE()
 class ReturnValExpr : public Expr
 {
     FIDDLE(...)
-    SLANG_UNREFLECTED
+
     Scope* scope = nullptr;
 };
 
@@ -673,6 +719,14 @@ class DispatchKernelExpr : public HigherOrderInvokeExpr
     FIDDLE() Expr* dispatchSize;
 };
 
+FIDDLE()
+class LambdaExpr : public Expr
+{
+    FIDDLE(...)
+    FIDDLE() ScopeDecl* paramScopeDecl;
+    FIDDLE() Stmt* bodyStmt;
+};
+
 /// An express to mark its inner expression as an intended non-differential call.
 FIDDLE()
 class TreatAsDifferentiableExpr : public Expr
@@ -707,10 +761,21 @@ FIDDLE()
 class ThisTypeExpr : public Expr
 {
     FIDDLE(...)
-    SLANG_UNREFLECTED
+
     Scope* scope = nullptr;
 };
 
+
+/// A type expression of the form `ThisInterface`
+///
+/// Refers to the interface type itself, not the conforming type from an interface decl.
+///
+FIDDLE()
+class ThisInterfaceExpr : public VarExpr
+{
+    FIDDLE(...)
+};
+
 /// A type expression of the form `Left & Right`.
 FIDDLE()
 class AndTypeExpr : public Expr
@@ -762,6 +827,7 @@ class PartiallyAppliedGenericExpr : public Expr
     FIDDLE(...)
 public:
     Expr* originalExpr = nullptr;
+    Expr* baseExpr = nullptr;
 
     /// The generic being applied
     DeclRef<GenericDecl> baseGenericDeclRef;
@@ -817,40 +883,41 @@ public:
     };
 
     // The flavour and token describes how this was parsed
-    Flavor flavor;
+    FIDDLE() Flavor flavor;
     // The single token this came from
     Token token;
 
     // If this was a SlangValue or SlangValueAddr or SlangType, then we also
     // store the expression, which should be a single VarExpr because we only
     // parse single idents at the moment
-    Expr* expr = nullptr;
+    FIDDLE() Expr* expr = nullptr;
 
     // If this is part of a bitwise or expression, this will point to the
     // remaining operands values in such an expression must be of flavour
     // Literal or NamedValue
-    List<SPIRVAsmOperand> bitwiseOrWith = List<SPIRVAsmOperand>();
+    FIDDLE() List<SPIRVAsmOperand> bitwiseOrWith = List<SPIRVAsmOperand>();
 
     // If this is a named value then we calculate the value here during
     // checking. If this is an opcode, then the parser will populate this too
     // (or set it to 0xffffffff);
-    SpvWord knownValue = 0xffffffff;
+    FIDDLE() SpvWord knownValue = 0xffffffff;
     // Although this might be a constant in the source we should actually pass
     // it as an id created with OpConstant
-    bool wrapInId = false;
+    FIDDLE() bool wrapInId = false;
 
     // Once we've checked things, the SlangType and BuiltinVar flavour operands
     // will have this type populated.
-    TypeExp type = TypeExp();
+    FIDDLE() TypeExp type = TypeExp();
 };
 
 FIDDLE()
 struct SPIRVAsmInst
 {
     FIDDLE(...)
+
 public:
-    SPIRVAsmOperand opcode;
-    List<SPIRVAsmOperand> operands;
+    FIDDLE() SPIRVAsmOperand opcode;
+    FIDDLE() List<SPIRVAsmOperand> operands;
 };
 
 FIDDLE()
