@@ -2,6 +2,9 @@
 #pragma once
 
 #include "slang-ast-base.h"
+#include "slang-ir-insts-enum.h"
+
+//
 #include "slang-ast-modifier.h.fiddle"
 
 FIDDLE()
@@ -66,6 +69,21 @@ class InternalModifier : public VisibilityModifier
     FIDDLE(...)
 };
 
+FIDDLE()
+class OverrideModifier : public Modifier
+{
+    FIDDLE(...)
+};
+
+// Marks that a decl is verified to be overriding another decl defined in a base type.
+FIDDLE()
+class IsOverridingModifier : public Modifier
+{
+    FIDDLE(...)
+
+    FIDDLE() Decl* overridedDecl = nullptr;
+};
+
 FIDDLE()
 class RequireModifier : public Modifier
 {
@@ -84,6 +102,12 @@ class ExternModifier : public Modifier
     FIDDLE(...)
 };
 
+FIDDLE()
+class DynModifier : public Modifier
+{
+    FIDDLE(...)
+};
+
 FIDDLE()
 class HLSLExportModifier : public Modifier
 {
@@ -158,6 +182,21 @@ class SynthesizedModifier : public Modifier
     FIDDLE(...)
 };
 
+// Marks that the definition of a func decl is synthesized static invoke func for
+// a lambda that doesn't capture anything.
+FIDDLE()
+class SynthesizedStaticLambdaFuncModifier : public Modifier
+{
+    FIDDLE(...)
+};
+
+FIDDLE()
+class ExplicitlyDeclaredCapabilityModifier : public Modifier
+{
+    FIDDLE(...)
+    FIDDLE() CapabilitySetVal* declaredCapabilityRequirements = nullptr;
+};
+
 // Marks a synthesized variable as local temporary variable.
 FIDDLE()
 class LocalTempVarModifier : public Modifier
@@ -190,6 +229,13 @@ class IgnoreForLookupModifier : public Modifier
     FIDDLE(...)
 };
 
+/// A modifier that indicates an `TypeConstraintDecl` is optional.
+FIDDLE()
+class OptionalConstraintModifier : public Modifier
+{
+    FIDDLE(...)
+};
+
 // A modifier that marks something as an operation that
 // has a one-to-one translation to the IR, and thus
 // has no direct definition in the high-level language.
@@ -293,16 +339,16 @@ class InOutModifier : public OutModifier
 };
 
 
-// `__ref` modifier for by-reference parameter passing
+// `ref` modifier for by-reference parameter passing
 FIDDLE()
 class RefModifier : public Modifier
 {
     FIDDLE(...)
 };
 
-// `__ref` modifier for by-reference parameter passing
+// `borrow` modifier for borrow parameter passing
 FIDDLE()
-class ConstRefModifier : public Modifier
+class BorrowModifier : public Modifier
 {
     FIDDLE(...)
 };
@@ -537,7 +583,6 @@ class BuiltinRequirementModifier : public Modifier
     FIDDLE() BuiltinRequirementKind kind;
 };
 
-
 // A modifier applied to declarations of builtin types to indicate how they
 // should be lowered to the IR.
 //
@@ -717,6 +762,14 @@ class HLSLVolatileModifier : public Modifier
     FIDDLE(...)
 };
 
+// Indicate that an interface method requirement has a default impl.
+FIDDLE()
+class HasInterfaceDefaultImplModifier : public Modifier
+{
+    FIDDLE(...)
+public:
+    FIDDLE() Decl* defaultImplDecl = nullptr;
+};
 
 FIDDLE()
 class AttributeTargetModifier : public Modifier
@@ -746,7 +799,7 @@ FIDDLE()
 class UncheckedAttribute : public AttributeBase
 {
     FIDDLE(...)
-    SLANG_UNREFLECTED
+
     Scope* scope = nullptr;
 };
 
@@ -755,7 +808,6 @@ FIDDLE()
 class UncheckedGLSLLayoutAttribute : public AttributeBase
 {
     FIDDLE(...)
-    SLANG_UNREFLECTED
 };
 
 // GLSL `binding` layout qualifier, does not include `set`.
@@ -763,7 +815,6 @@ FIDDLE()
 class UncheckedGLSLBindingLayoutAttribute : public UncheckedGLSLLayoutAttribute
 {
     FIDDLE(...)
-    SLANG_UNREFLECTED
 };
 
 // GLSL `set` layout qualifier, does not include `binding`.
@@ -771,7 +822,6 @@ FIDDLE()
 class UncheckedGLSLSetLayoutAttribute : public UncheckedGLSLLayoutAttribute
 {
     FIDDLE(...)
-    SLANG_UNREFLECTED
 };
 
 // GLSL `offset` layout qualifier.
@@ -779,70 +829,60 @@ FIDDLE()
 class UncheckedGLSLOffsetLayoutAttribute : public UncheckedGLSLLayoutAttribute
 {
     FIDDLE(...)
-    SLANG_UNREFLECTED
 };
 
 FIDDLE()
 class UncheckedGLSLInputAttachmentIndexLayoutAttribute : public UncheckedGLSLLayoutAttribute
 {
     FIDDLE(...)
-    SLANG_UNREFLECTED
 };
 
 FIDDLE()
 class UncheckedGLSLLocationLayoutAttribute : public UncheckedGLSLLayoutAttribute
 {
     FIDDLE(...)
-    SLANG_UNREFLECTED
 };
 
 FIDDLE()
 class UncheckedGLSLIndexLayoutAttribute : public UncheckedGLSLLayoutAttribute
 {
     FIDDLE(...)
-    SLANG_UNREFLECTED
 };
 
 FIDDLE()
 class UncheckedGLSLConstantIdAttribute : public UncheckedGLSLLayoutAttribute
 {
     FIDDLE(...)
-    SLANG_UNREFLECTED
 };
 
 FIDDLE()
 class UncheckedGLSLRayPayloadAttribute : public UncheckedGLSLLayoutAttribute
 {
     FIDDLE(...)
-    SLANG_UNREFLECTED
 };
 
 FIDDLE()
 class UncheckedGLSLRayPayloadInAttribute : public UncheckedGLSLLayoutAttribute
 {
     FIDDLE(...)
-    SLANG_UNREFLECTED
 };
 
 FIDDLE()
 class UncheckedGLSLHitObjectAttributesAttribute : public UncheckedGLSLLayoutAttribute
 {
     FIDDLE(...)
-    SLANG_UNREFLECTED
 };
 
 FIDDLE()
 class UncheckedGLSLCallablePayloadAttribute : public UncheckedGLSLLayoutAttribute
 {
     FIDDLE(...)
-    SLANG_UNREFLECTED
 };
 
 FIDDLE()
 class UncheckedGLSLCallablePayloadInAttribute : public UncheckedGLSLLayoutAttribute
 {
     FIDDLE(...)
-    SLANG_UNREFLECTED
 };
 
 // A `[name(arg0, ...)]` style attribute that has been validated.
@@ -876,7 +916,7 @@ FIDDLE()
 class RequireCapabilityAttribute : public Attribute
 {
     FIDDLE(...)
-    FIDDLE() CapabilitySet capabilitySet;
+    FIDDLE() CapabilitySetVal* capabilitySet = nullptr;
 };
 
 
@@ -1037,7 +1077,6 @@ FIDDLE()
 class GLSLImplicitOffsetLayoutAttribute : public AttributeBase
 {
     FIDDLE(...)
-    SLANG_UNREFLECTED
 };
 
 FIDDLE()
@@ -1235,7 +1274,7 @@ class EntryPointAttribute : public Attribute
 {
     FIDDLE(...)
     // The resolved capailities for our entry point.
-    FIDDLE() CapabilitySet capabilitySet;
+    FIDDLE() CapabilitySetVal* capabilitySet = nullptr;
 };
 
 // A `[__vulkanRayPayload(location)]` attribute, which is used in the
@@ -1568,7 +1607,7 @@ class DifferentiableAttribute : public Attribute
     /// Mapping from types to subtype witnesses for conformance to IDifferentiable.
     const OrderedDictionary<Type*, SubtypeWitness*>& getMapTypeToIDifferentiableWitness();
 
-    SLANG_UNREFLECTED ValSet m_typeRegistrationWorkingSet;
+    ValSet m_typeRegistrationWorkingSet;
 
 private:
     OrderedDictionary<Type*, SubtypeWitness*> m_mapToIDifferentiableWitness;
@@ -1650,7 +1689,7 @@ FIDDLE()
 class DerivativeMemberAttribute : public Attribute
 {
     FIDDLE(...)
-    FIDDLE() DeclRefExpr* memberDeclRef;
+    FIDDLE() DeclRefExpr* memberDeclRef = nullptr;
 };
 
 /// An attribute that marks an interface type as a COM interface declaration.
@@ -1675,7 +1714,7 @@ FIDDLE()
 class RequirePreludeAttribute : public Attribute
 {
     FIDDLE(...)
-    FIDDLE() CapabilitySet capabilitySet;
+    FIDDLE() CapabilitySetVal* capabilitySet = nullptr;
     FIDDLE() String prelude;
 };
 
@@ -1879,16 +1918,6 @@ class RequireFullQuadsAttribute : public Attribute
     FIDDLE(...)
 };
 
-/// A `[payload]` attribute indicates that a `struct` type will be used as
-/// a ray payload for `TraceRay()` calls, and thus also as input/output
-/// for shaders in the ray tracing pipeline that might be invoked for
-/// such a ray.
-///
-FIDDLE()
-class PayloadAttribute : public Attribute
-{
-    FIDDLE(...)
-};
 
 /// A `[raypayload]` attribute indicates that a `struct` type will be used as
 /// a ray payload for `TraceRay()` calls, and thus also as input/output
@@ -1925,7 +1954,7 @@ class NoSideEffectAttribute : public Attribute
     FIDDLE(...)
 };
 
-/// A `[KnownBuiltin("name")]` attribute allows the compiler to
+/// A `[KnownBuiltin(name)]` attribute allows the compiler to
 /// identify this declaration during compilation, despite obfuscation or
 /// linkage removing optimizations
 ///
@@ -1933,7 +1962,7 @@ FIDDLE()
 class KnownBuiltinAttribute : public Attribute
 {
     FIDDLE(...)
-    FIDDLE() String name;
+    FIDDLE() IntVal* name;
 };
 
 /// A modifier that applies to types rather than declarations.
@@ -2087,4 +2116,10 @@ public:
     List<Modifier*> getModifiers() { return memoryModifiers; }
 };
 
+FIDDLE()
+class ExperimentalModuleAttribute : public Attribute
+{
+    FIDDLE(...)
+};
+
 } // namespace Slang
