@@ -4,6 +4,7 @@
 #include "slang-ast-builder.h"
 #include "slang-ast-dispatch.h"
 #include "slang-ast-modifier.h"
+#include "slang-check.h"
 #include "slang-syntax.h"
 
 #include <assert.h>
@@ -13,6 +14,7 @@ namespace Slang
 
 bool isAbstractTypePack(Type* type)
 {
+    type = unwrapModifiedType(type);
     if (as<ExpandType>(type))
         return true;
     if (isDeclRefTypeOf<GenericTypePackParamDecl>(type))
@@ -22,6 +24,7 @@ bool isAbstractTypePack(Type* type)
 
 bool isTypePack(Type* type)
 {
+    type = unwrapModifiedType(type);
     if (as<ConcreteTypePack>(type))
         return true;
     return isAbstractTypePack(type);
@@ -396,8 +399,7 @@ Type* TypeType::_createCanonicalTypeOverride()
 
 void GenericDeclRefType::_toTextOverride(StringBuilder& out)
 {
-    // TODO: what is appropriate here?
-    out << toSlice("<DeclRef<GenericDecl>>");
+    out << getDeclRef();
 }
 
 Type* GenericDeclRefType::_createCanonicalTypeOverride()
@@ -440,11 +442,32 @@ Type* NativeRefType::getValueType()
     return as<Type>(_getGenericTypeArg(this, 0));
 }
 
-Val* PtrTypeBase::getAddressSpace()
+
+Val* PtrTypeBase::getAccessQualifier()
 {
     return _getGenericTypeArg(this, 1);
 }
 
+Val* PtrTypeBase::getAddressSpace()
+{
+    return _getGenericTypeArg(this, 2);
+}
+
+std::optional<AccessQualifier> tryGetAccessQualifierValue(Val* val)
+{
+    if (auto cintVal = as<ConstantIntVal>(val))
+    {
+        return AccessQualifier(cintVal->getValue());
+    }
+    return std::optional<AccessQualifier>();
+}
+
+std::optional<AccessQualifier> PtrTypeBase::tryGetAccessQualifierValue()
+{
+    auto accessQualifierArg = this->getAccessQualifier();
+    return Slang::tryGetAccessQualifierValue(accessQualifierArg);
+}
+
 AddressSpace tryGetAddressSpaceValue(Val* addrSpaceVal)
 {
     AddressSpace addrSpace = AddressSpace::Generic;
@@ -461,19 +484,41 @@ void maybePrintAddrSpaceOperand(StringBuilder& out, AddressSpace addrSpace)
     switch (addrSpace)
     {
     case AddressSpace::Generic:
+        out << toSlice(", AddressSpace.Generic");
+        break;
     case AddressSpace::UserPointer:
+        // We expose UserPointer as Device to users
+        out << toSlice(", AddressSpace.Device");
         break;
     case AddressSpace::GroupShared:
-        out << toSlice(", groupshared");
+        out << toSlice(", AddressSpace.GroupShared");
         break;
     case AddressSpace::Global:
-        out << toSlice(", global");
+        out << toSlice(", AddressSpace.Global");
         break;
     case AddressSpace::ThreadLocal:
-        out << toSlice(", threadlocal");
+        out << toSlice(", AddressSpace.ThreadLocal");
         break;
     case AddressSpace::Uniform:
-        out << toSlice(", uniform");
+        out << toSlice(", AddressSpace.Uniform");
+        break;
+    default:
+        break;
+    }
+}
+
+void maybePrintAccessQualifierOperand(StringBuilder& out, AccessQualifier accessQualifier)
+{
+    switch (accessQualifier)
+    {
+    case AccessQualifier::ReadWrite:
+        out << toSlice(", Access.ReadWrite");
+        break;
+    case AccessQualifier::Read:
+        out << toSlice(", Access.Read");
+        break;
+    case AccessQualifier::Immutable:
+        out << toSlice(", Access.Immutable");
         break;
     default:
         break;
@@ -483,22 +528,43 @@ void maybePrintAddrSpaceOperand(StringBuilder& out, AddressSpace addrSpace)
 void PtrType::_toTextOverride(StringBuilder& out)
 {
     auto addrSpace = tryGetAddressSpaceValue(getAddressSpace());
-    if (addrSpace == AddressSpace::Generic)
-        out << toSlice("Addr<") << getValueType();
-    else
-        out << toSlice("Ptr<") << getValueType();
+    out << toSlice("Ptr<") << getValueType();
+    if (auto optionalAccessQualifier = tryGetAccessQualifierValue())
+        maybePrintAccessQualifierOperand(out, *optionalAccessQualifier);
     maybePrintAddrSpaceOperand(out, addrSpace);
     out << toSlice(">");
 }
 
-void RefType::_toTextOverride(StringBuilder& out)
+void ExplicitRefType::_toTextOverride(StringBuilder& out)
 {
+    auto addrSpace = tryGetAddressSpaceValue(getAddressSpace());
     out << toSlice("Ref<") << getValueType();
-    auto addressSpaceVal = getAddressSpace();
-    maybePrintAddrSpaceOperand(out, tryGetAddressSpaceValue(addressSpaceVal));
+    if (auto optionalAccessQualifier = tryGetAccessQualifierValue())
+        maybePrintAccessQualifierOperand(out, *optionalAccessQualifier);
+    maybePrintAddrSpaceOperand(out, addrSpace);
     out << toSlice(">");
 }
 
+void OutParamType::_toTextOverride(StringBuilder& out)
+{
+    out << toSlice("out ") << getValueType();
+}
+
+void BorrowInOutParamType::_toTextOverride(StringBuilder& out)
+{
+    out << toSlice("inout ") << getValueType();
+}
+
+void RefParamType::_toTextOverride(StringBuilder& out)
+{
+    out << toSlice("ref ") << getValueType();
+}
+
+void BorrowInParamType::_toTextOverride(StringBuilder& out)
+{
+    out << toSlice("borrow ") << getValueType();
+}
+
 
 // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! NamedExpressionType !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 
@@ -520,31 +586,45 @@ Type* NamedExpressionType::_createCanonicalTypeOverride()
 
 // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! FuncType !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 
-ParameterDirection FuncType::getParamDirection(Index index)
+ParamPassingMode getParamPassingModeFromPossiblyWrappedParamType(Type* paramType)
 {
-    auto paramType = getParamType(index);
-    if (as<RefType>(paramType))
+    if (as<RefParamType>(paramType))
     {
-        return kParameterDirection_Ref;
+        return ParamPassingMode::Ref;
     }
-    else if (as<ConstRefType>(paramType))
+    else if (as<BorrowInParamType>(paramType))
     {
-        return kParameterDirection_ConstRef;
+        return ParamPassingMode::BorrowIn;
     }
-    else if (as<InOutType>(paramType))
+    else if (as<BorrowInOutParamType>(paramType))
     {
-        return kParameterDirection_InOut;
+        return ParamPassingMode::BorrowInOut;
     }
     else if (as<OutType>(paramType))
     {
-        return kParameterDirection_Out;
+        return ParamPassingMode::Out;
     }
     else
     {
-        return kParameterDirection_In;
+        return ParamPassingMode::In;
     }
 }
 
+ParamPassingMode FuncType::getParamPassingMode(Index index)
+{
+    auto paramType = getParamTypeWithModeWrapper(index);
+    return getParamPassingModeFromPossiblyWrappedParamType(paramType);
+}
+
+Type* FuncType::getParamValueType(Index index)
+{
+    auto paramType = getParamTypeWithModeWrapper(index);
+    if (auto wrappedParamType = as<ParamPassingModeType>(paramType))
+        return wrappedParamType->getValueType();
+    return paramType;
+}
+
+
 void FuncType::_toTextOverride(StringBuilder& out)
 {
     Index paramCount = getParamCount();
@@ -555,7 +635,7 @@ void FuncType::_toTextOverride(StringBuilder& out)
         {
             out << toSlice(", ");
         }
-        out << getParamType(pp);
+        out << getParamTypeWithModeWrapper(pp);
     }
     out << ") -> " << getResultType();
 
@@ -579,7 +659,20 @@ Val* FuncType::_substituteImplOverride(ASTBuilder* astBuilder, SubstitutionSet s
     List<Type*> substParamTypes;
     for (Index pp = 0; pp < getParamCount(); pp++)
     {
-        substParamTypes.add(as<Type>(getParamType(pp)->substituteImpl(astBuilder, subst, &diff)));
+        auto substParamType =
+            as<Type>(getParamTypeWithModeWrapper(pp)->substituteImpl(astBuilder, subst, &diff));
+        if (auto typePack = as<ConcreteTypePack>(substParamType))
+        {
+            // Unwrap the ConcreteTypePack and add each element as a parameter
+            for (Index i = 0; i < typePack->getTypeCount(); ++i)
+            {
+                substParamTypes.add(typePack->getElementType(i));
+            }
+        }
+        else
+        {
+            substParamTypes.add(substParamType);
+        }
     }
 
     // early exit for no change...
@@ -602,7 +695,7 @@ Type* FuncType::_createCanonicalTypeOverride()
     List<Type*> canParamTypes;
     for (Index pp = 0; pp < getParamCount(); pp++)
     {
-        canParamTypes.add(getParamType(pp)->getCanonicalType());
+        canParamTypes.add(getParamTypeWithModeWrapper(pp)->getCanonicalType());
     }
 
     FuncType* canType = getCurrentASTBuilder()->getFuncType(
@@ -775,7 +868,18 @@ Val* ConcreteTypePack::_substituteImplOverride(
     for (Index i = 0; i < getTypeCount(); i++)
     {
         auto substType = as<Type>(getElementType(i)->substituteImpl(astBuilder, subst, &diff));
-        substElementTypes.add(substType);
+        if (auto typePack = as<ConcreteTypePack>(substType))
+        {
+            // Unwrap the ConcreteTypePack and add each element as a parameter
+            for (Index j = 0; j < typePack->getTypeCount(); ++j)
+            {
+                substElementTypes.add(typePack->getElementType(j));
+            }
+        }
+        else
+        {
+            substElementTypes.add(substType);
+        }
     }
     if (!diff)
         return this;
@@ -841,13 +945,7 @@ DeclRef<ThisTypeDecl> ExtractExistentialType::getThisTypeDeclRef()
 
     SubtypeWitness* openedWitness = getSubtypeWitness();
 
-    ThisTypeDecl* thisTypeDecl = nullptr;
-    for (auto member : interfaceDecl->members)
-        if (as<ThisTypeDecl>(member))
-        {
-            thisTypeDecl = as<ThisTypeDecl>(member);
-            break;
-        }
+    ThisTypeDecl* thisTypeDecl = interfaceDecl->getThisTypeDecl();
     SLANG_ASSERT(thisTypeDecl);
 
     DeclRef<ThisTypeDecl> specialiedInterfaceDeclRef =
@@ -1102,6 +1200,8 @@ SlangResourceShape ResourceType::getShape()
         baseShape = (SlangResourceShape)((uint32_t)baseShape | SLANG_TEXTURE_SHADOW_FLAG);
     if (isFeedback())
         baseShape = (SlangResourceShape)((uint32_t)baseShape | SLANG_TEXTURE_FEEDBACK_FLAG);
+    if (isCombined())
+        baseShape = (SlangResourceShape)((uint32_t)baseShape | SLANG_TEXTURE_COMBINED_FLAG);
     return baseShape;
 }
 
@@ -1306,14 +1406,9 @@ Val* TextureTypeBase::getFormat()
     return as<Type>(_getGenericTypeArg(this, 8));
 }
 
-Type* removeParamDirType(Type* type)
+bool isCopyableType(Type* type)
 {
-    for (auto paramDirType = as<ParamDirectionType>(type); paramDirType;)
-    {
-        type = paramDirType->getValueType();
-        paramDirType = as<ParamDirectionType>(type);
-    }
-    return type;
+    return !isNonCopyableType(type);
 }
 
 bool isNonCopyableType(Type* type)
