@@ -1,8 +1,10 @@
 // slang-ast-type.cpp
 #include "slang-ast-val.h"
 
+#include "core/slang-uint-set.h"
 #include "slang-ast-builder.h"
 #include "slang-ast-dispatch.h"
+#include "slang-ast-natural-layout.h"
 #include "slang-check-impl.h"
 #include "slang-diagnostics.h"
 #include "slang-mangle.h"
@@ -142,7 +144,7 @@ Val* Val::_substituteImplOverride(ASTBuilder* astBuilder, SubstitutionSet subst,
 void Val::_toTextOverride(StringBuilder& out)
 {
     SLANG_UNUSED(out);
-    SLANG_UNEXPECTED("Val::_toStringOverride not overridden");
+    SLANG_UNEXPECTED("Val::_toTextOverride not overridden");
 }
 
 // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! ConstantIntVal !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
@@ -176,9 +178,9 @@ void ConstantIntVal::_toTextOverride(StringBuilder& out)
     out << getValue();
 }
 
-// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! GenericParamIntVal !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
+// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! DeclRefIntVal !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 
-void GenericParamIntVal::_toTextOverride(StringBuilder& out)
+void DeclRefIntVal::_toTextOverride(StringBuilder& out)
 {
     Name* name = getDeclRef().getName();
     if (name)
@@ -212,7 +214,7 @@ Val* maybeSubstituteGenericParam(Val* paramVal, Decl* paramDecl, SubstitutionSet
     Count argCount = args.getCount();
 
     Count argIndex = 0;
-    for (auto m : outerGeneric->members)
+    for (auto m : outerGeneric->getDirectMemberDecls())
     {
         // If we have run out of arguments, then we can stop
         // iterating over the parameters, because `this`
@@ -248,7 +250,7 @@ Val* maybeSubstituteGenericParam(Val* paramVal, Decl* paramDecl, SubstitutionSet
     return paramVal;
 }
 
-Val* GenericParamIntVal::_substituteImplOverride(
+Val* DeclRefIntVal::_substituteImplOverride(
     ASTBuilder* /* astBuilder */,
     SubstitutionSet subst,
     int* ioDiff)
@@ -259,12 +261,12 @@ Val* GenericParamIntVal::_substituteImplOverride(
     return this;
 }
 
-bool GenericParamIntVal::_isLinkTimeValOverride()
+bool DeclRefIntVal::_isLinkTimeValOverride()
 {
     return getDeclRef().getDecl()->hasModifier<ExternModifier>();
 }
 
-Val* GenericParamIntVal::_linkTimeResolveOverride(Dictionary<String, IntVal*>& map)
+Val* DeclRefIntVal::_linkTimeResolveOverride(Dictionary<String, IntVal*>& map)
 {
     auto name = getMangledName(getCurrentASTBuilder(), getDeclRef().declRefBase);
     IntVal* v;
@@ -540,17 +542,15 @@ Val* DeclaredSubtypeWitness::_substituteImplOverride(
 
         bool found = false;
         Index index = 0;
-        for (auto m : genericDecl->members)
+        for (auto constraintParam :
+             genericDecl->getDirectMemberDeclsOfType<GenericTypeConstraintDecl>())
         {
-            if (auto constraintParam = as<GenericTypeConstraintDecl>(m))
+            if (constraintParam == getDeclRef().getDecl())
             {
-                if (constraintParam == getDeclRef().getDecl())
-                {
-                    found = true;
-                    break;
-                }
-                index++;
+                found = true;
+                break;
             }
+            index++;
         }
         if (found)
         {
@@ -897,6 +897,18 @@ Val* TypeCoercionWitness::_resolveImplOverride()
     return this;
 }
 
+// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! NoneWitness !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
+
+void NoneWitness::_toTextOverride(StringBuilder& out)
+{
+    out.append("none");
+}
+
+Val* NoneWitness::_resolveImplOverride()
+{
+    return this;
+}
+
 // UNormModifierVal
 
 void UNormModifierVal::_toTextOverride(StringBuilder& out)
@@ -1189,7 +1201,9 @@ Val* PolynomialIntVal::_substituteImplOverride(
 
             if (auto constantVal = as<ConstantIntVal>(substResult))
             {
-                evaluatedTermConstFactor *= constantVal->getValue();
+                auto power = factor->getPower();
+                for (IntegerLiteralValue i = 0; i < power; i++)
+                    evaluatedTermConstFactor *= constantVal->getValue();
             }
             else if (auto intResult = as<IntVal>(substResult))
             {
@@ -1742,6 +1756,106 @@ Val* FuncCallIntVal::_substituteImplOverride(
     return this;
 }
 
+// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! SizeOfIntVal !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
+
+void SizeOfIntVal::_toTextOverride(StringBuilder& out)
+{
+    out << "sizeof(";
+    getTypeArg()->toText(out);
+    out << ")";
+}
+
+Val* SizeOfIntVal::tryFoldOrNull(ASTBuilder* astBuilder, Type* intType, Type* newType)
+{
+    ASTNaturalLayoutContext context(astBuilder, nullptr);
+    const auto size = context.calcSize(newType);
+
+    if (!size)
+        return nullptr;
+
+    return astBuilder->getIntVal(intType, size.size);
+}
+
+Val* SizeOfIntVal::tryFold(ASTBuilder* astBuilder, Type* intType, Type* newType)
+{
+    if (auto result = tryFoldOrNull(astBuilder, intType, newType))
+        return result;
+    auto result = astBuilder->getOrCreate<SizeOfIntVal>(intType, newType);
+    return result;
+}
+
+Val* SizeOfIntVal::_substituteImplOverride(
+    ASTBuilder* astBuilder,
+    SubstitutionSet subst,
+    int* ioDiff)
+{
+    int diff = 0;
+    auto newType = as<Type>(getTypeArg()->substituteImpl(astBuilder, subst, &diff));
+    if (!diff)
+        return this;
+
+    (*ioDiff)++;
+    return tryFold(astBuilder, getType(), newType);
+}
+
+Val* SizeOfIntVal::_resolveImplOverride()
+{
+    auto resolvedTypeArg = getTypeArg()->resolve();
+    if (resolvedTypeArg == getTypeArg())
+        return this;
+    return tryFold(getCurrentASTBuilder(), getType(), as<Type>(resolvedTypeArg));
+}
+
+// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! AlignOfIntVal !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
+
+void AlignOfIntVal::_toTextOverride(StringBuilder& out)
+{
+    out << "alignof(";
+    getTypeArg()->toText(out);
+    out << ")";
+}
+
+Val* AlignOfIntVal::tryFoldOrNull(ASTBuilder* astBuilder, Type* intType, Type* newType)
+{
+    ASTNaturalLayoutContext context(astBuilder, nullptr);
+    const auto size = context.calcSize(newType);
+
+    if (!size)
+        return nullptr;
+
+    return astBuilder->getIntVal(intType, size.alignment);
+}
+
+Val* AlignOfIntVal::tryFold(ASTBuilder* astBuilder, Type* intType, Type* newType)
+{
+    if (auto result = tryFoldOrNull(astBuilder, intType, newType))
+        return result;
+    auto result = astBuilder->getOrCreate<AlignOfIntVal>(intType, newType);
+    return result;
+}
+
+Val* AlignOfIntVal::_substituteImplOverride(
+    ASTBuilder* astBuilder,
+    SubstitutionSet subst,
+    int* ioDiff)
+{
+    int diff = 0;
+    auto newType = as<Type>(getTypeArg()->substituteImpl(astBuilder, subst, &diff));
+    if (!diff)
+        return this;
+
+    (*ioDiff)++;
+    return tryFold(astBuilder, getType(), newType);
+}
+
+Val* AlignOfIntVal::_resolveImplOverride()
+{
+    auto resolvedTypeArg = getTypeArg()->resolve();
+    if (resolvedTypeArg == getTypeArg())
+        return this;
+    return tryFold(getCurrentASTBuilder(), getType(), as<Type>(resolvedTypeArg));
+}
+
 // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! CountOfIntVal !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 
 void CountOfIntVal::_toTextOverride(StringBuilder& out)
@@ -2004,4 +2118,61 @@ Val* IntVal::linkTimeResolve(Dictionary<String, IntVal*>& mapMangledNameToVal)
     SLANG_AST_NODE_VIRTUAL_CALL(IntVal, linkTimeResolve, (mapMangledNameToVal));
 }
 
+//
+// UIntSetVal
+//
+
+UIntSet UIntSetVal::toUIntSet() const
+{
+    UIntSet result;
+
+    // Convert each bitmask operand back to UIntSet::Element
+    result.resizeBackingBufferDirectly(getBitmaskCount());
+    for (Index i = 0; i < getBitmaskCount(); i++)
+    {
+        result.m_buffer[i] = getBitmask(i);
+    }
+
+    return result;
+}
+
+void UIntSet::unionWith(const UIntSetVal& set)
+{
+    // UIntSetVal has getBitmask accessor that returns Elements
+    const Index setCount = set.getBitmaskCount();
+    const Index minCount = Math::Min(setCount, m_buffer.getCount());
+    m_buffer.reserve(setCount);
+
+    for (Index i = 0; i < minCount; ++i)
+    {
+        m_buffer[i] |= set.getBitmask(i);
+    }
+
+    // Add remaining elements from the UIntSetVal if it's larger
+    if (setCount > m_buffer.getCount())
+    {
+        for (Index i = m_buffer.getCount(); i < setCount; ++i)
+        {
+            m_buffer.add(set.getBitmask(i));
+        }
+    }
+}
+
+UIntSetVal* UIntSetVal::fromUIntSet(ASTBuilder* astBuilder, const UIntSet& uintSet)
+{
+    return astBuilder->getUIntSetVal(uintSet);
+}
+
+void UIntSetVal::_toTextOverride(StringBuilder& out)
+{
+    out << "UIntSetVal{";
+    for (Index i = 0; i < getBitmaskCount(); i++)
+    {
+        if (i > 0)
+            out << ", ";
+        out << getBitmask(i);
+    }
+    out << "}";
+}
+
 } // namespace Slang
