@@ -3,6 +3,8 @@
 
 #include "slang-ast-base.h"
 #include "slang-ast-decl.h"
+
+//
 #include "slang-ast-val.h.fiddle"
 
 FIDDLE()
@@ -54,8 +56,8 @@ public:
 };
 
 
-// Represent a lookup of SuperType::`m_decl` from `lookupSourceType` type that we know conforms to
-// SuperType.
+// Represent a lookup of witness of SuperType::`m_decl` from `lookupSource` type that we know
+// conforms to SuperType.
 FIDDLE()
 class LookupDeclRef : public DeclRefBase
 {
@@ -66,7 +68,7 @@ public:
     // The source type that we are looking up from.
     Type* getLookupSource() { return as<Type>(getOperand(1)); }
 
-    // Witness that `lookupSourceType`:SuperType.
+    // Witness that `lookupSource`:SuperType.
     SubtypeWitness* getWitness() { return as<SubtypeWitness>(getOperand(2)); }
 
     LookupDeclRef(Decl* declToLookup, Type* lookupSource, SubtypeWitness* witness)
@@ -168,7 +170,7 @@ class ConstantIntVal : public IntVal
 
 // The logical "value" of a reference to a generic value parameter
 FIDDLE()
-class GenericParamIntVal : public IntVal
+class DeclRefIntVal : public IntVal
 {
     FIDDLE(...)
     DeclRef<VarDeclBase> getDeclRef() { return as<DeclRefBase>(getOperand(1)); }
@@ -177,10 +179,7 @@ class GenericParamIntVal : public IntVal
     void _toTextOverride(StringBuilder& out);
     Val* _substituteImplOverride(ASTBuilder* astBuilder, SubstitutionSet subst, int* ioDiff);
 
-    GenericParamIntVal(Type* inType, DeclRef<VarDeclBase> inDeclRef)
-    {
-        setOperands(inType, inDeclRef);
-    }
+    DeclRefIntVal(Type* inType, DeclRef<VarDeclBase> inDeclRef) { setOperands(inType, inDeclRef); }
 
     bool _isLinkTimeValOverride();
     Val* _linkTimeResolveOverride(Dictionary<String, IntVal*>& map);
@@ -258,21 +257,65 @@ class FuncCallIntVal : public IntVal
     Val* _linkTimeResolveOverride(Dictionary<String, IntVal*>& map);
 };
 
-FIDDLE()
-class CountOfIntVal : public IntVal
+FIDDLE(abstract)
+class SizeOfLikeIntVal : public IntVal
 {
     FIDDLE(...)
-    CountOfIntVal(Type* inType, Type* typeArg) { setOperands(inType, typeArg); }
+    SizeOfLikeIntVal(Type* inType, Type* typeArg) { setOperands(inType, typeArg); }
 
     Val* getTypeArg() { return getOperand(1); }
 
+    bool _isLinkTimeValOverride() { return false; }
+};
+
+FIDDLE()
+class SizeOfIntVal : public SizeOfLikeIntVal
+{
+    FIDDLE(...)
+    SizeOfIntVal(Type* inType, Type* typeArg)
+        : SizeOfLikeIntVal(inType, typeArg)
+    {
+    }
+
     void _toTextOverride(StringBuilder& out);
     Val* _substituteImplOverride(ASTBuilder* astBuilder, SubstitutionSet subst, int* ioDiff);
     Val* _resolveImplOverride();
-    bool _isLinkTimeValOverride() { return false; }
 
     static Val* tryFoldOrNull(ASTBuilder* astBuilder, Type* intType, Type* newType);
+    static Val* tryFold(ASTBuilder* astBuilder, Type* intType, Type* newType);
+};
 
+FIDDLE()
+class AlignOfIntVal : public SizeOfLikeIntVal
+{
+    FIDDLE(...)
+    AlignOfIntVal(Type* inType, Type* typeArg)
+        : SizeOfLikeIntVal(inType, typeArg)
+    {
+    }
+
+    void _toTextOverride(StringBuilder& out);
+    Val* _substituteImplOverride(ASTBuilder* astBuilder, SubstitutionSet subst, int* ioDiff);
+    Val* _resolveImplOverride();
+
+    static Val* tryFoldOrNull(ASTBuilder* astBuilder, Type* intType, Type* newType);
+    static Val* tryFold(ASTBuilder* astBuilder, Type* intType, Type* newType);
+};
+
+FIDDLE()
+class CountOfIntVal : public SizeOfLikeIntVal
+{
+    FIDDLE(...)
+    CountOfIntVal(Type* inType, Type* typeArg)
+        : SizeOfLikeIntVal(inType, typeArg)
+    {
+    }
+
+    void _toTextOverride(StringBuilder& out);
+    Val* _substituteImplOverride(ASTBuilder* astBuilder, SubstitutionSet subst, int* ioDiff);
+    Val* _resolveImplOverride();
+
+    static Val* tryFoldOrNull(ASTBuilder* astBuilder, Type* intType, Type* newType);
     static Val* tryFold(ASTBuilder* astBuilder, Type* intType, Type* newType);
 };
 
@@ -319,9 +362,9 @@ public:
     // for sorting only.
     bool operator<(const PolynomialIntValFactor& other) const
     {
-        if (auto thisGenParam = as<GenericParamIntVal>(getParam()))
+        if (auto thisGenParam = as<DeclRefIntVal>(getParam()))
         {
-            if (auto thatGenParam = as<GenericParamIntVal>(other.getParam()))
+            if (auto thatGenParam = as<DeclRefIntVal>(other.getParam()))
             {
                 if (thisGenParam->equals(thatGenParam))
                     return getPower() < other.getPower();
@@ -336,7 +379,7 @@ public:
         }
         else
         {
-            if (const auto thatGenParam = as<GenericParamIntVal>(other.getParam()))
+            if (const auto thatGenParam = as<DeclRefIntVal>(other.getParam()))
             {
                 return false;
             }
@@ -347,9 +390,9 @@ public:
     // for sorting only.
     bool operator==(const PolynomialIntValFactor& other) const
     {
-        if (auto thisGenParam = as<GenericParamIntVal>(getParam()))
+        if (auto thisGenParam = as<DeclRefIntVal>(getParam()))
         {
-            if (auto thatGenParam = as<GenericParamIntVal>(other.getParam()))
+            if (auto thatGenParam = as<DeclRefIntVal>(other.getParam()))
             {
                 if (thisGenParam->equals(thatGenParam) && getPower() == other.getPower())
                     return true;
@@ -656,6 +699,13 @@ class DeclaredSubtypeWitness : public SubtypeWitness
         return false;
     }
 
+    bool isOptional()
+    {
+        if (auto declRef = getDeclRef().as<GenericTypeConstraintDecl>())
+            return declRef.getDecl()->hasModifier<OptionalConstraintModifier>();
+        return false;
+    }
+
     // Overrides should be public so base classes can access
     void _toTextOverride(StringBuilder& out);
     Val* _substituteImplOverride(ASTBuilder* astBuilder, SubstitutionSet subst, int* ioDiff);
@@ -791,6 +841,16 @@ class ExtractFromConjunctionSubtypeWitness : public SubtypeWitness
     ConversionCost _getOverloadResolutionCostOverride();
 };
 
+/// A witness for the "none" value of optional constraints.
+FIDDLE()
+class NoneWitness : public Witness
+{
+    FIDDLE(...)
+
+    void _toTextOverride(StringBuilder& out);
+    Val* _resolveImplOverride();
+};
+
 /// A value that represents a modifier attached to some other value
 FIDDLE()
 class ModifierVal : public Val
@@ -899,6 +959,42 @@ class BackwardDifferentiatePropagateVal : public DifferentiateVal
     }
 };
 
+/// A value that represents a UInt set stored as a list of bitmask operands.
+/// This is used for efficient deduplication of capability sets through ASTBuilder caching.
+FIDDLE()
+class UIntSetVal : public Val
+{
+    FIDDLE(...)
+
+    /// Get the number of bitmask operands
+    Index getBitmaskCount() const { return getOperandCount(); }
+
+    /// Get a specific bitmask operand as a constant integer value
+    ConstantIntVal* getBitmaskOperand(Index index) const
+    {
+        return as<ConstantIntVal>(getOperand(index));
+    }
+    UIntSet::Element getBitmask(Index index) const
+    {
+        return static_cast<UIntSet::Element>(getBitmaskOperand(index)->getValue());
+    }
+
+    /// Get all bitmask operands as a view
+    Val::OperandView<ConstantIntVal> getBitmasks() const
+    {
+        return Val::OperandView<ConstantIntVal>(this, 0, getOperandCount());
+    }
+
+    /// Convert this UIntSetVal to a UIntSet
+    UIntSet toUIntSet() const;
+
+    /// Create a UIntSetVal from a UIntSet using the given ASTBuilder
+    static UIntSetVal* fromUIntSet(ASTBuilder* astBuilder, const UIntSet& uintSet);
+
+    void _toTextOverride(StringBuilder& out);
+    Val* _resolveImplOverride() { return this; }
+};
+
 
 template<typename F>
 void SubstitutionSet::forEachGenericSubstitution(F func) const
@@ -954,6 +1050,10 @@ inline bool isTypeEqualityWitness(Val* witness)
         }
         return true;
     }
+    else if (auto expandWitness = as<ExpandSubtypeWitness>(witness))
+    {
+        return isTypeEqualityWitness(expandWitness->getPatternTypeWitness());
+    }
     return false;
 }
 
