@@ -156,7 +156,7 @@ void EntryPoint::_collectGenericSpecializationParamsRec(Decl* decl)
     if (!genericDecl)
         return;
 
-    for (auto m : genericDecl->members)
+    for (auto m : genericDecl->getDirectMemberDecls())
     {
         if (auto genericTypeParam = as<GenericTypeParamDecl>(m))
         {
@@ -238,6 +238,25 @@ DeclRef<FuncDecl> findFunctionDeclByName(Module* translationUnit, Name* name, Di
     {
         entryPointFuncDeclRef = declRefExpr->declRef.as<FuncDecl>();
 
+        if (!entryPointFuncDeclRef)
+        {
+            if (auto genDeclRef = as<GenericDecl>(declRefExpr->declRef))
+            {
+                SharedSemanticsContext context(
+                    translationUnit->getLinkage(),
+                    translationUnit,
+                    sink);
+                SemanticsVisitor visitor(&context);
+                entryPointFuncDeclRef = createDefaultSubstitutionsIfNeeded(
+                                            translationUnit->getASTBuilder(),
+                                            &visitor,
+                                            translationUnit->getASTBuilder()->getMemberDeclRef(
+                                                genDeclRef,
+                                                genDeclRef.getDecl()->inner))
+                                            .as<FuncDecl>();
+            }
+        }
+
         if (entryPointFuncDeclRef && getModule(entryPointFuncDeclRef.getDecl()) != translationUnit)
             entryPointFuncDeclRef = DeclRef<FuncDecl>();
     }
@@ -322,6 +341,7 @@ bool doStructFieldsHaveSemantic(Type* type)
     return doStructFieldsHaveSemanticImpl(type, seenTypes);
 }
 
+
 // Validate that an entry point function conforms to any additional
 // constraints based on the stage (and profile?) it specifies.
 void validateEntryPoint(EntryPoint* entryPoint, DiagnosticSink* sink)
@@ -357,7 +377,30 @@ void validateEntryPoint(EntryPoint* entryPoint, DiagnosticSink* sink)
     auto entryPointName = entryPointFuncDecl->getName();
 
     auto module = getModule(entryPointFuncDecl);
-    auto linkage = module->getLinkage();
+    auto linkage = entryPoint->getLinkage();
+
+    // Check if the return type is valid for a shader entry point
+    auto returnType = entryPointFuncDecl->returnType.type;
+    if (returnType)
+    {
+        // Use the existing getTypeTags functionality to check for resource types
+        // Create a temporary SemanticsVisitor to access getTypeTags
+        SharedSemanticsContext shared(linkage, module, sink);
+        SemanticsVisitor visitor(&shared);
+
+        auto typeTags = visitor.getTypeTags(returnType);
+        bool hasResourceOrUnsizedTypes = (((int)typeTags & (int)TypeTag::Opaque) != 0) ||
+                                         (((int)typeTags & (int)TypeTag::Unsized) != 0);
+
+        if (hasResourceOrUnsizedTypes)
+        {
+            sink->diagnose(
+                entryPointFuncDecl,
+                Diagnostics::entryPointCannotReturnResourceType,
+                entryPointName,
+                returnType);
+        }
+    }
 
     // Every entry point needs to have a stage specified either via
     // command-line/API options, or via an explicit `[shader("...")]` attribute.
@@ -448,6 +491,7 @@ void validateEntryPoint(EntryPoint* entryPoint, DiagnosticSink* sink)
     }
 
     bool canHaveVaryingInput = false;
+    bool shouldWarnOnNonUniformParam = true;
     switch (stage)
     {
     case Stage::Vertex:
@@ -462,6 +506,9 @@ void validateEntryPoint(EntryPoint* entryPoint, DiagnosticSink* sink)
     case Stage::Domain:
         canHaveVaryingInput = true;
         break;
+    case Stage::Dispatch:
+        shouldWarnOnNonUniformParam = false;
+        break;
     default:
         break;
     }
@@ -499,10 +546,57 @@ void validateEntryPoint(EntryPoint* entryPoint, DiagnosticSink* sink)
         // support varying input/output. We will automatically convert it to a 'uniform' parameter,
         // and diagnose a warning.
         addModifier(param, getCurrentASTBuilder()->create<HLSLUniformModifier>());
-        sink->diagnose(
-            param,
-            Diagnostics::nonUniformEntryPointParameterTreatedAsUniform,
-            param->getName());
+        if (shouldWarnOnNonUniformParam)
+        {
+            sink->diagnose(
+                param,
+                Diagnostics::nonUniformEntryPointParameterTreatedAsUniform,
+                param->getName());
+        }
+    }
+
+    // Attribute and keyword diagnostics. Check for the [[vk::binding]] and [[vk::push_constants]]
+    // attributes, and the register() and packoffset() keywords on entry point parameters. Slang
+    // currently ignores these, which can lead to user confusion whenever the output does not
+    // correspond to what was requested. Conversely, Slang silently generating output that just
+    // happens to align with what's requested can also lead to user confusion, with the user
+    // mistakenly believing that the modifiers are working as intended.
+    //
+    // Note that this only checks when they're used on entry point parameters.
+    for (const auto& param : entryPointFuncDecl->getParameters())
+    {
+        if (param->findModifier<GLSLBindingAttribute>())
+        {
+            sink->diagnose(
+                param,
+                Diagnostics::unhandledModOnEntryPointParameter,
+                "attribute '[[vk::binding(...)]]'",
+                param->getName());
+        }
+        if (param->findModifier<PushConstantAttribute>())
+        {
+            sink->diagnose(
+                param,
+                Diagnostics::unhandledModOnEntryPointParameter,
+                "attribute '[[vk::push_constant]]'",
+                param->getName());
+        }
+        if (param->findModifier<HLSLRegisterSemantic>())
+        {
+            sink->diagnose(
+                param,
+                Diagnostics::unhandledModOnEntryPointParameter,
+                "keyword 'register'",
+                param->getName());
+        }
+        if (param->findModifier<HLSLPackOffsetSemantic>())
+        {
+            sink->diagnose(
+                param,
+                Diagnostics::unhandledModOnEntryPointParameter,
+                "keyword 'packoffset'",
+                param->getName());
+        }
     }
 
     for (auto target : linkage->targets)
@@ -510,7 +604,8 @@ void validateEntryPoint(EntryPoint* entryPoint, DiagnosticSink* sink)
         auto targetCaps = target->getTargetCaps();
         auto stageCapabilitySet = entryPoint->getProfile().getCapabilityName();
         targetCaps.join(stageCapabilitySet);
-        if (targetCaps.isIncompatibleWith(entryPointFuncDecl->inferredCapabilityRequirements))
+        if (targetCaps.isIncompatibleWith(
+                CapabilitySet{entryPointFuncDecl->inferredCapabilityRequirements}))
         {
             // Incompatable means we don't support a set of abstract atoms.
             // Diagnose that we lack support for 'stage' and 'target' atoms with our provided
@@ -542,18 +637,42 @@ void validateEntryPoint(EntryPoint* entryPoint, DiagnosticSink* sink)
                 targetOptionSet.hasOption(CompilerOptionName::Profile) &&
                 (targetOptionSet.getIntOption(CompilerOptionName::Profile) !=
                  SLANG_PROFILE_UNKNOWN);
-            bool specificCapabilityRequested =
-                targetOptionSet.hasOption(CompilerOptionName::Capability) &&
-                (targetOptionSet.getIntOption(CompilerOptionName::Capability) !=
-                 SLANG_CAPABILITY_UNKNOWN);
+            bool specificCapabilityRequested = false;
+            for (auto atomVal : targetOptionSet.getArray(CompilerOptionName::Capability))
+            {
+                switch (atomVal.kind)
+                {
+                case CompilerOptionValueKind::Int:
+                    if (atomVal.intValue != SLANG_CAPABILITY_UNKNOWN)
+                        specificCapabilityRequested = true;
+                    break;
+                case CompilerOptionValueKind::String:
+                    // User made a specific capability request
+                    specificCapabilityRequested = true;
+                    break;
+                }
+                if (specificCapabilityRequested)
+                    break;
+            }
+
+            if (auto declaredCapsMod =
+                    entryPointFuncDecl->findModifier<ExplicitlyDeclaredCapabilityModifier>())
+            {
+                // If the entry point has an explicitly declared capability, then we
+                // will merge that with the target capability set before checking if
+                // there is an implicit upgrade.
+                targetCaps.nonDestructiveJoin(declaredCapsMod->declaredCapabilityRequirements);
+            }
+
             // Only attempt to error if a specific profile or capability is requested
             if ((specificCapabilityRequested || specificProfileRequested) &&
                 targetCaps.atLeastOneSetImpliedInOther(
-                    entryPointFuncDecl->inferredCapabilityRequirements) ==
+                    CapabilitySet{entryPointFuncDecl->inferredCapabilityRequirements}) ==
                     CapabilitySet::ImpliesReturnFlags::NotImplied)
             {
                 CapabilitySet combinedSets = targetCaps;
-                combinedSets.join(entryPointFuncDecl->inferredCapabilityRequirements);
+                combinedSets.join(
+                    CapabilitySet{entryPointFuncDecl->inferredCapabilityRequirements});
                 CapabilityAtomSet addedAtoms{};
                 if (auto targetCapSet = targetCaps.getAtomSets())
                 {
@@ -590,7 +709,8 @@ bool resolveStageOfProfileWithEntryPoint(
     if (auto entryPointAttr = entryPointFuncDecl->findModifier<EntryPointAttribute>())
     {
         auto entryPointProfileStage = entryPointProfile.getStage();
-        auto entryPointStage = getStageFromAtom(entryPointAttr->capabilitySet.getTargetStage());
+        auto entryPointStage =
+            getStageFromAtom(CapabilitySet{entryPointAttr->capabilitySet}.getTargetStage());
 
         // Ensure every target is specifying the same stage as an entry-point
         // if a profile+stage was set, else user will not be aware that their
@@ -623,7 +743,7 @@ bool resolveStageOfProfileWithEntryPoint(
                 entryPointFuncDecl->getName(),
                 entryPointProfileStage,
                 entryPointStage);
-        entryPointProfile.additionalCapabilities.add(entryPointAttr->capabilitySet);
+        entryPointProfile.additionalCapabilities.add(CapabilitySet{entryPointAttr->capabilitySet});
         return true;
     }
     return false;
@@ -709,7 +829,7 @@ Name* getReflectionName(VarDeclBase* varDecl)
     return varDecl->getName();
 }
 
-Type* getParamType(ASTBuilder* astBuilder, DeclRef<VarDeclBase> paramDeclRef)
+Type* getParamValueType(ASTBuilder* astBuilder, DeclRef<ParamDecl> paramDeclRef)
 {
     auto paramType = getType(astBuilder, paramDeclRef);
     if (paramDeclRef.getDecl()->findModifier<NoDiffModifier>())
@@ -720,6 +840,36 @@ Type* getParamType(ASTBuilder* astBuilder, DeclRef<VarDeclBase> paramDeclRef)
     return paramType;
 }
 
+Type* getParamTypeWithModeWrapper(ASTBuilder* astBuilder, DeclRef<ParamDecl> paramDeclRef)
+{
+    auto paramValueType = getParamValueType(astBuilder, paramDeclRef);
+    auto paramMode = getParamPassingMode(paramDeclRef.getDecl());
+    return getParamTypeWithModeWrapper(astBuilder, paramValueType, paramMode);
+}
+
+Type* getParamTypeWithModeWrapper(
+    ASTBuilder* astBuilder,
+    Type* paramValueType,
+    ParamPassingMode paramMode)
+{
+    switch (paramMode)
+    {
+    case ParamPassingMode::In:
+        return paramValueType;
+    case ParamPassingMode::BorrowIn:
+        return astBuilder->getConstRefParamType(paramValueType);
+    case ParamPassingMode::Out:
+        return astBuilder->getOutParamType(paramValueType);
+    case ParamPassingMode::BorrowInOut:
+        return astBuilder->getBorrowInOutParamType(paramValueType);
+    case ParamPassingMode::Ref:
+        return astBuilder->getRefParamType(paramValueType);
+    default:
+        SLANG_UNEXPECTED("unhandled parameter-passing mode");
+        UNREACHABLE_RETURN(paramValueType);
+    }
+}
+
 void Module::_collectShaderParams()
 {
     // We are going to walk the global declarations in the body of the
@@ -746,7 +896,7 @@ void Module::_collectShaderParams()
     for (Index i = 0; i < workList.getCount(); i++)
     {
         auto moduleDecl = workList[i];
-        for (auto globalDecl : moduleDecl->members)
+        for (auto globalDecl : moduleDecl->getDirectMemberDecls())
         {
             if (auto globalVar = as<VarDecl>(globalDecl))
             {
@@ -887,6 +1037,51 @@ RefPtr<ComponentType> fillRequirements(ComponentType* inComponentType)
     return componentType;
 }
 
+bool parseTypeConformanceArgString(
+    UnownedStringSlice optionString,
+    UnownedStringSlice& outTypeName,
+    UnownedStringSlice& outInterfaceName,
+    Index& outSequentialId)
+{
+    // The expected format for the type conformance argument is:
+    // `TypeName:InterfaceName[=SequentialId]`
+    //
+    // Where `TypeName` is the name of a concrete type, `InterfaceName`
+    // is the name of an interface type, and `SequentialId` is an optional
+    // integer that specifies a sequential ID for the conformance.
+    //
+    // If the string does not match this format, we will return false.
+
+    outTypeName = UnownedStringSlice();
+    outInterfaceName = UnownedStringSlice();
+    outSequentialId = -1;
+    auto colonPos = optionString.indexOf(':');
+    if (colonPos < 0)
+    {
+        // If there is no colon, then the string is invalid.
+        return false;
+    }
+    outTypeName = optionString.head(colonPos);
+    auto interfaceNameStart = colonPos + 1;
+    auto equalsPos = optionString.indexOf('=');
+    if (equalsPos < interfaceNameStart)
+    {
+        // If there is no equals sign, then the interface name goes to the end of the string.
+        outInterfaceName = optionString.tail(interfaceNameStart);
+    }
+    else
+    {
+        // If there is an equals sign, then the interface name goes up to that point.
+        outInterfaceName =
+            optionString.subString(interfaceNameStart, equalsPos - interfaceNameStart);
+        // The sequential ID is the part after the equals sign.
+        auto sequentialIdString = optionString.tail(equalsPos + 1);
+        if (SLANG_FAILED(StringUtil::parseInt(sequentialIdString, outSequentialId)))
+            return false;
+    }
+    return true;
+}
+
 /// Create a component type to represent the "global scope" of a compile request.
 ///
 /// This component type will include all the modules and their global
@@ -937,6 +1132,85 @@ RefPtr<ComponentType> createUnspecializedGlobalComponentType(FrontEndCompileRequ
             CompositeComponentType::create(linkage, translationUnitComponentTypes);
     }
 
+    List<RefPtr<ComponentType>> conformanceComponents;
+
+    // Find and include all type conformances specified through compiler options.
+    for (auto conformances :
+         compileRequest->optionSet.getArray(CompilerOptionName::TypeConformance))
+    {
+        auto stringValue = conformances.stringValue.getUnownedSlice();
+        UnownedStringSlice typeName, interfaceName;
+        Index sequentialId = -1;
+        if (!parseTypeConformanceArgString(stringValue, typeName, interfaceName, sequentialId))
+        {
+            compileRequest->getSink()->diagnose(
+                SourceLoc(),
+                Diagnostics::invalidTypeConformanceOptionString,
+                stringValue);
+            continue;
+        }
+        auto concreteType = globalComponentType->getTypeFromString(
+            String(typeName).getBuffer(),
+            compileRequest->getSink());
+        if (!concreteType)
+        {
+            compileRequest->getSink()->diagnose(
+                SourceLoc(),
+                Diagnostics::invalidTypeConformanceOptionNoType,
+                stringValue,
+                typeName);
+            continue;
+        }
+        auto interfaceType = globalComponentType->getTypeFromString(
+            String(interfaceName).getBuffer(),
+            compileRequest->getSink());
+        if (!interfaceType)
+        {
+            compileRequest->getSink()->diagnose(
+                SourceLoc(),
+                Diagnostics::invalidTypeConformanceOptionNoType,
+                stringValue,
+                interfaceName);
+            continue;
+        }
+        ComPtr<slang::ITypeConformance> conformanceComponent;
+        ComPtr<ISlangBlob> diagnostics;
+        compileRequest->getLinkage()->createTypeConformanceComponentType(
+            (slang::TypeReflection*)concreteType,
+            (slang::TypeReflection*)interfaceType,
+            conformanceComponent.writeRef(),
+            sequentialId,
+            diagnostics.writeRef());
+        if (!conformanceComponent)
+        {
+            // If we failed to create the conformance component, then
+            // we should report the diagnostics that were generated.
+            //
+            compileRequest->getSink()->diagnose(
+                SourceLoc(),
+                Diagnostics::cannotCreateTypeConformance,
+                stringValue);
+            if (diagnostics)
+            {
+                compileRequest->getSink()->diagnoseRaw(
+                    Severity::Error,
+                    UnownedStringSlice((char*)diagnostics->getBufferPointer()));
+            }
+            continue;
+        }
+        conformanceComponents.add(static_cast<TypeConformance*>(conformanceComponent.get()));
+    }
+
+    if (conformanceComponents.getCount() > 0)
+    {
+        // If we found any type conformances, then we will
+        // create a composite component type that includes
+        // the global component type and the conformance components.
+        //
+        conformanceComponents.add(globalComponentType);
+        globalComponentType = CompositeComponentType::create(linkage, conformanceComponents);
+    }
+
     return fillRequirements(globalComponentType);
 }
 
@@ -1076,9 +1350,20 @@ RefPtr<ComponentType> createUnspecializedGlobalAndEntryPointsComponentType(
 RefPtr<ComponentType::SpecializationInfo> Module::_validateSpecializationArgsImpl(
     SpecializationArg const* args,
     Index argCount,
+    Index& outConsumedArgCount,
     DiagnosticSink* sink)
 {
-    SLANG_ASSERT(argCount == getSpecializationParamCount());
+    if (argCount < getSpecializationParamCount())
+    {
+        sink->diagnose(
+            SourceLoc(),
+            Diagnostics::mismatchSpecializationArguments,
+            getSpecializationParamCount(),
+            argCount);
+        return nullptr;
+    }
+    outConsumedArgCount = getSpecializationParamCount();
+    argCount = outConsumedArgCount;
 
     SharedSemanticsContext semanticsContext(getLinkage(), this, sink);
     SemanticsVisitor visitor(&semanticsContext);
@@ -1293,23 +1578,27 @@ static void _extractSpecializationArgs(
 RefPtr<ComponentType::SpecializationInfo> EntryPoint::_validateSpecializationArgsImpl(
     SpecializationArg const* inArgs,
     Index inArgCount,
+    Index& outConsumedArgCount,
     DiagnosticSink* sink)
 {
+    SLANG_AST_BUILDER_RAII(getLinkage()->getASTBuilder());
+
     auto args = inArgs;
     auto argCount = inArgCount;
 
     SharedSemanticsContext sharedSemanticsContext(getLinkage(), nullptr, sink);
     SemanticsVisitor visitor(&sharedSemanticsContext);
 
-    // The first N arguments will be for the explicit generic parameters
+    // The last N arguments will be for the implicit existential arguments
     // of the entry point (if it has any).
     //
+    auto existentialSpecializationParamCount = getExistentialSpecializationParamCount();
     auto genericSpecializationParamCount = getGenericSpecializationParamCount();
-    SLANG_ASSERT(argCount >= genericSpecializationParamCount);
 
     RefPtr<EntryPointSpecializationInfo> info = new EntryPointSpecializationInfo();
 
     DeclRef<FuncDecl> specializedFuncDeclRef = m_funcDeclRef;
+    Index genericArgCount = genericSpecializationParamCount;
     if (genericSpecializationParamCount)
     {
         // We need to construct a generic application and use
@@ -1319,83 +1608,93 @@ RefPtr<ComponentType::SpecializationInfo> EntryPoint::_validateSpecializationArg
         auto genericDeclRef = m_funcDeclRef.getParent().as<GenericDecl>();
         SLANG_ASSERT(genericDeclRef); // otherwise we wouldn't have generic parameters
 
-        List<Val*> genericArgs;
+        bool isVariadic =
+            (genericDeclRef.getDecl()->getMembersOfType<GenericTypePackParamDecl>().getCount() !=
+             0);
 
-        for (Index ii = 0; ii < genericSpecializationParamCount; ++ii)
+        // If function is variadic generic, it will consume all the provided arguments.
+        if (isVariadic)
+            genericArgCount = argCount - existentialSpecializationParamCount;
+
+        if (genericArgCount < 0)
+        {
+            sink->diagnose(
+                SourceLoc(),
+                Diagnostics::mismatchSpecializationArguments,
+                genericSpecializationParamCount + existentialSpecializationParamCount,
+                argCount);
+            return nullptr;
+        }
+
+        List<Expr*> genericArgs;
+
+        auto astBuilder = getLinkage()->getASTBuilder();
+        for (Index ii = 0; ii < genericArgCount; ++ii)
         {
             auto specializationArg = args[ii];
-            genericArgs.add(specializationArg.val);
-        }
-        auto astBuilder = getLinkage()->getASTBuilder();
-        for (auto constraintDecl : getMembersOfType<GenericTypeConstraintDecl>(
-                 getLinkage()->getASTBuilder(),
-                 DeclRef<ContainerDecl>(genericDeclRef)))
-        {
-            DeclRef<GenericTypeConstraintDecl> constraintDeclRef =
-                astBuilder->getDirectDeclRef(constraintDecl.getDecl());
-            int argIndex = -1;
-            int ii = 0;
-
-            // Find the generic parameter type (T) that this constraint (T:IFoo) is applying to.
-            auto genericParamType = getSub(astBuilder, constraintDeclRef);
-            auto genParamDeclRefType = as<DeclRefType>(genericParamType);
-            if (!genParamDeclRefType)
+            if (specializationArg.expr)
             {
+                genericArgs.add(specializationArg.expr);
                 continue;
             }
-            auto genParamDeclRef = genParamDeclRefType->getDeclRef();
-
-            // Find the generic argument index of the corresponding generic parameter type in the
-            // generic parameter set.
-            //
-            for (auto member : genericDeclRef.getDecl()->getMembersOfType<GenericTypeParamDecl>())
+            if (auto typeVal = as<Type>(specializationArg.val))
             {
-                if (member == genParamDeclRef.getDecl())
+                auto typeExpr = astBuilder->create<SharedTypeExpr>();
+                typeExpr->type = astBuilder->getTypeType(typeVal);
+                genericArgs.add(typeExpr);
+            }
+            else if (auto intVal = as<ConstantIntVal>(specializationArg.val))
+            {
+                if (intVal->getType() == astBuilder->getBoolType())
                 {
-                    argIndex = ii;
-                    break;
+                    auto intExpr = astBuilder->create<BoolLiteralExpr>();
+                    intExpr->type = intVal->getType();
+                    intExpr->value = intVal->getValue() != 0;
+                    genericArgs.add(intExpr);
+                }
+                else
+                {
+                    auto intExpr = astBuilder->create<IntegerLiteralExpr>();
+                    intExpr->type = intVal->getType();
+                    intExpr->value = intVal->getValue();
+                    genericArgs.add(intExpr);
                 }
-                ii++;
-            }
-            if (argIndex == -1)
-            {
-                SLANG_ASSERT(!"generic parameter not found in generic decl");
-                continue;
-            }
-            auto sub = as<Type>(args[argIndex].val);
-            if (!sub)
-            {
-                sink->diagnose(
-                    constraintDecl,
-                    Diagnostics::expectedTypeForSpecializationArg,
-                    argIndex);
-                continue;
-            }
-
-            auto sup = getSup(astBuilder, constraintDeclRef);
-            auto subTypeWitness = visitor.isSubtype(sub, sup, IsSubTypeOptions::None);
-            if (subTypeWitness)
-            {
-                genericArgs.add(subTypeWitness);
             }
             else
             {
-                // TODO: diagnose a problem here
-                sink->diagnose(
-                    constraintDecl,
-                    Diagnostics::typeArgumentDoesNotConformToInterface,
-                    sub,
-                    sup);
-                continue;
+                sink->diagnose(SourceLoc(), Diagnostics::invalidFormOfSpecializationArg, ii + 1);
             }
         }
+        auto genAppExpr = astBuilder->create<GenericAppExpr>();
+        auto genExpr = astBuilder->create<VarExpr>();
+        genExpr->declRef = genericDeclRef;
+        genExpr->type = astBuilder->getOrCreate<GenericDeclRefType>();
+        genExpr->checked = true;
+        genAppExpr->functionExpr = genExpr;
+        genAppExpr->arguments = _Move(genericArgs);
+        auto checkedExpr = visitor.CheckTerm(genAppExpr);
+        if (auto partiallyAppliedExpr = as<PartiallyAppliedGenericExpr>(checkedExpr))
+        {
+            // If checked generic is partially applied generic, we try to force conversion into
+            // a fully defined declref by calling `trySolveConstraintSystem`.
+            SemanticsVisitor::ConstraintSystem system;
+            system.genericDecl = genericDeclRef.getDecl();
+            ConversionCost outCost;
+            specializedFuncDeclRef = visitor
+                                         .trySolveConstraintSystem(
+                                             &system,
+                                             genericDeclRef,
+                                             partiallyAppliedExpr->knownGenericArgs.getArrayView(),
+                                             outCost)
+                                         .as<FuncDecl>();
+        }
+        else if (auto declRefExpr = as<DeclRefExpr>(checkedExpr))
+        {
+            specializedFuncDeclRef = declRefExpr->declRef.as<FuncDecl>();
+        }
 
-        specializedFuncDeclRef =
-            getLinkage()
-                ->getASTBuilder()
-                ->getGenericAppDeclRef(genericDeclRef, genericArgs.getArrayView())
-                .as<FuncDecl>();
-        SLANG_ASSERT(specializedFuncDeclRef);
+        if (!specializedFuncDeclRef)
+            return nullptr;
     }
 
     info->specializedFuncDeclRef = specializedFuncDeclRef;
@@ -1405,11 +1704,19 @@ RefPtr<ComponentType::SpecializationInfo> EntryPoint::_validateSpecializationArg
     // specialization parameters, attached to the value parameters
     // of the entry point.
     //
-    args += genericSpecializationParamCount;
-    argCount -= genericSpecializationParamCount;
+    args += genericArgCount;
+    argCount -= genericArgCount;
+    outConsumedArgCount = genericArgCount + existentialSpecializationParamCount;
 
-    auto existentialSpecializationParamCount = getExistentialSpecializationParamCount();
-    SLANG_ASSERT(argCount == existentialSpecializationParamCount);
+    if (argCount < existentialSpecializationParamCount)
+    {
+        sink->diagnose(
+            SourceLoc(),
+            Diagnostics::mismatchSpecializationArguments,
+            genericSpecializationParamCount + existentialSpecializationParamCount,
+            argCount);
+        return nullptr;
+    }
 
     for (Index ii = 0; ii < existentialSpecializationParamCount; ++ii)
     {
@@ -1578,7 +1885,7 @@ Type* Linkage::specializeType(
         unspecializedType,
         SourceLoc());
 
-    assert(specializationParams.getCount() == argCount);
+    SLANG_ASSERT(specializationParams.getCount() == argCount);
 
     ExpandedSpecializationArgs specializationArgs;
     for (Int aa = 0; aa < argCount; ++aa)
