@@ -228,6 +228,30 @@ public:
     /// Get the diagnostic sink
     DiagnosticSink* getSink() { return m_codeGenContext->getSink(); }
 
+    /// Diagnose a diagnostic only once per diagnostic ID and all parameters
+    template<typename... Args>
+    void diagnoseOnce(SourceLoc loc, DiagnosticInfo const& diagnostic, Args&&... args)
+    {
+        // For diagnostics with parameters, we'll use all parameters to create a unique key
+        // This prevents duplicate diagnostics while allowing different parameter combinations
+        if constexpr (sizeof...(args) > 0)
+        {
+            String key = String(diagnostic.id);
+            ((key = key + "|" + String(args)), ...); // Fold expression to append all args
+            if (!m_reportedDiagnosticKeys.add(key))
+                return;
+        }
+        else
+        {
+            // For diagnostics without parameters, just use the ID
+            if (!m_reportedDiagnosticIds.add(diagnostic.id))
+                return;
+        }
+
+        // Report the diagnostic
+        getSink()->diagnose(loc, diagnostic, std::forward<Args>(args)...);
+    }
+
     /// Get the code gen target
     CodeGenTarget getTarget() { return m_target; }
     /// Get the source style
@@ -516,6 +540,7 @@ public:
     static IRWaveSizeDecoration* getComputeWaveSize(IRFunc* func, Int* outWaveSize);
 
 protected:
+    virtual bool shouldEmitOnlyHeader() { return false; }
     virtual void emitGlobalParamDefaultVal(IRGlobalParam* inst) { SLANG_UNUSED(inst); }
     virtual void emitPostDeclarationAttributesForType(IRInst* type) { SLANG_UNUSED(type); }
     virtual String getTargetBuiltinVarName(IRInst* inst, IRTargetBuiltinVarName builtinName);
@@ -615,6 +640,11 @@ protected:
     virtual void emitSwitchDecorationsImpl(IRSwitch* switchInst) { SLANG_UNUSED(switchInst); }
     virtual void emitSwitchCaseSelectorsImpl(const SwitchRegion::Case* currentCase, bool isDefault);
 
+    /// Returns true if this target supports fall-through in switch statements.
+    /// Targets like HLSL (FXC) and WGSL don't support fall-through.
+    /// This is used by the restructure pass to decide whether to preserve fall-through.
+    virtual bool supportsSwitchFallThrough() { return true; }
+
     virtual void emitFuncDecorationImpl(IRDecoration* decoration) { SLANG_UNUSED(decoration); }
     virtual void emitLivenessImpl(IRInst* inst);
 
@@ -738,6 +768,19 @@ protected:
     OrderedHashSet<IRStringLit*> m_requiredPreludes;
 
     Dictionary<const char*, IRStringLit*> m_builtinPreludes;
+
+    // Rename entry point if target doesn't allow the name (e.g., 'main')
+    virtual String maybeMakeEntryPointNameValid(String name, DiagnosticSink* sink);
+
+    // Indicates if we are emiting for Optix cooperative vector.
+    bool isOptixCoopVec = false;
+
+    // Set of diagnostic IDs that have already been reported to prevent duplicates
+    HashSet<int> m_reportedDiagnosticIds;
+
+    // Set of diagnostic keys (ID + all parameters) that have already been reported to prevent
+    // duplicates
+    HashSet<String> m_reportedDiagnosticKeys;
 };
 
 } // namespace Slang
