@@ -46,13 +46,25 @@ vec<T,A> _slang_vectorReshape(vec<T,N> v)
 }
 )";
 
+static const char* kMetalBuiltinPreludeMatrixFmod = R"(
+template<typename T, int A, int B>
+matrix<T,A,B> _slang_matrixFmod(matrix<T,A,B> m1, matrix<T,A,B> m2)
+{
+    matrix<T,A,B> result;
+    for (int i = 0; i < A; i++)
+        result[i] = fmod(m1[i], m2[i]);
+    return result;
+}
+)";
+
+
 void MetalSourceEmitter::_emitHLSLDecorationSingleString(
     const char* name,
     IRFunc* entryPoint,
     IRStringLit* val)
 {
     SLANG_UNUSED(entryPoint);
-    assert(val);
+    SLANG_ASSERT(val);
 
     m_writer->emit("[[");
     m_writer->emit(name);
@@ -138,9 +150,9 @@ void MetalSourceEmitter::_emitHLSLTextureType(IRTextureTypeBase* texType)
     {
     case SLANG_RESOURCE_ACCESS_READ:
         {
-            // Metal does not support access::sample for texture buffers, so we need to emit
-            // access::read instead.
-            if (texType->GetBaseShape() == SLANG_TEXTURE_BUFFER)
+            // Metal does not support access::sample for texture buffers and multisampled textures,
+            // so we need to emit access::read instead.
+            if (texType->GetBaseShape() == SLANG_TEXTURE_BUFFER || texType->isMultisample())
                 m_writer->emit("access::read");
             else
                 m_writer->emit("access::sample");
@@ -290,9 +302,35 @@ static IRImageSubscript* isTextureAccess(IRInst* inst)
     return as<IRImageSubscript>(getRootAddr(inst->getOperand(0)));
 }
 
+void MetalSourceEmitter::emitImageOperandWithAccessor(IRInst* imageOperand)
+{
+    emitOperand(imageOperand, getInfo(EmitOp::Postfix));
+
+    // Check if the image operand is a pointer type
+    if (as<IRPtrTypeBase>(imageOperand->getDataType()))
+    {
+        m_writer->emit("->");
+    }
+    else
+    {
+        m_writer->emit(".");
+    }
+}
+
 void MetalSourceEmitter::emitAtomicImageCoord(IRImageSubscript* inst)
 {
-    auto resourceType = as<IRResourceTypeBase>(inst->getImage()->getDataType());
+    auto imageDataType = inst->getImage()->getDataType();
+    auto resourceType = as<IRResourceTypeBase>(imageDataType);
+
+    // If the image data type is a pointer, get the value type
+    if (!resourceType)
+    {
+        if (auto ptrType = as<IRPtrTypeBase>(imageDataType))
+        {
+            resourceType = as<IRResourceTypeBase>(ptrType->getValueType());
+        }
+    }
+
     if (auto textureType = as<IRTextureType>(resourceType))
     {
         if (as<IRVectorType>(textureType->getElementType()))
@@ -303,7 +341,7 @@ void MetalSourceEmitter::emitAtomicImageCoord(IRImageSubscript* inst)
                 "atomic operation on non-scalar texture");
         }
     }
-    bool isArray = getIntVal(resourceType->getIsArrayInst()) != 0;
+    bool isArray = resourceType && getIntVal(resourceType->getIsArrayInst()) != 0;
     if (isArray)
     {
         emitOperand(inst->getCoord(), getInfo(EmitOp::Postfix));
@@ -375,8 +413,7 @@ bool MetalSourceEmitter::tryEmitInstStmtImpl(IRInst* inst)
         bool isImageOp = false;
         if (auto imageSubscript = isTextureAccess(inst))
         {
-            emitOperand(imageSubscript->getImage(), getInfo(EmitOp::Postfix));
-            m_writer->emit(".");
+            emitImageOperandWithAccessor(imageSubscript->getImage());
             m_writer->emit(imageFunc);
             m_writer->emit("(");
             emitAtomicImageCoord(imageSubscript);
@@ -400,16 +437,16 @@ bool MetalSourceEmitter::tryEmitInstStmtImpl(IRInst* inst)
         else
             m_writer->emit(");\n");
     };
-    auto diagnoseFloatAtommic = [&]()
+    auto diagnoseFloatAtomic = [&]()
     {
         getSink()->diagnose(
             inst,
             Diagnostics::unsupportedTargetIntrinsic,
-            "floating point atomic operation");
+            "Unsupported floating point atomic operation");
     };
     switch (inst->getOp())
     {
-    case kIROp_discard:
+    case kIROp_Discard:
         m_writer->emit("discard_fragment();\n");
         return true;
     case kIROp_MetalAtomicCast:
@@ -433,14 +470,14 @@ bool MetalSourceEmitter::tryEmitInstStmtImpl(IRInst* inst)
     case kIROp_AtomicLoad:
         {
             if (isFloatingType(inst->getDataType()))
-                diagnoseFloatAtommic();
+                diagnoseFloatAtomic();
 
             emitInstResultDecl(inst);
             bool isImageOp = false;
             if (auto imageSubscript = isTextureAccess(inst))
             {
-                emitOperand(imageSubscript->getImage(), getInfo(EmitOp::Postfix));
-                m_writer->emit(".atomic_load(");
+                emitImageOperandWithAccessor(imageSubscript->getImage());
+                m_writer->emit("atomic_load(");
                 emitAtomicImageCoord(imageSubscript);
                 isImageOp = true;
             }
@@ -465,8 +502,8 @@ bool MetalSourceEmitter::tryEmitInstStmtImpl(IRInst* inst)
             bool isImageOp = false;
             if (auto imageSubscript = isTextureAccess(inst))
             {
-                emitOperand(imageSubscript->getImage(), getInfo(EmitOp::Postfix));
-                m_writer->emit(".atomic_store(");
+                emitImageOperandWithAccessor(imageSubscript->getImage());
+                m_writer->emit("atomic_store(");
                 emitAtomicImageCoord(imageSubscript);
                 isImageOp = true;
             }
@@ -488,7 +525,7 @@ bool MetalSourceEmitter::tryEmitInstStmtImpl(IRInst* inst)
     case kIROp_AtomicExchange:
         {
             if (isFloatingType(inst->getDataType()))
-                diagnoseFloatAtommic();
+                diagnoseFloatAtomic();
 
             emitAtomicOp("atomic_exchange", "atomic_exchange_explicit");
             return true;
@@ -496,7 +533,7 @@ bool MetalSourceEmitter::tryEmitInstStmtImpl(IRInst* inst)
     case kIROp_AtomicCompareExchange:
         {
             if (isFloatingType(inst->getDataType()))
-                diagnoseFloatAtommic();
+                diagnoseFloatAtomic();
 
             bool isImageOp = false;
             auto imageSubscript = isTextureAccess(inst);
@@ -515,8 +552,8 @@ bool MetalSourceEmitter::tryEmitInstStmtImpl(IRInst* inst)
             m_writer->emit(";\n");
             if (imageSubscript)
             {
-                emitOperand(imageSubscript->getImage(), getInfo(EmitOp::Postfix));
-                m_writer->emit(".atomic_compare_exchange_weak(");
+                emitImageOperandWithAccessor(imageSubscript->getImage());
+                m_writer->emit("atomic_compare_exchange_weak(");
                 emitAtomicImageCoord(imageSubscript);
             }
             else
@@ -543,17 +580,11 @@ bool MetalSourceEmitter::tryEmitInstStmtImpl(IRInst* inst)
         }
     case kIROp_AtomicAdd:
         {
-            if (isFloatingType(inst->getDataType()))
-                diagnoseFloatAtommic();
-
             emitAtomicOp("atomic_fetch_add", "atomic_fetch_add_explicit");
             return true;
         }
     case kIROp_AtomicSub:
         {
-            if (isFloatingType(inst->getDataType()))
-                diagnoseFloatAtommic();
-
             emitAtomicOp("atomic_fetch_sub", "atomic_fetch_sub_explicit");
             return true;
         }
@@ -575,7 +606,7 @@ bool MetalSourceEmitter::tryEmitInstStmtImpl(IRInst* inst)
     case kIROp_AtomicMin:
         {
             if (isFloatingType(inst->getDataType()))
-                diagnoseFloatAtommic();
+                diagnoseFloatAtomic();
 
             emitAtomicOp("atomic_fetch_min", "atomic_fetch_min_explicit");
             return true;
@@ -583,7 +614,7 @@ bool MetalSourceEmitter::tryEmitInstStmtImpl(IRInst* inst)
     case kIROp_AtomicMax:
         {
             if (isFloatingType(inst->getDataType()))
-                diagnoseFloatAtommic();
+                diagnoseFloatAtomic();
 
             emitAtomicOp("atomic_fetch_max", "atomic_fetch_max_explicit");
             return true;
@@ -594,8 +625,8 @@ bool MetalSourceEmitter::tryEmitInstStmtImpl(IRInst* inst)
             bool isImageOp = false;
             if (auto imageSubscript = isTextureAccess(inst))
             {
-                emitOperand(imageSubscript->getImage(), getInfo(EmitOp::Postfix));
-                m_writer->emit(".atomic_fetch_add(");
+                emitImageOperandWithAccessor(imageSubscript->getImage());
+                m_writer->emit("atomic_fetch_add(");
                 emitAtomicImageCoord(imageSubscript);
                 isImageOp = true;
             }
@@ -622,8 +653,8 @@ bool MetalSourceEmitter::tryEmitInstStmtImpl(IRInst* inst)
             bool isImageOp = false;
             if (auto imageSubscript = isTextureAccess(inst))
             {
-                emitOperand(imageSubscript->getImage(), getInfo(EmitOp::Postfix));
-                m_writer->emit(".atomic_fetch_sub(");
+                emitImageOperandWithAccessor(imageSubscript->getImage());
+                m_writer->emit("atomic_fetch_sub(");
                 emitAtomicImageCoord(imageSubscript);
                 isImageOp = true;
             }
@@ -750,7 +781,16 @@ bool MetalSourceEmitter::tryEmitInstExprImpl(IRInst* inst, const EmitOpInfo& inO
         }
     case kIROp_FRem:
         {
-            m_writer->emit("fmod(");
+            if (as<IRMatrixType>(inst->getOperand(0)->getDataType()) &&
+                as<IRMatrixType>(inst->getOperand(1)->getDataType()))
+            {
+                ensurePrelude(kMetalBuiltinPreludeMatrixFmod);
+                m_writer->emit("_slang_matrixFmod(");
+            }
+            else
+            {
+                m_writer->emit("fmod(");
+            }
             emitOperand(inst->getOperand(0), getInfo(EmitOp::General));
             m_writer->emit(", ");
             emitOperand(inst->getOperand(1), getInfo(EmitOp::General));
@@ -937,6 +977,11 @@ bool MetalSourceEmitter::tryEmitInstExprImpl(IRInst* inst, const EmitOpInfo& inO
             }
             return true;
         }
+    case kIROp_PtrLit:
+        {
+            m_writer->emit("nullptr");
+            return true;
+        }
     default:
         break;
     }
@@ -1072,9 +1117,7 @@ void MetalSourceEmitter::emitSimpleTypeImpl(IRType* type)
     {
     case kIROp_VoidType:
     case kIROp_BoolType:
-    case kIROp_Int8Type:
     case kIROp_IntType:
-    case kIROp_UInt8Type:
     case kIROp_UIntType:
     case kIROp_FloatType:
     case kIROp_HalfType:
@@ -1082,6 +1125,12 @@ void MetalSourceEmitter::emitSimpleTypeImpl(IRType* type)
             m_writer->emit(getDefaultBuiltinTypeName(type->getOp()));
             return;
         }
+    case kIROp_Int8Type:
+        m_writer->emit("char");
+        return;
+    case kIROp_UInt8Type:
+        m_writer->emit("uchar");
+        return;
     case kIROp_Int64Type:
         m_writer->emit("long");
         return;
@@ -1103,6 +1152,9 @@ void MetalSourceEmitter::emitSimpleTypeImpl(IRType* type)
     case kIROp_StructType:
         m_writer->emit(getName(type));
         return;
+    case kIROp_DescriptorHandleType:
+        emitType(type);
+        return;
 
     case kIROp_DoubleType:
         SLANG_UNEXPECTED("'double' type emitted");
@@ -1149,13 +1201,13 @@ void MetalSourceEmitter::emitSimpleTypeImpl(IRType* type)
             return;
         }
     case kIROp_PtrType:
-    case kIROp_InOutType:
-    case kIROp_OutType:
-    case kIROp_RefType:
-    case kIROp_ConstRefType:
+    case kIROp_BorrowInOutParamType:
+    case kIROp_OutParamType:
+    case kIROp_RefParamType:
+    case kIROp_BorrowInParamType:
         {
             auto ptrType = cast<IRPtrTypeBase>(type);
-            if (type->getOp() == kIROp_ConstRefType)
+            if (type->getOp() == kIROp_BorrowInParamType)
             {
                 m_writer->emit("const ");
             }
@@ -1163,6 +1215,7 @@ void MetalSourceEmitter::emitSimpleTypeImpl(IRType* type)
             switch (ptrType->getAddressSpace())
             {
             case AddressSpace::Global:
+            case AddressSpace::UserPointer:
                 m_writer->emit(" device");
                 m_writer->emit("*");
                 break;
@@ -1182,6 +1235,9 @@ void MetalSourceEmitter::emitSimpleTypeImpl(IRType* type)
                 m_writer->emit(" object_data");
                 m_writer->emit("*");
                 break;
+            default:
+                SLANG_UNEXPECTED("Unknown addressspace encountered.");
+                break;
             }
             return;
         }
@@ -1243,7 +1299,8 @@ void MetalSourceEmitter::emitSimpleTypeImpl(IRType* type)
             m_writer->emit("uint32_t device*");
             break;
         case kIROp_RaytracingAccelerationStructureType:
-            m_writer->emit("acceleration_structure<instancing>");
+            m_writer->emit(
+                "metal::raytracing::acceleration_structure<metal::raytracing::instancing>");
             break;
         default:
             SLANG_DIAGNOSE_UNEXPECTED(getSink(), SourceLoc(), "unhandled buffer type");
