@@ -174,7 +174,6 @@ static bool _isImageFormatCompatible(ImageFormat imageFormat, IRType* dataType)
 
     if (numElems != imageFormatInfo.channelCount)
     {
-        SLANG_ASSERT(!"Format doesn't match channel count");
         return false;
     }
 
@@ -446,7 +445,7 @@ const char* IntrinsicExpandContext::_emitSpecial(const char* cursor)
             // there is a format conversion required between the type associated by the resource and
             // the backing ImageFormat. Currently this is only implemented on CUDA, where there are
             // specialized versions of the RWTexture writes that will do a format conversion.
-            if (m_emitter->getTarget() == CodeGenTarget::CUDASource)
+            if (isCUDATarget(m_emitter->getTargetReq()))
             {
                 IRInst* resourceInst = m_callInst->getArg(0);
 
