@@ -40,7 +40,7 @@ struct RegionEndpoint
 
 struct BreakableRegionInfo
 {
-    IRVar* breakVar;
+    IRVar* keepRunningVar;
     IRBlock* breakBlock;
     IRBlock* headerBlock;
 };
@@ -119,11 +119,11 @@ struct CFGNormalizationPass
 
         builder->setInsertInto(currBlock);
 
-        builder->emitStore(info->breakVar, builder->getBoolValue(false));
+        builder->emitStore(info->keepRunningVar, builder->getBoolValue(false));
         builder->emitBranch(afterBlock);
 
         // Is after-block unreachable?
-        if (auto unreachInst = as<IRUnreachable>(afterBlock->getFirstOrdinaryInst()))
+        if (auto unreachInst = as<IRUnreachableBase>(afterBlock->getFirstOrdinaryInst()))
         {
             // Link it to the parentAfterBlock.
             builder->setInsertInto(afterBlock);
@@ -240,9 +240,13 @@ struct CFGNormalizationPass
             auto afterSplitAfterBlock = builder.emitBlock();
 
             builder.setInsertInto(afterSplitBlock);
-            auto breakFlagValue = builder.emitLoad(parentRegion->breakVar);
+            auto keepRunningFlagValue = builder.emitLoad(parentRegion->keepRunningVar);
 
-            builder.emitIfElse(breakFlagValue, block, afterSplitAfterBlock, afterSplitAfterBlock);
+            builder.emitIfElse(
+                keepRunningFlagValue,
+                block,
+                afterSplitAfterBlock,
+                afterSplitAfterBlock);
 
             // At this point, we need to place afterSplitAfterBlock between
             // at the _end_ of this region, but we aren't there yet (and
@@ -261,14 +265,14 @@ struct CFGNormalizationPass
             auto terminator = currentBlock->getTerminator();
             switch (terminator->getOp())
             {
-            case kIROp_unconditionalBranch:
+            case kIROp_UnconditionalBranch:
                 {
                     auto targetBlock = as<IRUnconditionalBranch>(terminator)->getTargetBlock();
                     currentBlock = targetBlock;
                     break;
                 }
 
-            case kIROp_ifElse:
+            case kIROp_IfElse:
                 {
                     auto ifElse = as<IRIfElse>(terminator);
 
@@ -373,7 +377,7 @@ struct CFGNormalizationPass
                     // originally unreachable, all potential paths to it must have
                     // broken out of the region.
                     //
-                    if (auto unreachInst = as<IRUnreachable>(afterBlock->getTerminator()))
+                    if (auto unreachInst = as<IRUnreachableBase>(afterBlock->getTerminator()))
                     {
                         // Link it to the parentAfterBlock.
                         builder.setInsertInto(afterBlock);
@@ -437,7 +441,7 @@ struct CFGNormalizationPass
                     break;
                 }
 
-            case kIROp_loop:
+            case kIROp_Loop:
             case kIROp_Switch:
                 {
                     auto breakBlock = normalizeBreakableRegion(terminator);
@@ -505,7 +509,7 @@ struct CFGNormalizationPass
 
         switch (branchInst->getOp())
         {
-        case kIROp_loop:
+        case kIROp_Loop:
             {
                 BreakableRegionInfo info;
                 info.breakBlock = as<IRLoop>(branchInst)->getBreakBlock();
@@ -514,13 +518,13 @@ struct CFGNormalizationPass
                 // Emit var into parent block.
                 builder.setInsertBefore(as<IRBlock>(branchInst->getParent())->getTerminator());
 
-                // Create and initialize break var to true
+                // Create and initialize keepRunning var to true
                 // true -> no break yet.
                 // false -> atleast one break statement hit.
                 //
-                info.breakVar = builder.emitVar(builder.getBoolType());
-                builder.addNameHintDecoration(info.breakVar, UnownedStringSlice("_bflag"));
-                builder.emitStore(info.breakVar, builder.getBoolValue(true));
+                info.keepRunningVar = builder.emitVar(builder.getBoolType());
+                builder.addNameHintDecoration(info.keepRunningVar, UnownedStringSlice("_runFlag"));
+                builder.emitStore(info.keepRunningVar, builder.getBoolValue(true));
 
                 // If the loop is trivial (i.e. single iteration, with no
                 // edges actually in a loop), we're just going to remove
@@ -624,7 +628,7 @@ struct CFGNormalizationPass
                     auto cond = ifElse->getCondition();
 
                     builder.setInsertBefore(ifElse);
-                    auto breakFlagVal = builder.emitLoad(info.breakVar);
+                    auto breakFlagVal = builder.emitLoad(info.keepRunningVar);
 
                     // Need to invert the break flag if the loop is
                     // on the false side.
@@ -665,8 +669,8 @@ struct CFGNormalizationPass
                 // true -> no break yet.
                 // false -> atleast one break statement hit.
                 //
-                info.breakVar = builder.emitVar(builder.getBoolType());
-                builder.emitStore(info.breakVar, builder.getBoolValue(true));
+                info.keepRunningVar = builder.emitVar(builder.getBoolType());
+                builder.emitStore(info.keepRunningVar, builder.getBoolValue(true));
 
                 // Go over case labels and normalize all sub-regions.
                 for (UIndex ii = 0; ii < switchInst->getCaseCount(); ii++)
@@ -708,6 +712,7 @@ struct CFGNormalizationPass
 };
 
 void normalizeCFG(
+    TargetProgram* targetProgram,
     IRModule* module,
     IRGlobalValueWithCode* func,
     IRCFGNormalizationPass const& options)
@@ -753,11 +758,12 @@ void normalizeCFG(
     // no longer dominate the use. We fix these up by going through the IR and create temp
     // vars for such uses.
     sortBlocksInFunc(func);
-    legalizeDefUse(func);
+    legalizeDefUse(func, targetProgram);
 
-    disableIRValidationAtInsert();
-    constructSSA(module, func);
-    enableIRValidationAtInsert();
+    {
+        auto validationScope = disableIRValidationScope();
+        constructSSA(module, func);
+    }
 
     module->invalidateAnalysisForInst(func);
 #if _DEBUG
