@@ -8,6 +8,6 @@ class TargetRequest;
 
 void checkForRecursiveTypes(IRModule* module, DiagnosticSink* sink);
 
-void checkForRecursiveFunctions(TargetRequest* target, IRModule* module, DiagnosticSink* sink);
+void checkForRecursiveFunctions(IRModule* module, TargetRequest* target, DiagnosticSink* sink);
 
 } // namespace Slang
