@@ -75,7 +75,7 @@ struct DeadCodeEliminationContext
         }
     }
 
-    IRInst* getUndefInst()
+    IRInst* getUnitPoisonVal()
     {
         if (!undefInst)
         {
@@ -84,7 +84,7 @@ struct DeadCodeEliminationContext
                 builder.setInsertBefore(firstChild);
             else
                 builder.setInsertInto(module->getModuleInst());
-            undefInst = Slang::getUndefInst(builder, module);
+            undefInst = Slang::getUnitPoisonVal(builder, module);
         }
         return undefInst;
     }
@@ -109,12 +109,20 @@ struct DeadCodeEliminationContext
             //
             markInstAsLive(root);
 
-            // Ensure there is a global undef inst that is always alive.
-            // This undef inst will be used to fill in weak-referencing uses
-            // whose used value is marked as dead and eliminated.
-            // We always make sure this undef inst is available to prevent
-            // infiniate oscilating loops.
-            markInstAsLive(getUndefInst());
+            // We will use a single instruction representing an undefined
+            // value (using the `poison` instruction) as the replacement
+            // for any lingering uses of a value that is marked as dead
+            // (those uses logically represent "weak" references to the
+            // original value, since they didn't cause it to be kept alive).
+            //
+            // We need to make sure that this unique undefined value is
+            // treated as live, because if it is allowed to be removed then
+            // the compiler can get into an infinite oscillating loop where
+            // we determine that this instruction is not live and thus try
+            // to remove it and replace it with... a new instruction that
+            // is exactly like it.
+            //
+            markInstAsLive(getUnitPoisonVal());
 
             // Marking the module as live should have
             // seeded our work list, so we can now start
@@ -233,7 +241,7 @@ struct DeadCodeEliminationContext
             //
             if (inst->hasUses())
             {
-                inst->replaceUsesWith(getUndefInst());
+                inst->replaceUsesWith(getUnitPoisonVal());
             }
 
             if (inst->getOp() == kIROp_Param)
@@ -283,16 +291,6 @@ struct DeadCodeEliminationContext
     }
 };
 
-bool isFirstBlock(IRInst* inst)
-{
-    auto block = as<IRBlock>(inst);
-    if (!block)
-        return false;
-    if (!block->getParent())
-        return false;
-    return block->getParent()->getFirstBlock() == block;
-}
-
 bool isPtrUsed(IRInst* ptrInst)
 {
     for (auto use = ptrInst->firstUse; use; use = use->nextUse)
@@ -589,17 +587,22 @@ bool shouldInstBeLiveIfParentIsLive(IRInst* inst, IRDeadCodeEliminationOptions o
                 break;
             }
         }
-        for (auto decor : innerInst->getDecorations())
+
+        if (innerInst)
         {
-            switch (decor->getOp())
+            for (auto decor : innerInst->getDecorations())
             {
-            case kIROp_ForwardDerivativeDecoration:
-            case kIROp_UserDefinedBackwardDerivativeDecoration:
-            case kIROp_PrimalSubstituteDecoration:
-                shouldKeptAliveIfImported = true;
-                break;
+                switch (decor->getOp())
+                {
+                case kIROp_ForwardDerivativeDecoration:
+                case kIROp_UserDefinedBackwardDerivativeDecoration:
+                case kIROp_PrimalSubstituteDecoration:
+                    shouldKeptAliveIfImported = true;
+                    break;
+                }
             }
         }
+
         if (isImported && shouldKeptAliveIfImported)
             return true;
     }
@@ -667,16 +670,17 @@ bool shouldInstBeLiveIfParentIsLive(IRInst* inst, IRDeadCodeEliminationOptions o
 
 bool isWeakReferenceOperand(IRInst* inst, UInt operandIndex)
 {
+    SLANG_UNUSED(operandIndex);
     // There are some type of operands that needs to be treated as
     // "weak" references -- they can never hold things alive, and
     // whenever we delete the referenced value, these operands needs
     // to be replaced with `undef`.
     switch (inst->getOp())
     {
-    case kIROp_BoundInterfaceType:
+    /*case kIROp_BoundInterfaceType:
         if (inst->getOperand(operandIndex)->getOp() == kIROp_WitnessTable)
             return true;
-        break;
+        break;*/
     case kIROp_SpecializationDictionaryItem:
         // Ignore all operands of SpecializationDictionaryItem.
         // This inst is used as a cache and shouldn't hold anything alive.
