@@ -53,6 +53,7 @@ struct IntroduceExplicitGlobalContextPass
                 hoistGlobalVarOptions = HoistGlobalVarOptions::PlainGlobal;
                 break;
             case CodeGenTarget::CUDASource:
+            case CodeGenTarget::CUDAHeader:
                 hoistableGlobalObjectKind = GlobalObjectKind::GlobalVar;
 
                 // One important exception is that CUDA *does* support
@@ -221,19 +222,14 @@ struct IntroduceExplicitGlobalContextPass
                     // skip any global-scope parameters that are varying instead of
                     // uniform.
                     //
-                    switch (m_target)
+                    if (isCUDATarget(m_target) || isCPUTarget(m_target))
                     {
-                    case CodeGenTarget::CUDASource:
-                    case CodeGenTarget::CPPSource:
-                        {
-                            auto layoutDecor = globalParam->findDecoration<IRLayoutDecoration>();
-                            SLANG_ASSERT(layoutDecor);
-                            auto layout = as<IRVarLayout>(layoutDecor->getLayout());
-                            SLANG_ASSERT(layout);
-                            if (isVaryingParameter(layout))
-                                continue;
-                        }
-                        break;
+                        auto layoutDecor = globalParam->findDecoration<IRLayoutDecoration>();
+                        SLANG_ASSERT(layoutDecor);
+                        auto layout = as<IRVarLayout>(layoutDecor->getLayout());
+                        SLANG_ASSERT(layout);
+                        if (isVaryingParameter(layout))
+                            continue;
                     }
 
                     // Because of upstream passes, we expect there to be only a
@@ -246,7 +242,8 @@ struct IntroduceExplicitGlobalContextPass
                     // For CUDA output, we want to leave the global uniform
                     // parameter where it is, because it will translate to
                     // a global `__constant__` variable.
-                    if (m_target == CodeGenTarget::CUDASource)
+                    if (m_target == CodeGenTarget::CUDASource ||
+                        m_target == CodeGenTarget::CUDAHeader)
                         continue;
 
                     GlobalParamInfo globalParamInfo;
@@ -294,7 +291,7 @@ struct IntroduceExplicitGlobalContextPass
         // it is responsible for introducing the explicit entry-point
         // parameter that is used for passing in the global param(s).
         //
-        if (m_target != CodeGenTarget::CPPSource)
+        if (!isCPUTarget(m_target))
         {
             if (m_globalParams.getCount() == 0 && m_globalVars.getCount() == 0)
             {
@@ -309,6 +306,12 @@ struct IntroduceExplicitGlobalContextPass
         // type with a name hint of `KernelContext`.
         //
         m_contextStructType = builder.createStructType();
+        if (m_target == CodeGenTarget::CPPHeader)
+        {
+            builder.addExternCppDecoration(
+                m_contextStructType,
+                UnownedTerminatedStringSlice("KernelContext"));
+        }
         builder.addNameHintDecoration(
             m_contextStructType,
             UnownedTerminatedStringSlice("KernelContext"));
@@ -316,8 +319,7 @@ struct IntroduceExplicitGlobalContextPass
         // The context will usually be passed around by pointer,
         // so we get and cache that pointer type up front.
         //
-        m_contextStructPtrType =
-            builder.getPtrType(kIROp_PtrType, m_contextStructType, getAddressSpaceOfLocal());
+        m_contextStructPtrType = builder.getPtrType(m_contextStructType, getAddressSpaceOfLocal());
 
 
         // The first step will be to create fields in the `KernelContext`
@@ -441,6 +443,12 @@ struct IntroduceExplicitGlobalContextPass
         // Clone all original decorations to the new struct key.
         IRCloneEnv cloneEnv;
         cloneInstDecorationsAndChildren(&cloneEnv, m_module, originalInst, key);
+        if (m_target == CodeGenTarget::CPPHeader)
+        {
+            builder.addExternCppDecoration(
+                key,
+                originalInst->findDecoration<IRNameHintDecoration>()->getName());
+        }
 
         // We end by making note of the key that was created
         // for the instruction, so that we can use the key
@@ -498,7 +506,7 @@ struct IntroduceExplicitGlobalContextPass
             globalParam.entryPointParam->insertBefore(firstOrdinary);
         }
 
-        if (m_target == CodeGenTarget::CPPSource && m_globalParams.getCount() == 0)
+        if (isCPUTarget(m_target) && m_globalParams.getCount() == 0)
         {
             // The nature of our current ABI for entry points on CPU
             // means that we need an explicit parameter to be *declared*
@@ -565,6 +573,9 @@ struct IntroduceExplicitGlobalContextPass
                 builder.emitStore(fieldPtr, var);
             }
         }
+
+        // Update entry point function type after potentially adding parameters.
+        fixUpFuncType(entryPointFunc);
     }
 
     void replaceUsesOfGlobalParam(IRGlobalParam* globalParam)
@@ -630,7 +641,7 @@ struct IntroduceExplicitGlobalContextPass
 
         auto ptrType = getGlobalVarPtrType(globalVar);
         if (fieldInfo.needDereference)
-            ptrType = builder.getPtrType(kIROp_PtrType, ptrType, getAddressSpaceOfLocal());
+            ptrType = builder.getPtrType(ptrType, getAddressSpaceOfLocal());
 
         // We then iterate over the uses of the variable,
         // being careful to defend against the use/def information
@@ -661,6 +672,12 @@ struct IntroduceExplicitGlobalContextPass
                 ptr = builder.emitLoad(ptr);
             use->set(ptr);
         }
+
+        // We've replaced all uses of the global var so we should remove it.
+        // We leave decorations on the global var above, so if we do not remove it
+        // here, the global var will not be processed for elimination in
+        // eliminateDeadCode.
+        globalVar->removeAndDeallocate();
     }
 
     IRInst* findOrCreateContextPtrForInst(IRInst* inst)
@@ -718,6 +735,9 @@ struct IntroduceExplicitGlobalContextPass
         addKernelContextNameHint(contextParam);
         contextParam->insertBefore(firstBlock->getFirstOrdinaryInst());
 
+        // Update the type of the function to reflect this new parameter.
+        fixUpFuncType(func);
+
         // The new parameter can be registered as the context value
         // to be used for `func` right away.
         //
