@@ -2,7 +2,10 @@
 #include "slang-ir-inline.h"
 
 #include "../core/slang-performance-profiler.h"
+#include "slang-ir-specialize-address-space.h"
 #include "slang-ir-ssa-simplification.h"
+#include "slang-ir-util.h"
+
 // This file provides general facilities for inlining function calls.
 
 //
@@ -290,6 +293,126 @@ struct InliningPassBase
         return true;
     }
 
+    // Find an existing debug function for the given function
+    IRInst* findExistingDebugFunc(IRFunc* func)
+    {
+        if (auto debugFuncDecor = func->findDecoration<IRDebugFuncDecoration>())
+        {
+            return debugFuncDecor->getDebugFunc();
+        }
+        return nullptr;
+    }
+
+    struct DebugInlineInfo
+    {
+        IRInst* newDebugInlinedAt = nullptr;
+        IRInst* calleeDebugFunc = nullptr;
+    };
+
+    // Sets up the initial debug information structures required *before* inlining a call site.
+    //
+    // This function performs the following steps:
+    // 1. Checks if the callee function has associated debug location information.
+    // 2. Obtain the call inst's existing debug information.
+    // 3. Finds the last `IRDebugLine` preceding the `call` instruction to determine the source
+    //    location (line, col, file) of the call site.
+    // 4. Emits an `IRDebugInlinedAt` instruction with outer set to callDebugInlinedAt.
+    // 5. Inserts the newly created `IRDebugInlinedAt` instruction immediately *before* the `call`
+    // instruction.
+    DebugInlineInfo emitCalleeDebugInlinedAt(IRCall* call, IRFunc* callee, IRBuilder& builder)
+    {
+        IRDebugLine* lastDebugLine = nullptr;
+
+        if (!callee->findDecoration<IRDebugLocationDecoration>())
+        {
+            return DebugInlineInfo();
+        }
+
+        // Check if the call inst is part of an existing scope. If yes, then we restore
+        // that scope after the inlining of callee. This case can occur when we have out of
+        // order inlining. See forceinline-basic-block-inline-order.slang test for that use
+        // case. If we are travesing back the call inst and if we find a DebugNoScope, it means
+        // that there's another function that was inlined. We don't want that scope. If the call
+        // inst truly belongs to another DebugScope, then we should hit a DebugScope inst
+        // *before* we see a DebugNoScope
+        IRDebugScope* callDebugScope = nullptr;
+        builder.setInsertAfter(call);
+        for (IRInst* inst = call->getPrevInst(); inst; inst = inst->getPrevInst())
+        {
+            if (as<IRDebugNoScope>(inst))
+            {
+                break;
+            }
+            if (as<IRDebugScope>(inst))
+            {
+                callDebugScope = as<IRDebugScope>(inst);
+                builder.emitDebugScope(callDebugScope->getScope(), callDebugScope->getInlinedAt());
+                break;
+            }
+        }
+        if (!callDebugScope)
+        {
+            builder.emitDebugNoScope();
+        }
+
+        IRDebugInlinedAt* callDebugInlinedAt = nullptr;
+        for (IRInst* inst = call->getPrevInst(); inst; inst = inst->getPrevInst())
+        {
+            if (as<IRDebugNoScope>(inst))
+            {
+                break;
+            }
+            if (as<IRDebugInlinedAt>(inst))
+            {
+                callDebugInlinedAt = as<IRDebugInlinedAt>(inst);
+                break;
+            }
+        }
+
+        // Find the last IRDebugLine to extract debug info.
+        for (IRInst* inst = call->getPrevInst(); inst; inst = inst->getPrevInst())
+        {
+            if (auto debugLine = as<IRDebugLine>(inst))
+            {
+                lastDebugLine = debugLine;
+                break;
+            }
+        }
+
+        if (!lastDebugLine)
+            return DebugInlineInfo();
+
+        auto calleeDebugFunc = findExistingDebugFunc(callee);
+
+        // The caller func is the right lexical scope needed for nsight to show where
+        // the function is getting inlined inside.
+        if (auto callerFunc = getParentFunc(call))
+        {
+            // When `maybeAddDebugLocationDecoration()` failed to find the source
+            // location, IRDebugFuncDecoration is expected to be absent.
+            if (auto callerDebugFunc = findExistingDebugFunc(callerFunc))
+            {
+                builder.setInsertBefore(call);
+                auto newDebugInlinedAt = builder.emitDebugInlinedAt(
+                    lastDebugLine->getLineStart(),
+                    lastDebugLine->getColStart(),
+                    lastDebugLine->getSource(),
+                    callerDebugFunc,
+                    callDebugInlinedAt);
+
+                return DebugInlineInfo{newDebugInlinedAt, calleeDebugFunc};
+            }
+            else
+            {
+                // It is more likely to be a bug when IRDebugLocationDecoration exists and
+                // IRDebugFuncDecoration doesn't exist.
+                SLANG_ASSERT(nullptr == callerFunc->findDecoration<IRDebugLocationDecoration>());
+            }
+        }
+
+        return DebugInlineInfo();
+    }
+
     /// Inline the given `callSite`, which is assumed to have been validated
     void inlineCallSite(CallSiteInfo const& callSite)
     {
@@ -481,7 +604,9 @@ struct InliningPassBase
     void inlineSingleBlockFuncBody(
         CallSiteInfo const& callSite,
         IRCloneEnv* env,
-        IRBuilder* builder)
+        IRBuilder* builder,
+        IRInst* newDebugInlinedAt,
+        IRInst* calleeDebugFunc)
     {
         auto call = callSite.call;
         auto callee = callSite.callee;
@@ -495,18 +620,22 @@ struct InliningPassBase
         // them into the same basic block as the `call`.
         //
         builder->setInsertBefore(call);
+        IRInst* calleeDebugScope = nullptr;
+        if (calleeDebugFunc && newDebugInlinedAt)
+        {
+            calleeDebugScope = builder->emitDebugScope(calleeDebugFunc, newDebugInlinedAt);
+        }
 
         // Along the way, we will detect any `return` instruction,
         // and remember the (clone of the) returned value.
         //
         IRInst* returnVal = nullptr;
-
+        List<IRDebugInlinedAt*> debugInlinedInsts;
         for (auto inst : firstBlock->getChildren())
         {
             switch (inst->getOp())
             {
             default:
-                // In the common case we just clone the instruction as-is
                 _cloneInstWithSourceLoc(callSite, env, builder, inst);
                 break;
 
@@ -524,9 +653,40 @@ struct InliningPassBase
                 SLANG_ASSERT(!returnVal);
                 returnVal = findCloneForOperand(env, inst->getOperand(0));
                 break;
+
+            case kIROp_DebugNoScope:
+                {
+                    if (calleeDebugScope)
+                        _cloneInstWithSourceLoc(callSite, env, builder, calleeDebugScope);
+                    break;
+                }
+
+            case kIROp_DebugInlinedAt:
+                {
+                    auto clonedInst = _cloneInstWithSourceLoc(callSite, env, builder, inst);
+                    if (!as<IRDebugInlinedAt>(clonedInst)->isOuterInlinedPresent())
+                        debugInlinedInsts.add(as<IRDebugInlinedAt>(clonedInst));
+                    break;
+                }
+            }
+        }
+        // For any debugInlinedAt without an outerinlinedAt, emit a new debugInlinedAt with the
+        // outer set, and delete the older debugInlinedAt
+        for (auto inst : debugInlinedInsts)
+        {
+            if (newDebugInlinedAt && !inst->isOuterInlinedPresent())
+            {
+                builder->setInsertAfter(inst);
+                auto newInlinedAt = builder->emitDebugInlinedAt(
+                    inst->getLine(),
+                    inst->getCol(),
+                    inst->getFile(),
+                    inst->getDebugFunc(),
+                    newDebugInlinedAt);
+                inst->replaceUsesWith(newInlinedAt);
+                inst->removeAndDeallocate();
             }
         }
-
         // We are going to remove the original `call` now that the callee
         // has been inlined, but before we do that we need to replace
         // all uses of the `call` with whatever value was produced by the
@@ -547,10 +707,38 @@ struct InliningPassBase
     }
 
     /// Inline the body of the callee for `callSite`.
+    // Here is the algorithm for inserting debug information for slang inlined functions:
+    // 1. Check if the call inst belongs to an existing debug scope and find corresponding
+    // debugInlinedAt. [callDebugScope, callDebugInlinedAt]
+    //    1a. If callDebugScope exists, emit this debug Scope* after* the call inst.
+    // 2. Emit a new DebugInlinedAt inst, with debugFunc of the callee, and outer debugInlinedAt is
+    // callDebugInlinedAt. [newDebugInlinedAt]
+    //    2a. If calleDebugScope does not exist, emit debugNoScope after the call inst.
+    // 3. Clone the callee body.
+    // 4. For each cloned block, do this:
+    //    4a.Emit a new DebugScope inst setting the current scope to newDebugInlinedAt.
+    //    [calleeDebugScope] 4b.Emit a DebugNoScope at the end of each block. 4c.If callDebugScope
+    //    exists, do not emit a DebugNoScope for the last block.
+    // 5. For each cloned debugInlinedAt inst, if its outer inlined at operand is null, set it to
+    // the new DebugInlinedAt inst inserted at the top of the block.
+    // 6. For each cloned debugNoScope inst, replace it with calleeDebugScope. (This is because all
+    // cloned insts are in callee's scope).
     void inlineFuncBody(CallSiteInfo const& callSite, IRCloneEnv* env, IRBuilder* builder)
     {
-        auto call = callSite.call;
         auto callee = callSite.callee;
+        auto call = callSite.call;
+
+        auto debugInlineInfo = emitCalleeDebugInlinedAt(call, callee, *builder);
+
+        // Collect all arguments that are pointers, so we can propagate their address
+        // spaces to the cloned instructions after inlining.
+        List<IRInst*> ptrArgList;
+        for (UInt i = 0; i < call->getArgCount(); i++)
+        {
+            auto arg = call->getArg(i);
+            if (as<IRPtrTypeBase>(arg->getDataType()))
+                ptrArgList.add(arg);
+        }
 
         // If the callee consists of a single basic block *and* that block
         // ends with a `return` instruction, then we can apply a simple approach
@@ -561,9 +749,38 @@ struct InliningPassBase
         SLANG_ASSERT(firstBlock);
         if (!firstBlock->getNextBlock() && as<IRReturn>(firstBlock->getTerminator()))
         {
-            inlineSingleBlockFuncBody(callSite, env, builder);
-            return;
+            inlineSingleBlockFuncBody(
+                callSite,
+                env,
+                builder,
+                debugInlineInfo.newDebugInlinedAt,
+                debugInlineInfo.calleeDebugFunc);
         }
+        else
+        {
+            // If the callee has multiple blocks, use the more complex inlining approach
+            inlineMultipleBlockFuncBody(
+                callSite,
+                env,
+                builder,
+                debugInlineInfo.newDebugInlinedAt,
+                debugInlineInfo.calleeDebugFunc);
+        }
+
+        // Propagate the address space from the argument to the cloned instructions.
+        propagateAddressSpaceFromInsts(_Move(ptrArgList));
+    }
+
+    // Inline the body of the callee for `callSite`, for a callee that has multiple basic blocks.
+    void inlineMultipleBlockFuncBody(
+        CallSiteInfo const& callSite,
+        IRCloneEnv* env,
+        IRBuilder* builder,
+        IRInst* newDebugInlinedAt,
+        IRInst* calleeDebugFunc)
+    {
+        auto call = callSite.call;
+        auto callee = callSite.callee;
 
         // If the callee has any non-trivial control flow (multiple basic blocks
         // and terminators other than `return`), we will need to split the control
@@ -680,7 +897,53 @@ struct InliningPassBase
             }
             isFirstBlock = false;
         }
+        // For each existing debugNoScope inst, replace it with new debug scope we emit.
+        // For any debugInlinedAt without an outerinlinedAt, emit a new debugInlinedAt with the
+        // outer set, and delete the older debugInlinedAt
+        if (newDebugInlinedAt && callee->findDecoration<IRDebugLocationDecoration>())
+        {
+            for (auto calleeBlock : callee->getBlocks())
+            {
+                IRBlock* clonedBlock = as<IRBlock>(env->mapOldValToNew.getValue(calleeBlock));
+                setInsertBeforeOrdinaryInst(builder, clonedBlock->getFirstOrdinaryInst());
+                builder->emitDebugScope(calleeDebugFunc, newDebugInlinedAt);
 
+                List<IRInst*> debugNoScopeToRemove;
+                List<IRDebugInlinedAt*> debugInlinedAtToProcess;
+                for (auto inst : clonedBlock->getChildren())
+                {
+                    if (as<IRDebugNoScope>(inst))
+                    {
+                        debugNoScopeToRemove.add(inst);
+                    }
+                    if (auto inlinedAt = as<IRDebugInlinedAt>(inst))
+                    {
+                        if (!inlinedAt->isOuterInlinedPresent())
+                        {
+                            debugInlinedAtToProcess.add(inlinedAt);
+                        }
+                    }
+                }
+                for (auto inst : debugNoScopeToRemove)
+                {
+                    builder->setInsertAfter(inst);
+                    builder->emitDebugScope(calleeDebugFunc, newDebugInlinedAt);
+                    inst->removeAndDeallocate();
+                }
+                for (auto inlinedAt : debugInlinedAtToProcess)
+                {
+                    builder->setInsertAfter(inlinedAt);
+                    auto newInlinedAt = builder->emitDebugInlinedAt(
+                        inlinedAt->getLine(),
+                        inlinedAt->getCol(),
+                        inlinedAt->getFile(),
+                        inlinedAt->getDebugFunc(),
+                        newDebugInlinedAt);
+                    inlinedAt->replaceUsesWith(newInlinedAt);
+                    inlinedAt->removeAndDeallocate();
+                }
+            }
+        }
         // If there was a `returnVal` instruction that established
         // the return value of the inlined function, then that value
         // should be used to replace any uses of the original call.
@@ -741,12 +1004,20 @@ struct TypeInliningPass : InliningPassBase
 {
     typedef InliningPassBase Super;
 
-    TypeInliningPass(IRModule* module)
-        : Super(module)
+    TargetProgram* targetProgram;
+
+    bool shouldInlineBorrowRefTypesForSPIRV = false;
+
+    TypeInliningPass(IRModule* module, TargetProgram* inTargetProgram)
+        : Super(module), targetProgram(inTargetProgram)
     {
+        if (targetProgram->shouldEmitSPIRVDirectly())
+        {
+            shouldInlineBorrowRefTypesForSPIRV = true;
+        }
     }
 
-    bool doesTypeRequireInline(IRType* type, IRFunc* callee)
+    bool doesTypeRequireInline(IRType* type, IRInst* arg, IRFunc* callee)
     {
         // TODO(JS):
         // I guess there is a question here about what type around string requires
@@ -756,12 +1027,43 @@ struct TypeInliningPass : InliningPassBase
         const auto op = type->getOp();
         switch (op)
         {
-        case kIROp_RefType:
+        case kIROp_RefParamType:
             {
                 if (callee->findDecoration<IRNoRefInlineDecoration>())
                     return false;
                 return true;
             }
+        case kIROp_BorrowInParamType:
+        case kIROp_BorrowInOutParamType:
+            {
+                if (shouldInlineBorrowRefTypesForSPIRV)
+                {
+                    if (!arg)
+                        return true;
+
+                    // If the argument is a refernece to a local variable "memory object",
+                    // we don't need to inline.
+                    auto rootAddr = getRootAddr(arg);
+                    if (as<IRVar>(rootAddr) || as<IRGlobalVar>(rootAddr))
+                        return false;
+
+                    // Otherwise, we should inline the function before emitting SPIR-V,
+                    // because:
+                    // 1) SPIRV does not support expressing a function that takes a pointer
+                    // parameter whose address space is not Function or Private.
+                    // 2) SPIRV does not allow passing a pointer-typed argument that isn't a direct
+                    // reference to a variable "memory object". For example, you cannot pass the
+                    // result of an access chain to a pointer-typed parameter.
+                    //
+                    // Note that despite restriction rule (2) above, we still allow passing
+                    // an access chain to a local variable to a function in the Slang IR, and we
+                    // will introduce an additional copy when we emit SPIR-V. We do this to minimize
+                    // inlining in the Slang IR, and introducing additional copy for things already
+                    // in Function address space is generally less harmful.
+                    return true;
+                }
+                return false;
+            }
         case kIROp_StringType:
         case kIROp_NativeStringType:
             {
@@ -778,7 +1080,7 @@ struct TypeInliningPass : InliningPassBase
     {
         auto callee = info.callee;
 
-        if (doesTypeRequireInline(callee->getResultType(), callee))
+        if (doesTypeRequireInline(callee->getResultType(), nullptr, callee))
         {
             return true;
         }
@@ -786,7 +1088,7 @@ struct TypeInliningPass : InliningPassBase
         const auto count = Count(callee->getParamCount());
         for (Index i = 0; i < count; ++i)
         {
-            if (doesTypeRequireInline(callee->getParamType(UInt(i)), callee))
+            if (doesTypeRequireInline(callee->getParamType(UInt(i)), info.call->getArg(i), callee))
             {
                 return true;
             }
@@ -798,36 +1100,12 @@ struct TypeInliningPass : InliningPassBase
 
 } // namespace
 
-Result performTypeInlining(IRModule* module, DiagnosticSink* sink)
+Result performTypeInlining(IRModule* module, TargetProgram* targetProgram, DiagnosticSink* sink)
 {
     SLANG_UNUSED(sink);
 
-    // TODO(JS):
-    // This is perhaps not as efficient as might be desirable.
-    // A more optimized version might not need to pass over all of the module
-    // to find new call sites.
-    //
-    // Another problem here is recursion. Right now Slang compiler doesn't accept recursive input,
-    // but the Slang language is supposed to support recursion on targets that support it.
-    // There are GPU targets that allow recursion such as CUDA.
-    //
-    // Another approach would be (when enabled) when inlining occurs, would be instead of continuing
-    // *after*, to start the checks/inlining from where the inline took place.
-    //
-    while (true)
-    {
-        TypeInliningPass pass(module);
-        if (pass.considerAllCallSites())
-        {
-            // If there was a change try inlining again
-            continue;
-        }
-
-        // Done.
-        break;
-    }
-
-
+    TypeInliningPass pass(module, targetProgram);
+    pass.considerAllCallSites();
     return SLANG_OK;
 }
 
@@ -969,7 +1247,7 @@ struct GLSLResourceReturnFunctionInliningPass : InliningPassBase
         {
             if (isIllegalGLSLParameterType(param->getDataType()))
                 return true;
-            auto outType = as<IROutTypeBase>(param->getDataType());
+            auto outType = as<IROutParamTypeBase>(param->getDataType());
             if (!outType)
                 continue;
             auto outValueType = outType->getValueType();
@@ -980,7 +1258,7 @@ struct GLSLResourceReturnFunctionInliningPass : InliningPassBase
     }
 };
 
-void performGLSLResourceReturnFunctionInlining(TargetProgram* targetProgram, IRModule* module)
+void performGLSLResourceReturnFunctionInlining(IRModule* module, TargetProgram* targetProgram)
 {
     GLSLResourceReturnFunctionInliningPass pass(module);
     bool changed = true;
@@ -988,7 +1266,7 @@ void performGLSLResourceReturnFunctionInlining(TargetProgram* targetProgram, IRM
     while (changed)
     {
         changed = pass.considerAllCallSites();
-        simplifyIR(nullptr, module, IRSimplificationOptions::getFast(targetProgram));
+        simplifyIR(module, nullptr, IRSimplificationOptions::getFast(targetProgram));
     }
 }
 
@@ -1023,7 +1301,7 @@ struct IntrinsicFunctionInliningPass : InliningPassBase
                 hasSpvAsm = true;
                 continue;
             case kIROp_Load:
-            case kIROp_swizzle:
+            case kIROp_Swizzle:
             case kIROp_Store:
                 continue;
             default:
