@@ -1,8 +1,8 @@
 // slang-ir-layout.cpp
 #include "slang-ir-layout.h"
 
-#include "slang-ir-generics-lowering-context.h"
 #include "slang-ir-insts.h"
+#include "slang-ir-util.h"
 
 // This file implements facilities for computing and caching layout
 // information on IR types.
@@ -50,7 +50,7 @@
 namespace Slang
 {
 static Result _calcArraySizeAndAlignment(
-    CompilerOptionSet& optionSet,
+    TargetRequest* targetReq,
     IRTypeLayoutRules* rules,
     IRType* elementType,
     IRInst* elementCountInst,
@@ -68,7 +68,7 @@ static Result _calcArraySizeAndAlignment(
     }
 
     IRSizeAndAlignment elementTypeLayout;
-    SLANG_RETURN_ON_FAIL(getSizeAndAlignment(optionSet, rules, elementType, &elementTypeLayout));
+    SLANG_RETURN_ON_FAIL(getSizeAndAlignment(targetReq, rules, elementType, &elementTypeLayout));
 
     elementTypeLayout = rules->alignCompositeElement(elementTypeLayout);
     *outSizeAndAlignment = IRSizeAndAlignment(
@@ -83,22 +83,12 @@ IRIntegerValue getIntegerValueFromInst(IRInst* inst)
     return as<IRIntLit>(inst)->value.intVal;
 }
 
-static Result _calcSizeAndAlignment(
-    CompilerOptionSet& optionSet,
-    IRTypeLayoutRules* rules,
+Result IRTypeLayoutRules::calcSizeAndAlignment(
+    TargetRequest* targetReq,
     IRType* type,
     IRSizeAndAlignment* outSizeAndAlignment)
 {
-    int kPointerSize = 8;
-    switch (optionSet.getTarget())
-    {
-    case CodeGenTarget::HostCPPSource:
-    case CodeGenTarget::HostHostCallable:
-    case CodeGenTarget::HostExecutable:
-    case CodeGenTarget::HostSharedLibrary:
-        kPointerSize = (int)sizeof(void*);
-        break;
-    }
+    TargetBuiltinTypeLayoutInfo builtinTypeInfo = getBuiltinTypeLayoutInfo(targetReq);
 
     switch (type->getOp())
     {
@@ -124,17 +114,11 @@ static Result _calcSizeAndAlignment(
 
         BASE(Int64, 8);
         BASE(UInt64, 8);
-        BASE(IntPtr, kPointerSize);
-        BASE(UIntPtr, kPointerSize);
         BASE(Double, 8);
 
         // We are currently handling `bool` following the HLSL
         // precednet of storing it in 4 bytes.
         //
-        // TODO: It would be good to try to make this follow
-        // per-platform conventions, or at least to be able
-        // to use a 1-byte encoding where available.
-        //
         BASE(Bool, 4);
 
         // The Slang `void` type is treated as a zero-byte
@@ -142,7 +126,7 @@ static Result _calcSizeAndAlignment(
         //
         CASE(Void, 0, 1);
 
-#undef CASE
+#undef BASE
 
 #undef CASE
 
@@ -151,6 +135,8 @@ static Result _calcSizeAndAlignment(
             auto structType = cast<IRStructType>(type);
             IRSizeAndAlignment structLayout;
             IRIntegerValue offset = 0;
+            IRIntegerValue lastFieldAlignment = 0;
+            IRType* lastFieldType = NULL;
             bool seenFinalUnsizedArrayField = false;
             for (auto field : structType->getFields())
             {
@@ -159,17 +145,25 @@ static Result _calcSizeAndAlignment(
                 // subsequent offsets
                 SLANG_ASSERT(!seenFinalUnsizedArrayField);
 
+                IRSizeAndAlignment fieldTypeLayout;
+                SLANG_RETURN_ON_FAIL(
+                    getSizeAndAlignment(targetReq, this, field->getFieldType(), &fieldTypeLayout));
+                seenFinalUnsizedArrayField =
+                    fieldTypeLayout.size == IRSizeAndAlignment::kIndeterminateSize;
+
                 if (auto offsetDecor =
                         field->getKey()->findDecoration<IRVkStructOffsetDecoration>())
                 {
                     offset = offsetDecor->getOffset()->getValue();
                 }
-
-                IRSizeAndAlignment fieldTypeLayout;
-                SLANG_RETURN_ON_FAIL(
-                    getSizeAndAlignment(optionSet, rules, field->getFieldType(), &fieldTypeLayout));
-                seenFinalUnsizedArrayField =
-                    fieldTypeLayout.size == IRSizeAndAlignment::kIndeterminateSize;
+                else
+                {
+                    offset = adjustOffset(
+                        offset,
+                        fieldTypeLayout.size,
+                        lastFieldType,
+                        lastFieldAlignment);
+                }
 
                 structLayout.size = align(offset, fieldTypeLayout.alignment);
                 structLayout.alignment =
@@ -189,22 +183,16 @@ static Result _calcSizeAndAlignment(
                     builder.addDecoration(
                         field,
                         kIROp_OffsetDecoration,
-                        builder.getIntValue(intType, (IRIntegerValue)rules->ruleName),
+                        builder.getIntValue(intType, (IRIntegerValue)ruleName),
                         builder.getIntValue(intType, fieldOffset));
                 }
                 if (!seenFinalUnsizedArrayField)
                     structLayout.size += fieldTypeLayout.size;
                 offset = structLayout.size;
-                if (as<IRMatrixType>(field->getFieldType()) ||
-                    as<IRArrayTypeBase>(field->getFieldType()) ||
-                    as<IRStructType>(field->getFieldType()))
-                {
-                    offset = rules->adjustOffsetForNextAggregateMember(
-                        offset,
-                        fieldTypeLayout.alignment);
-                }
+                lastFieldType = field->getFieldType();
+                lastFieldAlignment = fieldTypeLayout.alignment;
             }
-            *outSizeAndAlignment = rules->alignCompositeElement(structLayout);
+            *outSizeAndAlignment = alignCompositeElement(structLayout);
             return SLANG_OK;
         }
         break;
@@ -214,8 +202,8 @@ static Result _calcSizeAndAlignment(
             auto arrayType = cast<IRArrayType>(type);
 
             return _calcArraySizeAndAlignment(
-                optionSet,
-                rules,
+                targetReq,
+                this,
                 arrayType->getElementType(),
                 arrayType->getElementCount(),
                 outSizeAndAlignment);
@@ -225,11 +213,7 @@ static Result _calcSizeAndAlignment(
     case kIROp_AtomicType:
         {
             auto atomicType = cast<IRAtomicType>(type);
-            _calcSizeAndAlignment(
-                optionSet,
-                rules,
-                atomicType->getElementType(),
-                outSizeAndAlignment);
+            calcSizeAndAlignment(targetReq, atomicType->getElementType(), outSizeAndAlignment);
             return SLANG_OK;
         }
         break;
@@ -238,8 +222,8 @@ static Result _calcSizeAndAlignment(
         {
             auto unsizedArrayType = cast<IRUnsizedArrayType>(type);
             getSizeAndAlignment(
-                optionSet,
-                rules,
+                targetReq,
+                this,
                 unsizedArrayType->getElementType(),
                 outSizeAndAlignment);
             outSizeAndAlignment->size = IRSizeAndAlignment::kIndeterminateSize;
@@ -251,8 +235,8 @@ static Result _calcSizeAndAlignment(
         {
             auto vecType = cast<IRVectorType>(type);
             IRSizeAndAlignment elementTypeLayout;
-            getSizeAndAlignment(optionSet, rules, vecType->getElementType(), &elementTypeLayout);
-            *outSizeAndAlignment = rules->getVectorSizeAndAlignment(
+            getSizeAndAlignment(targetReq, this, vecType->getElementType(), &elementTypeLayout);
+            *outSizeAndAlignment = getVectorSizeAndAlignment(
                 elementTypeLayout,
                 getIntegerValueFromInst(vecType->getElementCount()));
             return SLANG_OK;
@@ -263,7 +247,7 @@ static Result _calcSizeAndAlignment(
             auto anyValType = cast<IRAnyValueType>(type);
             outSizeAndAlignment->size = getIntVal(anyValType->getSize());
             outSizeAndAlignment->alignment = 4;
-            *outSizeAndAlignment = rules->alignCompositeElement(*outSizeAndAlignment);
+            *outSizeAndAlignment = alignCompositeElement(*outSizeAndAlignment);
             return SLANG_OK;
         }
         break;
@@ -271,17 +255,28 @@ static Result _calcSizeAndAlignment(
         {
             auto tupleType = cast<IRTupleType>(type);
             IRSizeAndAlignment resultLayout;
+            IRIntegerValue lastFieldAlignment = 0;
+            IRType* lastFieldType = NULL;
             for (UInt i = 0; i < tupleType->getOperandCount(); i++)
             {
                 auto elementType = tupleType->getOperand(i);
                 IRSizeAndAlignment fieldTypeLayout;
                 SLANG_RETURN_ON_FAIL(
-                    getSizeAndAlignment(optionSet, rules, (IRType*)elementType, &fieldTypeLayout));
+                    getSizeAndAlignment(targetReq, this, (IRType*)elementType, &fieldTypeLayout));
+                resultLayout.size = adjustOffset(
+                    resultLayout.size,
+                    fieldTypeLayout.size,
+                    lastFieldType,
+                    lastFieldAlignment);
                 resultLayout.size = align(resultLayout.size, fieldTypeLayout.alignment);
                 resultLayout.alignment =
                     std::max(resultLayout.alignment, fieldTypeLayout.alignment);
+
+                resultLayout.size += fieldTypeLayout.size;
+                lastFieldType = as<IRType>(elementType);
+                lastFieldAlignment = fieldTypeLayout.alignment;
             }
-            *outSizeAndAlignment = rules->alignCompositeElement(resultLayout);
+            *outSizeAndAlignment = alignCompositeElement(resultLayout);
             return SLANG_OK;
         }
         break;
@@ -294,18 +289,23 @@ static Result _calcSizeAndAlignment(
             return SLANG_OK;
         }
         break;
+    case kIROp_SetTagType:
+        {
+            outSizeAndAlignment->size = 4;
+            outSizeAndAlignment->alignment = 4;
+            return SLANG_OK;
+        }
+        break;
     case kIROp_InterfaceType:
         {
             auto interfaceType = cast<IRInterfaceType>(type);
-            auto size = SharedGenericsLoweringContext::getInterfaceAnyValueSize(
-                interfaceType,
-                interfaceType->sourceLoc);
+            auto size = getInterfaceAnyValueSize(interfaceType, interfaceType->sourceLoc);
             size += kRTTIHeaderSize;
             size = align(size, 4);
             IRSizeAndAlignment resultLayout;
             resultLayout.size = size;
             resultLayout.alignment = 4;
-            *outSizeAndAlignment = rules->alignCompositeElement(resultLayout);
+            *outSizeAndAlignment = alignCompositeElement(resultLayout);
             return SLANG_OK;
         }
         break;
@@ -318,8 +318,8 @@ static Result _calcSizeAndAlignment(
                 auto colVector =
                     builder.getVectorType(matType->getElementType(), matType->getRowCount());
                 return _calcArraySizeAndAlignment(
-                    optionSet,
-                    rules,
+                    targetReq,
+                    this,
                     colVector,
                     matType->getColumnCount(),
                     outSizeAndAlignment);
@@ -329,18 +329,20 @@ static Result _calcSizeAndAlignment(
                 auto rowVector =
                     builder.getVectorType(matType->getElementType(), matType->getColumnCount());
                 return _calcArraySizeAndAlignment(
-                    optionSet,
-                    rules,
+                    targetReq,
+                    this,
                     rowVector,
                     matType->getRowCount(),
                     outSizeAndAlignment);
             }
         }
         break;
-    case kIROp_OutType:
-    case kIROp_InOutType:
-    case kIROp_RefType:
-    case kIROp_ConstRefType:
+    case kIROp_IntPtrType:
+    case kIROp_UIntPtrType:
+    case kIROp_OutParamType:
+    case kIROp_BorrowInOutParamType:
+    case kIROp_RefParamType:
+    case kIROp_BorrowInParamType:
     case kIROp_RawPointerType:
     case kIROp_PtrType:
     case kIROp_NativePtrType:
@@ -348,12 +350,18 @@ static Result _calcSizeAndAlignment(
     case kIROp_NativeStringType:
     case kIROp_RaytracingAccelerationStructureType:
     case kIROp_FuncType:
+        // If we don't know the target, we can't tell the size of these types
+        // yet as it is target-dependent.
+        if (targetReq)
         {
-            *outSizeAndAlignment = IRSizeAndAlignment(kPointerSize, kPointerSize);
+            *outSizeAndAlignment = IRSizeAndAlignment(
+                builtinTypeInfo.genericPointerSize,
+                builtinTypeInfo.genericPointerSize);
             return SLANG_OK;
         }
         break;
     case kIROp_ScalarBufferLayoutType:
+    case kIROp_CBufferLayoutType:
     case kIROp_Std140BufferLayoutType:
     case kIROp_Std430BufferLayoutType:
     case kIROp_DefaultBufferLayoutType:
@@ -365,18 +373,25 @@ static Result _calcSizeAndAlignment(
             builder.setInsertBefore(type);
             auto uintType = builder.getUIntType();
             auto uint2Type = builder.getVectorType(uintType, 2);
-            return getSizeAndAlignment(optionSet, rules, uint2Type, outSizeAndAlignment);
+            return getSizeAndAlignment(targetReq, this, uint2Type, outSizeAndAlignment);
         }
     case kIROp_AttributedType:
         {
             auto attributedType = cast<IRAttributedType>(type);
             SLANG_ASSERT(attributedType->getAttr()->getOp() == kIROp_NoDiffAttr);
             return getSizeAndAlignment(
-                optionSet,
-                rules,
+                targetReq,
+                this,
                 attributedType->getBaseType(),
                 outSizeAndAlignment);
         }
+    case kIROp_EnumType:
+        {
+            auto enumType = cast<IREnumType>(type);
+            auto tagType = enumType->getTagType();
+            return calcSizeAndAlignment(targetReq, tagType, outSizeAndAlignment);
+        }
+        break;
     default:
         break;
     }
@@ -405,7 +420,7 @@ IRSizeAndAlignmentDecoration* findSizeAndAlignmentDecorationForLayout(
 }
 
 Result getSizeAndAlignment(
-    CompilerOptionSet& optionSet,
+    TargetRequest* targetReq,
     IRTypeLayoutRules* rules,
     IRType* type,
     IRSizeAndAlignment* outSizeAndAlignment)
@@ -417,7 +432,7 @@ Result getSizeAndAlignment(
     }
 
     IRSizeAndAlignment sizeAndAlignment;
-    SLANG_RETURN_ON_FAIL(_calcSizeAndAlignment(optionSet, rules, type, &sizeAndAlignment));
+    SLANG_RETURN_ON_FAIL(rules->calcSizeAndAlignment(targetReq, type, &sizeAndAlignment));
 
     if (auto module = type->getModule())
     {
@@ -452,7 +467,7 @@ IROffsetDecoration* findOffsetDecorationForLayout(
 }
 
 Result getOffset(
-    CompilerOptionSet& optionSet,
+    TargetRequest* targetReq,
     IRTypeLayoutRules* rules,
     IRStructField* field,
     IRIntegerValue* outOffset)
@@ -473,7 +488,7 @@ Result getOffset(
         return SLANG_FAIL;
 
     IRSizeAndAlignment structTypeLayout;
-    SLANG_RETURN_ON_FAIL(getSizeAndAlignment(optionSet, rules, structType, &structTypeLayout));
+    SLANG_RETURN_ON_FAIL(getSizeAndAlignment(targetReq, rules, structType, &structTypeLayout));
 
     if (auto decor = findOffsetDecorationForLayout(field, rules->ruleName))
     {
@@ -491,17 +506,66 @@ Result getOffset(
 struct NaturalLayoutRules : IRTypeLayoutRules
 {
     NaturalLayoutRules() { ruleName = IRTypeLayoutRuleName::Natural; }
-    virtual IRIntegerValue adjustOffsetForNextAggregateMember(
-        IRIntegerValue currentSize,
-        IRIntegerValue lastElementAlignment)
+    virtual IRIntegerValue adjustOffset(
+        IRIntegerValue offset,
+        IRIntegerValue elementSize,
+        IRType* lastFieldType,
+        IRIntegerValue lastFieldAlignment)
     {
-        SLANG_UNUSED(lastElementAlignment);
-        return currentSize;
+        SLANG_UNUSED(elementSize);
+        SLANG_UNUSED(lastFieldType);
+        SLANG_UNUSED(lastFieldAlignment);
+        return offset;
     }
+
     virtual IRSizeAndAlignment alignCompositeElement(IRSizeAndAlignment elementSize)
     {
         return elementSize;
     }
+
+    virtual IRSizeAndAlignment getVectorSizeAndAlignment(
+        IRSizeAndAlignment element,
+        IRIntegerValue count)
+    {
+        return IRSizeAndAlignment(element.size * count, element.alignment);
+    }
+};
+
+struct CLayoutRules : IRTypeLayoutRules
+{
+    CLayoutRules() { ruleName = IRTypeLayoutRuleName::C; }
+
+    virtual Result calcSizeAndAlignment(
+        TargetRequest* targetReq,
+        IRType* type,
+        IRSizeAndAlignment* outSizeAndAlignment)
+    {
+        if (type->getOp() == kIROp_BoolType)
+        {
+            *outSizeAndAlignment = IRSizeAndAlignment(1, 1);
+            return SLANG_OK;
+        }
+        return IRTypeLayoutRules::calcSizeAndAlignment(targetReq, type, outSizeAndAlignment);
+    }
+
+    virtual IRIntegerValue adjustOffset(
+        IRIntegerValue offset,
+        IRIntegerValue elementSize,
+        IRType* lastFieldType,
+        IRIntegerValue lastFieldAlignment)
+    {
+        SLANG_UNUSED(elementSize);
+        SLANG_UNUSED(lastFieldType);
+        return align(offset, (int)lastFieldAlignment);
+    }
+
+    virtual IRSizeAndAlignment alignCompositeElement(IRSizeAndAlignment elementSize)
+    {
+        IRSizeAndAlignment alignedSize = elementSize;
+        alignedSize.size = align(alignedSize.size, alignedSize.alignment);
+        return alignedSize;
+    }
+
     virtual IRSizeAndAlignment getVectorSizeAndAlignment(
         IRSizeAndAlignment element,
         IRIntegerValue count)
@@ -521,22 +585,30 @@ struct ConstantBufferLayoutRules : IRTypeLayoutRules
         return IRSizeAndAlignment(currentSize.size, 16);
     }
 
-    virtual IRIntegerValue adjustOffsetForNextAggregateMember(
-        IRIntegerValue currentSize,
-        IRIntegerValue lastElementAlignment)
+    virtual IRIntegerValue adjustOffset(
+        IRIntegerValue offset,
+        IRIntegerValue elementSize,
+        IRType* lastFieldType,
+        IRIntegerValue lastFieldAlignment)
     {
-        SLANG_UNUSED(lastElementAlignment);
-        return currentSize;
+        SLANG_UNUSED(lastFieldType);
+        SLANG_UNUSED(lastFieldAlignment);
+
+        // If the element would cross a 16-byte boundary, align to the next boundary
+        auto currentChunk = offset / 16;
+        auto endChunk = (offset + elementSize - 1) / 16;
+        if (currentChunk != endChunk)
+        {
+            return align(offset, 16);
+        }
+        return offset;
     }
 
     virtual IRSizeAndAlignment getVectorSizeAndAlignment(
         IRSizeAndAlignment element,
         IRIntegerValue count)
     {
-        IRIntegerValue countForAlignment = count;
-        return IRSizeAndAlignment(
-            (int)(element.size * count),
-            (int)(element.size * countForAlignment));
+        return IRSizeAndAlignment(element.size * count, element.alignment);
     }
 };
 
@@ -544,16 +616,25 @@ struct Std430LayoutRules : IRTypeLayoutRules
 {
     Std430LayoutRules() { ruleName = IRTypeLayoutRuleName::Std430; }
 
+    virtual IRIntegerValue adjustOffset(
+        IRIntegerValue offset,
+        IRIntegerValue elementSize,
+        IRType* lastFieldType,
+        IRIntegerValue lastFieldAlignment)
+    {
+        SLANG_UNUSED(elementSize);
+        if (as<IRMatrixType>(lastFieldType) || as<IRArrayTypeBase>(lastFieldType) ||
+            as<IRStructType>(lastFieldType))
+        {
+            return align(offset, (int)lastFieldAlignment);
+        }
+        return offset;
+    }
+
     virtual IRSizeAndAlignment alignCompositeElement(IRSizeAndAlignment elementSize)
     {
         return elementSize;
     }
-    virtual IRIntegerValue adjustOffsetForNextAggregateMember(
-        IRIntegerValue currentSize,
-        IRIntegerValue lastElementAlignment)
-    {
-        return align(currentSize, (int)lastElementAlignment);
-    }
 
     virtual IRSizeAndAlignment getVectorSizeAndAlignment(
         IRSizeAndAlignment element,
@@ -572,18 +653,28 @@ struct Std140LayoutRules : IRTypeLayoutRules
 {
     Std140LayoutRules() { ruleName = IRTypeLayoutRuleName::Std140; }
 
-    virtual IRIntegerValue adjustOffsetForNextAggregateMember(
-        IRIntegerValue currentSize,
-        IRIntegerValue lastElementAlignment)
+    virtual IRIntegerValue adjustOffset(
+        IRIntegerValue offset,
+        IRIntegerValue elementSize,
+        IRType* lastFieldType,
+        IRIntegerValue lastFieldAlignment)
     {
-        return align(currentSize, (int)lastElementAlignment);
+        SLANG_UNUSED(elementSize);
+        if (as<IRMatrixType>(lastFieldType) || as<IRArrayTypeBase>(lastFieldType) ||
+            as<IRStructType>(lastFieldType))
+        {
+            return align(offset, (int)lastFieldAlignment);
+        }
+        return offset;
     }
+
     virtual IRSizeAndAlignment alignCompositeElement(IRSizeAndAlignment elementSize)
     {
         elementSize.alignment = (int)align(elementSize.alignment, 16);
         elementSize.size = align(elementSize.size, elementSize.alignment);
         return elementSize;
     }
+
     virtual IRSizeAndAlignment getVectorSizeAndAlignment(
         IRSizeAndAlignment element,
         IRIntegerValue count)
@@ -597,24 +688,68 @@ struct Std140LayoutRules : IRTypeLayoutRules
     }
 };
 
+struct LLVMLayoutRules : IRTypeLayoutRules
+{
+    LLVMLayoutRules() { ruleName = IRTypeLayoutRuleName::LLVM; }
+
+    virtual Result calcSizeAndAlignment(
+        TargetRequest* targetReq,
+        IRType* type,
+        IRSizeAndAlignment* outSizeAndAlignment)
+    {
+        if (type->getOp() == kIROp_BoolType)
+        {
+            *outSizeAndAlignment = IRSizeAndAlignment(1, 1);
+            return SLANG_OK;
+        }
+        return IRTypeLayoutRules::calcSizeAndAlignment(targetReq, type, outSizeAndAlignment);
+    }
+
+    virtual IRIntegerValue adjustOffset(
+        IRIntegerValue offset,
+        IRIntegerValue elementSize,
+        IRType* lastFieldType,
+        IRIntegerValue lastFieldAlignment)
+    {
+        SLANG_UNUSED(elementSize);
+        SLANG_UNUSED(lastFieldType);
+        return align(offset, (int)lastFieldAlignment);
+    }
+
+    virtual IRSizeAndAlignment alignCompositeElement(IRSizeAndAlignment elementSize)
+    {
+        IRSizeAndAlignment alignedSize = elementSize;
+        alignedSize.size = align(alignedSize.size, alignedSize.alignment);
+        return alignedSize;
+    }
+
+    virtual IRSizeAndAlignment getVectorSizeAndAlignment(
+        IRSizeAndAlignment element,
+        IRIntegerValue count)
+    {
+        // Round alignment to next power of two
+        IRIntegerValue alignment = element.alignment;
+        while (alignment < element.size * count)
+            alignment *= 2;
+        return IRSizeAndAlignment(element.size * count, (int)alignment);
+    }
+};
+
 Result getNaturalSizeAndAlignment(
-    CompilerOptionSet& optionSet,
+    TargetRequest* targetReq,
     IRType* type,
     IRSizeAndAlignment* outSizeAndAlignment)
 {
     return getSizeAndAlignment(
-        optionSet,
+        targetReq,
         IRTypeLayoutRules::getNatural(),
         type,
         outSizeAndAlignment);
 }
 
-Result getNaturalOffset(
-    CompilerOptionSet& optionSet,
-    IRStructField* field,
-    IRIntegerValue* outOffset)
+Result getNaturalOffset(TargetRequest* targetReq, IRStructField* field, IRIntegerValue* outOffset)
 {
-    return getOffset(optionSet, IRTypeLayoutRules::getNatural(), field, outOffset);
+    return getOffset(targetReq, IRTypeLayoutRules::getNatural(), field, outOffset);
 }
 
 
@@ -623,23 +758,20 @@ Result getNaturalOffset(
 //////////////////////////
 
 Result getStd430SizeAndAlignment(
-    CompilerOptionSet& optionSet,
+    TargetRequest* targetReq,
     IRType* type,
     IRSizeAndAlignment* outSizeAndAlignment)
 {
     return getSizeAndAlignment(
-        optionSet,
+        targetReq,
         IRTypeLayoutRules::getStd430(),
         type,
         outSizeAndAlignment);
 }
 
-Result getStd430Offset(
-    CompilerOptionSet& optionSet,
-    IRStructField* field,
-    IRIntegerValue* outOffset)
+Result getStd430Offset(TargetRequest* targetReq, IRStructField* field, IRIntegerValue* outOffset)
 {
-    return getOffset(optionSet, IRTypeLayoutRules::getStd430(), field, outOffset);
+    return getOffset(targetReq, IRTypeLayoutRules::getStd430(), field, outOffset);
 }
 
 IRTypeLayoutRules* IRTypeLayoutRules::getStd430()
@@ -658,6 +790,18 @@ IRTypeLayoutRules* IRTypeLayoutRules::getNatural()
     return &rules;
 }
 
+IRTypeLayoutRules* IRTypeLayoutRules::getC()
+{
+    static CLayoutRules rules;
+    return &rules;
+}
+
+IRTypeLayoutRules* IRTypeLayoutRules::getLLVM()
+{
+    static LLVMLayoutRules rules;
+    return &rules;
+}
+
 IRTypeLayoutRules* IRTypeLayoutRules::getConstantBuffer()
 {
     static ConstantBufferLayoutRules rules;
@@ -673,9 +817,14 @@ IRTypeLayoutRules* IRTypeLayoutRules::get(IRTypeLayoutRuleName name)
     case IRTypeLayoutRuleName::Std140:
         return getStd140();
     case IRTypeLayoutRuleName::Natural:
+    case IRTypeLayoutRuleName::MetalParameterBlock:
         return getNatural();
+    case IRTypeLayoutRuleName::C:
+        return getC();
     case IRTypeLayoutRuleName::D3DConstantBuffer:
         return getConstantBuffer();
+    case IRTypeLayoutRuleName::LLVM:
+        return getLLVM();
     default:
         return nullptr;
     }
