@@ -25,6 +25,15 @@ struct LinkedIR
 //
 LinkedIR linkIR(CodeGenContext* codeGenContext);
 
+// Prelinking is a step that happens immediately after visiting all AST nodes during IR lowering,
+// and before any IR validation steps. Prelinking copys all extern symbols that are
+// [unsafeForceInlineEarly] into the current module being lowered, so that we can perform necessary
+// inlining passes before any data-flow analysis.
+// `externalSymbolsToLink` is a list of IRInsts defined in the imported modules that we need to pull
+// into the current module.
+//
+void prelinkIR(Module* module, IRModule* irModule, const List<IRInst*>& externalSymbolsToLink);
+
 // Replace any global constants in the IR module with their
 // definitions, if possible.
 //
