@@ -16,6 +16,7 @@ namespace Slang
 struct SharedSCCPContext
 {
     IRModule* module;
+    TargetProgram* targetProgram;
     DiagnosticSink* sink;
 };
 //
@@ -400,12 +401,13 @@ struct SCCPContext
         return LatticeVal::getConstant(resultVal);
     }
 
-    template<typename TIntFunc, typename TFloatFunc>
+    template<typename TIntFunc, typename TUintFunc, typename TFloatFunc>
     LatticeVal evalBinaryImpl(
         IRType* type,
         LatticeVal v0,
         LatticeVal v1,
         const TIntFunc& intFunc,
+        const TUintFunc& uintFunc,
         const TFloatFunc& floatFunc)
     {
         SLANG_SCCP_RETURN_IF_NONE_OR_ANY(v0)
@@ -415,17 +417,20 @@ struct SCCPContext
         IRInst* resultVal = nullptr;
         switch (type->getOp())
         {
-        case kIROp_Int8Type:
-        case kIROp_Int16Type:
-        case kIROp_IntType:
-        case kIROp_Int64Type:
         case kIROp_UInt8Type:
         case kIROp_UInt16Type:
         case kIROp_UIntType:
         case kIROp_UInt64Type:
-        case kIROp_IntPtrType:
         case kIROp_UIntPtrType:
+            resultVal =
+                getBuilder()->getIntValue(type, uintFunc(c0->value.uintVal, c1->value.uintVal));
+            break;
         case kIROp_BoolType:
+        case kIROp_Int8Type:
+        case kIROp_Int16Type:
+        case kIROp_IntType:
+        case kIROp_Int64Type:
+        case kIROp_IntPtrType:
             resultVal =
                 getBuilder()->getIntValue(type, intFunc(c0->value.intVal, c1->value.intVal));
             break;
@@ -511,7 +516,7 @@ struct SCCPContext
 
     template<typename TIntFunc, typename TFloatFunc>
     LatticeVal evalComparisonImpl(
-        IRType* type,
+        IRType*,
         LatticeVal v0,
         LatticeVal v1,
         const TIntFunc& intFunc,
@@ -522,29 +527,31 @@ struct SCCPContext
         SLANG_SCCP_RETURN_IF_NONE_OR_ANY(v1)
         auto c1 = as<IRConstant>(v1.value);
         IRInst* resultVal = nullptr;
-        switch (type->getOp())
+
+        // Check the operand types, not the result type (which is always bool for comparisons)
+        // For mixed-type comparisons, use floating-point path if either operand is floating-point
+        IRType* operandType0 = c0->getDataType();
+        IRType* operandType1 = c1->getDataType();
+
+        // Helper function to check if a type is floating-point
+        auto isFloatingPointType = [](IROp op) -> bool
+        { return op == kIROp_FloatType || op == kIROp_DoubleType || op == kIROp_HalfType; };
+
+        IROp op0 = operandType0->getOp();
+        IROp op1 = operandType1->getOp();
+
+        // Use floating-point path if either operand is floating-point
+        // Otherwise use integer path
+        if (isFloatingPointType(op0) || isFloatingPointType(op1))
         {
-        case kIROp_Int8Type:
-        case kIROp_Int16Type:
-        case kIROp_IntType:
-        case kIROp_Int64Type:
-        case kIROp_IntPtrType:
-        case kIROp_UInt8Type:
-        case kIROp_UInt16Type:
-        case kIROp_UIntType:
-        case kIROp_UInt64Type:
-        case kIROp_UIntPtrType:
-        case kIROp_BoolType:
-            resultVal = getBuilder()->getBoolValue(intFunc(c0->value.intVal, c1->value.intVal));
-            break;
-        case kIROp_FloatType:
-        case kIROp_DoubleType:
-        case kIROp_HalfType:
+            // Floating-point path - C++ operators follow IEEE 754 automatically
             resultVal =
                 getBuilder()->getBoolValue(floatFunc(c0->value.floatVal, c1->value.floatVal));
-            break;
-        default:
-            break;
+        }
+        else
+        {
+            // Integer path - all integer types
+            resultVal = getBuilder()->getBoolValue(intFunc(c0->value.intVal, c1->value.intVal));
         }
         if (!resultVal)
             return LatticeVal::getAny();
@@ -558,6 +565,7 @@ struct SCCPContext
             v0,
             v1,
             [](IRIntegerValue c0, IRIntegerValue c1) { return c0 + c1; },
+            [](IRUnsignedIntegerValue c0, IRUnsignedIntegerValue c1) { return c0 + c1; },
             [](IRFloatingPointValue c0, IRFloatingPointValue c1) { return c0 + c1; });
     }
     LatticeVal evalSub(IRType* type, LatticeVal v0, LatticeVal v1)
@@ -567,6 +575,7 @@ struct SCCPContext
             v0,
             v1,
             [](IRIntegerValue c0, IRIntegerValue c1) { return c0 - c1; },
+            [](IRUnsignedIntegerValue c0, IRUnsignedIntegerValue c1) { return c0 - c1; },
             [](IRFloatingPointValue c0, IRFloatingPointValue c1) { return c0 - c1; });
     }
     LatticeVal evalMul(IRType* type, LatticeVal v0, LatticeVal v1)
@@ -576,6 +585,7 @@ struct SCCPContext
             v0,
             v1,
             [](IRIntegerValue c0, IRIntegerValue c1) { return c0 * c1; },
+            [](IRUnsignedIntegerValue c0, IRUnsignedIntegerValue c1) { return c0 * c1; },
             [](IRFloatingPointValue c0, IRFloatingPointValue c1) { return c0 * c1; });
     }
     LatticeVal evalDiv(IRType* type, LatticeVal v0, LatticeVal v1)
@@ -585,6 +595,7 @@ struct SCCPContext
             v0,
             v1,
             [](IRIntegerValue c0, IRIntegerValue c1) { return c0 / c1; },
+            [](IRUnsignedIntegerValue c0, IRUnsignedIntegerValue c1) { return c0 / c1; },
             [](IRFloatingPointValue c0, IRFloatingPointValue c1) { return c0 / c1; });
     }
     LatticeVal evalRem(IRType* type, LatticeVal v0, LatticeVal v1)
@@ -594,6 +605,7 @@ struct SCCPContext
             v0,
             v1,
             [](IRIntegerValue c0, IRIntegerValue c1) { return c0 % c1; },
+            [](IRUnsignedIntegerValue c0, IRUnsignedIntegerValue c1) { return c0 % c1; },
             [](IRFloatingPointValue c0, IRFloatingPointValue c1) { return fmod(c0, c1); });
     }
     LatticeVal evalEql(IRType* type, LatticeVal v0, LatticeVal v1)
@@ -700,8 +712,8 @@ struct SCCPContext
     }
     LatticeVal evalLsh(IRType* type, LatticeVal v0, LatticeVal v1)
     {
-        IntInfo info = getIntTypeInfo(type);
-        if (info.isSigned == false)
+        bool isSigned = getIntTypeSigned(type);
+        if (isSigned == false)
         {
             return evalBinaryIntImpl(
                 type,
@@ -717,8 +729,8 @@ struct SCCPContext
     }
     LatticeVal evalRsh(IRType* type, LatticeVal v0, LatticeVal v1)
     {
-        IntInfo info = getIntTypeInfo(type);
-        if (info.isSigned == false)
+        bool isSigned = getIntTypeSigned(type);
+        if (isSigned == false)
         {
             return evalBinaryIntImpl(
                 type,
@@ -806,19 +818,24 @@ struct SCCPContext
         {
         case kIROp_Int64Type:
         case kIROp_UInt64Type:
-#if SLANG_PTR_IS_64
-        case kIROp_IntPtrType:
-        case kIROp_UIntPtrType:
-#endif
             resultVal = getBuilder()->getIntValue(type, sourceValueBits);
             break;
         case kIROp_IntType:
         case kIROp_UIntType:
-#if SLANG_PTR_IS_32
+            resultVal = getBuilder()->getIntValue(type, (uint32_t)sourceValueBits);
+            break;
         case kIROp_IntPtrType:
         case kIROp_UIntPtrType:
-#endif
-            resultVal = getBuilder()->getIntValue(type, (uint32_t)sourceValueBits);
+            // If running without targetProgram, we can't propagate
+            // pointer-sized types because we don't know what the pointer size
+            // is.
+            if (shared->targetProgram)
+            {
+                if (getPointerSize(shared->targetProgram->getTargetReq()) == sizeof(uint64_t))
+                    resultVal = getBuilder()->getIntValue(type, sourceValueBits);
+                else
+                    resultVal = getBuilder()->getIntValue(type, (uint32_t)sourceValueBits);
+            }
             break;
         case kIROp_FloatType:
             {
@@ -1798,13 +1815,17 @@ static bool applySparseConditionalConstantPropagationRec(
     return changed;
 }
 
-bool applySparseConditionalConstantPropagation(IRModule* module, DiagnosticSink* sink)
+bool applySparseConditionalConstantPropagation(
+    IRModule* module,
+    TargetProgram* targetProgram,
+    DiagnosticSink* sink)
 {
     if (sink && sink->getErrorCount())
         return false;
 
     SharedSCCPContext shared;
     shared.module = module;
+    shared.targetProgram = targetProgram;
     shared.sink = sink;
 
     // First we fold constants at global scope.
@@ -1819,13 +1840,17 @@ bool applySparseConditionalConstantPropagation(IRModule* module, DiagnosticSink*
     return changed;
 }
 
-bool applySparseConditionalConstantPropagationForGlobalScope(IRModule* module, DiagnosticSink* sink)
+bool applySparseConditionalConstantPropagationForGlobalScope(
+    IRModule* module,
+    TargetProgram* targetProgram,
+    DiagnosticSink* sink)
 {
     if (sink && sink->getErrorCount())
         return false;
 
     SharedSCCPContext shared;
     shared.module = module;
+    shared.targetProgram = targetProgram;
     shared.sink = sink;
     SCCPContext globalContext;
     globalContext.shared = &shared;
@@ -1834,13 +1859,17 @@ bool applySparseConditionalConstantPropagationForGlobalScope(IRModule* module, D
     return changed;
 }
 
-bool applySparseConditionalConstantPropagation(IRInst* func, DiagnosticSink* sink)
+bool applySparseConditionalConstantPropagation(
+    IRInst* func,
+    TargetProgram* targetProgram,
+    DiagnosticSink* sink)
 {
     if (sink && sink->getErrorCount())
         return false;
 
     SharedSCCPContext shared;
     shared.module = func->getModule();
+    shared.targetProgram = targetProgram;
     shared.sink = sink;
 
     SCCPContext globalContext;
@@ -1851,10 +1880,11 @@ bool applySparseConditionalConstantPropagation(IRInst* func, DiagnosticSink* sin
     return applySparseConditionalConstantPropagationRec(globalContext, func);
 }
 
-IRInst* tryConstantFoldInst(IRModule* module, IRInst* inst)
+IRInst* tryConstantFoldInst(IRModule* module, TargetProgram* targetProgram, IRInst* inst)
 {
     SharedSCCPContext shared;
     shared.module = module;
+    shared.targetProgram = targetProgram;
     SCCPContext instContext;
     instContext.shared = &shared;
     instContext.code = nullptr;
