@@ -6,6 +6,7 @@ namespace Slang
 struct IRModule;
 struct IRInst;
 class DiagnosticSink;
+class TargetProgram;
 
 /// Apply Sparse Conditional Constant Propagation (SCCP) to a module.
 ///
@@ -15,12 +16,19 @@ class DiagnosticSink;
 /// always evaluate to a constant (which can lead to entire blocks
 /// becoming dead code)
 /// Returns true if IR is changed.
-bool applySparseConditionalConstantPropagation(IRModule* module, DiagnosticSink* sink);
+bool applySparseConditionalConstantPropagation(
+    IRModule* module,
+    TargetProgram* targetProgram,
+    DiagnosticSink* sink);
 bool applySparseConditionalConstantPropagationForGlobalScope(
     IRModule* module,
+    TargetProgram* targetProgram,
     DiagnosticSink* sink);
 
-bool applySparseConditionalConstantPropagation(IRInst* func, DiagnosticSink* sink);
+bool applySparseConditionalConstantPropagation(
+    IRInst* func,
+    TargetProgram* targetProgram,
+    DiagnosticSink* sink);
 
-IRInst* tryConstantFoldInst(IRModule* module, IRInst* inst);
+IRInst* tryConstantFoldInst(IRModule* module, TargetProgram* targetProgram, IRInst* inst);
 } // namespace Slang
