@@ -56,7 +56,7 @@ static IRInst* findBreakableRegionHeaderInst(IRDominatorTree* domTree, IRBlock*
         switch (terminator->getOp())
         {
         case kIROp_Switch:
-        case kIROp_loop:
+        case kIROp_Loop:
             return terminator;
         }
     }
@@ -138,7 +138,7 @@ bool isTrivialSingleIterationLoop(
             //
             switch (terminator->getOp())
             {
-            case kIROp_loop:
+            case kIROp_Loop:
                 if (isBlockInRegion(domTree, as<IRLoop>(terminator), breakOriginBlock))
                     return false;
                 break;
@@ -348,12 +348,22 @@ static bool isTrivialIfElseBranch(IRIfElse* condBranch, IRBlock* branchBlock)
 {
     if (branchBlock != condBranch->getAfterBlock())
     {
-        if (auto br = as<IRUnconditionalBranch>(branchBlock->getFirstOrdinaryInst()))
+        for (auto inst = branchBlock->getFirstOrdinaryInst(); inst != nullptr;
+             inst = inst->getNextInst())
         {
-            if (br->getTargetBlock() == condBranch->getAfterBlock() &&
-                br->getOp() == kIROp_unconditionalBranch)
+            switch (inst->getOp())
             {
-                return true;
+            case kIROp_DebugLine:
+                continue;
+
+            case kIROp_UnconditionalBranch:
+                {
+                    auto br = as<IRUnconditionalBranch>(inst);
+                    return br->getTargetBlock() == condBranch->getAfterBlock();
+                }
+
+            default:
+                return false;
             }
         }
     }
@@ -433,12 +443,21 @@ static bool isTrivialSwitchBranch(IRSwitch* switchInst, IRBlock* branchBlock)
 {
     if (branchBlock != switchInst->getBreakLabel())
     {
-        if (auto br = as<IRUnconditionalBranch>(branchBlock->getFirstOrdinaryInst()))
+        for (auto inst = branchBlock->getFirstOrdinaryInst(); inst != nullptr; inst = inst->next)
         {
-            if (br->getTargetBlock() == switchInst->getBreakLabel() &&
-                br->getOp() == kIROp_unconditionalBranch)
+            switch (inst->getOp())
             {
-                return true;
+            case kIROp_DebugLine:
+                continue;
+
+            case kIROp_UnconditionalBranch:
+                {
+                    auto br = as<IRUnconditionalBranch>(inst);
+                    return br->getTargetBlock() == switchInst->getBreakLabel();
+                }
+
+            default:
+                return false;
             }
         }
     }
@@ -564,9 +583,20 @@ static bool trySimplifySwitch(IRBuilder& builder, IRSwitch* switchInst)
         for (;;)
         {
             auto block = as<IRBlock>(targetUse->get());
-            if (block->getFirstInst()->getOp() != kIROp_unconditionalBranch)
-                return;
-            auto branch = as<IRUnconditionalBranch>(block->getFirstInst());
+            IRUnconditionalBranch* branch = nullptr;
+            for (auto inst = block->getFirstInst(); inst && !branch; inst = inst->next)
+            {
+                switch (inst->getOp())
+                {
+                case kIROp_DebugLine:
+                    continue;
+                case kIROp_UnconditionalBranch:
+                    branch = as<IRUnconditionalBranch>(inst);
+                    break;
+                default:
+                    return;
+                }
+            }
             // We can't fuse the block if there are phi arguments.
             if (branch->getArgCount() != 0)
                 return;
@@ -580,8 +610,8 @@ static bool trySimplifySwitch(IRBuilder& builder, IRSwitch* switchInst)
                     continue;
                 switch (use->getUser()->getOp())
                 {
-                case kIROp_loop:
-                case kIROp_ifElse:
+                case kIROp_Loop:
+                case kIROp_IfElse:
                 case kIROp_Switch:
                     // If the target block is used by a special control flow inst,
                     // it is likely a merge block and we can't fuse it.
@@ -690,7 +720,7 @@ static bool simplifyBoolPhiParams(IRBlock* block)
     Array<IRBlock*, 2> preds;
     for (auto pred : block->getPredecessors())
     {
-        if (pred->getTerminator()->getOp() != kIROp_unconditionalBranch)
+        if (pred->getTerminator()->getOp() != kIROp_UnconditionalBranch)
             return false;
         preds.add(pred);
     }
@@ -914,7 +944,7 @@ static bool processFunc(IRGlobalValueWithCode* func, CFGSimplificationOptions op
                 }
 
                 // If `block` does not end with an unconditional branch, bail.
-                if (block->getTerminator()->getOp() != kIROp_unconditionalBranch)
+                if (block->getTerminator()->getOp() != kIROp_UnconditionalBranch)
                     break;
                 auto branch = as<IRUnconditionalBranch>(block->getTerminator());
                 auto successor = branch->getTargetBlock();
@@ -945,7 +975,7 @@ static bool processFunc(IRGlobalValueWithCode* func, CFGSimplificationOptions op
                     inst = next;
                 }
                 branch->removeAndDeallocate();
-                assert(!successor->hasUses());
+                SLANG_ASSERT(!successor->hasUses());
                 successor->removeAndDeallocate();
 
                 break;
