@@ -4,6 +4,8 @@
 namespace Slang
 {
 struct IRModule;
+struct IRInst;
+struct IRSpecialize;
 class DiagnosticSink;
 class TargetProgram;
 
@@ -13,15 +15,25 @@ struct SpecializationOptions
     // wherever possible to open up more specialization opportunities.
     //
     bool lowerWitnessLookups = false;
+
+    // Option to report dynamic dispatch sites.
+    bool reportDynamicDispatchSites = false;
 };
 
 /// Specialize generic and interface-based code to use concrete types.
 bool specializeModule(
-    TargetProgram* target,
     IRModule* module,
+    TargetProgram* target,
     DiagnosticSink* sink,
     SpecializationOptions options);
 
 void finalizeSpecialization(IRModule* module);
 
+IRInst* specializeGeneric(IRSpecialize* specInst);
+
+// Specialize a generic with one or more arguments that are collections rather
+// than single concrete values.
+//
+IRInst* specializeDynamicGeneric(IRSpecialize* specializeInst);
+
 } // namespace Slang
