@@ -75,8 +75,80 @@ struct GlobalVarTranslationContext
             builder.setInsertBefore(entryPointFunc);
             auto inputStructType = builder.createStructType();
             IRStructTypeLayout::Builder inputStructTypeLayoutBuilder(&builder);
+
+            // Go through the existing parameter layouts and add them to the
+            // replacement struct type first.
+            //
+            // We'll also need to fix up the offsets of the global varing inputs
+            // being added if we have any varing input entry point params. Set
+            // these to some defaults for now.
+            UInt nextOffset = 0;
+            IRVarLayout* lastFieldVarLayout = nullptr;
+
+            // If we have an existing entry point struct layout, we need to go through it and
+            // add any of the struct field layout attributes as fields in the new struct type
+            // layout that we are building to combine global varing ins and entry point ins.
+            IRLayoutDecoration* entryPointLayoutDecor =
+                entryPointFunc->findDecoration<IRLayoutDecoration>();
+            IREntryPointLayout* entryPointLayout = nullptr;
+            IRStructTypeLayout* entryPointParamsStructLayout = nullptr;
+
+            if (entryPointLayoutDecor)
+            {
+                entryPointLayout = as<IREntryPointLayout>(entryPointLayoutDecor->getLayout());
+                if (entryPointLayout)
+                {
+                    // The parameter layout for an entry point will either be a structure
+                    // type layout, or a parameter group type layout.
+                    entryPointParamsStructLayout = getScopeStructLayout(entryPointLayout);
+                    if (entryPointParamsStructLayout)
+                    {
+                        for (auto attr : entryPointParamsStructLayout->getFieldLayoutAttrs())
+                        {
+                            // Add existing parameter layouts to the replacement struct type layout.
+                            IRVarLayout* fieldLayout = attr->getLayout();
+                            IRInst* key = attr->getFieldKey();
+                            inputStructTypeLayoutBuilder.addField(key, fieldLayout);
+
+                            // Save the last VaryingInput type. This will be used to help
+                            // recalculate offsets for the global varying inputs.
+                            if (fieldLayout->usesResourceKind(LayoutResourceKind::VaryingInput))
+                                lastFieldVarLayout = fieldLayout;
+                        }
+
+                        // Calculate the nextOffset if necessary. If entry params had no varying
+                        // inputs nextOffset will just be 0.
+                        if (lastFieldVarLayout)
+                        {
+                            // Find the size and offset of the last varying "in" kind
+                            // param in the entry point param struct. Adding these will give us
+                            // the starting offset that should be used for the first global.
+                            if (auto sizeAttr = lastFieldVarLayout->getTypeLayout()->findSizeAttr(
+                                    LayoutResourceKind::VaryingInput))
+                            {
+                                const auto finiteSize = sizeAttr->getFiniteSize();
+                                if (auto offsetAttr = lastFieldVarLayout->findOffsetAttr(
+                                        LayoutResourceKind::VaryingInput))
+                                {
+                                    UInt lastFieldOffset = offsetAttr->getOffset();
+                                    nextOffset = finiteSize + lastFieldOffset;
+                                }
+                            }
+                        }
+                    }
+                }
+            }
+
+            // Add the global vars to the replacement struct and add new params to
+            // the entry point func based on the global input vars.
+            List<IRInst*> newParams;
             UInt inputVarIndex = 0;
             List<IRStructKey*> inputKeys;
+
+            // Setup the location where we will insert the new params
+            auto firstBlock = entryPointFunc->getFirstBlock();
+            builder.setInsertInto(firstBlock);
+
             for (auto input : inputVars)
             {
                 auto inputType = cast<IRPtrTypeBase>(input->getDataType())->getValueType();
@@ -112,67 +184,91 @@ struct GlobalVarTranslationContext
                 }
                 else
                 {
-                    if (!hasExistingLayout)
-                    {
-                        fieldTypeLayoutBuilder.addResourceUsage(
-                            LayoutResourceKind::VaryingInput,
-                            LayoutSize(1));
-                    }
+                    // Start off the offset as nextOffset. If the global "in"s have existing
+                    // offsetAttr's, we will add the offsets from those as well.
+                    auto resInfo =
+                        varLayoutBuilder.findOrAddResourceInfo(LayoutResourceKind::VaryingInput);
+                    resInfo->offset = nextOffset;
+
                     if (auto layoutDecor = findVarLayout(input))
                     {
                         if (auto offsetAttr =
                                 layoutDecor->findOffsetAttr(LayoutResourceKind::VaryingInput))
                         {
-                            varLayoutBuilder
-                                .findOrAddResourceInfo(LayoutResourceKind::VaryingInput)
-                                ->offset = (UInt)offsetAttr->getOffset();
+                            resInfo->offset += (UInt)offsetAttr->getOffset();
                         }
                     }
                     if (entryPointDecor->getProfile().getStage() == Stage::Fragment)
                     {
                         varLayoutBuilder.setUserSemantic("COLOR", inputVarIndex);
+                        if (!key->findDecoration<IRSemanticDecoration>())
+                            builder.addSemanticDecoration(key, toSlice("COLOR"), inputVarIndex);
                     }
                     else if (entryPointDecor->getProfile().getStage() == Stage::Vertex)
                     {
                         varLayoutBuilder.setUserSemantic("VERTEX_IN_", inputVarIndex);
+                        if (!key->findDecoration<IRSemanticDecoration>())
+                            builder.addSemanticDecoration(
+                                key,
+                                toSlice("VERTEX_IN_"),
+                                inputVarIndex);
                     }
                     inputVarIndex++;
                 }
-                inputStructTypeLayoutBuilder.addField(key, varLayoutBuilder.build());
+                auto varLayout = varLayoutBuilder.build();
+                inputStructTypeLayoutBuilder.addField(key, varLayout);
                 input->transferDecorationsTo(key);
+
+                // Emit a new param here to represent the global input var.
+                auto inputParam =
+                    builder.emitParam(builder.getBorrowInParamType(inputType, AddressSpace::Input));
+
+                // Copy the global input vars original decorations onto the new param.
+                // We need to do this to ensure that we can do things like get system
+                // value info in later passes like when we legalize entry points for WGSL.
+                IRCloneEnv cloneEnv;
+                cloneInstDecorationsAndChildren(&cloneEnv, builder.getModule(), key, inputParam);
+
+                // Add the layout to the new param
+                builder.addLayoutDecoration(inputParam, varLayout);
+
+                // Add the param to our list of new params. This will allow us
+                // to connect the old "global variable" to a "global parameter"
+                // later on.
+                newParams.add(inputParam);
             }
+
+            // Build the new layout that will replace the old entryPointLayout.
             auto paramTypeLayout = inputStructTypeLayoutBuilder.build();
             IRVarLayout::Builder paramVarLayoutBuilder(&builder, paramTypeLayout);
             paramLayout = paramVarLayoutBuilder.build();
 
-            // Add an entry point parameter for all the inputs.
-            auto firstBlock = entryPointFunc->getFirstBlock();
-            builder.setInsertInto(firstBlock);
-            auto inputParam = builder.emitParam(
-                builder.getPtrType(kIROp_ConstRefType, inputStructType, AddressSpace::Input));
-            builder.addLayoutDecoration(inputParam, paramLayout);
-
             // Initialize all global variables in the order of struct member declaration.
             for (Index i = inputVars.getCount() - 1; i >= 0; i--)
             {
                 auto input = inputVars[i];
                 setInsertBeforeOrdinaryInst(&builder, firstBlock->getFirstOrdinaryInst());
-                auto inputType = cast<IRPtrTypeBase>(input->getDataType())->getValueType();
+                // TODO: Does the below TODO apply if we no longer use emitFieldExtract?
+#if 0
                 // TODO: This could be more efficient as a Load(FieldAddress(inputParam, i))
                 // operation instead of a FieldExtract(Load(inputParam)).
                 builder.emitStore(
                     input,
                     builder
                         .emitFieldExtract(inputType, builder.emitLoad(inputParam), inputKeys[i]));
-                // Relate "global variable" to a "global parameter" for use later in compilation
-                // to resolve a "global variable" shadowing a "global parameter" relationship.
+#endif
+                builder.emitStore(input, builder.emitLoad(newParams[i]));
+
+                // Relate the old "global variable" to a "global parameter" for use later in
+                // compilation to resolve a "global variable" shadowing a "global parameter"
+                // relationship.
                 builder.addGlobalVariableShadowingGlobalParameterDecoration(
-                    inputParam,
+                    newParams[i],
                     input,
                     inputKeys[i]);
             }
 
-            // For each entry point, introduce a new parameter to represent each input parameter,
+            // For each entry point, introduce a new parameter to represent each output parameter,
             // and return all outputs via a struct value.
             if (hasOutput)
             {
@@ -194,8 +290,27 @@ struct GlobalVarTranslationContext
                     auto key = builder.createStructKey();
                     auto ptrType = as<IRPtrTypeBase>(output->getDataType());
                     builder.createStructField(resultType, key, ptrType->getValueType());
-                    IRTypeLayout::Builder fieldTypeLayout(&builder);
-                    IRVarLayout::Builder varLayoutBuilder(&builder, fieldTypeLayout.build());
+
+                    IRTypeLayout::Builder fieldTypeLayoutBuilder(&builder);
+                    IRTypeLayout* fieldTypeLayout = nullptr;
+                    bool hasExistingLayout = false;
+                    if (auto existingLayoutDecoration =
+                            output->findDecoration<IRLayoutDecoration>())
+                    {
+                        if (auto existingVarLayout =
+                                as<IRVarLayout>(existingLayoutDecoration->getLayout()))
+                        {
+                            fieldTypeLayout = existingVarLayout->getTypeLayout();
+                            hasExistingLayout = true;
+                        }
+                    }
+
+                    if (!hasExistingLayout)
+                    {
+                        fieldTypeLayout = fieldTypeLayoutBuilder.build();
+                    }
+
+                    IRVarLayout::Builder varLayoutBuilder(&builder, fieldTypeLayout);
                     varLayoutBuilder.setStage(entryPointDecor->getProfile().getStage());
                     if (auto semanticDecor = output->findDecoration<IRSemanticDecoration>())
                     {
@@ -205,9 +320,6 @@ struct GlobalVarTranslationContext
                     }
                     else
                     {
-                        fieldTypeLayout.addResourceUsage(
-                            LayoutResourceKind::VaryingOutput,
-                            LayoutSize(1));
                         if (auto layoutDecor = findVarLayout(output))
                         {
                             if (auto offsetAttr =
@@ -221,10 +333,20 @@ struct GlobalVarTranslationContext
                         if (entryPointDecor->getProfile().getStage() == Stage::Fragment)
                         {
                             varLayoutBuilder.setSystemValueSemantic("SV_TARGET", outputVarIndex);
+                            if (!key->findDecoration<IRSemanticDecoration>())
+                                builder.addSemanticDecoration(
+                                    key,
+                                    toSlice("SV_TARGET"),
+                                    outputVarIndex);
                         }
                         else if (entryPointDecor->getProfile().getStage() == Stage::Vertex)
                         {
                             varLayoutBuilder.setUserSemantic("COLOR", outputVarIndex);
+                            if (!key->findDecoration<IRSemanticDecoration>())
+                                builder.addSemanticDecoration(
+                                    key,
+                                    toSlice("COLOR"),
+                                    outputVarIndex);
                         }
                         outputVarIndex++;
                     }
@@ -256,18 +378,43 @@ struct GlobalVarTranslationContext
                     }
                 }
             }
-            if (auto entryPointLayoutDecor = entryPointFunc->findDecoration<IRLayoutDecoration>())
+            if (entryPointLayout)
             {
-                if (auto entryPointLayout =
-                        as<IREntryPointLayout>(entryPointLayoutDecor->getLayout()))
+                if (paramLayout)
                 {
-                    if (paramLayout)
-                        builder.replaceOperand(entryPointLayout->getOperands(), paramLayout);
-                    if (resultVarLayout)
+                    // We try to respect the original entry point type layout when replacing it
+                    // below. The IRParameterGroupTypeLayout type layout is used when there are
+                    // things like uniform entry point params, otherwise the IRStructTypeLayout type
+                    // layout will normally represent the entry point type layout.
+                    if (auto paramGroupTypeLayout =
+                            as<IRParameterGroupTypeLayout>(entryPointParamsStructLayout))
+                    {
+                        // Build a new IRParameterGroupTypeLayout to replace the old one
+                        // representing the entryPointLayout. The ContainerVarLayout shouldn't have
+                        // changed, so resue that. Replace the rest with the new paramTypeLayout and
+                        // paramLayout that we calculated above
+                        IRParameterGroupTypeLayout::Builder paramGroupTypeLayoutBuilder(&builder);
+                        paramGroupTypeLayoutBuilder.setContainerVarLayout(
+                            paramGroupTypeLayout->getContainerVarLayout());
+                        paramGroupTypeLayoutBuilder.setElementVarLayout(paramLayout);
+                        paramGroupTypeLayoutBuilder.setOffsetElementTypeLayout(paramTypeLayout);
                         builder.replaceOperand(
-                            entryPointLayout->getOperands() + 1,
-                            resultVarLayout);
+                            entryPointLayout->getParamsLayout()->getOperands(),
+                            paramGroupTypeLayoutBuilder.build());
+                    }
+                    else if (as<IRStructTypeLayout>(entryPointParamsStructLayout))
+                    {
+                        builder.replaceOperand(
+                            entryPointLayout->getParamsLayout()->getOperands(),
+                            paramTypeLayout);
+                    }
+                    else
+                    {
+                        SLANG_UNEXPECTED("uhandled global-scope binding layout");
+                    }
                 }
+                if (resultVarLayout)
+                    builder.replaceOperand(entryPointLayout->getOperands() + 1, resultVarLayout);
             }
             // Update func type for the entry point.
             List<IRType*> paramTypes;
@@ -373,7 +520,7 @@ struct GlobalVarTranslationContext
     }
 };
 
-void translateGlobalVaryingVar(CodeGenContext* context, IRModule* module)
+void translateGlobalVaryingVar(IRModule* module, CodeGenContext* context)
 {
     GlobalVarTranslationContext ctx;
     ctx.context = context;
