@@ -1,6 +1,7 @@
 // slang-ir-validate.cpp
 #include "slang-ir-validate.h"
 
+#include "slang-compiler.h"
 #include "slang-ir-dominators.h"
 #include "slang-ir-insts.h"
 #include "slang-ir-util.h"
@@ -25,6 +26,31 @@ struct IRValidateContext
     HashSet<IRInst*> seenInsts;
 };
 
+// Context class for structured buffer validation
+class StructuredBufferValidationContext
+{
+public:
+    StructuredBufferValidationContext(DiagnosticSink* sink, TargetRequest* targetRequest)
+        : m_sink(sink), m_targetRequest(targetRequest), m_hasErrors(false)
+    {
+    }
+
+    bool validate(IRModule* module);
+
+private:
+    DiagnosticSink* m_sink;
+    TargetRequest* m_targetRequest;
+    bool m_hasErrors;
+
+    // Cache of types we've already checked for containing opaque handles
+    HashSet<IRType*> m_checkedTypes;
+    HashSet<IRType*> m_typesWithOpaqueHandles;
+
+    bool containsOpaqueHandleTypeCached(IRType* type);
+    bool containsOpaqueHandleTypeInternal(IRType* type, HashSet<IRType*>& visitedInCurrentCheck);
+    void validateStructuredBufferVariable(IRInst* inst);
+};
+
 void validateIRInst(IRValidateContext* context, IRInst* inst);
 
 void validate(IRValidateContext* context, bool condition, IRInst* inst, char const* message)
@@ -175,9 +201,13 @@ void validateIRInstOperand(IRValidateContext* context, IRInst* inst, IRUse* oper
                 // in order.
                 if (context)
                 {
+                    // There is exception that the use of an inst is defined before the inst,
+                    // e.g. generic parameter can be defined before its data type in some cases.
+                    // In those cases we allow relaxing the rule.
                     validate(
                         context,
-                        context->seenInsts.contains(operandValue),
+                        context->seenInsts.contains(operandValue) ||
+                            canRelaxInstOrderRule(operandValue, inst),
                         inst,
                         "def must come before use in same block");
                 }
@@ -220,7 +250,7 @@ void validateIRInstOperand(IRValidateContext* context, IRInst* inst, IRUse* oper
     }
 
     // We allow out-of-order def-use in global scope.
-    bool allInGlobalScope = inst->getParent() && inst->getParent()->getOp() == kIROp_Module;
+    bool allInGlobalScope = inst->getParent() && inst->getParent()->getOp() == kIROp_ModuleInst;
     if (allInGlobalScope)
     {
         for (UInt i = 0; i < inst->getOperandCount(); i++)
@@ -230,7 +260,7 @@ void validateIRInstOperand(IRValidateContext* context, IRInst* inst, IRUse* oper
                 continue;
             if (!op->getParent())
                 continue;
-            if (op->getParent()->getOp() != kIROp_Module)
+            if (op->getParent()->getOp() != kIROp_ModuleInst)
             {
                 allInGlobalScope = false;
                 break;
@@ -244,6 +274,7 @@ void validateIRInstOperand(IRValidateContext* context, IRInst* inst, IRUse* oper
     switch (inst->getOp())
     {
     case kIROp_DifferentiableTypeDictionaryItem:
+    case kIROp_DebugScope:
         return;
     }
     //
@@ -272,24 +303,29 @@ void validateIRInstOperands(IRValidateContext* context, IRInst* inst)
 }
 
 static thread_local bool _enableIRValidationAtInsert = false;
-void disableIRValidationAtInsert()
+
+// RAII class implementation for exception-safe IR validation state management
+IRValidationScope::IRValidationScope(bool enableValidation)
+    : m_previousState(_enableIRValidationAtInsert)
 {
-    _enableIRValidationAtInsert = false;
+    _enableIRValidationAtInsert = enableValidation;
 }
-void enableIRValidationAtInsert()
+
+IRValidationScope::~IRValidationScope()
 {
-    _enableIRValidationAtInsert = true;
+    _enableIRValidationAtInsert = m_previousState;
 }
+
 void validateIRInstOperands(IRInst* inst)
 {
     if (!_enableIRValidationAtInsert)
         return;
     switch (inst->getOp())
     {
-    case kIROp_loop:
-    case kIROp_ifElse:
-    case kIROp_unconditionalBranch:
-    case kIROp_conditionalBranch:
+    case kIROp_Loop:
+    case kIROp_IfElse:
+    case kIROp_UnconditionalBranch:
+    case kIROp_ConditionalBranch:
     case kIROp_Switch:
         return;
     default:
@@ -319,12 +355,12 @@ void validateCodeBody(IRValidateContext* context, IRGlobalValueWithCode* code)
         validate(context, terminator, block, "block must have valid terminator inst.");
         switch (terminator->getOp())
         {
-        case kIROp_conditionalBranch:
+        case kIROp_ConditionalBranch:
             validateBranchTarget(terminator, as<IRConditionalBranch>(terminator)->getTrueBlock());
             validateBranchTarget(terminator, as<IRConditionalBranch>(terminator)->getFalseBlock());
             break;
-        case kIROp_loop:
-        case kIROp_unconditionalBranch:
+        case kIROp_Loop:
+        case kIROp_UnconditionalBranch:
             validateBranchTarget(
                 terminator,
                 as<IRUnconditionalBranch>(terminator)->getTargetBlock());
@@ -452,7 +488,7 @@ static bool isValidAtomicDest(bool skipFuncParamValidation, IRInst* dst)
     if (auto param = as<IRParam>(dst))
     {
         auto paramType = param->getDataType();
-        if (auto outType = as<IROutTypeBase>(paramType))
+        if (auto outType = as<IROutParamTypeBase>(paramType))
         {
             if (outType->getAddressSpace() == AddressSpace::GroupShared)
             {
@@ -510,4 +546,238 @@ void validateAtomicOperations(bool skipFuncParamValidation, DiagnosticSink* sink
     }
 }
 
+static void validateVectorOrMatrixElementType(
+    DiagnosticSink* sink,
+    SourceLoc sourceLoc,
+    IRType* elementType,
+    uint32_t allowedWidths,
+    const DiagnosticInfo& disallowedElementTypeEncountered,
+    TargetRequest* targetRequest)
+{
+    if (!isFloatingType(elementType))
+    {
+        if (isIntegralType(elementType))
+        {
+            IntInfo info = getIntTypeInfo(targetRequest, elementType);
+            if (allowedWidths == 0U)
+            {
+                sink->diagnose(sourceLoc, disallowedElementTypeEncountered, elementType);
+            }
+            else
+            {
+                bool widthAllowed = false;
+                SLANG_ASSERT((allowedWidths & ~(0xfU << 3)) == 0U);
+                for (uint32_t p = 3U; p <= 6U; p++)
+                {
+                    uint32_t width = 1U << p;
+                    if (!(allowedWidths & width))
+                        continue;
+                    widthAllowed = widthAllowed || (info.width == width);
+                }
+                if (!widthAllowed)
+                {
+                    sink->diagnose(sourceLoc, disallowedElementTypeEncountered, elementType);
+                }
+            }
+        }
+        else if (!as<IRBoolType>(elementType))
+        {
+            sink->diagnose(sourceLoc, disallowedElementTypeEncountered, elementType);
+        }
+    }
+}
+
+static void validateVectorElementCount(DiagnosticSink* sink, IRVectorType* vectorType)
+{
+    const auto elementCount = as<IRIntLit>(vectorType->getElementCount())->getValue();
+
+    // 1-vectors are supported and are legalized/transformed properly when targetting unsupported
+    // backends.
+    const IRIntegerValue minCount = 1;
+    const IRIntegerValue maxCount = 4;
+    if ((elementCount < minCount) || (elementCount > maxCount))
+    {
+        sink->diagnose(
+            vectorType->sourceLoc,
+            Diagnostics::vectorWithInvalidElementCountEncountered,
+            elementCount,
+            "1",
+            maxCount);
+    }
+}
+
+void validateVectorsAndMatrices(
+    IRModule* module,
+    DiagnosticSink* sink,
+    TargetRequest* targetRequest)
+{
+    for (auto globalInst : module->getGlobalInsts())
+    {
+        if (auto matrixType = as<IRMatrixType>(globalInst))
+        {
+            // Matrices with row/col dimension 1 are only well-supported on D3D targets
+            if (!isD3DTarget(targetRequest))
+            {
+                // Verify that neither row nor col count is 1
+                auto colCount = as<IRIntLit>(matrixType->getColumnCount());
+                auto rowCount = as<IRIntLit>(matrixType->getRowCount());
+
+                if ((rowCount && (rowCount->getValue() == 1)) ||
+                    (colCount && (colCount->getValue() == 1)))
+                {
+                    sink->diagnose(matrixType->sourceLoc, Diagnostics::matrixColumnOrRowCountIsOne);
+                }
+            }
+
+            // Matrix element type validation removed to allow integer/bool matrices
+            // which will be lowered to arrays of vectors on targets that don't support them
+            // natively
+        }
+        else if (auto vectorType = as<IRVectorType>(globalInst))
+        {
+            // Verify that the element type is a floating point type, or an allowed integral type
+            auto elementType = vectorType->getElementType();
+            uint32_t allowedWidths = 0U;
+            if (isWGPUTarget(targetRequest))
+                allowedWidths = 32U;
+            else
+                allowedWidths = 8U | 16U | 32U | 64U;
+
+            validateVectorOrMatrixElementType(
+                sink,
+                vectorType->sourceLoc,
+                elementType,
+                allowedWidths,
+                Diagnostics::vectorWithDisallowedElementTypeEncountered,
+                targetRequest);
+
+            validateVectorElementCount(sink, vectorType);
+        }
+    }
+}
+
+//
+// Structure buffer resource types
+//
+
+bool StructuredBufferValidationContext::containsOpaqueHandleTypeCached(IRType* type)
+{
+    // Check cache first
+    if (m_checkedTypes.contains(type))
+    {
+        return m_typesWithOpaqueHandles.contains(type);
+    }
+
+    // Not in cache, need to check
+    HashSet<IRType*> visitedInCurrentCheck;
+    bool result = containsOpaqueHandleTypeInternal(type, visitedInCurrentCheck);
+
+    // Cache the result
+    m_checkedTypes.add(type);
+    if (result)
+    {
+        m_typesWithOpaqueHandles.add(type);
+    }
+
+    return result;
+}
+
+bool StructuredBufferValidationContext::containsOpaqueHandleTypeInternal(
+    IRType* type,
+    HashSet<IRType*>& visitedInCurrentCheck)
+{
+    // Prevent infinite recursion in current check
+    if (!visitedInCurrentCheck.add(type))
+        return false;
+
+    // Check if the type itself is an opaque handle
+    if (isResourceType(type))
+        return true;
+
+    // Check struct types
+    if (auto structType = as<IRStructType>(type))
+    {
+        for (auto field : structType->getFields())
+        {
+            if (containsOpaqueHandleTypeInternal(field->getFieldType(), visitedInCurrentCheck))
+                return true;
+        }
+    }
+    else if (auto arrayType = as<IRArrayTypeBase>(type))
+    {
+        return containsOpaqueHandleTypeInternal(arrayType->getElementType(), visitedInCurrentCheck);
+    }
+    else if (auto ptrType = as<IRPtrTypeBase>(type))
+    {
+        return containsOpaqueHandleTypeInternal(ptrType->getValueType(), visitedInCurrentCheck);
+    }
+
+    return false;
+}
+
+void StructuredBufferValidationContext::validateStructuredBufferVariable(IRInst* inst)
+{
+    IRType* type = inst->getDataType();
+
+    // Unwrap arrays if present
+    type = unwrapArrayAndPointers(type);
+
+    // Check if this is a structured buffer type
+    auto structuredBufferType = as<IRHLSLStructuredBufferTypeBase>(type);
+    if (!structuredBufferType)
+        return;
+
+    // Get the element type
+    auto elementType = structuredBufferType->getElementType();
+
+    // Check if the element type contains any resource/opaque handle types
+    if (containsOpaqueHandleTypeCached(elementType))
+    {
+        m_sink->diagnose(
+            inst->sourceLoc,
+            Diagnostics::cannotUseResourceTypeInStructuredBuffer,
+            elementType);
+        m_hasErrors = true;
+    }
+}
+
+bool StructuredBufferValidationContext::validate(IRModule* module)
+{
+    // Skip validation if bindless is enabled for this target
+    if (m_targetRequest && areResourceTypesBindlessOnTarget(m_targetRequest))
+        return true;
+
+    // Iterate through all global instructions
+    for (auto globalInst : module->getGlobalInsts())
+    {
+        if (auto globalVar = as<IRGlobalParam>(globalInst))
+        {
+            validateStructuredBufferVariable(globalVar);
+        }
+        else if (auto func = as<IRFunc>(globalInst))
+        {
+            for (auto param : func->getParams())
+            {
+                validateStructuredBufferVariable(param);
+            }
+        }
+    }
+
+    return !m_hasErrors;
+}
+
+bool validateStructuredBufferResourceTypes(
+    IRModule* module,
+    DiagnosticSink* sink,
+    TargetRequest* targetRequest)
+{
+    StructuredBufferValidationContext context(sink, targetRequest);
+    return context.validate(module);
+}
+
+void validateAtomicOperations(IRModule* module, bool skipFuncParamValidation, DiagnosticSink* sink)
+{
+    validateAtomicOperations(skipFuncParamValidation, sink, module->getModuleInst());
+}
+
 } // namespace Slang
