@@ -1,24 +1,26 @@
 // slang-ir.h
-#ifndef SLANG_IR_H_INCLUDED
-#define SLANG_IR_H_INCLUDED
+#pragma once
 
 // This file defines the intermediate representation (IR) used for Slang
 // shader code. This is a typed static single assignment (SSA) IR,
 // similar in spirit to LLVM (but much simpler).
 //
 
-#if defined(__cpp_lib_bit_cast)
-#include <bit>
-#endif
 #include "../compiler-core/slang-source-loc.h"
 #include "../compiler-core/slang-source-map.h"
 #include "../core/slang-basic.h"
 #include "../core/slang-memory-arena.h"
+#include "slang-ast-type.h"
 #include "slang-container-pool.h"
+#include "slang-ir-insts-enum.h"
 #include "slang-type-system-shared.h"
 
 #include <functional>
 
+//
+#include "slang-ir.h.fiddle"
+
+FIDDLE()
 namespace Slang
 {
 
@@ -30,6 +32,7 @@ class Layout;
 class Type;
 class Session;
 class Name;
+class TargetRequest;
 struct IRBuilder;
 struct IRFunc;
 struct IRGlobalValueWithCode;
@@ -38,6 +41,8 @@ struct IRModule;
 struct IRStructField;
 struct IRStructKey;
 
+FIDDLE(instStructForwardDecls())
+
 typedef unsigned int IROpFlags;
 enum : IROpFlags
 {
@@ -50,37 +55,6 @@ enum : IROpFlags
         1 << 3, ///< If set this op should always be hoisted but should never be deduplicated.
 };
 
-/* Bit usage of IROp is a follows
-
-          MainOp | Other
-Bit range: 0-10   | Remaining bits
-
-For doing range checks (for example for doing isa tests), the value is masked by kIROpMask_OpMask,
-such that the Other bits don't interfere. The other bits can be used for storage for anything that
-needs to identify as a different 'op' or 'type'. It is currently used currently for storing the
-TextureFlavor of a IRResourceTypeBase derived types for example.
-
-TODO: We should eliminate the use of the "other" bits so that the entire value/state
-of an instruction is manifest in its opcode, operands, and children.
-*/
-enum IROp : int32_t
-{
-#define INST(ID, MNEMONIC, ARG_COUNT, FLAGS) kIROp_##ID,
-#include "slang-ir-inst-defs.h"
-
-    /// The total number of valid opcodes
-    kIROpCount,
-
-    /// An invalid opcode used to represent a missing or unknown opcode value.
-    kIROp_Invalid = kIROpCount,
-
-#define INST(ID, MNEMONIC, ARG_COUNT, FLAGS) /* empty */
-#define INST_RANGE(BASE, FIRST, LAST) \
-    kIROp_First##BASE = kIROp_##FIRST, kIROp_Last##BASE = kIROp_##LAST,
-
-#include "slang-ir-inst-defs.h"
-};
-
 /* IROpMeta describes values for the layout of IROps */
 enum IROpMeta
 {
@@ -105,11 +79,7 @@ enum IRMemoryOrder
 
 inline int32_t operator&(const IROpMask m, const IROp o)
 {
-#if defined(__cpp_lib_bit_cast)
-    return std::bit_cast<int32_t>(m) & std::bit_cast<int32_t>(o);
-#else
-    return (int32_t)m & (int32_t)o;
-#endif
+    return int32_t{m} & int32_t{o};
 }
 
 inline int32_t operator&(const IROp o, const IROpMask m)
@@ -555,6 +525,9 @@ enum class IRTypeLayoutRuleName
     Std430,
     Std140,
     D3DConstantBuffer,
+    MetalParameterBlock,
+    C,
+    LLVM,
     _Count,
 };
 
@@ -563,8 +536,10 @@ struct IRBlock;
 // Every value in the IR is an instruction (even things
 // like literal values).
 //
+FIDDLE()
 struct IRInst
 {
+    FIDDLE(...)
     // The operation that this value represents
     IROp m_op;
 
@@ -723,6 +698,12 @@ struct IRInst
 
     IRUse* getOperands();
 
+    IRUse* getOperandUse(UInt index)
+    {
+        SLANG_ASSERT(index < getOperandCount());
+        return getOperands() + index;
+    }
+
     IRInst* getOperand(UInt index)
     {
         SLANG_ASSERT(index < getOperandCount());
@@ -969,39 +950,18 @@ typename IRFilteredInstList<T>::Iterator IRFilteredInstList<T>::end()
 
 // Types
 
-#define IR_LEAF_ISA(NAME)                               \
-    static bool isaImpl(IROp op)                        \
-    {                                                   \
-        return (kIROpMask_OpMask & op) == kIROp_##NAME; \
-    }
-#define IR_PARENT_ISA(NAME)                                       \
-    static bool isaImpl(IROp opIn)                                \
-    {                                                             \
-        const int op = (kIROpMask_OpMask & opIn);                 \
-        return op >= kIROp_First##NAME && op <= kIROp_Last##NAME; \
-    }
-
-#define SIMPLE_IR_TYPE(NAME, BASE) \
-    struct IR##NAME : IR##BASE     \
-    {                              \
-        IR_LEAF_ISA(NAME)          \
-    };
-#define SIMPLE_IR_PARENT_TYPE(NAME, BASE) \
-    struct IR##NAME : IR##BASE            \
-    {                                     \
-        IR_PARENT_ISA(NAME)               \
-    };
-
 
 // All types in the IR are represented as instructions which conceptually
 // execute before run time.
+FIDDLE()
 struct IRType : IRInst
 {
+    FIDDLE(baseInst{noIsaImpl = true})
     IRType* getCanonicalType() { return this; }
 
     // Hack: specialize can also be a type. We should consider using a
     // separate `specializeType` op code for types so we can use the normal
-    // `IR_PARENT_ISA` macro here.
+    // isaImpl definition macro here.
     static bool isaImpl(IROp opIn)
     {
         const int op = (kIROpMask_OpMask & opIn);
@@ -1010,33 +970,22 @@ struct IRType : IRInst
 };
 
 IRType* unwrapArray(IRType* type);
+IRType* unwrapArrayAndPointers(IRType* type);
 
+FIDDLE()
 struct IRBasicType : IRType
 {
+    FIDDLE(baseInst())
     BaseType getBaseType() { return BaseType(getOp() - kIROp_FirstBasicType); }
-
-    IR_PARENT_ISA(BasicType)
 };
 
-struct IRVoidType : IRBasicType
-{
-    IR_LEAF_ISA(VoidType)
-};
-
-struct IRBoolType : IRBasicType
-{
-    IR_LEAF_ISA(BoolType)
-};
 
+FIDDLE()
 struct IRStringTypeBase : IRType
 {
-    IR_PARENT_ISA(StringTypeBase)
+    FIDDLE(baseInst())
 };
 
-SIMPLE_IR_TYPE(StringType, StringTypeBase)
-SIMPLE_IR_TYPE(NativeStringType, StringTypeBase)
-
-SIMPLE_IR_TYPE(DynamicType, Type)
 
 // True if types are equal
 // Note compares nominal types by name alone
@@ -1054,11 +1003,17 @@ struct IntInfo
     bool operator==(const IntInfo& i) const { return width == i.width && isSigned == i.isSigned; }
 };
 
-IntInfo getIntTypeInfo(const IRType* intType);
+// The size of some integer types (IntPtr & UIntPtr) depend on the target.
+Int getIntTypeWidth(TargetRequest* targetReq, IRType* intType);
+std::optional<Int> maybeGetIntTypeWidth(IRType* intType);
+bool getIntTypeSigned(IRType* intType);
+IntInfo getIntTypeInfo(TargetRequest* targetReq, IRType* intType);
 
 // left-inverse of getIntTypeInfo
 IROp getIntTypeOpFromInfo(const IntInfo info);
 
+IROp getOppositeSignIntTypeOp(IROp op);
+
 struct FloatInfo
 {
     Int width;
@@ -1079,8 +1034,10 @@ typedef int64_t IRIntegerValue;
 typedef uint64_t IRUnsignedIntegerValue;
 typedef double IRFloatingPointValue;
 
+FIDDLE()
 struct IRConstant : IRInst
 {
+    FIDDLE(baseInst())
     enum class FloatKind
     {
         Finite,
@@ -1103,6 +1060,7 @@ struct IRConstant : IRInst
     union ValueUnion
     {
         IRIntegerValue intVal; ///< Used for integrals and boolean
+        IRUnsignedIntegerValue uintVal;
         IRFloatingPointValue floatVal;
         void* ptrVal;
 
@@ -1133,7 +1091,6 @@ struct IRConstant : IRInst
     /// Get the hash
     HashCode getHashCode();
 
-    IR_PARENT_ISA(Constant)
 
     // Must be last member, because data may be held behind
     // NOTE! The total size of IRConstant may not be allocated - only enough space is allocated for
@@ -1141,58 +1098,57 @@ struct IRConstant : IRInst
     ValueUnion value;
 };
 
+FIDDLE()
 struct IRIntLit : IRConstant
 {
+    FIDDLE(leafInst())
     IRIntegerValue getValue() { return value.intVal; }
-
-    IR_LEAF_ISA(IntLit);
 };
 
+FIDDLE()
 struct IRFloatLit : IRConstant
 {
+    FIDDLE(leafInst())
     IRFloatingPointValue getValue() { return value.floatVal; }
-
-    IR_LEAF_ISA(FloatLit);
 };
 
+FIDDLE()
 struct IRBoolLit : IRConstant
 {
+    FIDDLE(leafInst())
     bool getValue() { return value.intVal != 0; }
+};
 
-    IR_LEAF_ISA(BoolLit);
+FIDDLE()
+struct IRStringLit : IRConstant
+{
+    FIDDLE(leafInst());
 };
 
 // Get the compile-time constant integer value of an instruction,
 // if it has one, and assert-fail otherwise.
 IRIntegerValue getIntVal(IRInst* inst);
 
-struct IRStringLit : IRConstant
-{
+// If it's a specialization constant sized array or unsized array, returns
+// kUnsizedArrayMagicLength if it's an unsized array. Otherwise just returns
+// the actual size.
+IRIntegerValue getArraySizeVal(IRInst* inst);
 
-    IR_LEAF_ISA(StringLit);
-};
-
-struct IRBlobLit : IRConstant
-{
-    IR_LEAF_ISA(BlobLit);
-};
 
+FIDDLE()
 struct IRPtrLit : IRConstant
 {
-    IR_LEAF_ISA(PtrLit);
+    FIDDLE(leafInst())
 
     void* getValue() { return value.ptrVal; }
 };
 
-struct IRVoidLit : IRConstant
-{
-    IR_LEAF_ISA(VoidLit);
-};
 
 // A instruction that ends a basic block (usually because of control flow)
+FIDDLE()
 struct IRTerminatorInst : IRInst
 {
-    IR_PARENT_ISA(TerminatorInst)
+    FIDDLE(baseInst())
 };
 
 // A function parameter is owned by a basic block, and represents
@@ -1203,12 +1159,12 @@ struct IRTerminatorInst : IRInst
 // In each case, the basic idea is that a block is a "label with
 // arguments."
 //
+FIDDLE()
 struct IRParam : IRInst
 {
+    FIDDLE(leafInst())
     IRParam* getNextParam();
     IRParam* getPrevParam();
-
-    IR_LEAF_ISA(Param)
 };
 
 /// A control-flow edge from one basic block to another
@@ -1238,8 +1194,10 @@ private:
 // no function declarations, or nested blocks). We also expect
 // that the previous/next instruction are always a basic block.
 //
+FIDDLE()
 struct IRBlock : IRInst
 {
+    FIDDLE(leafInst())
     // Linked list of the instructions contained in this block
     //
     IRInst* getFirstInst() { return getChildren().first; }
@@ -1348,19 +1306,19 @@ struct IRBlock : IRInst
 
     struct SuccessorList
     {
-        SuccessorList(IRUse* begin, IRUse* end, UInt stride = 1)
+        SuccessorList(IRUse* begin, IRUse* end, Int stride = 1)
             : begin_(begin), end_(end), stride(stride)
         {
         }
         IRUse* begin_;
         IRUse* end_;
-        UInt stride;
+        Int stride;
 
         UInt getCount();
 
         struct Iterator
         {
-            Iterator(IRUse* use, UInt stride)
+            Iterator(IRUse* use, Int stride)
                 : use(use), stride(stride)
             {
             }
@@ -1374,25 +1332,26 @@ struct IRBlock : IRInst
             IREdge getEdge() const { return IREdge(use); }
 
             IRUse* use;
-            UInt stride;
+            Int stride;
         };
 
         Iterator begin() { return Iterator(begin_, stride); }
         Iterator end() { return Iterator(end_, stride); }
+
+        SuccessorList reverse() { return SuccessorList(end_ - stride, begin_ - stride, -stride); }
     };
 
     PredecessorList getPredecessors();
     SuccessorList getSuccessors();
 
     //
-
-    IR_LEAF_ISA(Block)
 };
 
-SIMPLE_IR_TYPE(BasicBlockType, Type)
 
+FIDDLE()
 struct IRResourceTypeBase : IRType
 {
+    FIDDLE(baseInst())
     IRInst* getShapeInst() { return getOperand(kCoreModule_TextureShapeParameterIndex); }
     IRInst* getIsArrayInst() { return getOperand(kCoreModule_TextureIsArrayParameterIndex); }
     IRInst* getIsMultisampleInst()
@@ -1460,152 +1419,105 @@ struct IRResourceTypeBase : IRType
         }
         return SLANG_RESOURCE_ACCESS_UNKNOWN;
     }
-
-    IR_PARENT_ISA(ResourceTypeBase);
 };
 
+FIDDLE()
 struct IRResourceType : IRResourceTypeBase
 {
+    FIDDLE(baseInst())
     IRType* getElementType() { return (IRType*)getOperand(0); }
     IRInst* getSampleCount() { return getSampleCountInst(); }
     bool hasFormat() { return getOperandCount() >= 9; }
     IRIntegerValue getFormat() { return getIntVal(getFormatInst()); }
-
-    IR_PARENT_ISA(ResourceType)
 };
 
+FIDDLE()
 struct IRTextureTypeBase : IRResourceType
 {
-    IR_PARENT_ISA(TextureTypeBase)
-};
-
-struct IRTextureType : IRTextureTypeBase
-{
-    IR_LEAF_ISA(TextureType)
-};
-
-struct IRGLSLImageType : IRTextureTypeBase
-{
-    IR_LEAF_ISA(GLSLImageType)
+    FIDDLE(baseInst())
 };
 
+FIDDLE()
 struct IRSubpassInputType : IRType
 {
-    IRType* getElementType() { return (IRType*)getOperand(0); }
-    IRInst* getIsMultisampleInst() { return getOperand(1); }
+    FIDDLE(leafInst())
     bool isMultisample() { return getIntVal(getIsMultisampleInst()) == 1; }
-
-    IR_LEAF_ISA(SubpassInputType)
 };
 
+FIDDLE()
 struct IRSamplerStateTypeBase : IRType
 {
-    IR_PARENT_ISA(SamplerStateTypeBase)
+    FIDDLE(baseInst())
 };
 
-SIMPLE_IR_TYPE(SamplerStateType, SamplerStateTypeBase)
-SIMPLE_IR_TYPE(SamplerComparisonStateType, SamplerStateTypeBase)
 
+FIDDLE()
 struct IRBuiltinGenericType : IRType
 {
+    FIDDLE(baseInst())
     IRType* getElementType() { return (IRType*)getOperand(0); }
-
-    IR_PARENT_ISA(BuiltinGenericType)
 };
 
-SIMPLE_IR_PARENT_TYPE(PointerLikeType, BuiltinGenericType);
 
+FIDDLE()
 struct IRHLSLStructuredBufferTypeBase : IRBuiltinGenericType
 {
-    IRType* getDataLayout() { return (IRType*)getOperand(1); }
-
-    IR_PARENT_ISA(HLSLStructuredBufferTypeBase)
+    FIDDLE(baseInst())
+    IRType* getDataLayout() { return getOperandCount() > 1 ? (IRType*)getOperand(1) : nullptr; }
 };
 
-SIMPLE_IR_TYPE(HLSLStructuredBufferType, HLSLStructuredBufferTypeBase)
-SIMPLE_IR_TYPE(HLSLRWStructuredBufferType, HLSLStructuredBufferTypeBase)
-SIMPLE_IR_TYPE(HLSLRasterizerOrderedStructuredBufferType, HLSLStructuredBufferTypeBase)
-
-SIMPLE_IR_PARENT_TYPE(UntypedBufferResourceType, Type)
-SIMPLE_IR_PARENT_TYPE(ByteAddressBufferTypeBase, UntypedBufferResourceType)
-SIMPLE_IR_TYPE(HLSLByteAddressBufferType, ByteAddressBufferTypeBase)
-SIMPLE_IR_TYPE(HLSLRWByteAddressBufferType, ByteAddressBufferTypeBase)
-SIMPLE_IR_TYPE(HLSLRasterizerOrderedByteAddressBufferType, ByteAddressBufferTypeBase)
-
-SIMPLE_IR_TYPE(HLSLAppendStructuredBufferType, HLSLStructuredBufferTypeBase)
-SIMPLE_IR_TYPE(HLSLConsumeStructuredBufferType, HLSLStructuredBufferTypeBase)
 
+FIDDLE()
 struct IRHLSLPatchType : IRType
 {
+    FIDDLE(baseInst())
     IRType* getElementType() { return (IRType*)getOperand(0); }
     IRInst* getElementCount() { return getOperand(1); }
-
-    IR_PARENT_ISA(HLSLPatchType)
 };
 
-SIMPLE_IR_TYPE(HLSLInputPatchType, HLSLPatchType)
-SIMPLE_IR_TYPE(HLSLOutputPatchType, HLSLPatchType)
-
-SIMPLE_IR_PARENT_TYPE(HLSLStreamOutputType, BuiltinGenericType)
-SIMPLE_IR_TYPE(HLSLPointStreamType, HLSLStreamOutputType)
-SIMPLE_IR_TYPE(HLSLLineStreamType, HLSLStreamOutputType)
-SIMPLE_IR_TYPE(HLSLTriangleStreamType, HLSLStreamOutputType)
 
 // Mesh shaders
 // TODO: Ellie, should this parent struct be shared with Patch?
 // IRArrayLikeType? IROpaqueArrayLikeType?
+FIDDLE()
 struct IRMeshOutputType : IRType
 {
+    FIDDLE(baseInst())
     IRType* getElementType() { return (IRType*)getOperand(0); }
     IRInst* getMaxElementCount() { return getOperand(1); }
-
-    IR_PARENT_ISA(MeshOutputType)
 };
 
-SIMPLE_IR_TYPE(VerticesType, MeshOutputType)
-SIMPLE_IR_TYPE(IndicesType, MeshOutputType)
-SIMPLE_IR_TYPE(PrimitivesType, MeshOutputType)
 
-struct IRMetalMeshType : IRType
+FIDDLE()
+struct IRPointerLikeType : IRBuiltinGenericType
 {
-    IR_LEAF_ISA(MetalMeshType);
-
-    IRType* getVerticesType() { return (IRType*)getOperand(0); }
-    IRType* getPrimitivesType() { return (IRType*)getOperand(1); }
-    IRInst* getNumVertices() { return (IRInst*)getOperand(2); }
-    IRInst* getNumPrimitives() { return (IRInst*)getOperand(3); }
-    IRIntLit* getTopology() { return (IRIntLit*)getOperand(4); }
+    FIDDLE(baseInst())
 };
 
-SIMPLE_IR_TYPE(MetalMeshGridPropertiesType, Type)
-
-SIMPLE_IR_TYPE(GLSLInputAttachmentType, Type)
-SIMPLE_IR_PARENT_TYPE(ParameterGroupType, PointerLikeType)
+FIDDLE()
+struct IRParameterGroupType : IRPointerLikeType
+{
+    FIDDLE(baseInst())
+};
 
+FIDDLE()
 struct IRUniformParameterGroupType : IRParameterGroupType
 {
-    IR_PARENT_ISA(UniformParameterGroupType)
+    FIDDLE(baseInst())
 
     IRType* getDataLayout() { return getOperandCount() > 1 ? (IRType*)getOperand(1) : nullptr; }
 };
 
-SIMPLE_IR_PARENT_TYPE(VaryingParameterGroupType, ParameterGroupType)
-SIMPLE_IR_TYPE(ConstantBufferType, UniformParameterGroupType)
-SIMPLE_IR_TYPE(TextureBufferType, UniformParameterGroupType)
-SIMPLE_IR_TYPE(GLSLInputParameterGroupType, VaryingParameterGroupType)
-SIMPLE_IR_TYPE(GLSLOutputParameterGroupType, VaryingParameterGroupType)
-SIMPLE_IR_TYPE(ParameterBlockType, UniformParameterGroupType)
-SIMPLE_IR_TYPE(DynamicResourceType, Type)
-
-struct IRGLSLShaderStorageBufferType : IRBuiltinGenericType
+FIDDLE()
+struct IRGLSLShaderStorageBufferType : IRPointerLikeType
 {
-    IRType* getDataLayout() { return (IRType*)getOperand(1); }
-
-    IR_LEAF_ISA(GLSLShaderStorageBufferType)
+    FIDDLE(leafInst())
 };
 
+FIDDLE()
 struct IRArrayTypeBase : IRType
 {
+    FIDDLE(baseInst())
     IRType* getElementType() { return (IRType*)getOperand(0); }
 
     // Returns the element count for an `IRArrayType`, and null
@@ -1628,50 +1540,13 @@ struct IRArrayTypeBase : IRType
         }
         return nullptr;
     }
-
-    IR_PARENT_ISA(ArrayTypeBase)
 };
 
-struct IRArrayType : IRArrayTypeBase
-{
-    IR_LEAF_ISA(ArrayType)
-};
-
-SIMPLE_IR_TYPE(UnsizedArrayType, ArrayTypeBase)
-
-struct IRAtomicType : IRType
-{
-    IR_LEAF_ISA(AtomicType)
-
-    IRType* getElementType() { return (IRType*)getOperand(0); }
-};
-
-SIMPLE_IR_PARENT_TYPE(Rate, Type)
-SIMPLE_IR_TYPE(ConstExprRate, Rate)
-SIMPLE_IR_TYPE(GroupSharedRate, Rate)
-SIMPLE_IR_TYPE(ActualGlobalRate, Rate)
-
-struct IRRateQualifiedType : IRType
-{
-    IRRate* getRate() { return (IRRate*)getOperand(0); }
-    IRType* getValueType() { return (IRType*)getOperand(1); }
-
-    IR_LEAF_ISA(RateQualifiedType)
-};
-
-struct IRDescriptorHandleType : IRType
-{
-    IRType* getResourceType() { return (IRType*)getOperand(0); }
-    IR_LEAF_ISA(DescriptorHandleType)
-};
 
 // Unlike the AST-level type system where `TypeType` tracks the
 // underlying type, the "type of types" in the IR is a simple
 // value with no operands, so that all type nodes have the
 // same type.
-SIMPLE_IR_PARENT_TYPE(Kind, Type);
-SIMPLE_IR_TYPE(TypeKind, Kind);
-SIMPLE_IR_TYPE(TypeParameterPackKind, Kind);
 
 // The kind of any and all generics.
 //
@@ -1682,155 +1557,48 @@ SIMPLE_IR_TYPE(TypeParameterPackKind, Kind);
 // "higher-kinded" generics (e.g., a generic that takes another
 // generic as a parameter).
 //
-SIMPLE_IR_TYPE(GenericKind, Kind)
 
+FIDDLE()
 struct IRDifferentialPairTypeBase : IRType
 {
+    FIDDLE(baseInst())
     IRType* getValueType() { return (IRType*)getOperand(0); }
     IRInst* getWitness() { return (IRInst*)getOperand(1); }
-
-    IR_PARENT_ISA(DifferentialPairTypeBase)
 };
 
-struct IRDifferentialPairType : IRDifferentialPairTypeBase
-{
-    IR_LEAF_ISA(DifferentialPairType)
-};
-
-struct IRDifferentialPtrPairType : IRDifferentialPairTypeBase
-{
-    IR_LEAF_ISA(DifferentialPtrPairType)
-};
-
-struct IRDifferentialPairUserCodeType : IRDifferentialPairTypeBase
-{
-    IR_LEAF_ISA(DifferentialPairUserCodeType)
-};
-
-struct IRBackwardDiffIntermediateContextType : IRType
-{
-    IRInst* getFunc() { return getOperand(0); }
-    IR_LEAF_ISA(BackwardDiffIntermediateContextType)
-};
-
-struct IRVectorType : IRType
-{
-    IRType* getElementType() { return (IRType*)getOperand(0); }
-    IRInst* getElementCount() { return getOperand(1); }
-
-    IR_LEAF_ISA(VectorType)
-};
-
-struct IRMatrixType : IRType
-{
-    IRType* getElementType() { return (IRType*)getOperand(0); }
-    IRInst* getRowCount() { return getOperand(1); }
-    IRInst* getColumnCount() { return getOperand(2); }
-    IRInst* getLayout() { return getOperand(3); }
-
-    IR_LEAF_ISA(MatrixType)
-};
-
-struct IRArrayListType : IRType
-{
-    IRType* getElementType() { return (IRType*)getOperand(0); }
-
-    IR_LEAF_ISA(ArrayListType)
-};
-
-struct IRTensorViewType : IRType
-{
-    IRType* getElementType() { return (IRType*)getOperand(0); }
-
-    IR_LEAF_ISA(TensorViewType)
-};
-
-struct IRTorchTensorType : IRType
-{
-    IR_LEAF_ISA(TorchTensorType)
-};
-
-struct IRSPIRVLiteralType : IRType
-{
-    IR_LEAF_ISA(SPIRVLiteralType)
-
-    IRType* getValueType() { return static_cast<IRType*>(getOperand(0)); }
-};
 
+FIDDLE()
 struct IRPtrTypeBase : IRType
 {
+    FIDDLE(baseInst())
     IRType* getValueType() { return (IRType*)getOperand(0); }
 
+    AccessQualifier getAccessQualifier()
+    {
+        return getOperandCount() > 1
+                   ? (AccessQualifier) static_cast<IRIntLit*>(getOperand(1))->getValue()
+                   : AccessQualifier::ReadWrite;
+    }
+
     bool hasAddressSpace()
     {
-        return getOperandCount() > 1 && getAddressSpace() != AddressSpace::Generic;
+        return getOperandCount() > 2 && getAddressSpace() != AddressSpace::Generic;
     }
 
     AddressSpace getAddressSpace()
     {
-        return getOperandCount() > 1
-                   ? (AddressSpace) static_cast<IRIntLit*>(getOperand(1))->getValue()
+        return getOperandCount() > 2
+                   ? (AddressSpace) static_cast<IRIntLit*>(getOperand(2))->getValue()
                    : AddressSpace::Generic;
     }
-
-    IR_PARENT_ISA(PtrTypeBase)
 };
 
-SIMPLE_IR_TYPE(PtrType, PtrTypeBase)
-SIMPLE_IR_TYPE(RefType, PtrTypeBase)
-SIMPLE_IR_TYPE(ConstRefType, PtrTypeBase)
-SIMPLE_IR_PARENT_TYPE(OutTypeBase, PtrTypeBase)
-SIMPLE_IR_TYPE(OutType, OutTypeBase)
-SIMPLE_IR_TYPE(InOutType, OutTypeBase)
-
-struct IRComPtrType : public IRType
-{
-    IR_LEAF_ISA(ComPtrType);
-    IRType* getValueType() { return (IRType*)getOperand(0); }
-};
-
-struct IRNativePtrType : public IRType
-{
-    IR_LEAF_ISA(NativePtrType);
-    IRType* getValueType() { return (IRType*)getOperand(0); }
-};
-
-struct IRPseudoPtrType : public IRPtrTypeBase
-{
-    IR_LEAF_ISA(PseudoPtrType);
-};
 
 /// The base class of RawPointerType and RTTIPointerType.
+FIDDLE()
 struct IRRawPointerTypeBase : IRType
 {
-    IR_PARENT_ISA(RawPointerTypeBase);
-};
-
-/// Represents a pointer to an object of unknown type.
-struct IRRawPointerType : IRRawPointerTypeBase
-{
-    IR_LEAF_ISA(RawPointerType)
-};
-
-/// Represents a pointer to an object whose type is determined at runtime,
-/// with type information available through `rttiOperand`.
-///
-struct IRRTTIPointerType : IRRawPointerTypeBase
-{
-    IRInst* getRTTIOperand() { return getOperand(0); }
-    IR_LEAF_ISA(RTTIPointerType)
-};
-
-struct IRGlobalHashedStringLiterals : IRInst
-{
-    IR_LEAF_ISA(GlobalHashedStringLiterals)
-};
-
-struct IRGetStringHash : IRInst
-{
-    IR_LEAF_ISA(GetStringHash)
-
-    IRStringLit* getStringLit() { return as<IRStringLit>(getOperand(0)); }
+    FIDDLE(baseInst())
 };
 
 /// Get the type pointed to be `ptrType`, or `nullptr` if it is not a pointer(-like) type.
@@ -1838,63 +1606,44 @@ struct IRGetStringHash : IRInst
 /// The given IR `builder` will be used if new instructions need to be created.
 IRType* tryGetPointedToType(IRBuilder* builder, IRType* type);
 
+IRType* tryGetPointedToOrBufferElementType(IRBuilder* builder, IRType* type);
+
+FIDDLE()
 struct IRFuncType : IRType
 {
-    IRType* getResultType() { return (IRType*)getOperand(0); }
+    FIDDLE(leafInst())
     UInt getParamCount() { return getOperandCount() - 1; }
     IRType* getParamType(UInt index) { return (IRType*)getOperand(1 + index); }
     IROperandList<IRType> getParamTypes()
     {
         return IROperandList<IRType>(getOperands() + 1, getOperands() + getOperandCount());
     }
-
-    IR_LEAF_ISA(FuncType)
 };
 
-struct IRRayQueryType : IRType
+FIDDLE()
+struct IRTensorAddressingTensorViewType : IRType
 {
-    IR_LEAF_ISA(RayQueryType)
-};
-
-struct IRHitObjectType : IRType
-{
-    IR_LEAF_ISA(HitObjectType)
-};
-
-struct IRCoopVectorType : IRType
-{
-    IRType* getElementType() { return (IRType*)getOperand(0); }
-    IRInst* getElementCount() { return getOperand(1); }
-
-    IR_LEAF_ISA(CoopVectorType)
-};
-
-struct IRCoopMatrixType : IRType
-{
-    IRType* getElementType() { return (IRType*)getOperand(0); }
-    IRInst* getScope() { return getOperand(1); }
-    IRInst* getRowCount() { return getOperand(2); }
-    IRInst* getColumnCount() { return getOperand(3); }
-    IRInst* getMatrixUse() { return getOperand(4); }
-
-    IR_LEAF_ISA(CoopMatrixType)
+    FIDDLE(leafInst())
+    IRInst* getPermutation(int index) { return getOperand(2 + index); }
 };
 
 bool isDefinition(IRInst* inVal);
 
 // A structure type is represented as a parent instruction,
-// where the child instructions represent the fields of the
+// where the child instructions repres ent the fields of the
 // struct.
 //
-// The space of fields that a given struct type supports
-// are defined as its "keys", which are global values
-// (that is, they have mangled names that can be used
+// The space of fields that a given st ruct type supports
+// are defined as its "keys", which ar e global values
+// (that is, they have mangled names t hat can be used
 // for linkage).
 //
+FIDDLE()
 struct IRStructKey : IRInst
 {
-    IR_LEAF_ISA(StructKey)
+    FIDDLE(leafInst())
 };
+
 //
 // The fields of the struct are then defined as mappings
 // from those keys to the associated type (in the case of
@@ -1903,8 +1652,10 @@ struct IRStructKey : IRInst
 // A struct field thus has two operands: the key, and the
 // type of the field.
 //
+FIDDLE()
 struct IRStructField : IRInst
 {
+    FIDDLE(leafInst())
     IRStructKey* getKey() { return cast<IRStructKey>(getOperand(0)); }
     IRType* getFieldType()
     {
@@ -1916,8 +1667,6 @@ struct IRStructField : IRInst
         return (IRType*)getOperand(1);
     }
     void setFieldType(IRType* type) { setOperand(1, type); }
-
-    IR_LEAF_ISA(StructField)
 };
 //
 // The struct type is then represented as a parent instruction
@@ -1925,183 +1674,103 @@ struct IRStructField : IRInst
 // *not* contain the keys, because code needs to be able to
 // reference the keys from scopes outside of the struct.
 //
+FIDDLE()
 struct IRStructType : IRType
 {
+    FIDDLE(leafInst())
     IRFilteredInstList<IRStructField> getFields()
     {
         return IRFilteredInstList<IRStructField>(getChildren());
     }
-
-    IR_LEAF_ISA(StructType)
 };
 
+FIDDLE()
 struct IRClassType : IRType
 {
+    FIDDLE(leafInst())
     IRFilteredInstList<IRStructField> getFields()
     {
         return IRFilteredInstList<IRStructField>(getChildren());
     }
-
-    IR_LEAF_ISA(ClassType)
-};
-
-struct IRAssociatedType : IRType
-{
-    IR_LEAF_ISA(AssociatedType)
 };
 
+FIDDLE()
 struct IRThisType : IRType
 {
-    IR_LEAF_ISA(ThisType)
+    FIDDLE(leafInst())
 
     IRInst* getConstraintType() { return getOperand(0); }
 };
 
+FIDDLE()
 struct IRThisTypeWitness : IRInst
 {
-    IR_LEAF_ISA(ThisTypeWitness)
+    FIDDLE(leafInst())
 
     IRInst* getConstraintType() { return getOperand(0); }
 };
 
+FIDDLE()
 struct IRInterfaceRequirementEntry : IRInst
 {
-    IRInst* getRequirementKey() { return getOperand(0); }
-    IRInst* getRequirementVal() { return getOperand(1); }
+    FIDDLE(leafInst())
     void setRequirementKey(IRInst* val) { setOperand(0, val); }
     void setRequirementVal(IRInst* val) { setOperand(1, val); }
-
-    IR_LEAF_ISA(InterfaceRequirementEntry);
 };
 
+FIDDLE()
 struct IRInterfaceType : IRType
 {
-    IR_LEAF_ISA(InterfaceType)
+    FIDDLE(leafInst())
 
     UInt getRequirementCount() { return getOperandCount(); }
 };
 
+FIDDLE()
 struct IRConjunctionType : IRType
 {
-    IR_LEAF_ISA(ConjunctionType)
+    FIDDLE(leafInst())
 
     Int getCaseCount() { return getOperandCount(); }
     IRType* getCaseType(Int index) { return (IRType*)getOperand(index); }
 };
 
-struct IRAttributedType : IRType
-{
-    IR_LEAF_ISA(AttributedType)
-
-    IRType* getBaseType() { return (IRType*)getOperand(0); }
-    IRInst* getAttr() { return getOperand(1); }
-};
-
-struct IRTupleTypeBase : IRType
-{
-    IR_PARENT_ISA(TupleTypeBase)
-};
-
-/// Represents a tuple. Tuples are created by `IRMakeTuple` and its elements
-/// are accessed via `GetTupleElement(tupleValue, IRIntLit)`.
-struct IRTupleType : IRTupleTypeBase
-{
-    IR_LEAF_ISA(TupleType)
-};
-
-/// Represents a type pack. Type packs behave like tuples, but they have a
-/// "flattening" semantics, so that MakeTypePack(MakeTypePack(T1,T2), T3) is
-/// MakeTypePack(T1,T2,T3).
-struct IRTypePack : IRTupleTypeBase
-{
-    IR_LEAF_ISA(TypePack)
-};
-
 // A placeholder struct key for tuple type layouts that will be replaced with
 // the actual struct key when the tuple type is materialized into a struct type.
+FIDDLE()
 struct IRIndexedFieldKey : IRInst
 {
-    IR_LEAF_ISA(IndexedFieldKey)
-    IRInst* getBaseType() { return getOperand(0); }
-    IRInst* getIndex() { return getOperand(1); }
-};
-
-/// Represents a tuple in target language. TargetTupleType will not be lowered to structs.
-struct IRTargetTupleType : IRType
-{
-    IR_LEAF_ISA(TargetTupleType)
+    FIDDLE(leafInst())
 };
 
 /// Represents a `expand T` type used in variadic generic decls in Slang. Expected to be substituted
 /// by actual types during specialization.
-struct IRExpandType : IRType
+FIDDLE()
+struct IRExpandTypeOrVal : IRType
 {
-    IR_LEAF_ISA(ExpandTypeOrVal)
+    FIDDLE(leafInst())
     IRType* getPatternType() { return (IRType*)(getOperand(0)); }
     UInt getCaptureCount() { return getOperandCount() - 1; }
     IRType* getCaptureType(UInt index) { return (IRType*)(getOperand(index + 1)); }
 };
 
-/// Represents an `Result<T,E>`, used by functions that throws error codes.
-struct IRResultType : IRType
-{
-    IR_LEAF_ISA(ResultType)
-
-    IRType* getValueType() { return (IRType*)getOperand(0); }
-    IRType* getErrorType() { return (IRType*)getOperand(1); }
-};
-
-/// Represents an `Optional<T>`.
-struct IROptionalType : IRType
-{
-    IR_LEAF_ISA(OptionalType)
-
-    IRType* getValueType() { return (IRType*)getOperand(0); }
-};
-
-struct IRTypeType : IRType
-{
-    IR_LEAF_ISA(TypeType);
-};
-
-/// Represents the IR type for an `IRRTTIObject`.
-struct IRRTTIType : IRType
-{
-    IR_LEAF_ISA(RTTIType);
-};
-
-/// Represents a handle to an RTTI object.
-/// This is lowered as an integer number identifying a type.
-struct IRRTTIHandleType : IRType
-{
-    IR_LEAF_ISA(RTTIHandleType);
-};
-
-struct IRAnyValueType : IRType
-{
-    IR_LEAF_ISA(AnyValueType);
-    IRInst* getSize() { return getOperand(0); }
-};
-
+FIDDLE()
 struct IRWitnessTableTypeBase : IRType
 {
+    FIDDLE(baseInst())
     IRInst* getConformanceType() { return getOperand(0); }
-    IR_PARENT_ISA(WitnessTableTypeBase);
 };
 
+FIDDLE()
 struct IRWitnessTableType : IRWitnessTableTypeBase
 {
-    IR_LEAF_ISA(WitnessTableType);
-};
-
-struct IRWitnessTableIDType : IRWitnessTableTypeBase
-{
-    IR_LEAF_ISA(WitnessTableIDType);
+    FIDDLE(leafInst())
 };
 
+FIDDLE()
 struct IRBindExistentialsTypeBase : IRType
 {
-    IR_PARENT_ISA(BindExistentialsTypeBase)
+    FIDDLE(baseInst())
 
     IRType* getBaseType() { return (IRType*)getOperand(0); }
     UInt getExistentialArgCount() { return getOperandCount() - 1; }
@@ -2109,14 +1778,10 @@ struct IRBindExistentialsTypeBase : IRType
     IRInst* getExistentialArg(UInt index) { return getExistentialArgs()[index].get(); }
 };
 
-struct IRBindExistentialsType : IRBindExistentialsTypeBase
-{
-    IR_LEAF_ISA(BindExistentialsType)
-};
-
+FIDDLE()
 struct IRBoundInterfaceType : IRBindExistentialsTypeBase
 {
-    IR_LEAF_ISA(BoundInterfaceType)
+    FIDDLE(leafInst())
 
     IRType* getInterfaceType() { return getBaseType(); }
     IRType* getConcreteType() { return (IRType*)getExistentialArg(0); }
@@ -2126,20 +1791,22 @@ struct IRBoundInterfaceType : IRBindExistentialsTypeBase
 
 /// @brief A global value that potentially holds executable code.
 ///
+FIDDLE()
 struct IRGlobalValueWithCode : IRInst
 {
+    FIDDLE(baseInst())
     // The children of a value with code will be the basic
     // blocks of its definition.
     IRBlock* getFirstBlock() { return cast<IRBlock>(getFirstChild()); }
     IRBlock* getLastBlock() { return cast<IRBlock>(getLastChild()); }
     IRInstList<IRBlock> getBlocks() { return IRInstList<IRBlock>(getChildren()); }
-
-    IR_PARENT_ISA(GlobalValueWithCode)
 };
 
 // A value that has parameters so that it can conceptually be called.
+FIDDLE()
 struct IRGlobalValueWithParams : IRGlobalValueWithCode
 {
+    FIDDLE(baseInst())
     // Convenience accessor for the IR parameters,
     // which are actually the parameters of the first
     // block.
@@ -2147,16 +1814,16 @@ struct IRGlobalValueWithParams : IRGlobalValueWithCode
     IRParam* getLastParam();
     IRInstList<IRParam> getParams();
     IRInst* getFirstOrdinaryInst();
-
-    IR_PARENT_ISA(GlobalValueWithParams)
 };
 
 // A function is a parent to zero or more blocks of instructions.
 //
 // A function is itself a value, so that it can be a direct operand of
 // an instruction (e.g., a call).
+FIDDLE()
 struct IRFunc : IRGlobalValueWithParams
 {
+    FIDDLE(leafInst())
     // The type of the IR-level function
     IRFuncType* getDataType() { return (IRFuncType*)IRInst::getDataType(); }
 
@@ -2167,8 +1834,6 @@ struct IRFunc : IRGlobalValueWithParams
     IRType* getParamType(UInt index);
 
     bool isDefinition() { return getFirstBlock() != nullptr; }
-
-    IR_LEAF_ISA(Func)
 };
 
 /// Adjust the type of an IR function based on its parameter list.
@@ -2186,15 +1851,10 @@ void fixUpFuncType(IRFunc* func, IRType* resultType);
 ///
 void fixUpFuncType(IRFunc* func);
 
-// A generic is akin to a function, but is conceptually executed
-// before runtime, to specialize the code nested within.
-//
-// In practice, a generic always holds only a single block, and ends
-// with a `return` instruction for the value that the generic yields.
-struct IRGeneric : IRGlobalValueWithParams
-{
-    IR_LEAF_ISA(Generic)
-};
+/// If the function has a DebugFuncDecoration, replaces the function type in
+/// that decoration to match the current type of the function.
+///
+void fixUpDebugFuncType(IRFunc* func);
 
 // Find the value that is returned from a generic, so that
 // a pass can glean information from it.
@@ -2228,16 +1888,16 @@ IRInst* getResolvedInstForDecorations(IRInst* inst, bool resolveThroughDifferent
 
 // The IR module itself is represented as an instruction, which
 // serves at the root of the tree of all instructions in the module.
+FIDDLE()
 struct IRModuleInst : IRInst
 {
+    FIDDLE(leafInst())
     // Pointer back to the non-instruction object that represents
     // the module, so that we can get back to it in algorithms
     // that need it.
     IRModule* module;
 
     IRInstListBase getGlobalInsts() { return getChildren(); }
-
-    IR_LEAF_ISA(Module)
 };
 
 struct IRModule;
@@ -2361,8 +2021,10 @@ struct IRAnalysis
     IRDominatorTree* getDominatorTree();
 };
 
+FIDDLE()
 struct IRModule : RefObject
 {
+    FIDDLE(...)
 public:
     enum
     {
@@ -2395,6 +2057,8 @@ public:
 
     IRDeduplicationContext* getDeduplicationContext() const { return &m_deduplicationContext; }
 
+    Dictionary<IRInst*, UInt>* getUniqueIdMap() { return &m_mapInstToUniqueId; }
+
     IRDominatorTree* findDominatorTree(IRGlobalValueWithCode* func)
     {
         IRAnalysis* analysis = m_mapInstToAnalysis.tryGetValue(func);
@@ -2445,7 +2109,29 @@ public:
 
     ContainerPool& getContainerPool() { return m_containerPool; }
 
+    //
+    // The range of module versions this compiler supports
+    //
+    // This will need to be updated if for example an instruction is removed,
+    // the max supported version should be incremented and the min supported
+    // version set to above the last version an instance of that instruction
+    // could be found
+    //
+    // Additionally this should be updated when new instructions are added,
+    // however only k_maxSupportedModuleVersion needs to be incremented in that
+    // case
+    //
+    // It represents the version of module regarding semantics and doesn't have
+    // anything to do with serialization format
+    //
+    const static UInt k_minSupportedModuleVersion = 4;
+    const static UInt k_maxSupportedModuleVersion = 5;
+    static_assert(k_minSupportedModuleVersion <= k_maxSupportedModuleVersion);
+
 private:
+    friend struct IRSerialReadContext;
+    friend struct IRSerialWriteContext;
+    friend struct Fossilized_IRModule;
     IRModule() = delete;
 
     /// Ctor
@@ -2464,10 +2150,13 @@ private:
     /// instructions from an arbitrary IR instruction we expect to find the
     /// `IRModuleInst` for the module the instruction belongs to, if any.
     ///
-    IRModuleInst* m_moduleInst = nullptr;
+    FIDDLE() IRModuleInst* m_moduleInst = nullptr;
 
     // The name of the module.
-    Name* m_name = nullptr;
+    FIDDLE() Name* m_name = nullptr;
+
+    // The version of the module as it was loaded
+    FIDDLE() UInt64 m_version = k_maxSupportedModuleVersion;
 
     /// The memory arena from which all IR instructions (and any associated state) in this module
     /// are allocated.
@@ -2486,6 +2175,12 @@ private:
     Dictionary<IRInst*, IRAnalysis> m_mapInstToAnalysis;
 
     Dictionary<ImmutableHashedString, List<IRInst*>> m_mapMangledNameToGlobalInst;
+
+    /// Hold a mapping for inst -> uniqueID. This mapping is generated on
+    /// demand if passes need them, rather than eagerly storing them on
+    /// insts when unnecessary.
+    ///
+    Dictionary<IRInst*, UInt> m_mapInstToUniqueId;
 };
 
 
@@ -2575,12 +2270,6 @@ struct InstHashSet
     void clear() { set->clear(); }
 };
 
-
-struct IRSpecializationDictionaryItem : public IRInst
-{
-    IR_LEAF_ISA(SpecializationDictionaryItem)
-};
-
 struct IRDumpOptions
 {
     typedef uint32_t Flags;
@@ -2668,6 +2357,10 @@ bool isMovableInst(IRInst* inst);
 
 #if SLANG_ENABLE_IR_BREAK_ALLOC
 uint32_t& _debugGetIRAllocCounter();
+extern uint32_t _slangIRAllocBreak;
+extern bool _slangIRPrintStackAtBreak;
+void _debugSetInstBeingCloned(uint32_t uid);
+void _debugResetInstBeingCloned();
 #endif
 
 // TODO: Ellie, comment and move somewhere more appropriate?
@@ -2805,5 +2498,3 @@ R* composeGetters(T* t)
 }
 
 } // namespace Slang
-
-#endif
