@@ -14,6 +14,8 @@ struct ManglingContext
     }
     ASTBuilder* astBuilder;
     StringBuilder sb;
+    Decl* targetDecl = nullptr; // The decl we are mangling name for.
+    bool isEmittingTarget = false;
 };
 
 void emitRaw(ManglingContext* context, char const* text)
@@ -191,6 +193,7 @@ void emitBaseType(ManglingContext* context, BaseType baseType)
     case BaseType::IntPtr:
         emitRaw(context, "ip");
         break;
+
     default:
         SLANG_UNEXPECTED("unimplemented case in base type mangling");
         break;
@@ -252,7 +255,7 @@ void emitType(ManglingContext* context, Type* type)
         auto n = funcType->getParamCount();
         emit(context, n);
         for (Index i = 0; i < n; ++i)
-            emitType(context, funcType->getParamType(i));
+            emitType(context, funcType->getParamTypeWithModeWrapper(i));
         emitType(context, funcType->getResultType());
         emitType(context, funcType->getErrorType());
     }
@@ -323,7 +326,7 @@ void emitVal(ManglingContext* context, Val* val)
         // to mangle in the constraints even when
         // the whole thing is specialized...
     }
-    else if (auto genericParamIntVal = dynamicCast<GenericParamIntVal>(val))
+    else if (auto genericParamIntVal = dynamicCast<DeclRefIntVal>(val))
     {
         // TODO: we shouldn't be including the names of generic parameters
         // anywhere in mangled names, since changing parameter names
@@ -357,6 +360,21 @@ void emitVal(ManglingContext* context, Val* val)
         emitVal(context, lookupIntVal->getWitness());
         emitName(context, lookupIntVal->getKey()->getName());
     }
+    else if (auto sizeOfIntVal = dynamicCast<SizeOfIntVal>(val))
+    {
+        emitRaw(context, "KSO");
+        emitVal(context, sizeOfIntVal->getTypeArg());
+    }
+    else if (auto alignOfIntVal = dynamicCast<AlignOfIntVal>(val))
+    {
+        emitRaw(context, "KAO");
+        emitVal(context, alignOfIntVal->getTypeArg());
+    }
+    else if (auto countOfIntVal = dynamicCast<CountOfIntVal>(val))
+    {
+        emitRaw(context, "KCO");
+        emitVal(context, countOfIntVal->getTypeArg());
+    }
     else if (const auto polynomialIntVal = dynamicCast<PolynomialIntVal>(val))
     {
         emitRaw(context, "KX");
@@ -391,6 +409,31 @@ void emitVal(ManglingContext* context, Val* val)
 
 void emitQualifiedName(ManglingContext* context, DeclRef<Decl> declRef, bool includeModuleName)
 {
+    if (declRef.getDecl() == context->targetDecl)
+    {
+        if (!context->isEmittingTarget)
+        {
+            context->isEmittingTarget = true;
+        }
+        else
+        {
+            // We are already in the middle of mangling the target decl,
+            // so we must have run into a cycle.
+            //
+            // For example, this can happen when mangling a struct defined
+            // inline at a parameter-type locataion inside a function.
+            // During mangling that struct, we will mangle the parent function,
+            // but the mangled name of that parent function will contain
+            // parameter types, which means we will attempt to mangle the struct
+            // again.
+            //
+            // In this case we will just skip emitting for the current declRef
+            // to break the cycle.
+            return;
+        }
+    }
+
+    bool ignoreName = false;
     if (!includeModuleName)
     {
         if (as<ModuleDecl>(declRef))
@@ -445,19 +488,6 @@ void emitQualifiedName(ManglingContext* context, DeclRef<Decl> declRef, bool inc
         return;
     }
 
-    // Inheritance declarations don't have meaningful names,
-    // and so we should emit them based on the type
-    // that is doing the inheriting.
-    if (auto inheritanceDeclRef = declRef.as<TypeConstraintDecl>())
-    {
-        emit(context, "I");
-        emitType(context, getSup(context->astBuilder, inheritanceDeclRef));
-        return;
-    }
-
-    // Similarly, an extension doesn't have a name worth
-    // emitting, and we should base things on its target
-    // type instead.
     if (auto extensionDeclRef = declRef.as<ExtensionDecl>())
     {
         // TODO: as a special case, an "unconditional" extension
@@ -471,7 +501,25 @@ void emitQualifiedName(ManglingContext* context, DeclRef<Decl> declRef, bool inc
             emit(context, "I");
             emitType(context, getSup(context->astBuilder, inheritanceDecl));
         }
-        return;
+        // A non generic extension doesn't have a name worth
+        // emitting, and we should base things on its target
+        // type instead.
+        if (parentGenericDeclRef)
+        {
+            ignoreName = true;
+        }
+    }
+    // Inheritance declarations don't have meaningful names,
+    // and so we should emit them based on the type
+    // that is doing the inheriting.
+    else if (auto inheritanceDeclRef = declRef.as<TypeConstraintDecl>())
+    {
+        emit(context, "I");
+        emitType(context, getSup(context->astBuilder, inheritanceDeclRef));
+        if (parentGenericDeclRef)
+        {
+            ignoreName = true;
+        }
     }
 
     // TODO: we should special case GenericTypeParamDecl and GenericValueParamDecl nodes
@@ -480,7 +528,10 @@ void emitQualifiedName(ManglingContext* context, DeclRef<Decl> declRef, bool inc
     // For each generic parameter, we should assign it a unique ID (i, j), where i is the
     // nesting level of the generic, and j is the sequential order of the parameter within
     // its generic parent, and use this 2D ID to refer to such a parameter.
-    emitName(context, declRef.getName());
+    if (!ignoreName)
+    {
+        emitName(context, declRef.getName());
+    }
 
     // Special case: accessors need some way to distinguish themselves
     // so that a getter/setter/ref-er don't all compile to the same name.
@@ -576,14 +627,23 @@ void emitQualifiedName(ManglingContext* context, DeclRef<Decl> declRef, bool inc
             }
 
             auto canonicalizedConstraints =
-                getCanonicalGenericConstraints(context->astBuilder, parentGenericDeclRef);
+                getCanonicalGenericConstraints2(context->astBuilder, parentGenericDeclRef);
             for (auto& constraint : canonicalizedConstraints)
             {
-                for (auto type : constraint.value)
+                if (constraint.value.getCount() > 0)
                 {
                     emitRaw(context, "C");
-                    emitQualifiedName(context, makeDeclRef(constraint.key), true);
-                    emitType(context, type);
+                    emitType(context, constraint.key);
+                    int counter = 0;
+                    for (auto type : constraint.value)
+                    {
+                        if (counter > 0)
+                        {
+                            emitRaw(context, "_");
+                        }
+                        ++counter;
+                        emitType(context, type);
+                    }
                 }
             }
         }
@@ -610,27 +670,27 @@ void emitQualifiedName(ManglingContext* context, DeclRef<Decl> declRef, bool inc
             // parameter modifier makes big difference in the spirv code generation, for example
             // "out"/"inout" parameter will be passed by pointer. Therefore, we need to
             // distinguish them in the mangled name to avoid name collision.
-            ParameterDirection paramDirection = getParameterDirection(paramDeclRef.getDecl());
+            ParamPassingMode paramDirection = getParamPassingMode(paramDeclRef.getDecl());
             switch (paramDirection)
             {
-            case kParameterDirection_Ref:
+            case ParamPassingMode::Ref:
                 emitRaw(context, "r_");
                 break;
-            case kParameterDirection_ConstRef:
+            case ParamPassingMode::BorrowIn:
                 emitRaw(context, "c_");
                 break;
-            case kParameterDirection_Out:
+            case ParamPassingMode::Out:
                 emitRaw(context, "o_");
                 break;
-            case kParameterDirection_InOut:
+            case ParamPassingMode::BorrowInOut:
                 emitRaw(context, "io_");
                 break;
-            case kParameterDirection_In:
+            case ParamPassingMode::In:
                 emitRaw(context, "i_");
                 break;
             default:
                 StringBuilder errMsg;
-                errMsg << "Unknown parameter direction: " << paramDirection;
+                errMsg << "Unknown parameter direction: " << int(paramDirection);
                 SLANG_ABORT_COMPILATION(errMsg.toString().begin());
                 break;
             }
@@ -693,6 +753,7 @@ void mangleName(ManglingContext* context, DeclRef<Decl> declRef)
     // TODO: catch cases where the declaration should
     // forward to something else? E.g., what if we
     // are asked to mangle the name of a `typedef`?
+    context->targetDecl = declRef.getDecl();
 
     auto decl = declRef.getDecl();
     if (!decl)
@@ -754,6 +815,10 @@ void mangleName(ManglingContext* context, DeclRef<Decl> declRef)
         emitQualifiedName(context, bwdReq->originalRequirementDecl, true);
         return;
     }
+    else if (as<AttributeDecl>(decl))
+    {
+        emitRaw(context, "A");
+    }
     else
     {
         // TODO: handle other cases
