@@ -3,6 +3,7 @@
 
 #include "../core/slang-blob.h"
 #include "slang-ast-support-types.h"
+#include "slang.h"
 
 template<typename T>
 struct Range
@@ -56,21 +57,35 @@ namespace Slang
 static void emitReflectionVarInfoJSON(PrettyWriter& writer, slang::VariableReflection* var);
 static void emitReflectionTypeLayoutJSON(PrettyWriter& writer, slang::TypeLayoutReflection* type);
 static void emitReflectionTypeJSON(PrettyWriter& writer, slang::TypeReflection* type);
+static slang::ShaderReflection* g_inProgramLayout = nullptr;
+
+static void emitReflectionSize(PrettyWriter& writer, size_t size)
+{
+    size == SLANG_UNBOUNDED_SIZE ? writer << "\"unbounded\""
+    : size == SLANG_UNKNOWN_SIZE ? writer << "\"unknown\""
+                                 : writer << (uint64_t)size;
+}
 
 static void emitReflectionVarBindingInfoJSON(
     PrettyWriter& writer,
     SlangParameterCategory category,
     SlangUInt index,
     SlangUInt count,
+    SlangUInt stride,
     SlangUInt space = 0)
 {
     if (category == SLANG_PARAMETER_CATEGORY_UNIFORM)
     {
         writer << "\"kind\": \"uniform\"";
         writer << ", ";
-        writer << "\"offset\": " << index;
+        writer << "\"offset\": ";
+        emitReflectionSize(writer, index);
         writer << ", ";
-        writer << "\"size\": " << count;
+        writer << "\"size\": ";
+        emitReflectionSize(writer, count);
+        writer << ", ";
+        writer << "\"elementStride\": ";
+        emitReflectionSize(writer, stride);
     }
     else
     {
@@ -100,30 +115,24 @@ static void emitReflectionVarBindingInfoJSON(
 
         default:
             writer << "unknown";
-            assert(!"unhandled case");
+            SLANG_ASSERT(!"unhandled case");
             break;
         }
         writer << "\"";
         if (space && category != SLANG_PARAMETER_CATEGORY_REGISTER_SPACE)
         {
             writer << ", ";
-            writer << "\"space\": " << space;
+            writer << "\"space\": ";
+            emitReflectionSize(writer, space);
         }
         writer << ", ";
         writer << "\"index\": ";
-        writer << index;
+        emitReflectionSize(writer, index);
         if (count != 1)
         {
             writer << ", ";
             writer << "\"count\": ";
-            if (count == SLANG_UNBOUNDED_SIZE)
-            {
-                writer << "\"unbounded\"";
-            }
-            else
-            {
-                writer << count;
-            }
+            emitReflectionSize(writer, count);
         }
     }
 }
@@ -159,6 +168,12 @@ static void emitReflectionVarBindingInfoJSON(
         case SLANG_STAGE_COMPUTE:
             stageName = "compute";
             break;
+        case SLANG_STAGE_MESH:
+            stageName = "mesh";
+            break;
+        case SLANG_STAGE_AMPLIFICATION:
+            stageName = "amplification";
+            break;
 
         default:
             break;
@@ -189,6 +204,7 @@ static void emitReflectionVarBindingInfoJSON(
             auto index = var->getOffset(category);
             auto space = var->getBindingSpace(category);
             auto count = typeLayout->getSize(category);
+            auto elementStride = typeLayout->getElementStride(category);
 
             // Query the paramater usage for the specified entry point.
             // Note: both `request` and `entryPointIndex` may be invalid here, but that should just
@@ -208,7 +224,7 @@ static void emitReflectionVarBindingInfoJSON(
 
             writer << "{";
 
-            emitReflectionVarBindingInfoJSON(writer, category, index, count, space);
+            emitReflectionVarBindingInfoJSON(writer, category, index, count, elementStride, space);
 
             if (usedAvailable)
             {
@@ -361,6 +377,26 @@ static void emitUserAttributes(PrettyWriter& writer, slang::FunctionReflection*
     }
 }
 
+static slang::TypeLayoutReflection* maybeChangeTypeLayoutToAgumentBufferTier2(
+    slang::VariableLayoutReflection* varLayout)
+{
+    if (varLayout->getCategoryCount() != 0)
+    {
+        for (unsigned int categoryIdx = 0; categoryIdx < varLayout->getCategoryCount();
+             categoryIdx++)
+        {
+            auto category = varLayout->getCategoryByIndex(categoryIdx);
+            if (category == slang::MetalArgumentBufferElement)
+            {
+                return g_inProgramLayout->getTypeLayout(
+                    varLayout->getTypeLayout()->getType(),
+                    slang::LayoutRules::MetalArgumentBufferTier2);
+            }
+        }
+    }
+    return nullptr;
+}
+
 static void emitReflectionVarLayoutJSON(PrettyWriter& writer, slang::VariableLayoutReflection* var)
 {
     writer << "{\n";
@@ -376,13 +412,21 @@ static void emitReflectionVarLayoutJSON(PrettyWriter& writer, slang::VariableLay
 
     writer.maybeComma();
     writer << "\"type\": ";
-    emitReflectionTypeLayoutJSON(writer, var->getTypeLayout());
+    if (auto newTypeLayout = maybeChangeTypeLayoutToAgumentBufferTier2(var))
+    {
+        emitReflectionTypeLayoutJSON(writer, newTypeLayout);
+    }
+    else
+    {
+        emitReflectionTypeLayoutJSON(writer, var->getTypeLayout());
+    }
 
-    emitReflectionModifierInfoJSON(writer, var->getVariable());
+    if (auto variable = var->getVariable())
+    {
+        emitReflectionModifierInfoJSON(writer, variable);
+    }
 
     emitReflectionVarBindingInfoJSON(writer, var);
-
-    emitUserAttributes(writer, var->getVariable());
     writer.dedent();
     writer << "\n}";
 }
@@ -394,7 +438,7 @@ static void emitReflectionScalarTypeInfoJSON(PrettyWriter& writer, SlangScalarTy
     {
     default:
         writer << "unknown";
-        assert(!"unhandled case");
+        SLANG_ASSERT(!"unhandled case");
         break;
 #define CASE(TAG, ID)                                                          \
     case static_cast<SlangScalarType>(slang::TypeReflection::ScalarType::TAG): \
@@ -434,7 +478,7 @@ static void emitReflectionResourceTypeBaseInfoJSON(
     {
     default:
         writer << "unknown";
-        assert(!"unhandled case");
+        SLANG_ASSERT(!"unhandled case");
         break;
 
 #define CASE(SHAPE, NAME)             \
@@ -467,6 +511,11 @@ static void emitReflectionResourceTypeBaseInfoJSON(
         writer.maybeComma();
         writer << "\"feedback\": true";
     }
+    if (shape & SLANG_TEXTURE_COMBINED_FLAG)
+    {
+        writer.maybeComma();
+        writer << "\"combined\": true";
+    }
 
     if (access != SLANG_RESOURCE_ACCESS_READ)
     {
@@ -476,7 +525,7 @@ static void emitReflectionResourceTypeBaseInfoJSON(
         {
         default:
             writer << "unknown";
-            assert(!"unhandled case");
+            SLANG_ASSERT(!"unhandled case");
             break;
 
         case SLANG_RESOURCE_ACCESS_READ:
@@ -504,7 +553,6 @@ static void emitReflectionResourceTypeBaseInfoJSON(
     }
 }
 
-
 static void emitReflectionTypeInfoJSON(PrettyWriter& writer, slang::TypeReflection* type)
 {
     auto kind = type->getKind();
@@ -589,7 +637,7 @@ static void emitReflectionTypeInfoJSON(PrettyWriter& writer, slang::TypeReflecti
         writer << "\"kind\": \"vector\"";
         writer.maybeComma();
         writer << "\"elementCount\": ";
-        writer << int(type->getElementCount());
+        emitReflectionSize(writer, type->getElementCount());
         writer.maybeComma();
         writer << "\"elementType\": ";
         emitReflectionTypeJSON(writer, type->getElementType());
@@ -616,7 +664,7 @@ static void emitReflectionTypeInfoJSON(PrettyWriter& writer, slang::TypeReflecti
             writer << "\"kind\": \"array\"";
             writer.maybeComma();
             writer << "\"elementCount\": ";
-            writer << int(arrayType->getElementCount());
+            emitReflectionSize(writer, arrayType->getElementCount());
             writer.maybeComma();
             writer << "\"elementType\": ";
             emitReflectionTypeJSON(writer, arrayType->getElementType());
@@ -676,8 +724,22 @@ static void emitReflectionTypeInfoJSON(PrettyWriter& writer, slang::TypeReflecti
         writer.maybeComma();
         writer << "\"kind\": \"DynamicResource\"";
         break;
+    case slang::TypeReflection::Kind::MeshOutput:
+        writer.maybeComma();
+        writer << "\"kind\": \"MeshOutput\"";
+        break;
+    case slang::TypeReflection::Kind::Specialized:
+        writer.maybeComma();
+        writer << "\"kind\": \"Specialized\"";
+        break;
+    case slang::TypeReflection::Kind::None:
+        writer.maybeComma();
+        writer << "\"kind\": \"None\"";
+        break;
     default:
-        assert(!"unhandled case");
+        SLANG_ASSERT(!"Unhandled type kind");
+        writer.maybeComma();
+        writer << "\"kind\": \"Unknown\"";
         break;
     }
     emitUserAttributes(writer, type);
@@ -693,7 +755,18 @@ static void emitReflectionParameterGroupTypeLayoutInfoJSON(
     writer << "\"";
 
     writer << ",\n\"elementType\": ";
-    emitReflectionTypeLayoutJSON(writer, typeLayout->getElementTypeLayout());
+
+    if (auto newElementTypeLayout =
+            maybeChangeTypeLayoutToAgumentBufferTier2(typeLayout->getElementVarLayout()))
+    {
+        // If we are in argument buffer tier 2, we need to use the new type layout
+        // that has the correct binding information.
+        emitReflectionTypeLayoutJSON(writer, newElementTypeLayout);
+    }
+    else
+    {
+        emitReflectionTypeLayoutJSON(writer, typeLayout->getElementTypeLayout());
+    }
 
     // Note: There is a subtle detail below when it comes to the
     // container/element variable layouts that get nested inside
@@ -793,7 +866,7 @@ static void emitReflectionTypeLayoutInfoJSON(
 
             writer.maybeComma();
             writer << "\"elementCount\": ";
-            writer << int(arrayTypeLayout->getElementCount());
+            emitReflectionSize(writer, arrayTypeLayout->getElementCount());
 
             writer.maybeComma();
             writer << "\"elementType\": ";
@@ -803,7 +876,9 @@ static void emitReflectionTypeLayoutInfoJSON(
             {
                 writer.maybeComma();
                 writer << "\"uniformStride\": ";
-                writer << int(arrayTypeLayout->getElementStride(SLANG_PARAMETER_CATEGORY_UNIFORM));
+                emitReflectionSize(
+                    writer,
+                    arrayTypeLayout->getElementStride(SLANG_PARAMETER_CATEGORY_UNIFORM));
             }
         }
         break;
@@ -840,6 +915,10 @@ static void emitReflectionTypeLayoutInfoJSON(
         emitReflectionParameterGroupTypeLayoutInfoJSON(writer, typeLayout, "constantBuffer");
         break;
 
+    case slang::TypeReflection::Kind::OutputStream:
+        emitReflectionParameterGroupTypeLayoutInfoJSON(writer, typeLayout, "outputStream");
+        break;
+
     case slang::TypeReflection::Kind::ParameterBlock:
         emitReflectionParameterGroupTypeLayoutInfoJSON(writer, typeLayout, "parameterBlock");
         break;
@@ -963,7 +1042,10 @@ static void emitReflectionParamJSON(PrettyWriter& writer, slang::VariableLayoutR
         emitReflectionNameInfoJSON(writer, name);
     }
 
-    emitReflectionModifierInfoJSON(writer, param->getVariable());
+    if (auto var = param->getVariable())
+    {
+        emitReflectionModifierInfoJSON(writer, var);
+    }
 
     emitReflectionVarBindingInfoJSON(writer, param);
 
@@ -1060,6 +1142,12 @@ static void emitReflectionEntryPointJSON(
     case SLANG_STAGE_COMPUTE:
         writer << ",\n\"stage\": \"compute\"";
         break;
+    case SLANG_STAGE_MESH:
+        writer << ",\n\"stage\": \"mesh\"";
+        break;
+    case SLANG_STAGE_AMPLIFICATION:
+        writer << ",\n\"stage\": \"amplification\"";
+        break;
     default:
         break;
     }
@@ -1231,6 +1319,7 @@ void emitReflectionJSON(
     PrettyWriter& writer)
 {
     auto programReflection = (slang::ShaderReflection*)reflection;
+    g_inProgramLayout = programReflection;
     emitReflectionJSON(writer, request, programReflection);
 }
 
