@@ -12,25 +12,40 @@
 #include "slang-serialize-ast.h"
 #include "slang-serialize-ir.h"
 #include "slang-serialize-source-loc.h"
+#include "slang-serialize-types.h"
 
 namespace Slang
 {
 struct ModuleEncodingContext
 {
+private:
+    Stream* _stream = nullptr;
+
+    // The string pool used across the whole of the container
+    StringSlicePool _containerStringPool;
+    RefPtr<SerialSourceLocWriter> _sourceLocWriter;
+
+    RIFF::Builder _riff;
+    RIFF::BuildCursor _cursor;
+
 public:
     ModuleEncodingContext(SerialContainerUtil::WriteOptions const& options, Stream* stream)
-        : options(options), encoder(stream), containerStringPool(StringSlicePool::Style::Default)
+        : _stream(stream), _containerStringPool(StringSlicePool::Style::Default)
     {
-        if (options.optionFlags & SerialOptionFlag::SourceLocation)
+        if (options.sourceManagerToUseWhenSerializingSourceLocs)
         {
-            sourceLocWriter = new SerialSourceLocWriter(options.sourceManager);
+            _sourceLocWriter =
+                new SerialSourceLocWriter(options.sourceManagerToUseWhenSerializingSourceLocs);
         }
+
+        _cursor = RIFF::BuildCursor(_riff);
     }
 
     ~ModuleEncodingContext()
     {
-        encoder.setRIFFChunk(encoder.getRIFF()->getRoot());
+        _cursor = RIFF::BuildCursor(_riff.getRootChunk());
         encodeFinalPieces();
+        _riff.writeTo(_stream);
     }
 
     SlangResult encodeModuleList(FrontEndCompileRequest* frontEndReq)
@@ -39,7 +54,7 @@ public:
         // is simply a matter of encoding the module for each
         // of the translation units that got compiled.
         //
-        Encoder::WithKeyValuePair withArray(&encoder, SerialBinary::kModuleListFourCc);
+        SLANG_SCOPED_RIFF_BUILDER_LIST_CHUNK(_cursor, SerialBinary::kModuleListFourCc);
         for (TranslationUnitRequest* translationUnit : frontEndReq->translationUnits)
         {
             SLANG_RETURN_ON_FAIL(encode(translationUnit->module));
@@ -49,14 +64,14 @@ public:
 
     SlangResult encode(FrontEndCompileRequest* frontEndReq)
     {
-        Encoder::WithObject withObject(&encoder, SerialBinary::kContainerFourCc);
+        SLANG_SCOPED_RIFF_BUILDER_LIST_CHUNK(_cursor, SerialBinary::kContainerFourCc);
         SLANG_RETURN_ON_FAIL(encodeModuleList(frontEndReq));
         return SLANG_OK;
     }
 
     SlangResult encode(EndToEndCompileRequest* request)
     {
-        Encoder::WithObject withObject(&encoder, SerialBinary::kContainerFourCc);
+        SLANG_SCOPED_RIFF_BUILDER_LIST_CHUNK(_cursor, SerialBinary::kContainerFourCc);
 
         // Encoding an end-to-end compile request starts with the same
         // work as for a front-end request: we encode each of
@@ -85,7 +100,7 @@ public:
         auto sink = request->getSink();
         auto program = request->getSpecializedGlobalAndEntryPointsComponentType();
         {
-            Encoder::WithArray withArray(&encoder); // kContainerFourCc
+            SLANG_SCOPED_RIFF_BUILDER_LIST_CHUNK(_cursor, SerialBinary::kArrayFourCC);
 
             for (auto target : linkage->targets)
             {
@@ -98,7 +113,7 @@ public:
         // and we need to encode information about each of them.
         //
         {
-            Encoder::WithArray withArray(&encoder, SerialBinary::kEntryPointListFourCc);
+            SLANG_SCOPED_RIFF_BUILDER_LIST_CHUNK(_cursor, SerialBinary::kEntryPointListFourCc);
 
             auto entryPointCount = program->getEntryPointCount();
             for (Index ii = 0; ii < entryPointCount; ++ii)
@@ -123,39 +138,44 @@ public:
         IRModule* irModule = targetProgram->getOrCreateIRModuleForLayout(sink);
 
         // Okay, we need to serialize this target program and its IR too...
-        IRSerialData serialData;
-        IRSerialWriter writer;
-
-        SLANG_RETURN_ON_FAIL(
-            writer.write(irModule, sourceLocWriter, options.optionFlags, &serialData));
-        SLANG_RETURN_ON_FAIL(IRSerialWriter::writeContainer(serialData, encoder.getRIFF()));
+        {
+            SLANG_SCOPED_RIFF_BUILDER_LIST_CHUNK(_cursor, PropertyKeys<IRModule>::IRModule);
+            writeSerializedModuleIR(_cursor, irModule, _sourceLocWriter);
+        }
 
         return SLANG_OK;
     }
 
-    void encode(Name* name) { encoder.encode(name->text); }
+    void encodeData(void const* data, size_t size, FourCC type)
+    {
+        _cursor.addDataChunk(type, data, size);
+    }
 
-    void encode(String const& value) { encoder.encode(value); }
+    void encode(String const& value, FourCC type = SerialBinary::kStringFourCC)
+    {
+        encodeData(value.getBuffer(), value.getLength(), type);
+    }
 
-    void encode(uint32_t value) { encoder.encode(UInt(value)); }
+    void encode(Name* name, FourCC type = SerialBinary::kNameFourCC) { encode(name->text, type); }
 
-    void encodeData(void const* data, size_t size) { encoder.encodeData(data, size); }
+
+    void encode(uint32_t value, FourCC type = SerialBinary::kUInt32FourCC)
+    {
+        encodeData(&value, sizeof(value), type);
+    }
 
     SlangResult encode(EntryPoint* entryPoint, String const& entryPointMangledName)
     {
-        Encoder::WithObject withObject(&encoder, SerialBinary::kEntryPointFourCc);
+        SLANG_SCOPED_RIFF_BUILDER_LIST_CHUNK(_cursor, SerialBinary::kEntryPointFourCc);
 
         {
-            Encoder::WithObject withProperty(&encoder, SerialBinary::kNameFourCC);
-            encode(entryPoint->getName());
+            encode(entryPoint->getName(), SerialBinary::kNameFourCC);
         }
         {
-            Encoder::WithObject withProperty(&encoder, SerialBinary::kProfileFourCC);
-            encode(entryPoint->getProfile().raw);
+            encode(entryPoint->getProfile().raw, SerialBinary::kProfileFourCC);
         }
         {
-            Encoder::WithObject withProperty(&encoder, SerialBinary::kMangledNameFourCC);
-            encode(entryPointMangledName);
+            encode(entryPointMangledName, SerialBinary::kMangledNameFourCC);
         }
 
         return SLANG_OK;
@@ -164,10 +184,7 @@ public:
 
     SlangResult encode(Module* module)
     {
-        if (!(options.optionFlags & (SerialOptionFlag::IRModule | SerialOptionFlag::ASTModule)))
-            return SLANG_OK;
-
-        Encoder::WithObject withModule(&encoder, SerialBinary::kModuleFourCC);
+        SLANG_SCOPED_RIFF_BUILDER_LIST_CHUNK(_cursor, SerialBinary::kModuleFourCC);
 
         // The first piece that we write for a module is its header.
         // The header is intended to provide information that can be
@@ -180,15 +197,14 @@ public:
             // sense to serialize it separately from all the rest.
             //
             {
-                Encoder::WithObject withProperty(&encoder, SerialBinary::kNameFourCC);
-                encoder.encodeString(module->getNameObj()->text);
+                encode(module->getNameObj(), SerialBinary::kNameFourCC);
             }
 
             // The header includes a digest of all the compile options and
             // the files that the compiled result depended on.
             //
             auto digest = module->computeDigest();
-            encoder.encodeData(PropertyKeys<Module>::Digest, digest.data, sizeof(digest.data));
+            _cursor.addDataChunk(PropertyKeys<Module>::Digest, digest.data, sizeof(digest.data));
 
             // The header includes an array of the paths of all of the
             // files that the compiled result depended on.
@@ -196,34 +212,20 @@ public:
             encodeModuleDependencyPaths(module);
         }
 
-        // If serialization of Slang IR modules is enabled, and there
-        // is IR available for this module, then we we encode it.
+        // If there is IR available for this module, then we we encode it.
         //
-        if ((options.optionFlags & SerialOptionFlag::IRModule))
+        if (auto irModule = module->getIRModule())
         {
-            if (auto irModule = module->getIRModule())
-            {
-                Encoder::WithKeyValuePair withKey(&encoder, PropertyKeys<Module>::IRModule);
-
-                IRSerialData serialData;
-                IRSerialWriter writer;
-                SLANG_RETURN_ON_FAIL(
-                    writer.write(irModule, sourceLocWriter, options.optionFlags, &serialData));
-                SLANG_RETURN_ON_FAIL(IRSerialWriter::writeContainer(serialData, encoder.getRIFF()));
-            }
+            SLANG_SCOPED_RIFF_BUILDER_LIST_CHUNK(_cursor, PropertyKeys<IRModule>::IRModule);
+            writeSerializedModuleIR(_cursor, irModule, _sourceLocWriter);
         }
 
-        // If serialization of AST information is enabled, and we have AST
-        // information available, then we serialize it here.
+        // If we have AST information available, then we serialize it here.
         //
-        if (options.optionFlags & SerialOptionFlag::ASTModule)
+        if (auto moduleDecl = module->getModuleDecl())
         {
-            if (auto moduleDecl = module->getModuleDecl())
-            {
-                Encoder::WithKeyValuePair withKey(&encoder, PropertyKeys<Module>::ASTModule);
-
-                writeSerializedModuleAST(&encoder, moduleDecl, sourceLocWriter);
-            }
+            SLANG_SCOPED_RIFF_BUILDER_LIST_CHUNK(_cursor, PropertyKeys<Module>::ASTModule);
+            writeSerializedModuleAST(_cursor, moduleDecl, _sourceLocWriter);
         }
 
         return SLANG_OK;
@@ -231,7 +233,7 @@ public:
 
     SlangResult encodeModuleDependencyPaths(Module* module)
     {
-        Encoder::WithObject withProperty(&encoder, PropertyKeys<Module>::FileDependencies);
+        SLANG_SCOPED_RIFF_BUILDER_LIST_CHUNK(_cursor, PropertyKeys<Module>::FileDependencies);
 
         // TODO(tfoley): This is some of the most complicated logic
         // in the encoding system, because it tries to translate
@@ -305,7 +307,6 @@ public:
         }
         Path::getCanonical(linkageRoot, linkageRoot);
 
-        Encoder::WithArray withArray(&encoder);
         for (auto file : fileDependencies)
         {
             if (file->getPathInfo().hasFoundPath())
@@ -322,26 +323,26 @@ public:
                         auto relativeModulePath =
                             Path::getRelativePath(linkageRoot, canonicalModulePath);
 
-                        encoder.encodeString(relativeModulePath);
+                        encode(relativeModulePath);
                     }
                     else
                     {
                         // For all other dependnet files, store them as relative paths with respect
                         // to the module's path.
                         canonicalFilePath = Path::getRelativePath(moduleDir, canonicalFilePath);
-                        encoder.encodeString(canonicalFilePath);
+                        encode(canonicalFilePath);
                     }
                 }
                 else
                 {
                     // If the module is coming from string instead of an actual file, store it as
                     // is.
-                    encoder.encodeString(canonicalModulePath);
+                    encode(canonicalModulePath);
                 }
             }
             else
             {
-                encoder.encodeString(file->getPathInfo().getMostUniqueIdentity());
+                encode(file->getPathInfo().getMostUniqueIdentity());
             }
         }
 
@@ -351,38 +352,29 @@ public:
     SlangResult encodeFinalPieces()
     {
         // We can now output the debug information. This is for all IR and AST
-        if (sourceLocWriter)
+        if (_sourceLocWriter)
         {
             // Write out the debug info
             SerialSourceLocData debugData;
-            sourceLocWriter->write(&debugData);
+            _sourceLocWriter->write(&debugData);
 
-            debugData.writeContainer(encoder.getRIFF());
+            debugData.writeTo(_cursor);
         }
 
         // Write the container string table
-        if (containerStringPool.getAdded().getCount() > 0)
+        if (_containerStringPool.getAdded().getCount() > 0)
         {
-            Encoder::WithKeyValuePair withKey(&encoder, SerialBinary::kStringTableFourCc);
-
             List<char> encodedTable;
-            SerialStringTableUtil::encodeStringTable(containerStringPool, encodedTable);
+            SerialStringTableUtil::encodeStringTable(_containerStringPool, encodedTable);
 
-            encoder.encodeData(encodedTable.getBuffer(), encodedTable.getCount());
+            _cursor.addDataChunk(
+                SerialBinary::kStringTableFourCc,
+                encodedTable.getBuffer(),
+                encodedTable.getCount());
         }
 
         return SLANG_OK;
     }
-
-
-private:
-    SerialContainerUtil::WriteOptions const& options;
-    RefPtr<SerialSourceLocWriter> sourceLocWriter;
-
-    // The string pool used across the whole of the container
-    StringSlicePool containerStringPool;
-
-    Encoder encoder;
 };
 
 //
@@ -421,154 +413,137 @@ private:
     return SLANG_OK;
 }
 
-String StringChunkRef::getValue()
+String StringChunk::getValue() const
 {
-    return Decoder(ptr()).decodeString();
+    return String(UnownedStringSlice((char const*)getPayload(), getPayloadSize()));
 }
 
-ChunkRefList<StringChunkRef> ModuleChunkRef::getFileDependencies()
+RIFF::ChunkList<StringChunk> ModuleChunk::getFileDependencies() const
 {
-    Decoder decoder(ptr());
-    Decoder::WithProperty withProperty(decoder, PropertyKeys<Module>::FileDependencies);
-    return ChunkRefList<StringChunkRef>(as<RiffContainer::ListChunk>(decoder.getCursor()));
+    auto found = findListChunk(PropertyKeys<Module>::FileDependencies);
+    if (!found)
+        return RIFF::ChunkList<StringChunk>();
+    return found->getChildren().cast<StringChunk>();
 }
 
-ModuleChunkRef ModuleChunkRef::find(RiffContainer* container)
+ModuleChunk const* ModuleChunk::find(RIFF::ListChunk const* baseChunk)
 {
-    auto found = container->getRoot()->findListRec(SerialBinary::kModuleFourCC);
-    return ModuleChunkRef(found);
+    auto found = baseChunk->findListChunkRec(SerialBinary::kModuleFourCC);
+    return static_cast<ModuleChunk const*>(found);
 }
 
-SHA1::Digest ModuleChunkRef::getDigest()
+SHA1::Digest ModuleChunk::getDigest() const
 {
-    auto foundChunk =
-        static_cast<RiffContainer::DataChunk*>(ptr()->findContained(PropertyKeys<Module>::Digest));
+    auto foundChunk = findDataChunk(PropertyKeys<Module>::Digest);
     if (!foundChunk)
     {
         SLANG_UNEXPECTED("module chunk had no digest");
     }
-    if (foundChunk->calcPayloadSize() != sizeof(SHA1::Digest))
+    return foundChunk->readPayloadAs<SHA1::Digest>();
+}
+
+String ModuleChunk::getName() const
+{
+    auto found = findDataChunk(SerialBinary::kNameFourCC);
+    if (!found)
     {
-        SLANG_UNEXPECTED("module digest chunk had wrong size");
+        SLANG_UNEXPECTED("module chunk had no name");
     }
-
-    SHA1::Digest digest;
-    foundChunk->getPayload(&digest);
-    return digest;
-}
-
-String ModuleChunkRef::getName()
-{
-    // TODO(tfoley): This kind of logic needs a way
-    // to be greatly simplified, so that we don't
-    // have to express such complicated logic for
-    // simply extracting a single string property...
-    //
-    Decoder decoder(ptr());
-    Decoder::WithProperty withProperty(decoder, SerialBinary::kNameFourCC);
-    return decoder.decodeString();
+    return static_cast<StringChunk const*>(found)->getValue();
 }
 
 
-IRModuleChunkRef ModuleChunkRef::findIR()
+IRModuleChunk const* ModuleChunk::findIR() const
 {
-    auto foundProperty = ptr()->findContainedList(PropertyKeys<Module>::IRModule);
+    auto foundProperty = findListChunk(PropertyKeys<IRModule>::IRModule);
     if (!foundProperty)
-        return IRModuleChunkRef(nullptr);
-    return IRModuleChunkRef(
-        static_cast<RiffContainer::ListChunk*>(foundProperty->getFirstContainedChunk()));
+        return nullptr;
+
+    return static_cast<IRModuleChunk const*>(foundProperty->getFirstChild().get());
 }
 
-ASTModuleChunkRef ModuleChunkRef::findAST()
+ASTModuleChunk const* ModuleChunk::findAST() const
 {
-    auto foundProperty = ptr()->findContainedList(PropertyKeys<Module>::ASTModule);
+    auto foundProperty = findListChunk(PropertyKeys<Module>::ASTModule);
     if (!foundProperty)
-        return ASTModuleChunkRef(nullptr);
-    return ASTModuleChunkRef(
-        static_cast<RiffContainer::ListChunk*>(foundProperty->getFirstContainedChunk()));
+        return nullptr;
+
+    return static_cast<ASTModuleChunk const*>(foundProperty->getFirstChild().get());
 }
 
-ContainerChunkRef ContainerChunkRef::find(RiffContainer* container)
+ContainerChunk const* ContainerChunk::find(RIFF::ListChunk const* baseChunk)
 {
-    auto found = container->getRoot()->findListRec(SerialBinary::kContainerFourCc);
-    return ContainerChunkRef(found);
+    auto found = baseChunk->findListChunkRec(SerialBinary::kContainerFourCc);
+    return static_cast<ContainerChunk const*>(found);
 }
 
-ChunkRefList<ModuleChunkRef> ContainerChunkRef::getModules()
+RIFF::ChunkList<ModuleChunk> ContainerChunk::getModules() const
 {
-    auto found = ptr()->findContainedList(SerialBinary::kModuleListFourCc);
-    return ChunkRefList<ModuleChunkRef>(found);
+    auto found = findListChunk(SerialBinary::kModuleListFourCc);
+    if (!found)
+        return RIFF::ChunkList<ModuleChunk>();
+    return found->getChildren().cast<ModuleChunk>();
 }
 
-ChunkRefList<EntryPointChunkRef> ContainerChunkRef::getEntryPoints()
+RIFF::ChunkList<EntryPointChunk> ContainerChunk::getEntryPoints() const
 {
-    auto found = ptr()->findContainedList(SerialBinary::kEntryPointListFourCc);
-    return ChunkRefList<EntryPointChunkRef>(found);
+    auto found = findListChunk(SerialBinary::kEntryPointListFourCc);
+    if (!found)
+        return RIFF::ChunkList<EntryPointChunk>();
+    return found->getChildren().cast<EntryPointChunk>();
 }
 
-String EntryPointChunkRef::getMangledName() const
+String EntryPointChunk::getMangledName() const
 {
-    // TODO(tfoley): This kind of logic needs a way
-    // to be greatly simplified, so that we don't
-    // have to express such complicated logic for
-    // simply extracting a single string property...
-    //
-    Decoder decoder(ptr());
-    Decoder::WithProperty withProperty(decoder, SerialBinary::kMangledNameFourCC);
-    return decoder.decodeString();
+    auto found = findDataChunk(SerialBinary::kMangledNameFourCC);
+    if (!found)
+    {
+        SLANG_UNEXPECTED("entry point chunk had no mangled name");
+    }
+    return static_cast<StringChunk const*>(found)->getValue();
 }
 
-String EntryPointChunkRef::getName() const
+String EntryPointChunk::getName() const
 {
-    // TODO(tfoley): This kind of logic needs a way
-    // to be greatly simplified, so that we don't
-    // have to express such complicated logic for
-    // simply extracting a single string property...
-    //
-    Decoder decoder(ptr());
-    Decoder::WithProperty withProperty(decoder, SerialBinary::kNameFourCC);
-    return decoder.decodeString();
+    auto found = findDataChunk(SerialBinary::kNameFourCC);
+    if (!found)
+    {
+        SLANG_UNEXPECTED("entry point chunk had no name");
+    }
+    return static_cast<StringChunk const*>(found)->getValue();
 }
 
-Profile EntryPointChunkRef::getProfile() const
+Profile EntryPointChunk::getProfile() const
 {
-    // TODO(tfoley): This kind of logic needs a way
-    // to be greatly simplified, so that we don't
-    // have to express such complicated logic for
-    // simply extracting a single string property...
-    //
-    Decoder decoder(ptr());
-    Decoder::WithProperty withProperty(decoder, SerialBinary::kProfileFourCC);
-
-    Profile::RawVal rawVal;
-    decoder.decode(rawVal);
-
+    auto found = findDataChunk(SerialBinary::kProfileFourCC);
+    if (!found)
+    {
+        SLANG_UNEXPECTED("entry point chunk had no profile");
+    }
+    auto rawVal = found->readPayloadAs<Profile::RawVal>();
     return Profile(rawVal);
 }
 
 
-RiffContainer::ListChunk* findDebugChunk(RiffContainer::Chunk* startingChunk)
+DebugChunk const* DebugChunk::find(RIFF::ListChunk const* baseChunk)
 {
-    if (!startingChunk)
-        return nullptr;
-
-    RiffContainer::ListChunk* container = as<RiffContainer::ListChunk>(startingChunk);
-    if (!container)
-        container = startingChunk->m_parent;
-
-    for (; container; container = container->m_parent)
-    {
-        if (auto debugChunk = container->findContainedList(SerialSourceLocData::kDebugFourCc))
-        {
-            return debugChunk;
-        }
-    }
+    auto found = baseChunk->findListChunkRec(SerialSourceLocData::kDebugFourCc);
+    return static_cast<DebugChunk const*>(found);
+}
 
+DebugChunk const* DebugChunk::find(
+    RIFF::ListChunk const* baseChunk,
+    RIFF::ListChunk const* containerChunk)
+{
+    if (auto found = find(baseChunk))
+        return found;
+    if (containerChunk)
+        return find(containerChunk);
     return nullptr;
 }
 
 SlangResult readSourceLocationsFromDebugChunk(
-    RiffContainer::ListChunk* debugChunk,
+    DebugChunk const* debugChunk,
     SourceManager* sourceManager,
     RefPtr<SerialSourceLocReader>& outReader)
 {
@@ -585,7 +560,7 @@ SlangResult readSourceLocationsFromDebugChunk(
     // from the RIFF into the intermediate structure.
     //
     SerialSourceLocData intermediateData;
-    SLANG_RETURN_ON_FAIL(intermediateData.readContainer(debugChunk));
+    SLANG_RETURN_ON_FAIL(intermediateData.readFrom(debugChunk));
 
     // After reading the data into the intermediate representation,
     // we turn it into a `SerialSourceLocReader`, which vends source
@@ -599,38 +574,15 @@ SlangResult readSourceLocationsFromDebugChunk(
     return SLANG_OK;
 }
 
-SlangResult decodeModuleIR(
-    RefPtr<IRModule>& outIRModule,
-    RiffContainer::Chunk* chunk,
-    Session* session,
-    SerialSourceLocReader* sourceLocReader)
+static void calcModuleInstructionList(IRModule* module, List<IRInst*>& instsOut)
 {
-    // IR serialization still uses the older approach, where
-    // data gets deserialized from the RIFF into an intermediate
-    // data structure (`IRSerialData`), and then the actual
-    // in-memory structures are created based on the intermediate.
-    //
-    // Thus we start by running the `IRSerialReader::readContainer`
-    // logic to get the `IRSerialData` representation.
-    //
-    // TODO(tfoley): This should all get streamlined so that we
-    // are deserializing IR nodes directly from the format written
-    // into the RIFF.
-    //
-    auto listChunk = as<RiffContainer::ListChunk>(chunk);
-    if (!listChunk)
-        return SLANG_FAIL;
-    IRSerialData serialData;
-    SLANG_RETURN_ON_FAIL(IRSerialReader::readContainer(listChunk, &serialData));
+    // We reserve 0 for null
+    instsOut.setCount(1);
+    instsOut[0] = nullptr;
 
-    // Next we read the actual IR representation out from the
-    // `serialData`. This is the step that may pull source-location
-    // information from the provided `sourceLocReader`.
-    //
-    IRSerialReader reader;
-    SLANG_RETURN_ON_FAIL(reader.read(serialData, session, sourceLocReader, outIRModule));
+    IRModuleInst* moduleInst = module->getModuleInst();
 
-    return SLANG_OK;
+    traverseInstsInSerializationOrder(moduleInst, [&](IRInst* inst) { instsOut.add(inst); });
 }
 
 /* static */ SlangResult SerialContainerUtil::verifyIRSerialize(
@@ -641,35 +593,28 @@ SlangResult decodeModuleIR(
     // Verify if we can stream out with raw source locs
 
     List<IRInst*> originalInsts;
-    IRSerialWriter::calcInstructionList(module, originalInsts);
-
-    IRSerialData irData;
+    calcModuleInstructionList(module, originalInsts);
 
     OwnedMemoryStream memoryStream(FileAccess::ReadWrite);
 
     {
-        RiffContainer riffContainer;
-
-        // Need to put all of this in a container
-        RiffContainer::ScopeChunk containerScope(
-            &riffContainer,
-            RiffContainer::Chunk::Kind::List,
-            SerialBinary::kContainerFourCc);
+        RIFF::Builder builder;
+        RIFF::BuildCursor cursor(builder);
 
+        // Need to put all of this in a module chunk
+        SLANG_SCOPED_RIFF_BUILDER_LIST_CHUNK(cursor, SerialBinary::kModuleFourCC);
         RefPtr<SerialSourceLocWriter> sourceLocWriter;
 
-        if (options.optionFlags & SerialOptionFlag::SourceLocation)
+        if (options.sourceManagerToUseWhenSerializingSourceLocs)
         {
-            sourceLocWriter = new SerialSourceLocWriter(options.sourceManager);
+            sourceLocWriter =
+                new SerialSourceLocWriter(options.sourceManagerToUseWhenSerializingSourceLocs);
         }
 
         {
-            // Write IR out to serialData - copying over SourceLoc information directly
-            IRSerialWriter writer;
-            SLANG_RETURN_ON_FAIL(
-                writer.write(module, sourceLocWriter, options.optionFlags, &irData));
+            SLANG_SCOPED_RIFF_BUILDER_LIST_CHUNK(cursor, PropertyKeys<IRModule>::IRModule);
+            writeSerializedModuleIR(cursor, module, sourceLocWriter);
         }
-        SLANG_RETURN_ON_FAIL(IRSerialWriter::writeContainer(irData, &riffContainer));
 
         // Write the debug info Riff container
         if (sourceLocWriter)
@@ -677,71 +622,67 @@ SlangResult decodeModuleIR(
             SerialSourceLocData serialData;
             sourceLocWriter->write(&serialData);
 
-            SLANG_RETURN_ON_FAIL(serialData.writeContainer(&riffContainer));
+            SLANG_RETURN_ON_FAIL(serialData.writeTo(cursor));
         }
 
-        SLANG_RETURN_ON_FAIL(RiffUtil::write(&riffContainer, &memoryStream));
+        SLANG_RETURN_ON_FAIL(builder.writeTo(&memoryStream));
     }
 
     // Reset stream
     memoryStream.seek(SeekOrigin::Start, 0);
 
     SourceManager workSourceManager;
-    workSourceManager.initialize(options.sourceManager, nullptr);
+    workSourceManager.initialize(options.sourceManagerToUseWhenSerializingSourceLocs, nullptr);
 
     // The read ir module
     RefPtr<IRModule> irReadModule;
     {
-        RiffContainer riffContainer;
-        SLANG_RETURN_ON_FAIL(RiffUtil::read(&memoryStream, riffContainer));
+        auto streamContents = memoryStream.getContents();
 
-        RiffContainer::ListChunk* rootList = riffContainer.getRoot();
+        auto rootChunk =
+            RIFF::RootChunk::getFromBlob(streamContents.getBuffer(), streamContents.getCount());
+        if (!rootChunk)
+        {
+            return SLANG_FAIL;
+        }
+
+        auto moduleChunk = ModuleChunk::find(rootChunk);
+        if (!moduleChunk)
+        {
+            return SLANG_FAIL;
+        }
 
         RefPtr<SerialSourceLocReader> sourceLocReader;
 
         // If we have debug info then find and read it
-        if (options.optionFlags & SerialOptionFlag::SourceLocation)
+        if (options.sourceManagerToUseWhenSerializingSourceLocs)
         {
-            RiffContainer::ListChunk* debugList =
-                rootList->findContainedList(SerialSourceLocData::kDebugFourCc);
-            if (!debugList)
+            auto debugChunk = DebugChunk::find(moduleChunk);
+            if (!debugChunk)
             {
                 return SLANG_FAIL;
             }
             SerialSourceLocData sourceLocData;
-            SLANG_RETURN_ON_FAIL(sourceLocData.readContainer(debugList));
+            SLANG_RETURN_ON_FAIL(sourceLocData.readFrom(debugChunk));
 
             sourceLocReader = new SerialSourceLocReader;
             SLANG_RETURN_ON_FAIL(sourceLocReader->read(&sourceLocData, &workSourceManager));
         }
 
         {
-            RiffContainer::ListChunk* irList =
-                rootList->findContainedList(IRSerialBinary::kIRModuleFourCc);
-            if (!irList)
+            auto irChunk = moduleChunk->findIR();
+            if (!irChunk)
             {
                 return SLANG_FAIL;
             }
 
-            {
-                IRSerialData irReadData;
-                IRSerialReader reader;
-                SLANG_RETURN_ON_FAIL(reader.readContainer(irList, &irReadData));
-
-                // Check the stream read data is the same
-                if (irData != irReadData)
-                {
-                    SLANG_ASSERT(!"Streamed in data doesn't match");
-                    return SLANG_FAIL;
-                }
-
-                SLANG_RETURN_ON_FAIL(reader.read(irData, session, sourceLocReader, irReadModule));
-            }
+            SLANG_RETURN_ON_FAIL(
+                readSerializedModuleIR(irChunk, session, sourceLocReader, irReadModule));
         }
     }
 
     List<IRInst*> readInsts;
-    IRSerialWriter::calcInstructionList(irReadModule, readInsts);
+    calcModuleInstructionList(irReadModule, readInsts);
 
     if (readInsts.getCount() != originalInsts.getCount())
     {
@@ -749,23 +690,7 @@ SlangResult decodeModuleIR(
         return SLANG_FAIL;
     }
 
-    if (options.optionFlags & SerialOptionFlag::RawSourceLocation)
-    {
-        SLANG_ASSERT(readInsts[0] == originalInsts[0]);
-        // All the source locs should be identical
-        for (Index i = 1; i < readInsts.getCount(); ++i)
-        {
-            IRInst* origInst = originalInsts[i];
-            IRInst* readInst = readInsts[i];
-
-            if (origInst->sourceLoc.getRaw() != readInst->sourceLoc.getRaw())
-            {
-                SLANG_ASSERT(!"Source locs don't match");
-                return SLANG_FAIL;
-            }
-        }
-    }
-    else if (options.optionFlags & SerialOptionFlag::SourceLocation)
+    if (options.sourceManagerToUseWhenSerializingSourceLocs)
     {
         // They should be on the same line nos
         for (Index i = 1; i < readInsts.getCount(); ++i)
@@ -779,7 +704,9 @@ SlangResult decodeModuleIR(
             }
 
             // Work out the
-            SourceView* origSourceView = options.sourceManager->findSourceView(origInst->sourceLoc);
+            SourceView* origSourceView =
+                options.sourceManagerToUseWhenSerializingSourceLocs->findSourceView(
+                    origInst->sourceLoc);
             SourceView* readSourceView = workSourceManager.findSourceView(readInst->sourceLoc);
 
             // if both are null we are done
