@@ -912,30 +912,23 @@ FuncType* getFuncType(ASTBuilder* astBuilder, DeclRef<CallableDecl> const& declR
 {
     List<Type*> paramTypes;
     auto resultType = getResultType(astBuilder, declRef);
+
+    if (!resultType)
+        resultType = astBuilder->getErrorType();
+
     auto errorType = getErrorCodeType(astBuilder, declRef);
     auto visitParamDecl = [&](DeclRef<ParamDecl> paramDeclRef)
     {
+        auto paramValueType = getParamValueType(astBuilder, paramDeclRef);
+        if (!paramValueType)
+        {
+            paramValueType = astBuilder->getErrorType();
+        }
+
         auto paramDecl = paramDeclRef.getDecl();
-        auto paramType = getParamType(astBuilder, paramDeclRef);
-        if (paramDecl->findModifier<RefModifier>())
-        {
-            paramType = astBuilder->getRefType(paramType, AddressSpace::Generic);
-        }
-        else if (paramDecl->findModifier<ConstRefModifier>())
-        {
-            paramType = astBuilder->getConstRefType(paramType);
-        }
-        else if (paramDecl->findModifier<OutModifier>())
-        {
-            if (paramDecl->findModifier<InOutModifier>() || paramDecl->findModifier<InModifier>())
-            {
-                paramType = astBuilder->getInOutType(paramType);
-            }
-            else
-            {
-                paramType = astBuilder->getOutType(paramType);
-            }
-        }
+        auto paramMode = getParamPassingMode(paramDecl);
+        auto paramType = getParamTypeWithModeWrapper(astBuilder, paramValueType, paramMode);
+
         paramTypes.add(paramType);
     };
     auto parent = declRef.getParent();
@@ -951,8 +944,7 @@ FuncType* getFuncType(ASTBuilder* astBuilder, DeclRef<CallableDecl> const& declR
         visitParamDecl(paramDeclRef);
     }
 
-    FuncType* funcType =
-        astBuilder->getOrCreate<FuncType>(paramTypes.getArrayView(), resultType, errorType);
+    FuncType* funcType = astBuilder->getFuncType(paramTypes.getArrayView(), resultType, errorType);
     return funcType;
 }
 
@@ -1093,44 +1085,44 @@ ModuleDecl* getModuleDecl(Scope* scope)
     return nullptr;
 }
 
-Decl* getParentDecl(Decl* decl)
+ContainerDecl* getParentDecl(Decl* decl)
 {
-    decl = decl->parentDecl;
-    while (as<GenericDecl>(decl))
-        decl = decl->parentDecl;
-    return decl;
+    auto parentDecl = decl->parentDecl;
+    while (auto genericDecl = as<GenericDecl>(parentDecl))
+        parentDecl = genericDecl->parentDecl;
+    return parentDecl;
 }
 
-Decl* getParentAggTypeDecl(Decl* decl)
+AggTypeDecl* getParentAggTypeDecl(Decl* decl)
 {
     decl = decl->parentDecl;
     while (decl)
     {
-        if (as<AggTypeDecl>(decl))
-            return decl;
+        if (auto found = as<AggTypeDecl>(decl))
+            return found;
         decl = decl->parentDecl;
     }
     return nullptr;
 }
 
-Decl* getParentAggTypeDeclBase(Decl* decl)
+AggTypeDeclBase* getParentAggTypeDeclBase(Decl* decl)
 {
     decl = decl->parentDecl;
     while (decl)
     {
-        if (as<AggTypeDeclBase>(decl))
-            return decl;
+        if (auto found = as<AggTypeDeclBase>(decl))
+            return found;
         decl = decl->parentDecl;
     }
     return nullptr;
 }
 
-Decl* getParentFunc(Decl* decl)
+FunctionDeclBase* getParentFunc(Decl* decl)
 {
     while (decl)
     {
-        if (as<FunctionDeclBase>(decl))
-            return decl;
+        if (auto found = as<FunctionDeclBase>(decl))
+            return found;
         decl = decl->parentDecl;
     }
     return nullptr;
@@ -1185,8 +1177,15 @@ bool findVkImageFormatByName(const UnownedStringSlice& name, ImageFormat* outFor
     if (name.endsWith(kSNorm))
     {
         StringBuilder buf;
-        //  format names end with snormal after a '_', so replace with that
-        buf << name.head(name.getLength() - kSNorm.getLength()) << "_" << kSNorm;
+        auto prefix = name.head(name.getLength() - kSNorm.getLength());
+        buf << prefix;
+        // format names end with snormal after a '_', so add an underscore
+        // if the prefix doesn't already end with one
+        if (prefix.getLength() == 0 || prefix[prefix.getLength() - 1] != '_')
+        {
+            buf << "_";
+        }
+        buf << kSNorm;
         return findImageFormatByName(buf.getUnownedSlice(), outFormat);
     }
 
