@@ -149,7 +149,7 @@ SLANG_NO_THROW SlangResult SLANG_MCALL DeviceImpl::initialize(const Desc& desc)
         desc.extendedDescCount,
         desc.extendedDescs,
         SLANG_PTX,
-        "sm_5_1",
+        "cuda_sm_5_0",
         makeArray(slang::PreprocessorMacroDesc{"__CUDA_COMPUTE__", "1"}).getView()));
 
     SLANG_RETURN_ON_FAIL(RendererBase::initialize(desc));
@@ -180,8 +180,9 @@ SLANG_NO_THROW SlangResult SLANG_MCALL DeviceImpl::initialize(const Desc& desc)
 
     SLANG_CUDA_RETURN_ON_FAIL(cuDeviceGet(&m_device, m_deviceIndex));
 
+    // Use version-aware context creation that works with both CUDA 12 and CUDA 13
     SLANG_CUDA_RETURN_WITH_REPORT_ON_FAIL(
-        cuCtxCreate(&m_context->m_context, 0, m_device),
+        createCudaContext(&m_context->m_context, 0, m_device),
         reportType);
 
     {
