@@ -214,7 +214,8 @@ Result DeviceImpl::initVulkanInstanceAndDevice(
 #endif
         }
 
-        if (ENABLE_VALIDATION_LAYER || isGfxDebugLayerEnabled())
+        gfxEnableDebugLayer(useValidationLayer);
+        if (isGfxDebugLayerEnabled())
             instanceExtensions.add(VK_EXT_DEBUG_REPORT_EXTENSION_NAME);
 
         VkInstanceCreateInfo instanceCreateInfo = {VK_STRUCTURE_TYPE_INSTANCE_CREATE_INFO};
@@ -716,7 +717,7 @@ Result DeviceImpl::initVulkanInstanceAndDevice(
         SIMPLE_EXTENSION_FEATURE(
             extendedFeatures.computeShaderDerivativeFeatures,
             computeDerivativeGroupLinear,
-            VK_NV_COMPUTE_SHADER_DERIVATIVES_EXTENSION_NAME,
+            VK_KHR_COMPUTE_SHADER_DERIVATIVES_EXTENSION_NAME,
             "computeDerivativeGroupLinear");
 
         // Only enable raytracing validation if both requested and supported
@@ -1027,7 +1028,7 @@ SlangResult DeviceImpl::initialize(const Desc& desc)
         descriptorSetAllocator.m_api = &m_api;
         initDeviceResult = initVulkanInstanceAndDevice(
             desc.existingDeviceHandles.handles,
-            ENABLE_VALIDATION_LAYER != 0 || isGfxDebugLayerEnabled());
+            isGfxDebugLayerEnabled());
         if (initDeviceResult == SLANG_OK)
             break;
     }
