@@ -233,92 +233,6 @@ Result ShaderObjectImpl::_writeOrdinaryData(
 
     encoder->uploadBufferDataImpl(buffer, offset, srcSize, src);
 
-    // In the case where this object has any sub-objects of
-    // existential/interface type, we need to recurse on those objects
-    // that need to write their state into an appropriate "pending" allocation.
-    //
-    // Note: Any values that could fit into the "payload" included
-    // in the existential-type field itself will have already been
-    // written as part of `setObject()`. This loop only needs to handle
-    // those sub-objects that do not "fit."
-    //
-    // An implementers looking at this code might wonder if things could be changed
-    // so that *all* writes related to sub-objects for interface-type fields could
-    // be handled in this one location, rather than having some in `setObject()` and
-    // others handled here.
-    //
-    Index subObjectRangeCounter = 0;
-    for (auto const& subObjectRangeInfo : specializedLayout->getSubObjectRanges())
-    {
-        Index subObjectRangeIndex = subObjectRangeCounter++;
-        auto const& bindingRangeInfo =
-            specializedLayout->getBindingRange(subObjectRangeInfo.bindingRangeIndex);
-
-        // We only need to handle sub-object ranges for interface/existential-type fields,
-        // because fields of constant-buffer or parameter-block type are responsible for
-        // the ordinary/uniform data of their own existential/interface-type sub-objects.
-        //
-        if (bindingRangeInfo.bindingType != slang::BindingType::ExistentialValue)
-            continue;
-
-        // Each sub-object range represents a single "leaf" field, but might be nested
-        // under zero or more outer arrays, such that the number of existential values
-        // in the same range can be one or more.
-        //
-        auto count = bindingRangeInfo.count;
-
-        // We are not concerned with the case where the existential value(s) in the range
-        // git into the payload part of the leaf field.
-        //
-        // In the case where the value didn't fit, the Slang layout strategy would have
-        // considered the requirements of the value as a "pending" allocation, and would
-        // allocate storage for the ordinary/uniform part of that pending allocation inside
-        // of the parent object's type layout.
-        //
-        // Here we assume that the Slang reflection API can provide us with a single byte
-        // offset and stride for the location of the pending data allocation in the
-        // specialized type layout, which will store the values for this sub-object range.
-        //
-        // TODO: The reflection API functions we are assuming here haven't been implemented
-        // yet, so the functions being called here are stubs.
-        //
-        // TODO: It might not be that a single sub-object range can reliably map to a single
-        // contiguous array with a single stride; we need to carefully consider what the
-        // layout logic does for complex cases with multiple layers of nested arrays and
-        // structures.
-        //
-        Offset subObjectRangePendingDataOffset = subObjectRangeInfo.offset.pendingOrdinaryData;
-        Size subObjectRangePendingDataStride = subObjectRangeInfo.stride.pendingOrdinaryData;
-
-        // If the range doesn't actually need/use the "pending" allocation at all, then
-        // we need to detect that case and skip such ranges.
-        //
-        // TODO: This should probably be handled on a per-object basis by caching a "does it
-        // fit?" bit as part of the information for bound sub-objects, given that we already
-        // compute the "does it fit?" status as part of `setObject()`.
-        //
-        if (subObjectRangePendingDataOffset == 0)
-            continue;
-
-        for (Slang::Index i = 0; i < count; ++i)
-        {
-            auto subObject = m_objects[bindingRangeInfo.subObjectIndex + i];
-
-            RefPtr<ShaderObjectLayoutImpl> subObjectLayout;
-            SLANG_RETURN_ON_FAIL(subObject->_getSpecializedLayout(subObjectLayout.writeRef()));
-
-            auto subObjectOffset =
-                subObjectRangePendingDataOffset + i * subObjectRangePendingDataStride;
-
-            subObject->_writeOrdinaryData(
-                encoder,
-                buffer,
-                offset + subObjectOffset,
-                destSize - subObjectOffset,
-                subObjectLayout);
-        }
-    }
-
     return SLANG_OK;
 }
 
@@ -504,7 +418,6 @@ void ShaderObjectImpl::writeAccelerationStructureDescriptor(
             static_cast<AccelerationStructureImpl*>(resourceViews[i].Ptr());
         VkWriteDescriptorSetAccelerationStructureKHR writeAS = {};
         writeAS.sType = VK_STRUCTURE_TYPE_WRITE_DESCRIPTOR_SET_ACCELERATION_STRUCTURE_KHR;
-        VkAccelerationStructureKHR nullHandle = VK_NULL_HANDLE;
         if (accelerationStructure)
         {
             writeAS.accelerationStructureCount = 1;
@@ -512,6 +425,12 @@ void ShaderObjectImpl::writeAccelerationStructureDescriptor(
         }
         else
         {
+            // The Vulkan spec states: If the nullDescriptor feature is not enabled, each element of
+            // pAccelerationStructures must not be VK_NULL_HANDLE
+            SLANG_ASSERT(
+                context.device->m_api.m_extendedFeatures.robustness2Features.nullDescriptor);
+
+            static const VkAccelerationStructureKHR nullHandle = VK_NULL_HANDLE;
             writeAS.accelerationStructureCount = 1;
             writeAS.pAccelerationStructures = &nullHandle;
         }
@@ -838,8 +757,8 @@ Result ShaderObjectImpl::bindAsValue(
                 // For the purposes of nested binding, what used to be the pending offset
                 // will now be used as the primary offset.
                 //
-                SimpleBindingOffset objOffset = rangeOffset.pending;
-                SimpleBindingOffset objStride = rangeStride.pending;
+                SimpleBindingOffset objOffset = {};
+                SimpleBindingOffset objStride = {};
                 for (Index i = 0; i < count; ++i)
                 {
                     // An existential-type sub-object is always bound just as a value,
@@ -1138,7 +1057,6 @@ Result RootShaderObjectImpl::bindAsRoot(
     RootShaderObjectLayout* layout)
 {
     BindingOffset offset = {};
-    offset.pending = layout->getPendingDataOffset();
 
     // Note: the operations here are quite similar to what `bindAsParameterBlock` does.
     // The key difference in practice is that we do *not* make use of the adjustment
