@@ -158,7 +158,7 @@ static SlangResult _compileProgramImpl(
         break;
 
     default:
-        assert(!"unexpected");
+        SLANG_ASSERT(!"unexpected");
         break;
     }
 
@@ -186,6 +186,15 @@ static SlangResult _compileProgramImpl(
         sessionOptionEntries.add(entry);
     }
 
+    if (options.useLLVMDirectly)
+    {
+        slang::CompilerOptionEntry entry;
+        entry.name = slang::CompilerOptionName::EmitCPUMethod;
+        entry.value.kind = slang::CompilerOptionValueKind::Int;
+        entry.value.intValue0 = int(SlangEmitCPUMethod::SLANG_EMIT_CPU_VIA_LLVM);
+        sessionOptionEntries.add(entry);
+    }
+
     sessionTargetDesc.format = input.target;
     if (input.profile.getLength()) // do not set profile unless requested
         sessionTargetDesc.profile = globalSession->findProfile(input.profile.getBuffer());
@@ -194,6 +203,22 @@ static SlangResult _compileProgramImpl(
     else
         sessionTargetDesc.flags = 0;
 
+    {
+        slang::CompilerOptionEntry entry;
+        entry.value.kind = slang::CompilerOptionValueKind::Int;
+        if (options.generateSPIRVDirectly)
+        {
+            entry.name = slang::CompilerOptionName::EmitSpirvDirectly;
+            entry.value.intValue0 = int(options.generateSPIRVDirectly);
+        }
+        else
+        {
+            entry.name = slang::CompilerOptionName::EmitSpirvViaGLSL;
+            entry.value.intValue0 = int(!options.generateSPIRVDirectly);
+        }
+        sessionOptionEntries.add(entry);
+    }
+
     // Not expecting argument parsing to have added any targets
     SLANG_ASSERT(sessionDesc.targetCount == 0);
     sessionDesc.targetCount = 1;
@@ -211,6 +236,15 @@ static SlangResult _compileProgramImpl(
         sessionOptionEntries.add(entry);
     }
 
+    for (auto& capability : options.capabilities)
+    {
+        slang::CompilerOptionEntry entry;
+        entry.name = slang::CompilerOptionName::Capability;
+        entry.value.kind = slang::CompilerOptionValueKind::String;
+        entry.value.stringValue0 = capability.getBuffer();
+        sessionOptionEntries.add(entry);
+    }
+
     sessionDesc.compilerOptionEntryCount = sessionOptionEntries.getCount();
     sessionDesc.compilerOptionEntries = sessionOptionEntries.getBuffer();
 
@@ -426,6 +460,22 @@ static SlangResult _compileProgramImpl(
         specializedEntryPoints.add(specializedEntryPoint);
     }
 
+    // If no explicit entry points were provided, check if the module has any
+    // defined entry points (e.g., functions marked with [shader(...)] attributes)
+    if (explicitEntryPointCount == 0 && !options.dontAddDefaultEntryPoints)
+    {
+        SlangInt32 definedEntryPointCount = module->getDefinedEntryPointCount();
+        for (SlangInt32 ee = 0; ee < definedEntryPointCount; ++ee)
+        {
+            ComPtr<slang::IEntryPoint> entryPoint;
+            SLANG_RETURN_ON_FAIL(module->getDefinedEntryPoint(ee, entryPoint.writeRef()));
+
+            // For now, we'll assume no specialization is needed for discovered entry points
+            // If specialization is needed, this would need to be updated
+            specializedEntryPoints.add(entryPoint);
+        }
+    }
+
     if (input.passThrough == SLANG_PASS_THROUGH_NONE)
     {
         componentsRawPtr.add(specializedModule);
