@@ -1,11 +1,11 @@
 // slang-fiddle-script.h
 #pragma once
 
-#include "../external/lua/lapi.h"
-#include "../external/lua/lauxlib.h"
 #include "compiler-core/slang-source-loc.h"
 #include "core/slang-list.h"
 #include "core/slang-string.h"
+#include "lua/lapi.h"
+#include "lua/lauxlib.h"
 #include "slang-fiddle-diagnostics.h"
 #include "slang-fiddle-scrape.h"
 
@@ -15,5 +15,15 @@ using namespace Slang;
 
 lua_State* getLuaState();
 
-String evaluateScriptCode(String originalFileName, String scriptSource, DiagnosticSink* sink);
+String evaluateScriptCode(
+    SourceLoc loc,
+    String originalFileName,
+    String scriptSource,
+    DiagnosticSink* sink);
+
+String evaluateLuaExpression(
+    SourceLoc loc,
+    String originalFileName,
+    String scriptSource,
+    DiagnosticSink* sink);
 } // namespace fiddle
