@@ -1,4 +1,4 @@
-// test-context.cpp
+// options.cpp
 #include "options.h"
 
 #include "../../source/core/slang-io.h"
@@ -69,12 +69,14 @@ static bool _isSubCommand(const char* arg)
         "  -bindir <path>                 Set directory for binaries (default: the path to the "
         "slang-test executable)\n"
         "  -test-dir <path>               Set directory for test files (default: tests/)\n"
-        "  -v                             Enable verbose output\n"
+        "  -v [level]                     Set verbosity level (verbose, info, failure)\n"
+        "                                 Default: verbose when -v used, info otherwise\n"
         "  -hide-ignored                  Hide results from ignored tests\n"
         "  -api-only                      Only run tests that use specified APIs\n"
         "  -verbose-paths                 Use verbose paths in output\n"
         "  -category <name>               Only run tests in specified category\n"
         "  -exclude <name>                Exclude tests in specified category\n"
+        "  -exclude-prefix <prefix>       Exclude tests with specified path prefix\n"
         "  -api <expr>                    Enable specific APIs (e.g., 'vk+dx12' or '+dx11')\n"
         "  -synthesizedTestApi <expr>     Set APIs for synthesized tests\n"
         "  -skip-api-detection            Skip API availability detection\n"
@@ -87,6 +89,23 @@ static bool _isSubCommand(const char* arg)
         "  -use-shared-library            Run tests in-process using shared library\n"
         "  -use-test-server               Run tests using test server\n"
         "  -use-fully-isolated-test-server  Run each test in isolated server\n"
+        "  -capability <name>             Compile with the given capability\n"
+        "  -shuffle-tests                 Shuffle tests in directories\n"
+        "  -shuffle-seed <seed>           Set shuffle seed (default: 1)\n"
+
+        // Recent Windows runtime versions started opening a dialog popup window when
+        // `abort()` is called, which breaks the CI workflow and some scripts that
+        // expect a normal termination.
+        // It can be helpful for debugging but we should ignore it for CI.
+        "  -ignore-abort-msg              Ignore abort message dialog popup on Windows\n"
+
+        "  -enable-debug-layers [true|false] Enable or disable Validation Layer for Vulkan\n"
+        "                                 and Debug Device for DX\n"
+        "  -cache-rhi-device [true|false] Enable or disable RHI device caching (default: true)\n"
+#if _DEBUG
+        "  -disable-debug-layers          Disable the debug layers (default enabled in debug "
+        "build)\n"
+#endif
         "\n"
         "Output modes:\n"
         "  -appveyor                      Use AppVeyor output format\n"
@@ -103,6 +122,7 @@ static bool _isSubCommand(const char* arg)
     int argc,
     char** argv,
     TestCategorySet* categorySet,
+    Slang::WriterHelper stdOut,
     Slang::WriterHelper stdError,
     Options* optionsOut)
 {
@@ -115,6 +135,19 @@ static bool _isSubCommand(const char* arg)
     char const* const* argCursor = argv;
     char const* const* argEnd = argCursor + argCount;
 
+#if _DEBUG
+    // Enabling debug layers by default in debug builds.
+    // For DX12 it will use the debug layer, for Vulkan it will enable validation layers.
+    //
+    // CI/CD will explicitly disable this until we address all of VUID errors.
+    // https://github.com/shader-slang/slang/issues/4798
+    //
+    // When you run the Debug build locally, you may see more errors if not disabled with
+    // '-enable-debug-layers false'.
+    //
+    optionsOut->enableDebugLayers = true;
+#endif
+
     // first argument is the application name
     if (argCursor != argEnd)
     {
@@ -126,7 +159,7 @@ static bool _isSubCommand(const char* arg)
     {
         if (strcmp(argv[i], "-h") == 0 || strcmp(argv[i], "--help") == 0)
         {
-            showHelp(stdError);
+            showHelp(stdOut);
             return SLANG_FAIL;
         }
     }
@@ -195,7 +228,35 @@ static bool _isSubCommand(const char* arg)
         }
         else if (strcmp(arg, "-v") == 0)
         {
-            optionsOut->shouldBeVerbose = true;
+            if (argCursor == argEnd)
+            {
+                // Default to verbose if no argument provided (backward compatibility)
+                optionsOut->verbosity = VerbosityLevel::Verbose;
+            }
+            else
+            {
+                const char* verbosityArg = *argCursor;
+                if (strcmp(verbosityArg, "verbose") == 0)
+                {
+                    optionsOut->verbosity = VerbosityLevel::Verbose;
+                    argCursor++;
+                }
+                else if (strcmp(verbosityArg, "info") == 0)
+                {
+                    optionsOut->verbosity = VerbosityLevel::Info;
+                    argCursor++;
+                }
+                else if (strcmp(verbosityArg, "failure") == 0)
+                {
+                    optionsOut->verbosity = VerbosityLevel::Failure;
+                    argCursor++;
+                }
+                else
+                {
+                    // Not a verbosity level, treat as old-style -v
+                    optionsOut->verbosity = VerbosityLevel::Verbose;
+                }
+            }
         }
         else if (strcmp(arg, "-hide-ignored") == 0)
         {
@@ -213,6 +274,24 @@ static bool _isSubCommand(const char* arg)
         {
             optionsOut->generateHLSLBaselines = true;
         }
+        else if (strcmp(arg, "-shuffle-tests") == 0)
+        {
+            optionsOut->shuffleTests = true;
+        }
+        else if (strcmp(arg, "-shuffle-seed") == 0)
+        {
+            if (argCursor == argEnd)
+            {
+                stdError.print("error: expected operand for '%s'\n", arg);
+                showHelp(stdError);
+                return SLANG_FAIL;
+            }
+            optionsOut->shuffleSeed = stringToInt(*argCursor++);
+            if (optionsOut->shuffleSeed <= 0)
+            {
+                optionsOut->shuffleSeed = 1;
+            }
+        }
         else if (strcmp(arg, "-release") == 0)
         {
             // Assumed to be handle by .bat file that called us
@@ -307,6 +386,18 @@ static bool _isSubCommand(const char* arg)
                 optionsOut->excludeCategories.add(category, category);
             }
         }
+        else if (strcmp(arg, "-exclude-prefix") == 0)
+        {
+            if (argCursor == argEnd)
+            {
+                stdError.print("error: expected operand for '%s'\n", arg);
+                showHelp(stdError);
+                return SLANG_FAIL;
+            }
+            Slang::StringBuilder sb;
+            Slang::Path::simplify(*argCursor++, Slang::Path::SimplifyStyle::NoRoot, sb);
+            optionsOut->excludePrefixes.add(sb);
+        }
         else if (strcmp(arg, "-api") == 0)
         {
             if (argCursor == argEnd)
@@ -359,6 +450,23 @@ static bool _isSubCommand(const char* arg)
         {
             optionsOut->emitSPIRVDirectly = false;
         }
+        else if (strcmp(arg, "-capability") == 0)
+        {
+            if (argCursor == argEnd)
+            {
+                stdError.print("error: expected operand for '%s'\n", arg);
+                showHelp(stdError);
+                return SLANG_FAIL;
+            }
+            optionsOut->capabilities.add(*argCursor++);
+        }
+        else if (strcmp(arg, "-ignore-abort-msg") == 0)
+        {
+            optionsOut->ignoreAbortMsg = true;
+#ifdef _MSC_VER
+            _set_abort_behavior(0, _WRITE_ABORT_MSG);
+#endif
+        }
         else if (strcmp(arg, "-expected-failure-list") == 0)
         {
             if (argCursor == argEnd)
@@ -374,7 +482,20 @@ static bool _isSubCommand(const char* arg)
             StringUtil::split(text.getUnownedSlice(), '\n', lines);
             for (auto line : lines)
             {
-                optionsOut->expectedFailureList.add(line);
+                // Remove comments (everything after '#' character)
+                auto trimmedLine = line;
+                auto commentIndex = line.indexOf('#');
+                if (commentIndex != -1)
+                {
+                    trimmedLine = line.head(commentIndex);
+                }
+
+                // Trim whitespace and skip empty lines
+                trimmedLine = trimmedLine.trim();
+                if (trimmedLine.getLength() > 0)
+                {
+                    optionsOut->expectedFailureList.add(trimmedLine);
+                }
             }
         }
         else if (strcmp(arg, "-test-dir") == 0)
@@ -395,6 +516,54 @@ static bool _isSubCommand(const char* arg)
         {
             optionsOut->skipReferenceImageGeneration = true;
         }
+        else if (strcmp(arg, "-enable-debug-layers") == 0)
+        {
+            optionsOut->enableDebugLayers = true;
+
+            if (argCursor == argEnd)
+            {
+                stdError.print("error: expected operand for '%s'\n", arg);
+                showHelp(stdError);
+                return SLANG_FAIL;
+            }
+
+            // Check for false variants
+            const char* value = *argCursor++;
+            if (value[0] == 'f' || value[0] == 'F' || value[0] == 'n' || value[0] == 'N' ||
+                value[0] == '0' ||
+                ((value[0] == 'o' || value[0] == 'O') && (value[1] == 'f' || value[1] == 'F')))
+            {
+                optionsOut->enableDebugLayers = false;
+            }
+        }
+        else if (strcmp(arg, "-cache-rhi-device") == 0)
+        {
+            optionsOut->cacheRhiDevice = true;
+
+            if (argCursor == argEnd)
+            {
+                stdError.print("error: expected operand for '%s'\n", arg);
+                showHelp(stdError);
+                return SLANG_FAIL;
+            }
+
+            // Check for false variants
+            const char* value = *argCursor++;
+            if (value[0] == 'f' || value[0] == 'F' || value[0] == 'n' || value[0] == 'N' ||
+                value[0] == '0' ||
+                ((value[0] == 'o' || value[0] == 'O') && (value[1] == 'f' || value[1] == 'F')))
+            {
+                optionsOut->cacheRhiDevice = false;
+            }
+        }
+#if _DEBUG
+        else if (strcmp(arg, "-disable-debug-layers") == 0)
+        {
+            stdError.print("warning: '-disable-debug-layers' is deprecated, use "
+                           "'-enable-debug-layers false'\n");
+            optionsOut->enableDebugLayers = false;
+        }
+#endif
         else
         {
             stdError.print("unknown option '%s'\n", arg);
