@@ -3,6 +3,7 @@
 
 #include "../../source/core/slang-process-util.h"
 #include "../../source/core/slang-string-util.h"
+#include "options.h"
 
 #include <mutex>
 #include <stdio.h>
@@ -73,7 +74,7 @@ static void appendXmlEncode(const String& in, StringBuilder& out)
         if (cur < end)
         {
             const char encodeChar = *cur++;
-            assert(isXmlEncodeChar(encodeChar));
+            SLANG_ASSERT(isXmlEncodeChar(encodeChar));
             appendXmlEncode(encodeChar, out);
         }
     }
@@ -91,7 +92,7 @@ TestReporter::TestReporter()
 
     m_inTest = false;
     m_dumpOutputOnFailure = false;
-    m_isVerbose = false;
+    m_verbosity = VerbosityLevel::Info;
 }
 
 Result TestReporter::init(
@@ -124,8 +125,8 @@ bool TestReporter::canWriteStdError() const
 void TestReporter::startTest(const char* testName)
 {
     // Must be in a suite
-    assert(m_suiteStack.getCount());
-    assert(!m_inTest);
+    SLANG_ASSERT(m_suiteStack.getCount());
+    SLANG_ASSERT(!m_inTest);
 
     m_inTest = true;
 
@@ -139,8 +140,8 @@ void TestReporter::startTest(const char* testName)
 
 void TestReporter::endTest()
 {
-    assert(m_suiteStack.getCount());
-    assert(m_inTest);
+    SLANG_ASSERT(m_suiteStack.getCount());
+    SLANG_ASSERT(m_inTest);
 
     m_currentInfo.message = m_currentMessage;
 
@@ -151,7 +152,7 @@ void TestReporter::endTest()
 
 void TestReporter::addResult(TestResult result)
 {
-    assert(m_inTest);
+    SLANG_ASSERT(m_inTest);
 
     std::lock_guard<std::recursive_mutex> lock(m_mutex);
     if (result == TestResult::Fail && m_expectedFailureList.contains(m_currentInfo.name))
@@ -160,6 +161,11 @@ void TestReporter::addResult(TestResult result)
     m_numCurrentResults++;
 }
 
+TestResult TestReporter::getResult() const
+{
+    return m_currentInfo.testResult;
+}
+
 void TestReporter::addExecutionTime(double time)
 {
     std::lock_guard<std::recursive_mutex> lock(m_mutex);
@@ -173,7 +179,7 @@ void TestReporter::addResultWithLocation(
     const char* file,
     int line)
 {
-    assert(m_inTest);
+    SLANG_ASSERT(m_inTest);
 
     std::lock_guard<std::recursive_mutex> lock(m_mutex);
     result = adjustResult(m_currentInfo.name.getUnownedSlice(), result);
@@ -334,10 +340,29 @@ static void _appendTime(double timeInSec, StringBuilder& out)
 
 void TestReporter::_addResult(TestInfo info)
 {
+    // If test result is still uninitialized, no result was ever set for this test
+    // This likely indicates a bug in the test framework usage
+    if (info.testResult == TestResult::Uninitialized)
+    {
+        SLANG_ASSERT(!"Test ended without setting a result - this is likely a bug");
+        // Don't count uninitialized tests in statistics
+        return;
+    }
+
     if (info.testResult == TestResult::Ignored && m_hideIgnored)
     {
         return;
     }
+
+    // If test is pending retry, don't count it in any statistics yet
+    // The test will be re-run and reported again with a final result
+    if (info.testResult == TestResult::PendingRetry)
+    {
+        printf("failed(pending retry) '%S'\n", info.name.toWString().begin());
+        fflush(stdout);
+        return;
+    }
+
     info.testResult = adjustResult(info.name.getUnownedSlice(), info.testResult);
 
     m_totalTestCount++;
@@ -360,14 +385,23 @@ void TestReporter::_addResult(TestInfo info)
         break;
 
     default:
-        assert(!"unexpected");
+        SLANG_ASSERT(!"unexpected");
         break;
     }
 
     m_testInfos.add(info);
 
-    auto defaultOutputFunc = [](const TestInfo& info)
+    auto defaultOutputFunc = [this](const TestInfo& info)
     {
+        // Skip output for passed/ignored tests if verbosity < Info
+        if (m_verbosity < VerbosityLevel::Info)
+        {
+            if (info.testResult == TestResult::Pass || info.testResult == TestResult::Ignored)
+            {
+                return;
+            }
+        }
+
         char const* resultString = "UNEXPECTED";
         switch (info.testResult)
         {
@@ -384,7 +418,7 @@ void TestReporter::_addResult(TestInfo info)
             resultString = "ignored";
             break;
         default:
-            assert(!"unexpected");
+            SLANG_ASSERT(!"unexpected");
             break;
         }
 
@@ -483,7 +517,7 @@ void TestReporter::_addResult(TestInfo info)
                     break;
                 }
             default:
-                assert(!"unexpected");
+                SLANG_ASSERT(!"unexpected");
                 break;
             }
 
@@ -516,7 +550,7 @@ void TestReporter::_addResult(TestInfo info)
                 break;
 
             default:
-                assert(!"unexpected");
+                SLANG_ASSERT(!"unexpected");
                 break;
             }
 
@@ -570,7 +604,7 @@ void TestReporter::_addResult(TestInfo info)
 void TestReporter::addTest(const String& testName, TestResult testResult)
 {
     // Can't add this way if in test
-    assert(!m_inTest);
+    SLANG_ASSERT(!m_inTest);
 
     TestInfo info;
     info.name = testName;
@@ -584,7 +618,7 @@ void TestReporter::message(TestMessageType type, const String& message)
 
     if (type == TestMessageType::Info)
     {
-        if (m_isVerbose && canWriteStdError())
+        if (m_verbosity == VerbosityLevel::Verbose && canWriteStdError())
         {
             fputs(message.getBuffer(), stderr);
         }
@@ -595,9 +629,9 @@ void TestReporter::message(TestMessageType type, const String& message)
 
     if (canWriteStdError())
     {
+        fprintf(stderr, "[%s] ", m_currentInfo.name.getBuffer());
         if (type == TestMessageType::RunError || type == TestMessageType::TestFailure)
         {
-            fprintf(stderr, "error: ");
             fputs(message.getBuffer(), stderr);
             fprintf(stderr, "\n");
         }
@@ -612,6 +646,9 @@ void TestReporter::message(TestMessageType type, const String& message)
     {
         m_currentMessage << "\n";
     }
+
+    if (m_currentInfo.name.getLength())
+        m_currentMessage << "[" << m_currentInfo.name << "] ";
     m_currentMessage.append(message);
 }
 
@@ -688,7 +725,7 @@ void TestReporter::outputSummary()
             printf("\n===\n\n");
             if (m_failedTestCount)
             {
-                printf("failing tests:\n");
+                printf("%d failing tests:\n", m_failedTestCount);
                 printf("---\n");
                 for (const auto& testInfo : m_testInfos)
                 {
@@ -769,7 +806,7 @@ void TestReporter::outputSummary()
     case TestOutputMode::XUnit2:
         {
             // https://xunit.github.io/docs/format-xml-v2
-            assert("Not currently supported");
+            SLANG_ASSERT("Not currently supported");
             break;
         }
     case TestOutputMode::TeamCity:
@@ -803,7 +840,7 @@ void TestReporter::startSuite(const String& name)
 
 void TestReporter::endSuite()
 {
-    assert(m_suiteStack.getCount());
+    SLANG_ASSERT(m_suiteStack.getCount());
 
     switch (m_outputMode)
     {
